@@ -1,22 +1,24 @@
-#undef bzread
-#undef bzopen
-#undef bzdopen
-#undef bzwrite
-#undef bzerror
-#undef bzclose
-#undef bzflush
+#undef EAZEL_BUILD_TIMESTAMP
+#undef EAZEL_SERVICES
+#undef ENABLE_NLS
+#undef ENABLE_PROFILER
+#undef HAVE_CATGETS
+#undef HAVE_GETTEXT
+#undef HAVE_LC_MESSAGES
+#undef HAVE_MEDUSA
+#undef HAVE_PAM
+#undef HAVE_RPM
+#undef HAVE_RPM_30
+#undef HAVE_RPM_40
+#undef HAVE_STPCPY
 #undef bzCompress
 #undef bzCompressInit
 #undef bzDecompress
 #undef bzDecompressInit
-#undef EAZEL_SERVICES
-#undef ENABLE_PROFILER
-#undef ENABLE_NLS
-#undef HAVE_CATGETS
-#undef HAVE_GETTEXT
-#undef HAVE_LC_MESSAGES
-#undef HAVE_PAM
-#undef HAVE_RPM
-#undef HAVE_STPCPY
-#undef USING_OAF
-#undef EAZEL_BUILD_TIMESTAMP
+#undef bzclose
+#undef bzdopen
+#undef bzerror
+#undef bzflush
+#undef bzopen
+#undef bzread
+#undef bzwrite
