@@ -32,6 +32,7 @@
 
 #include "nautilus-adapter-control-embed-strategy.h"
 #include "nautilus-adapter-embed-strategy-private.h"
+#include "nautilus-zoomable-proxy.h"
 
 #include <gtk/gtkobject.h>
 #include <gtk/gtksignal.h>
@@ -42,6 +43,7 @@
 struct NautilusAdapterControlEmbedStrategyDetails {
 	Bonobo_Control      control;
 	BonoboControlFrame *control_frame;
+	BonoboObject       *zoomable;
 	GtkWidget          *widget;
 };
 
@@ -54,6 +56,7 @@ static void nautilus_adapter_control_embed_strategy_deactivate       (NautilusAd
 static void nautilus_adapter_control_embed_strategy_destroy          (GtkObject                              *object);
 
 static GtkWidget *nautilus_adapter_control_embed_strategy_get_widget (NautilusAdapterEmbedStrategy *strategy);
+static BonoboObject *nautilus_adapter_control_embed_strategy_get_zoomable (NautilusAdapterEmbedStrategy *strategy);
 
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusAdapterControlEmbedStrategy, nautilus_adapter_control_embed_strategy, NAUTILUS_TYPE_ADAPTER_EMBED_STRATEGY)
@@ -72,6 +75,7 @@ nautilus_adapter_control_embed_strategy_initialize_class (NautilusAdapterControl
 	adapter_embed_strategy_class = NAUTILUS_ADAPTER_EMBED_STRATEGY_CLASS (klass);
 
 	adapter_embed_strategy_class->get_widget = nautilus_adapter_control_embed_strategy_get_widget;
+	adapter_embed_strategy_class->get_zoomable = nautilus_adapter_control_embed_strategy_get_zoomable;
 	adapter_embed_strategy_class->activate = nautilus_adapter_control_embed_strategy_activate;
 	adapter_embed_strategy_class->deactivate = nautilus_adapter_control_embed_strategy_deactivate;
 }
@@ -145,9 +149,9 @@ activate_uri_callback (BonoboControlFrame *frame,
 		       gboolean relative,
 		       NautilusAdapterControlEmbedStrategy *strategy)
 {
-	/* FIXME: ignoring `relative' parameter as the concept is kind
-           of broken. */
-
+	/* FIXME bugzilla.eazel.com 4404: ignoring `relative' parameter as
+	 * the concept is kind of broken. 
+	 */
 	nautilus_adapter_embed_strategy_emit_open_location (NAUTILUS_ADAPTER_EMBED_STRATEGY (strategy), 
 							    uri);
 }
@@ -157,6 +161,8 @@ nautilus_adapter_control_embed_strategy_new (Bonobo_Control control,
 					     Bonobo_UIContainer ui_container)
 {
 	NautilusAdapterControlEmbedStrategy *strategy;
+	Bonobo_Zoomable corba_zoomable;
+	CORBA_Environment ev;
 
 	strategy = NAUTILUS_ADAPTER_CONTROL_EMBED_STRATEGY (gtk_object_new (NAUTILUS_TYPE_ADAPTER_CONTROL_EMBED_STRATEGY, NULL));
 	gtk_object_ref (GTK_OBJECT (strategy));
@@ -172,6 +178,18 @@ nautilus_adapter_control_embed_strategy_new (Bonobo_Control control,
 	gtk_signal_connect (GTK_OBJECT (strategy->details->control_frame),
 			    "activate_uri", GTK_SIGNAL_FUNC (activate_uri_callback), strategy);
 
+	CORBA_exception_init (&ev);
+	corba_zoomable = Bonobo_Unknown_queryInterface (control,
+							"IDL:Bonobo/Zoomable:1.0",
+							&ev);
+	if ((ev._major == CORBA_NO_EXCEPTION) &&
+	    !CORBA_Object_is_nil (corba_zoomable, &ev)) {
+		strategy->details->zoomable = nautilus_zoomable_proxy_get
+			(corba_zoomable);
+	}
+
+	CORBA_exception_free (&ev);
+
 	return NAUTILUS_ADAPTER_EMBED_STRATEGY (strategy);
 }
 
@@ -185,5 +203,14 @@ nautilus_adapter_control_embed_strategy_get_widget (NautilusAdapterEmbedStrategy
 	return strategy->details->widget;
 }
 
+static BonoboObject *
+nautilus_adapter_control_embed_strategy_get_zoomable (NautilusAdapterEmbedStrategy *abstract_strategy)
+{
+	NautilusAdapterControlEmbedStrategy *strategy;
+
+	strategy = NAUTILUS_ADAPTER_CONTROL_EMBED_STRATEGY (abstract_strategy);
+
+	return strategy->details->zoomable;
+}
 
 
