@@ -45,9 +45,13 @@
 #include <libnautilus-adapter/nautilus-adapter-factory.h>
 
 struct NautilusAdapterDetails {
-	NautilusAdapterLoadStrategy  *load_strategy;
-	NautilusAdapterEmbedStrategy *embed_strategy;
 	NautilusView                 *nautilus_view;
+	NautilusAdapterEmbedStrategy *embed_strategy;
+	NautilusAdapterLoadStrategy  *load_strategy;
+	guint report_load_underway_id;
+	guint report_load_progress_id;
+	guint report_load_complete_id;
+        guint report_load_failed_id;
 };
 
 
@@ -111,20 +115,29 @@ nautilus_adapter_initialize (NautilusAdapter *adapter)
 static void
 nautilus_adapter_destroy (GtkObject *object)
 {
-	NautilusAdapter *server;
+	NautilusAdapter *adapter;
 	
-	server = NAUTILUS_ADAPTER (object);
+	adapter = NAUTILUS_ADAPTER (object);
 
-	if (server->details->load_strategy != NULL) {
-		nautilus_adapter_load_strategy_stop_loading (server->details->load_strategy);
-		gtk_object_unref (GTK_OBJECT (server->details->load_strategy));
+	gtk_signal_disconnect (GTK_OBJECT (adapter->details->load_strategy),
+			       adapter->details->report_load_underway_id);
+	gtk_signal_disconnect (GTK_OBJECT (adapter->details->load_strategy),
+			       adapter->details->report_load_progress_id);
+	gtk_signal_disconnect (GTK_OBJECT (adapter->details->load_strategy),
+			       adapter->details->report_load_complete_id);
+	gtk_signal_disconnect (GTK_OBJECT (adapter->details->load_strategy),
+			       adapter->details->report_load_failed_id);
+
+	if (adapter->details->load_strategy != NULL) {
+		nautilus_adapter_load_strategy_stop_loading (adapter->details->load_strategy);
+		gtk_object_unref (GTK_OBJECT (adapter->details->load_strategy));
 	}
 
-	if (server->details->embed_strategy != NULL) {
-		gtk_object_unref (GTK_OBJECT (server->details->embed_strategy));
+	if (adapter->details->embed_strategy != NULL) {
+		gtk_object_unref (GTK_OBJECT (adapter->details->embed_strategy));
 	}
 
-	g_free (server->details);
+	g_free (adapter->details);
 	
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
@@ -135,10 +148,13 @@ nautilus_adapter_new (Bonobo_Unknown component)
 	NautilusAdapter      *adapter;
 	BonoboControl        *control;
 	GtkWidget            *bin;
+	BonoboObject         *zoomable;
 	CORBA_Environment     ev;
 
 
-	/* FIXME: should be done with construct args */
+	/* FIXME bugzilla.eazel.com 4405: should be done with
+	 * construct args 
+	 */
 
 	CORBA_exception_init (&ev);
 
@@ -166,6 +182,11 @@ nautilus_adapter_new (Bonobo_Unknown component)
 		return NULL;
 	}
 
+	/* Get the NautilusAdapterZoomable proxy object. */
+	zoomable = nautilus_adapter_embed_strategy_get_zoomable (adapter->details->embed_strategy);
+	if (zoomable != NULL)
+		bonobo_object_add_interface (BONOBO_OBJECT (control), zoomable);
+
 	gtk_signal_connect (GTK_OBJECT (control), "activate",
 			    GTK_SIGNAL_FUNC (nautilus_adapter_activate_callback),
 			    adapter);
@@ -186,28 +207,26 @@ nautilus_adapter_new (Bonobo_Unknown component)
 	}
 
 	/* hook up load strategy signals */
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (adapter->details->load_strategy),
-					       "report_load_underway",
-					       nautilus_adapter_load_underway_callback,
-					       GTK_OBJECT (adapter));
-
-	/* hook up load strategy signals */
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (adapter->details->load_strategy),
-					       "report_load_progress",
-					       nautilus_adapter_load_progress_callback,
-					       GTK_OBJECT (adapter));
-
-	/* hook up load strategy signals */
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (adapter->details->load_strategy),
-					       "report_load_complete",
-					       nautilus_adapter_load_complete_callback,
-					       GTK_OBJECT (adapter));
-
-	/* hook up load strategy signals */
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (adapter->details->load_strategy),
-					       "report_load_failed",
-					       nautilus_adapter_load_failed_callback,
-					       GTK_OBJECT (adapter));
+	adapter->details->report_load_underway_id =
+		gtk_signal_connect_object (GTK_OBJECT (adapter->details->load_strategy),
+					   "report_load_underway",
+					   nautilus_adapter_load_underway_callback,
+					   GTK_OBJECT (adapter));
+	adapter->details->report_load_progress_id =
+		gtk_signal_connect_object (GTK_OBJECT (adapter->details->load_strategy),
+					   "report_load_progress",
+					   nautilus_adapter_load_progress_callback,
+					   GTK_OBJECT (adapter));
+	adapter->details->report_load_complete_id =
+		gtk_signal_connect_object (GTK_OBJECT (adapter->details->load_strategy),
+					   "report_load_complete",
+					   nautilus_adapter_load_complete_callback,
+					   GTK_OBJECT (adapter));
+	adapter->details->report_load_failed_id =
+		gtk_signal_connect_object (GTK_OBJECT (adapter->details->load_strategy),
+					   "report_load_failed",
+					   nautilus_adapter_load_failed_callback,
+					   GTK_OBJECT (adapter));
 
 	/* complete the embedding */
 
@@ -262,8 +281,8 @@ nautilus_adapter_open_location_callback  (NautilusAdapterEmbedStrategy *strategy
 					  const char                   *uri,
 					  NautilusAdapter              *adapter)
 {
-	nautilus_view_open_location (adapter->details->nautilus_view,
-				     uri);
+	nautilus_view_open_location_in_this_window
+		(adapter->details->nautilus_view, uri);
 }
 
 
