@@ -28,51 +28,39 @@
 #include <config.h>
 #include "nautilus-hardware-view.h"
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <dirent.h>
-
-#include <libnautilus/libnautilus.h>
+#include <fcntl.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gnome.h>
+#include <gtk/gtksignal.h>
+#include <libgnorba/gnorba.h>
 #include <libnautilus-extensions/nautilus-background.h>
 #include <libnautilus-extensions/nautilus-directory-background.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-file.h>
-#include <libnautilus-extensions/nautilus-font-factory.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
+#include <libnautilus-extensions/nautilus-image.h>
+#include <libnautilus-extensions/nautilus-label.h>
 #include <libnautilus-extensions/nautilus-metadata.h>
 #include <libnautilus-extensions/nautilus-string.h>
-#include <libnautilus-extensions/nautilus-label.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtksignal.h>
-#include <gnome.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnorba/gnorba.h>
+#include <libnautilus/libnautilus.h>
 #include <limits.h>
-
-#define HARDWARE_FONT_SIZE 14
-#define HARDWARE_TITLE_FONT_SIZE 24
+#include <sys/stat.h>
+#include <sys/types.h>
 
 struct _NautilusHardwareViewDetails {
-        char *uri;
         NautilusView *nautilus_view;
-        
         GtkWidget *form;
 };
 
-
 enum {
-	TARGET_URI_LIST,
 	TARGET_COLOR,
-        TARGET_GNOME_URI_LIST
 };
 
 static GtkTargetEntry hardware_dnd_target_table[] = {
-	{ "text/uri-list",  0, TARGET_URI_LIST },
 	{ "application/x-color", 0, TARGET_COLOR },
-	{ "x-special/gnome-icon-list",  0, TARGET_GNOME_URI_LIST }
 };
 
 static void nautilus_hardware_view_drag_data_received (GtkWidget                 *widget,
@@ -137,9 +125,10 @@ nautilus_hardware_view_initialize (NautilusHardwareView *hardware_view)
 static void
 nautilus_hardware_view_destroy (GtkObject *object)
 {
-	NautilusHardwareView *hardware_view = NAUTILUS_HARDWARE_VIEW (object);
+	NautilusHardwareView *hardware_view;
+
+        hardware_view = NAUTILUS_HARDWARE_VIEW (object);
 
-	g_free (hardware_view->details->uri);
 	g_free (hardware_view->details);
 
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
@@ -152,25 +141,25 @@ nautilus_hardware_view_get_nautilus_view (NautilusHardwareView *hardware_view)
 	return hardware_view->details->nautilus_view;
 }
 
-static char* 
-read_proc_info(const gchar* proc_filename)
+static char * 
+read_proc_info (const char* proc_filename)
 {
 	FILE *thisFile;
 	char *result;
 	char buffer[256];
-	char* path_name = g_strdup_printf("/proc/%s", proc_filename);	
+	char* path_name;
 	GString* string_data = g_string_new("");
 	
-	thisFile = fopen(  path_name, "r");
-	
-	while (fgets(buffer, 255, thisFile) != NULL) {
-		g_string_append(string_data, buffer);		
+        path_name = g_strdup_printf ("/proc/%s", proc_filename);	
+	thisFile = fopen (path_name, "r");
+	g_free (path_name);
+	while (fgets (buffer, 255, thisFile) != NULL) {
+		g_string_append (string_data, buffer);		
 	}
-	fclose(thisFile);
+	fclose (thisFile);
 	
-	result = g_strdup(string_data->str);
-	g_string_free(string_data, TRUE);
-	g_free(path_name);
+	result = string_data->str;
+	g_string_free(string_data, FALSE);
 
 	return result;
 }
@@ -178,21 +167,23 @@ read_proc_info(const gchar* proc_filename)
 /* utility routine to extract information from a string and add it to an XML node */
 
 static char*
-extract_info(gchar* data, const gchar *field_name, gint nth)
+extract_info (char* data, const char *field_name, int nth)
 {
 	int index;
 	char **info_array;
 	char *field_data = NULL;
 	
 	/* parse the data into a string array */
-	info_array = g_strsplit(data, "\n", 32);
+	info_array = g_strsplit (data, "\n", 32);
 	/* iterate through the data isolating the field */
 	for (index = 0; index < 32; index++) {
-		if (info_array[index] == NULL)
+		if (info_array[index] == NULL) {
 			break;
+                }
 		if (nautilus_str_has_prefix(info_array[index], field_name)) {
-			if(nth>0) nth--;
-			else {
+			if (nth > 0) {
+                                nth--;
+			} else {
 				field_data = info_array[index] + strlen(field_name);
 				field_data = strchr(field_data, ':') + 1;
 				field_data =  g_strchug(field_data);
@@ -202,49 +193,49 @@ extract_info(gchar* data, const gchar *field_name, gint nth)
 	}
 	
 	/* add the requested node if the field was found */
-	if (field_data == NULL)
-		return NULL;
-	field_data = g_strdup(field_data);
-	g_strfreev(info_array);
+	field_data = g_strdup (field_data);
+	g_strfreev (info_array);
 	return field_data;
 }
 
 /* get descriptive text about the CPU */
 
-static char*
-get_CPU_description(gint nth)
+static char *
+get_CPU_description (int nth)
 {
-	char *temp_str, *result;
-	GString* string_data = g_string_new("");
-	char* proc_data = read_proc_info("cpuinfo");
+	char *proc_data;
+        char *model, *speed, *cache_size;
+	char *result;
 
-	temp_str = extract_info(proc_data, "processor", nth);
-	if(temp_str == NULL) {
-		/* can't find nth processor */
-		g_free(proc_data);
-		g_string_free(string_data, TRUE);
-		return NULL;
-	}
-	g_free (temp_str);
-	
-	temp_str = extract_info(proc_data, "model name", nth);
-	g_string_append(string_data, temp_str);
-	g_string_append(string_data, " CPU\n");
-	g_free(temp_str);
-	
-	temp_str = extract_info(proc_data, "cpu MHz", nth);
-	g_string_append(string_data, temp_str);
-	g_string_append(string_data, " MHz\n");
-	g_free(temp_str);
-	
-	temp_str = extract_info(proc_data, "cache size", nth);
-	g_string_append(string_data, temp_str);
-	g_string_append(string_data, " cache size\n");
-	g_free(temp_str);
-		
-	g_free(proc_data);
-	result = g_strdup(string_data->str);
-	g_string_free(string_data, TRUE);
+        proc_data = read_proc_info ("cpuinfo");
+	model = extract_info (proc_data, "model name", nth);
+	speed = extract_info (proc_data, "cpu MHz", nth);
+	cache_size = extract_info (proc_data, "cache size", nth);
+	g_free (proc_data);
+
+        /* FIXME: The MHz value always comes in with a "." as the
+         * radix character. We need to change that to the local one
+         * ("," for many European countries).
+         */
+
+        /* FIXME bugzilla.eazel.com 5298: The KB string that comes
+         * from the proc data is wrong -- "kB" is correct, and we use
+         * "K" for file sizes as of this writing (although we use "MB"
+         * and "GB").
+         */
+        
+        if (model == NULL || speed == NULL || cache_size == NULL) {
+                result = NULL;
+        } else {
+                result = g_strdup_printf (_("%s CPU\n"
+                                            "%s MHz\n"
+                                            "%s cache size"),
+                                          model, speed, cache_size);
+        }
+
+        g_free (model);
+        g_free (speed);
+        g_free (cache_size);
 
 	return result;	
 }
@@ -253,82 +244,98 @@ get_CPU_description(gint nth)
 static char *
 get_RAM_description (void)
 {
-	char *temp_str, *num_str, *result;
-	GString* string_data = g_string_new("");
-	char* proc_data = read_proc_info("meminfo");
-	
+	char *temp_str, *result;
+	char *proc_data;
+        gulong ram_size;
+
+        proc_data = read_proc_info ("meminfo");
+
 	temp_str = extract_info (proc_data, "MemTotal", 0);
-	/* strip kbyte suffix */
+        if (temp_str == NULL || strlen (temp_str) < 3) {
+                g_free (temp_str);
+                return NULL;
+        }
+
+        /* strip kbyte suffix */
 	temp_str[strlen(temp_str) - 3] = '\0';
 
-        num_str = gnome_vfs_format_file_size_for_display (1024 * atoi (temp_str));
-	
-	g_string_append(string_data, num_str);
-	g_string_append(string_data, " RAM");
-	g_free(num_str);
-	g_free(temp_str);
+	ram_size = (strtoul (temp_str, NULL, 10) + 500) / 1000;
+	if (ram_size >= 1000) {
+		result = g_strdup_printf (_("%lu GB RAM"), ram_size / 1000);
+	} else {
+		result = g_strdup_printf (_("%lu MB RAM"), ram_size);
+	}
 
-	g_free(proc_data);
-	result = g_strdup(string_data->str);
-	g_string_free(string_data, TRUE);
+ 	g_free (temp_str);
+
+	g_free (proc_data);
 
 	return result;
 }
 
-static char*
-get_IDE_description(char *device)
+static char *
+get_IDE_description (const char *device)
 {
         char *temp_str, *num_str, *result;
-        GString* string_data = g_string_new("");
+        GString *string_data = g_string_new("");
         char *proc_file;
+        gulong capacity;
 
         /* Read model information string */
-        proc_file = g_strdup_printf("%s/model", device);
-        temp_str = read_proc_info(proc_file);
-        temp_str[strlen(temp_str) - 1] = '\0';
-        g_string_append(string_data, temp_str);
-        g_free(temp_str);
-        g_free(proc_file);
+        proc_file = g_strdup_printf ("%s/model", device);
+        temp_str = read_proc_info (proc_file);
+        temp_str[strlen (temp_str) - 1] = '\0';
+        g_string_append (string_data, temp_str);
+        g_free (temp_str);
+        g_free (proc_file);
         
         /* Read media type */
-        proc_file = g_strdup_printf("%s/media", device);
-        temp_str = read_proc_info(proc_file);
-        g_free(proc_file);
+        proc_file = g_strdup_printf ("%s/media", device);
+        temp_str = read_proc_info (proc_file);
+        g_free (proc_file);
         
         /* If a hard disk, get the size */
-        if(!strcmp(temp_str, "disk\n")) {
-                unsigned long capacity;
+        if (strcmp (temp_str, "disk\n") == 0) {
+                g_free (temp_str);
 
-                g_free(temp_str);
+                proc_file = g_strdup_printf ("%s/capacity", device);
+                temp_str = read_proc_info (proc_file);
+                temp_str[strlen (temp_str) - 1] = '\0';
 
-                proc_file = g_strdup_printf("%s/capacity", device);
-                temp_str = read_proc_info(proc_file);
-                temp_str[strlen(temp_str) - 1] = '\0';
-
-                /* NOTE: this should be 
-                   capacity = strtoul (...)
-                   num_str = gnome_vfs_format_file_size_for_display (512 * capacity);
-                   
-                   (512 bytes pr sector)
-                   but 512 * 23579136 (numsectors) = 12072517632 > unsigned long,
-                   which is the type that gnome_vfs uses */
-                /* This converts & calcs the sectors into MB's */
-                capacity = 512 * (strtoul (temp_str, NULL, 10) / (1024 * 1024));
-                if (capacity > 1024) {
-                        num_str = g_strdup_printf ("%lu GB", capacity / 1024);
-                } else {
-                        num_str = g_strdup_printf ("%lu MB", capacity);
-                }
-                g_string_append(string_data, "\n");
-                g_string_append(string_data, num_str);
-                g_free(temp_str);
-                g_free(proc_file);
+                 /* NOTE: this should be 
+		  *  capacity = strtoul (...)
+		  *  num_str = gnome_vfs_format_file_size_for_display (512 * numsectors);   
+		  *               
+		  *  (512 bytes per sector)
+		  *
+		  *  but with large disks we overflow an unsigned long, which is the
+		  *  the type that gnome_vfs uses (Darin: Not true, gnome-vfs uses 64-bit integers).  
+		  *
+		  *  ALSO, in keeping with disk manufacturer convention, disk sizes
+		  *  are quoted in powers of 10 (i.e., MB is 10^6, GB is 10^9).
+		  *  (see http://www.maxtor.com/technology/Digi_vs_Deci.html
+		  *  So as to not confuse the user too much, we will follow the
+		  *  same convention.)
+		  *
+		  */
+   
+		capacity = (512 * (strtoul (temp_str, NULL, 10) / 1000)) / 1000;
+		if (capacity >= 1000) {
+			num_str = g_strdup_printf (_("%lu GB"), capacity / 1000);
+		} else {
+			num_str = g_strdup_printf (_("%lu MB"), capacity);
+		}
+             
+		g_string_append (string_data, "\n");
+                g_string_append (string_data, num_str);
+                g_free (temp_str);
+                g_free (proc_file);
         } else {
-                g_free(temp_str);
+                g_free (temp_str);
         }
         
-        result = strdup(string_data->str);
-        g_string_free(string_data, TRUE);
+        result = string_data->str;
+        g_string_free (string_data, FALSE);
         
         return result;
 }
@@ -337,7 +344,9 @@ get_IDE_description(char *device)
 /* shared utility to allocate a title for a form */
 
 static void
-setup_form_title (NautilusHardwareView *view, const char* image_name, const char* title_text)
+setup_form_title (NautilusHardwareView *view,
+                  const char *image_name,
+                  const char *title_text)
 {
 	GtkWidget *temp_widget;
 	char *file_name;	
@@ -349,15 +358,15 @@ setup_form_title (NautilusHardwareView *view, const char* image_name, const char
 	if (image_name != NULL) {
  		file_name = gnome_pixmap_file (image_name);
 		if (file_name != NULL) {
-			temp_widget = GTK_WIDGET (gnome_pixmap_new_from_file (file_name));
-			gtk_box_pack_start(GTK_BOX(temp_container), temp_widget, 0, 0, 8);		
-			gtk_widget_show(temp_widget);
+			temp_widget = nautilus_image_new (file_name);
+			gtk_box_pack_start (GTK_BOX(temp_container), temp_widget, 0, 0, 8);		
+			gtk_widget_show (temp_widget);
 			g_free (file_name);
 		}
 	}
 	
  	temp_widget = nautilus_label_new (title_text);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_TITLE_FONT_SIZE);
+	nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 10);
 
 	gtk_box_pack_start (GTK_BOX (temp_container), temp_widget, 0, 0, 8);			 	
 	gtk_widget_show (temp_widget);
@@ -378,7 +387,8 @@ add_element_to_table (GtkWidget *table, GtkWidget *element, int element_index)
 }
 
 /* set up the widgetry for the overview page */
-static void setup_overview_form (NautilusHardwareView *view)
+static void
+setup_overview_form (NautilusHardwareView *view)
 {
 	char  *file_name, *temp_text;
 	GtkWidget *temp_widget, *pixmap_widget, *temp_box;
@@ -394,7 +404,7 @@ static void setup_overview_form (NautilusHardwareView *view)
 	gtk_widget_show(view->details->form);
 
 	/* set up the title */	
-	setup_form_title (view, NULL, "Hardware Overview");
+	setup_form_title (view, NULL, _("Hardware Overview"));
 
 	/* allocate a table to hold the elements */
 	table =  gtk_table_new (3, 3, FALSE);	
@@ -409,13 +419,13 @@ static void setup_overview_form (NautilusHardwareView *view)
 		gtk_widget_show (temp_box);
 
 		file_name = nautilus_pixmap_file ("cpu.png");
-                temp_widget = GTK_WIDGET (gnome_pixmap_new_from_file (file_name));
-		gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0);		
-			gtk_widget_show(temp_widget);
-			g_free (file_name);
+                temp_widget = nautilus_image_new (file_name);
+		gtk_box_pack_start (GTK_BOX(temp_box), temp_widget, 0, 0, 0);		
+		gtk_widget_show (temp_widget);
+		g_free (file_name);
 		
 		temp_widget = nautilus_label_new (temp_text);
-		nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
+		nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 2);
 		g_free(temp_text);
 		gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0 );			
 		gtk_widget_show (temp_widget);
@@ -429,16 +439,16 @@ static void setup_overview_form (NautilusHardwareView *view)
 	gtk_widget_show (temp_box);
 
  	file_name = nautilus_pixmap_file ("memory_chip.gif");
-  	temp_widget = GTK_WIDGET (gnome_pixmap_new_from_file (file_name));
+  	temp_widget = nautilus_image_new (file_name);
 	gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0);		
   	gtk_widget_show(temp_widget);
   	g_free (file_name);
 	
 	temp_text = get_RAM_description ();
 	temp_widget = nautilus_label_new (temp_text);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
-	g_free(temp_text);
-	gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0 );			
+	nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 2);
+	g_free (temp_text);
+	gtk_box_pack_start (GTK_BOX(temp_box), temp_widget, 0, 0, 0 );			
  	gtk_widget_show (temp_widget);
 	
         /* Set up ide devices : by Shane Butler <shane_b@bigfoot.com> */
@@ -467,7 +477,7 @@ static void setup_overview_form (NautilusHardwareView *view)
                                         file_name = nautilus_pixmap_file("i-harddisk.png");
                                 }
                                 
-				pixmap_widget = GTK_WIDGET (gnome_pixmap_new_from_file(file_name));
+				pixmap_widget = nautilus_image_new (file_name);
 				gtk_box_pack_start (GTK_BOX(temp_box), pixmap_widget, 0, 0, 0);
 				gtk_widget_show(pixmap_widget);
 				g_free(file_name);
@@ -475,8 +485,8 @@ static void setup_overview_form (NautilusHardwareView *view)
                                 
 				temp_text = get_IDE_description (device);
 				temp_widget = nautilus_label_new (temp_text);
-				nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
-				nautilus_label_set_text_justification (NAUTILUS_LABEL (temp_widget), GTK_JUSTIFY_CENTER);
+				nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 2);
+				nautilus_label_set_justify (NAUTILUS_LABEL (temp_widget), GTK_JUSTIFY_CENTER);
 
 				g_free(temp_text);
                                 gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0);
@@ -489,9 +499,12 @@ static void setup_overview_form (NautilusHardwareView *view)
         }
 }
 
+#ifdef ENABLE_SUBVIEWS
+
 /* set up the widgetry for the CPU page */
 
-static void setup_CPU_form(NautilusHardwareView *view)
+static void
+setup_CPU_form(NautilusHardwareView *view)
 {
 	char *message;
 	GtkWidget *temp_widget;
@@ -506,8 +519,8 @@ static void setup_CPU_form(NautilusHardwareView *view)
 	
 	message = "This is a placeholder for the CPU page.";
 	temp_widget = nautilus_label_new (message);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
- 	gtk_label_set_line_wrap(GTK_LABEL(temp_widget), TRUE);
+	nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 2);
+ 	nautilus_label_set_wrap(NAUTILUS_LABEL(temp_widget), TRUE);
 	
 	gtk_box_pack_start(GTK_BOX(view->details->form), temp_widget, 0, 0, 12);			
  	gtk_widget_show (temp_widget);
@@ -515,7 +528,8 @@ static void setup_CPU_form(NautilusHardwareView *view)
 
 /* set up the widgetry for the RAM page */
 
-static void setup_RAM_form(NautilusHardwareView *view)
+static void
+setup_RAM_form(NautilusHardwareView *view)
 {
 	char *message;
 	GtkWidget *temp_widget;
@@ -530,8 +544,8 @@ static void setup_RAM_form(NautilusHardwareView *view)
 	
 	message = "This is a placeholder for the RAM page.";
 	temp_widget = nautilus_label_new (message);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
- 	gtk_label_set_line_wrap(GTK_LABEL(temp_widget), TRUE);
+	nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 2);
+ 	nautilus_label_set_wrap(NAUTILUS_LABEL(temp_widget), TRUE);
 	
 	gtk_box_pack_start(GTK_BOX(view->details->form), temp_widget, 0, 0, 12);			
  	gtk_widget_show (temp_widget);
@@ -539,7 +553,8 @@ static void setup_RAM_form(NautilusHardwareView *view)
 
 /* set up the widgetry for the IDE page */
 
-static void setup_IDE_form(NautilusHardwareView *view)
+static void
+setup_IDE_form(NautilusHardwareView *view)
 {
         char *message;
         GtkWidget *temp_widget;
@@ -554,8 +569,8 @@ static void setup_IDE_form(NautilusHardwareView *view)
         
         message = "This is a placeholder for the IDE page.";
         temp_widget = nautilus_label_new (message);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
-        gtk_label_set_line_wrap(GTK_LABEL(temp_widget), TRUE);
+	nautilus_label_make_larger (NAUTILUS_LABEL (temp_widget), 2);
+        nautilus_label_set_wrap(NAUTILUS_LABEL(temp_widget), TRUE);
         
         gtk_box_pack_start(GTK_BOX(view->details->form), temp_widget, 0, 0, 12);            
         gtk_widget_show (temp_widget);
@@ -563,43 +578,48 @@ static void setup_IDE_form(NautilusHardwareView *view)
 
 
 /* utility for checking uri */
-static gboolean is_location(char *document_str, const char *place_str)
+static gboolean
+is_location (const char *document_str, const char *place_str)
 {
-	return document_str && !strncmp(document_str + 1, place_str, strlen(place_str));
+	return document_str && strncmp(document_str + 1, place_str, strlen (place_str)) == 0;
 }
 
+#endif /* ENABLE_SUBVIEWS */
+
 /* load the uri by casing out on the path */
 
 void
 nautilus_hardware_view_load_uri (NautilusHardwareView *view, const char *uri)
 {
-	char *document_name;
+#ifdef ENABLE_SUBVIEWS
+	const char *document_name;
+#endif
 	
-	/* dispose of the old uri and copy the new one */
-	g_free (view->details->uri);
-	view->details->uri = g_strdup (uri);
-
 	/* dispose of any old form that was installed */
 	if (view->details->form != NULL) {
-		gtk_widget_destroy(view->details->form);
+		gtk_widget_destroy (view->details->form);
 		view->details->form = NULL;	
 	}
 		
+#ifndef ENABLE_SUBVIEWS
+        setup_overview_form (view);
+#else
 	/* extract the document part of the uri */
-	document_name = strchr(uri, ':');
+	document_name = strchr (uri, ':');
 	
 	/* load the appropriate form, based on the uri and the registration state */
-	if (is_location(document_name, "overview")) {
-		setup_overview_form(view);
-	} else if (is_location(document_name, "CPU")) {
-		setup_CPU_form(view);
-	} else if (is_location(document_name, "RAM")) {
-		setup_RAM_form(view);
-        } else if (is_location(document_name, "IDE")) {
-                setup_IDE_form(view);
+	if (is_location (document_name, "overview")) {
+		setup_overview_form (view);
+	} else if (is_location (document_name, "CPU")) {
+		setup_CPU_form (view);
+	} else if (is_location (document_name, "RAM")) {
+		setup_RAM_form (view);
+        } else if (is_location (document_name, "IDE")) {
+                setup_IDE_form (view);
 	} else {
-		setup_overview_form(view); /* if we don't understand it, go to the overview */
+		setup_overview_form (view); /* if we don't understand it, go to the overview */
         }
+#endif
 }
 
 static void
@@ -622,12 +642,6 @@ nautilus_hardware_view_drag_data_received (GtkWidget *widget, GdkDragContext *co
 	g_return_if_fail (NAUTILUS_IS_HARDWARE_VIEW (widget));
 
 	switch (info) {
-        case TARGET_GNOME_URI_LIST:
-        case TARGET_URI_LIST: 	
-                g_message ("dropped data on hardware_view: %s", selection_data->data); 			
-                break;
-  		
-                
         case TARGET_COLOR:
                 /* Let the background change based on the dropped color. */
                 nautilus_background_receive_dropped_color (nautilus_get_widget_background (widget),
