@@ -14,6 +14,26 @@
 
 /* Generic Function. Used by toc_elements and sect_preparse */
 
+void
+glossentry_stack_add (Context *context,
+		   const char *name,
+		   const xmlChar **atrs)
+{
+	/* This function adds a glosssentry id to the context->glossentry_stack */
+	char **atrs_ptr;
+	
+	atrs_ptr = (char **) atrs;
+	while (atrs_ptr &&  *atrs_ptr) {
+		if (g_strcasecmp (*atrs_ptr, "id") == 0) {
+			atrs_ptr++;
+			context->glossentry_stack =
+				g_list_prepend (context->glossentry_stack, g_strdup (*atrs_ptr));
+			break;
+		}
+		atrs_ptr += 2;
+	}
+}
+
 void
 sect1id_stack_add (Context *context,
 		   const char *name,
@@ -43,6 +63,13 @@ article_start_element (Context *context, const char *name, const xmlChar **atrs)
 	context->doctype = ARTICLE_DOC;
 }
 
+void
+glossary_start_element (Context *context, const char *name, const xmlChar **atrs)
+{
+	g_print ("<HTML>\n");
+	context->doctype = GLOSSARY_DOC;
+}
+
 void
 book_start_element (Context *context, const char *name, const xmlChar **atrs)
 {
@@ -160,8 +187,8 @@ find_first_element (Context *context, GSList *args)
 
 	for (ptr = context->stack; ptr; ptr = ptr->next) {
 		for (element_ptr = args; element_ptr; element_ptr = element_ptr->next) {
-			if (((StackElement*) ptr->data)->info &&
-			    ((StackElement*) ptr->data)->info->index == GPOINTER_TO_INT (element_ptr->data))
+			if (((StackElement *) ptr->data)->info &&
+			    ((StackElement *) ptr->data)->info->index == GPOINTER_TO_UINT (element_ptr->data))
 				return (StackElement *) ptr->data;
 		}
 	}
@@ -196,6 +223,105 @@ find_element_info (ElementInfo *elements,
 
 	return NULL;
 }
+/* new init_entities function and data structure
+   to handle i18n entities */ 
+struct gdb_xmlPredefinedEntityValue {
+     const char *name;
+     const char *value;
+};
+
+struct gdb_xmlPredefinedEntityValue gdbXmlPredefinedEntityValues[] = {
+/* i18 entities from iso-latin1 character set */
+     { "aacute", "&#x00E1;" },
+     { "Aacute", "&#x00C1;" },
+     { "acirc", "&#x00E2;" },
+     { "Acirc", "&#x00C2;" },
+     { "agrave", "&#x00E0;" },
+     { "Agrave", "&#x00C0;" },
+     { "aring", "&#x00E5;" },
+     { "Aring", "&#x00C5;" },
+     { "atilde", "&#x00E3;" },
+     { "Atilde", "&#x00C3;" },
+     { "auml", "&#x00E4;" },
+     { "Auml", "&#x00C4;" },
+     { "aelig", "&#x00E6;" },
+     { "AElig", "&#x00C6;" },
+     { "ccedil", "&#x00E7;" },
+     { "Ccedil", "&#x00C7;" },
+     { "eth", "&#x00D0;" },
+     { "ETH", "&#x00F0;" },
+     { "eacute", "&#x00E9;" },
+     { "Eacute", "&#x00C9;" },
+     { "ecirc", "&#x00EA;" },
+     { "Ecirc", "&#x00CA;" },
+     { "egrave", "&#x00E8;" },
+     { "Egrave", "&#x00C8;" },
+     { "euml", "&#x00EB;" },
+     { "Euml", "&#x00CB;" },
+     { "iacute", "&#x00ED;" },
+     { "Iacute", "&#x00CD;" },
+     { "icirc", "&#x00EE;" },
+     { "Icirc", "&#x00CE;" },
+     { "igrave", "&#x00EC;" },
+     { "Igrave", "&#x00CC;" },
+     { "iuml", "&#x00EF;" },
+     { "Iuml", "&#x00CF;" },
+     { "ntilde", "&#x00F1;" },
+     { "Ntilde", "&#x00D1;" },
+     { "oacute", "&#x00F3;" },
+     { "Oacute", "&#x00D3;" },
+     { "ocirc", "&#x00F4;" },
+     { "Ocirc", "&#x00D4;" },
+     { "ograve", "&#x00F2;" },
+     { "Ograve", "&#x00D2;" },
+     { "oslash", "&#x2298;" },
+     { "Oslash", "&#x00D8;" },
+     { "otilde", "&#x00F5;" },
+     { "Otilde", "&#x00D5;" },
+     { "ouml", "&#x00F6;" },
+     { "Ouml", "&#x00D6;" },
+     { "szlig", "&#x00DF;" },
+     { "thorn", "&#x00FE;" },
+     { "THORN", "&#x00DE;" },
+     { "uacute", "&#x00FA;" },
+     { "Uacute", "&#x00DA;" },
+     { "ucirc", "&#x00FB;" },
+     { "Ucirc", "&#x00DB;" },
+     { "ugrave", "&#x00F9;" },
+     { "Ugrave", "&#x00D9;" },
+     { "uuml", "&#x00FC;" },
+     { "Uuml", "&#x00DC;" },
+     { "yacute", "&#x00FD;" },
+     { "Yacute", "&#x00DD;" },
+     { "yuml", "&#x00FF;" },
+/* other miscellaneous entities we need */
+     { "mdash", "&mdash;" },
+     { "hellip", "&hellip;" },
+     { "percnt", "&#37;" }
+};
+
+static void
+init_entities (xmlDocPtr doc) {
+    unsigned int i;
+    xmlChar name[50];
+    xmlChar value[50];
+    const char *in;
+    xmlChar *out;
+
+    for (i = 0;i < sizeof(gdbXmlPredefinedEntityValues) / 
+                   sizeof(gdbXmlPredefinedEntityValues[0]);i++) {
+        in = gdbXmlPredefinedEntityValues[i].name;
+	out = &name[0];
+	for (;(*out++ = (xmlChar) *in);)in++;
+        in = gdbXmlPredefinedEntityValues[i].value;
+	out = &value[0];
+	for (;(*out++ = (xmlChar) *in);)in++;
+        xmlAddDocEntity(doc, &name[0],
+	             XML_INTERNAL_PREDEFINED_ENTITY, NULL, NULL,
+		     &value[0]);
+    }
+}
+
 
 /* our callbacks for the xmlSAXHandler */
 
@@ -408,6 +534,10 @@ gdb3html_internalSubset (Context *context, const xmlChar *name,
 	ctxt = context->ParserCtxt;
 	
 	xmlCreateIntSubset (ctxt->myDoc, name, ExternalID, SystemID);
+	/* Initialize our "custom" entities - This is a libxml 1.x SAX hack */
+	/* FIXME: Remove the below when we use DOM and/or libxml 2.x */
+	init_entities (ctxt->myDoc);
+
 	if (((ExternalID != NULL) || (SystemID != NULL)) &&
 	    (ctxt->validate && ctxt->wellFormed && ctxt->myDoc)) {
 		xmlDtdPtr ret = NULL;
@@ -564,42 +694,6 @@ static xmlSAXHandler parser = {
 	(cdataBlockSAXFunc) cdata_block
 };
 
-static xmlDocPtr
-xml_parse_document (gchar *filename)
-{
-	/* This function is ripped from parser.c in libxml but slightly
-	 * modified so as not to spew debug warnings all around */
-	xmlDocPtr ret;
-	xmlParserCtxtPtr ctxt;
-	char *directory;
-
-	directory = NULL;
-
-	ctxt = xmlCreateFileParserCtxt(filename);
-	if (ctxt == NULL) {
-		return (NULL);
-	}
-	ctxt->sax = NULL; /* This line specifically stops the warnings */
-
-	if ((ctxt->directory == NULL) && (directory == NULL))
-		directory = xmlParserGetDirectory (filename);
-	if ((ctxt->directory == NULL) && (directory != NULL))
-		ctxt->directory = (char *) xmlStrdup ((xmlChar *) directory);
-
-	xmlParseDocument (ctxt);
-
-	if (ctxt->wellFormed) {
-		ret = ctxt->myDoc;
-	} else {
-		ret = NULL;
-		xmlFreeDoc (ctxt->myDoc);
-		ctxt->myDoc = NULL;
-	}
-	xmlFreeParserCtxt (ctxt);
-	
-	return (ret);
-}
-
 void
 print_footer (const char *prev, const char *home, const char *next)
 {
@@ -610,21 +704,27 @@ print_footer (const char *prev, const char *home, const char *next)
 	if (prev == NULL) {
 		g_print ("&nbsp;");
 	} else {
-		g_print ("<A HREF=\"%s\">&#60;&#60;&#60; Previous</A>", prev);
+		g_print ("<A HREF=\"%s\">&#60;&#60;&#60; ", prev);
+		g_print (_("Previous"));
+		g_print ("</A>");
 	}
 
 	g_print ("</TD>\n<TD WIDTH=\"34%%\" ALIGN=\"CENTER\" VALIGN=\"TOP\">");
 	if (home == NULL) {
 		g_print ("&nbsp;");
 	} else {
-		g_print ("<A HREF=\"%s\">Home</A>", home);
+		g_print ("<A HREF=\"%s\">", home);
+		g_print (_("Home"));
+		g_print ("</A>");
 	}
 
 	g_print ("</TD>\n<TD WIDTH=\"33%%\" ALIGN=\"RIGHT\" VALIGN=\"TOP\">");
 	if (next == NULL) {
 		g_print ("&nbsp;");
 	} else {
-		g_print ("<A HREF=\"%s\">Next &#62;&#62;&#62;</A>", next);
+		g_print ("<A HREF=\"%s\">", next);
+		g_print (_("Next"));
+		g_print (" &#62;&#62;&#62;</A>");
 	}
 
 	g_print ("</TD>\n</TR></TABLE>\n");
@@ -690,11 +790,7 @@ parse_file (gchar *filename, gchar *section, char *arg)
 	xmlInitParserCtxt (context->ParserCtxt);
 	context->ParserCtxt->sax = &parser;
 	context->ParserCtxt->validate = 1;
-	/* FIXME bugzilla.eazel.com 2399: 
-	 * Is the below correct? version needs to be set so as not to
-	 * segfault in startDocument (in SAX.h) */
-	context->ParserCtxt->version = xmlStrdup ("1.0"); 
-	context->ParserCtxt->myDoc = xml_parse_document (filename);
+	context->ParserCtxt->version = xmlStrdup (XML_DEFAULT_VERSION); 
 	xmlSubstituteEntitiesDefault (1);
 
 	dirpointer = strrchr (arg, '/');
@@ -742,9 +838,17 @@ parse_file (gchar *filename, gchar *section, char *arg)
 			print_footer (NULL, NULL, temp_uri);
 			g_free (temp_uri);
 		}
+		toc_free_data (context->data);
 	}
 
+	/* Set this to NULL so xmlFreeParserCtxt does not try to free this static memory */
+	context->ParserCtxt->sax = NULL;
+	if (context->ParserCtxt->myDoc) {
+		xmlFreeDoc (context->ParserCtxt->myDoc);
+	}
+	xmlFreeParserCtxt (context->ParserCtxt);
 	g_free (context->base_path);
+	g_free (context);
 }
 
 int
@@ -755,7 +859,6 @@ main (int argc, char *argv[])
 	gchar *ptr2;
 
 	if (argc != 2) {
-		/* It is '?SECTIONID' not '#SECTIONID' */
 		g_print ("Usage:  gnome-db2html2 FILE[?SECTIONID]\n\n");
 		return 0;
 	}
