@@ -26,7 +26,7 @@ ElementInfo sect_elements[] = {
 	{ AFFILIATION, "affiliation", NULL, NULL, NULL},
 	{ EMAIL, "email", NULL, NULL, (charactersSAXFunc) sect_email_characters },
 	{ ORGNAME, "orgname", NULL, NULL, (charactersSAXFunc) sect_author_characters },
-	{ ADDRESS, "address", NULL, NULL, (charactersSAXFunc) sect_write_characters},
+	{ ADDRESS, "address", (startElementSAXFunc) sect_address_start_element, (endElementSAXFunc) sect_address_end_element, (charactersSAXFunc) sect_address_characters},
 	{ COPYRIGHT, "copyright", NULL, NULL, NULL},
 	{ YEAR, "year", NULL, NULL, (charactersSAXFunc) sect_copyright_characters},
 	{ HOLDER, "holder", NULL, NULL, (charactersSAXFunc) sect_copyright_characters},
@@ -112,7 +112,25 @@ ElementInfo sect_elements[] = {
 	{ ANSWER, "answer", (startElementSAXFunc) sect_answer_start_element, (endElementSAXFunc) sect_formalpara_end_element, NULL /* charactersSAXFunc) sect_write_characters */},
 	{ CHAPTER, "chapter", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
 	{ PREFACE, "preface", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
+	{ TERM, "term", NULL, NULL, (charactersSAXFunc) sect_write_characters},
 	{ APPENDIX, "appendix", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
+	{ DOCINFO, "docinfo", NULL, NULL, NULL},
+	{ GLOSSARY, "glossary", (startElementSAXFunc) glossary_start_element, NULL, NULL},
+	{ GLOSSDIV, "glossdiv", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
+	{ GLOSSENTRY, "glossentry", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
+	{ GLOSSTERM, "glossterm", (startElementSAXFunc) sect_title_start_element, (endElementSAXFunc) sect_title_end_element, (charactersSAXFunc) sect_title_characters },
+	{ GLOSSSEE, "glosssee", (startElementSAXFunc) sect_glosssee_start_element, NULL,  NULL },
+	{ GLOSSSEEALSO, "glossseealso", (startElementSAXFunc) sect_glossseealso_start_element, NULL, NULL },
+	{ EXAMPLE, "example", NULL, NULL, NULL },
+	{ VARLISTENTRY, "varlistentry", (startElementSAXFunc) sect_varlistentry_start_element, (endElementSAXFunc) sect_varlistentry_end_element, NULL },
+	{ STREET, "street", (startElementSAXFunc) sect_street_start_element, NULL, (charactersSAXFunc) sect_write_characters },
+	{ CITY, "city", (startElementSAXFunc) sect_city_start_element, NULL, (charactersSAXFunc) sect_write_characters },
+	{ COUNTRY, "country", (startElementSAXFunc) sect_country_start_element, NULL, (charactersSAXFunc) sect_write_characters },
+	{ STATE, "state", NULL, NULL, (charactersSAXFunc) sect_write_characters },
+	{ POSTCODE, "postcode", NULL, NULL, (charactersSAXFunc) sect_write_characters },
+	{ LITERALLAYOUT, "literallayout", (startElementSAXFunc) sect_literallayout_start_element, (endElementSAXFunc) sect_literallayout_end_element, (charactersSAXFunc) sect_write_characters },
+	{ QANDAENTRY, "qandaentry", NULL, NULL, NULL, },
+	{ QANDASET, "qandaset", NULL, NULL, NULL, },
 	{ UNDEFINED, NULL, NULL, NULL, NULL}
 };
 
@@ -233,8 +251,7 @@ sect_article_end_element (Context *context, const gchar *name)
 		GSList *list;
 		gint i = 1;
 		
-		g_print ("<HR>");
-		g_print ("<H4>Notes:</H4>");
+		g_print ("<HR><H4>%s</H4>", _("Notes"));
 		g_print ("<TABLE BORDER=\"0\" WIDTH=\"100%%\">\n\n");
 		for (list = context->footnotes; list; list = list->next) {
 			g_print ("<TR><TD ALIGN=\"LEFT\" VALIGN=\"TOP\" WIDTH=\"5%%\">\n");
@@ -244,7 +261,7 @@ sect_article_end_element (Context *context, const gchar *name)
 		}
 		g_print ("</TABLE>");
 	}
-	/* FIXME: Is the below needed now that we have the footer printed out */
+	/* FIXME bugzilla.eazel.com 4408: Is the below needed now that we have the footer printed out */
 /*	sect_print (context, "</BODY>\n</HEAD>\n"); */
 }
 
@@ -555,6 +572,7 @@ sect_author_characters (Context *context, const gchar *chars, int len)
 	case ARTHEADER:
 		break;
 	default:
+		sect_write_characters (context, chars, len);
 		return;
 	};
 
@@ -643,6 +661,8 @@ sect_copyright_characters (Context *context,
 	};
 }
 
+static gboolean is_in_sect_title = FALSE;
+
 void
 sect_title_start_element (Context *context,
 			  const gchar *name,
@@ -652,6 +672,7 @@ sect_title_start_element (Context *context,
 	StackElement *stack_el;
 	gchar **atrs_ptr;
 	SectContext *sect_context;
+	char *table_msg;
 
 	sect_context = (SectContext *) context->data;
 	if (!IS_IN_SECT (context)) 
@@ -670,6 +691,8 @@ sect_title_start_element (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSENTRY));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (EXAMPLE));
 	stack_el = find_first_element (context, element_list);
 
 	g_slist_free (element_list);
@@ -678,7 +701,10 @@ sect_title_start_element (Context *context,
 
 	switch (stack_el->info->index) {
 	case TABLE:
-		sect_print (context, "<P><B>Table %d. ", sect_context->table_count); 
+		table_msg = g_strdup_printf (_("Table %d."), sect_context->table_count);
+		sect_print (context, "<P><B>");
+		sect_print (context, "%s ", table_msg); 
+		g_free (table_msg);
 		break;
 	case PREFACE:
 	case CHAPTER:
@@ -696,6 +722,7 @@ sect_title_start_element (Context *context,
 		else
 			sect_print (context, "<H3>");
 		sect_print (context, "<A name=\"");
+		is_in_sect_title = TRUE;
 
 		atrs_ptr = (stack_el->atrs);
 		while (atrs_ptr && *atrs_ptr) {
@@ -711,6 +738,12 @@ sect_title_start_element (Context *context,
 	case FORMALPARA:
 		sect_print (context, "<B>");
 		break;
+	case GLOSSENTRY:
+		sect_print (context, "<B><H2>");
+		break;
+	case EXAMPLE:
+		sect_print (context, "<H3>");
+		break;
 	default:
 		break;
 	};
@@ -740,6 +773,8 @@ sect_title_end_element (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSENTRY));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (EXAMPLE));
 
 	index = find_first_parent (context, element_list);
 
@@ -752,23 +787,35 @@ sect_title_end_element (Context *context,
 	        sect_print (context, "</A></H1>\n");
 		break;
 	case CHAPTER:
-		/* FIXME: This sometimes causes the output of invalid HTML because in sect_title_characters
-		 * we could switch states (from LOOKING_FOR_SECT_TITLE to IN_SECT */
-		sect_print (context, "</A></H1>\n");
+		if (is_in_sect_title == TRUE) {
+			sect_print (context, "</A></H1>\n");
+			is_in_sect_title = FALSE;
+		}
 		break;
 	case SECT1:
 	case SECTION:
-		sect_print (context, "</A></H2>\n");
+		if (is_in_sect_title == TRUE) {
+			sect_print (context, "</A></H2>\n");
+			is_in_sect_title = FALSE;
+		}
 		break;
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
-		sect_print (context, "</A></H3>\n");
+		if (is_in_sect_title == TRUE) {
+			sect_print (context, "</A></H3>\n");
+			is_in_sect_title = FALSE;
+		}
 		break;
 	case FORMALPARA:
 		sect_print (context, ".</B>");
 		break;
+	case GLOSSENTRY:
+	        sect_print (context, "</H2></B>");
+		break;
+	case EXAMPLE:
+		sect_print (context, "</H3>");
 	default:
 		break;
 	};
@@ -799,6 +846,7 @@ sect_title_characters (Context *context,
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
+		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSENTRY));
 
 		stack_el = find_first_element (context, element_list);
 
@@ -806,9 +854,11 @@ sect_title_characters (Context *context,
 
 		sect_print (context, "<HEAD>\n<TITLE>%s</TITLE>\n</HEAD>\n", temp);
 		sect_print (context, "<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#840084\" ALINK=\"#0000FF\">\n");
-		if (stack_el == NULL)
-			sect_print (context, "<A href=\"gnome-help:%s\"><font size=3>Up to Table of Contents</font></A><BR>\n",
-				 context->base_file);
+		if (stack_el == NULL) {
+			sect_print (context, "<A href=\"gnome-help:%s\"><font size=3>", context->base_file);
+			sect_print (context, _("Up to Table of Contents"));
+			sect_print (context, "</font></A><BR>\n");
+		}
 #if 0
 		else
 			sect_print (context, "<A href=\"gnome-help:%s#%s\"><font size=3>Up to %s</font></A><BR>\n",
@@ -841,6 +891,7 @@ sect_title_characters (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSENTRY));
 
 	index = find_first_parent (context, element_list);
 
@@ -855,6 +906,7 @@ sect_title_characters (Context *context,
 	case APPENDIX:
 	case SECTION:
 	case FORMALPARA:
+	case GLOSSENTRY:
 		sect_print (context, temp);
 		g_free (temp);
 		break;
@@ -950,7 +1002,11 @@ sect_xref_start_element (Context *context,
 	}
 
 	if (fignum_from_figure_id != NULL) {
-		sect_print (context, "Figure %s", fignum_from_figure_id);
+		char *output_msg;
+
+		output_msg = g_strdup_printf (_("Figure %s"), fignum_from_figure_id);
+		sect_print (context, "%s", output_msg);
+		g_free (output_msg);
 		return;
 	}
 	
@@ -968,9 +1024,9 @@ sect_xref_start_element (Context *context,
 		title = g_hash_table_lookup (((SectContext *)context->data)->title_hash, *atrs_ptr);
 
 	if (title == NULL) {
-			sect_print (context, "\">the section here</A>");
+		sect_print (context, "\">%s</A>", _("the section here"));
 	} else {
-			sect_print (context, "\">the section <EM>%s</EM></A>", title);
+		sect_print (context, "\">%s <EM>%s</EM></A>", _("the section"), title);
 	}
 }
 
@@ -996,6 +1052,9 @@ sect_figure_start_element (Context *context,
 {
 	SectContext *sect_context = (SectContext *)context->data;
 
+	/* Increment figure count, before checking for IS_IN_SECT */
+	sect_context->figure_count++;
+
 	if (!IS_IN_SECT (context))
 		return;
 
@@ -1003,7 +1062,6 @@ sect_figure_start_element (Context *context,
 		return;
 
 	sect_context->figure = g_new0 (FigureInfo, 1);
-	sect_context->figure_count ++;
 }
 
 void
@@ -1019,11 +1077,14 @@ sect_figure_end_element (Context *context,
 		return;
 
 	if (sect_context->figure->img != NULL) {
+		char *output_msg;
+		
  		if (sect_context->figure->id)
 			sect_print (context, "<A NAME=\"%s\">",
 				    sect_context->figure->id);
-		sect_print (context, "<P><B>Figure %d.",
-			    sect_context->figure_count);
+		output_msg = g_strdup_printf (_("Figure %d"), sect_context->figure_count);
+		sect_print (context, "<P><B>%s.", output_msg);
+		g_free (output_msg);
  		if (sect_context->figure->title)
 			sect_print (context, " %s",
 				    sect_context->figure->title);
@@ -1033,7 +1094,7 @@ sect_figure_end_element (Context *context,
 		sect_print (context, "<IMG SRC=\"file://%s%s\" ALT=\"%s\"><P>",
 			    context->base_path,
 			    sect_context->figure->img,
-			    (sect_context->figure->alt) ? sect_context->figure->alt : "IMAGE");
+			    (sect_context->figure->alt) ? sect_context->figure->alt : _("IMAGE"));
 	}
 	g_free (sect_context->figure->title);
 	g_free (sect_context->figure->alt);
@@ -1078,7 +1139,7 @@ sect_inlinegraphic_start_element (Context *context,
 		return;
 	}
 
-	/* FIXME: Should we put an 'ALT' tag here? */	
+	/* FIXME bugzilla.eazel.com 4407: Should we put an 'ALT' tag here? */	
 	if (format == NULL) {
 		/* Default is PNG */
 		sect_print (context, "<IMG SRC=\"file://%s%s.png\">", context->base_path, fileref);
@@ -1403,20 +1464,58 @@ sect_listitem_start_element (Context *context,
 			     const gchar *name,
 			     const xmlChar **atrs)
 {
+	GSList *element_list = NULL;
+	StackElement *stack_el;
+
 	if (!IS_IN_SECT (context))
 		return;
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ITEMIZEDLIST));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ORDEREDLIST));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (VARIABLELIST));
+	stack_el = find_first_element (context, element_list);
 
-	sect_print (context, "<LI>");
+	g_slist_free (element_list);
+	if (stack_el == NULL)
+		return;
+
+	switch (stack_el->info->index) {
+	case VARIABLELIST:
+		return;
+	case ITEMIZEDLIST:
+	case ORDEREDLIST:
+		sect_print (context, "<LI>");
+	default:
+		break;
+	};
 }
 
 void
 sect_listitem_end_element (Context *context,
 			   const gchar *name)
 {
+	GSList *element_list = NULL;
+	StackElement *stack_el;
+
 	if (!IS_IN_SECT (context))
 		return;
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ITEMIZEDLIST));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ORDEREDLIST));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (VARIABLELIST));
+	stack_el = find_first_element (context, element_list);
 
-	sect_print (context, "<P>");
+	g_slist_free (element_list);
+	if (stack_el == NULL)
+		return;
+
+	switch (stack_el->info->index) {
+	case VARIABLELIST:
+		return;
+	case ITEMIZEDLIST:
+	case ORDEREDLIST:
+		sect_print (context, "</LI>");
+	default:
+		break;
+	};
 }
 
 void
@@ -1681,8 +1780,9 @@ sect_informaltable_start_element (Context *context,
 	if (!IS_IN_SECT (context))
 		return;
 
-	sect_print (context, "<TABLE BORDER=\"0\" BGCOLOR=\"#E0E0E0\"
-                           CELLSPACING=\"0\" CELLPADDING=\"4\">\n");
+	sect_print (context,
+		    "<TABLE BORDER=\"0\" BGCOLOR=\"#E0E0E0\" "
+		    "CELLSPACING=\"0\" CELLPADDING=\"4\">\n");
 }
 
 void
@@ -1960,9 +2060,9 @@ sect_legalnotice_start_element (Context *context,
 	}
 
 	sect_context->state = IN_SECT;
-	sect_print (context, "<HEAD>\n<TITLE>Legal Notice</TITLE>\n</HEAD>\n");
+	sect_print (context, "<HEAD>\n<TITLE>%s</TITLE>\n</HEAD>\n", _("Legal Notice"));
 	sect_print (context, "<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#840084\" ALINK=\"#0000FF\">\n");
-	sect_print (context, "<H1>Legal Notice</H1>\n");
+	sect_print (context, "<H1>%s</H1>\n", _("Legal Notice"));
 
 }
 
@@ -2007,7 +2107,8 @@ sect_question_start_element (Context *context,
       if (!IS_IN_SECT (context))
               return;
 
-      sect_print (context, "<P><B>Q: </B>");
+      /* This 'Q' is short for 'Question:' */
+      sect_print (context, "<P><B>%s: </B>", _("Q"));
 }
 
 void
@@ -2018,6 +2119,161 @@ sect_answer_start_element (Context *context,
       if (!IS_IN_SECT (context))
               return;
 
-      sect_print (context, "<P><B>A: </B>");
+      /* This 'A' is short for 'Answer' */
+      sect_print (context, "<P><B>%s: </B>", _("A"));
+}
+
+
+void
+sect_glosssee_start_element (Context *context, const gchar *name, const xmlChar **atrs)
+{
+	gchar **atrs_ptr;
+	gchar *temp;
+
+	if (!IS_IN_SECT (context))
+        	return;
+   
+	atrs_ptr = (gchar **) atrs;
+	while (atrs_ptr && *atrs_ptr) {
+		if (!g_strcasecmp (*atrs_ptr, "otherterm")) {
+			atrs_ptr++;
+			((StackElement *)context->stack->data)->atrs = g_new0 (gchar *, 3);
+			((StackElement *)context->stack->data)->atrs[0] = g_strdup ("otherterm");
+			((StackElement *)context->stack->data)->atrs[1] = g_strdup (*atrs_ptr);
+			       
+                        if (g_hash_table_lookup (context->glossary_data, *atrs_ptr)) {
+				temp = g_hash_table_lookup (context->glossary_data, *atrs_ptr);
+                        	g_print("<BR>%s: <A HREF=\"gnome-help:%s?%s\">%s</A>\n", _("See"), context->base_file, *atrs_ptr, temp);
+		        }
+			atrs_ptr += 2;
+		}
+	}
+}
+
+void
+sect_glossseealso_start_element (Context *context, const gchar *name, const xmlChar **atrs)
+{
+
+	gchar **atrs_ptr;
+	gchar *temp;
+ 
+	if (!IS_IN_SECT (context))
+		return;
+
+	atrs_ptr = (gchar **) atrs;
+	while (atrs_ptr && *atrs_ptr) {
+		if (!g_strcasecmp (*atrs_ptr, "otherterm")) {
+			atrs_ptr++;
+			((StackElement *)context->stack->data)->atrs = g_new0 (gchar *, 3);
+			((StackElement *)context->stack->data)->atrs[0] = g_strdup ("otherterm");
+			((StackElement *)context->stack->data)->atrs[1] = g_strdup (*atrs_ptr);
+			       
+                        if ( g_hash_table_lookup (context->glossary_data, *atrs_ptr)) {
+				temp = g_hash_table_lookup (context->glossary_data, *atrs_ptr);
+                        	g_print("<BR>%s: <A HREF=\"gnome-help:%s?%s\">%s</A>\n", _("See also"), context->base_file, *atrs_ptr, temp);
+		        }
+			atrs_ptr += 2;
+		}
+	}
+}
+
+void
+sect_varlistentry_start_element (Context *context,
+			     	 const gchar *name,
+			     	 const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<LI>");
+}
+
+void
+sect_varlistentry_end_element (Context *context,
+			       const gchar *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "</LI>");
+}
+
+void
+sect_address_characters (Context *context,
+			 const char *chars,
+			 int len)
+{
+	gchar *temp;
+
+	if (!IS_IN_SECT (context))
+		return;
+		
+	if (context->start_address_character == TRUE) {
+		g_print ("<BR>&nbsp;&nbsp;&nbsp;");
+		context->start_address_character = FALSE;
+	}
+	
+	temp = g_strndup (chars, len);
+	sect_print (context, temp);
+	g_free (temp);
+}
+void
+sect_address_start_element (Context *context,
+			    const gchar *name,
+			    const xmlChar **atrs)
+{
+	context->start_address_character = TRUE;
+}
+
+void
+sect_address_end_element (Context *context,
+			  const gchar *name)
+{
+	context->start_address_character = FALSE;
+}
+
+void
+sect_street_start_element (Context *context,
+			   const gchar *name,
+			   const xmlChar **atrs)
+{
+	sect_print (context, "<BR>&nbsp;&nbsp;&nbsp;");
+}
+
+void
+sect_city_start_element (Context *context,
+			 const gchar *name,
+			 const xmlChar **atrs)
+{
+	sect_print (context, "<BR>&nbsp;&nbsp;&nbsp;");
+}
+
+void
+sect_country_start_element (Context *context,
+			    const gchar *name,
+			    const xmlChar **atrs)
+{
+	sect_print (context, "<BR>&nbsp;&nbsp;&nbsp;");
+}
+
+void
+sect_literallayout_start_element (Context *context,
+			     	  const gchar *name,
+			     	  const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<PRE>");
+}
+
+void
+sect_literallayout_end_element (Context *context,
+			   	const gchar *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "</PRE>");
 }
 
