@@ -128,4 +128,17 @@ void sect_legalnotice_end_element (Context *context, const char *name);
 void sect_legalnotice_characters (Context *context, const char *chars, int len);
 void sect_question_start_element (Context *context, const char *name, const xmlChar **atrs);
 void sect_answer_start_element (Context *context, const char *name, const xmlChar **atrs);
+void sect_glosssee_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_glossseealso_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_glossee_end_element (Context *context, const gchar *name);
+void sect_varlistentry_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_varlistentry_end_element (Context *context, const gchar *name);
+void sect_address_characters (Context *context, const char *chars, int len);
+void sect_address_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_address_end_element (Context *context, const gchar *name);
+void sect_street_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_city_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_country_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_literallayout_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+void sect_literallayout_end_element (Context *context, const gchar *name);
 #endif
