@@ -8,6 +8,13 @@ static void sect_preparse_sect_start_element (Context *context, const gchar *nam
 static void sect_preparse_title_characters (Context *context, const gchar *chars, gint len);
 static void sect_preparse_figure_start_element (Context *context, const char *name, const xmlChar **atrs);
 static void sect_preparse_set_doctype (Context *context, const char *name, const xmlChar **atrs);
+static void sect_preparse_glossterm_start_element (Context *context, const char *name, const xmlChar **atrs);
+static void sect_preparse_glossterm_characters (Context *context, const gchar *chars, int len);
+static void sect_preparse_glossterm_end_element (Context *context, const char *name, const xmlChar **atrs);
+static void sect_preparse_glossentry_start_element (Context *context, const char *name, const xmlChar **atrs);
+static void sect_preparse_acronym_characters (Context *context, const gchar *chars, int len);
+static void gloss_term_append (Context *context, const gchar *chars, int len);
+
 
 ElementInfo sect_preparse[] = {
 	{ ARTICLE, "article", (startElementSAXFunc) sect_preparse_set_doctype, NULL, NULL},
@@ -99,7 +106,7 @@ ElementInfo sect_preparse[] = {
         { ENTRY, "entry", NULL, NULL, NULL},
         { THEAD, "thead", NULL, NULL, NULL},
         { TBODY, "tbody", NULL, NULL, NULL},
-        { ACRONYM, "acronym", NULL, NULL, NULL},
+        { ACRONYM, "acronym", NULL, NULL, (charactersSAXFunc) sect_preparse_acronym_characters},
         { MARKUP, "markup", NULL, NULL, NULL},
         { SIMPLELIST, "simplelist", NULL, NULL, NULL},
         { MEMBER, "member", NULL, NULL, NULL},
@@ -118,7 +125,25 @@ ElementInfo sect_preparse[] = {
         { ANSWER, "answer", NULL, NULL, NULL},
         { CHAPTER, "chapter", (startElementSAXFunc) sect_preparse_sect_start_element, NULL, NULL},
 	{ PREFACE, "preface", (startElementSAXFunc) sect_preparse_sect_start_element, NULL, NULL},
+	{ TERM, "term", NULL, NULL, NULL},
 	{ APPENDIX, "appendix", (startElementSAXFunc) sect_preparse_sect_start_element, NULL, NULL},
+	{ DOCINFO, "docinfo", NULL, NULL, NULL},
+	{ GLOSSARY, "glossary", (startElementSAXFunc) sect_preparse_set_doctype, NULL, NULL},
+	{ GLOSSDIV, "glossdiv", (startElementSAXFunc) sect_preparse_sect_start_element, NULL, NULL},
+	{ GLOSSENTRY, "glossentry", (startElementSAXFunc) sect_preparse_glossentry_start_element, NULL, NULL},
+	{ GLOSSTERM, "glossterm", (startElementSAXFunc) sect_preparse_glossterm_start_element, (endElementSAXFunc) sect_preparse_glossterm_end_element, (charactersSAXFunc) sect_preparse_glossterm_characters},
+	{ GLOSSSEE, "glosssee", NULL, NULL, NULL},
+	{ GLOSSSEEALSO, "glossseealso", NULL, NULL, NULL},
+	{ EXAMPLE, "example", NULL, NULL, NULL},
+	{ VARLISTENTRY, "varlistentry", NULL, NULL, NULL},
+	{ STREET, "street", NULL, NULL, NULL},
+	{ CITY, "city", NULL, NULL, NULL},
+	{ COUNTRY, "country", NULL, NULL, NULL},
+	{ STATE, "state", NULL, NULL, NULL},
+	{ POSTCODE, "postcode", NULL, NULL, NULL},
+	{ LITERALLAYOUT, "literallayout", NULL, NULL, NULL},
+	{ QANDAENTRY, "quandaentry", NULL, NULL, NULL},
+	{ QANDASET, "quandaset", NULL, NULL, NULL},
 	{ UNDEFINED, NULL, NULL, NULL, NULL}
 };
 
@@ -128,9 +153,11 @@ sect_preparse_set_doctype (Context *context,
 			   const xmlChar **atrs)
 {
 	if (g_strcasecmp (name, "article") == 0) {
-		context->doctype = ARTICLE;
+		context->doctype = ARTICLE_DOC;
 	} else if (g_strcasecmp (name, "book") == 0) {
-		context->doctype = BOOK;
+		context->doctype = BOOK_DOC;
+	} else if (g_strcasecmp (name, "glossary") == 0) {
+	        context->doctype = GLOSSARY_DOC;
 	}
 }
 
@@ -140,10 +167,36 @@ sect_preparse_sect_start_element (Context *context,
 				  const gchar *name,
 				  const xmlChar **atrs)
 {
+	GSList *element_list;
+	StackElement *stack_el;
 	gchar **atrs_ptr;
 
-	if (g_strcasecmp(name, "section") == 0) {
-		return;
+	element_list = NULL;
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (NOTE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TIP));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (WARNING));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GRAPHIC));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSDIV));
+	stack_el = find_first_element (context, element_list);
+
+	g_slist_free (element_list);
+
+	if (stack_el == NULL) {
+		if (g_strcasecmp(name, "section") == 0) {
+			return;
+		}
 	}
 
 
@@ -154,11 +207,17 @@ sect_preparse_sect_start_element (Context *context,
 	case 't':
 		sect1id_stack_add (context, name, atrs);
 		break;
+	case 's':
+		sect1id_stack_add (context, name, atrs);
+		break;
 	case '1':
-		if (context->doctype == ARTICLE_DOC) {
+		if (context->doctype == ARTICLE_DOC && stack_el->info->index != APPENDIX) {
 			sect1id_stack_add (context, name, atrs);
 		}
 		break;
+	case 'n':
+	        sect1id_stack_add (context, name, atrs);
+		break;
 	default:
 		break;
 	}
@@ -246,5 +305,91 @@ sect_preparse_figure_start_element (Context *context,
                 }
                 atrs_ptr += 2;
         }
+}
 
+static void
+sect_preparse_glossterm_characters (Context *context,
+				    const gchar *chars,
+				    int len)
+{
+	gloss_term_append (context, chars, len);
+}
+
+static void
+sect_preparse_acronym_characters (Context *context,
+				  const gchar *chars,
+				  int len)
+{
+	gloss_term_append (context, chars, len);
+}
+
+static void
+gloss_term_append (Context *context,
+		   const gchar *chars,
+		   int len)
+{
+        GList *temp_list;
+        char *temp_glossentry;
+	char *temp;
+	char *new_string;
+ 	char *origkey;
+ 	char *origdata;
+
+	if (context->in_glossterm == TRUE) {
+		temp = g_strndup (chars, len);
+		temp_glossentry = NULL;
+		/* get most recent entry to glossentry stack */
+		temp_list = g_list_first (context->glossentry_stack);
+
+		if (temp_list != NULL) {
+			temp_glossentry = g_strdup_printf ("%s", (char *)temp_list->data);
+		}
+		
+		/* check to see if glossary_data hash exists - if not create it */
+		if (context->glossary_data == NULL) {
+			context->glossary_data = g_hash_table_new (g_str_hash, g_str_equal);
+		}
+		/* check to see if an entry for temp_glossentry exists - if not create it */
+		if (g_hash_table_lookup (context->glossary_data, temp_glossentry) == NULL) {
+			/* The key is the 'glossentry id' - The data is the 'glossterm' */
+			g_hash_table_insert (context->glossary_data, g_strdup (temp_glossentry), temp);
+		} else {
+			/* An entry already exists. Add to it. */
+	 		if (g_hash_table_lookup_extended (context->glossary_data, temp_glossentry,
+ 					  (gpointer *)&origkey,
+ 					  (gpointer *)&origdata)) {
+ 			new_string = g_strconcat ((gchar *)origdata, temp, NULL );
+ 			g_free(origdata);
+ 			g_hash_table_remove(context->glossary_data, origkey);
+ 			g_hash_table_insert(context->glossary_data, origkey, new_string);
+ 			g_free(temp_glossentry);
+			} 
+		}
+	}
+}
+
+static void
+sect_preparse_glossentry_start_element (Context *context,
+				  	const gchar *name,
+				  	const xmlChar **atrs)
+{
+	glossentry_stack_add (context, name, atrs);
+}
+
+static void
+sect_preparse_glossterm_start_element (Context *context,
+				  	const gchar *name,
+				  	const xmlChar **atrs)
+{
+	context->in_glossterm = TRUE;
+	return;
+}
+
+static void
+sect_preparse_glossterm_end_element (Context *context,
+				  	const gchar *name,
+				  	const xmlChar **atrs)
+{
+	context->in_glossterm = FALSE;
+	return;
 }
