@@ -1,8 +1,10 @@
 
 #include <config.h>
+#include <gnome.h>
+#include "sect-elements.h"
 #include "toc-elements.h"
 
-
+static gboolean in_printed_title = FALSE;
 typedef struct _TocContext TocContext;
 struct _TocContext {
 	HeaderInfo *header;
@@ -17,7 +19,13 @@ static void toc_copyright_characters (Context *context, const gchar *chars, int
 static void toc_title_start_element (Context *context, const gchar *name, const xmlChar **atrs);
 static void toc_title_end_element (Context *context, const gchar *name);
 static void toc_title_characters (Context *context, const gchar *chars, int len);
-
+static void toc_glossdiv_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+static void toc_glossdiv_end_element (Context *context, const gchar *name);
+static void toc_glossentry_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+static void toc_glossentry_end_element (Context *context, const gchar *name);
+static void toc_glossterm_start_element (Context *context, const gchar *name, const xmlChar **atrs);
+static void toc_glossterm_end_element (Context *context, const gchar *name);
+static void toc_tag_characters (Context *context, const gchar *chars, int len);
 
 ElementInfo toc_elements[] = {
 	{ ARTICLE, "article", (startElementSAXFunc) article_start_element, NULL, NULL},
@@ -48,21 +56,21 @@ ElementInfo toc_elements[] = {
 	{ HOLDER, "holder", NULL, NULL, (charactersSAXFunc) toc_copyright_characters},
 	{ TITLE, "title", (startElementSAXFunc) toc_title_start_element, (endElementSAXFunc) toc_title_end_element, (charactersSAXFunc) toc_title_characters },
 	{ SUBTITLE, "subtitle", (startElementSAXFunc) toc_title_start_element, (endElementSAXFunc) toc_title_end_element, (charactersSAXFunc) toc_title_characters },
-	{ ULINK, "ulink", NULL, NULL, NULL},
+	{ ULINK, "ulink", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ XREF, "xref", NULL, NULL, NULL},
-	{ FOOTNOTE, "footnote", NULL, NULL, NULL},
+	{ FOOTNOTE, "footnote", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ FIGURE, "figure", NULL, NULL, NULL},
 	{ GRAPHIC, "graphic", NULL, NULL, NULL},
-	{ CITETITLE, "citetitle", NULL, NULL, NULL},
-	{ APPLICATION, "application", NULL, NULL, NULL},
-	{ FILENAME, "filename", NULL, NULL, NULL},
+	{ CITETITLE, "citetitle", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ APPLICATION, "application", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ FILENAME, "filename", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ ITEMIZEDLIST, "itemizedlist", NULL, NULL, NULL},
 	{ ORDEREDLIST, "orderedlist", NULL, NULL, NULL},
 	{ VARIABLELIST, "variablelist", NULL, NULL, NULL},
 	{ LISTITEM, "listitem", NULL, NULL, NULL},
 	{ PROGRAMLISTING, "programlisting", NULL, NULL, NULL},
-	{ SGMLTAG, "sgmltag", NULL, NULL, NULL},
-	{ EMPHASIS, "emphasis", NULL, NULL, NULL},
+	{ SGMLTAG, "sgmltag", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ EMPHASIS, "emphasis", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ TIP, "tip", NULL, NULL, NULL},
 	{ WARNING, "warning", NULL, NULL, NULL},
 	{ IMPORTANT, "important", NULL, NULL, NULL},
@@ -71,64 +79,82 @@ ElementInfo toc_elements[] = {
 	{ SCREEN, "screen", NULL, NULL, NULL},
 	{ SCREENSHOT, "screenshot", NULL, NULL, NULL},
 	{ SCREENINFO, "screeninfo", NULL, NULL, NULL},
-	{ COMMAND, "command", NULL, NULL, NULL},
-	{ REPLACEABLE, "replaceable", NULL, NULL, NULL},
-	{ FUNCTION, "function", NULL, NULL, NULL},
-	{ GUIBUTTON, "guibutton", NULL, NULL, NULL},
-	{ GUIICON, "guiicon", NULL, NULL, NULL},
-	{ GUILABEL, "guilabel", NULL, NULL, NULL},
-	{ GUIMENU, "guimenu", NULL, NULL, NULL},
-	{ GUIMENUITEM, "guimenuitem", NULL, NULL, NULL},
-	{ HARDWARE, "hardware", NULL, NULL, NULL},
-	{ KEYCAP, "keycap", NULL, NULL, NULL},
-	{ KEYCODE, "keycode", NULL, NULL, NULL},
-	{ KEYSYM, "keysym", NULL, NULL, NULL},
-	{ LITERAL, "literal", NULL, NULL, NULL},
-	{ PARAMETER, "parameter", NULL, NULL, NULL},
-	{ PROMPT, "prompt", NULL, NULL, NULL},
-	{ SYMBOL, "symbol", NULL, NULL, NULL},
-	{ USERINPUT, "userinput", NULL, NULL, NULL},
+	{ COMMAND, "command", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ REPLACEABLE, "replaceable", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ FUNCTION, "function", NULL, NULL, (charactersSAXFunc) toc_tag_characters},
+	{ GUIBUTTON, "guibutton", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ GUIICON, "guiicon", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ GUILABEL, "guilabel", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ GUIMENU, "guimenu", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ GUIMENUITEM, "guimenuitem", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ HARDWARE, "hardware", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ KEYCAP, "keycap", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ KEYCODE, "keycode", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ KEYSYM, "keysym", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ LITERAL, "literal", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ PARAMETER, "parameter", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ PROMPT, "prompt", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ SYMBOL, "symbol", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ USERINPUT, "userinput", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ CAUTION, "caution", NULL, NULL, NULL},
 	{ LEGALPARA, "legalpara", NULL, NULL, NULL},
-	{ FIRSTTERM, "firstterm", NULL, NULL, NULL},
-	{ STRUCTNAME, "structname", NULL, NULL, NULL},
-	{ STRUCTFIELD, "structfield", NULL, NULL, NULL},
+	{ FIRSTTERM, "firstterm", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ STRUCTNAME, "structname", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ STRUCTFIELD, "structfield", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ FUNCSYNOPSIS, "funcsynopsis", NULL, NULL, NULL},
 	{ FUNCPROTOTYPE, "funcprototype", NULL, NULL, NULL},
 	{ FUNCDEF, "funcdef", NULL, NULL, NULL},
 	{ FUNCPARAMS, "funcparams", NULL, NULL, NULL},
 	{ PARAMDEF, "paramdef", NULL, NULL, NULL},
 	{ VOID, "void", NULL, NULL, NULL},
-	{ GUISUBMENU, "guisubmenu", NULL, NULL, NULL},
-	{ INTERFACE, "interface", NULL, NULL, NULL},
-	{ LINK, "link", NULL, NULL, NULL},
-	{ MENUCHOICE, "menuchoice", NULL, NULL, NULL},
+	{ GUISUBMENU, "guisubmenu", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ INTERFACE, "interface", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ LINK, "link", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ MENUCHOICE, "menuchoice", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ TABLE, "table", NULL, NULL, NULL},
 	{ INFORMALTABLE, "informaltable", NULL, NULL, NULL},
 	{ ROW, "row",  NULL, NULL, NULL},
 	{ ENTRY, "entry", NULL, NULL, NULL},
 	{ THEAD, "thead", NULL, NULL, NULL},
 	{ TBODY, "tbody", NULL, NULL, NULL},
-	{ ACRONYM, "acronym", NULL, NULL, NULL},
-	{ MARKUP, "markup", NULL, NULL, NULL},
+	{ ACRONYM, "acronym", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ MARKUP, "markup", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ SIMPLELIST, "simplelist", NULL, NULL, NULL},
 	{ MEMBER, "member", NULL, NULL, NULL},
-	{ MOUSEBUTTON, "mousebutton", NULL, NULL, NULL},
-	{ SUPERSCRIPT, "superscript", NULL, NULL, NULL},
-	{ SYSTEMITEM, "systemitem", NULL, NULL, NULL},
-	{ VARNAME, "varname", NULL, NULL, NULL},
+	{ MOUSEBUTTON, "mousebutton", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ SUPERSCRIPT, "superscript", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ SYSTEMITEM, "systemitem", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ VARNAME, "varname", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ BLOCKQUOTE, "blockquote", NULL, NULL, NULL},
-	{ QUOTE, "quote", NULL, NULL, NULL},
-	{ OPTION, "option", NULL, NULL, NULL},
-	{ ENVAR, "envar", NULL, NULL, NULL},
-	{ COMPUTEROUTPUT, "computeroutput", NULL, NULL, NULL},
+	{ QUOTE, "quote", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ OPTION, "option", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ ENVAR, "envar", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
+	{ COMPUTEROUTPUT, "computeroutput", NULL, NULL, (charactersSAXFunc) toc_tag_characters },
 	{ INLINEGRAPHIC, "inlinegraphic", NULL, NULL, NULL},
 	{ LEGALNOTICE, "legalnotice", NULL, NULL, NULL},
 	{ QUESTION, "question", NULL, NULL, NULL},
 	{ ANSWER, "answer", NULL, NULL, NULL},
 	{ CHAPTER, "chapter", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
 	{ PREFACE, "preface", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
+	{ TERM, "term", NULL, NULL, NULL},
 	{ APPENDIX, "appendix", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
+	{ DOCINFO, "docinfo", (startElementSAXFunc) artheader_start_element, (endElementSAXFunc) toc_artheader_end_element, NULL},
+	{ GLOSSARY, "glossary", (startElementSAXFunc) glossary_start_element, NULL, NULL},
+	{ GLOSSDIV, "glossdiv", (startElementSAXFunc) toc_glossdiv_start_element, (endElementSAXFunc) toc_glossdiv_end_element, NULL},
+	{ GLOSSENTRY, "glossentry", (startElementSAXFunc) toc_glossentry_start_element, (endElementSAXFunc) toc_glossentry_end_element, NULL},
+	{ GLOSSTERM, "glossterm", (startElementSAXFunc) toc_glossterm_start_element, (endElementSAXFunc) toc_glossterm_end_element, (charactersSAXFunc) toc_title_characters},
+	{ GLOSSSEE, "glosssee", NULL, NULL, NULL},
+	{ GLOSSSEEALSO, "glossseealso", NULL, NULL, NULL},
+	{ EXAMPLE, "example", NULL, NULL, NULL},
+	{ VARLISTENTRY, "varlistentry", NULL, NULL, NULL},
+	{ STREET, "street", NULL, NULL, NULL},
+	{ CITY, "city", NULL, NULL, NULL},
+	{ COUNTRY, "country", NULL, NULL, NULL},
+	{ STATE, "state", NULL, NULL, NULL},
+	{ POSTCODE, "postcode", NULL, NULL, NULL},
+	{ LITERALLAYOUT, "literallayout", NULL, NULL, NULL},
+	{ QANDAENTRY, "qandaentry", NULL, NULL, NULL, },
+	{ QANDASET, "qandaset", NULL, NULL, NULL, },
 	{ UNDEFINED, NULL, NULL, NULL, NULL}
 };
 
@@ -141,6 +167,15 @@ toc_init_data (void)
 	return (gpointer) retval;
 }
 
+void
+toc_free_data (gpointer data)
+{
+	TocContext *to_free;
+
+	to_free = (TocContext *) data;
+	g_free (to_free->header);
+	g_free (to_free);
+}
 
 static void
 toc_sect_start_element (Context *context,
@@ -235,11 +270,10 @@ toc_sect_end_element (Context *context,
 	if (g_strcasecmp (name, "section") == 0) {
 		return;
 	}
-	
 
 	switch (name[4]) {
 	case 'a':
-		/* FIXME: should chapter be set to zero? */
+		/* FIXME bugzilla.eazel.com 4410: should chapter be set to zero? */
 	        context->preface = 0;
 		context->sect1 = 0;
 		break;
@@ -259,7 +293,7 @@ toc_sect_end_element (Context *context,
 		context->sect5 = 0;
 		break;
 	case 'n':
-		/* FIXME: should chapter be set to zero? */
+		/* FIXME bugzilla.eazel.com 4410: should chapter be set to zero? */
 		context->appendix = 0;
 		context->sect1 = 0;
 		break;
@@ -278,7 +312,7 @@ toc_artheader_end_element (Context *context, const gchar *name)
 	if (header->title)
 		g_print ("<TITLE>%s</TITLE>\n</HEAD>\n", header->title);
 	else
-		g_print ("<TITLE>GNOME Documentation</TITLE>\n</HEAD>\n");
+		g_print ("<TITLE>%s</TITLE>\n</HEAD>\n", _("GNOME Documentation"));
 
 	g_print ("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#840084\" ALINK=\"#0000FF\">\n");
 	g_print ("<TABLE><TR><TD VALIGN=\"TOP\">\n");
@@ -307,9 +341,11 @@ toc_artheader_end_element (Context *context, const gchar *name)
 		g_print ("<BR>");
 	}
 	g_print ("<P>");
-	if ((header->copyright_holder) && (header->copyright_year))
-		g_print ("<A HREF=\"gnome-help:%s?legalnotice\">Copyright</A> &copy; %s by %s", context->base_file, header->copyright_year, header->copyright_holder);
-	g_print ("<HR>\n<H2>Table of Contents</H2>\n\n");
+	if ((header->copyright_holder) && (header->copyright_year)) {
+		g_print ("<A HREF=\"gnome-help:%s?legalnotice\">%s</A> &copy; %s %s %s",
+			 context->base_file, _("Copyright"), header->copyright_year, _("by"),header->copyright_holder);
+	}
+	g_print ("<HR>\n<H2>%s</H2>\n\n", _("Table of Contents"));
 	g_print ("<P>\n");
 }
 
@@ -324,12 +360,16 @@ toc_author_start_element (Context *context,
 
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ARTHEADER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (BOOKINFO));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (DOCINFO));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSARYINFO));
 	index = find_first_parent (context, element_list);
 	g_slist_free (element_list);
 
 	switch (index) {
 	case ARTHEADER:
 	case BOOKINFO:
+	case DOCINFO:
+	case GLOSSARYINFO:
 		break;
 	default:
 		return;
@@ -348,12 +388,16 @@ toc_author_characters (Context *context, const gchar *chars, int len)
 
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ARTHEADER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (BOOKINFO));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (DOCINFO));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSARYINFO));
 	index = find_first_parent (context, element_list);
 	g_slist_free (element_list);
 
 	switch (index) {
 	case ARTHEADER:
 	case BOOKINFO:
+	case DOCINFO:
+	case GLOSSARYINFO:
 		break;
 	default:
 		return;
@@ -391,6 +435,8 @@ toc_copyright_characters (Context *context,
 
 	temp = g_strndup (chars, len);
 
+	/* FIXME: Memory is leaked here, especially if we have multiple
+	 * copyright holders. Not a big priority though. */
 	switch (index) {
 	case COPYRIGHT:
 		if (((StackElement *) context->stack->data)->info->index == YEAR)
@@ -432,6 +478,8 @@ toc_title_start_element (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSDIV));	
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (QANDASET));	
 	stack_el = find_first_element (context, element_list);
 
 	g_slist_free (element_list);
@@ -444,6 +492,8 @@ toc_title_start_element (Context *context,
 	case SECT1:
 	case SECTION:
 	case APPENDIX:
+	case GLOSSDIV:
+		in_printed_title = TRUE;
 		if (context->sect1 == 0) {
 			g_print ("<DT>");
 		} else if (context->sect2 == 0) {
@@ -460,7 +510,7 @@ toc_title_start_element (Context *context,
 		}
 
 		if (context->preface > 0) {
-		       if (context->sect1 == 0) g_print ("PREFACE:<BR>");
+		       if (context->sect1 == 0) g_print ("%s:<BR>", _("PREFACE"));
 		       if (context->sect1 > 0) g_print ("&nbsp;&nbsp;%d", context->sect1); 
 		        
 		} else if (context->chapter > 0) {
@@ -469,7 +519,7 @@ toc_title_start_element (Context *context,
 									
 		} else if (context->appendix > 0) {
 			if (context->sect1 == 0) {
-				g_print ("APPENDIX:<BR>");
+				g_print ("%s:<BR>", _("APPENDIX"));
 			} else if (context->sect1 > 0) {
 				g_print ("&nbsp;&nbsp;%d",context->sect1);
 			}
@@ -483,18 +533,21 @@ toc_title_start_element (Context *context,
                 if (context->sect4 > 0) g_print (".%d", context->sect4);
                 if (context->sect5 > 0) g_print (".%d", context->sect5);
 
-		/* Don't print the "." if you are in the preface or appendix title */
-		if (stack_el->info->index != PREFACE && stack_el->info->index != APPENDIX) {
+		/* Don't print the "." if you are in the preface or appendix or glossdiv title */
+		if ((stack_el->info->index != PREFACE) &&
+		    (stack_el->info->index != APPENDIX) &&
+		    (stack_el->info->index != GLOSSDIV)) {
 		       g_print (".&nbsp;&nbsp;");
 		}
 		
 		/* Only print the link if we are the chapter tag, or the sect1 tag
-		 * (and the document is an article) or preface or appendix */
+		 * (and the document is an article) or preface or appendix or glossdiv */
 		print_link = (((stack_el->info->index == SECT1) && (context->doctype != BOOK_DOC)
 				&& (context->appendix == 0))
 				|| stack_el->info->index == CHAPTER
 		                || stack_el->info->index == PREFACE
-				|| stack_el->info->index == APPENDIX);
+				|| stack_el->info->index == APPENDIX
+				|| stack_el->info->index == GLOSSDIV);
 
 		if (print_link) {
 			g_print ("<A href=\"gnome-help:%s", context->base_file);
@@ -515,6 +568,7 @@ toc_title_start_element (Context *context,
 	case SECT3:
 	case SECT4:
 	case SECT5:
+		in_printed_title = TRUE;
 		if (context->sect2 == 0) {
 			g_print ("<DT>");
 		} else if (context->sect3 == 0) {
@@ -564,23 +618,27 @@ toc_title_end_element (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSDIV));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (QANDASET));
 
 	index = find_first_parent (context, element_list);
 
+	in_printed_title = FALSE;
 	switch (index) {
 	case PREFACE:
 	case APPENDIX:
-		g_print ("</A></DT>");
+	case GLOSSDIV:
+		g_print ("</A></DT>\n");
 		break;
 	case CHAPTER:
-		g_print ("</A></DT>");
+		g_print ("</A></DT>\n");
 		break;
 	case SECT1:
 	case SECTION:
 		if (context->doctype == ARTICLE_DOC) {
 			g_print ("</A></DT>\n");
 		} else {
-			g_print ("</DT>");
+			g_print ("</DT>\n");
 		}
 		break;
 	case SECT2:
@@ -625,6 +683,11 @@ toc_title_characters (Context *context, const gchar *chars, int len)
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (DOCINFO));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSDIV));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSTERM));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSARYINFO));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (QANDASET));
 
 	index = find_first_parent (context, element_list);
 
@@ -640,11 +703,15 @@ toc_title_characters (Context *context, const gchar *chars, int len)
 	case SECT5:
 	case SECTION:
 	case APPENDIX:
+	case GLOSSDIV:
+	case GLOSSTERM:
 		g_print (temp);
 		g_free (temp);
 		break;
 	case ARTHEADER:
 	case BOOKINFO:
+	case DOCINFO:
+	case GLOSSARYINFO:
 		if (((StackElement *)context->stack->data)->info->index == TITLE)
 			((TocContext *) context->data)->header->title = temp;
 		else if (((StackElement *)context->stack->data)->info->index == SUBTITLE)
@@ -657,3 +724,124 @@ toc_title_characters (Context *context, const gchar *chars, int len)
 
 	g_slist_free (element_list);
 }
+
+static void
+toc_glossdiv_start_element (Context *context,
+			    const gchar *name,
+			    const xmlChar **atrs)
+{
+	gchar **atrs_ptr;
+
+	atrs_ptr = (gchar **) atrs;
+	while (atrs_ptr && *atrs_ptr) {
+        	if (!g_strcasecmp (*atrs_ptr, "id")) {
+			atrs_ptr++;
+			((StackElement *)context->stack->data)->atrs = g_new0 (gchar *, 3);
+			((StackElement *)context->stack->data)->atrs[0] = g_strdup ("id");
+			((StackElement *)context->stack->data)->atrs[1] = g_strdup (*atrs_ptr);
+			break;
+		}
+		atrs_ptr += 2;
+	}  
+}
+
+static void
+toc_glossdiv_end_element (Context *context,
+		      const gchar *name)
+{
+/*  FIXME: what do we do here? */
+}
+
+static void
+toc_glossentry_start_element (Context *context,
+			      const gchar *name,
+			      const xmlChar **atrs)
+{
+	gchar **atrs_ptr;
+
+	atrs_ptr = (gchar **) atrs;
+	while (atrs_ptr && *atrs_ptr) {
+		if (!g_strcasecmp (*atrs_ptr, "id")) {
+			atrs_ptr++;
+			((StackElement *)context->stack->data)->atrs = g_new0 (gchar *, 3);
+			((StackElement *)context->stack->data)->atrs[0] = g_strdup ("id");
+			((StackElement *)context->stack->data)->atrs[1] = g_strdup (*atrs_ptr);
+			break;
+		}
+		atrs_ptr += 2;
+	}  
+}
+
+static void
+toc_glossentry_end_element (Context *context,
+			    const gchar *name)
+{
+/*  FIXME: what do we do here? */
+}
+
+static void
+toc_glossterm_start_element (Context *context,
+			     const gchar *name,
+			     const xmlChar **atrs)
+{
+	StackElement *stack_el;
+	gchar **atrs_ptr;
+	GSList *element_list = NULL;
+
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSENTRY));
+	stack_el = find_first_element (context, element_list);
+
+	g_print ("<DT>");
+	g_print ("&nbsp;&nbsp;<A href=\"gnome-help:%s", context->base_file);
+
+	atrs_ptr = (stack_el->atrs);
+	while (atrs_ptr && *atrs_ptr) {
+		if (!g_strcasecmp (*atrs_ptr, "id")) {
+			atrs_ptr++;
+			g_print ("?%s", *atrs_ptr);
+			break;
+		}
+		atrs_ptr += 2;
+	}
+	g_print ("\">");
+}
+
+static void
+toc_glossterm_end_element (Context *context,
+			   const gchar *name)
+{
+	g_print ("</A>\n");
+}
+
+static void
+toc_tag_characters (Context *context, const gchar *chars, int len)
+{
+	GSList *element_list = NULL;
+	ElementIndex index;
+	char *temp;
+
+	if (in_printed_title == FALSE) {
+		return;
+	}
+	
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TITLE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GLOSSTERM));
+
+	index = find_first_parent (context, element_list);
+	temp = g_strndup (chars, len);
+
+	switch (index) {
+	case TITLE:
+	case GLOSSTERM:
+		temp = g_strndup (chars, len);
+		sect_print (context, temp);
+		g_free (temp);
+		break;
+	default:
+		g_free (temp);
+		break;
+	};
+
+	g_slist_free (element_list);
+
+}
