@@ -55,8 +55,6 @@ typedef struct {
 
 
 static gboolean file_in_info_path (const char *file);
-static char *old_help_file (const char *old_uri);
-
 
 /* This is a copy of nautilus_shell_quote. The best thing to do is to
  * use the one on GNOME for GNOME 2.0, but we could also move this
@@ -196,7 +194,8 @@ static gboolean
 convert_file_to_uri (HelpURI *help_uri, char *file)
 {
 	const char *mime_type;
-	
+
+	/* FIXME: This test is no longer necessary since we know the file exists from calling function */
 	if (!g_file_test (file, G_FILE_TEST_ISFILE | G_FILE_TEST_ISLINK)) { 
 		return FALSE;
 	}
@@ -232,8 +231,7 @@ transform_file (const char *old_uri,
 {
 	HelpURI *help_uri;
 	char *p;
-	char *base, *new_uri, *new_uri_with_extension;
-	char *old_help;
+	char *base, *new_uri;
 
 	help_uri = help_uri_new ();
 
@@ -258,9 +256,10 @@ transform_file (const char *old_uri,
 
         /* Call the passed in function to compute the URI. */
 	new_uri = (* compute_uri_function) (base);
-	old_help = old_help_file (base);
         g_free (base);
+	
         if (new_uri == NULL) {
+		/* there is no SGML/XML or old HTML help path */
                 help_uri_free (help_uri);
                 return NULL;
         }
@@ -270,35 +269,8 @@ transform_file (const char *old_uri,
 		return help_uri;
 	}
 
-	new_uri_with_extension = g_strconcat (new_uri, ".xml", NULL);
-	if (convert_file_to_uri (help_uri, new_uri_with_extension)) {
-		g_free (new_uri);
-		g_free (old_help);
-		return help_uri;
-	}
-	g_free(new_uri_with_extension);
-
-	/* Try with an sgml extension. */
-	new_uri_with_extension = g_strconcat (new_uri, ".sgml", NULL);
-	if (convert_file_to_uri (help_uri, new_uri_with_extension)) {
-		g_free (new_uri);
-		g_free (old_help);
-		return help_uri;
-	}
-	g_free (new_uri_with_extension);
-
-	/* Try to look for the 'index.html' file */	
-	new_uri_with_extension = g_strconcat (old_help, ".html", NULL);
-	if (convert_file_to_uri (help_uri, new_uri_with_extension)) {
-		g_free (new_uri);
-		g_free (old_help);
-		return help_uri;
-	}
-        g_free (new_uri_with_extension);
-
         /* Failed, so return. */
 	g_free (new_uri);
-	g_free (old_help);
         help_uri_free (help_uri);
 	return NULL;
 }
@@ -355,27 +327,15 @@ file_from_path (const char *path)
 	}
 }
 
-static char *
-old_help_file (const char *old_uri) 
-{
-	char *base_name, *new_uri;
-
-	base_name = file_from_path (old_uri);
-	if (base_name == NULL || base_name[0] == '\0') {
-		g_free (base_name);
-		return NULL;
-	}
-
-	new_uri = gnome_help_file_path (base_name, "index");
-	g_free (base_name);
-
-	return new_uri;
-}
-
+/* If the help file exists returns the appropriate PATH (taking locale into account)
+ * otherwise it returns NULL */
 static char *
 find_help_file (const char *old_uri)
 {
-        char *base_name, *new_uri;
+        char *base_name, *new_uri, *buf;
+	GList *language_list;
+	char *new_uri_with_extension;
+	char *old_help;
 
         base_name = file_from_path (old_uri);
         if (base_name == NULL || base_name[0] == '\0') {
@@ -383,12 +343,46 @@ find_help_file (const char *old_uri)
                 return NULL;
         }
 
-        /* FIXME bugzilla.eazel.com 2403: 
-         * gnome_help_file_path should take const char * parameters. */
-        new_uri = gnome_help_file_path (base_name, (char *) old_uri);
-        g_free (base_name);
+	new_uri_with_extension = NULL;
+	new_uri = NULL;
+	
+	language_list = gnome_i18n_get_language_list ("LC_MESSAGES");
 
-        return new_uri;
+	while (!new_uri_with_extension && language_list) {
+		const char *lang;
+
+		lang = language_list->data;
+		buf = g_strdup_printf ("gnome/help/%s/%s/%s", base_name, lang, old_uri);
+		new_uri = gnome_unconditional_datadir_file (buf);
+		g_free (buf);
+
+		new_uri_with_extension = g_strconcat (new_uri, ".xml", NULL);
+		/* FIXME: Should we use g_file_test instead? */
+		if (!g_file_exists (new_uri_with_extension)) {
+			/* XML file doesn't exist - now try SGML */
+			g_free (new_uri_with_extension);
+	
+			new_uri_with_extension = g_strconcat (new_uri, ".sgml", NULL);
+			if (!g_file_exists (new_uri_with_extension)) {
+				/* SGML file doesn't exist - fallback to SGML */
+				g_free (new_uri_with_extension);
+
+				old_help = g_strdup_printf ("gnome/help/%s/%s/index.html", base_name, lang);
+				new_uri_with_extension = gnome_unconditional_datadir_file (old_help);
+				g_free (old_help);
+				
+				if (!g_file_exists (new_uri_with_extension)) {
+					/* HTML file doesn't exist - next language */
+					g_free (new_uri_with_extension);
+					new_uri_with_extension = NULL;
+				}
+			}
+		}
+		g_free (new_uri);
+		new_uri = NULL;
+		language_list = language_list->next;
+	}
+	return new_uri_with_extension;			
 }
 
 static HelpURI *
@@ -447,7 +441,8 @@ file_in_info_path (const char *file)
         /* Check some hardcoded locations. */
         if (strncmp (file, "/usr/info/", strlen ("/usr/info/")) == 0 ||
             strncmp (file, "/usr/local/info/", strlen ("/usr/local/info/")) ||
-            strncmp (file, "/usr/local/info/", strlen ("/usr/gnome/info/"))) {
+            strncmp (file, "/usr/gnome/info/", strlen ("/usr/gnome/info/")) ||
+	    strncmp (file, "/usr/share/info/", strlen ("/usr/share/info/"))) {
                 return TRUE;
         }
 
