@@ -33,6 +33,12 @@
 #include <liboaf/liboaf.h>
 #include <bonobo.h>
 
+#include <gconf/gconf.h>
+
+#ifdef EAZEL_SERVICES
+#include <libtrilobite/libammonite-gtk.h>
+#endif
+
 #include <stdlib.h>
 
 #define nopeDEBUG_mfleming 1
@@ -44,7 +50,15 @@ mozilla_object_destroyed (GtkObject *obj)
 {
 	object_count--;
 
+#ifdef DEBUG_mfleming
+	g_print ("mozilla_object_destroyed\n");
+#endif
+
 	if (object_count <= 0) {
+#ifdef DEBUG_mfleming
+	g_print ("...final mozilla_object_destroyed, quiting\n");
+#endif
+
 		gtk_main_quit ();
 	}
 }
@@ -60,6 +74,10 @@ mozilla_make_object (BonoboGenericFactory *factory,
 	if (strcmp (goad_id, "OAFIID:nautilus_mozilla_content_view:1ee70717-57bf-4079-aae5-922abdd576b1")) {
 		return NULL;
 	}
+
+#ifdef DEBUG_mfleming
+	g_print ("+mozilla_make_object\n");
+#endif
 	
 	view = NAUTILUS_MOZILLA_CONTENT_VIEW (gtk_object_new (NAUTILUS_TYPE_MOZILLA_CONTENT_VIEW, NULL));
 
@@ -69,6 +87,10 @@ mozilla_make_object (BonoboGenericFactory *factory,
 
 	gtk_signal_connect (GTK_OBJECT (nautilus_view), "destroy", mozilla_object_destroyed, NULL);
 
+#ifdef DEBUG_mfleming
+	g_print ("-mozilla_make_object\n");
+#endif
+
 	return BONOBO_OBJECT (nautilus_view);
 }
 
@@ -80,12 +102,23 @@ run_test_cases (void)
 	return test_make_full_uri_from_relative ();
 }
 
+#ifdef EAZEL_SERVICES
+/*Defined in nautilus-mozilla-content-view.c*/
+extern EazelProxy_UserControl nautilus_mozilla_content_view_user_control;
+#endif
+
 int
 main (int argc, char *argv[])
 {
 	BonoboGenericFactory *factory;
 	CORBA_ORB orb;
 	char *registration_id;
+	GError *error_gconf = NULL;
+	char *fake_argv[] = { "nautilus-mozilla-content-view", NULL };
+
+#ifdef DEBUG_mfleming
+	g_print ("nautilus-mozilla-content-view: starting...\n");
+#endif
 
 	if (argc == 2 && 0 == strcmp (argv[1], "--self-test")) {
 		gboolean success;
@@ -98,11 +131,18 @@ main (int argc, char *argv[])
 	gnome_init_with_popt_table ("nautilus-mozilla-content-view", VERSION, 
 				    argc, argv,
 				    oaf_popt_options, 0, NULL); 
+	gdk_rgb_init ();
 	
 	orb = oaf_init (argc, argv);
 	
 	bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
-	
+
+	/* the fake_argv thing is just superstition.  I just don't
+	 * want gconf mucking with my args
+	 */
+	if (!gconf_init (1, fake_argv, &error_gconf)) {
+		g_warning ("Couldn't init gconf");
+	}
 	
         registration_id = oaf_make_registration_id ("OAFIID:nautilus_mozilla_content_view_factory:020a0285-6b96-4685-84a1-4a56eb6baa2b", getenv ("DISPLAY"));
 	factory = bonobo_generic_factory_new_multi (registration_id, 
@@ -111,14 +151,22 @@ main (int argc, char *argv[])
 	g_free (registration_id);
 	
 	gnome_vfs_init ();
-	
+
+#ifdef EAZEL_SERVICES
+	if (ammonite_init ((PortableServer_POA) bonobo_poa)) {
+		nautilus_mozilla_content_view_user_control = ammonite_get_user_control ();
+	}
+#endif
+
 #ifdef DEBUG_mfleming
-	g_print ("OAF registration complete.\n");
+	g_print ("nautilus-mozilla-content-view: OAF registration complete.\n");
 #endif
 
 	do {
 		bonobo_main ();
 	} while (object_count > 0);
-	
+
+        gnome_vfs_shutdown ();
+
 	return 0;
 }
