@@ -52,14 +52,26 @@
 
 #ifdef EAZEL_SERVICES
 #include <libtrilobite/libammonite-gtk.h>
-/* for bonobo_poa() */
-#include <bonobo/bonobo-main.h>
 #endif
 
 #define NUM_ELEMENTS_IN_ARRAY(_a) (sizeof (_a) / sizeof ((_a)[0]))
 
+/* Code-copied from nsGUIEvent.h */
+
+enum nsEventStatus {  
+	/* The event is ignored, do default processing */
+ 	nsEventStatus_eIgnore,            
+ 	/* The event is consumed, don't do default processing */
+ 	nsEventStatus_eConsumeNoDefault, 
+ 	/* The event is consumed, but do default processing */
+	nsEventStatus_eConsumeDoDefault  
+};
+ 
+#define NS_DOM_EVENT_IGNORED ((enum nsEventStatus)nsEventStatus_eIgnore)
+#define NS_DOM_EVENT_CONSUMED ((enum nsEventStatus)nsEventStatus_eConsumeNoDefault)
+
 #ifdef EAZEL_SERVICES
-static EazelProxy_UserControl global_user_control = CORBA_OBJECT_NIL;
+EazelProxy_UserControl nautilus_mozilla_content_view_user_control = CORBA_OBJECT_NIL;
 #endif
 
 struct NautilusMozillaContentViewDetails {
@@ -86,6 +98,8 @@ static void     mozilla_net_state_callback                     (GtkMozEmbed
 								gint                             state,
 								guint                            status,
 								gpointer                         user_data);
+static void     mozilla_net_stop_callback                      (GtkMozEmbed                     *mozilla,
+								gpointer                         user_data);
 static void     mozilla_link_message_callback                  (GtkMozEmbed                     *mozilla,
 								gpointer                         user_data);
 static void     mozilla_progress_callback                      (GtkMozEmbed                     *mozilla,
@@ -95,6 +109,9 @@ static void     mozilla_progress_callback                      (GtkMozEmbed
 static  gint    mozilla_open_uri_callback                      (GtkMozEmbed                     *mozilla,
 								const char                      *uri,
 								gpointer                         user_data);
+static  gint    mozilla_dom_key_press_callback                 (GtkMozEmbed                     *mozilla,
+								gpointer                         dom_event,
+								gpointer                         user_data);
 static  gint    mozilla_dom_mouse_click_callback               (GtkMozEmbed                     *mozilla,
 								gpointer                         dom_event,
 								gpointer                         user_data);
@@ -110,7 +127,7 @@ static char *   mozilla_translate_uri_if_needed                (NautilusMozillaC
 static char *   mozilla_untranslate_uri_if_needed              (NautilusMozillaContentView      *view,
 								const char                      *uri);
 static void     mozilla_content_view_one_time_happenings       (void);
-
+static void     mozilla_content_view_setup_profile_directory   (void);
 
 #ifdef EAZEL_SERVICES
 
@@ -156,6 +173,8 @@ nautilus_mozilla_content_view_initialize_class (NautilusMozillaContentViewClass
 	parent_class = gtk_type_class (GTK_TYPE_VBOX);
 	
 	object_class->destroy = nautilus_mozilla_content_view_destroy;
+
+	mozilla_content_view_setup_profile_directory ();
 }
 
 static void
@@ -191,6 +210,12 @@ nautilus_mozilla_content_view_initialize (NautilusMozillaContentView *view)
 					GTK_SIGNAL_FUNC (mozilla_net_state_callback),
 					view,
 					GTK_OBJECT (view->details->mozilla));
+
+	gtk_signal_connect_while_alive (GTK_OBJECT (view->details->mozilla), 
+					"net_stop",
+					GTK_SIGNAL_FUNC (mozilla_net_stop_callback),
+					view,
+					GTK_OBJECT (view->details->mozilla));
 	
 	gtk_signal_connect_while_alive (GTK_OBJECT (view->details->mozilla), 
 					"link_message",
@@ -209,6 +234,12 @@ nautilus_mozilla_content_view_initialize (NautilusMozillaContentView *view)
 					GTK_SIGNAL_FUNC (mozilla_open_uri_callback),
 					view,
 					GTK_OBJECT (view->details->mozilla));
+
+	gtk_signal_connect_while_alive (GTK_OBJECT (view->details->mozilla), 
+					"dom_key_press",
+					GTK_SIGNAL_FUNC (mozilla_dom_key_press_callback),
+					view,
+					GTK_OBJECT (view->details->mozilla));
 	
 	gtk_signal_connect_while_alive (GTK_OBJECT (view->details->mozilla), 
 					"dom_mouse_click",
@@ -248,10 +279,6 @@ nautilus_mozilla_content_view_destroy (GtkObject *object)
 		gdk_cursor_destroy (view->details->busy_cursor);
 	}
 
-#ifdef EAZEL_SERVICES
-	ammonite_shutdown ();
-#endif /*EAZEL_SERVICES*/
-
 	g_free (view->details);
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy) {
@@ -402,6 +429,10 @@ nautilus_mozilla_content_view_load_uri (NautilusMozillaContentView	*view,
 
 	g_assert (uri != NULL);
 
+#ifdef DEBUG_mfleming
+	g_print ("+%s uri='%s'\n", __FUNCTION__, uri);
+#endif
+
 	view->details->got_called_by_nautilus = TRUE;
 
 	/* Check whether its the same uri.  Ignore the mozilla
@@ -424,12 +455,21 @@ nautilus_mozilla_content_view_load_uri (NautilusMozillaContentView	*view,
 	g_print ("nautilus_mozilla_content_view_load_uri (uri = %s, same = %d)\n", 
 		 view->details->uri, same_uri);
 #endif
+	/* NOTE: (mfleming) I placed this here to fix bugzilla.eazel.com 5249
+	 * The belief is that calling load_underway here will cause
+	 * the Bonobo::Control::realize call to be made while this call is
+	 * pending, rather than later.  It's somewhat safer for that to occur
+	 * here.  Anyway, no harm is done: this is a totally legit place
+	 * to call load_underway
+	 */
+
+	nautilus_view_report_load_underway (view->details->nautilus_view);
+
 	/* If the request can be handled by mozilla, pass the uri as is.  Otherwise,
 	 * use gnome-vfs to open the uri and later stream the data into the gtkmozembed
 	 * widget.
 	 */
 	if (mozilla_is_uri_handled_by_mozilla (uri)) {
-
 		if (same_uri) {
 			gtk_moz_embed_reload (GTK_MOZ_EMBED (view->details->mozilla),
 					      GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
@@ -439,7 +479,6 @@ nautilus_mozilla_content_view_load_uri (NautilusMozillaContentView	*view,
 						view->details->uri);
 		}
 	} else {
-		nautilus_view_report_load_underway (view->details->nautilus_view);
 		gnome_vfs_async_open (&async_handle, uri, GNOME_VFS_OPEN_READ, mozilla_vfs_callback, view);	
 	}
 }
@@ -481,6 +520,10 @@ mozilla_load_location_callback (NautilusView *nautilus_view,
 
 	g_assert (nautilus_view == view->details->nautilus_view);
 
+#ifdef DEBUG_mfleming
+	g_print ("+%s location='%s'\n", __FUNCTION__, location);
+#endif
+
 	translated_location = mozilla_translate_uri_if_needed (view, location);
 
 	if (translated_location) {
@@ -573,6 +616,10 @@ mozilla_location_changed_callback (GtkMozEmbed *mozilla, gpointer user_data)
 
 	new_location = gtk_moz_embed_get_location (GTK_MOZ_EMBED (view->details->mozilla));
 
+#ifdef DEBUG_mfleming
+	g_print ("+%s current moz location='%s'\n", __FUNCTION__, new_location);
+#endif
+
 #ifdef DEBUG_ramiro
 	g_print ("mozilla_location_changed_callback (%s)\n", new_location);
 #endif
@@ -645,6 +692,9 @@ mozilla_net_state_callback (GtkMozEmbed	*mozilla,
 
 	g_assert (GTK_MOZ_EMBED (mozilla) == GTK_MOZ_EMBED (view->details->mozilla));
 
+#if defined(DEBUG_mfleming)
+	g_print ("gtkembedmoz signal: 'net_state'\n");
+#endif
 #if defined(DEBUG_ramiro)
 	g_print ("%s\n", __FUNCTION__);
 	debug_print_state_flags (state_flags);
@@ -663,19 +713,43 @@ mozilla_net_state_callback (GtkMozEmbed	*mozilla,
 	}
 }
 
+
+static void
+mozilla_net_stop_callback (GtkMozEmbed 	*mozilla,
+			   gpointer	user_data)
+{
+ 	NautilusMozillaContentView	*view;
+
+	view = NAUTILUS_MOZILLA_CONTENT_VIEW (user_data);
+
+	g_assert (GTK_MOZ_EMBED (mozilla) == GTK_MOZ_EMBED (view->details->mozilla));
+
+#if defined(DEBUG_mfleming)
+	g_print ("gtkembedmoz signal: 'net_stop'\n");
+#endif
+
+	nautilus_view_report_load_complete (view->details->nautilus_view);
+}
+
 static void
 mozilla_link_message_callback (GtkMozEmbed *mozilla, gpointer user_data)
 {
  	NautilusMozillaContentView	*view;
 	char				*link_message;
+	char				*translated_link_message;
 
 	view = NAUTILUS_MOZILLA_CONTENT_VIEW (user_data);
 
 	g_assert (GTK_MOZ_EMBED (mozilla) == GTK_MOZ_EMBED (view->details->mozilla));
 
 	link_message = gtk_moz_embed_get_link_message (GTK_MOZ_EMBED (view->details->mozilla));
+
+	/* This is actually not that efficient */
+	translated_link_message = mozilla_untranslate_uri_if_needed (view, link_message);
+
 	nautilus_view_report_status (view->details->nautilus_view,
-				     link_message);
+				     translated_link_message);
+	g_free (translated_link_message);
 	g_free (link_message);
 }
 
@@ -694,11 +768,20 @@ mozilla_progress_callback (GtkMozEmbed *mozilla,
 	g_print ("mozilla_progress_callback (max = %d, current = %d)\n", max_progress, current_progress);
 #endif
 
-	if (max_progress == current_progress || max_progress == 0) {
-		nautilus_view_report_load_complete (view->details->nautilus_view);
+	/* NOTE:
+	 * "max_progress" will be -1 if the filesize cannot be determined
+	 * On occasion, it appears that current_progress may actuall exceed max_progress
+	 */
+
+	if (max_progress == -1 || max_progress == 0) {
+		nautilus_view_report_load_progress (view->details->nautilus_view, 
+							    0);
+	} else if (max_progress < current_progress) {
+		nautilus_view_report_load_progress (view->details->nautilus_view, 
+							    1.0);
 	} else {
 		nautilus_view_report_load_progress (view->details->nautilus_view, 
-						    current_progress / max_progress);
+							    current_progress / max_progress);
 	}
 }
 
@@ -710,6 +793,7 @@ mozilla_open_uri_callback (GtkMozEmbed *mozilla,
 {
 	gint                            abort_uri_open;
  	NautilusMozillaContentView     *view;
+	/*FIXME this static is shared between multiple browser windows! */
 	static gboolean                 do_nothing = FALSE;
 
 	view = NAUTILUS_MOZILLA_CONTENT_VIEW (user_data);
@@ -719,6 +803,10 @@ mozilla_open_uri_callback (GtkMozEmbed *mozilla,
 	/* Determine whether we want to abort this uri load */
 	abort_uri_open = mozilla_is_uri_handled_by_nautilus (uri);
 
+#ifdef DEBUG_mfleming
+	g_print ("+%s do_nothing = %d, got_called_by_nautilus = %d\n", __FUNCTION__, do_nothing,  view->details->got_called_by_nautilus);
+#endif
+
 	/* FIXME: I believe that this will cause a navigation from
 	 * a page containing a specific iframe URL to that same URL
 	 * to fail.
@@ -730,6 +818,9 @@ mozilla_open_uri_callback (GtkMozEmbed *mozilla,
 	 */
 
 	if ( mozilla_events_is_url_in_iframe (mozilla, uri)) {
+#ifdef DEBUG_mfleming
+		g_print ("URI is in an iframe;ignoring '%s'\n", uri);
+#endif
 		return FALSE;
 	}
 	
@@ -755,7 +846,8 @@ mozilla_open_uri_callback (GtkMozEmbed *mozilla,
 
 		/*do untranslate here*/
 		untranslated_uri = mozilla_untranslate_uri_if_needed (view, uri);
-		nautilus_view_open_location (view->details->nautilus_view, untranslated_uri);
+		nautilus_view_open_location_in_this_window
+			(view->details->nautilus_view, untranslated_uri);
 		g_free (untranslated_uri);
 
 		bonobo_object_unref (BONOBO_OBJECT (view->details->nautilus_view));
@@ -783,7 +875,88 @@ is_uri_partial (const char *uri)
 	return  !(':' == *current);
 }
 
+/*
+ * Remove "./" segments
+ * Compact "../" segments inside the URI
+ * Remove "." at the end of the URL 
+ * Leave any ".."'s at the beginning of the URI
+ */
 
+/* in case if you were wondering, this is probably one of the least time-efficient ways to do this*/
+static void
+remove_internal_relative_components (char *uri_current)
+{
+	char *segment_prev, *segment_cur;
+	size_t len_prev, len_cur;
+
+	len_prev = len_cur = 0;
+	segment_prev = NULL;
+
+	segment_cur = uri_current;
+
+	while (*segment_cur) {
+		len_cur = strcspn (segment_cur, "/");
+
+		if (len_cur == 1 && segment_cur[0] == '.') {
+			/* Remove "." 's */
+			if (segment_cur[1] == '\0') {
+				segment_cur[0] = '\0';
+				break;
+			} else {
+				memmove (segment_cur, segment_cur + 2, strlen (segment_cur + 2) + 1);
+				continue;
+			}
+		} else if (len_cur == 2 && segment_cur[0] == '.' && segment_cur[1] == '.' ) {
+			/* Remove ".."'s (and the component to the left of it) that aren't at the
+			 * beginning or to the right of other ..'s
+			 */
+			if (segment_prev) {
+				if (! (len_prev == 2
+				       && segment_prev[0] == '.'
+				       && segment_prev[1] == '.')) {
+				       	if (segment_cur[2] == '\0') {
+						segment_prev[0] = '\0';
+						break;
+				       	} else {
+						memmove (segment_prev, segment_cur + 3, strlen (segment_cur + 3) + 1);
+
+						segment_cur = segment_prev;
+						len_cur = len_prev;
+
+						/* now we find the previous segment_prev */
+						if (segment_prev == uri_current) {
+							segment_prev = NULL;
+						} else if (segment_prev - uri_current >= 2) {
+							segment_prev -= 2;
+							for ( ; segment_prev > uri_current && segment_prev[0] != '/' 
+							      ; segment_prev-- );
+							if (segment_prev[0] == '/') {
+								segment_prev++;
+							}
+						}
+						continue;
+					}
+				}
+			}
+		}
+
+		/*Forward to next segment */
+
+		if (segment_cur [len_cur] == '\0') {
+			break;
+		}
+		 
+		segment_prev = segment_cur;
+		len_prev = len_cur;
+		segment_cur += len_cur + 1;	
+	}
+	
+}
+
+
+/* If I had known this relative uri code would have ended up this long, I would
+ * have done it a different way
+ */
 static char *
 make_full_uri_from_relative (const char *base_uri, const char *uri)
 {
@@ -791,20 +964,21 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 
 	/* See section 5.2 in RFC 2396 */
 
-	/* FIXME This function does not take into account a BASE tag
-	 * in an HTML document, so its functionality differs
-	 * from what Mozilla itself would do.
+	/* FIXME bugzilla.eazel.com 4413: This function does not take
+	 * into account a BASE tag in an HTML document, so its
+	 * functionality differs from what Mozilla itself would do.
 	 */
 
 	if (is_uri_partial (uri)) {
 		char *mutable_base_uri;
+		char *mutable_uri;
 
-		const char *uri_current;
+		char *uri_current;
 		size_t base_uri_length;
 		char *separator;
 
 		mutable_base_uri = g_strdup (base_uri);
-		uri_current = uri;
+		uri_current = mutable_uri = g_strdup (uri);
 
 		/* Chew off Fragment and Query from the base_url */
 
@@ -820,10 +994,21 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 			*separator = '\0';
 		}
 
-		/* Relative URI's beginning with '/' absolute-path based
-		 * at the root of the base uri
-		 */
-		if ('/' == uri_current[0]) {
+		if ('/' == uri_current[0] && '/' == uri_current [1]) {
+			/* Relative URI's beginning with the authority
+			 * component inherit only the scheme from their parents
+			 */
+
+			separator = strchr (mutable_base_uri, ':');
+
+			if (separator) {
+				separator[1] = '\0';
+			}			  
+		} else if ('/' == uri_current[0]) {
+			/* Relative URI's beginning with '/' absolute-path based
+			 * at the root of the base uri
+			 */
+
 			separator = strchr (mutable_base_uri, ':');
 
 			/* g_assert (separator), really */
@@ -858,11 +1043,9 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 				}
 			}
 
-			/* (the following are all rarely seen cases */
-			/* FIXME remove './' segments */
-			/* FIXME Remove . at the end of a URL (that is, if the URL ends in /. */   
-			/* FIXME Remove "../" inside the uri current */
+			remove_internal_relative_components (uri_current);
 
+			/* handle the "../"'s at the beginning of the relative URI */
 			while (0 == strncmp ("../", uri_current, 3)) {
 				uri_current += 3;
 				separator = strrchr (mutable_base_uri, '/');
@@ -874,6 +1057,17 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 				}
 			}
 
+			/* handle a ".." at the end */
+			if (uri_current[0] == '.' && uri_current[1] == '.' 
+			    && uri_current[2] == '\0') {
+
+			    	uri_current += 2;
+				separator = strrchr (mutable_base_uri, '/');
+				if (separator) {
+					*separator = '\0';
+				}
+			}
+
 			/* Re-append the '/' */
 			mutable_base_uri [strlen(mutable_base_uri)+1] = '\0';
 			mutable_base_uri [strlen(mutable_base_uri)] = '/';
@@ -881,9 +1075,10 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 
 		result = g_strconcat (mutable_base_uri, uri_current, NULL);
 		g_free (mutable_base_uri); 
+		g_free (mutable_uri); 
 
 #ifdef DEBUG_mfleming
-		g_print ("Relative URI converted base '%s' uri '%s' to '%s'", base_uri, uri, result);
+		g_print ("Relative URI converted base '%s' uri '%s' to '%s'\n", base_uri, uri, result);
 #endif
 	} else {
 		result = g_strdup (uri);
@@ -892,6 +1087,28 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 	return result;
 }
 
+static gint
+mozilla_dom_key_press_callback (GtkMozEmbed                     *mozilla,
+				gpointer                         dom_event,
+				gpointer                         user_data)
+{
+	g_return_val_if_fail (dom_event != NULL, NS_DOM_EVENT_IGNORED);
+
+#ifdef DEBUG_mfleming
+	g_print ("%s (%p)\n", __FUNCTION__, dom_event);
+#endif
+
+	/* If this keyboard event is going to trigger a URL navigation, we need
+	 * to fake it out like the mouse event below
+	 */
+
+	if (mozilla_events_is_key_return (dom_event)) {
+		return mozilla_dom_mouse_click_callback (mozilla, dom_event, user_data);
+	} else {
+		return NS_DOM_EVENT_IGNORED;
+	}
+}
+
 static gint
 mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
 				  gpointer	dom_event,
@@ -900,9 +1117,9 @@ mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
  	NautilusMozillaContentView	*view;
 	char				*href;
 
-	g_return_val_if_fail (GTK_IS_MOZ_EMBED (mozilla), TRUE);
-	g_return_val_if_fail (dom_event != NULL, TRUE);
-	g_return_val_if_fail (NAUTILUS_IS_MOZILLA_CONTENT_VIEW (user_data), TRUE);
+	g_return_val_if_fail (GTK_IS_MOZ_EMBED (mozilla), NS_DOM_EVENT_IGNORED);
+	g_return_val_if_fail (dom_event != NULL, NS_DOM_EVENT_IGNORED);
+	g_return_val_if_fail (NAUTILUS_IS_MOZILLA_CONTENT_VIEW (user_data), NS_DOM_EVENT_IGNORED);
 	
 	view = NAUTILUS_MOZILLA_CONTENT_VIEW (user_data);
 
@@ -922,20 +1139,12 @@ mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
 		 * its context, we use a hack.  The hack is to lie Nautilus
 		 * that the POST event was instigated by Nautilus.  Nautilus
 		 * will the call our load_location() away without having 
-		 * gnome-vfs try and do the post and thus spoling it.
+		 * gnome-vfs try and do the post and thus spoiling it.
 		 */
 		view->details->got_called_by_nautilus = TRUE;
 	} else {
 		
-		href = mozilla_events_get_href_for_mouse_event (dom_event);
-
-		/* 
-		 * The return value over here needs to be psycho-analyzed some.
-		 * In theory, a return value of NS_OK, which is 0 (yes, zero)
-		 * in the Mozilla universe means that the dom event got handled
-		 *
-		 * Need to look at the gtkmozembed.cpp code more carefully.
-		 */
+		href = mozilla_events_get_href_for_event (dom_event);
 
 		if (href) {
 			char * href_full = NULL;
@@ -948,7 +1157,7 @@ mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
 
 #if 0
 /* FIXME mfleming I think I should do this here, but I'm not sure */
-			href_full = mozilla_untranslate_uri_if_needed (href);
+			href_full = mozilla_untranslate_uri_if_needed (view, href);
 			g_free (href);
 			href = href_full;
 			href_full = NULL;
@@ -958,22 +1167,27 @@ mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
 			g_print ("%s() href = %s\n", __FUNCTION__, href);
 #endif
 			bonobo_object_ref (BONOBO_OBJECT (view->details->nautilus_view));
-			nautilus_view_open_location (view->details->nautilus_view, href);
+			nautilus_view_open_location_in_this_window
+				(view->details->nautilus_view, href);
 			bonobo_object_unref (BONOBO_OBJECT (view->details->nautilus_view));
 			g_free (href);
 
-			return 0;
+			return NS_DOM_EVENT_CONSUMED;
+		} else {
+#ifdef DEBUG_ramiro
+			g_print ("%s() no href;ignoring\n", __FUNCTION__);
+#endif
 		}
 	}
 	
-	return TRUE;
+	return NS_DOM_EVENT_IGNORED;
 }
 
 #define STRING_LIST_NOT_FOUND -1
 static gint
 string_list_get_index_of_string (const char *string_list[], guint num_strings, const char *string)
 {
-	gint i;
+	guint i;
 
 	g_return_val_if_fail (string != NULL, STRING_LIST_NOT_FOUND);
 	g_return_val_if_fail (string_list != NULL, STRING_LIST_NOT_FOUND);
@@ -1059,7 +1273,7 @@ mozilla_translate_uri_if_needed (NautilusMozillaContentView *view, const char *u
 		ret = eazel_services_scheme_translate (view, uri, FALSE);
 
 #ifdef DEBUG_mfleming
-		g_message ("Mozilla: translated uri '%s' to '%s'", uri, ret ? ret : "<no such page>");
+		g_message ("Mozilla: translated uri '%s' to '%s'\n", uri, ret ? ret : "<no such page>");
 #endif
 
 	} else
@@ -1120,7 +1334,7 @@ eazel_services_scheme_translate	(NautilusMozillaContentView	*view,
 	char *ret = NULL;
 	AmmoniteError err;
 
-	if (CORBA_OBJECT_NIL == global_user_control) {
+	if (CORBA_OBJECT_NIL == nautilus_mozilla_content_view_user_control) {
 		return NULL;
 	}
 
@@ -1155,7 +1369,7 @@ eazel_services_scheme_translate	(NautilusMozillaContentView	*view,
 			parsed = ammonite_url_parse (uri);
 			g_assert (parsed);
 			
-			/*FIXME it's possible that "view" will be gone by the time the reponse comes back */
+			/* FIXME bugzilla.eazel.com 4412: it's possible that "view" will be gone by the time the reponse comes back */
 			if (! ammonite_do_prompt_login_async (parsed->user, NULL, NULL, p_state,  prompt_login_cb) ) {
 				ret = NULL;
 				g_free (p_state->uri);
@@ -1187,7 +1401,7 @@ eazel_services_scheme_untranslate (NautilusMozillaContentView	*view,
 
 	if (ERR_Success == err) {
 #ifdef DEBUG_mfleming
-		g_message ("Mozilla: untranslated uri '%s' to '%s'", uri, ret );
+		g_message ("Mozilla: untranslated uri '%s' to '%s'\n", uri, ret );
 #endif
 	} else {
 		ret = g_strdup (uri);
@@ -1250,12 +1464,14 @@ test_make_full_uri_from_relative (void)
 	 * as fixmes
 	 */
 
+	/* From the RFC */
+
 	TEST_PARTIAL ("g", "http://a/b/c/g");
-	TEST_PARTIAL ("./g", "http://a/b/c/g");		/* FAILS */
+	TEST_PARTIAL ("./g", "http://a/b/c/g");
 	TEST_PARTIAL ("g/", "http://a/b/c/g/");
 	TEST_PARTIAL ("/g", "http://a/g");
 
-	TEST_PARTIAL ("//g", "http://g");		/* FAILS */
+	TEST_PARTIAL ("//g", "http://g");
 	
 	TEST_PARTIAL ("?y", "http://a/b/c/?y");
 	TEST_PARTIAL ("g?y", "http://a/b/c/g?y");
@@ -1266,18 +1482,53 @@ test_make_full_uri_from_relative (void)
 	TEST_PARTIAL ("g;x", "http://a/b/c/g;x");
 	TEST_PARTIAL ("g;x?y#s", "http://a/b/c/g;x?y#s");
 
-	TEST_PARTIAL (".", "http://a/b/c/");  		/* FAILS */
-	TEST_PARTIAL ("./", "http://a/b/c/");  		/* FAILS */
+	TEST_PARTIAL (".", "http://a/b/c/");
+	TEST_PARTIAL ("./", "http://a/b/c/");
 
-	TEST_PARTIAL ("..", "http://a/b/");		/* FAILS */
+	TEST_PARTIAL ("..", "http://a/b/");
 	TEST_PARTIAL ("../g", "http://a/b/g");
-	TEST_PARTIAL ("../..", "http://a/");		/* FAILS */
+	TEST_PARTIAL ("../..", "http://a/");
 	TEST_PARTIAL ("../../", "http://a/");
 	TEST_PARTIAL ("../../g", "http://a/g");
 
+	/* Others */
+	TEST_PARTIAL ("g/..", "http://a/b/c/");
+	TEST_PARTIAL ("g/../", "http://a/b/c/");
+	TEST_PARTIAL ("g/../g", "http://a/b/c/g");
+
 	return success;
 }
 
+static void
+ensure_profile_dir (void)
+{
+	char *profile_dir;
+	
+	profile_dir = g_strdup_printf ("%s/.nautilus/MozillaProfile", g_get_home_dir ());
+
+	/* Make sure the profile directory exists */
+	mkdir (profile_dir, 0777);
+	
+	g_free (profile_dir);
+}
+
+static void
+ensure_cache_dir (void)
+{
+	char *cache_dir;
+
+	ensure_profile_dir ();
+	
+	cache_dir = g_strdup_printf ("%s/.nautilus/MozillaProfile/Cache", g_get_home_dir ());
+
+	/* Make sure the cache directory exists */
+	mkdir (cache_dir, 0777);
+	
+	mozilla_preference_set ("browser.cache.directory", cache_dir);
+	
+	g_free (cache_dir);
+}
+
 static void
 mozilla_content_view_one_time_happenings (void)
 {
@@ -1293,7 +1544,7 @@ mozilla_content_view_one_time_happenings (void)
 	mozilla_preference_set_boolean ("security.checkloaduri", FALSE);
 
 	/* Change http protocol user agent to include the string 'Nautilus' */
-	mozilla_preference_set ("general.useragent.misc", "Nautilus/1.0");
+	mozilla_preference_set ("general.useragent.misc", "Nautilus/1.0PR3");
 
 	/* We dont want to use the proxy for localhost */
 	mozilla_preference_set ("network.proxy.no_proxies_on", "localhost");
@@ -1301,10 +1552,26 @@ mozilla_content_view_one_time_happenings (void)
 	/* Setup routing of proxy preferences from gconf to mozilla */
 	mozilla_gconf_listen_for_proxy_changes ();
 
-#ifdef EAZEL_SERVICES
-	if (ammonite_init ((PortableServer_POA) bonobo_poa)) {
-		global_user_control = ammonite_get_user_control ();
-	}
-#endif 
+	ensure_cache_dir ();
+}
+
+/* The "Mozilla Profile" directory is the place where mozilla stores 
+ * things like cookies and cache.  Here we tell the mozilla embedding
+ * widget to use ~/.nautilus/MozillaProfile for this purpose.
+ *
+ * We need mozilla M19 to support this feature.
+ */
+static void
+mozilla_content_view_setup_profile_directory (void)
+{
+#if (MOZILLA_MILESTONE >= 19)
+	const char *profile_name = "MozillaProfile";
+	char *profile_dir;
+	
+	profile_dir = g_strdup_printf ("%s/.nautilus", g_get_home_dir ());
+
+	/* Its a bug in mozilla embedding that we need to cast the const away */
+	gtk_moz_embed_set_profile_path (profile_dir, (char *) profile_name);
+#endif
 }
 
