@@ -103,6 +103,7 @@ main (int argc, char *argv[])
 				   argc, argv,
 				   oaf_popt_options, 0, NULL); 
 
+	gdk_rgb_init ();
 	orb = oaf_init (argc, argv);
 	
 	bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
@@ -122,5 +123,7 @@ main (int argc, char *argv[])
 		bonobo_main ();
 	} while (object_count > 0);
 	
+        gnome_vfs_shutdown ();
+
 	return 0;
 }
