@@ -109,7 +109,7 @@ notes_load_metainfo (Notes *notes)
                 return;
         }
 
-        /* FIXME: should monitor file metadata, not just call_when_ready */
+        /* FIXME bugzilla.eazel.com 4422: should monitor file metadata, not just call_when_ready */
 
         attributes = g_list_append (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
         nautilus_file_call_when_ready (notes->file, attributes, finish_loading_note, notes);
@@ -202,7 +202,8 @@ make_notes_view (BonoboGenericFactory *Factory, const char *goad_id, gpointer cl
         /* handle selections */
         nautilus_clipboard_set_up_editable_in_control
                 (GTK_EDITABLE (notes->note_text_field),
-                 nautilus_view_get_bonobo_control (notes->view));
+                 nautilus_view_get_bonobo_control (notes->view),
+                 FALSE);
 
 /* FIXME bugzilla.eazel.com 4436: 
  * Undo not working in notes-view.
@@ -240,8 +241,9 @@ main(int argc, char *argv[])
 
         gnome_init_with_popt_table("nautilus-notes", VERSION,
 				   argc, argv,
-				   oaf_popt_options, 0, NULL); 
-	orb = oaf_init (argc, argv);
+				   oaf_popt_options, 0, NULL); 	
+	gdk_rgb_init ();
+        orb = oaf_init (argc, argv);
 
         bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
         
@@ -261,5 +263,6 @@ main(int argc, char *argv[])
                 bonobo_main();
         } while (notes_object_count > 0);
         
+        gnome_vfs_shutdown ();
         return EXIT_SUCCESS;
 }
