@@ -21,22 +21,18 @@
  */
 
 #include "nautilus-rpm-view-install.h"
-#include <libtrilobite/libtrilobite.h>
-#include <libeazelinstall.h>
+#include "libtrilobite/libtrilobite.h"
+#include "libeazelinstall.h"
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
 #include <libnautilus-extensions/nautilus-password-dialog.h>
-#include <libtrilobite/libtrilobite.h>
+#include "libtrilobite/libtrilobite.h"
 #include "nautilus-rpm-view-private.h"
 
-/* don't try to access a remote server for install */
-#define DEFAULT_SERVICES_HOST	""
-#define DEFAULT_SERVICES_PORT	80
-
 #define OAF_ID "OAFIID:trilobite_eazel_install_service:8ff6e815-1992-437c-9771-d932db3b4a17"
 
 static void 
 nautilus_rpm_view_download_progress_signal (EazelInstallCallback *service, 
-					    const char *name,
+					    const PackageData *pack,
 					    int amount, 
 					    int total,
 					    NautilusRPMView *rpm_view) 
@@ -88,10 +84,11 @@ nautilus_rpm_view_install_progress_signal (EazelInstallCallback *service,
 
 static void
 nautilus_rpm_view_download_failed (EazelInstallCallback *service, 
-				   const char *name,
+				   const PackageData *pack,
 				   NautilusRPMView *rpm_view)
 {
-	trilobite_debug ("Download of %s FAILED", name);
+	g_assert (pack->name != NULL);
+	trilobite_debug ("Download of %s FAILED", pack->name);
 }
 
 static void
@@ -253,7 +250,7 @@ nautilus_rpm_view_install_done (EazelInstallCallback *service,
 							detailed,
 							GTK_WINDOW (window));
 							
-		/* gnome_dialog_run_and_close (d); */
+		gnome_dialog_run_and_close (d);
 		g_free (terse);
 		g_free (dialog_title);
 		g_free (detailed);
@@ -269,9 +266,8 @@ nautilus_rpm_view_install_done (EazelInstallCallback *service,
 		CORBA_exception_free (&ev);
 	}
 	
-
 	nautilus_rpm_view_finished_working (rpm_view);
-
+	
 	tmp = g_strdup (nautilus_rpm_view_get_uri (rpm_view));
 	nautilus_rpm_view_load_uri (rpm_view, tmp);
 	g_free (tmp);
@@ -389,23 +385,39 @@ preflight_check (EazelInstallCallback *cb, const GList *packages,
 	return TRUE;
 }
 
+static void
+nautilus_rpm_view_set_server (NautilusRPMView *rpm_view,
+			      EazelInstallCallback *cb,
+			      CORBA_Environment *ev)
+{
+	int port;
+	char *host, *p;
+
+	/* get default host/port */
+	host = g_strdup (trilobite_get_services_address ());
+	if ((p = strchr (host, ':')) != NULL) {
+		*p = 0;
+	}
+	/* always go for the no auth port */
+	port = 80;
+
+	GNOME_Trilobite_Eazel_Install__set_server (eazel_install_callback_corba_objref (cb), host, ev);
+	GNOME_Trilobite_Eazel_Install__set_server_port (eazel_install_callback_corba_objref (cb), port, ev);
+
+	/* For now always set auth to FALSE, so users are not required to 
+	   login to services to install local rpm files */
+	GNOME_Trilobite_Eazel_Install__set_auth (eazel_install_callback_corba_objref (cb), FALSE, ev);
+}
 
 void 
 nautilus_rpm_view_install_package_callback (GtkWidget *widget,
                                             NautilusRPMView *rpm_view)
 {
 	GList *categories;
-	PackageData *pack;
 	CategoryData *category;
 	CORBA_Environment ev;
 	EazelInstallCallback *cb;
 
-	/* Check that we're on a redhat system */
-	if (!check_for_redhat ()) {
-		fprintf (stderr, "*** This tool can only be used on RedHat.\n");
-		return;
-	}
-
 	CORBA_exception_init (&ev);
 
 	categories = NULL;
@@ -414,21 +426,19 @@ nautilus_rpm_view_install_package_callback (GtkWidget *widget,
 	gtk_widget_set_sensitive (GTK_WIDGET (rpm_view->details->package_uninstall_button), FALSE);
 	nautilus_view_report_load_underway (nautilus_rpm_view_get_view (rpm_view));
 
-	pack = (PackageData *) gtk_object_get_data (GTK_OBJECT (rpm_view), "packagedata");
-	g_assert (pack);
-
+	g_assert (rpm_view->details->package);
 	category = categorydata_new ();
-	category->packages = g_list_prepend (NULL, pack);
+	category->packages = g_list_prepend (NULL, rpm_view->details->package);
 	categories = g_list_prepend (NULL, category);
 
 	cb = eazel_install_callback_new ();
+
 	rpm_view->details->installer = cb;
 	rpm_view->details->root_client = set_root_client (eazel_install_callback_bonobo (cb), rpm_view);
 	
-	Trilobite_Eazel_Install__set_protocol (eazel_install_callback_corba_objref (cb), Trilobite_Eazel_PROTOCOL_HTTP, &ev);
-	Trilobite_Eazel_Install__set_server (eazel_install_callback_corba_objref (cb), DEFAULT_SERVICES_HOST, &ev);
-	Trilobite_Eazel_Install__set_server_port (eazel_install_callback_corba_objref (cb), DEFAULT_SERVICES_PORT, &ev);
-	
+	GNOME_Trilobite_Eazel_Install__set_protocol (eazel_install_callback_corba_objref (cb), GNOME_Trilobite_Eazel_PROTOCOL_HTTP, &ev);
+	nautilus_rpm_view_set_server (rpm_view, cb, &ev);
+
 	gtk_signal_connect (GTK_OBJECT (cb), "download_progress", nautilus_rpm_view_download_progress_signal, rpm_view);
 	gtk_signal_connect (GTK_OBJECT (cb), "install_progress", nautilus_rpm_view_install_progress_signal, rpm_view);
 	gtk_signal_connect (GTK_OBJECT (cb), "dependency_check", nautilus_rpm_view_dependency_check, rpm_view);
@@ -440,9 +450,8 @@ nautilus_rpm_view_install_package_callback (GtkWidget *widget,
 
 	eazel_install_callback_install_packages (cb, categories, NULL, &ev);
 
-	g_list_foreach (categories, (GFunc)categorydata_destroy_foreach, NULL);
-	gtk_object_set_data (GTK_OBJECT (rpm_view), "packagedata", NULL);
-	
+	/* Leak the categories here */
+
 	CORBA_exception_free (&ev);               
 }
 
@@ -452,16 +461,9 @@ nautilus_rpm_view_uninstall_package_callback (GtkWidget *widget,
 {
 	CategoryData *category;
 	GList *categories;
-	PackageData *pack;
 	CORBA_Environment ev;
 	EazelInstallCallback *cb;		
 
-	/* Check that we're on a redhat system */
-	if (!check_for_redhat ()) {
-		fprintf (stderr, "*** This tool can only be used on RedHat.\n");
-		return;
-	}
-
 	CORBA_exception_init (&ev);
 
 	categories = NULL;
@@ -470,18 +472,18 @@ nautilus_rpm_view_uninstall_package_callback (GtkWidget *widget,
 	gtk_widget_set_sensitive (GTK_WIDGET (rpm_view->details->package_uninstall_button), FALSE);
 	nautilus_view_report_load_underway (nautilus_rpm_view_get_view (rpm_view));
 
-	pack = (PackageData *) gtk_object_get_data (GTK_OBJECT (rpm_view), "packagedata");
-	g_assert (pack);
+	g_assert (rpm_view->details->package);
 	category = categorydata_new ();
-	category->packages = g_list_prepend (NULL, pack);
+	category->packages = g_list_prepend (NULL, rpm_view->details->package);
 	categories = g_list_prepend (NULL, category);
 
 	cb = eazel_install_callback_new ();
+
 	rpm_view->details->installer = cb;
 	rpm_view->details->root_client = set_root_client (eazel_install_callback_bonobo (cb), rpm_view);
 	
-	Trilobite_Eazel_Install__set_protocol (eazel_install_callback_corba_objref (cb), Trilobite_Eazel_PROTOCOL_HTTP, &ev);
-	Trilobite_Eazel_Install__set_tmp_dir (eazel_install_callback_corba_objref (cb), "/tmp/eazel-install", &ev);
+	GNOME_Trilobite_Eazel_Install__set_protocol (eazel_install_callback_corba_objref (cb), GNOME_Trilobite_Eazel_PROTOCOL_HTTP, &ev);
+	GNOME_Trilobite_Eazel_Install__set_tmp_dir (eazel_install_callback_corba_objref (cb), "/tmp/eazel-install", &ev);
 	gtk_signal_connect (GTK_OBJECT (cb), "download_progress", nautilus_rpm_view_download_progress_signal, rpm_view);
 	gtk_signal_connect (GTK_OBJECT (cb), "uninstall_progress", nautilus_rpm_view_install_progress_signal, rpm_view);
 	gtk_signal_connect (GTK_OBJECT (cb), "uninstall_failed", nautilus_rpm_view_uninstall_failed, rpm_view);
@@ -491,9 +493,7 @@ nautilus_rpm_view_uninstall_package_callback (GtkWidget *widget,
 	gtk_signal_connect (GTK_OBJECT (cb), "preflight_check", GTK_SIGNAL_FUNC (preflight_check), NULL);
 	
 	eazel_install_callback_uninstall_packages (cb, categories, NULL, &ev);
-
-	g_list_foreach (categories, (GFunc)categorydata_destroy_foreach, NULL);
-	gtk_object_set_data (GTK_OBJECT (rpm_view), "packagedata", NULL);
+	/* Leak the categories here */
 	
 	CORBA_exception_free (&ev);               
 }
