@@ -27,20 +27,35 @@
    It should be a good basis for writing out-of-proc content views.
  */
 
+/* WHAT YOU NEED TO CHANGE: You need to rename everything. Then look
+ * for the individual CHANGE comments to see some things you could
+ * change to make your view do what you want.  
+ */
+
 #include <config.h>
 #include "nautilus-sample-content-view.h"
 
+/* CHANGE: #ifdef this back in if this component comes as part of
+ * Nautilus. If not, you can't rely on the nautilus-gtk-macros.h header.
+ */
+#if 0
+#include <libnautilus-extensions/nautilus-gtk-macros.h>
+#endif
+
+#include <libnautilus/nautilus-bonobo-ui.h>
 #include <bonobo/bonobo-control.h>
-#include <gtk/gtksignal.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-stock.h>
-#include <libnautilus/nautilus-bonobo-ui.h>
-#include <libnautilus-extensions/nautilus-gtk-macros.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtksignal.h>
 
+/* CHANGE: You probably want some different widget than a label to be
+ * your main view contents.  
+ */
 struct NautilusSampleContentViewDetails {
-	char *location;
-	NautilusView *nautilus_view;
+	char      *location;
+	GtkWidget *label;
 };
 
 static void nautilus_sample_content_view_initialize_class (NautilusSampleContentViewClass *klass);
@@ -53,13 +68,51 @@ static void sample_merge_bonobo_items_callback            (BonoboControl
 							   gboolean                        state,
 							   gpointer                        user_data);
 
-/* FIXME bugzilla.eazel.com 2410: 
- * Should we use this Nautilus-only macro in a class that's
- * supposed to be a sample?
+
+/* CHANGE: #ifdef this back in and remove the else clause if this
+ * component comes as part of Nautilus. If not, you can't rely on the
+ * nautilus-gtk-macros.h header, so remove the #if 0 part, and use the
+ * #else portion.
  */
+
+#if 0
+
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusSampleContentView,
 				   nautilus_sample_content_view,
-				   GTK_TYPE_LABEL)
+				   NAUTILUS_TYPE_VIEW)
+#else
+
+static gpointer parent_class;
+                                                                                                        
+GtkType
+nautilus_sample_content_view_get_type (void)                                                         
+{                                                                                                       
+	GtkType parent_type;                                                                            
+	static GtkType type;                                                                            
+                                                                                                        
+	if (type == 0) {                                                                                
+		static GtkTypeInfo info = {
+		        "NautilusSampleContentView",
+			sizeof (NautilusSampleContentView),
+			sizeof (NautilusSampleContentViewClass),
+			(GtkClassInitFunc)nautilus_sample_content_view_initialize_class,
+			(GtkObjectInitFunc)nautilus_sample_content_view_initialize,
+			NULL,
+			NULL,
+			NULL
+		};
+
+		parent_type = (NAUTILUS_TYPE_VIEW);
+		type = gtk_type_unique (parent_type, &info);
+		parent_class = gtk_type_class (parent_type);
+	}
+
+	return type;
+}
+
+#endif
+
+
      
 static void
 nautilus_sample_content_view_initialize_class (NautilusSampleContentViewClass *klass)
@@ -80,25 +133,25 @@ nautilus_sample_content_view_initialize (NautilusSampleContentView *view)
 
 	view->details = g_new0 (NautilusSampleContentViewDetails, 1);
 	
-	gtk_label_set_text (GTK_LABEL (view), g_strdup (_("(none)")));
+	view->details->label = gtk_label_new (_("(none)"));
+	gtk_widget_show (view->details->label);
 	
-	view->details->nautilus_view = nautilus_view_new (GTK_WIDGET (view));
+	nautilus_view_construct (NAUTILUS_VIEW (view), 
+				 view->details->label);
 	
-	gtk_signal_connect (GTK_OBJECT (view->details->nautilus_view), 
+	gtk_signal_connect (GTK_OBJECT (view), 
 			    "load_location",
 			    sample_load_location_callback, 
-			    view);
+			    NULL);
 
 	/* Get notified when our bonobo control is activated so we can
 	 * merge menu & toolbar items into the shell's UI.
 	 */
-        gtk_signal_connect (GTK_OBJECT (nautilus_view_get_bonobo_control
-					(view->details->nautilus_view)),
+        gtk_signal_connect (GTK_OBJECT (nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view))),
                             "activate",
                             sample_merge_bonobo_items_callback,
                             view);
 	
-	gtk_widget_show (GTK_WIDGET (view));
 }
 
 static void
@@ -110,25 +163,16 @@ nautilus_sample_content_view_destroy (GtkObject *object)
 	
 	g_free (view->details->location);
 	g_free (view->details);
-	
+
+	/* CHANGE: Pick one. */
+#if 0 /* nautilus-specific */	
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+#else /* non-nautilus-specific */
+	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+#endif
 }
 
-/**
- * nautilus_sample_content_view_get_nautilus_view:
- *
- * Return the NautilusView object associated with this view; this
- * is needed to export the view via CORBA/Bonobo.
- *
- * @view: NautilusSampleContentView to get the nautilus_view from..
- **/
-NautilusView *
-nautilus_sample_content_view_get_nautilus_view (NautilusSampleContentView *view)
-{
-	g_return_val_if_fail (NAUTILUS_IS_SAMPLE_CONTENT_VIEW (view), NULL);
 
-	return view->details->nautilus_view;
-}
 
 static void
 load_location (NautilusSampleContentView *view,
@@ -143,10 +187,15 @@ load_location (NautilusSampleContentView *view,
 	view->details->location = g_strdup (location);
 
 	label_text = g_strdup_printf (_("%s\n\nThis is a sample Nautilus content view component."), location);
-	gtk_label_set_text (GTK_LABEL (view), label_text);
+	gtk_label_set_text (GTK_LABEL (view->details->label), label_text);
 	g_free (label_text);
 }
 
+/* CHANGE: Do your own loading here. If loading can be a long-running
+ * operation, you should consider doing it async, in which case you
+ * should only call load_complete when the load is actually done.
+ */
+
 static void
 sample_load_location_callback (NautilusView *nautilus_view, 
 			       const char *location,
@@ -157,9 +206,7 @@ sample_load_location_callback (NautilusView *nautilus_view,
 	g_assert (NAUTILUS_IS_VIEW (nautilus_view));
 	g_assert (location != NULL);
 	
-	view = NAUTILUS_SAMPLE_CONTENT_VIEW (user_data);
-	
-	g_assert (nautilus_view == view->details->nautilus_view);
+	view = NAUTILUS_SAMPLE_CONTENT_VIEW (nautilus_view);
 	
 	/* It's mandatory to send an underway message once the
 	 * component starts loading, otherwise nautilus will assume it
@@ -185,7 +232,9 @@ sample_load_location_callback (NautilusView *nautilus_view,
 }
 
 static void
-bonobo_sample_callback (BonoboUIComponent *ui, gpointer user_data, const char *verb)
+bonobo_sample_callback (BonoboUIComponent *ui, 
+			gpointer           user_data, 
+			const char        *verb)
 {
  	NautilusSampleContentView *view;
 	char *label_text;
@@ -204,12 +253,15 @@ bonobo_sample_callback (BonoboUIComponent *ui, gpointer user_data, const char *v
 					      view->details->location);
 	}
 	
-	gtk_label_set_text (GTK_LABEL (view), label_text);
+	gtk_label_set_text (GTK_LABEL (view->details->label), label_text);
 	g_free (label_text);
 }
 
+/* CHANGE: Do your own menu/toolbar merging here. */
 static void
-sample_merge_bonobo_items_callback (BonoboControl *control, gboolean state, gpointer user_data)
+sample_merge_bonobo_items_callback (BonoboControl *control, 
+				    gboolean       state, 
+				    gpointer       user_data)
 {
  	NautilusSampleContentView *view;
 	BonoboUIComponent *ui_component;
@@ -224,7 +276,7 @@ sample_merge_bonobo_items_callback (BonoboControl *control, gboolean state, gpoi
 	view = NAUTILUS_SAMPLE_CONTENT_VIEW (user_data);
 
 	if (state) {
-		ui_component = nautilus_view_set_up_ui (view->details->nautilus_view,
+		ui_component = nautilus_view_set_up_ui (NAUTILUS_VIEW (view),
 							DATADIR,
 							"nautilus-sample-content-view-ui.xml",
 							"nautilus-sample-content-view");
