@@ -27,7 +27,7 @@
 #include <bonobo.h>
 #include <sys/utsname.h>
 
-#include <eazel-install-types.h>
+#include <eazel-package-system-types.h>
 #include <eazel-install-corba-types.h>
 #include <eazel-install-corba-callback.h>
 #include <eazel-install-problem.h>
@@ -57,7 +57,6 @@ int     arg_dry_run,
 	arg_ftp,
 	arg_local,
 	arg_debug,
-	arg_port,
 	arg_delay,
 	arg_file,
 	arg_force,
@@ -70,11 +69,13 @@ int     arg_dry_run,
 	arg_provides,
 	arg_verbose,
 	arg_id,
-	arg_no_pct;
+	arg_ei2,
+	arg_no_pct,
+	arg_no_auth;
 char    *arg_server,
+	*arg_cgi,
 	*arg_config_file,
 	*arg_package_list,
-	*arg_tmp_dir,
 	*arg_username,
 	*arg_root,
 	*arg_batch;
@@ -85,13 +86,17 @@ CORBA_ORB orb;
 CORBA_Environment ev;
 int cli_result = 0;
 GList *cases = NULL;
+GList *categories;
+gboolean downloaded_files = FALSE;
 
 static const struct poptOption options[] = {
 	{"batch", '\0', POPT_ARG_STRING, &arg_batch, 0, N_("Set the default answer to continue, also default delete to Yes"), NULL},
+	{"cgi-path", '\0', POPT_ARG_STRING, &arg_cgi, 0, N_("Specify search cgi"), NULL},
 	{"debug", '\0', POPT_ARG_NONE, &arg_debug, 0 , N_("Show debug output"), NULL},
 	{"delay", '\0', POPT_ARG_NONE, &arg_delay, 0 , N_("10 sec delay after starting service"), NULL},
 	{"downgrade", 'd', POPT_ARG_NONE, &arg_downgrade, 0, N_("Allow downgrades"), NULL},
 	{"erase", 'e', POPT_ARG_NONE, &arg_erase, 0, N_("Erase packages"), NULL},
+	{"ei2", '\0', POPT_ARG_NONE, &arg_ei2, 0, N_("enable ei2"), NULL},
 	{"file",'\0', POPT_ARG_NONE, &arg_file, 0, N_("RPM args are filename"), NULL},
 	{"force", 'F', POPT_ARG_NONE, &arg_force, 0, N_("Force install"), NULL},
 	{"ftp", 'f', POPT_ARG_NONE, &arg_ftp, 0, N_("Use ftp"), NULL},
@@ -99,16 +104,15 @@ static const struct poptOption options[] = {
 	{"http", 'h', POPT_ARG_NONE, &arg_http, 0, N_("Use http"), NULL},
 	{"id", 'i', POPT_ARG_NONE, &arg_id, 0, N_("RPM args are Eazel Ids"), NULL},
 	{"no-percent", '\0', POPT_ARG_NONE, &arg_no_pct, 0, N_("Don't print fancy percent output"), NULL},
+	{"no-auth", '\0', POPT_ARG_NONE, &arg_no_auth, 0, N_("don't use eazel auth stuff"), NULL},
 	{"packagefile", '\0', POPT_ARG_STRING, &arg_package_list, 0, N_("Specify package file"), NULL},
-	{"port", '\0', POPT_ARG_INT, &arg_port, 0 , N_("Set port numer (80)"), NULL},
 	{"provides", '\0', POPT_ARG_NONE, &arg_provides, 0, N_("RPM args are needed files"), NULL},
 	{"query", 'q', POPT_ARG_NONE, &arg_query, 0, N_("Run Query"), NULL},
 	{"revert", 'r', POPT_ARG_NONE, &arg_revert, 0, N_("Revert"), NULL},
 	{"root", '\0', POPT_ARG_STRING, &arg_root, 0, N_("Set root"), NULL},
 	{"server", '\0', POPT_ARG_STRING, &arg_server, 0, N_("Specify server"), NULL},
-	{"ssl_rename", 's', POPT_ARG_NONE, &arg_ssl_rename, 0, N_("Perform ssl renaming"), NULL},
+	{"ssl-rename", 's', POPT_ARG_NONE, &arg_ssl_rename, 0, N_("Perform ssl renaming"), NULL},
 	{"test", 't', POPT_ARG_NONE, &arg_dry_run, 0, N_("Test run"), NULL},
-	{"tmp", '\0', POPT_ARG_STRING, &arg_tmp_dir, 0, N_("Set tmp dir (/tmp)"), NULL},
 	{"username", '\0', POPT_ARG_STRING, &arg_username, 0, N_("Allow username"), NULL},
 	{"upgrade", 'u', POPT_ARG_NONE, &arg_upgrade, 0, N_("Allow upgrades"), NULL},
 	{"verbose", 'v', POPT_ARG_NONE, &arg_verbose, 0, N_("Verbose output"), NULL},
@@ -125,11 +129,14 @@ static void tree_helper (EazelInstallCallback *service,
 #define check_ev(s) if (ev._major!=CORBA_NO_EXCEPTION) { g_warning ("%s: Caught exception %s", s, CORBA_exception_id (&ev)); }
 
 static void
-set_parameters_from_command_line (Trilobite_Eazel_Install service)
+set_parameters_from_command_line (GNOME_Trilobite_Eazel_Install service)
 {
-	if (!arg_debug) {
-		Trilobite_Eazel_Install__set_log_file (service, DEFAULT_LOG_FILE, &ev);
+	if (!arg_debug) {		
+		GNOME_Trilobite_Eazel_Install__set_log_file (service, DEFAULT_LOG_FILE, &ev);
 		check_ev ("set_log_file");
+	} else {
+		GNOME_Trilobite_Eazel_Install__set_debug (service, TRUE, &ev);
+		check_ev ("set_debug");
 	}
 
 	/* We only want 1 protocol type */
@@ -138,16 +145,16 @@ set_parameters_from_command_line (Trilobite_Eazel_Install service)
 			exit (1);
 	}
 	if (arg_http) {
-		Trilobite_Eazel_Install__set_protocol (service, Trilobite_Eazel_PROTOCOL_HTTP, &ev);
+		GNOME_Trilobite_Eazel_Install__set_protocol (service, GNOME_Trilobite_Eazel_PROTOCOL_HTTP, &ev);
 		check_ev ("set_protocol");
 	} else if (arg_ftp) {
-		Trilobite_Eazel_Install__set_protocol (service, Trilobite_Eazel_PROTOCOL_FTP, &ev);
+		GNOME_Trilobite_Eazel_Install__set_protocol (service, GNOME_Trilobite_Eazel_PROTOCOL_FTP, &ev);
 		check_ev ("set_protocol");
 	} else if (arg_local) {
-		Trilobite_Eazel_Install__set_protocol (service, Trilobite_Eazel_PROTOCOL_LOCAL, &ev);
+		GNOME_Trilobite_Eazel_Install__set_protocol (service, GNOME_Trilobite_Eazel_PROTOCOL_LOCAL, &ev);
 		check_ev ("set_protocol");
 	} else {
-		Trilobite_Eazel_Install__set_protocol (service, Trilobite_Eazel_PROTOCOL_HTTP, &ev);
+		GNOME_Trilobite_Eazel_Install__set_protocol (service, GNOME_Trilobite_Eazel_PROTOCOL_HTTP, &ev);
 		check_ev ("set_protocol");
 	}
 	if (arg_downgrade + arg_upgrade + arg_erase +arg_revert > 1) {
@@ -155,57 +162,82 @@ set_parameters_from_command_line (Trilobite_Eazel_Install service)
 			exit (1);
 	}
 	if (arg_upgrade) {
-		Trilobite_Eazel_Install__set_update (service, TRUE, &ev);
+		GNOME_Trilobite_Eazel_Install__set_update (service, TRUE, &ev);
 		check_ev ("update");
 	}
 	if (arg_downgrade) {
-		Trilobite_Eazel_Install__set_downgrade (service, TRUE, &ev);
+		GNOME_Trilobite_Eazel_Install__set_downgrade (service, TRUE, &ev);
 		check_ev ("downgrade");
 	}
 	if (arg_server) {
-		Trilobite_Eazel_Install__set_server (service, arg_server, &ev);
+		char *colon = strchr (arg_server, ':');
+		if (colon) {
+			char *host;
+			int port;
+
+			host = g_new0(char, (colon - arg_server) + 1);
+			strncpy (host, arg_server, colon - arg_server);
+			colon++;
+			port = atoi (colon);
+			GNOME_Trilobite_Eazel_Install__set_server (service, host, &ev);
+			check_ev ("set_server");
+			GNOME_Trilobite_Eazel_Install__set_server_port (service, port, &ev);
+			check_ev ("set_port");
+			g_free (host);
+		} else {
+			GNOME_Trilobite_Eazel_Install__set_server (service, arg_server, &ev);
+			check_ev ("set_server");
+		}
+		GNOME_Trilobite_Eazel_Install__set_auth (service, FALSE, &ev);
+		check_ev ("set_auth");
+	} else if (arg_no_auth==0) {
+		char *host, *p;
+		int port;
+
+		host = g_strdup (trilobite_get_services_address ());
+		if ((p = strchr (host, ':')) != NULL) {
+			*p = 0;
+			port = atoi (p+1);
+		} else {
+			port = 80;
+		}
+		GNOME_Trilobite_Eazel_Install__set_auth (service, TRUE, &ev);
+		check_ev ("set_auth");
+		GNOME_Trilobite_Eazel_Install__set_server (service, host, &ev);
 		check_ev ("set_server");
+		GNOME_Trilobite_Eazel_Install__set_server_port (service, port, &ev);
+		check_ev ("set_port");
+		g_free (host);
 	}
 	if (arg_username) {
-		Trilobite_Eazel_Install__set_username (service, arg_username, &ev);
+		GNOME_Trilobite_Eazel_Install__set_username (service, arg_username, &ev);
 		check_ev ("set_username");
 	}
+	if (arg_cgi) {
+		GNOME_Trilobite_Eazel_Install__set_cgi (service, arg_cgi, &ev);
+		check_ev ("set_cgi");
+	}
 
 	if (arg_ssl_rename) {
-		Trilobite_Eazel_Install__set_ssl_rename (service, TRUE, &ev);
+		GNOME_Trilobite_Eazel_Install__set_ssl_rename (service, TRUE, &ev);
 		check_ev ("set_ssl_rename");
 	}
-
-#define RANDCHAR ('A' + (rand () % 23))
-	if (arg_tmp_dir == NULL) {
-		int tries;
-		srand (time (NULL));
-		for (tries = 0; tries < 50; tries++) {
-			arg_tmp_dir = g_strdup_printf ("/tmp/eazel-installer.%c%c%c%c%c%c%d",
-						  RANDCHAR, RANDCHAR, RANDCHAR, RANDCHAR,
-						  RANDCHAR, RANDCHAR, (rand () % 1000));
-			if (g_file_test (arg_tmp_dir, G_FILE_TEST_ISDIR)==0) {
-				break;
-			}
-			g_free (arg_tmp_dir);
-		}
-	}
-/*
-	Trilobite_Eazel_Install__set_tmp_dir (service, arg_tmp_dir, &ev);
-	check_ev ("set_tmp_dir");
-*/
-	if (arg_port) {
-		Trilobite_Eazel_Install__set_server_port (service, arg_port, &ev);
-		check_ev ("set_server_port");
+	if (arg_ei2) {
+		GNOME_Trilobite_Eazel_Install__set_ei2 (service, TRUE, &ev);
+		check_ev ("set_ei2");
 	}
+	if (arg_no_auth) {
+		GNOME_Trilobite_Eazel_Install__set_auth (service, FALSE, &ev);
+		check_ev ("set_auth");
+	} 
 	if (arg_dry_run) {
-		Trilobite_Eazel_Install__set_test_mode (service, TRUE, &ev);
+		GNOME_Trilobite_Eazel_Install__set_test_mode (service, TRUE, &ev);
 	}
 	if (arg_force) {
-		Trilobite_Eazel_Install__set_force (service, TRUE, &ev);
+		GNOME_Trilobite_Eazel_Install__set_force (service, TRUE, &ev);
 	}
 	if (arg_package_list) {
-		Trilobite_Eazel_Install__set_package_list (service, arg_package_list, &ev);
+		GNOME_Trilobite_Eazel_Install__set_package_list (service, arg_package_list, &ev);
 		check_ev ("packagelist");
 	}
 
@@ -230,28 +262,59 @@ set_parameters_from_command_line (Trilobite_Eazel_Install service)
 
 static void 
 eazel_download_progress_signal (EazelInstallCallback *service, 
-				const char *name,
+				const PackageData *pack,
 				int amount, 
 				int total,
 				char *title) 
 {
+	static time_t t;
+	static int pct;
+	static int old_pct;
+
+	time_t end;
+	time_t diff;
+	static float ks=0;
+
+	g_assert (pack->name != NULL);
+
+	downloaded_files = TRUE;
+
 	if (amount==0) {
-		fprintf (stdout, "Downloading %s...\n", name);
+		fprintf (stdout, "Downloading %s...", pack->name);
+		t = time (NULL);
+		old_pct = pct = 0;
 	} else if (amount != total ) {
 		if (arg_no_pct==0) {
-			fprintf (stdout, "(%d/%d) = %.1f %%\r", 
-				 amount, total,
-				 (float) (((float) amount * 100.0) / total));
-			fflush (stdout);
+			pct = amount / (total / 100);
+			if (pct > 5) {
+				if (old_pct != pct && pct%5==0) {
+					end = time (NULL);
+					diff = end - t;
+					ks = ((float)amount/1024)/diff;
+					old_pct = pct;
+				}
+				fprintf (stdout, "\rDownloading %s... (%d/%d) = %d%% %.1f Kb/s     \r", 
+					 pack->name,
+					 amount, total, pct,
+					 ks);
+			} else {
+				fprintf (stdout, "\rDownloading %s... (%d/%d) = %d%%", 
+					 pack->name,
+					 amount, total, pct);
+			}
 		}
 	} else if (amount == total && total!=0) {
 		if (arg_no_pct==0) {
-			fprintf (stdout, "(%d/%d) = %.1f %%\r\n",
-				 amount, total, 100.0);
+			fprintf (stdout, "\rDownloading %s... (%d/%d) %.1f Kb/s Done      \n",
+				 pack->name,
+				 amount, total, 
+				 ks);
+		} else {
+			fprintf (stdout, "Downloading %s... %3.1f KB/s Done\n", 
+				 pack->name, ks);
 		}
-		fprintf (stdout, "Done\n");
-		fflush (stdout);
 	}
+	fflush (stdout);
 }
 
 static void 
@@ -263,35 +326,39 @@ eazel_install_progress_signal (EazelInstallCallback *service,
 			       char *title)
 {
 	if (amount==0) {
-		fprintf (stdout, "%s %s: \"%20.20s\"...\n", title, pack->name, pack->description);
+		fprintf (stdout, "%s %s", title, pack->name);
 	} else if (amount != total ) {
 		if (arg_no_pct==0) {
-			fprintf (stdout, "(%d/%d), (%d/%d)b - (%d/%d) = %.1f%%\r", 
+			fprintf (stdout, "\r%s %s (%d/%d), (%d/%d)b - (%d/%d) = %d%%", 
+				 title, pack->name,
 				 package_num, num_packages,
 				 total_size_completed, total_size,
 				 amount, total,
-				 (float) (((float) amount * 100.0) / total));
-			fflush (stdout);
+				 amount / (total / 100));
 		}
 	}
 	if (amount == total && total!=0) {
 		if (arg_no_pct==0) {
-			fprintf (stdout, "(%d/%d), (%d/%d)b - (%d/%d) = %.1f %%\r\n",
+			fprintf (stdout, "\r%s %s (%d/%d), (%d/%d)b - (%d/%d) = %d%% Done\n",
+				 title, pack->name,
 				 package_num, num_packages,
 				 total_size_completed, total_size,
-				 amount, total, 100.0);
+				 amount, total, 100);
+		} else {
+			fprintf (stdout, "Done\n");
 		}
-		fprintf (stdout, "Done\n");
-		fflush (stdout);
+
 	}
+	fflush (stdout);
 }
 
 static void
 download_failed (EazelInstallCallback *service, 
-		 const char *name,
+		 const PackageData *pack,
 		 gpointer unused)
 {
-	fprintf (stdout, "Download of %s FAILED\n", name);
+	g_assert (pack->name != NULL);
+	fprintf (stdout, "Download of %s FAILED\n", pack->name);
 }
 
 /* This is ridiculous... */
@@ -303,12 +370,19 @@ tree_helper_helper(EazelInstallCallback *service,
 		   GList *iterator,
 		   GList *next_list) 
 {
-	PackageData *pack = (PackageData*)iterator->data;
+	PackageData *pack;
 	char *indent2;
 	char indenter;
 	gchar *extra_space=NULL;
 	int indent_level_cnt;
 
+	if (IS_PACKAGEDATA (iterator->data)) {
+		pack = PACKAGEDATA (iterator->data);
+	} else {
+		PackageDependency *dep = PACKAGEDEPENDENCY (iterator->data);
+		pack = dep->package;
+	}
+
 	if (indent_level>0) {
 		extra_space = g_new0 (char, indent_level+1);
 		for (indent_level_cnt = 0; indent_level_cnt < indent_level; indent_level_cnt++) {
@@ -338,18 +412,20 @@ tree_helper (EazelInstallCallback *service,
 	     int indent_level,
 	     char *title)
 {
+	char *readable_name;
 	GList *iterator;
 
 	if (title && pd->toplevel) {
 		fprintf (stdout, title);
 	}
 
-	
+	readable_name = packagedata_get_readable_name (pd);
 	fprintf (stdout, "%s%s%s (%s/%s)\n", 
 		 indent,  indent_type,
-		 rpmname_from_packagedata (pd),
+		 readable_name,
 		 packagedata_status_enum_to_str (pd->status),
 		 packagedata_modstatus_enum_to_str (pd->modify_status));
+	g_free (readable_name);
 
 	for (iterator = pd->soft_depends; iterator; iterator = iterator->next) {		
 		char *tmp;
@@ -372,6 +448,56 @@ tree_helper (EazelInstallCallback *service,
 	}
 }
 
+static void
+something_failed (EazelInstallCallback *service,
+		  const PackageData *pd,
+		  EazelInstallProblem *problem,
+		  gboolean uninstall)
+{
+	char *title;
+	GList *stuff = NULL;	
+
+	gtk_object_ref (GTK_OBJECT (pd));
+
+	if (uninstall) {
+		title = g_strdup_printf ("\nPackage %s failed to uninstall.\n", pd->name);
+	} else {
+		title = g_strdup_printf ("\nPackage %s failed to install.\n", pd->name);
+	}
+
+	if (arg_debug) {
+		tree_helper (service, pd, "", "", 4, title);
+		fprintf (stdout, "\n");
+	} else {
+		fprintf (stdout, "%s", title);
+	}
+
+	g_free (title);
+
+	if (problem) {
+		stuff = eazel_install_problem_tree_to_string (problem, pd, uninstall);
+		if (stuff) {
+			GList *it;
+			for (it = stuff; it; it = g_list_next (it)) {
+				fprintf (stdout, "Problem : %s\n", (char*)(it->data));
+			}
+		}
+		
+		eazel_install_problem_tree_to_case (problem, pd, uninstall, &cases);
+		stuff = eazel_install_problem_cases_to_string (problem, cases);
+		if (cases) {
+			stuff = eazel_install_problem_cases_to_string (problem, cases);
+			if (stuff) {
+				GList *it;
+				for (it = stuff; it; it = g_list_next (it)) {
+					fprintf (stdout, "Solution : %s\n", (char*)(it->data));
+				}
+			}
+		}
+	}
+	gtk_object_unref (GTK_OBJECT (pd));
+}
+
 /*
   This dumps the entire tree for the failed package.
  */
@@ -380,49 +506,15 @@ install_failed (EazelInstallCallback *service,
 		const PackageData *pd,
 		EazelInstallProblem *problem)
 {
-	char *title;
-	GList *stuff = NULL;	
-
-	title = g_strdup_printf ("\nPackage %s failed to install. Here's the tree...\n", pd->name);
-	if (arg_debug) {
-		tree_helper (service, pd, "", "", 4, title);
-	}
-	fprintf (stdout, "\n");
-
-	if (problem) {
-		stuff = eazel_install_problem_tree_to_string (problem, pd);
-		if (stuff) {
-			GList *it;
-			for (it = stuff; it; it = g_list_next (it)) {
-				fprintf (stdout, "Problem : %s\n", (char*)(it->data));
-			}
-		}
-		
-		eazel_install_problem_tree_to_case (problem, pd, &cases);
-		stuff = eazel_install_problem_cases_to_string (problem, cases);
-		if (cases) {
-			stuff = eazel_install_problem_cases_to_string (problem, cases);
-			if (stuff) {
-				GList *it;
-				for (it = stuff; it; it = g_list_next (it)) {
-					fprintf (stdout, "Solution : %s\n", (char*)(it->data));
-			}
-			}
-		}
-	}
-	
-	g_free (title);
+	something_failed (service, pd, problem, FALSE);
 }
 
 static void
 uninstall_failed (EazelInstallCallback *service,
 		  const PackageData *pd,		
-		  gpointer unused)
+		  EazelInstallProblem *problem)
 {
-	char *title;
-	title = g_strdup_printf ("\nPackage %s failed to uninstall. Here's the tree...\n", pd->name);
-	tree_helper (service, pd, "", "", 4, title);
-	g_free (title);
+	something_failed (service, pd, problem, TRUE);
 }
 
 static gboolean
@@ -448,29 +540,15 @@ eazel_preflight_check_signal (EazelInstallCallback *service,
 static void
 dep_check (EazelInstallCallback *service,
 	   const PackageData *package,
-	   const PackageData *needs,
+	   const PackageData *needs_package,
 	   gpointer unused) 
 {
-	if (needs->name && needs->version) {
-		fprintf (stdout, "Doing dependency check for %s-%s - need %s-%s\n", 
-			 package->name, package->version,
-			 needs->name, needs->version);
-	} else if (needs->name) {
-		fprintf (stdout, "Doing dependency check for %s-%s - need %s\n", 
-			 package->name, package->version,
-			 needs->name);
-	} else if (needs->provides) {
-		GList *iterator;
-		fprintf (stdout, "Doing dependency check for %s-%s - need", 
-			 package->name, package->version);
-		for (iterator = needs->provides; iterator; iterator = g_list_next (iterator)) {
-			fprintf (stdout, " %s", (char*)iterator->data);
-		}
-		fprintf (stdout, "\n");
-	} else {
-		fprintf (stdout, "Doing dependency check for %s-%s - needs something, but I don't know what it was...\n", 
-			 package->name, package->version);
-	}
+	char *pack, *needs;
+	pack = packagedata_get_readable_name (package);
+	needs = packagedata_get_readable_name (needs_package);
+	printf ("Dependency : %s needs %s\n", pack, needs);
+	g_free (pack);
+	g_free (needs);
 }
 
 static void
@@ -480,8 +558,8 @@ md5_check_failed (EazelInstallCallback *service,
 		  gpointer unused) 
 {
 	fprintf (stdout, "Package %s failed md5 check!\n", package->name);
-	fprintf (stdout, "server MD5 checksum is %s\n", package->md5);
-	fprintf (stdout, "actual MD5 checksum is %s\n", actual_md5);
+	fprintf (stdout, "\tserver MD5 checksum is %s\n", package->md5);
+	fprintf (stdout, "\tactual MD5 checksum is %s\n", actual_md5);
 }
 
 static PackageData*
@@ -542,7 +620,12 @@ delete_files (EazelInstallCallback *service, EazelInstallProblem *problem)
 		}
 		if (answer[0] == 'y' || answer[0] == 'Y') {
 			fflush (stdout);
-			eazel_install_problem_handle_cases (problem, service, &cases, arg_root);
+			eazel_install_problem_handle_cases (problem, 
+							    service, 
+							    &cases, 
+							    &categories, 
+							    NULL,
+							    arg_root);
 			ask_delete = FALSE;
 			result = FALSE;
 		} else {
@@ -551,7 +634,7 @@ delete_files (EazelInstallCallback *service, EazelInstallProblem *problem)
 		}		
 	} 
 
-	if (ask_delete) {
+	if (downloaded_files && !arg_query && !arg_erase && !arg_file && ask_delete) {
 		printf ("should i delete the RPM files? (y/n) ");
 		fflush (stdout);
 		if (arg_batch) {			
@@ -578,7 +661,7 @@ done (EazelInstallCallback *service,
 	cli_result = result ? 0 : 1;
 
 	if (delete_files (service, problem)) {
-		gtk_main_quit ();
+		trilobite_main_quit ();
 	}
 }
 
@@ -619,7 +702,6 @@ set_root_client (BonoboObjectClient *service)
 int main(int argc, char *argv[]) {
 	poptContext ctxt;
 	GList *packages;
-	GList *categories;
 	char *str;
 	GList *strs;
 	EazelInstallCallback *cb;		
@@ -632,13 +714,15 @@ int main(int argc, char *argv[]) {
 	/* Seems that bonobo_main doens't like
 	   not having gnome_init called, dies in a
 	   X call, yech */
-#if 1
+#if 0
 	gnome_init_with_popt_table ("Eazel Install", "1.0", argc, argv, options, 0, &ctxt);
+	orb = oaf_init (argc, argv);
+	if (!bonobo_init (NULL, NULL, NULL)) {
+		g_error ("Could not init bonobo");
+	}
 #else
-	gtk_type_init ();
-	gnomelib_init ("Eazel Install", "1.0");
-	gnomelib_register_popt_table (options, "Eazel Install");
-	ctxt = gnomelib_parse_args (argc, argv, 0);
+	trilobite_init ("Eazel Install", "1.0", NULL, options, argc, argv);
+	ctxt = trilobite_get_popt_context ();
 #endif
 
 	packages = NULL;
@@ -654,21 +738,10 @@ int main(int argc, char *argv[]) {
 		category->name = g_strdup ("files from commandline");
 		category->packages = packages;
 		categories = g_list_prepend (NULL, category);		
-	} 
-
-	/* Check that we're root and on a redhat system */
-	if (!check_for_redhat ()) {
-		fprintf (stderr, "*** This tool can only be used on RedHat.\n");
-		exit (1);
 	}
 
-	orb = oaf_init (argc, argv);
-	
-	if (!bonobo_init (NULL, NULL, NULL)) {
-		g_error ("Could not init bonobo");
-	}
 	bonobo_activate ();
-	
+
 	cb = eazel_install_callback_new ();
 	problem = eazel_install_problem_new (); 
 	gtk_object_ref (GTK_OBJECT (problem));
@@ -785,7 +858,7 @@ int main(int argc, char *argv[]) {
 	}
 	
 	if (!cli_result && !arg_query) {
-		gtk_main ();
+		trilobite_main ();
 	}
 
 	categorydata_list_destroy (categories);
