@@ -22,6 +22,7 @@
  */
 
 #include <config.h>
+#include <gnome-xml/parser.h>
 #include "eazel-install-corba-callback.h"
 #include "eazel-install-corba-types.h"
 #include "eazel-install-xml-package-list.h"
@@ -63,22 +64,26 @@ static BonoboObjectClass *eazel_install_callback_parent_class;
 static PortableServer_ServantBase__epv base_epv = { NULL, NULL, NULL };
 
 typedef struct {
-	POA_Trilobite_Eazel_InstallCallback poa;
+	POA_GNOME_Trilobite_Eazel_InstallCallback poa;
 	EazelInstallCallback *object;
-} impl_POA_Trilobite_Eazel_InstallCallback;
+} impl_POA_GNOME_Trilobite_Eazel_InstallCallback;
 
 static void
-impl_download_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
-			const char *name,
+impl_download_progress (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
+			const GNOME_Trilobite_Eazel_PackageDataStruct *corbapack,
 			const CORBA_long amount,
 			const CORBA_long total,
 			CORBA_Environment * ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DOWNLOAD_PROGRESS], name, amount, total);
+	PackageData *pack;
+
+	pack = packagedata_from_corba_packagedatastruct (*corbapack);
+	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DOWNLOAD_PROGRESS], pack, amount, total);
+	packagedata_destroy (pack, TRUE);
 }
 
 static CORBA_boolean
-impl_preflight_check (impl_POA_Trilobite_Eazel_InstallCallback *servant,
+impl_preflight_check (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
 		      const CORBA_char *xmlcorbapacks,
 		      const CORBA_long total_bytes,
 		      const CORBA_long total_packages,
@@ -111,17 +116,21 @@ impl_preflight_check (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static void
-impl_download_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
-		      const char *name,
+impl_download_failed (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
+		      const GNOME_Trilobite_Eazel_PackageDataStruct *corbapack,
 		      CORBA_Environment * ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DOWNLOAD_FAILED], name);
+	PackageData *pack;
+
+	pack = packagedata_from_corba_packagedatastruct (*corbapack);
+	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DOWNLOAD_FAILED], pack);
+	packagedata_destroy (pack, TRUE);
 }
 
 static void 
-impl_dep_check (impl_POA_Trilobite_Eazel_InstallCallback *servant,
-		const Trilobite_Eazel_PackageDataStruct *corbapack,
-		const Trilobite_Eazel_PackageDataStruct *corbaneeds, 
+impl_dep_check (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
+		const GNOME_Trilobite_Eazel_PackageDataStruct *corbapack,
+		const GNOME_Trilobite_Eazel_PackageDataStruct *corbaneeds, 
 		CORBA_Environment * ev)
 {
 	PackageData *pack, *needs;
@@ -133,8 +142,8 @@ impl_dep_check (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static void 
-impl_install_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
-		       const Trilobite_Eazel_PackageDataStruct *corbapack,
+impl_install_progress (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
+		       const GNOME_Trilobite_Eazel_PackageDataStruct *corbapack,
 		       const CORBA_long package_num, const CORBA_long num_packages, 
 		       const CORBA_long package_size_completed, const CORBA_long package_size_total,
 		       const CORBA_long total_size_completed, const CORBA_long total_size,
@@ -151,8 +160,8 @@ impl_install_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static void 
-impl_uninstall_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
-			 const Trilobite_Eazel_PackageDataStruct *corbapack,
+impl_uninstall_progress (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
+			 const GNOME_Trilobite_Eazel_PackageDataStruct *corbapack,
 			 const CORBA_long amount,
 			 const CORBA_long total,
 			 CORBA_Environment * ev)
@@ -164,20 +173,19 @@ impl_uninstall_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static void 
-impl_md5_check_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
-		       const Trilobite_Eazel_PackageDataStruct *corbapack,
+impl_md5_check_failed (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
+		       const GNOME_Trilobite_Eazel_PackageDataStruct *corbapack,
 		       const CORBA_char *actual_md5,
 		       CORBA_Environment * ev)
 {
 	PackageData *pack;
-	g_message ("impl_md5_check_failed (..., %s)", actual_md5);
 	pack = packagedata_from_corba_packagedatastruct (*corbapack);
 	gtk_signal_emit (GTK_OBJECT (servant->object), signals[MD5_CHECK_FAILED], pack, actual_md5);
 	packagedata_destroy (pack, TRUE);
 }
 
 static void 
-impl_install_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
+impl_install_failed (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
 		     const CORBA_char *xmlcorbapack,
 		     CORBA_Environment * ev)
 {
@@ -203,7 +211,7 @@ impl_install_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static void 
-impl_uninstall_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
+impl_uninstall_failed (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
 		       const CORBA_char *xmlcorbapack,
 		       CORBA_Environment * ev)
 {
@@ -229,7 +237,7 @@ impl_uninstall_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static CORBA_boolean
-impl_delete_files (impl_POA_Trilobite_Eazel_InstallCallback *servant,
+impl_delete_files (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant,
 		   CORBA_Environment *ev)
 {
 	gboolean result;
@@ -239,19 +247,19 @@ impl_delete_files (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 }
 
 static void 
-impl_done (impl_POA_Trilobite_Eazel_InstallCallback *servant, 
+impl_done (impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant, 
 	   CORBA_boolean result,
 	   CORBA_Environment * ev)
 {
 	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DONE], result);
 }
 
-POA_Trilobite_Eazel_InstallCallback__epv* 
+POA_GNOME_Trilobite_Eazel_InstallCallback__epv* 
 eazel_install_callback_get_epv () 
 {
-	POA_Trilobite_Eazel_InstallCallback__epv *epv;
+	POA_GNOME_Trilobite_Eazel_InstallCallback__epv *epv;
 
-	epv = g_new0 (POA_Trilobite_Eazel_InstallCallback__epv, 1);
+	epv = g_new0 (POA_GNOME_Trilobite_Eazel_InstallCallback__epv, 1);
 	epv->download_progress   = (gpointer)&impl_download_progress;
 	epv->preflight_check     = (gpointer)&impl_preflight_check;
 	epv->dependency_check    = (gpointer)&impl_dep_check;
@@ -267,23 +275,23 @@ eazel_install_callback_get_epv ()
 	return epv;
 };
 
-Trilobite_Eazel_InstallCallback
+GNOME_Trilobite_Eazel_InstallCallback
 eazel_install_callback_create_corba_object (BonoboObject *service) {
-	impl_POA_Trilobite_Eazel_InstallCallback *servant;
+	impl_POA_GNOME_Trilobite_Eazel_InstallCallback *servant;
 	CORBA_Environment ev;
 
 	g_assert (service != NULL);
-	g_assert (IS_EAZEL_INSTALL_CALLBACK (service));
+	g_assert (EAZEL_IS_INSTALL_CALLBACK (service));
 	
 	CORBA_exception_init (&ev);
 	
-	servant = g_new0 (impl_POA_Trilobite_Eazel_InstallCallback,1);
+	servant = g_new0 (impl_POA_GNOME_Trilobite_Eazel_InstallCallback,1);
 	servant->object = EAZEL_INSTALL_CALLBACK (service);
 
-	((POA_Trilobite_Eazel_InstallCallback*) servant)->vepv = 
+	((POA_GNOME_Trilobite_Eazel_InstallCallback*) servant)->vepv = 
 		EAZEL_INSTALL_CALLBACK_CLASS ( GTK_OBJECT (service)->klass)->servant_vepv;
-	POA_Trilobite_Eazel_InstallCallback__init (servant, &ev);
-	ORBIT_OBJECT_KEY (((POA_Trilobite_Eazel_InstallCallback*)servant)->_private)->object = NULL;
+	POA_GNOME_Trilobite_Eazel_InstallCallback__init (servant, &ev);
+	ORBIT_OBJECT_KEY (((POA_GNOME_Trilobite_Eazel_InstallCallback*)servant)->_private)->object = NULL;
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("Cannot instantiate Eazel_InstallCallback corba object");
@@ -295,7 +303,7 @@ eazel_install_callback_create_corba_object (BonoboObject *service) {
 	CORBA_exception_free (&ev);		
 
 	/* Return the bonobo activation of the servant */
-	return (Trilobite_Eazel_InstallCallback) bonobo_object_activate_servant (service, servant);
+	return (GNOME_Trilobite_Eazel_InstallCallback) bonobo_object_activate_servant (service, servant);
 }
 
 
@@ -348,10 +356,10 @@ eazel_install_callback_class_initialize (EazelInstallCallbackClass *klass)
 	object_class->finalize = eazel_install_callback_finalize;
 
 	eazel_install_callback_parent_class = gtk_type_class (bonobo_object_get_type ());
-	klass->servant_vepv = g_new0 (POA_Trilobite_Eazel_InstallCallback__vepv,1);
-	((POA_Trilobite_Eazel_InstallCallback__vepv*)klass->servant_vepv)->_base_epv = &base_epv; 
-	((POA_Trilobite_Eazel_InstallCallback__vepv*)klass->servant_vepv)->Bonobo_Unknown_epv = bonobo_object_get_epv ();
-	((POA_Trilobite_Eazel_InstallCallback__vepv*)klass->servant_vepv)->Trilobite_Eazel_InstallCallback_epv = 
+	klass->servant_vepv = g_new0 (POA_GNOME_Trilobite_Eazel_InstallCallback__vepv,1);
+	((POA_GNOME_Trilobite_Eazel_InstallCallback__vepv*)klass->servant_vepv)->_base_epv = &base_epv; 
+	((POA_GNOME_Trilobite_Eazel_InstallCallback__vepv*)klass->servant_vepv)->Bonobo_Unknown_epv = bonobo_object_get_epv ();
+	((POA_GNOME_Trilobite_Eazel_InstallCallback__vepv*)klass->servant_vepv)->GNOME_Trilobite_Eazel_InstallCallback_epv = 
 		eazel_install_callback_get_epv ();
 
 	signals[DOWNLOAD_PROGRESS] = 
@@ -440,29 +448,31 @@ eazel_install_callback_initialize (EazelInstallCallback *service) {
 	CORBA_Environment ev;
 
 	g_assert (service != NULL);
-	g_assert (IS_EAZEL_INSTALL_CALLBACK (service));
+	g_assert (EAZEL_IS_INSTALL_CALLBACK (service));
 
 	CORBA_exception_init (&ev);
 	service->cb = eazel_install_callback_create_corba_object (BONOBO_OBJECT (service));
 
 	service->installservice_bonobo = bonobo_object_activate (OAF_ID, 0);
 	if ( !service->installservice_bonobo) {
-		g_error ("Cannot activate %s\n", OAF_ID);
+		g_warning ("Cannot activate %s\n", OAF_ID);
+	} else {
+		if (! bonobo_object_client_has_interface (service->installservice_bonobo, "IDL:Trilobite/Service:1.0", &ev)) {
+			g_warning ("Object does not support IDL:Trilobite/Service:1.0");
+		}
+		if (! bonobo_object_client_has_interface (service->installservice_bonobo, "IDL:GNOME/Trilobite/Eazel/Install:1.0", &ev)) {
+			g_warning ("Object does not support IDL:GNOME/Trilobite/Eazel/Install:1.0");
+		}
+		service->installservice_corba = bonobo_object_query_interface (BONOBO_OBJECT (service->installservice_bonobo),
+									       "IDL:GNOME/Trilobite/Eazel/Install:1.0");
 	}
-	if (! bonobo_object_client_has_interface (service->installservice_bonobo, "IDL:Trilobite/Service:1.0", &ev)) {
-		g_error ("Object does not support IDL:Trilobite/Service:1.0");
-	}
-	if (! bonobo_object_client_has_interface (service->installservice_bonobo, "IDL:Trilobite/Eazel/Install:1.0", &ev)) {
-		g_error ("Object does not support IDL:Trilobite/Eazel/Time:1.0");
-	}
-	service->installservice_corba = bonobo_object_query_interface (BONOBO_OBJECT (service->installservice_bonobo),
-								       "IDL:Trilobite/Eazel/Install:1.0");
 
 	/* This sets the bonobo structures in service using the corba object */
 	if (!bonobo_object_construct (BONOBO_OBJECT (service), service->cb)) {
 		g_warning ("bonobo_object_construct failed");
-	}	
-	CORBA_exception_free (&ev);		
+	}
+
+	CORBA_exception_free (&ev);
 }
 
 GtkType
@@ -499,11 +509,15 @@ eazel_install_callback_new (void)
 	EazelInstallCallback *service;
 
 	service = EAZEL_INSTALL_CALLBACK (gtk_object_new (TYPE_EAZEL_INSTALL_CALLBACK, NULL));
+	if (! service->installservice_bonobo || ! service->installservice_corba) {
+		bonobo_object_unref (BONOBO_OBJECT (service));
+		service = NULL;
+	}
 
 	return service;
 }
 
-Trilobite_Eazel_Install 
+GNOME_Trilobite_Eazel_Install 
 eazel_install_callback_corba_objref (EazelInstallCallback *service)
 {
 	return service->installservice_corba;
@@ -521,9 +535,9 @@ eazel_install_callback_install_packages (EazelInstallCallback *service,
 					 const char *root,
 					 CORBA_Environment *ev)
 {
-	Trilobite_Eazel_CategoryStructList *corbacats;
+	GNOME_Trilobite_Eazel_CategoryStructList *corbacats;
 	corbacats = corba_category_list_from_categorydata_list (categories);
-	Trilobite_Eazel_Install_install_packages (service->installservice_corba, 
+	GNOME_Trilobite_Eazel_Install_install_packages (service->installservice_corba, 
 						  corbacats, 
 						  root ? root : "",
 						  service->cb, 
@@ -536,9 +550,9 @@ eazel_install_callback_uninstall_packages (EazelInstallCallback *service,
 					   const char *root,
 					   CORBA_Environment *ev)
 {
-	Trilobite_Eazel_CategoryStructList *corbacats;
+	GNOME_Trilobite_Eazel_CategoryStructList *corbacats;
 	corbacats = corba_category_list_from_categorydata_list (categories);
-	Trilobite_Eazel_Install_uninstall_packages (service->installservice_corba, 
+	GNOME_Trilobite_Eazel_Install_uninstall_packages (service->installservice_corba, 
 						    corbacats, 
 						    root ? root : "",
 						    service->cb, 
@@ -552,12 +566,12 @@ eazel_install_callback_simple_query (EazelInstallCallback *service,
 				     CORBA_Environment *ev)
 {
 	GList *result;
-	Trilobite_Eazel_PackageDataStructList *corbares;
+	GNOME_Trilobite_Eazel_PackageDataStructList *corbares;
 
-	corbares = Trilobite_Eazel_Install_simple_query (service->installservice_corba,
-							 query,
-							 root ? root : "",
-							 ev);
+	corbares = GNOME_Trilobite_Eazel_Install_simple_query (service->installservice_corba,
+							       query,
+							       root ? root : "",
+							       ev);
 	result = packagedata_list_from_corba_packagedatastructlist (*corbares);
 	CORBA_free (corbares); 
 	
@@ -578,7 +592,7 @@ eazel_install_callback_revert_transaction (EazelInstallCallback *service,
 	doc = xmlParseFile (xmlfile);
 	xmlDocDumpMemory (doc, &mem, &size);
 	arg = CORBA_string_dup (mem);
-	Trilobite_Eazel_Install_revert_transaction (service->installservice_corba, 
+	GNOME_Trilobite_Eazel_Install_revert_transaction (service->installservice_corba, 
 						    arg,
 						    root ? root : "",
 						    service->cb,
@@ -592,5 +606,5 @@ void
 eazel_install_callback_delete_files (EazelInstallCallback *service,
 				     CORBA_Environment *ev)
 {
-	Trilobite_Eazel_Install_delete_files (service->installservice_corba, ev);
+	GNOME_Trilobite_Eazel_Install_delete_files (service->installservice_corba, ev);
 }
