@@ -28,7 +28,7 @@
 #include "bonobo.h"
 #include "trilobite-eazel-install.h"
 
-#include "eazel-install-types.h"
+#include "eazel-package-system-types.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -37,8 +37,8 @@ extern "C" {
 #define TYPE_EAZEL_INSTALL_CALLBACK           (eazel_install_callback_get_type ())
 #define EAZEL_INSTALL_CALLBACK(obj)           (GTK_CHECK_CAST ((obj), TYPE_EAZEL_INSTALL_CALLBACK, EazelInstallCallback))
 #define EAZEL_INSTALL_CALLBACK_CLASS(klass)   (GTK_CHECK_CLASS_CAST ((klass), TYPE_EAZEL_INSTALL_CALLBACK, EazelInstallCallbackClass))
-#define IS_EAZEL_INSTALL_CALLBACK(obj)        (GTK_CHECK_TYPE ((obj), TYPE_EAZEL_INSTALL_CALLBACK))
-#define IS_EAZEL_INSTALL_CALLBACK_CLASS(klass)(GTK_CHECK_CLASS_TYPE ((klass), TYPE_EAZEL_INSTALL_CALLBACK))
+#define EAZEL_IS_INSTALL_CALLBACK(obj)        (GTK_CHECK_TYPE ((obj), TYPE_EAZEL_INSTALL_CALLBACK))
+#define EAZEL_IS_INSTALL_CALLBACK_CLASS(klass)(GTK_CHECK_CLASS_TYPE ((klass), TYPE_EAZEL_INSTALL_CALLBACK))
 
 typedef struct _EazelInstallCallback EazelInstallCallback;
 typedef struct _EazelInstallCallbackClass EazelInstallCallbackClass;
@@ -96,10 +96,10 @@ struct _EazelInstallCallbackClass
 struct _EazelInstallCallback
 {
 	BonoboObject parent;
-	Trilobite_Eazel_InstallCallback cb;
+	GNOME_Trilobite_Eazel_InstallCallback cb;
 
 	BonoboObjectClient *installservice_bonobo;
-	Trilobite_Eazel_Install installservice_corba;
+	GNOME_Trilobite_Eazel_Install installservice_corba;
 };
 
 /* Create a new eazel-install-callback object */
@@ -137,9 +137,9 @@ void eazel_install_callback_delete_files (EazelInstallCallback *service,
 
 /* Stuff */
 GtkType                                   eazel_install_callback_get_type   (void);
-POA_Trilobite_Eazel_InstallCallback__epv *eazel_install_callback_get_epv (void);
-Trilobite_Eazel_InstallCallback           eazel_install_callback_create_corba_object (BonoboObject *service);
-Trilobite_Eazel_Install                   eazel_install_callback_corba_objref (EazelInstallCallback *service);
+POA_GNOME_Trilobite_Eazel_InstallCallback__epv *eazel_install_callback_get_epv (void);
+GNOME_Trilobite_Eazel_InstallCallback           eazel_install_callback_create_corba_object (BonoboObject *service);
+GNOME_Trilobite_Eazel_Install                   eazel_install_callback_corba_objref (EazelInstallCallback *service);
 BonoboObjectClient                       *eazel_install_callback_bonobo (EazelInstallCallback *service);
 
 #ifdef __cplusplus
