@@ -28,8 +28,8 @@
  */
 
 #include <config.h>
+#include <libtrilobite/libtrilobite.h>
 #include "eazel-install-metadata.h"
-#include <libtrilobite/trilobite-core-utils.h>
 
 #ifndef EAZEL_INSTALL_SLIM
 #include <gconf/gconf.h>
@@ -40,7 +40,7 @@ static GConfEngine *conf_engine = NULL;
 #define INSTALL_GCONF_PATH	"/apps/eazel-trilobite/install"
 
 #define DEFAULT_SERVER		"services.eazel.com"
-#define DEFAULT_PORT		443
+#define DEFAULT_PORT		80
 #define DEFAULT_CGI_PATH	"/catalog/find"
 
 
@@ -187,7 +187,7 @@ init_default_install_configuration (void)
 	rv->protocol = get_urltype_from_string (temp);
 	g_free (temp);
 
-	rv->pkg_list = get_conf_string ("package-list", "/var/eazel/services/package-list.xml");
+	rv->pkg_list = get_conf_string ("package-list", NULL);
 	rv->transaction_dir = get_conf_string ("transaction-dir", NULL);
 	if (rv->transaction_dir == NULL) {
 		rv->transaction_dir = g_strdup_printf ("%s/.nautilus/transactions", g_get_home_dir ());
@@ -195,7 +195,7 @@ init_default_install_configuration (void)
 
 	rv->mode_verbose = get_conf_boolean ("verbose", TRUE);
 	rv->mode_silent = get_conf_boolean ("silent", FALSE);
-	rv->mode_debug = get_conf_boolean ("debug", TRUE);
+	rv->mode_debug = get_conf_boolean ("debug", FALSE);
 	rv->mode_test = get_conf_boolean ("dry-run", FALSE);
 	rv->mode_force = get_conf_boolean ("force", FALSE);
 	rv->mode_depend = get_conf_boolean ("depend", FALSE);
@@ -209,36 +209,65 @@ TransferOptions *
 init_default_transfer_configuration (void)
 {
 	TransferOptions *rv;
-	char *p;
 
 	check_gconf_init ();
 	rv = g_new0 (TransferOptions, 1);
 
-	rv->port_number = get_conf_int ("server/port", DEFAULT_PORT);
-	rv->hostname = get_conf_string ("server/hostname", DEFAULT_SERVER);
-	rv->username = NULL;
-	if ((p = strchr (rv->hostname, ':')) != NULL) {
-		/* make "server/port" optional -- could just be in "server/hostname" */
-		*p = 0;
-		rv->port_number = atoi (p+1);
-	}
 	rv->tmp_dir = get_conf_string ("server/temp-dir", NULL);
 	rv->rpmrc_file = get_conf_string ("server/rpmrc", "/usr/lib/rpm/rpmrc");
-	rv->cgi_path = get_conf_string ("server/cgi-path", DEFAULT_CGI_PATH);
-	rv->eazel_auth = get_conf_boolean ("server/eazel-auth", TRUE);
 
 	return rv;
 }
 
+void
+eazel_install_configure_softcat (EazelSoftCat *softcat)
+{
+	char *p;
+	char *hostname, *cgi_path;
+	int port;
+
+	check_gconf_init ();
+	port = get_conf_int ("server/port", DEFAULT_PORT);
+	hostname = get_conf_string ("server/hostname", DEFAULT_SERVER);
+	if ((p = strchr (hostname, ':')) != NULL) {
+		/* make "server/port" optional -- could just be in "server/hostname" */
+		*p = 0;
+		port = atoi (p+1);
+	}
+	eazel_softcat_set_server_host (softcat, hostname);
+	eazel_softcat_set_server_port (softcat, port);
+	g_free (hostname);
+
+	cgi_path = get_conf_string ("server/cgi-path", DEFAULT_CGI_PATH);
+	eazel_softcat_set_cgi_path (softcat, cgi_path);
+	g_free (cgi_path);
+	
+	eazel_softcat_set_authn (softcat, get_conf_boolean ("server/eazel-auth", TRUE), NULL);
+}
+
+gboolean
+eazel_install_configure_check_jump_after_install (char **url)
+{
+	char *new_url;
+
+	check_gconf_init ();
+	if (! get_conf_boolean ("jump-after-install", TRUE)) {
+		return FALSE;
+	}
+
+	new_url = get_conf_string ("jump-url", NULL);
+	if (new_url != NULL) {
+		g_free (*url);
+		*url = new_url;
+	}
+	return TRUE;
+}
+
 void 
 transferoptions_destroy (TransferOptions *topts)
 {
 	g_return_if_fail (topts!=NULL);
 
-	g_free (topts->hostname);
-	topts->hostname = NULL;
-	g_free (topts->username);
-	topts->username = NULL;
 	g_free (topts->pkg_list_storage_path);
 	topts->pkg_list_storage_path = NULL;
 	g_free (topts->tmp_dir);
