@@ -28,11 +28,14 @@
 #ifndef EAZEL_INSTALL_METADATA_H
 #define EAZEL_INSTALL_METADATA_H
 
-#include "eazel-install-types.h"
+#include "eazel-package-system-types.h"
+#include "eazel-softcat.h"
 
 InstallOptions *init_default_install_configuration (void);
 TransferOptions *init_default_transfer_configuration (void);
 void transferoptions_destroy (TransferOptions *topts);
 void installoptions_destroy (InstallOptions *iopts);
+void eazel_install_configure_softcat (EazelSoftCat *softcat);
+gboolean eazel_install_configure_check_jump_after_install (char **url);
 
 #endif /* EAZEL_INSTALL_METADATA_H */
