@@ -25,6 +25,8 @@
 #define EAZEL_INSTALL_PRIVATE_H
 
 #include "eazel-install-public.h"
+#include "eazel-package-system.h"
+#include "eazel-softcat.h"
 
 /* Funky define to step a GList iterator one ahead */
 #define glist_step(iterator) iterator = g_list_next (iterator)
@@ -44,30 +46,25 @@ struct _EazelInstallPrivate {
 	int problem_filters; 
 	char *transaction_dir;
 
+	gboolean ei2; /* This for the transition period. ei2==true means use the
+			 new ei2 stuff */
+
 	gboolean ssl_rename; /* If true, rename the hosts part in all urls to 
 				localhost. This is to make stuff work
 				with ssl tunneling */
 
+	gboolean ignore_file_conflicts; /* Instructs the installer to ignore file conflicts */
+
 	gboolean revert; /* If true, the current operation is a reversion */
 
 	/* This holds the files that were downloaded */
 	GList *downloaded_files;
 
 	gboolean use_local_package_list;
-	
-	PackageSystem package_system;
-	union {
-		struct {
-			GHashTable *dbs;
-			unsigned long total_size, 
-				      current_installed_size, 
-				      num_packages, 
-				      packages_installed;
-			int install_flags,
-			    interface_flags,
-			    problem_filters;
-		} rpm;
-	} packsys;
+  	
+	EazelPackageSystem *package_system;
+	/* hacky way to implement the old style signals with the new package system object */
+	unsigned long infoblock[6];
 
 	/* This hash maps from package name-version-release to the 
 	   package. Packages are added in eazel-install-rpm-glue.c do_rpm_install
@@ -94,6 +91,27 @@ struct _EazelInstallPrivate {
 
 	/* used to cancel a download before it finished */
 	gboolean cancel_download;
+
+	/* hacky way to notice if the disk is full */
+	gboolean disk_full;
+
+	/* look in these local directories for packages before downloading */
+	GList *local_repositories;
+
+	/* context to use for softcat queries */
+	EazelSoftCat *softcat;
+
+	/* These are hashtables used in eazel-install-logic2.c */
+	GHashTable *dedupe_hash; /* This hash matches from eazel_id to PackageData objects, 
+				    and is used to quickly identify dupes after getting the softcat
+				    information */
+	GHashTable *dep_ok_hash; /* This hash matches from eazel_id to enum (found in eazel-install-logic2.c), 
+				    and is used during check for dependencies which are required. If the
+				    string occurs in the hash, the package has already been checked
+				    for, and the returned enum denotes the dependency state.
+				    The hash also matches from a eazel_id-sense-version concatenation
+				    to a PackageData for the case where the dependency has both a version
+				    and sense */
 };
 
 #endif /* EAZEL_INSTALL_PRIVATE_H */
