@@ -18,7 +18,9 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Authors: J Shane Culpepper <pepper@eazel.com>
+ * Authors: Eskil Heyn Olsen <eskil@eazel.com>
+ *          Robey Pointer <robey@eazel.com>
+ *          J Shane Culpepper <pepper@eazel.com>
  */
 
 /* eazel-install - services command line install/update/uninstall
@@ -28,13 +30,18 @@
 
 #include "eazel-install-protocols.h"
 #include "eazel-install-private.h"
-#include "eazel-install-xml-package-list.h"
+#include "eazel-softcat.h"
 #include <ghttp.h>
 #include <config.h>
-#include <sys/utsname.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <errno.h>
 
+#include "eazel-package-system.h"
+
 /* We use rpmvercmp to compare versions... */
+#include <rpm/rpmlib.h>
 #include <rpm/misc.h>
 
 #include <libtrilobite/trilobite-core-utils.h>
@@ -43,20 +50,19 @@
 #include <libgnomevfs/gnome-vfs.h>
 #endif /* EAZEL_INSTALL_SLIM */
 
-/* #define EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI */
-
 /* evil evil hack because RPM doesn't understand that a package for i386 is still okay to run on i686! */
 #define ASSUME_ix86_IS_i386
 
 typedef struct {
 	EazelInstall *service;
-	const char *file_to_report;
+	const PackageData *package;
 } gnome_vfs_callback_struct;
 
 typedef gboolean (*eazel_install_file_fetch_function) (gpointer *obj, 
 						       char *url,
 						       const char *file_to_report,
-						       const char *target_file);
+						       const char *target_file,
+						       const PackageData *package);
 
 
 #ifdef EAZEL_INSTALL_SLIM	       
@@ -64,61 +70,42 @@ typedef gboolean (*eazel_install_file_fetch_function) (gpointer *obj,
 gboolean http_fetch_remote_file (EazelInstall *service,
 				 char *url, 
 				 const char *file_to_report,
-				 const char* target_file);
+				 const char* target_file,
+				 const PackageData *package);
 gboolean ftp_fetch_remote_file (EazelInstall *service,
 				char *url, 
 				const char *file_to_report,
-				const char* target_file);
+				const char* target_file,
+				const PackageData *package);
 #else /* EAZEL_INSTALL_SLIM */
 gboolean  gnome_vfs_fetch_remote_file (EazelInstall *service, 
 				       char *url, 
 				       const char *file_to_report, 
-				       const char *target_file);
+				       const char *target_file,
+				       const PackageData *package);
 #endif /* EAZEL_INSTALL_SLIM */
 gboolean local_fetch_remote_file (EazelInstall *service,
 				  char *url, 
 				  const char *file_to_report,
-				  const char* target_file);
+				  const char* target_file,
+				  const PackageData *package);
 
-typedef enum { RPMSEARCH_ENTRY_NAME, RPMSEARCH_ENTRY_PROVIDES, RPMSEARCH_ENTRY_ID } RpmSearchEntry;
-
-/* This method takes a RpmSearch, which describes the thing to search for
-   (ie. a package name or a package that provides file foo).
-   If RPMSEARCH_ENTRY_NAME, data must be a PackageData pointer,
-   if PROVIDES, it must be a string containing the file which is needed.
-   if ID, it must be a string containing the id of the package (provided by the rpm service).
-   It creates a search url when can be used to get info about the package */
-char* get_search_url_for_package (EazelInstall *service, 
-				  RpmSearchEntry, 
-				  const gpointer data);
-
-/* This method takes a RpmSearch, which describes the thing to search for
-   (ie. a package name or a package that provides file foo).
-   If RPMSEARCH_ENTRY_NAME, data must be a PackageData pointer,
-   if PROVIDES, it must be a string containing the file which is needed.
-   if ID, it must be a string containing the id of the package
-   The last argument is a PackageData structure to insert info into, 
-   eg. the url (->filename), the server md5 (->md5).
-   It uses get_search_url_for_package, downloads the contents of the url and
-   parses it and returns a url for the package itself */
-static char* get_url_for_package (EazelInstall *service, 
-				  RpmSearchEntry, 
-				  const gpointer data, 
-				  PackageData *pack);
 
 #ifdef EAZEL_INSTALL_SLIM
 gboolean
 http_fetch_remote_file (EazelInstall *service,
 			char *url, 
 			const char *file_to_report,
-			const char* target_file) 
+			const char* target_file,
+			const PackageData *package) 
 {
         int length, get_failed;
         ghttp_request* request;
         ghttp_status status;
         char* body;
         FILE* file;
-	int total_bytes=0;
+	int total_bytes = 0;
+	int last_flush_bytes = 0;
 	gboolean first_emit;
 	const char *report;
 	char *target_file_premove;
@@ -161,7 +148,7 @@ http_fetch_remote_file (EazelInstall *service,
         }
 
         ghttp_set_header (request, http_hdr_Connection, "close");
-        ghttp_set_header (request, http_hdr_User_Agent, trilobite_get_useragent_string (FALSE, NULL));
+        ghttp_set_header (request, http_hdr_User_Agent, trilobite_get_useragent_string (NULL));
         if (ghttp_prepare (request) != 0) {
                 g_warning (_("Could not prepare http request !"));
                 get_failed = 1;
@@ -178,20 +165,42 @@ http_fetch_remote_file (EazelInstall *service,
 		total_bytes = curStat.bytes_total;
 		/* Ensure first emit is with amount==0 */
 		if (first_emit && total_bytes > 0) {
-			eazel_install_emit_download_progress (service, report, 0, total_bytes);
+			eazel_install_emit_download_progress (service, package, 0, total_bytes);
 			first_emit = FALSE;
 		}
 		/* And that amount==0 & amount==total only occurs once */
 		if (curStat.bytes_read!=0 && (curStat.bytes_read != curStat.bytes_total)) {
 			eazel_install_emit_download_progress (service, 
-							      report,
+							      package,
 							      curStat.bytes_read, 
 							      curStat.bytes_total);
 		}
+
+		/* arbitrary -- flush every 16k or so */
+		if (curStat.bytes_read > last_flush_bytes + 16384) {
+			ghttp_flush_response_buffer (request);
+			length = ghttp_get_body_len (request);
+			body = ghttp_get_body (request);
+			if (body != NULL) {
+				if (fwrite (body, length, 1, file) < 1) {
+					/* probably out of disk space */
+					g_warning (_("DISK FULL: could not write %s"), target_file);
+					service->private->disk_full = TRUE;
+					get_failed = 1;
+					break;
+				}
+			} else {
+				g_warning (_("Could not get request body!"));
+				get_failed = 1;
+				break;
+			}
+			last_flush_bytes = curStat.bytes_read;
+		}
+
 		g_main_iteration (FALSE);
         }
 	/* Last emit amount==total */
-	eazel_install_emit_download_progress (service, report, total_bytes, total_bytes);		
+	eazel_install_emit_download_progress (service, package, total_bytes, total_bytes);
 
         if (ghttp_status_code (request) != 200) {
                 g_warning (_("HTTP error: %d %s"), ghttp_status_code (request),
@@ -202,7 +211,12 @@ http_fetch_remote_file (EazelInstall *service,
 		length = ghttp_get_body_len (request);
 		body = ghttp_get_body (request);
 		if (body != NULL) {
-			fwrite (body, length, 1, file);
+			if (fwrite (body, length, 1, file) < 1) {
+				/* probably out of disk space */
+				g_warning (_("DISK FULL: could not write %s"), target_file);
+				service->private->disk_full = TRUE;
+				get_failed = 1;
+			}
 		} else {
 			g_warning (_("Could not get request body!"));
 			get_failed = 1;
@@ -214,7 +228,11 @@ http_fetch_remote_file (EazelInstall *service,
         if (request) {
                 ghttp_request_destroy (request);
         }
-        fclose (file);
+        if (fclose (file) != 0) {
+		g_warning (_("DISK FULL: could not write %s"), target_file);
+		service->private->disk_full = TRUE;
+		get_failed = 1;
+	}
 
 	if (! get_failed) {
 		rename (target_file_premove, target_file);
@@ -232,7 +250,8 @@ gboolean
 ftp_fetch_remote_file (EazelInstall *service,
 		       char *url, 
 		       const char *file_to_report,
-		       const char* target_file) 
+		       const char* target_file,
+		       const PackageData *package)
 {
 	trilobite_debug (_("Downloading %s..."), url);
 	trilobite_debug (_("FTP not supported yet"));
@@ -249,12 +268,8 @@ gnome_vfs_xfer_callback (GnomeVFSXferProgressInfo *info,
 	static gboolean initial_emit;
 	static gboolean last_emit;
 	EazelInstall *service = cbstruct->service;
-	const char *file_to_report = cbstruct->file_to_report;
+	const PackageData *package = cbstruct->package;
 
-#ifdef GTK_MAIN_ITERATION_ACTUALLY_CHECKED_FOR_CORBA_CALLS_WHICH_IT_APPARENTLY_DOESNT
-	gtk_main_iteration_do (FALSE);
-#endif
-	
 	switch (info->status) {
 	case GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR:
 		trilobite_debug ("GnomeVFS Error: %s\n",
@@ -288,19 +303,19 @@ gnome_vfs_xfer_callback (GnomeVFSXferProgressInfo *info,
 		case GNOME_VFS_XFER_PHASE_COPYING:
 			if (initial_emit && info->file_size>0) {
 				initial_emit = FALSE;
-				eazel_install_emit_download_progress (service, 
-								      file_to_report ? file_to_report : info->source_name,
+				eazel_install_emit_download_progress (service,
+								      package,
 								      0,
 								      info->file_size);
 			} else if (!last_emit && info->bytes_copied == info->file_size) {
 				last_emit = TRUE;
 				eazel_install_emit_download_progress (service, 
-								      file_to_report ? file_to_report : info->source_name,
+								      package,
 								      info->file_size,
 								      info->file_size);
 			} else if (info->bytes_copied > 0) {
 				eazel_install_emit_download_progress (service, 
-								      file_to_report ? file_to_report : info->source_name,
+								      package,
 								      info->bytes_copied,
 								      info->file_size);
 
@@ -329,7 +344,7 @@ gnome_vfs_xfer_callback (GnomeVFSXferProgressInfo *info,
 			if (!last_emit) {
 				last_emit = TRUE;
 				eazel_install_emit_download_progress (service, 
-								      file_to_report ? file_to_report : info->source_name,
+								      package,
 								      info->file_size,
 								      info->file_size);
 			}
@@ -354,22 +369,21 @@ gboolean
 gnome_vfs_fetch_remote_file (EazelInstall *service, 
 			     char *url, 
 			     const char *file_to_report, 
-			     const char *target_file)
+			     const char *target_file,
+			     const PackageData *package)
 {
 	GnomeVFSResult result;
 	GnomeVFSXferOptions xfer_options = 0;
 	GnomeVFSURI *src_uri;
 	GnomeVFSURI *dest_uri;
 	char *t_file;
+	char *target_file_premove;
 	gnome_vfs_callback_struct *cbstruct;
 
-	/* Ensure the target_file has a protocol://,
-	   if not, prefix a file:// */
-	if (strstr (target_file, "://")==NULL) {
-		t_file = g_strdup_printf ("file://%s", target_file);
-	} else {
-		t_file = g_strdup (target_file);
-	}
+	target_file_premove = g_strdup_printf ("%s~", target_file);
+
+	/* this will always be a file: uri */
+	t_file = g_strdup_printf ("file://%s", target_file_premove);
 
 	trilobite_debug ("gnome_vfs_xfer_uri ( %s %s )", url, t_file);
 	
@@ -380,14 +394,14 @@ gnome_vfs_fetch_remote_file (EazelInstall *service,
 		gnome_vfs_uri_set_host_name (src_uri, "localhost");
 	}
 	
-	dest_uri = gnome_vfs_uri_new (t_file);
+	dest_uri = gnome_vfs_uri_new (target_file_premove);
 	g_assert (dest_uri != NULL);
 	
 	/* Setup the userdata for the callback, I need both the
 	   service object to emit signals too, and the filename to report */
 	cbstruct = g_new0 (gnome_vfs_callback_struct, 1);
 	cbstruct->service = service;
-	cbstruct->file_to_report = file_to_report;
+	cbstruct->package = package;
 
 	/* Execute the gnome_vfs copy */
 	service->private->cancel_download = FALSE;
@@ -399,7 +413,9 @@ gnome_vfs_fetch_remote_file (EazelInstall *service,
 				     cbstruct);
 
 	if (result==GNOME_VFS_OK) {
-		trilobite_debug ("File download successfull");
+		chmod (target_file, 0600);
+		trilobite_debug ("File download successful");
+		rename (target_file_premove, target_file);
 	} else {
 		trilobite_debug ("File download failed");
 		if (result == GNOME_VFS_ERROR_BAD_PARAMETERS) {
@@ -409,7 +425,7 @@ gnome_vfs_fetch_remote_file (EazelInstall *service,
 			trilobite_debug ("download was cancelled from afar");
 		}
 	}
- 
+
 	/* Free the various stuff */
 	g_free (t_file);
 	g_free (cbstruct);
@@ -426,7 +442,8 @@ gboolean
 local_fetch_remote_file (EazelInstall *service,
 			 char *url, 
 			 const char *file_to_report,
-			 const char* target_file) 
+			 const char* target_file,
+			 const PackageData *package)
 {
 	gboolean result;
 	const char *report;
@@ -438,8 +455,8 @@ local_fetch_remote_file (EazelInstall *service,
 		struct stat sbuf;
 		stat (target_file, &sbuf);
 		/* Emit bogus download progress */
-		eazel_install_emit_download_progress (service, report,            0, sbuf.st_size);
-		eazel_install_emit_download_progress (service, report, sbuf.st_size, sbuf.st_size);
+		eazel_install_emit_download_progress (service, package,            0, sbuf.st_size);
+		eazel_install_emit_download_progress (service, package, sbuf.st_size, sbuf.st_size);
 		result = TRUE;
 	} 
 	return result;
@@ -463,13 +480,56 @@ eazel_install_fill_file_fetch_table (void)
 	return res;
 }
 
+static void
+my_copy_file (const char *orig, const char *dest)
+{
+	int ofd, nfd;
+	char buffer[1024];
+	int n, n2, count;
+
+	ofd = open (orig, O_RDONLY);
+	if (ofd < 0) {
+		return;
+	}
+	nfd = open (dest, O_CREAT|O_WRONLY, 0644);
+	if (nfd < 0) {
+		close (ofd);
+		return;
+	}
+
+	while (1) {
+		n = read (ofd, buffer, sizeof (buffer));
+		if (n <= 0) {
+			break;
+		}
+		count = 0;
+		while (count < n) {
+			n2 = write (nfd, buffer+count, n-count);
+			if (n2 < 0) {
+				/* error */
+				close (ofd);
+				close (nfd);
+				unlink (dest);
+				return;
+			}
+			count += n2;
+		}
+	}
+	close (ofd);
+	close (nfd);
+	return;
+}
+
 gboolean
 eazel_install_fetch_file (EazelInstall *service,
 			  char *url, 
 			  const char *file_to_report,
-			  const char* target_file) 
+			  const char* target_file,
+			  const PackageData *package)
 {
 	gboolean result;
+	GList *iter;
+	char *filename;
 
 	static eazel_install_file_fetch_function *func_table = NULL;
 	
@@ -482,21 +542,37 @@ eazel_install_fetch_file (EazelInstall *service,
 	g_return_val_if_fail (url!=NULL, FALSE);
 	g_return_val_if_fail (target_file!=NULL, FALSE);
 
+	for (iter = g_list_first (service->private->local_repositories); iter != NULL; iter = g_list_next (iter)) {
+		filename = g_strdup_printf ("%s/%s", (char *)(iter->data), g_basename (target_file));
+		if (g_file_test (filename, G_FILE_TEST_ISFILE)) {
+			/* copy this file to target_file */
+			trilobite_debug ("%s found at %s, copying", file_to_report, filename);
+			my_copy_file (filename, target_file);
+		}
+		g_free (filename);
+	}
+
 	if (g_file_test (target_file, G_FILE_TEST_ISFILE)) {
+		/* File is already present, so just emit to progress callbacks and get on with
+		   your life */
+		struct stat buf;
+		stat (target_file, &buf);
 		trilobite_debug ("%s already present, not downloading", target_file);
+		eazel_install_emit_download_progress (service, package, 0, buf.st_size);
+		eazel_install_emit_download_progress (service, package, buf.st_size, buf.st_size);
 		result = TRUE;
 	} else {
 		result = (func_table [eazel_install_get_protocol (service)])((gpointer)service, 
 									     url, 
 									     file_to_report, 
-									     target_file);
+									     target_file,
+									     package);
 	}
  	
 	if (!result) {
 		g_warning (_("Failed to retrieve %s!"), 
 			   file_to_report ? file_to_report : g_basename (target_file));
-		eazel_install_emit_download_failed (service, 
-						    file_to_report ? file_to_report : g_basename (target_file));
+		eazel_install_emit_download_failed (service, package);
 	} 
 
 	return result;
@@ -536,29 +612,27 @@ eazel_install_fetch_package (EazelInstall *service,
 	char *name = g_strdup (package->name);
 	char *version = g_strdup (package->version);
 
-	switch (eazel_install_get_protocol (service)) {
-	case PROTOCOL_FTP:
-	case PROTOCOL_HTTP: 
-	{
-		if (package->remote_url) {
-			url = g_strdup (package->remote_url);
-		} else if (package->eazel_id) {
-			url = get_url_for_package (service, RPMSEARCH_ENTRY_ID, 
-						   package->eazel_id, 
-						   package);
-		} else if (g_list_length (package->provides)==1) {
-			url = get_url_for_package (service, RPMSEARCH_ENTRY_PROVIDES, 
-						   package->provides->data,
-						   package);
-		} else {
-			url = get_url_for_package (service, RPMSEARCH_ENTRY_NAME, package, package);
-		}
+	if (package->remote_url != NULL) {
+		url = g_strdup (package->remote_url);
+	} else {
+		switch (eazel_install_get_protocol (service)) {
+		case PROTOCOL_FTP:
+		case PROTOCOL_HTTP: 
+			{
+				if (eazel_softcat_get_info (service->private->softcat, package, EAZEL_SOFTCAT_SENSE_GE,
+							    PACKAGE_FILL_NO_PROVIDES | PACKAGE_FILL_NO_DEPENDENCIES) 
+				    == EAZEL_SOFTCAT_SUCCESS) {
+					url = g_strdup (package->remote_url);
+				} else {
+					url = NULL;
+				}
+			}
+			break;
+		case PROTOCOL_LOCAL:
+			url = g_strdup_printf ("%s", rpmfilename_from_packagedata (package));
+			break;
+		};
 	}
-	break;
-	case PROTOCOL_LOCAL:
-		url = g_strdup_printf ("%s", rpmfilename_from_packagedata (package));
-		break;
-	};
 
 	if (url == NULL) {
 		char *rname = packagedata_get_readable_name (package);
@@ -568,17 +642,13 @@ eazel_install_fetch_package (EazelInstall *service,
 		targetname = g_strdup_printf ("%s/%s",
 					      eazel_install_get_tmp_dir (service),
 					      filename_from_url (url));
-#ifndef EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI
-		result = eazel_install_fetch_file (service, url, package->name, targetname);
-#else /*  EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI */
-		if (filename_from_url (url) && strlen (filename_from_url (url))>1) {
-			result = eazel_install_fetch_file (service, url, package->name, targetname);
-		} else {
-			trilobite_debug ("cannot handle %s", url);
-		}
-#endif /* EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI */
+		result = eazel_install_fetch_file (service, url, package->name, targetname, package);
 		if (result) {
-			packagedata_fill_from_file (package, targetname); 
+			package = eazel_package_system_load_package (service->private->package_system,
+								     package, 
+								     targetname,
+								     PACKAGE_FILL_NO_DIRS_IN_PROVIDES|PACKAGE_FILL_NO_DEPENDENCIES);
+							   
 			if (name) {
 				if (strcmp (name, package->name)) {
 					g_warning (_("Downloaded package does not have the correct name"));
@@ -612,293 +682,14 @@ eazel_install_fetch_package (EazelInstall *service,
 		g_warning (_("File download failed"));
 		unlink (targetname);
 	} 
+	g_free (name);
+	g_free (version);
 	g_free (targetname);
 	g_free (url);
 
 	return result;
 }
 
-#define EVILCHAR(c)  (((c) == '+') || ((c) < '-') || ((c) == '?') || ((c) == '\\') || ((c) > 'z'))
-static void
-add_to_url (char **url,
-	    const char *cgi_string,
-	    const char *val)
-{
-#ifndef EAZEL_INSTALL_SLIM
-	char *tmp;
-
-	if (val) {
-		tmp = g_strconcat (*url, 
-				   cgi_string, 
-				   gnome_vfs_escape_string (val), 
-				   NULL);
-		g_free (*url);
-		(*url) = tmp;
-	} 
-#else /*  EAZEL_INSTALL_SLIM */
-	char *tmp, *quoted, *q;
-	const char *p;
-	int needs_quoting;
-
-	g_assert (url != NULL);
-	g_assert ((*url) != NULL);
-	g_assert (cgi_string != NULL);
-
-	needs_quoting = 0;
-	for (p = val; p && *p; p++) {
-		if (EVILCHAR (*p)) {
-			needs_quoting++;
-		}
-	}
-
-	if (needs_quoting) {
-		/* url quote the sucker. */
-		q = quoted = g_malloc (strlen (val) + (needs_quoting*2) + 1);
-		for (p = val; p && *p; p++) {
-			if (EVILCHAR (*p)) {
-				*q++ = '%';
-				*q++ = "0123456789ABCDEF"[*p / 16];
-				*q++ = "0123456789ABCDEF"[*p % 16];
-			} else {
-				*q++ = *p;
-			}
-		}
-		*q = 0;
-	} else {
-		quoted = (char *)val;
-	}
-
-	if (quoted) {
-		tmp = g_strconcat (*url, cgi_string, quoted, NULL);
-		g_free (*url);
-		(*url) = tmp;
-	}
-
-	if (needs_quoting) {
-		g_free (quoted);
-	}
-#endif /*  EAZEL_INSTALL_SLIM */
-}
-
-static char*
-get_url_for_package  (EazelInstall *service, 
-		      RpmSearchEntry entry,
-		      gpointer data,
-		      PackageData *out_package)		      
-{
-	char *search_url = NULL;
-	char *url = NULL;
-	char *body = NULL;
-	int length;
-
-	search_url = get_search_url_for_package (service, entry, data);
-	if (search_url == NULL) {
-		trilobite_debug ("No search URL");
-		return NULL;
-	}
-
-	trilobite_debug ("Search URL: %s", search_url);
-
-	trilobite_setenv ("GNOME_VFS_HTTP_USER_AGENT", trilobite_get_useragent_string (FALSE, NULL), TRUE);
-
-	if (trilobite_fetch_uri (search_url, &body, &length)) {
-#ifndef EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI
-		/* Parse the returned xml */
-		GList *packages;
-		
-		packages = parse_osd_xml_from_memory (body, length);
-		if (g_list_length (packages) == 0) {
-			trilobite_debug ("No url for file");
-		} else if (g_list_length (packages) > 1) {
-			trilobite_debug ("Ugh, more then one match, using first");
-		}
-		
-		if (g_list_length (packages) > 0) {
-			/* Get the first package returned */
-			PackageData *pack;
-			
-			g_assert (packages->data != NULL);
-			pack = (PackageData*)packages->data;
-			out_package->remote_url = g_strdup (pack->remote_url);
-			url = g_strdup (pack->remote_url);
-			out_package->md5 = g_strdup (pack->md5);
-			if (! out_package->name) {
-				out_package->name = g_strdup (pack->name);
-			}
-			if (! out_package->version) {
-				out_package->version = g_strdup (pack->version);
-			}
-			if (! out_package->summary) {
-				out_package->summary = g_strdup (pack->summary);
-			}
-			if (! out_package->description) {
-				out_package->description = g_strdup (pack->description);
-			}
-			if (out_package->bytesize == 0) {
-				out_package->bytesize = pack->bytesize;
-			}
-			
-			g_list_foreach (packages, 
-					(GFunc)packagedata_destroy, 
-					GINT_TO_POINTER (TRUE));
-			g_list_free (packages);
-		}						
-#else /* EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI */
-		trilobite_debug ("using old cgi");
-		if (body) {
-			/* body is already null-terminated, luckily */
-			url = g_strdup (body);
-		}
-#endif /* EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI */
-		g_free (body);				
-	} else {
-		switch (entry) {
-		case RPMSEARCH_ENTRY_NAME:
-			g_warning (_("Could not retrieve a URL for %s"), 
-				   rpmfilename_from_packagedata ((PackageData*)data));
-			trilobite_debug ("entry type was NAME");
-			break;
-		case RPMSEARCH_ENTRY_PROVIDES:
-			g_warning (_("Could not retrieve a URL for %s"),
-				   (char*)data);
-			trilobite_debug ("entry type was PROVIDES");
-			break;
-		case RPMSEARCH_ENTRY_ID:
-			g_warning (_("Could not retrieve a URL for id %s"),
-				   (char *)data);
-			trilobite_debug ("entry type was ID");
-			break;
-		}
-	}
-	
-	g_free (search_url);
-	return url;
-}
-
-
-static char *
-real_arch_name (const char *arch)
-{
-	char *arch_copy;
-
-	arch_copy = g_strdup (arch);
-#ifdef ASSUME_ix86_IS_i386
-	if ((strlen (arch_copy) == 4) && (arch_copy[0] == 'i') &&
-	    ((arch_copy[1] >= '3') && (arch_copy[1] <= '9')) &&
-	    (arch_copy[2] == '8') && (arch_copy[3] == '6')) {
-		arch_copy[1] = '3';
-	}
-#endif
-	return arch_copy;
-}
-
-static char*
-get_eazel_auth_path (EazelInstall *service)
-{
-	char *result = NULL;
-	if (eazel_install_get_username (service)) {
-		result = g_strdup_printf ("//%s%s",
-					  eazel_install_get_username (service),
-					  eazel_install_get_cgi_path (service));
-	} else {
-		result = g_strdup (eazel_install_get_cgi_path (service));
-	}
-	return result;
-}
-
-char* get_search_url_for_package (EazelInstall *service, 
-				  RpmSearchEntry entry,
-				  const gpointer data)
-{
-	char *url;
-	DistributionInfo dist;
-	char *arch;
-
-	if (! strlen (eazel_install_get_server (service))) {
-		return NULL;
-	}
-
-	if (eazel_install_get_eazel_auth (service)) {
-		char *cgipath;
-		cgipath = get_eazel_auth_path (service);
-		g_assert (cgipath);
-		/* use eazel-auth: uri */
-		url = g_strdup_printf ("eazel-services:%s", cgipath);
-		g_free (cgipath);
-	} else {
-		url = g_strdup_printf ("http://%s:%d%s",
-				       eazel_install_get_server (service),
-				       eazel_install_get_server_port (service),
-				       eazel_install_get_cgi_path (service));
-	}
-
-	dist = trilobite_get_distribution ();
-
-	switch (entry) {
-	case RPMSEARCH_ENTRY_NAME: {
-		PackageData *pack;
-
-		pack = (PackageData*)data;
-		arch = real_arch_name (pack->archtype);
-		add_to_url (&url, "?name=", pack->name);
-		add_to_url (&url, "&arch=", arch);
-		add_to_url (&url, "&version=", pack->version); 
-		/* FIXME bugzilla.eazel.com 3482
-		   support other flags then 8 
-		*/
-		if (pack->version) {
-			if (service->private->revert) {
-				add_to_url (&url, "&flags=", "8");
-			} else {
-				add_to_url (&url, "&flags=", "12");
-			}
-		}
-		if (pack->distribution.name != DISTRO_UNKNOWN) {
-			dist = pack->distribution;
-		}
-		g_free (arch);
-	}
-	break;
-	case RPMSEARCH_ENTRY_PROVIDES: {
-		struct utsname buf;
-
-		uname (&buf);
-		arch = real_arch_name (buf.machine);
-		add_to_url (&url, "?provides=", (char*)data);
-		add_to_url (&url, "&arch=", arch);
-		/* hack, FIXME bugzilla.eazel.com 3481 
-		add_to_url (&url, "&flag=", "0");
-		add_to_url (&url, "&version=", "-");
-		*/
-		g_free (arch);
-	}
-	break;
-	case RPMSEARCH_ENTRY_ID: {
-		struct utsname buf;
-
-		uname (&buf);
-		arch = real_arch_name (buf.machine);
-		add_to_url (&url, "?rpm_id=", (char *)data);
-		add_to_url (&url, "&arch=", arch);
-		g_free (arch);
-	}
-	break;
-	}
-
-#ifndef EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI
-	if (dist.name != DISTRO_UNKNOWN) {
-		char *distro;
-		distro = g_strdup_printf ("%s", 
-					  trilobite_get_distribution_name (dist, TRUE, TRUE));
-		add_to_url (&url, "&distro=", distro);
-		g_free (distro);
-	}
-#endif /* EAZEL_INSTALL_PROTOCOL_USE_OLD_CGI */
-
-	add_to_url (&url, "&protocol=", protocol_as_string (eazel_install_get_protocol (service)));
-
-	return url;
-}
 
 static void
 flatten_tree_func (PackageData *pack, GList **out)
@@ -912,6 +703,8 @@ flatten_tree_func (PackageData *pack, GList **out)
 	pack->hard_depends = pack->soft_depends = NULL;
 }
 
+/* this was never used yet */
+#if 0
 /* given a list of packages with incomplete info (for example, the initial bootstrap install list),
  * go ask for real info and compile a new list of packages.
  */
@@ -931,7 +724,7 @@ eazel_install_fetch_definitive_package_info (EazelInstall *service, PackageData
 	}
 
 	trilobite_debug ("Search URL: %s", search_url);
-	trilobite_setenv ("GNOME_VFS_HTTP_USER_AGENT", trilobite_get_useragent_string (FALSE, NULL), TRUE);
+	trilobite_setenv ("GNOME_VFS_HTTP_USER_AGENT", trilobite_get_useragent_string (NULL), TRUE);
 
 	if (! trilobite_fetch_uri (search_url, &body, &length)) {
 		g_free (search_url);
@@ -974,3 +767,4 @@ eazel_install_fetch_definitive_category_info (EazelInstall *service, CategoryDat
 	g_list_free (category->packages);
 	category->packages = real_packages;
 }
+#endif
