@@ -36,7 +36,7 @@
 #include "trilobite-eazel-install.h"
 #endif /*  EAZEL_INSTALL_NO_CORBA */
 
-#include "eazel-install-types.h"
+#include "eazel-package-system-types.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -45,13 +45,23 @@ extern "C" {
 #define TYPE_EAZEL_INSTALL           (eazel_install_get_type ())
 #define EAZEL_INSTALL(obj)           (GTK_CHECK_CAST ((obj), TYPE_EAZEL_INSTALL, EazelInstall))
 #define EAZEL_INSTALL_CLASS(klass)   (GTK_CHECK_CLASS_CAST ((klass), TYPE_EAZEL_INSTALL, EazelInstallClass))
-#define IS_EAZEL_INSTALL(obj)        (GTK_CHECK_TYPE ((obj), TYPE_EAZEL_INSTALL))
-#define IS_EAZEL_INSTALL_CLASS(klass)(GTK_CHECK_CLASS_TYPE ((klass), TYPE_EAZEL_INSTALL))
+#define EAZEL_IS_INSTALL(obj)        (GTK_CHECK_TYPE ((obj), TYPE_EAZEL_INSTALL))
+#define EAZEL_IS_INSTALL_CLASS(klass)(GTK_CHECK_CLASS_TYPE ((klass), TYPE_EAZEL_INSTALL))
 
 typedef enum {
-	EAZEL_INSTALL_USE_RPM
-} PackageSystem;;
+	EAZEL_INSTALL_NOTHING = 0,
+	EAZEL_INSTALL_INSTALL_OK = 1<<0,
+	EAZEL_INSTALL_UNINSTALL_OK = 1<<1,
+	EAZEL_INSTALL_REVERSION_OK = 1<<2
+} EazelInstallOperationStatus;
 
+typedef enum {
+	EAZEL_INSTALL_STATUS_NEW_PACKAGE,
+	EAZEL_INSTALL_STATUS_UPGRADES,
+	EAZEL_INSTALL_STATUS_DOWNGRADES,
+	EAZEL_INSTALL_STATUS_QUO
+} EazelInstallStatus;
+	
 typedef struct _EazelInstall EazelInstall;
 typedef struct _EazelInstallClass EazelInstallClass;
 
@@ -63,7 +73,7 @@ struct _EazelInstallClass
 	BonoboObjectClass parent_class;
 #endif /* EAZEL_INSTALL_NO_CORBA */
 	/* signal prototypes */
-	void (*download_progress) (EazelInstall *service, const char *file, int amount, int total);
+	void (*download_progress) (EazelInstall *service, const PackageData *package, int amount, int total);
 
 	gboolean (*preflight_check) (EazelInstall *service, 
 				     GList *packages,
@@ -79,7 +89,7 @@ struct _EazelInstallClass
 	/* 
 	   if the set URLType is PROTOCOL_HTTP, info is a HTTPError struc 
 	*/
-	void (*download_failed) (EazelInstall *service, const char *name);
+	void (*download_failed) (EazelInstall *service, const PackageData *package);
 	/*
 	  if RPM_FAIL is RPM_SRC_NOT_SUPPORTED, info is NULL
 	                 RPM_DEP_FAIL, info is a GSList of required packages (PackageData objects)
@@ -105,7 +115,7 @@ struct _EazelInstall
 	GtkObject parent;
 #else 
 	BonoboObject parent;
-	Trilobite_Eazel_InstallCallback callback;
+	GNOME_Trilobite_Eazel_InstallCallback callback;
 #endif /* EAZEL_INSTALL_NO_CORBA */
 	EazelInstallPrivate *private;
 };
@@ -116,8 +126,8 @@ GtkType                        eazel_install_get_type   (void);
 void                           eazel_install_unref       (GtkObject *object);
 
 #ifndef EAZEL_INSTALL_NO_CORBA
-POA_Trilobite_Eazel_Install__epv *eazel_install_get_epv (void);
-Trilobite_Eazel_Install eazel_install_create_corba_object (BonoboObject *service);
+POA_GNOME_Trilobite_Eazel_Install__epv *eazel_install_get_epv (void);
+GNOME_Trilobite_Eazel_Install eazel_install_create_corba_object (BonoboObject *service);
 #endif /* EAZEL_INSTALL_NO_CORBA */
 
 void eazel_install_set_log			  (EazelInstall *service,
@@ -126,6 +136,8 @@ void eazel_install_open_log                       (EazelInstall *service,
 						   const char *fname);
 void eazel_install_log_to_stderr		  (EazelInstall *service,
 						   gboolean log_to_stderr);
+gboolean eazel_install_failed_because_of_disk_full (EazelInstall *service);
+void eazel_install_add_repository		  (EazelInstall *service, const char *dir);
 
 /* This sets mode 500 on tmpdir and all files in
    private->downloaded_files */
@@ -138,14 +150,14 @@ void eazel_install_emit_install_progress          (EazelInstall *service,
 						   int package_num, int num_packages, 
 						   int package_size_completed, int package_size_total,
 						   int total_size_completed, int total_size);
-void eazel_install_emit_download_progress         (EazelInstall *service, 
-						   const char *name,
+void eazel_install_emit_download_progress         (EazelInstall *service,
+						   const PackageData *package,
 						   int amount, 
 						   int total);
 gboolean eazel_install_emit_preflight_check     (EazelInstall *service, 
 						 GList *packages);
 void eazel_install_emit_download_failed           (EazelInstall *service, 
-						   const char *name);
+						   const PackageData *package);
 void eazel_install_emit_md5_check_failed          (EazelInstall *service, 
 						   const PackageData *pd,
 						   const char *actual_md5);
@@ -154,6 +166,9 @@ void eazel_install_emit_install_failed            (EazelInstall *service,
 void eazel_install_emit_uninstall_failed          (EazelInstall *service, 
 						   const PackageData *pd);
 void eazel_install_emit_dependency_check          (EazelInstall *service, 
+						   const PackageData *package, 
+						   const PackageDependency *needed);
+void eazel_install_emit_dependency_check_pre_ei2  (EazelInstall *service, 
 						   const PackageData *package, 
 						   const PackageData *needed);
 void eazel_install_emit_done                      (EazelInstall *service, 
@@ -191,27 +206,30 @@ void eazel_install_delete_downloads (EazelInstall *service);
 /******************************************************************************/
 /* Beware, from hereonafter, it's #def madness, to make the get/set functions */
 
-#define SANITY_VAL(name, ret)\
+#define EAZEL_INSTALL_SANITY_VAL(name, ret)\
 	g_return_val_if_fail (name != NULL, ret); \
-	g_return_val_if_fail (IS_EAZEL_INSTALL (name), ret); \
+	g_return_val_if_fail (EAZEL_IS_INSTALL (name), ret); \
+	g_return_val_if_fail (name->private->softcat != NULL, ret); \
+	g_return_val_if_fail (EAZEL_IS_SOFTCAT (name->private->softcat), ret); \
 	g_assert (name->private != NULL); \
 	g_assert (name->private->iopts != NULL); \
 	g_assert (name->private->topts != NULL) 
 
-#define SANITY(name)\
+#define EAZEL_INSTALL_SANITY(name)\
 	g_return_if_fail (name != NULL); \
-	g_return_if_fail (IS_EAZEL_INSTALL (name)); \
+	g_return_if_fail (EAZEL_IS_INSTALL (name)); \
+	g_return_if_fail (name->private->softcat != NULL); \
+	g_return_if_fail (EAZEL_IS_SOFTCAT (name->private->softcat)); \
 	g_assert (name->private != NULL); \
 	g_assert (name->private->iopts != NULL); \
 	g_assert (name->private->topts != NULL) 
 
-
 #define ei_access_decl(name, type) \
 type eazel_install_get_##name (EazelInstall *service)
 
 #define ei_access_impl(name, type, var, defl) \
 type eazel_install_get_##name (EazelInstall *service) { \
-        SANITY_VAL (service, defl); \
+        EAZEL_INSTALL_SANITY_VAL (service, defl); \
 	return service->private->var; \
 }
 
@@ -222,14 +240,14 @@ void eazel_install_set_##name (EazelInstall *service, \
 #define ei_mutator_impl(name, type,var) \
 void eazel_install_set_##name (EazelInstall *service, \
                                          type name) { \
-        SANITY (service); \
+        EAZEL_INSTALL_SANITY (service); \
 	service->private->var = name; \
 }
 
 #define ei_mutator_impl_copy(name, type, var, copyfunc) \
 void eazel_install_set_##name (EazelInstall *service, \
                                          const type name) { \
-        SANITY (service); \
+        EAZEL_INSTALL_SANITY (service); \
         g_free (service->private->var); \
 	service->private->var = copyfunc ( name ); \
 }
@@ -265,8 +283,9 @@ ei_mutator_decl (transaction_dir, char*);
 ei_mutator_decl (server_port, guint);
 ei_mutator_decl (cgi_path, char*);
 ei_mutator_decl (eazel_auth, gboolean);
-ei_mutator_decl (package_system, int);
 ei_mutator_decl (ssl_rename, gboolean);
+ei_mutator_decl (ignore_file_conflicts, gboolean);
+ei_mutator_decl (ei2, gboolean);
 
 ei_access_decl (verbose, gboolean);
 ei_access_decl (silent, gboolean);
@@ -289,8 +308,9 @@ ei_access_decl (transaction_dir, char*);
 ei_access_decl (server_port, guint);
 ei_access_decl (cgi_path, char*);
 ei_access_decl (eazel_auth, gboolean);
-ei_access_decl (package_system, int);
 ei_access_decl (ssl_rename, gboolean);
+ei_access_decl (ignore_file_conflicts, gboolean);
+ei_access_decl (ei2, gboolean);
 
 #ifdef __cplusplus
 }
