@@ -24,16 +24,14 @@
  */
 
 #include "eazel-install-rpm-glue.h"
-#include "eazel-install-md5.h"
 #include "eazel-install-public.h"
 #include "eazel-install-private.h"
 #include "eazel-install-logic.h"
 
 #include <libtrilobite/trilobite-core-utils.h>
+#include <libtrilobite/trilobite-md5-tools.h>
 
 #include <rpm/rpmlib.h>
-#include <rpm/rpmmacro.h>
-#include <rpm/dbindex.h>
 #include <string.h>
 #include <time.h>
 
@@ -48,284 +46,37 @@
 #include <sys/wait.h>
 #endif
 
-#define PERCENTS_PER_RPM_HASH 2
-#undef DEBUG_RPM_OUTPUT
+#include "eazel-package-system.h"
 
-void 
-eazel_install_rpm_set_settings (EazelInstall *service) {
-	int install_flags, interface_flags, problem_filters;
+/* WAAAAAAHH! */
+#include "eazel-package-system-rpm3.h"
+#include "eazel-package-system-rpm3-private.h"
+
+PackageData* packagedata_new_from_rpm_conflict (struct rpmDependencyConflict);
+PackageData* packagedata_new_from_rpm_conflict_reversed (struct rpmDependencyConflict);
+
+PackageData*
+packagedata_new_from_rpm_conflict (struct rpmDependencyConflict conflict) 
+{
+	PackageData *result;
 	
-	install_flags = 0;
-	interface_flags = 0;
-	problem_filters = 0;
-	
-	if (eazel_install_get_test (service)) {
-		trilobite_debug (_("Dry Run Mode Activated.  Packages will not actually be installed ..."));
-		install_flags |= RPMTRANS_FLAG_TEST;
-	}
-
-	if (eazel_install_get_update (service)) {
-		interface_flags |= INSTALL_UPGRADE;
-	}
-
-	if (eazel_install_get_verbose (service)) {
-		rpmSetVerbosity (RPMMESS_VERBOSE);
-	} else {
-		rpmSetVerbosity (RPMMESS_NORMAL);
-	}
-
-	if (eazel_install_get_force (service)) {
-		trilobite_debug ("Force install is enabled");
-		problem_filters |= RPMPROB_FILTER_REPLACEPKG |
-			RPMPROB_FILTER_REPLACEOLDFILES |
-			RPMPROB_FILTER_REPLACENEWFILES |
-			RPMPROB_FILTER_OLDPACKAGE;
-	}
-
-
-	service->private->packsys.rpm.install_flags = install_flags;
-	service->private->packsys.rpm.interface_flags = interface_flags;
-	service->private->packsys.rpm.problem_filters = problem_filters;
-
-	/* this spams to stderr -- is that what we really want? */
-	if (eazel_install_get_debug (service) && 0) {
-		rpmSetVerbosity (RPMMESS_DEBUG);
-	}
-
-	trilobite_debug ("Read rpmrc file %s", eazel_install_get_rpmrc_file (service));
-	rpmReadConfigFiles (eazel_install_get_rpmrc_file (service), NULL);
+	result = packagedata_new ();
 
+	result->name = g_strdup (conflict.needsName);
+	result->version = (conflict.needsVersion && (strlen (conflict.needsVersion) > 1)) ? g_strdup (conflict.needsVersion) : NULL;
+	return result;
 }
 
-void
-eazel_install_start_transaction_make_rpm_argument_list (EazelInstall *service,
-							GList **args)
+PackageData*
+packagedata_new_from_rpm_conflict_reversed (struct rpmDependencyConflict conflict) 
 {
-	if (eazel_install_get_test (service)) {
-		(*args) = g_list_prepend (*args, g_strdup ("--test"));
-	} 
-	if (eazel_install_get_force (service)) {
-		g_warning ("Force install mode!");
-		if (!eazel_install_get_uninstall (service)) {
-			(*args) = g_list_prepend (*args, g_strdup ("--force"));
-		}
-		(*args) = g_list_prepend (*args, g_strdup ("--nodeps"));
-	}
-	if (eazel_install_get_uninstall (service)) {
-		(*args) = g_list_prepend (*args, g_strdup ("-e"));
-	} else if (eazel_install_get_update (service) || eazel_install_get_downgrade (service)) {
-		if (eazel_install_get_downgrade (service)) {
-			(*args) = g_list_prepend (*args, g_strdup ("--oldpackage"));
-		}
-		(*args) = g_list_prepend (*args, g_strdup ("-Uvh"));
-	} else {
-		(*args) = g_list_prepend (*args, g_strdup ("-ivh"));
-	}
-}
-
-
-gboolean
-eazel_install_monitor_rpm_process_pipe (GIOChannel *source,
-					GIOCondition condition,
-					EazelInstall *service)
-{
-	char         tmp;
-	static       int package_name_length = 256;
-	static       char package_name [256];
-	ssize_t      bytes_read;
-	static       PackageData *pack = NULL;
-	static       int pct;
-#ifdef DEBUG_RPM_OUTPUT
-	static char *rpmoutname = NULL;
-	FILE *rpmoutput;
-#endif
+	PackageData *result;
 	
-	g_io_channel_read (source, &tmp, 1, &bytes_read);
-	
-/* 1.39 has the code to parse --percent output */
-	if (bytes_read) {
-#ifdef DEBUG_RPM_OUTPUT
-		if (rpmoutname == NULL) {
-			rpmoutname = tempnam ("/tmp", "rpmSt");
-		}
-		rpmoutput = fopen (rpmoutname, "at");
-		fprintf (rpmoutput, "%c", tmp); 
-		fflush (rpmoutput); 
-		fclose (rpmoutput);
-#endif
-		/* Percentage output, parse and emit... */
-		if (tmp=='#') {
-			int amount;
-			if (pack == NULL) {
-				return TRUE;
-			}
-			pct += PERCENTS_PER_RPM_HASH;
-			if (pct == 100) {
-				amount = pack->bytesize;
-			} else {
-				amount =  (pack->bytesize / 100) * pct;
-			}
-			if (pack && amount) {
-				eazel_install_emit_install_progress (service, 
-								     pack, 
-								     service->private->packsys.rpm.packages_installed, 
-								     service->private->packsys.rpm.num_packages,
-								     amount, 
-								     pack->bytesize,
-								     service->private->packsys.rpm.current_installed_size + amount,
-								     service->private->packsys.rpm.total_size);
-			}
-					/* By invalidating the pointer here, we
-					   only emit with amount==total once */
-			if (pct==100) {
-				service->private->packsys.rpm.current_installed_size += pack->bytesize;
-					/* If a toplevel pacakge completed, add to transction list */
-				if (pack->toplevel) {
-					eazel_install_do_transaction_add_to_transaction (service, pack);
-				}
-				pack = NULL;
-				pct = 0;
-				package_name [0] = 0;
-			}
-		}  else  if (tmp != ' ') {
-			/* Read untill we hit a space */
-			package_name[0] = 0;
-			while (bytes_read && tmp != ' ') {
-				if (strlen (package_name) < package_name_length) {
-					/* Add char to package */
-					int x;
-					x = strlen (package_name);
-					if (!isspace (tmp)) {
-						package_name [x] = tmp;
-						package_name [x+1] = 0;
-					}
-				}
-					/* read next byte */
-				g_io_channel_read (source, &tmp, 1, &bytes_read);
-				if (tmp=='\n') {
-					package_name[0] = '\0';
-					break;
-				}
-			}
-			
-			/* Not percantage mark, that means filename, step ahead one file */
-			pack = g_hash_table_lookup (service->private->name_to_package_hash, package_name);
-			if (pack==NULL) {
-				/* might be "warning:" */
-				if (strcmp (package_name, "warning:") == 0) {
-					package_name[0] = 0;
-					while (tmp != '\n') {
-						g_io_channel_read (source, &tmp, 1, &bytes_read);
-					}
-					trilobite_debug ("warning received");
-				} else {
-					trilobite_debug ("lookup \"%s\" failed", package_name);
-				}
-			} else {
-				trilobite_debug ("matched \"%s\"", package_name);
-				pct = 0;
-				service->private->packsys.rpm.packages_installed ++;
-				eazel_install_emit_install_progress (service, 
-								     pack, 
-								     service->private->packsys.rpm.packages_installed, 
-								     service->private->packsys.rpm.num_packages,
-								     0, 
-								     pack->bytesize,
-								     service->private->packsys.rpm.current_installed_size,
-								     service->private->packsys.rpm.total_size);
-			}
-		}
-	} 
+	result = packagedata_new ();
 
-	if (bytes_read == 0) {
-		return FALSE;
-	} else {
-		return TRUE;
-	}
-}
-
-static gboolean
-eazel_install_free_rpm_system_close_db_foreach (char *key, rpmdb db, gpointer unused)
-{
-	if (db) {
-		trilobite_debug (_("Closing db for %s (open)"), key);
-		rpmdbClose (db);
-		db = NULL;
-		g_free (key);
-	} else {
-		trilobite_debug (_("Closing db for %s (not open)"), key);
-	}
-
-	return TRUE;
-}
-
-gboolean
-eazel_install_free_rpm_system (EazelInstall *service)
-{
-	/* Close all the db's */
-	g_assert (service->private->packsys.rpm.dbs);
-	trilobite_debug ("service->private->packsys.rpm.dbs.size = %d", g_hash_table_size (service->private->packsys.rpm.dbs));
-	g_hash_table_foreach_remove (service->private->packsys.rpm.dbs, 
-				     (GHRFunc)eazel_install_free_rpm_system_close_db_foreach,
-				     NULL);
-	trilobite_debug ("service->private->packsys.rpm.dbs.size = %d", g_hash_table_size (service->private->packsys.rpm.dbs));
-	
-/*
-  This crashes, so it's commented out. 
-  These are the vars used for this.
-  
-	rpmTransactionSet *set;
-	set = &(service->private->packsys.rpm.set);
-
-	if (*set != NULL) {
-		rpmtransFree (*set);
-		(*set) = NULL;
-	}
-*/
-	return TRUE;
-}
-
-gboolean
-eazel_install_prepare_rpm_system(EazelInstall *service)
-{
-	GList *iterator;
-
-	g_assert (service->private->packsys.rpm.dbs);
-
-	rpmReadConfigFiles (eazel_install_get_rpmrc_file (service), NULL);
-
-	g_assert (g_list_length (eazel_install_get_root_dirs (service)));
-
-	addMacro(NULL, "_dbpath", NULL, "/", 0);
-
-	if (g_hash_table_size (service->private->packsys.rpm.dbs) > 0) {
-		trilobite_debug ("db already open ?");
-		return TRUE;
-	}
-	
-	for (iterator = eazel_install_get_root_dirs (service); iterator; iterator = g_list_next (iterator)) {
-		const char *root_dir;	
-		rpmdb db;
-		
-		root_dir = (char*)iterator->data;
-		
-		if (rpmdbOpen (root_dir, &db, O_RDONLY, 0644)) {
-			trilobite_debug ("Opening packages database in %s failed", root_dir);
-			trilobite_debug ("(first time)");
-		} else {			
-			if (db) {
-				trilobite_debug (_("Opened packages database in %s"), root_dir);
-				g_hash_table_insert (service->private->packsys.rpm.dbs,
-						     g_strdup (root_dir),
-						     db);
-			} else {
-				trilobite_debug (_("Opening packages database in %s failed"), root_dir);
-			}
-		}
-	}
-
-
-	return TRUE;
+	result->name = g_strdup (conflict.byName);
+	result->version = (conflict.byVersion && (strlen (conflict.byVersion) > 1)) ? g_strdup (conflict.byVersion) : NULL;
+	return result;
 }
 
 /*
@@ -357,11 +108,10 @@ eazel_install_add_to_rpm_set (EazelInstall *service,
 		int err;
 
 		pack = (PackageData*)iterator->data;
-
 		if (!eazel_install_get_uninstall (service)) {
 			g_assert (pack->packsys_struc);
 			err = rpmtransAddPackage (set,
-						  *((Header*)pack->packsys_struc),
+						  ((Header) pack->packsys_struc),
 						  NULL, 
 						  NULL,
 						  interface_flags, 
@@ -377,10 +127,11 @@ eazel_install_add_to_rpm_set (EazelInstall *service,
 				   we're iterating it, so add to a tmp list and nuke later */
 				tmp_failed = g_list_prepend (tmp_failed, pack);
 			}
-			/* just flailing around here (robey) */
+			/* just flailing around here (robey) 
 			if (pack->soft_depends) {
 				eazel_install_add_to_rpm_set (service, set, &pack->soft_depends, failed);
 			}
+			*/
 		} else {
 			g_assert_not_reached ();
 		}
@@ -471,21 +222,22 @@ eazel_install_do_rpm_dependency_check (EazelInstall *service,
 
 	trilobite_debug ("eazel_install_do_rpm_dependency_check");
 
-	g_assert (service->private->packsys.rpm.dbs);
+	g_assert (EAZEL_PACKAGE_SYSTEM_RPM3 (service->private->package_system)->private->dbs);
 
-	db = (rpmdb)g_hash_table_lookup (service->private->packsys.rpm.dbs,
+	eazel_package_system_rpm3_open_dbs (EAZEL_PACKAGE_SYSTEM_RPM3 (service->private->package_system));
+	db = (rpmdb)g_hash_table_lookup (EAZEL_PACKAGE_SYSTEM_RPM3 (service->private->package_system)->private->dbs,
 					 service->private->cur_root);
 	if (!db) {
 		return;
 	}
 
 	set =  rpmtransCreateSet (db, service->private->cur_root);
-	
 	eazel_install_add_to_rpm_set (service, set, packages, failedpackages); 
 
 	/* Reorder the packages as per. deps and do the dep check */
 	rpmdepOrder (set);		
 	rpmdepCheck (set, &conflicts, &num_conflicts);
+	eazel_package_system_rpm3_close_dbs (EAZEL_PACKAGE_SYSTEM_RPM3 (service->private->package_system));
 
 	/* FIXME bugzilla.eazel.com 1512:
 	   This piece of code is rpm specific. It has some generic algorithm
@@ -500,7 +252,7 @@ eazel_install_do_rpm_dependency_check (EazelInstall *service,
 
 		/* Locate the package that caused the conflict */
 		pack_entry = g_list_find_custom (*packages, 
-						 conflict.byName,
+						 (char*)conflict.byName,
 						 (GCompareFunc)eazel_install_package_name_compare);
 
 		/* first time through, only do immediate matches */
@@ -536,12 +288,12 @@ eazel_install_do_rpm_dependency_check (EazelInstall *service,
 
 		/* Locate the package that caused the conflict */
 		pack_entry = g_list_find_custom (*packages, 
-						 conflict.byName,
+						 (char*)conflict.byName,
 						 (GCompareFunc)eazel_install_package_name_compare);
 		if (pack_entry == NULL) {
 			/* try our brand-new list of required packages, too */
 			pack_entry = g_list_find_custom (*requirements,
-							 conflict.byName,
+							 (char*)conflict.byName,
 							 (GCompareFunc)eazel_install_requirement_dep_name_compare);
 			if (pack_entry != NULL) {
 				trilobite_debug (_("package %s is already in requirements, whew!"), conflict.byName);
@@ -607,10 +359,10 @@ eazel_install_do_rpm_dependency_check (EazelInstall *service,
 							trilobite_debug ("Cannot lookup %s in *failedpackages",
 									 conflict.needsName);
 							
-							trilobite_debug ("packages = 0x%x", packages);
-							trilobite_debug ("*packages = 0x%x", *packages);
-							trilobite_debug ("failedpackages = 0x%x", failedpackages);
-							trilobite_debug ("*failedpackages = 0x%x", *failedpackages);
+							trilobite_debug ("packages = 0x%p", packages);
+							trilobite_debug ("*packages = 0x%p", *packages);
+							trilobite_debug ("failedpackages = 0x%p", failedpackages);
+							trilobite_debug ("*failedpackages = 0x%p", *failedpackages);
 							
 							trilobite_debug ("g_list_length (*packages) = %d", g_list_length (*packages));
 							trilobite_debug ("g_list_length (*failedpackages) = %d", g_list_length (*failedpackages));
