@@ -29,10 +29,13 @@
  */
 
 #include <config.h>
+#include "eazel-softcat.h"	/* for softcat sense flags */
 #include "eazel-install-xml-package-list.h"
 
 #include <libtrilobite/trilobite-core-utils.h>
+#include <libtrilobite/trilobite-core-network.h>
 #include <gnome-xml/parser.h>
+#include <gnome-xml/xmlmemory.h>
 
 static PackageData* parse_package (xmlNode* package, gboolean set_toplevel);
 static CategoryData* parse_category (xmlNode* cat);
@@ -40,48 +43,62 @@ static int parse_pkg_template (const char* pkg_template_file, char** result);
 static PackageData* osd_parse_softpkg (xmlNodePtr softpkg);
 
 
+
 static PackageData*
 parse_package (xmlNode* package, gboolean set_toplevel) {
 
 	xmlNodePtr dep;
 	PackageData* rv;
-	const char *temp;
+	char *tmp;
 
 	rv = packagedata_new ();
 
-	rv->name = g_strdup (xml_get_value (package, "NAME"));
-	rv->version = g_strdup (xml_get_value (package, "VERSION"));
-	rv->minor = g_strdup (xml_get_value (package, "MINOR"));
-	rv->archtype = g_strdup (xml_get_value (package, "ARCH"));
-	if (xml_get_value (package, "STATUS")) {
-		rv->status = packagedata_status_str_to_enum (xml_get_value (package, "STATUS"));
-	}
-	if (xml_get_value (package, "MODSTATUS")) {
-		rv->modify_status = packagedata_modstatus_str_to_enum (xml_get_value (package, "MODSTATUS"));
+	rv->name = trilobite_xml_get_string (package, "NAME");
+	rv->version = trilobite_xml_get_string (package, "VERSION");
+	rv->minor = trilobite_xml_get_string (package, "MINOR");
+	rv->archtype = trilobite_xml_get_string (package, "ARCH");
+	
+	tmp = trilobite_xml_get_string (package, "STATUS");
+	if (tmp) {
+		rv->status = packagedata_status_str_to_enum (tmp);
+		g_free (tmp);
 	}
 
-	temp = xml_get_value (package, "BYTESIZE");
-	if (temp) {
-		rv->bytesize = atoi (temp);
+	tmp = trilobite_xml_get_string (package, "MODSTATUS");
+	if (tmp) {
+		rv->modify_status = packagedata_modstatus_str_to_enum (tmp);
+		g_free (tmp);
+	}
+
+	tmp = trilobite_xml_get_string (package, "BYTESIZE");
+	if (tmp) {
+		rv->bytesize = atoi (tmp);
+		g_free (tmp);
 	} else {
 		rv->bytesize = 0;
 	}
-	temp = xml_get_value (package, "PROVIDES");
-	if (temp) {
-		const char *tmp1, *sep, *end;		
-		tmp1 = temp;
-		end = temp + strlen (temp);
-		while ((sep = strchr (tmp1, G_SEARCHPATH_SEPARATOR))!= NULL) {
-			rv->provides = g_list_prepend (rv->provides, g_strndup (tmp1, (int)(sep - tmp1)));
-			tmp1 = sep;
-			tmp1 ++;
+
+	tmp = trilobite_xml_get_string (package, "PROVIDES");
+	if (tmp) {
+		char **files;
+		int iterator;
+
+		files = g_strsplit (tmp, G_SEARCHPATH_SEPARATOR_S, 0);
+		if (files) {
+			/* we don't free the individual files[x]'s, as
+			   they end up in the package data object */
+			for (iterator = 0; files[iterator]; iterator++) {
+				rv->provides = g_list_prepend (rv->provides,
+							       files[iterator]);
+			}
+			rv->provides = g_list_reverse (rv->provides);
+			g_free (files);
 		}
-		rv->provides = g_list_prepend (rv->provides, g_strdup (tmp1));
-		rv->provides = g_list_reverse (rv->provides);
+		g_free (tmp);
 	}
 
-	rv->summary = g_strdup (xml_get_value (package, "SUMMARY"));
-	rv->description = g_strdup (xml_get_value (package, "DESCRIPTION"));
+	rv->summary = trilobite_xml_get_string (package, "SUMMARY");
+	rv->description = trilobite_xml_get_string (package, "DESCRIPTION");
 	rv->distribution = trilobite_get_distribution ();
 	if (set_toplevel) {
 		rv->toplevel = TRUE;
@@ -135,7 +152,7 @@ parse_category (xmlNode* cat) {
 	char *text, *textp;
 
 	category = categorydata_new ();
-	category->name = g_strdup (xml_get_value (cat, "name"));
+	category->name = trilobite_xml_get_string (cat, "name");
 
 	pkg = cat->xmlChildrenNode;
 	if (pkg == NULL) {
@@ -379,10 +396,7 @@ generate_xml_package_list (const char* pkg_template_file,
  * category name : package name : version : minor : archtype : bytesize : summary
  * 
  * Example:
- * 
- *
  * Essential Packages:anaconda:7.0:1:i386:2722261:The redhat installer
- * 
  */
 
 	xmlDocPtr doc;
@@ -520,11 +534,17 @@ parse_pkg_template (const char* pkg_template_file, char **result) {
    (primarily used for the recursiveness of PackageData objects.
    If not, creates a node with the name "PACKAGE" */
 xmlNodePtr
-eazel_install_packagedata_to_xml (const PackageData *pack, char *title, xmlNodePtr droot)
+eazel_install_packagedata_to_xml (const PackageData *pack, 
+				  char *title, 
+				  xmlNodePtr droot, 
+				  gboolean include_provides)
 {
 	xmlNodePtr root, node;
 	char *tmp;
 	GList *iterator;
+	/* This is a horrible crackpatch for PR3 used to check for recursive
+	   depends... */
+	static GList *path = NULL;
 
 	if (droot) {
 		g_assert (title != NULL);
@@ -559,7 +579,7 @@ eazel_install_packagedata_to_xml (const PackageData *pack, char *title, xmlNodeP
 	node = xmlNewChild (root, NULL, "BYTESIZE", tmp);
 	g_free (tmp);
 
-	if (pack->provides) {
+	if (include_provides && pack->provides) {
 		tmp = g_strdup ((char*)(pack->provides->data));
 		for (iterator = g_list_next (pack->provides); iterator; iterator = g_list_next (iterator)) {
 			char *fname = (char*)(iterator->data);
@@ -570,26 +590,48 @@ eazel_install_packagedata_to_xml (const PackageData *pack, char *title, xmlNodeP
 		}
 		node = xmlNewChild (root, NULL, "PROVIDES", tmp);
 	}
-
+	
+	for (iterator = pack->depends; iterator; iterator = iterator->next) {
+		if (g_list_find (path, ((PackageDependency*)iterator->data)->package) == NULL) {
+			path = g_list_prepend (path, ((PackageDependency*)iterator->data)->package);
+			eazel_install_packagedata_to_xml (((PackageDependency*)iterator->data)->package, 
+							  "SOFT_DEPEND", 
+							  root, 
+							  include_provides);
+			path = g_list_remove (path, ((PackageDependency*)iterator->data)->package);
+		}
+	}
 	for (iterator = pack->soft_depends; iterator; iterator = iterator->next) {
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, "SOFT_DEPEND", root);
+		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
+						  "SOFT_DEPEND", 
+						  root, 
+						  include_provides);
 	}
 	for (iterator = pack->hard_depends; iterator; iterator = iterator->next) {
 		node = xmlNewChild (root, NULL, "HARD_DEPEND", NULL);
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, "HARD_DEPEND", root);
+		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
+						  "HARD_DEPEND", 
+						  root,
+						  include_provides);
 	}
 	for (iterator = pack->breaks; iterator; iterator = iterator->next) {
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, "BREAKS", root);
+		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
+						  "BREAKS", 
+						  root,
+						  include_provides);
 	}
 	for (iterator = pack->modifies; iterator; iterator = iterator->next) {
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, "MODIFIES", root);
+		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
+						  "MODIFIES", 
+						  root,
+						  include_provides);
 	}
 
 	return root;
 }
 
 xmlNodePtr
-eazel_install_packagelist_to_xml (GList *packages) {
+eazel_install_packagelist_to_xml (GList *packages, gboolean include_provides) {
 	xmlNodePtr node;
 	GList *iterator;
 
@@ -598,7 +640,8 @@ eazel_install_packagelist_to_xml (GList *packages) {
 	for (iterator = packages; iterator; iterator = iterator->next) {
 		xmlAddChild (node, 
 			     eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
-							       NULL, NULL));
+							       NULL, NULL, include_provides)
+			);
 	}
 
 	return node;	
@@ -611,34 +654,131 @@ eazel_install_categorydata_to_xml (const CategoryData *category)
 
 	node = xmlNewNode (NULL, "CATEGORY");
 	xmlSetProp (node, "name", category->name);
-	xmlAddChild (node, eazel_install_packagelist_to_xml (category->packages));
+	xmlAddChild (node, eazel_install_packagelist_to_xml (category->packages, TRUE));
 
 	return node;
 }
 
+static PackageDependency *
+osd_parse_provides (PackageData *pack, xmlNodePtr node, GList **feature_list)
+{
+	xmlNodePtr child;
+	GList *list = NULL;
+	PackageDependency *dep = NULL;
+	char *tmp;
+	gboolean got_package = FALSE;
+
+	child = node->xmlChildrenNode;
+	while (child) {
+		if (g_strcasecmp (child->name, "FILE") == 0) {
+			tmp = xmlNodeGetContent (child);
+			list = g_list_prepend (list, g_strdup (tmp));
+			xmlFree (tmp);
+		} else if (g_strcasecmp (child->name, "PACKAGE") == 0) {
+			if (got_package) {
+				g_warning ("multiple packages in dep list for %s!", pack->name);
+			} else {
+				dep = packagedependency_new ();
+				dep->version = trilobite_xml_get_string (child, "version");
+				tmp = trilobite_xml_get_string (child, "sense");
+				dep->sense = eazel_softcat_convert_sense_flags (atoi (tmp));
+				g_free (tmp);
+				got_package = TRUE;
+			}
+		} else if (g_strcasecmp (child->name, "FEATURE") == 0) {
+			tmp = xmlNodeGetContent (child);
+			list = g_list_prepend (list, g_strdup (tmp));
+			xmlFree (tmp);
+		} else {
+			trilobite_debug ("Unknown tag %s in xml", child->name);
+		}
+		child = child->next;
+	}
+
+	list = g_list_reverse (list);
+	*feature_list = list;
+	return dep;
+}
+
 static void
 osd_parse_dependency (PackageData *pack, xmlNodePtr node)
 {
 	xmlNodePtr child;
-	PackageData *softpack;
+	PackageData *softpack = NULL;
+	PackageDependency *dep = NULL;
+	GList *features = NULL;
 
 	child = node->xmlChildrenNode;
 	while (child) {
 		if (g_strcasecmp (child->name, "PROVIDES") == 0) {
-			/* do nothing yet.  maybe someday? */
+			dep = osd_parse_provides (pack, child, &features);
 		} else if (g_strcasecmp (child->name, "SOFTPKG") == 0) {
 			/* dependent softpkg */
 			softpack = osd_parse_softpkg (child);
-			if (softpack != NULL) {
-				packagedata_add_pack_to_soft_depends (pack, softpack);
-			} else {
-				trilobite_debug ("SOFTPKG dependency parse failed");
-			}
 		} else {
 			/* unparsed part of dependency */
 		}
 		child = child->next;
 	}
+
+	if (softpack != NULL) {
+		if ((features != NULL) && (softpack->features == NULL)) {
+			/* softcat sends separate PROVIDES and SOFTPKG: connect them */
+			softpack->features = features;
+		}
+		if (dep == NULL) {
+			dep = packagedependency_new ();
+		}
+		/* attach this package to a depends struct */
+		pack->depends = g_list_prepend (pack->depends, dep);
+		dep->package = softpack;
+	} else if (features != NULL) {
+		g_warning ("stranded PROVIDES block in the dependency info!");
+		g_list_foreach (features, (GFunc)g_free, NULL);
+		g_list_free (features);
+		features = NULL;
+	}
+}
+
+static void
+osd_parse_file_list (PackageData *pack, xmlNodePtr node)
+{
+	xmlNodePtr child;
+	char *tmp;
+
+	child = node->xmlChildrenNode;
+	while (child) {
+		if (g_strcasecmp (child->name, "FILE") == 0) {
+			tmp = xmlNodeGetContent (child);
+			pack->provides = g_list_prepend (pack->provides, g_strdup (tmp));
+			xmlFree (tmp);
+		} else {
+			/* bad : thing in file list that isn't a file */
+			trilobite_debug ("XML file list contains %s (not FILE)", child->name);
+		}
+		child = child->next;
+	}
+	pack->provides = g_list_reverse (pack->provides);
+}
+
+static void
+osd_parse_feature_list (PackageData *pack, xmlNodePtr node)
+{
+	xmlNodePtr child;
+	char *tmp;
+
+	child = node->xmlChildrenNode;
+	while (child) {
+		if (g_strcasecmp (child->name, "FEATURE") == 0) {
+			tmp = xmlNodeGetContent (child);
+			pack->features = g_list_prepend (pack->features, g_strdup (tmp));
+			xmlFree (tmp);
+		} else {
+			trilobite_debug ("XML feature list contains %s (not FILE)", child->name);
+		}
+		child = child->next;
+	}
+	pack->features = g_list_reverse (pack->features);
 }
 
 static void
@@ -650,33 +790,37 @@ osd_parse_implementation (PackageData *pack,
 	child = node->xmlChildrenNode;
 	while (child) {
 		if (g_strcasecmp (child->name, "PROCESSOR")==0) {
-			pack->archtype = g_strdup (xml_get_value (child, "VALUE"));
+			pack->archtype = trilobite_xml_get_string (child, "VALUE");
 		} else if (g_strcasecmp (child->name, "OS")==0) {
-			char *dtmp = xmlGetProp (child, "VALUE");
+			char *dtmp = trilobite_xml_get_string (child, "VALUE");
 			if (dtmp) {
 				pack->distribution.name = trilobite_get_distribution_enum (dtmp,
 											   TRUE);
+				g_free (dtmp);
 			} 
 		} else if (g_strcasecmp (child->name, "CODEBASE")==0) {			
-			const char *temp;
-			pack->remote_url = g_strdup (xml_get_value (child, "HREF"));
-			temp = xml_get_value (child, "SIZE");
-			if (temp) {
-				pack->bytesize = atoi (temp);
+			char *tmp;
+			pack->remote_url = trilobite_xml_get_string (child, "HREF");
+			tmp = trilobite_xml_get_string (child, "SIZE");
+			if (tmp) {
+				pack->bytesize = atoi (tmp);
+				g_free (tmp);
 			} else {
 				pack->bytesize = 0;
 			}
+		} else if (g_strcasecmp (child->name, "FILES") == 0) {
+			/* oh boy... exhaustive file list */
+			osd_parse_file_list (pack, child);
+		} else if (g_strcasecmp (child->name, "FEATURES") == 0) {
+			/* list of "features" (usually shared libs) in the toplevel package */
+			osd_parse_feature_list (pack, child);
 		} else if (g_strcasecmp (child->name, "DEPENDENCY")==0) {
-			/* presume this is a soft-depends */
 			osd_parse_dependency (pack, child);
 		} else {
 			/* trilobite_debug ("unparsed tag \"%s\" in IMPLEMENTATION", child->name); */
 		}
 		child = child->next;
 	}
-	/* FIXME: bugzilla.eazel.com 2241
-	   we need the md5 in the xml, and then add it to the package here */
-	
 }
 
 static PackageData*
@@ -684,19 +828,29 @@ osd_parse_softpkg (xmlNodePtr softpkg)
 {
 	PackageData *result;
 	xmlNodePtr child;
+	char *tmp;
 
 	result = packagedata_new ();
 
-	result->name = g_strdup (xml_get_value (softpkg, "NAME"));
-	result->version = g_strdup (xml_get_value (softpkg, "VERSION"));
-	result->md5 = g_strdup (xml_get_value (softpkg, "MD5"));
+	result->name = trilobite_xml_get_string (softpkg, "NAME");
+	result->version = trilobite_xml_get_string (softpkg, "VERSION");
+	result->minor = trilobite_xml_get_string (softpkg, "REVISION");
+	result->md5 = trilobite_xml_get_string (softpkg, "MD5");
 	
 	child = softpkg->xmlChildrenNode;
 	while (child) {
 		if (g_strcasecmp (child->name, "ABSTRACT")==0) {
-			result->description = g_strdup (xmlNodeGetContent (child));
+			tmp = xmlNodeGetContent (child);
+			result->summary = g_strdup (tmp);
+			while ((strlen (result->summary) > 0) &&
+			       (result->summary[strlen (result->summary)-1] <= ' ')) {
+				result->summary[strlen (result->summary)-1] = '\0';
+			}
+			xmlFree (tmp);
 		} else if (g_strcasecmp (child->name, "IMPLEMENTATION")==0) {
 			osd_parse_implementation (result, child);
+		} else if (g_strcasecmp (child->name, "EAZEL_ID") == 0) {
+			result->eazel_id = trilobite_xml_get_string (child, "VALUE");
 		} else {
 			/* trilobite_debug ("unparsed tag \"%s\" in SOFTPKG", child->name); */
 		}
@@ -744,17 +898,17 @@ osd_parse_shared (xmlDocPtr doc)
 	return result;
 }
 
-GList*
-parse_osd_xml_from_memory (const char *mem, 
-			   int size)
+/* returns FALSE if the XML was all horked */
+gboolean
+eazel_install_packagelist_parse (GList **list, const char *mem, int size)
 {
 	xmlDocPtr doc;
-	GList *result;
 	char *ptr, *docptr, *end;
+	char *nextnl, *cur;
 
-	result = NULL;
-	if (mem==NULL) {
-		return result;
+	*list = NULL;
+	if (mem == NULL) {
+		return FALSE;
 	}
 
 	docptr = g_malloc (size+1);
@@ -762,27 +916,27 @@ parse_osd_xml_from_memory (const char *mem,
 	docptr [size] = 0;
 
 	/* libxml is very intolerant of whitespace */
-	ptr = docptr;
-	while ((size > 0) && (*ptr <= ' ')) {
-		ptr++, size--;
-	}
-	end = ptr + size - 1;
-	while ((size > 0) && (*end <= ' ')) {
-		*end-- = '\0', size--;
-	}
+	for (ptr = docptr; (size > 0) && (*ptr <= ' '); ptr++, size--)
+		;
+	for (end = ptr + size - 1; (size > 0) && (*end <= ' '); *end-- = '\0', size--)
+		;
 
 	doc = xmlParseMemory (ptr, size);
+
 	if (doc == NULL) {
-		trilobite_debug ("XML =\"%s\"", ptr);
 		g_warning (_("Could not parse the xml (length %d)"), size);
+		for (cur = ptr; (nextnl = strchr (cur, '\n')) != NULL; ) {
+			*nextnl = '\0';
+			trilobite_debug ("XML: %s", cur);
+			cur = nextnl + 1;
+		}
 		g_free (docptr);
-		return result;
+		return FALSE;
 	}
 
-	result = osd_parse_shared (doc);	
+	*list = osd_parse_shared (doc);	
 
 	xmlFreeDoc (doc);
 	g_free (docptr);
-	return result;
+	return TRUE;
 }
-
