@@ -25,7 +25,7 @@
 
 #include <libnautilus/nautilus-view.h>
 #include <gtk/gtk.h>
-#include <eazel-install-types.h>
+#include <eazel-package-system-types.h>
 #include <eazel-install-corba-types.h>
 #include <eazel-install-corba-callback.h>
 #include <eazel-install-problem.h>
@@ -51,6 +51,16 @@ struct _NautilusServiceInstallViewClass {
         GtkVBoxClass				parent_class;
 };
 
+typedef struct {
+	char *package_name;
+	GtkWidget *label;
+	GtkWidget *progress_bar;
+	GtkWidget *progress_label;
+	GtkWidget *vbox;	/* [ progress_bar, progress_label ] */
+	GtkWidget *hbox;	/* [ label, padding, vbox ] */
+	GtkWidget *line;
+} InstallMessage;
+
 /* A NautilusContentView's private information. */
 struct _NautilusServiceInstallViewDetails {
 	char            *uri;
@@ -64,10 +74,10 @@ struct _NautilusServiceInstallViewDetails {
 	GtkWidget       *total_progress_bar;
 	GtkWidget       *overall_feedback_text;
 	GtkWidget	*message_box;
-	GList		*message_left;
-	GList		*message_right;
 	GtkWidget       *current_feedback_text;
-	GtkWidget       *current_progress_bar;
+	InstallMessage	*current_im;
+	GtkWidget	*pane;
+	GtkWidget	*middle_title;
 
 	char		*current_rpm;
 	int		current_package;
@@ -77,18 +87,25 @@ struct _NautilusServiceInstallViewDetails {
 	int		using_local_file;
 	gboolean	failure;
 	gboolean	cancelled;
+	gboolean	already_installed;
+	int		last_k;			/* used to avoid flickering the KB count so much */
+	gboolean	cancelled_before_downloads;
+	unsigned long	download_bytes_total;
+	unsigned long	download_bytes_sofar;
+	char		*username;
 
 	EazelInstallCallback *installer;
 	EazelInstallProblem *problem;
 	TrilobiteRootClient *root_client;
-	
-	/* Crude fix for 
-	   FIXME bugzilla.eazel.com 3431
-	*/
+	GList		    *categories;	
+
 	gboolean core_package;
 
 	GList *problem_cases;
 	GList *desktop_files;
+
+	GList *message;		/* GList<InstallMessage *> */
+	GHashTable *deps;	/* package(char *) => package that needs it(char *) */
 };
 
 
