@@ -21,6 +21,8 @@
  *
  */
 
+#undef STOP_IN_DEBUG
+
 #include <config.h>
 #include <gnome.h>
 #include <liboaf/liboaf.h>
@@ -28,7 +30,15 @@
 #include <signal.h>
 
 #include <libtrilobite/libtrilobite-service.h>
-#include <libtrilobite/trilobite-core-utils.h>
+#include <libtrilobite/libtrilobite.h>
+
+#ifdef STOP_IN_DEBUG
+/* Also add 
+	$(top_builddir)/libnautilus-extensions/nautilus-debug.o				 	\
+to the ldadd line in Makefile.am
+*/
+#include <libnautilus-extensions/nautilus-debug.h>
+#endif
 
 #include <trilobite-eazel-install.h>
 #include <eazel-install-public.h>
@@ -69,7 +79,7 @@ trilobite_service_factory_destroy (GtkObject *object)
 	trilobite_debug ("Destroying factory object");
 
 	bonobo_object_unref (BONOBO_OBJECT (factory)); 
-	gtk_main_quit ();
+	trilobite_main_quit ();
 }
 
 static BonoboObject*
@@ -118,8 +128,6 @@ eazel_install_service_factory (BonoboGenericFactory *this_factory,
 
 int main(int argc, char *argv[]) {
 
-	GData *data;
-
 #ifdef ENABLE_NLS /* sadly we need this ifdef because otherwise the following get empty statement warnings */
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
@@ -127,14 +135,12 @@ int main(int argc, char *argv[]) {
 
 	signal (SIGSEGV, &sig_segv_handler);
 
-	g_datalist_init (&data);
-	g_datalist_set_data (&data, "debug", (void *)1);
 	if (!trilobite_init ("trilobite-eazel-install-service", "0.1", "~/.nautilus/trilobite-install.log",
-			     argc, argv, data)) {
+			     NULL, argc, argv)) {
 		g_error ("Could not initialize trilobite. :(");
 		exit (1);
 	}
-	g_datalist_clear (&data);
+	trilobite_set_debug_mode (TRUE);
 
 	gnome_vfs_init ();
 
@@ -146,10 +152,13 @@ int main(int argc, char *argv[]) {
 		g_error ("Could not register factory");
 	}
 
-	bonobo_activate();
+#ifdef STOP_IN_DEBUG
+	nautilus_make_warnings_and_criticals_stop_in_debugger ("GLib", NULL);
+#endif
 
 	do {
-		bonobo_main ();
+		bonobo_activate ();
+		trilobite_main ();
 	} while (trilobites_active > 0);
 
 	g_message ("EazelInstall service terminating");
