@@ -24,7 +24,7 @@
   This is based in the distribution stuff from helixcode-utils.c, from
   the HelixCode Installer.
   I had to revamp it because it used some evil float stuff (redhat v. 6.09999)
-  and it's enums were evil.
+  and its enums were evil.
  */
 
 #include <config.h>
@@ -33,7 +33,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-
+#include <sys/utsname.h>
 #include <string.h>
 
 #define RHAT "RedHat Linux"
@@ -56,6 +56,10 @@
 #define MANDc "Mandrake"
 #define UNKWc "unknown"
 
+/* avoid confusing RPM */
+#define ASSUME_ix86_IS_i386
+
+
 /* FIXME bugzilla.eazel.com 908
    need to implement the rest of the determine_FOO_version
 */
@@ -80,7 +84,7 @@ determine_turbolinux_version (DistributionInfo *distinfo)
 	text = strstr (buf, "release ");
 	if (text) {
 		text += 8;
-		v = g_strndup (text, 3);
+		v = g_strndup (text, 4);
 		sscanf (v, "%d.%d", &version_major, &version_minor);
 		g_free (v);
 		distinfo->version_major = version_major;
@@ -92,6 +96,31 @@ determine_turbolinux_version (DistributionInfo *distinfo)
 static void
 determine_mandrake_version (DistributionInfo *distinfo)
 {
+	FILE *f;
+	char buf[1024];
+	char *text, *v;
+	int version_major, version_minor;
+
+	/* contents, according to loiosh:
+	 * "Linux Mandrake release 7.1 (something)"
+	 */
+	f = fopen ("/etc/mandrake-release", "rt");
+	g_return_if_fail (f != NULL);
+
+	fread ((char *)buf, 1023, 1, f);
+	fclose (f);
+	buf[1023] = 0;
+
+	text = strstr (buf, "release ");
+	if (text) {
+		text += 8;
+		v = g_strndup (text, 4);
+		sscanf (v, "%d.%d", &version_major, &version_minor);
+		g_free (v);
+		distinfo->version_major = version_major;
+		distinfo->version_minor = version_minor;
+		return;
+	}
 }
 
 static void
@@ -259,8 +288,7 @@ trilobite_get_distribution_name (DistributionInfo distinfo,
 		} 
 	}
 
-	/* This slightly odd g_strconcat is odd, so it
-	   eg. can be expanded to have arch */
+	/* This g_strconcat is odd, so it can be expanded, eg. to add arch */
 	result = g_strconcat (name, 
 			      version,
 			      arch,
@@ -330,3 +358,25 @@ trilobite_get_distribution_enum (const char *name, gboolean compact)
 		return trilobite_get_distribution_enum_verbose (name);
 	}
 }
+
+/* returns something like "i386", we hope */
+char *
+trilobite_get_distribution_arch (void)
+{
+	struct utsname utsbuffer;
+	char *arch;
+
+	uname (&utsbuffer);
+	arch = g_strdup (utsbuffer.machine);
+
+#ifdef ASSUME_ix86_IS_i386
+	/* always assume that "ix86" should be "i386", because otherwise RPM can't figure it out */
+	if ((strlen (arch) == 4) && (arch[0] == 'i') &&
+	    ((arch[1] >= '3') && (arch[1] <= '9')) &&
+	    (arch[2] == '8') && (arch[3] == '6')) {
+		arch[1] = '3';
+	}
+#endif
+
+	return arch;
+}
