@@ -281,7 +281,7 @@ eazel_helper_start (int *pipe_stdin, int *pipe_stdout, int *child_pid)
 		case '7':
 		case '8':
 			/* informational messages from userhelper -- don't care */
-			trilobite_debug ("roothelper: info msg '%c'", buffer[0]);
+			/* trilobite_debug ("roothelper: info msg '%c'", buffer[0]); */
 			discard_line (*pipe_stdout);
 			continue;
 		case '*':
@@ -477,6 +477,11 @@ trilobite_root_helper_run_program (TrilobiteRootHelper *root_helper, const char
 
 	/* any argument containing an unprintable is an immediate error */
 	for (item = g_list_first (argv); item; item = g_list_next (item)) {
+		if (item->data == NULL) {
+			g_warning ("TrilobiteRootHelper: run %s: argument %d was (null)",
+				   program, g_list_position (argv, item));
+			continue;
+		}
 		for (p = (char *)item->data; *p; p++) {
 			if ((*p < ' ') || (*p == 127)) {
 				g_warning ("TrilobiteRootHelper: run %s: argument %d contained an "
@@ -488,7 +493,8 @@ trilobite_root_helper_run_program (TrilobiteRootHelper *root_helper, const char
 	}
 
 	out = g_strdup_printf ("%s %u\n", program, g_list_length (argv));
-	if (write (root_helper->pipe_stdin, out, strlen (out)) < strlen (out)) {
+	len = strlen (out);
+	if (write (root_helper->pipe_stdin, out, len) < len) {
 		g_free (out);
 		goto bail;
 	}
@@ -528,7 +534,8 @@ trilobite_root_helper_set_time (TrilobiteRootHelper *root_helper, GList *argv, i
 	GList *item;
 	char *out;
 	char ch;
-
+	int len;
+	
 	if (fd) {
 		*fd = -1;
 	}
@@ -540,7 +547,8 @@ trilobite_root_helper_set_time (TrilobiteRootHelper *root_helper, GList *argv, i
 	}
 
 	out = g_strdup_printf ("set-time %s\n", (char *)item->data);
-	if (write (root_helper->pipe_stdin, out, strlen (out)) < strlen (out)) {
+	len = strlen (out);
+	if (write (root_helper->pipe_stdin, out, len) < len) {
 		g_free (out);
 		goto bail;
 	}
