@@ -148,8 +148,7 @@ nautilus_tree_model_initialize (gpointer object, gpointer klass)
 
 	model->details = g_new0 (NautilusTreeModelDetails, 1);
 
-	model->details->file_to_node_map = g_hash_table_new (g_direct_hash, 
-							     g_direct_equal);
+	model->details->file_to_node_map = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -371,14 +370,34 @@ nautilus_tree_model_node_has_monitor_clients (NautilusTreeModel         *model,
 {
 	return (node->details->monitor_clients != NULL);
 }
+
+
+static void
+nautilus_tree_model_node_begin_monitoring_no_connect (NautilusTreeModel         *model,
+						      NautilusTreeNode          *node,
+						      gboolean                   force_reload)
+{
+	GList             *monitor_attributes;
+	NautilusDirectory *directory;
+
+	directory = nautilus_tree_node_get_directory (node);
+
+	monitor_attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY);
+	nautilus_directory_file_monitor_add (directory,
+					     model,
+					     TRUE, TRUE,
+					     monitor_attributes,
+					     force_reload);
+	g_list_free (monitor_attributes);
+}
+
+
 				 
 static void
 nautilus_tree_model_node_begin_monitoring (NautilusTreeModel         *model,
-					   NautilusTreeNode          *node)
+					   NautilusTreeNode          *node,
+					   gboolean                   force_reload)
 {
-	GList             *monitor_attributes;
-
-
 	NautilusDirectory *directory;
 	directory = nautilus_tree_node_get_directory (node);
 
@@ -402,12 +421,7 @@ nautilus_tree_model_node_begin_monitoring (NautilusTreeModel         *model,
 		 nautilus_tree_model_directory_done_loading_callback,
 		 model);
 
-	monitor_attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY);
-	nautilus_directory_file_monitor_add (directory,
-					     model,
-					     monitor_attributes,
-					     TRUE);
-	g_list_free (monitor_attributes);
+	nautilus_tree_model_node_begin_monitoring_no_connect (model, node, force_reload);
 }
 
 static void
@@ -417,7 +431,11 @@ nautilus_tree_model_node_end_monitoring (NautilusTreeModel         *model,
 	gtk_signal_disconnect (GTK_OBJECT (node->details->directory), node->details->files_added_id);
 	gtk_signal_disconnect (GTK_OBJECT (node->details->directory), node->details->files_changed_id);
 	gtk_signal_disconnect (GTK_OBJECT (node->details->directory), node->details->done_loading_id);
-	
+
+	node->details->files_added_id = 0;
+	node->details->files_changed_id = 0;
+	node->details->done_loading_id = 0;
+
 	nautilus_directory_file_monitor_remove (node->details->directory,
 						model);
 }
@@ -427,7 +445,8 @@ nautilus_tree_model_node_end_monitoring (NautilusTreeModel         *model,
 void
 nautilus_tree_model_monitor_node (NautilusTreeModel         *model,
 				  NautilusTreeNode          *node,
-				  gconstpointer              client)
+				  gconstpointer              client,
+				  gboolean                   force_reload)
 {
 	if (!nautilus_file_is_directory (nautilus_tree_node_get_file (node))) {
 		report_done_loading (model, node);
@@ -435,7 +454,9 @@ nautilus_tree_model_monitor_node (NautilusTreeModel         *model,
 	}
 
 	if (! nautilus_tree_model_node_has_monitor_clients (model, node)) {
-		nautilus_tree_model_node_begin_monitoring (model, node);
+		nautilus_tree_model_node_begin_monitoring (model, node, force_reload);
+	} else if (force_reload) {
+		nautilus_tree_model_node_begin_monitoring_no_connect (model, node, force_reload);
 	}
 
 	if (! g_list_find (node->details->monitor_clients, (gpointer) client)) {
@@ -481,9 +502,19 @@ nautilus_tree_model_stop_monitoring_node_recursive (NautilusTreeModel *model,
 }
 
 
+NautilusTreeNode *
+nautilus_tree_model_get_node_from_file (NautilusTreeModel *model,
+					NautilusFile      *file)
+{
+	g_return_val_if_fail (NAUTILUS_IS_TREE_MODEL (model), NULL);
+	g_return_val_if_fail (NAUTILUS_IS_FILE (file), NULL);
+	
+	return g_hash_table_lookup (model->details->file_to_node_map, file);
+}
+
 NautilusTreeNode *
 nautilus_tree_model_get_node (NautilusTreeModel *model,
-			      const char *uri)
+			      const char        *uri)
 {
 	NautilusFile *file;
 	NautilusTreeNode *node;
@@ -494,7 +525,7 @@ nautilus_tree_model_get_node (NautilusTreeModel *model,
 		return NULL;
 	}
 
-	node = g_hash_table_lookup (model->details->file_to_node_map, file);
+	node = nautilus_tree_model_get_node_from_file (model, file);
 	nautilus_file_unref (file);
 
 	return node;
@@ -512,10 +543,10 @@ dump_one_file_node (gpointer key, gpointer value, gpointer user_data)
 	g_assert (NAUTILUS_IS_TREE_NODE (value));
 	g_assert (user_data != NULL);
 
-	file_number = (guint *)user_data;
+	file_number = (guint *) user_data;
 	uri = nautilus_file_get_uri (NAUTILUS_FILE (key));
 
-	g_print ("%d: %s (0x%x)|\n", ++(*file_number), uri, (unsigned) key);
+	g_print ("%d: %s (%p)|\n", ++(*file_number), uri, key);
 	
 	g_free (uri);
 }
@@ -579,6 +610,15 @@ report_node_changed (NautilusTreeModel *model,
 	char *node_uri;
 	char *file_uri;
 
+	/* Bail out if we don't have all the info we need yet (we'll
+	 * end up reporting the change later, once the info is
+	 * ready). 
+	 */
+
+	if (nautilus_file_get_file_type (node->details->file) == GNOME_VFS_FILE_TYPE_UNKNOWN) {
+		return;
+	}
+
 	node_uri = nautilus_tree_node_get_uri (node);
 
 	if (node->details->directory == NULL && nautilus_file_is_directory (node->details->file)) {
@@ -591,8 +631,8 @@ report_node_changed (NautilusTreeModel *model,
 #endif
 	}
 
-	if (g_hash_table_lookup (model->details->file_to_node_map, 
-				 nautilus_tree_node_get_file (node)) == NULL) {
+	if (nautilus_tree_model_get_node_from_file (model,
+						    nautilus_tree_node_get_file (node)) == NULL) {
 		/* Actually added, go figure */
 		
 		parent_uri = uri_get_parent_text (node_uri);
@@ -668,8 +708,8 @@ report_node_removed_internal (NautilusTreeModel *model,
 		return;
 	}
 
-	if (g_hash_table_lookup (model->details->file_to_node_map, 
-				 nautilus_tree_node_get_file (node)) != NULL) {
+	if (nautilus_tree_model_get_node_from_file 
+	    (model,nautilus_tree_node_get_file (node)) != NULL) {
 
 		parent_node = nautilus_tree_node_get_parent (node);
 			
@@ -738,14 +778,15 @@ nautilus_tree_model_directory_files_changed_callback (NautilusDirectory        *
 	for (p = changed_files; p != NULL; p = p->next) {
 		file = NAUTILUS_FILE (p->data);
 		
-		node = (NautilusTreeNode *) g_hash_table_lookup (model->details->file_to_node_map, file);
+		node = nautilus_tree_model_get_node_from_file (model, file);
 
 		if (node == NULL) {
 			/* Do we need to add this node? */
 			uri = nautilus_file_get_uri (file);
 			g_free (uri);
 		} else {
-			if (!nautilus_directory_contains_file (directory, file)) {
+			if (!nautilus_directory_contains_file (directory, file)
+			    || nautilus_file_is_gone (file)) {
 				report_node_removed (model, node);
 			} else {			
 				report_node_changed (model, node);
@@ -766,7 +807,13 @@ nautilus_tree_model_directory_files_added_callback (NautilusDirectory        *di
 	for (p = added_files; p != NULL; p = p->next) {
 		file = (NautilusFile *) p->data;
 		
-		node = nautilus_tree_node_new (file);
+		node = nautilus_tree_model_get_node_from_file (model, file);
+
+		if (node == NULL) {
+			node = nautilus_tree_node_new (file);
+		} else {
+			gtk_object_ref (GTK_OBJECT (node));
+		}			
 		
 		report_node_changed (model, node);
 
@@ -779,12 +826,16 @@ static void
 nautilus_tree_model_directory_done_loading_callback (NautilusDirectory        *directory,
 						     NautilusTreeModel        *model)
 {
-	char *uri;
+	NautilusFile *file;
 	NautilusTreeNode *node;
 
-	uri = nautilus_directory_get_uri (directory);
-	node = nautilus_tree_model_get_node (model, uri);
-	g_free (uri);
+	file = nautilus_directory_get_corresponding_file (directory);
+	node = nautilus_tree_model_get_node_from_file (model, file);
+	nautilus_file_unref (file);
 
-	report_done_loading (model, node);
+	if (node != NULL) {
+		report_done_loading (model, node);
+	} else {
+		g_warning ("Got done loading notification for nonexistent node %s", nautilus_directory_get_uri (directory));
+	}
 }
