@@ -61,8 +61,7 @@ static void nautilus_background_draw_flat_box    (GtkStyle           *style,
 						  int                 y,
 						  int                 width,
 						  int                 height);
-static void nautilus_background_real_reset       (NautilusBackground *background);
-static void nautilus_background_start_loading_image (NautilusBackground *background);
+static void nautilus_background_start_loading_image (NautilusBackground *background, gboolean emit_appearance_change);
 static gboolean nautilus_background_is_image_load_in_progress (NautilusBackground *background);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusBackground, nautilus_background, GTK_TYPE_OBJECT)
@@ -94,6 +93,7 @@ struct NautilusBackgroundDetails {
 	int image_width_unscaled;
 	int image_height_unscaled;
 	NautilusPixbufLoadHandle *load_image_handle;
+	gboolean emit_after_load;
 	gboolean combine_mode;
 	NautilusBackgroundImagePlacement image_placement;
 
@@ -158,7 +158,6 @@ nautilus_background_initialize_class (gpointer klass)
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	object_class->destroy = nautilus_background_destroy;
-	background_class->reset = nautilus_background_real_reset;
 }
 
 static void
@@ -223,11 +222,8 @@ nautilus_background_set_combine_mode (NautilusBackground *background, gboolean n
 
 	if (new_combine_mode != background->details->combine_mode) {
 		background->details->combine_mode = new_combine_mode;
-		
-		gtk_signal_emit (GTK_OBJECT (background),
-			 signals[SETTINGS_CHANGED]);
-		gtk_signal_emit (GTK_OBJECT (background),
-			 signals[APPEARANCE_CHANGED]);
+		gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
+		gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
 	}
 }
 
@@ -239,16 +235,16 @@ nautilus_background_get_image_placement (NautilusBackground *background)
 	return background->details->image_placement;
 }
 
-void
-nautilus_background_set_image_placement (NautilusBackground *background,
-					 NautilusBackgroundImagePlacement new_placement)
+static gboolean
+nautilus_background_set_image_placement_no_emit (NautilusBackground *background,
+						 NautilusBackgroundImagePlacement new_placement)
 {
-	g_return_if_fail (NAUTILUS_IS_BACKGROUND (background));
+	g_return_val_if_fail (NAUTILUS_IS_BACKGROUND (background), FALSE);
 
 	if (new_placement != background->details->image_placement) {
 
 		if (nautilus_background_is_image_load_in_progress (background)) {
-			/* We try to smart and keep using the current image for updates
+			/* We try to be smart and keep using the current image for updates
 			 * while a new image is loading. However, changing the placement
 			 * foils these plans.
 			 */
@@ -256,7 +252,17 @@ nautilus_background_set_image_placement (NautilusBackground *background,
 		}
 
 		background->details->image_placement = new_placement;
-		
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+void
+nautilus_background_set_image_placement (NautilusBackground *background,
+					 NautilusBackgroundImagePlacement new_placement)
+{
+	if (nautilus_background_set_image_placement_no_emit (background, new_placement)) {
 		gtk_signal_emit (GTK_OBJECT (background),
 			 signals[SETTINGS_CHANGED]);
 		gtk_signal_emit (GTK_OBJECT (background),
@@ -340,9 +346,9 @@ nautilus_background_ensure_gradient_buffered (NautilusBackground *background, in
 		dg = new_color.green - cur_color.green;
 		db = new_color.blue  - cur_color.blue;
 
-		fill_pos   = 0;
-		fill_width = (percent * num_pixels) / 100;
-		fill_limit = MIN (buff_ptr + fill_width * 3, buff_limit);
+		fill_pos   = 1;
+		fill_limit = MIN (background->details->gradient_buffer + 3 * ((num_pixels * percent) / 100), buff_limit);
+		fill_width = (fill_limit - buff_ptr) / 3;
 		
 		while (buff_ptr < fill_limit) {
 			*buff_ptr++ = (cur_color.red   + (dr * fill_pos) / fill_width) >> 8;
@@ -350,7 +356,6 @@ nautilus_background_ensure_gradient_buffered (NautilusBackground *background, in
 			*buff_ptr++ = (cur_color.blue  + (db * fill_pos) / fill_width) >> 8;
 			++fill_pos;
 		}
-
 		cur_color = new_color;
 	}
 
@@ -402,7 +407,7 @@ fill_canvas_from_gradient_buffer (const GnomeCanvasBuf *buf, const NautilusBackg
 {
 	g_return_if_fail (background->details->gradient_buffer != NULL);
 
-	/* FIXME bugzilla.eazel.com 415: This hack is needed till we fix background
+	/* FIXME bugzilla.eazel.com 4876: This hack is needed till we fix background
 	 * scolling.
 	 *
 	 * I.e. currently you can scroll off the end of the gradient - and we
@@ -483,7 +488,7 @@ fill_drawable_from_gradient_buffer (GdkDrawable *drawable, GdkGC *gc,
 	
 	g_return_if_fail (background->details->gradient_buffer != NULL);
 
-	/* FIXME bugzilla.eazel.com 415: This hack is needed till we fix background
+	/* FIXME bugzilla.eazel.com 4876: This hack is needed till we fix background
 	 * scolling.
 	 *
 	 * I.e. currently you can scroll off the end of the gradient - and we
@@ -534,9 +539,9 @@ fill_drawable_from_gradient_buffer (GdkDrawable *drawable, GdkGC *gc,
  */
 static void
 update_drawable_with_pixbuf (GdkDrawable *dst, GdkGC *gc,
-							 int dst_x0, int dst_y0, int dst_x1, int dst_y1,
-							 GdkPixbuf *src,
-							 int src_x0, int src_y0, int src_x1, int src_y1)
+			     int dst_x0, int dst_y0, int dst_x1, int dst_y1,
+			     GdkPixbuf *src,
+			     int src_x0, int src_y0, int src_x1, int src_y1)
 {
 	if (src_x1 > dst_x0 && src_y1 > dst_y0 && src_x0 < dst_x1  && src_y0 < dst_y1) {
 		int src_offset_x;
@@ -573,10 +578,10 @@ update_drawable_with_pixbuf (GdkDrawable *dst, GdkGC *gc,
 		copy_y1 = MIN (src_y1, dst_y1);
 		
 		gdk_pixbuf_render_to_drawable (src, dst, gc,
-									   src_offset_x, src_offset_y,
-									   dst_offset_x, dst_offset_y,
-									   copy_x1 - copy_x0, copy_y1 - copy_y0,
-									   GDK_RGB_DITHER_NONE, 0, 0);
+					       src_offset_x, src_offset_y,
+					       dst_offset_x, dst_offset_y,
+					       copy_x1 - copy_x0, copy_y1 - copy_y0,
+					       GDK_RGB_DITHER_NONE, 0, 0);
 	}
 }
 
@@ -597,9 +602,9 @@ draw_pixbuf_tiled (GdkPixbuf *pixbuf, GdkDrawable *drawable, GdkGC *gc, int draw
 	for (y = start_y; y < drawable_bottom; y += tile_height) {
 		for (x = start_x; x < drawable_right; x += tile_width) {
 			update_drawable_with_pixbuf (drawable, gc,
-										 drawable_x, drawable_y, drawable_right, drawable_bottom,
-										 pixbuf,
-										 x, y, x + tile_width, y + tile_height);
+						     drawable_x, drawable_y, drawable_right, drawable_bottom,
+						     pixbuf,
+						     x, y, x + tile_width, y + tile_height);
 		}
 	}
 }
@@ -707,7 +712,7 @@ nautilus_background_ensure_image_scaled (NautilusBackground *background, int des
 		if (reload_image) {
 			gdk_pixbuf_unref (background->details->image);
 			background->details->image = NULL;
-			nautilus_background_start_loading_image (background);
+			nautilus_background_start_loading_image (background, TRUE);
 			background->details->image_rect_x = 0;
 			background->details->image_rect_y = 0;
 			background->details->image_rect_width = 0;
@@ -739,9 +744,9 @@ nautilus_background_pre_draw (NautilusBackground *background, int entire_width,
  */
 void
 nautilus_background_draw (NautilusBackground *background,
-						  GdkDrawable *drawable, GdkGC *gc,
-						  int drawable_x, int drawable_y,
-						  int drawable_width, int drawable_height)
+			  GdkDrawable *drawable, GdkGC *gc,
+			  int drawable_x, int drawable_y,
+			  int drawable_width, int drawable_height)
 {
 	int drawable_right;
 	int drawable_bottom;
@@ -750,6 +755,19 @@ nautilus_background_draw (NautilusBackground *background,
 
 	g_return_if_fail (NAUTILUS_IS_BACKGROUND (background));
 
+	/* Combine mode is currently used only by the sidebar. It allocates a pixbuf
+	 * on every draw, so further spread of its use should not be encouraged.
+	 * 
+	 * Combine mode allows the sidebar, regardless of aa mode, to alpha composite
+	 * an image over a color (an effect utilized by the sidebar in some themes)
+	 * and avoid striations in gradients by using dithering.
+	 * 
+	 * FIXME bugzila.eazel.com 5516:
+	 * The whole combine mode concept could be elimintated by some minor refactoring that
+	 * would basically move the code below to a place used only by nautilus_background_draw_flat_box.
+	 * Also, the various pieces of code that manage the combine setting could be elimintated
+	 * from nautilus-sidebar and nautilus-directory-background.
+	 */
 	if (background->details->combine_mode) {
 		GdkPixbuf *pixbuf;
 		GnomeCanvasBuf buffer;
@@ -768,7 +786,7 @@ nautilus_background_draw (NautilusBackground *background,
 					       0, 0,
 					       0, 0,
 					       drawable_width, drawable_height,
-					       GDK_RGB_DITHER_NONE, 0, 0);
+					       GDK_RGB_DITHER_MAX, 0, 0);
 		
 		/* free things up and we're done */
 		gdk_pixbuf_unref (pixbuf);
@@ -806,11 +824,11 @@ nautilus_background_draw (NautilusBackground *background,
 			/* Since the image has already been scaled, all these cases can be treated identically.
 			 */
 			update_drawable_with_pixbuf (drawable, gc,
-										 drawable_x, drawable_y,
-										 drawable_right, drawable_bottom,
-							 			 background->details->image,
-							 			 background->details->image_rect_x, background->details->image_rect_y,
-							 			 image_rect_right, image_rect_bottom);
+						     drawable_x, drawable_y,
+						     drawable_right, drawable_bottom,
+						     background->details->image,
+						     background->details->image_rect_x, background->details->image_rect_y,
+						     image_rect_right, image_rect_bottom);
 			break;
 		}
 	}
@@ -818,15 +836,37 @@ nautilus_background_draw (NautilusBackground *background,
 
 void
 nautilus_background_draw_to_drawable (NautilusBackground *background,
-						 			  GdkDrawable *drawable, GdkGC *gc,
-									  int drawable_x, int drawable_y,
-									  int drawable_width, int drawable_height,
-									  int entire_width, int entire_height)
+				      GdkDrawable *drawable, GdkGC *gc,
+				      int drawable_x, int drawable_y,
+				      int drawable_width, int drawable_height,
+				      int entire_width, int entire_height)
 {
 	nautilus_background_pre_draw (background, entire_width, entire_height);
 	nautilus_background_draw (background, drawable, gc, drawable_x, drawable_y, drawable_width, drawable_height);
 }
 
+void
+nautilus_background_draw_to_pixbuf (NautilusBackground *background,
+				    GdkPixbuf *pixbuf,
+				    int pixbuf_x,
+				    int pixbuf_y,
+				    int pixbuf_width,
+				    int pixbuf_height,
+				    int entire_width,
+				    int entire_height)
+{
+	GnomeCanvasBuf fake_buffer;
+
+	g_return_if_fail (background != NULL);
+	g_return_if_fail (pixbuf != NULL);
+
+	canvas_buf_from_pixbuf (&fake_buffer, pixbuf, pixbuf_x, pixbuf_y, pixbuf_width, pixbuf_height);
+
+	nautilus_background_draw_to_canvas (background,
+					    &fake_buffer,
+					    entire_width,
+					    entire_height);
+}
 
 /* fill the canvas buffer with a tiled pixbuf */
 static void
@@ -893,9 +933,9 @@ nautilus_background_draw_aa (NautilusBackground *background, GnomeCanvasBuf *buf
 			 * can be treated identically.
 			 */
 			nautilus_gnome_canvas_draw_pixbuf (buffer,
-											   background->details->image,
-											   background->details->image_rect_x,
-											   background->details->image_rect_y);
+							   background->details->image,
+							   background->details->image_rect_x,
+							   background->details->image_rect_y);
 			break;
 		}
 	}
@@ -906,9 +946,9 @@ nautilus_background_draw_aa (NautilusBackground *background, GnomeCanvasBuf *buf
 
 void
 nautilus_background_draw_to_canvas (NautilusBackground *background,
-								    GnomeCanvasBuf *buffer,
-									int entire_width,
-									int entire_height)
+				    GnomeCanvasBuf *buffer,
+				    int entire_width,
+				    int entire_height)
 {
 	nautilus_background_pre_draw (background, entire_width, entire_height);
 	nautilus_background_draw_aa (background, buffer);
@@ -960,8 +1000,8 @@ nautilus_background_set_color (NautilusBackground *background,
 
 static void
 nautilus_background_load_image_callback (GnomeVFSResult error,
-										 GdkPixbuf *pixbuf,
-										 gpointer callback_data)
+					 GdkPixbuf *pixbuf,
+					 gpointer callback_data)
 {
 	NautilusBackground *background;
 
@@ -979,8 +1019,11 @@ nautilus_background_load_image_callback (GnomeVFSResult error,
 		background->details->image_height_unscaled = gdk_pixbuf_get_height (pixbuf);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
 	gtk_signal_emit (GTK_OBJECT (background), signals[IMAGE_LOADING_DONE], pixbuf != NULL || background->details->image_uri == NULL);
+
+	if (background->details->emit_after_load) {
+		gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
+	}
 }
 
 static gboolean
@@ -1000,20 +1043,24 @@ nautilus_background_cancel_loading_image (NautilusBackground *background)
 }
 
 static void
-nautilus_background_start_loading_image (NautilusBackground *background)
+nautilus_background_start_loading_image (NautilusBackground *background, gboolean emit_appearance_change)
 {
+	background->details->emit_after_load = emit_appearance_change;
+
 	if (background->details->image_uri != NULL) {
 		background->details->load_image_handle = nautilus_gdk_pixbuf_load_async (background->details->image_uri,
-																				 nautilus_background_load_image_callback,
-																				 background);
+											 nautilus_background_load_image_callback,
+											 background);
 	} else {
 		nautilus_background_load_image_callback (0, NULL, background);
 	}
 }
 
 static gboolean
-nautilus_background_set_image_uri_no_emit (NautilusBackground *background,
-					const char *image_uri)
+nautilus_background_set_image_uri_helper (NautilusBackground *background,
+					const char *image_uri,
+					gboolean emit_setting_change,
+					gboolean emit_appearance_change)
 {
 	g_return_val_if_fail (NAUTILUS_IS_BACKGROUND (background), FALSE);
 
@@ -1031,38 +1078,35 @@ nautilus_background_set_image_uri_no_emit (NautilusBackground *background,
 	 * current image can be used if an update is needed before the load completes.
 	 */
 	
-	nautilus_background_start_loading_image (background);
+	nautilus_background_start_loading_image (background, emit_appearance_change);
 
+	if (emit_setting_change) {
+		gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
+	}
 
 	return TRUE;
 }
 
-static gboolean
-nautilus_background_set_image_uri_helper (NautilusBackground *background, const char *image_uri)
-{
-	if (nautilus_background_set_image_uri_no_emit (background, image_uri)) {
-		/* The APPEARANCE_CHANGED is emitted when the image is done loading,
-		 * in nautilus_background_load_image_callback. 
-		 */
-		gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
-		return TRUE;
-	} else {
-		return FALSE;
-	}
-}
-
 void
 nautilus_background_set_image_uri (NautilusBackground *background, const char *image_uri)
 {
-	nautilus_background_set_image_uri_helper (background, image_uri);
+	nautilus_background_set_image_uri_helper (background, image_uri, TRUE, TRUE);
 }
 
-
 static void
-set_image_and_color_image_loading_done_callback (NautilusBackground *background, gboolean successful_load, const char *color)
+set_image_and_color_image_loading_done_callback (NautilusBackground *background, gboolean successful_load, char *color)
 {
-		gtk_signal_disconnect_by_func (GTK_OBJECT (background), GTK_SIGNAL_FUNC (set_image_and_color_image_loading_done_callback), (gpointer) color);
-		nautilus_background_set_color (background, color);
+	gtk_signal_disconnect_by_func (GTK_OBJECT (background), GTK_SIGNAL_FUNC (set_image_and_color_image_loading_done_callback), (gpointer) color);
+
+	nautilus_background_set_color_no_emit (background, color);
+
+	g_free (color);
+	
+	/* We always emit , even if the color didn't change, because the image change
+	 * relies on us doing it here.
+	 */
+	gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
+	gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
 }
 
 /* Use this fn to set both the image and color and avoid flash. The color isn't
@@ -1072,15 +1116,34 @@ set_image_and_color_image_loading_done_callback (NautilusBackground *background,
 static void
 nautilus_background_set_image_uri_and_color (NautilusBackground *background, const char *image_uri, const char *color)
 {
+	char *color_copy;
+
+	if (nautilus_strcmp (background->details->color, color) == 0 &&
+	    nautilus_strcmp (background->details->image_uri, image_uri) == 0) {
+		return;
+	}
+
+	color_copy = g_strdup (color);
+
 	gtk_signal_connect (GTK_OBJECT (background),
-						"image_loading_done",
-						GTK_SIGNAL_FUNC (set_image_and_color_image_loading_done_callback),
-						(gpointer) color);
-	if (!nautilus_background_set_image_uri_helper (background, image_uri)) {
-		/* If the image loading was a no-op, then we have to manually call
-		 * set_image_and_color_image_loading_done_callback. 
-		 */
-		set_image_and_color_image_loading_done_callback (background, TRUE, color);
+			    "image_loading_done",
+			    GTK_SIGNAL_FUNC (set_image_and_color_image_loading_done_callback),
+			    (gpointer) color_copy);
+			    
+	/* set_image_and_color_image_loading_done_callback must always be called
+	 * because we rely on it to:
+	 *  - disconnect the image_loading_done signal handler
+	 *  - emit SETTINGS_CHANGED & APPEARANCE_CHANGED
+	 *  - free color_copy
+	 *  - prevent the common cold
+	 */
+	     
+	/* We use nautilus_background_set_image_uri_helper because its
+	 * return value (if false) tells us whether or not we need to
+	 * call set_image_and_color_image_loading_done_callback ourselves.
+	 */
+	if (!nautilus_background_set_image_uri_helper (background, image_uri, FALSE, FALSE)) {
+		set_image_and_color_image_loading_done_callback (background, TRUE, color_copy);
 	}
 }
 
@@ -1092,6 +1155,12 @@ nautilus_background_receive_dropped_background_image (NautilusBackground *backgr
 	if (nautilus_str_has_suffix (image_uri, "/" RESET_BACKGROUND_MAGIC_IMAGE_NAME)) {
 		nautilus_background_reset (background);
 	} else {
+		/* Currently, we only support tiled images. So we set the placement.
+		 * We rely on nautilus_background_set_image_uri_and_color to emit
+		 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
+		 */
+		nautilus_background_set_image_placement_no_emit (background, NAUTILUS_BACKGROUND_TILED);
+
 		nautilus_background_set_image_uri_and_color (background, image_uri, NULL);
 	}
 }
@@ -1168,6 +1237,8 @@ nautilus_background_draw_flat_box (GtkStyle *style,
 	nautilus_gdk_window_update_sizes (window, &width, &height);	
 
 	nautilus_background_draw_to_drawable (background, window, gc, 0, 0, width, height, widget->allocation.width, widget->allocation.height);
+	if (area)
+		gdk_gc_set_clip_rectangle (gc, NULL);
 }
 
 static GtkStyleClass *
@@ -1258,42 +1329,14 @@ nautilus_background_reset (NautilusBackground *background)
 {
 	g_return_if_fail (NAUTILUS_IS_BACKGROUND (background));
 
-	gtk_signal_emit (GTK_OBJECT (background),
-			 signals[RESET]);
-}
-
-static void
-nautilus_background_real_reset (NautilusBackground *background)
-{
-	nautilus_background_set_image_uri_and_color (background, NULL, NULL);
+	gtk_signal_emit (GTK_OBJECT (background), signals[RESET]);
 }
 
 static void
 nautilus_background_set_up_canvas (GtkWidget *widget)
 {
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-
-	/* Attach ourselves to a canvas in a way that will work.
-	   Changing the style is not sufficient.
-
-	   Since there's no signal to override in GnomeCanvas to control
-	   drawing the background, we change the class of the canvas root.
-	   This gives us a chance to draw the background before any of the
-	   objects draw themselves, and has no effect on the bounds or
-	   anything related to scrolling.
-
-	   We settled on this after less-than-thrilling results using a
-	   canvas item as the background. The canvas item contributed to
-	   the bounds of the canvas and had to constantly be resized.
-	*/
 	if (GNOME_IS_CANVAS (widget)) {
-		g_assert (GTK_OBJECT (GNOME_CANVAS (widget)->root)->klass
-			  == gtk_type_class (GNOME_TYPE_CANVAS_GROUP)
-			  || GTK_OBJECT (GNOME_CANVAS (widget)->root)->klass
-			  == gtk_type_class (NAUTILUS_TYPE_BACKGROUND_CANVAS_GROUP));
-
-		GTK_OBJECT (GNOME_CANVAS (widget)->root)->klass =
-			gtk_type_class (NAUTILUS_TYPE_BACKGROUND_CANVAS_GROUP);
+		nautilus_background_canvas_group_supplant_root_class (GNOME_CANVAS (widget));
 	}
 }
 
@@ -1356,6 +1399,22 @@ nautilus_widget_has_attached_background (GtkWidget *widget)
 	return gtk_object_get_data (GTK_OBJECT (widget), "nautilus_background") != NULL;
 }
 
+GtkWidget *
+nautilus_gtk_widget_find_background_ancestor (GtkWidget *widget)
+{
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+
+	while (widget != NULL) {
+		if (nautilus_widget_has_attached_background (widget)) {
+			return widget;
+		}
+
+		widget = widget->parent;
+	}
+
+	return NULL;
+}
+
 gboolean
 nautilus_background_is_too_complex_for_gtk_style (NautilusBackground *background)
 {
@@ -1368,7 +1427,7 @@ nautilus_background_is_too_complex_for_gtk_style (NautilusBackground *background
 	if (background->details->image_uri != NULL) {
 		return TRUE;
 	}
-	if (nautilus_gradient_is_gradient (background->details->color)) {
+	if (!background->details->is_solid_color) {
 		return TRUE;
 	}
 	
@@ -1388,8 +1447,9 @@ nautilus_background_is_dark (NautilusBackground *background)
 	
 	if (background->details->image != NULL) {
 		nautilus_gdk_pixbuf_average_value (background->details->image, &color);
-			
-	} else if (nautilus_gradient_is_gradient (background->details->color)) {
+	} else if (background->details->is_solid_color) {
+		nautilus_gdk_color_parse_with_white_default (background->details->color, &color);	
+	} else {
 		start_color_spec = nautilus_gradient_get_start_color_spec (background->details->color);
 		end_color_spec = nautilus_gradient_get_end_color_spec (background->details->color);
 		
@@ -1402,8 +1462,6 @@ nautilus_background_is_dark (NautilusBackground *background)
 		g_free (end_color_spec);
 		
 		return (intensity + intensity2) < 320;
-	} else {
-		nautilus_gdk_color_parse_with_white_default (background->details->color, &color);	
 	}
 	
 	intensity = (((color.red >> 8) * 77) + ((color.green >> 8) * 150) + ((color.blue >> 8) * 28)) >> 8;		 
