@@ -41,19 +41,24 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+/* turn this on to see messages about each load_directory call: */
+#if 0
+#define DEBUG_LOAD_DIRECTORY
+#endif
+
+/* turn this on to check if async. job calls are balanced */
+#if 0
+#define DEBUG_ASYNC_JOBS
+#endif
+
 #define METAFILE_PERMISSIONS (GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE \
 			      | GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_WRITE \
 			      | GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_WRITE)
 
 #define DIRECTORY_LOAD_ITEMS_PER_CALLBACK 32
 
-/* comment this back in to see messages about each load_directory call:
-#define DEBUG_LOAD_DIRECTORY
-*/
-
-/* comment this back in to check if async. job calls are balanced
-#define DEBUG_ASYNC_JOBS
-*/
+/* Keep async. jobs down to this number for all directories. */
+#define MAX_ASYNC_JOBS 10
 
 struct MetafileReadState {
 	gboolean use_public_metafile;
@@ -65,7 +70,7 @@ struct MetafileWriteState {
 	gboolean use_public_metafile;
 	GnomeVFSAsyncHandle *handle;
 	xmlChar *buffer;
-	int size;
+	GnomeVFSFileSize size;
 	gboolean write_again;
 };
 
@@ -79,18 +84,6 @@ struct ActivationURIReadState {
 	NautilusReadFileHandle *handle;
 };
 
-/* A request for information about one or more files. */
-typedef struct {
-	gboolean metafile;
-	gboolean file_list; /* always FALSE if file != NULL */
-	gboolean file_info;
-	gboolean directory_count;
-	gboolean deep_count;
-	gboolean mime_list;
-	gboolean top_left_text;
-	gboolean activation_uri;
-} Request;
-
 typedef struct {
 	NautilusFile *file; /* Which file, NULL means all. */
 	union {
@@ -103,6 +96,8 @@ typedef struct {
 
 typedef struct {
 	NautilusFile *file; /* Which file, NULL means all. */
+	gboolean monitor_hidden_files; /* defines whether "all" includes hidden files */
+	gboolean monitor_backup_files; /* defines whether "all" includes backup files */
 	gconstpointer client;
 	Request request;
 } Monitor;
@@ -110,9 +105,6 @@ typedef struct {
 typedef gboolean (* RequestCheck) (const Request *);
 typedef gboolean (* FileCheck) (NautilusFile *);
 
-/* Keep async. jobs down to this number for all directories. */
-#define MAX_ASYNC_JOBS 10
-
 /* Current number of async. jobs. */
 static int async_job_count;
 static GHashTable *waiting_directories;
@@ -121,12 +113,14 @@ static GHashTable *async_jobs;
 #endif
 
 /* Forward declarations for functions that need them. */
-static void     deep_count_load       (NautilusDirectory *directory,
-				       const char        *uri);
-static void     metafile_read_restart (NautilusDirectory *directory);
-static gboolean request_is_satisfied  (NautilusDirectory *directory,
-				       NautilusFile      *file,
-				       Request           *request);
+static void     deep_count_load           (NautilusDirectory *directory,
+					   const char        *uri);
+static void     metafile_read_restart     (NautilusDirectory *directory);
+static gboolean request_is_satisfied      (NautilusDirectory *directory,
+					   NautilusFile      *file,
+					   Request           *request);
+static void     cancel_loading_attributes (NautilusDirectory *directory,
+					   GList             *file_attributes);
 
 /* Some helpers for case-insensitive strings.
  * Move to nautilus-glib-extensions?
@@ -212,7 +206,7 @@ async_job_start (NautilusDirectory *directory,
 	if (async_job_count >= MAX_ASYNC_JOBS) {
 		if (waiting_directories == NULL) {
 			waiting_directories = nautilus_g_hash_table_new_free_at_exit
-				(g_direct_hash, g_direct_equal,
+				(NULL, NULL,
 				 "nautilus-directory-async.c: waiting_directories");
 		}
 
@@ -437,9 +431,7 @@ can_use_public_metafile (NautilusDirectory *directory)
 		return FALSE;
 	}
 
-	preference_value = nautilus_preferences_get_enum
-		(NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA,
-		 NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY);
+	preference_value = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA);
 
 	if (preference_value == NAUTILUS_SPEED_TRADEOFF_ALWAYS) {
 		return TRUE;
@@ -612,7 +604,8 @@ metafile_read_done_callback (GnomeVFSResult result,
 	/* The gnome-xml parser requires a zero-terminated array. */
 	buffer = g_realloc (file_contents, size + 1);
 	buffer[size] = '\0';
-	directory->details->metafile = xmlParseMemory (buffer, size);
+	nautilus_directory_set_metafile_contents (directory,
+						  xmlParseMemory (buffer, size));
 	g_free (buffer);
 
 	metafile_read_done (directory);
@@ -660,11 +653,14 @@ allow_metafile (NautilusDirectory *directory)
 	 * access inside gnome-vfs.
 	 */
 	uri = directory->details->uri;
-	if (nautilus_istr_has_prefix (uri, "info:")
+	if (nautilus_is_search_uri (uri)
+	    || nautilus_istr_has_prefix (uri, "ghelp:")
+	    || nautilus_istr_has_prefix (uri, "gnome-help:")
 	    || nautilus_istr_has_prefix (uri, "help:")
+	    || nautilus_istr_has_prefix (uri, "info:")
 	    || nautilus_istr_has_prefix (uri, "man:")
 	    || nautilus_istr_has_prefix (uri, "pipe:")
-	    || nautilus_is_search_uri (uri)) {
+	    ) {
 		return FALSE;
 	}
 	
@@ -855,6 +851,8 @@ nautilus_metafile_write_start (NautilusDirectory *directory)
 static void
 metafile_write (NautilusDirectory *directory)
 {
+	int xml_doc_size;
+	
 	g_assert (NAUTILUS_IS_DIRECTORY (directory));
 
 	nautilus_directory_ref (directory);
@@ -881,7 +879,8 @@ metafile_write (NautilusDirectory *directory)
 		= can_use_public_metafile (directory);
 	xmlDocDumpMemory (directory->details->metafile,
 			  &directory->details->metafile_write_state->buffer,
-			  &directory->details->metafile_write_state->size);
+			  &xml_doc_size);
+	directory->details->metafile_write_state->size = xml_doc_size;
 	nautilus_metafile_write_start (directory);
 }
 
@@ -953,7 +952,7 @@ find_monitor (NautilusDirectory *directory,
 	GList *result;
 	Monitor *monitor;
 
-	monitor = g_new (Monitor, 1);
+	monitor = g_new0 (Monitor, 1);
 	monitor->client = client;
 	monitor->file = file;
 
@@ -986,8 +985,8 @@ remove_monitor (NautilusDirectory *directory,
 	remove_monitor_link (directory, find_monitor (directory, file, client));
 }
 
-static void
-set_up_request_by_file_attributes (Request *request,
+void
+nautilus_directory_set_up_request (Request *request,
 				   GList *file_attributes)
 {
 	memset (request, 0, sizeof (*request));
@@ -995,44 +994,50 @@ set_up_request_by_file_attributes (Request *request,
 	request->directory_count = g_list_find_custom
 		(file_attributes,
 		 NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_COUNT,
-		 nautilus_str_compare) != NULL;
+		 nautilus_strcmp_compare_func) != NULL;
 	request->deep_count = g_list_find_custom
 		(file_attributes,
 		 NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS,
-		 nautilus_str_compare) != NULL;
+		 nautilus_strcmp_compare_func) != NULL;
 	request->mime_list = g_list_find_custom
 		(file_attributes,
 		 NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES,
-		 nautilus_str_compare) != NULL;
-	request->top_left_text = g_list_find_custom
-		(file_attributes,
-		 NAUTILUS_FILE_ATTRIBUTE_TOP_LEFT_TEXT,
-		 nautilus_str_compare) != NULL;
+		 nautilus_strcmp_compare_func) != NULL;
 	request->file_info = g_list_find_custom
 		(file_attributes,
 		 NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE,
-		 nautilus_str_compare) != NULL;
+		 nautilus_strcmp_compare_func) != NULL;
 	request->file_info |= g_list_find_custom
 		(file_attributes,
 		 NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY,
-		 nautilus_str_compare) != NULL;
+		 nautilus_strcmp_compare_func) != NULL;
+	request->file_info |= g_list_find_custom
+		(file_attributes,
+		 NAUTILUS_FILE_ATTRIBUTE_CAPABILITIES,
+		 nautilus_strcmp_compare_func) != NULL;
 	request->file_info |= g_list_find_custom
 		(file_attributes,
 		 NAUTILUS_FILE_ATTRIBUTE_FILE_TYPE,
-		 nautilus_str_compare) != NULL;
+		 nautilus_strcmp_compare_func) != NULL;
+	
+	if (g_list_find_custom (file_attributes,
+				NAUTILUS_FILE_ATTRIBUTE_TOP_LEFT_TEXT,
+				nautilus_strcmp_compare_func) != NULL) {
+		request->top_left_text = TRUE;
+		request->file_info = TRUE;
+	}
+	
 	if (g_list_find_custom (file_attributes,
 				NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI,
-				nautilus_str_compare) != NULL) {
+				nautilus_strcmp_compare_func) != NULL) {
 		request->file_info = TRUE;
 		request->activation_uri = TRUE;
 	}
 	
-	if (!request->metafile) {
-		request->metafile = g_list_find_custom
-			(file_attributes,
-			 NAUTILUS_FILE_ATTRIBUTE_METADATA,
-			 nautilus_str_compare) != NULL;
-	}
+	request->metafile |= g_list_find_custom
+		(file_attributes,
+		 NAUTILUS_FILE_ATTRIBUTE_METADATA,
+		 nautilus_strcmp_compare_func) != NULL;
 
 	/* FIXME bugzilla.eazel.com 2435:
 	 * Some file attributes are really pieces of metadata.
@@ -1045,12 +1050,10 @@ set_up_request_by_file_attributes (Request *request,
 	 * directly (would need some funky char trick to prevent
 	 * namespace collisions).
 	 */
-	if (!request->metafile) {
-		request->metafile = g_list_find_custom
-			(file_attributes,
-			 NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON,
-			 nautilus_str_compare) != NULL;
-	}
+	request->metafile |= g_list_find_custom
+		(file_attributes,
+		 NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON,
+		 nautilus_strcmp_compare_func) != NULL;
 }
 
 static gboolean
@@ -1082,6 +1085,8 @@ void
 nautilus_directory_monitor_add_internal (NautilusDirectory *directory,
 					 NautilusFile *file,
 					 gconstpointer client,
+					 gboolean monitor_hidden_files,
+					 gboolean monitor_backup_files,
 					 GList *file_attributes)
 {
 	Monitor *monitor;
@@ -1095,8 +1100,10 @@ nautilus_directory_monitor_add_internal (NautilusDirectory *directory,
 	/* Add the new monitor. */
 	monitor = g_new (Monitor, 1);
 	monitor->file = file;
+	monitor->monitor_hidden_files = monitor_hidden_files;
+	monitor->monitor_backup_files = monitor_backup_files;
 	monitor->client = client;
-	set_up_request_by_file_attributes (&monitor->request, file_attributes);
+	nautilus_directory_set_up_request (&monitor->request, file_attributes);
 
 	monitor->request.file_list = file == NULL;
 	directory->details->monitor_list =
@@ -1140,18 +1147,57 @@ set_file_unconfirmed (NautilusFile *file, gboolean unconfirmed)
 	}
 }
 
+static gboolean show_hidden_files = TRUE;
+static gboolean show_backup_files = TRUE;
+
+static void
+show_hidden_files_changed_callback (gpointer callback_data)
+{
+	show_hidden_files = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES);
+}
+
+static void
+show_backup_files_changed_callback (gpointer callback_data)
+{
+	show_backup_files = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES);
+}
+
 static GnomeVFSDirectoryFilterOptions
 get_filter_options_for_directory_count (void)
 {
+	static gboolean show_hidden_files_changed_callback_installed = FALSE;
+	static gboolean show_backup_files_changed_callback_installed = FALSE;
 	GnomeVFSDirectoryFilterOptions filter_options;
 	
 	filter_options = GNOME_VFS_DIRECTORY_FILTER_NOSELFDIR
 		| GNOME_VFS_DIRECTORY_FILTER_NOPARENTDIR;
+
+	/* Add the callback once for the life of our process */
+	if (show_hidden_files_changed_callback_installed == FALSE) {
+		nautilus_preferences_add_callback_while_process_is_running (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
+									    show_hidden_files_changed_callback,
+									    NULL);
+		show_hidden_files_changed_callback_installed = TRUE;
+		
+		/* Peek for the first time */
+		show_hidden_files_changed_callback (NULL);
+	}
+
+	/* Add the callback once for the life of our process */
+	if (show_backup_files_changed_callback_installed == FALSE) {
+		nautilus_preferences_add_callback_while_process_is_running (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES,
+									    show_backup_files_changed_callback,
+									    NULL);
+		show_backup_files_changed_callback_installed = TRUE;
+		
+		/* Peek for the first time */
+		show_backup_files_changed_callback (NULL);
+	}
 	
-	if (!nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES, FALSE)) {
+	if (!show_hidden_files) {
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_NODOTFILES;
 	}
-	if (!nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES, FALSE)) {
+	if (!show_backup_files) {
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES;
 	}
 
@@ -1219,7 +1265,7 @@ dequeue_pending_idle_callback (gpointer callback_data)
 		file_info = node->data;
 
 		/* Update the file count. */
-		/* FIXME: This could count a file twice if we get it
+		/* FIXME bugzilla.eazel.com 5063: This could count a file twice if we get it
 		 * from both load_directory and from
 		 * new_files_callback. Not too hard to fix by moving
 		 * this into the actual callback instead of waiting
@@ -1287,11 +1333,14 @@ dequeue_pending_idle_callback (gpointer callback_data)
 		nautilus_directory_emit_done_loading (directory);
 
 		file = directory->details->load_directory_file;
+
 		if (file != NULL) {
+			file->details->directory_count_is_up_to_date = TRUE;
 			file->details->got_directory_count = TRUE;
 			file->details->directory_count = directory->details->load_file_count;
 
 			file->details->got_mime_list = TRUE;
+			file->details->mime_list_is_up_to_date = TRUE;
 			file->details->mime_list = istr_set_get_as_list
 				(directory->details->load_mime_list_hash);
 
@@ -1352,7 +1401,11 @@ directory_load_done (NautilusDirectory *directory,
 	directory->details->directory_loaded = TRUE;
 	directory->details->directory_loaded_sent_notification = FALSE;
 
-	if (result != GNOME_VFS_ERROR_EOF) {
+	/* Note that GNOME_VFS_OK can make it this far when the file-list
+	 * length limit has been reached. In that case, don't treat it as
+	 * an error.
+	 */
+	if (result != GNOME_VFS_ERROR_EOF && result != GNOME_VFS_OK) {
 		/* The load did not complete successfully. This means
 		 * we don't know the status of the files in this directory.
 		 * We clear the unconfirmed bit on each file here so that
@@ -1363,6 +1416,9 @@ directory_load_done (NautilusDirectory *directory,
 		     node != NULL; node = node->next) {
 			set_file_unconfirmed (NAUTILUS_FILE (node->data), FALSE);
 		}
+
+		nautilus_directory_emit_load_error (directory,
+						    result);
 	}
 
 	/* Call the idle function right away. */
@@ -1397,6 +1453,7 @@ directory_load_callback (GnomeVFSAsyncHandle *handle,
 
 	directory = NAUTILUS_DIRECTORY (callback_data);
 
+	g_assert (directory->details->directory_load_in_progress != NULL);
 	g_assert (directory->details->directory_load_in_progress == handle);
 
 	/* Move items from the list onto our pending queue.
@@ -1522,7 +1579,7 @@ nautilus_directory_call_when_ready_internal (NautilusDirectory *directory,
 		callback.callback.file = file_callback;
 	}
 	callback.callback_data = callback_data;
-	set_up_request_by_file_attributes (&callback.request, file_attributes);
+	nautilus_directory_set_up_request (&callback.request, file_attributes);
 	callback.request.file_list = file == NULL && file_attributes != NULL;
 	
 	/* Handle the NULL case. */
@@ -1556,7 +1613,7 @@ nautilus_directory_check_if_ready_internal (NautilusDirectory *directory,
 
 	g_assert (NAUTILUS_IS_DIRECTORY (directory));
 
-	set_up_request_by_file_attributes (&request, file_attributes);
+	nautilus_directory_set_up_request (&request, file_attributes);
 	return request_is_satisfied (directory, file, &request);
 }					    
 
@@ -1635,6 +1692,8 @@ directory_count_callback (GnomeVFSAsyncHandle *handle,
 		return;
 	}
 
+	count_file->details->directory_count_is_up_to_date = TRUE;
+
 	/* Record either a failure or success. */
 	if (result != GNOME_VFS_ERROR_EOF) {
 		count_file->details->directory_count_failed = TRUE;
@@ -1777,8 +1836,7 @@ static gboolean
 lacks_directory_count (NautilusFile *file)
 {
 	return nautilus_file_is_directory (file)
-		&& !file->details->got_directory_count
-		&& !file->details->directory_count_failed;
+		&& !file->details->directory_count_is_up_to_date;
 }
 
 static gboolean
@@ -1799,7 +1857,7 @@ lacks_top_left (NautilusFile *file)
 {
 	return nautilus_file_should_get_top_left_text (file)
 		&& nautilus_file_contains_text (file)
-		&& !file->details->got_top_left_text;
+		&& !file->details->top_left_text_is_up_to_date;
 }
 
 static gboolean
@@ -1811,9 +1869,8 @@ wants_top_left (const Request *request)
 static gboolean
 lacks_info (NautilusFile *file)
 {
-	return file->details->info == NULL
-		&& !file->details->is_gone
-		&& !file->details->get_info_failed;
+	return !file->details->file_info_is_up_to_date
+		&& !file->details->is_gone;
 }
 
 static gboolean
@@ -1839,8 +1896,7 @@ static gboolean
 lacks_mime_list (NautilusFile *file)
 {
 	return nautilus_file_is_directory (file)
-		&& !file->details->got_mime_list
-		&& !file->details->mime_list_failed;
+		&& !file->details->mime_list_is_up_to_date;
 }
 
 static gboolean
@@ -1860,7 +1916,7 @@ static gboolean
 lacks_activation_uri (NautilusFile *file)
 {
 	return file->details->info != NULL
-		&& !file->details->got_activation_uri;
+		&& !file->details->activation_uri_is_up_to_date;
 }
 
 static gboolean
@@ -1914,7 +1970,13 @@ request_is_satisfied (NautilusDirectory *directory,
 			return FALSE;
 		}
 	}
-	
+
+	if (request->top_left_text) {
+		if (has_problem (directory, file, lacks_top_left)) {
+			return FALSE;
+		}
+	}
+
 	if (request->deep_count) {
 		if (has_problem (directory, file, lacks_deep_count)) {
 			return FALSE;
@@ -1943,6 +2005,7 @@ call_ready_callbacks (NautilusDirectory *directory)
 	GList *node, *next;
 	ReadyCallback *callback;
 
+	callback = NULL;
 	called_any = FALSE;
 	while (1) {
 		/* Check if any callbacks are satisifed and call them if they are. */
@@ -2015,6 +2078,69 @@ mark_all_files_unconfirmed (NautilusDirectory *directory)
 	}
 }
 
+static gboolean
+should_display_file_name (const char *name,
+			  GnomeVFSDirectoryFilterOptions options)
+{
+	/* Note that the name is URI-encoded, but this should not
+	 * affect the . or the ~.
+	 */
+
+	if ((options & GNOME_VFS_DIRECTORY_FILTER_NODOTFILES) != 0
+	    && nautilus_file_name_matches_hidden_pattern (name)) {
+		return FALSE;
+	}
+
+	if ((options & GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES) != 0
+	    && nautilus_file_name_matches_backup_pattern (name)) {
+		return FALSE;
+	}
+	
+	/* Note that we don't bother to check for "." or ".." here, because
+	 * this function is used only for search results, which never include
+	 * those special files. If we later use this function more generally,
+	 * we might have to change this.
+	 */
+	return TRUE;
+}
+
+/* Filter search results based on user preferences. This must be done
+ * differently than filtering other files because the search results
+ * are encoded: the entire file path is encoded and stored as the file
+ * name.
+ */
+static gboolean
+filter_search_uri (const GnomeVFSFileInfo *info, gpointer data)
+{
+	GnomeVFSDirectoryFilterOptions options;
+	char *real_file_uri;
+	gboolean result;
+
+	options = GPOINTER_TO_INT (data);
+	
+	real_file_uri = nautilus_get_target_uri_from_search_result_name (info->name);
+	result = should_display_file_name (g_basename (real_file_uri), options);	
+	g_free (real_file_uri);
+
+	return result;
+}
+
+static GnomeVFSDirectoryFilter *
+get_file_count_filter (NautilusDirectory *directory)
+{
+	if (nautilus_is_search_uri (directory->details->uri)) {
+		return gnome_vfs_directory_filter_new_custom
+			(filter_search_uri,
+			 GNOME_VFS_DIRECTORY_FILTER_NEEDS_NAME,
+			 GINT_TO_POINTER (get_filter_options_for_directory_count ()));
+	}
+
+	return gnome_vfs_directory_filter_new
+		(GNOME_VFS_DIRECTORY_FILTER_NONE,
+		 get_filter_options_for_directory_count (),
+		 NULL);
+}
+
 /* Start monitoring the file list if it isn't already. */
 static void
 start_monitoring_file_list (NautilusDirectory *directory)
@@ -2039,15 +2165,11 @@ start_monitoring_file_list (NautilusDirectory *directory)
 	g_assert (directory->details->uri != NULL);
 	directory->details->directory_load_list_last_handled
 		= GNOME_VFS_DIRECTORY_LIST_POSITION_NONE;
-	directory->details->load_file_count = 0;
         directory->details->load_directory_file =
 		nautilus_directory_get_corresponding_file (directory);
 	directory->details->load_directory_file->details->loading_directory = TRUE;
 	directory->details->load_file_count = 0;
-	directory->details->load_file_count_filter = gnome_vfs_directory_filter_new
-		(GNOME_VFS_DIRECTORY_FILTER_NONE,
-		 get_filter_options_for_directory_count (),
-		 NULL);
+	directory->details->load_file_count_filter = get_file_count_filter (directory);
 	directory->details->load_mime_list_hash = istr_set_new ();
 #ifdef DEBUG_LOAD_DIRECTORY
 	g_message ("load_directory called to monitor file list of %s", directory->details->uri);
@@ -2113,11 +2235,9 @@ nautilus_directory_invalidate_counts (NautilusDirectory *directory)
 			mime_list_cancel (parent_directory);
 		}
 
-		file->details->got_directory_count = FALSE;
-		file->details->directory_count_failed = FALSE;
+		file->details->directory_count_is_up_to_date = FALSE;
 		file->details->deep_counts_status = NAUTILUS_REQUEST_NOT_STARTED;
-		file->details->got_mime_list = FALSE;
-		file->details->mime_list_failed = FALSE;
+		file->details->mime_list_is_up_to_date = FALSE;
 
 		nautilus_file_unref (file);
 
@@ -2125,9 +2245,32 @@ nautilus_directory_invalidate_counts (NautilusDirectory *directory)
 	}
 }
 
-void
-nautilus_directory_force_reload (NautilusDirectory *directory)
+static void
+nautilus_directory_invalidate_file_attributes (NautilusDirectory *directory,
+					       GList             *file_attributes)
 {
+	GList *node;
+
+	cancel_loading_attributes (directory, file_attributes);
+
+	for (node = directory->details->file_list; node != NULL; node = node->next) {
+		nautilus_file_invalidate_attributes_internal (NAUTILUS_FILE (node->data),
+							      file_attributes);
+	}
+
+	if (directory->details->as_file != NULL) {
+		nautilus_file_invalidate_attributes_internal (directory->details->as_file,
+							      file_attributes);
+	}
+}
+
+void
+nautilus_directory_force_reload (NautilusDirectory *directory,
+				 GList             *file_attributes)
+{
+	/* invalidate attributes that are getting reloaded for all files */
+	nautilus_directory_invalidate_file_attributes (directory, file_attributes);
+
 	/* Start a new directory load. */
 	file_list_cancel (directory);
 	directory->details->directory_loaded = FALSE;
@@ -2138,6 +2281,24 @@ nautilus_directory_force_reload (NautilusDirectory *directory)
 	nautilus_directory_async_state_changed (directory);
 }
 
+static gboolean
+monitor_includes_file (const Monitor *monitor,
+		       NautilusFile *file)
+{
+	if (monitor->file == file) {
+		return TRUE;
+	}
+	if (monitor->file != NULL) {
+		return FALSE;
+	}
+	if (file == file->details->directory->details->as_file) {
+		return FALSE;
+	}
+	return nautilus_file_should_show (file,
+					  monitor->monitor_hidden_files,
+					  monitor->monitor_backup_files);
+}
+
 static gboolean
 is_needy (NautilusFile *file,
 	  FileCheck check_missing,
@@ -2172,11 +2333,7 @@ is_needy (NautilusFile *file,
 	     node != NULL; node = node->next) {
 		monitor = node->data;
 		if ((* check_wanted) (&monitor->request)) {
-			if (monitor->file == file) {
-				return TRUE;
-			}
-			if (monitor->file == NULL
-			    && file != directory->details->as_file) {
+			if (monitor_includes_file (monitor, file)) {
 				return TRUE;
 			}
 		}
@@ -2234,7 +2391,7 @@ select_needy_file (NautilusDirectory *directory,
 			for (node_2 = directory->details->monitor_list;
 			     node_2 != NULL; node_2 = node_2->next) {
 				monitor = node_2->data;
-				if ((monitor->file == NULL || monitor->file == file)
+				if (monitor_includes_file (monitor, file)
 				    && (* check_wanted) (&monitor->request)) {
 					break;
 				}
@@ -2545,6 +2702,8 @@ mime_list_callback (GnomeVFSAsyncHandle *handle,
 		return;
 	}
 
+	file->details->mime_list_is_up_to_date = TRUE;
+
 	/* Record either a failure or success. */
 	nautilus_g_list_free_deep (file->details->mime_list);
 	if (result != GNOME_VFS_ERROR_EOF) {
@@ -2673,22 +2832,34 @@ top_left_read_callback (GnomeVFSResult result,
 			gpointer callback_data)
 {
 	NautilusDirectory *directory;
+	NautilusFile *changed_file;
 
 	directory = NAUTILUS_DIRECTORY (callback_data);
 
 	directory->details->top_left_read_state->handle = NULL;
 
+	directory->details->top_left_read_state->file->details->top_left_text_is_up_to_date = TRUE;
+
+	changed_file = NULL;
 	if (result == GNOME_VFS_OK) {
 		g_free (directory->details->top_left_read_state->file->details->top_left_text);
 		directory->details->top_left_read_state->file->details->top_left_text =
 			nautilus_extract_top_left_text (file_contents, bytes_read);
 		
-		nautilus_file_changed (directory->details->top_left_read_state->file);
+		directory->details->top_left_read_state->file->details->got_top_left_text = TRUE;
+
+		changed_file = directory->details->top_left_read_state->file;
+		nautilus_file_ref (changed_file);
 		
 		g_free (file_contents);
 	}
 	
 	top_left_read_done (directory);
+
+	if (changed_file != NULL) {
+		nautilus_file_changed (changed_file);
+		nautilus_file_unref (changed_file);
+	}
 }
 
 static gboolean
@@ -2769,15 +2940,37 @@ get_info_callback (GnomeVFSAsyncHandle *handle,
 	
 	directory->details->get_info_file = NULL;
 	directory->details->get_info_in_progress = NULL;
-	
+
+	/* ref here because we might be removing the last ref when we
+	 * mark the file gone below, but we need to keep a ref at
+	 * least long enough to send the change notification. 
+	 */
+	nautilus_file_ref (get_info_file);
+
 	result = results->data;
+
 	if (result->result != GNOME_VFS_OK) {
+		get_info_file->details->file_info_is_up_to_date = TRUE;
+		if (get_info_file->details->info != NULL) {
+			gnome_vfs_file_info_unref (get_info_file->details->info);
+			get_info_file->details->info = NULL;
+		}
 		get_info_file->details->get_info_failed = TRUE;
 		get_info_file->details->get_info_error = result->result;
+		if (result->result == GNOME_VFS_ERROR_NOT_FOUND) {
+			/* mark file as gone */
+
+			get_info_file->details->is_gone = TRUE;
+			if (get_info_file != directory->details->as_file) {
+				nautilus_directory_remove_file (directory, get_info_file);
+			}
+		}
 	} else {
 		nautilus_file_update_info (get_info_file, result->file_info);
 	}
+
 	nautilus_file_changed (get_info_file);
+	nautilus_file_unref (get_info_file);
 
 	async_job_end (directory, "file info");
 	nautilus_directory_async_state_changed (directory);
@@ -2820,6 +3013,7 @@ file_info_start (NautilusDirectory *directory)
 		g_free (uri);
 		
 		if (vfs_uri == NULL) {
+			file->details->file_info_is_up_to_date = TRUE;
 			file->details->get_info_failed = TRUE;
 			file->details->get_info_error = GNOME_VFS_ERROR_INVALID_URI;
 			nautilus_file_changed (file);
@@ -2849,6 +3043,8 @@ activation_uri_done (NautilusDirectory *directory,
 		     NautilusFile *file,
 		     const char *uri)
 {
+	file->details->activation_uri_is_up_to_date = TRUE;
+
 	file->details->got_activation_uri = TRUE;
 	g_free (file->details->activation_uri);
 	file->details->activation_uri = g_strdup (uri);
@@ -2864,7 +3060,6 @@ activation_uri_read_done (NautilusDirectory *directory,
 	NautilusFile *file;
 
 	file = directory->details->activation_uri_read_state->file;
-
 	g_free (directory->details->activation_uri_read_state);
 	directory->details->activation_uri_read_state = NULL;
 
@@ -3077,3 +3272,128 @@ nautilus_directory_cancel (NautilusDirectory *directory)
 	/* Check if any directories should wake up. */
 	async_job_wake_up ();
 }
+
+
+static void
+cancel_directory_count_for_file (NautilusDirectory *directory,
+				 NautilusFile      *file)
+{
+	if (directory->details->count_file == file) {
+		directory_count_cancel (directory);
+	}
+}
+
+
+static void
+cancel_deep_counts_for_file (NautilusDirectory *directory,
+			     NautilusFile      *file)
+{
+	if (directory->details->deep_count_file == file) {
+		deep_count_cancel (directory);
+	}
+}
+
+
+static void
+cancel_mime_list_for_file (NautilusDirectory *directory,
+			   NautilusFile      *file)
+{
+	if (directory->details->mime_list_file == file) {
+		mime_list_cancel (directory);
+	}
+}
+
+static void
+cancel_top_left_text_for_file (NautilusDirectory *directory,
+			       NautilusFile      *file)
+{
+	if (directory->details->top_left_read_state != NULL &&
+	    directory->details->top_left_read_state->file == file) {
+		top_left_cancel (directory);
+	}
+}
+
+static void
+cancel_file_info_for_file (NautilusDirectory *directory,
+			   NautilusFile      *file)
+{
+	if (directory->details->get_info_file == file) {
+		file_info_cancel (directory);
+	}
+}
+
+static void
+cancel_activation_uri_for_file (NautilusDirectory *directory,
+				NautilusFile      *file)
+{
+	if (directory->details->activation_uri_read_state != NULL &&
+	    directory->details->activation_uri_read_state->file == file) {
+		activation_uri_cancel (directory);
+	}
+}
+
+
+static void
+cancel_loading_attributes (NautilusDirectory *directory,
+			   GList *file_attributes)
+{
+	Request request;
+	
+	nautilus_directory_set_up_request (&request,
+					   file_attributes);
+
+	if (request.directory_count) {
+		directory_count_cancel (directory);
+	}
+	if (request.deep_count) {
+		deep_count_cancel (directory);
+	}
+	if (request.mime_list) {
+		mime_list_cancel (directory);
+	}
+	if (request.top_left_text) {
+		top_left_cancel (directory);
+	}
+	if (request.file_info) {
+		file_info_cancel (directory);
+	}
+	if (request.activation_uri) {
+		file_info_cancel (directory);
+	}
+	
+	/* FIXME bugzilla.eazel.com 5064: implement cancelling metadata when we
+	   implement invalidating metadata */
+}
+
+void
+nautilus_directory_cancel_loading_file_attributes (NautilusDirectory *directory,
+						   NautilusFile      *file,
+						   GList             *file_attributes)
+{
+	Request request;
+	
+	nautilus_directory_set_up_request (&request,
+					   file_attributes);
+
+	if (request.directory_count) {
+		cancel_directory_count_for_file (directory, file);
+	}
+	if (request.deep_count) {
+		cancel_deep_counts_for_file (directory, file);
+	}
+	if (request.mime_list) {
+		cancel_mime_list_for_file (directory, file);
+	}
+	if (request.top_left_text) {
+		cancel_top_left_text_for_file (directory, file);
+	}
+	if (request.file_info) {
+		cancel_file_info_for_file (directory, file);
+	}
+	if (request.activation_uri) {
+		cancel_activation_uri_for_file (directory, file);
+	}
+
+	/* FIXME bugzilla.eazel.com 5064: implement cancelling metadata when we
+	   implement invalidating metadata */
+}
