@@ -52,18 +52,71 @@ static void saved_settings_changed_callback (NautilusFile       *file,
                                          
 static void nautilus_file_background_receive_root_window_changes (NautilusBackground *background);
 
+static void nautilus_file_update_desktop_pixmaps (NautilusBackground *background);
+
+static void nautilus_file_background_write_desktop_settings (char *color,
+							     char *image,
+							     NautilusBackgroundImagePlacement placement,
+							     gboolean combine);
+
+static gboolean nautilus_file_background_matches_default_settings (
+			const char* color, const char* default_color,
+			const char* image, const char* default_image,
+			NautilusBackgroundImagePlacement placement, NautilusBackgroundImagePlacement default_placement,
+			gboolean combine, gboolean default_combine);
+
+static void
+desktop_background_realized (NautilusIconContainer *icon_container, void *disconnect_signal)
+{
+	NautilusBackground *background;
+
+        if ((gboolean) GPOINTER_TO_INT (disconnect_signal)) {
+		gtk_signal_disconnect_by_func (GTK_OBJECT (icon_container),
+					       GTK_SIGNAL_FUNC (desktop_background_realized),
+					       disconnect_signal);
+	}
+
+	background = nautilus_get_widget_background (GTK_WIDGET (icon_container));
+
+	gtk_object_set_data (GTK_OBJECT (background), "icon_container", (gpointer) icon_container);
+
+	nautilus_file_update_desktop_pixmaps (background);
+}
+
 static const char *default_theme_source = "directory";
 static const char *desktop_theme_source = "desktop";
-                       
+
 void
-static nautilus_file_background_set_desktop (NautilusBackground *background)
+nautilus_connect_desktop_background_to_file_metadata (NautilusIconContainer *icon_container,
+                                                      NautilusFile *file)
 {
+	NautilusBackground *background;
+
+	background = nautilus_get_widget_background (GTK_WIDGET (icon_container));
+
 	gtk_object_set_data (GTK_OBJECT (background), "theme_source", (gpointer) desktop_theme_source);
+
+	/* Strictly speaking, we don't need to know about metadata changes, since
+	 * the desktop setting aren't stored there. But, hooking up to metadata
+	 * changes is actually a small part of what this fn does, and we do need
+	 * the other stuff (hooked up to background & theme changes, initialize
+	 * the background). Being notified of metadata changes on the file is a
+	 * waste, but won't hurt, so I don't think it's worth refactoring the fn
+	 * at this point.
+	 */
+	nautilus_connect_background_to_file_metadata (GTK_WIDGET (icon_container), file);
+
+	if (GTK_WIDGET_REALIZED (icon_container)) {
+		desktop_background_realized (icon_container, GINT_TO_POINTER (FALSE));
+	} else {
+		gtk_signal_connect (GTK_OBJECT (icon_container), "realize", GTK_SIGNAL_FUNC (desktop_background_realized), GINT_TO_POINTER (TRUE));
+	}
+
 	nautilus_file_background_receive_root_window_changes (background); 
 }
 
 static gboolean
-nautilus_file_background_is_desktop (NautilusBackground *background)
+nautilus_background_is_desktop (NautilusBackground *background)
 {
 	/* == works because we're carful to always use the same string.
 	 */
@@ -79,9 +132,15 @@ static const char *nautilus_file_background_peek_theme_source (NautilusBackgroun
 	return theme_source != NULL ? theme_source : default_theme_source;
 }
 
+static GdkWindow *
+nautilus_background_get_desktop_background_window (NautilusBackground *background)
+{
+	return GTK_LAYOUT (gtk_object_get_data (GTK_OBJECT (background), "icon_container"))->bin_window;
+}
+
 /* utility routine to handle mapping local image files in themes to a uri */
 static char*
-theme_image_path_to_uri (char *image_file)
+theme_image_path_to_uri (char *image_file, const char *theme_name)
 {
 	char *image_path;
 	char *image_uri;
@@ -89,7 +148,7 @@ theme_image_path_to_uri (char *image_file)
 	if (image_file != NULL && !nautilus_istr_has_prefix (image_file, "file://")) {
 		
 		if (nautilus_str_has_prefix (image_file, "./")) {
-			image_path = nautilus_theme_get_image_path (image_file + 2);
+			image_path = nautilus_theme_get_image_path_from_theme (image_file + 2, theme_name);
 		} else {
 			image_path = g_strdup_printf ("%s/%s", NAUTILUS_DATADIR, image_file);
 		}
@@ -112,6 +171,38 @@ theme_image_path_to_uri (char *image_file)
  * None of our themes currently specify this for the desktop.
  */
  
+static void
+nautilus_file_background_get_default_settings_for_theme (const char* theme_name,
+							  const char* theme_source,
+							  char **color,
+							  char **image,
+							  NautilusBackgroundImagePlacement *placement,
+							  gboolean *combine)
+{
+	char *combine_str;
+	char *image_local_path;
+
+	if (placement != NULL) {
+		*placement = NAUTILUS_BACKGROUND_TILED;
+	}
+
+	if (color != NULL) {
+		*color = nautilus_theme_get_theme_data_from_theme (theme_source, NAUTILUS_METADATA_KEY_LOCATION_BACKGROUND_COLOR, theme_name);
+	}
+
+	if (image != NULL) {
+		image_local_path = nautilus_theme_get_theme_data_from_theme (theme_source, NAUTILUS_METADATA_KEY_LOCATION_BACKGROUND_IMAGE, theme_name);
+		*image = theme_image_path_to_uri (image_local_path, theme_name);
+		g_free (image_local_path);
+	}
+
+	if (combine != NULL) {
+		combine_str = nautilus_theme_get_theme_data_from_theme (theme_source, "COMBINE", theme_name);
+		*combine = combine_str != NULL;
+		g_free (combine_str);
+	}
+}
+
 static void
 nautilus_file_background_get_default_settings (const char* theme_source,
                                                char **color,
@@ -119,28 +210,19 @@ nautilus_file_background_get_default_settings (const char* theme_source,
                                                NautilusBackgroundImagePlacement *placement,
                                                gboolean *combine)
 {
-	char *combine_str;
-	char *image_local_path;
-
-	*placement = NAUTILUS_BACKGROUND_TILED;
-	
-	*color = nautilus_theme_get_theme_data (theme_source, NAUTILUS_METADATA_KEY_LOCATION_BACKGROUND_COLOR);
-
-	image_local_path = nautilus_theme_get_theme_data (theme_source, NAUTILUS_METADATA_KEY_LOCATION_BACKGROUND_IMAGE);
-	*image = theme_image_path_to_uri (image_local_path);
-	g_free (image_local_path);
-
-	combine_str = nautilus_theme_get_theme_data (theme_source, "COMBINE");
-	*combine = combine_str != NULL;
-	g_free (combine_str);	
+	char *theme_name;
+	theme_name = nautilus_theme_get_theme ();
+	nautilus_file_background_get_default_settings_for_theme
+		(theme_name, theme_source, color, image, placement, combine);
+	g_free (theme_name);
 }
 
 static gboolean
-nautilus_gnome_config_string_match_no_case (const char *path, const char *test_value)
+nautilus_gnome_config_string_match_no_case_with_default (const char *path, const char *test_value, gboolean *was_default)
 {
 	char *value;
 	gboolean result;
-	value = gnome_config_get_string (path);
+	value = gnome_config_get_string_with_default (path, was_default);
 	result = !nautilus_strcasecmp (value, test_value);
 	g_free (value);
 	return result;
@@ -163,8 +245,9 @@ nautilus_file_background_read_desktop_settings (char **color,
 {
 	int	 image_alignment;
 	char*	 image_local_path;
-	char*	 default_image_file;
-	gboolean no_alignment;
+	char*	 default_image_uri;
+	gboolean no_alignment_set;
+	gboolean no_image_set;
 	NautilusBackgroundImagePlacement default_placement;
 	
 	char	*end_color;
@@ -172,25 +255,42 @@ nautilus_file_background_read_desktop_settings (char **color,
 	char	*default_color;
 	gboolean use_gradient;
 	gboolean is_horizontal;
-	gboolean no_start_color;
-	gboolean no_end_color;
+	gboolean no_start_color_set;
+	gboolean no_end_color_set;
+	gboolean no_gradient_set;
+	gboolean no_gradient_orientation_set;
 
-	nautilus_file_background_get_default_settings (desktop_theme_source, &default_color, &default_image_file, &default_placement, combine);
+	char *theme_name;
+	char *cur_theme_name;
+	gboolean no_theme_name_set;
+	gboolean switch_to_cur_theme_default;
+
+	theme_name = gnome_config_get_string_with_default ("/Background/Default/nautilus_theme", &no_theme_name_set);
+
+	if (no_theme_name_set) {
+		nautilus_file_background_get_default_settings
+			(desktop_theme_source, &default_color, &default_image_uri, &default_placement, combine);
+	} else {
+		nautilus_file_background_get_default_settings_for_theme
+			(theme_name, desktop_theme_source, &default_color, &default_image_uri, &default_placement, combine);
+	}
 	/* note - value of combine comes from the theme, not currently setable in gnome_config */
 
-	image_local_path = gnome_config_get_string ("/Background/Default/wallpaper=none");
-	image_alignment  = gnome_config_get_int_with_default ("/Background/Default/wallpaperAlign", &no_alignment);
+	image_local_path = gnome_config_get_string_with_default ("/Background/Default/wallpaper", &no_image_set);
 
-	if (nautilus_strcasecmp (image_local_path, "none") != 0) {
+	if (no_image_set) {
+		*image = g_strdup (default_image_uri);
+	} else if (nautilus_strcasecmp (image_local_path, "none") != 0) {
 		*image = gnome_vfs_get_uri_from_local_path (image_local_path);
 	} else {
 		*image = NULL;
 	}
 	
 	g_free(image_local_path);
-	g_free(default_image_file);	
 
-	if (no_alignment) {
+	image_alignment  = gnome_config_get_int_with_default ("/Background/Default/wallpaperAlign", &no_alignment_set);
+
+	if (no_alignment_set) {
 		*placement = default_placement;
 	} else {
 		 switch (image_alignment) {
@@ -215,24 +315,66 @@ nautilus_file_background_read_desktop_settings (char **color,
 		 }
 	}
 
-	end_color     = gnome_config_get_string_with_default ("/Background/Default/color2", &no_end_color);
-	start_color   = gnome_config_get_string_with_default ("/Background/Default/color1", &no_start_color);
-	use_gradient  = !nautilus_gnome_config_string_match_no_case ("/Background/Default/simple=solid", "solid");
-	is_horizontal = !nautilus_gnome_config_string_match_no_case ("/Background/Default/gradient=vertical", "vertical");
+	end_color     = gnome_config_get_string_with_default ("/Background/Default/color2", &no_end_color_set);
+	start_color   = gnome_config_get_string_with_default ("/Background/Default/color1", &no_start_color_set);
+	use_gradient  = !nautilus_gnome_config_string_match_no_case_with_default ("/Background/Default/simple", "solid", &no_gradient_set);
+	is_horizontal = !nautilus_gnome_config_string_match_no_case_with_default ("/Background/Default/gradient", "vertical", &no_gradient_orientation_set);
 
-	if (use_gradient) {
-		if (no_start_color || no_end_color) {
+	if (no_gradient_set || no_gradient_orientation_set || no_start_color_set) {
+		*color = g_strdup (default_color);
+	} else if (use_gradient) {
+		if (no_end_color_set) {
 			*color = g_strdup (default_color);
 		} else {
 			*color = nautilus_gradient_new (start_color, end_color , is_horizontal);
 		}
 	} else {
-		*color = g_strdup (no_start_color ? default_color : start_color);
+		*color = g_strdup (start_color);
 	}
 
 	g_free(start_color);
 	g_free(end_color);
+
+	/* Since we share our settings with the background-capplet, we can't
+	 * write the default values specially (e.g. by removing them entirely).
+	 * 
+	 * The best we can do is check to see if the settings match the default values
+	 * for the theme name that's stored with them. If so, we assume it means use
+	 * the default - i.e. the default from the current theme.
+	 * 
+	 *  - there must be a theme name stored with the settings
+	 *  - if the stored theme name matches the current theme, then
+	 *    we don't need to do anything since we're already using
+	 *    the current theme's default values. 
+	 * 
+	 */
+	cur_theme_name = nautilus_theme_get_theme ();
+
+	switch_to_cur_theme_default = !no_theme_name_set &&
+				      (nautilus_strcmp (theme_name, cur_theme_name) != 0) &&
+				      nautilus_file_background_matches_default_settings
+					(*color, default_color,
+					 *image, default_image_uri,
+					 *placement, default_placement,
+					 *combine, *combine);
+
+	if (switch_to_cur_theme_default) {
+		g_free (*color);
+		g_free (*image);
+		nautilus_file_background_get_default_settings (desktop_theme_source, color, image, placement, combine);
+	}
+
+	if (switch_to_cur_theme_default || no_theme_name_set) {
+		/* Writing out the actual settings for the current theme so that the
+		 * background capplet will show the right settings.
+		 */
+		nautilus_file_background_write_desktop_settings (*color, *image, *placement, *combine);
+	}
+
+	g_free (theme_name);	
+	g_free (cur_theme_name);	
 	g_free(default_color);
+	g_free(default_image_uri);
 }
 
 static void
@@ -241,6 +383,7 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, Nauti
 	char *end_color;
 	char *start_color;
 	char *image_local_path;
+	char *theme_name;
 
 	int wallpaper_align;
 
@@ -262,6 +405,14 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, Nauti
 
 		gnome_config_set_string ("/Background/Default/simple", nautilus_gradient_is_gradient (color) ? "gradient" : "solid");
 		gnome_config_set_string ("/Background/Default/gradient", nautilus_gradient_is_horizontal (color) ? "horizontal" : "vertical");
+	} else {
+		/* We set it to white here because that's how backgrounds with a NULL color
+		 * are drawn by Nautilus - due to usage of nautilus_gdk_color_parse_with_white_default.
+		 */
+		gnome_config_set_string ("/Background/Default/color1", "rgb:FFFF/FFFF/FFFF");		
+		gnome_config_set_string ("/Background/Default/color2", "rgb:FFFF/FFFF/FFFF");		
+		gnome_config_set_string ("/Background/Default/simple", "solid");
+		gnome_config_set_string ("/Background/Default/gradient", "vertical");
 	}
 
 	if (image != NULL) {
@@ -314,6 +465,10 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, Nauti
 		gnome_config_set_string ("/Background/Default/wallpaper", "none");
 	}
 
+	theme_name = nautilus_theme_get_theme ();
+	gnome_config_set_string ("/Background/Default/nautilus_theme", theme_name);
+	g_free (theme_name);
+
 	gnome_config_sync ();
 }
 
@@ -339,6 +494,13 @@ call_settings_changed (NautilusBackground *background)
 	return FALSE;
 }
 
+/* We don't want to respond to our own changes to the root pixmap.
+ * Since there's no way to determine the origin of the x-event (or mark it)
+ * we use this variable determine if we think the next PropertyNotify is
+ * due to us.
+ */
+static int set_root_pixmap_count = 0;
+
 static GdkFilterReturn
 nautilus_file_background_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 {
@@ -348,6 +510,14 @@ nautilus_file_background_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, g
 	xevent = (XEvent *) gdk_xevent;
 
 	if (xevent->type == PropertyNotify && xevent->xproperty.atom == gdk_atom_intern("ESETROOT_PMAP_ID", TRUE)) {
+
+		/* If we caused it, ignore it.
+		 */
+		if (set_root_pixmap_count > 0) {
+			--set_root_pixmap_count;
+			return GDK_FILTER_CONTINUE;
+		}
+		
 	    	background = NAUTILUS_BACKGROUND (data);
 	    	/* FIXME bugzilla.eazel.com 2220:
 	    	 * We'd like to call saved_settings_changed_callback right here, directly.
@@ -356,10 +526,6 @@ nautilus_file_background_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, g
 	    	 * i.e. after we get this event. How long afterwards is not knowable - we
 	    	 * guess half a second. Fixing this requires changing the capplet.
 	    	 */
-
-	    	/* FIXME bugzilla.eazel.com 3038:
-	    	 * We don't want to respond to an event we originated.
-	    	 */
 	    	gtk_timeout_add (500, (GtkFunction) (call_settings_changed), background);
 	}
 
@@ -403,17 +569,22 @@ nautilus_file_background_receive_root_window_changes (NautilusBackground *backgr
 static GdkPixmap *
 make_root_pixmap (gint width, gint height)
 {
+	Display *display;
 	Pixmap result;
 
 	gdk_flush ();
 
-	XSetCloseDownMode (gdk_display, RetainPermanent);
+	display = XOpenDisplay (gdk_display_name);
 
-	result = XCreatePixmap (gdk_display,
-				DefaultRootWindow (gdk_display),
+	XSetCloseDownMode (display, RetainPermanent);
+
+	result = XCreatePixmap (display,
+				DefaultRootWindow (display),
 				width, height,
 				DefaultDepthOfScreen (DefaultScreenOfDisplay (GDK_DISPLAY())));
 
+	XCloseDisplay (display);
+
 	return gdk_pixmap_foreign_new (result);
 }
 
@@ -445,27 +616,36 @@ dispose_root_pixmap (GdkPixmap *pixmap)
 static void 
 set_root_pixmap (GdkPixmap *pixmap)
 {
-	GdkAtom type;
-	gulong nitems, bytes_after;
-	gint format;
+	int     result;
+	gint    format;
+	gulong  nitems;
+	gulong  bytes_after;
 	guchar *data_esetroot;
-	Pixmap pixmap_id = GDK_WINDOW_XWINDOW (pixmap);
+	Pixmap  pixmap_id;
+	GdkAtom type;
+
+	data_esetroot = NULL;
 
 	XGrabServer (GDK_DISPLAY());
 
-	XGetWindowProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
-			    gdk_atom_intern("ESETROOT_PMAP_ID", FALSE),
-			    0L, 1L, False, XA_PIXMAP,
-			    &type, &format, &nitems, &bytes_after,
-			    &data_esetroot);
-
-	if (type == XA_PIXMAP) {
-		if (format == 32 && nitems == 4)
-			XKillClient(GDK_DISPLAY(), *((Pixmap*)data_esetroot));
+	result = XGetWindowProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+				     gdk_atom_intern("ESETROOT_PMAP_ID", FALSE),
+				     0L, 1L, False, XA_PIXMAP,
+				     &type, &format, &nitems, &bytes_after,
+				     &data_esetroot);
 
+	if (result == Success && type == XA_PIXMAP && format == 32 && nitems == 1) {
+		XKillClient(GDK_DISPLAY(), *(Pixmap*)data_esetroot);
+	}
+	
+	if (data_esetroot != NULL) {
 		XFree (data_esetroot);
 	}
 
+	++set_root_pixmap_count;
+	
+	pixmap_id = GDK_WINDOW_XWINDOW (pixmap);
+
 	XChangeProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
 			 gdk_atom_intern("ESETROOT_PMAP_ID", FALSE), XA_PIXMAP,
 			 32, PropModeReplace,
@@ -482,43 +662,46 @@ set_root_pixmap (GdkPixmap *pixmap)
 	
 	XFlush(GDK_DISPLAY());
 }
-
+	
 static void
 image_loading_done_callback (NautilusBackground *background, gboolean successful_load, void *disconnect_signal)
 {
-	GdkGC        *gc;
-	GdkPixmap    *bg_pixmap;
 	int	      width;
 	int	      height;
-
-        g_assert (NAUTILUS_IS_BACKGROUND (background));
-
+	GdkGC        *gc;
+	GdkPixmap    *pixmap;
+	
         if ((gboolean) GPOINTER_TO_INT (disconnect_signal)) {
 		gtk_signal_disconnect_by_func (GTK_OBJECT (background),
 					       GTK_SIGNAL_FUNC (image_loading_done_callback),
 					       disconnect_signal);
 	}
 
-	/* need to update the root view whether loading succeeded or not
-	 */
-	
 	width  = gdk_screen_width ();
 	height = gdk_screen_height ();
-	
-	bg_pixmap = make_root_pixmap (width, height);
-	gc = gdk_gc_new (bg_pixmap);
 
-	nautilus_background_draw_to_drawable (background, bg_pixmap, gc, 0, 0, width, height, width, height);
-			    
-	set_root_pixmap (bg_pixmap);
-	
-	dispose_root_pixmap (bg_pixmap);
+	pixmap = make_root_pixmap (width, height);
+	gc = gdk_gc_new (pixmap);
+	nautilus_background_draw_to_drawable (background, pixmap, gc, 0, 0, width, height, width, height);
 	gdk_gc_unref (gc);
+
+	set_root_pixmap (pixmap);
+
+	gdk_window_set_back_pixmap (nautilus_background_get_desktop_background_window (background), pixmap, FALSE);
+
+	/* We'd like to simply unref pixmap here, but due to a bug in gdk's handling of
+	 * foreign pixmaps, we can't - it would free the X resource.
+	 *
+	 * gdk_window_set_back_pixmap does not need the gdk pixmap object to stick around.
+	 * It simply uses X resource inside it. dispose_root_pixmap free's the gdk object
+	 * and not the X resource.
+	 */
+	dispose_root_pixmap (pixmap);
 }
 
 static void
-nautilus_file_update_root_window (NautilusBackground *background)
-{
+nautilus_file_update_desktop_pixmaps (NautilusBackground *background)
+{	
 	if (nautilus_background_is_loaded (background)) {
 		image_loading_done_callback (background, TRUE, GINT_TO_POINTER (FALSE));
 	} else {
@@ -529,6 +712,27 @@ nautilus_file_update_root_window (NautilusBackground *background)
 	}
 }
 
+static gboolean
+nautilus_file_background_matches_default_settings (
+			const char* color, const char* default_color,
+			const char* image, const char* default_image,
+			NautilusBackgroundImagePlacement placement, NautilusBackgroundImagePlacement default_placement,
+			gboolean combine, gboolean default_combine)
+{
+	gboolean match_color;
+	gboolean match_image;
+
+	/* A NULL default color or image is not considered when determining a match.
+	 */
+	
+	match_color = (default_color == NULL) || nautilus_strcmp (color, default_color) == 0;
+	
+	match_image = (default_image == NULL) || ((nautilus_strcmp (image, default_image) == 0) &&
+						  (placement == default_placement) &&
+						  (combine == default_combine));
+	return match_color && match_image;
+}
+
 /* return true if the background is not in the default state */
 gboolean
 nautilus_file_background_is_set (NautilusBackground *background)
@@ -538,7 +742,7 @@ nautilus_file_background_is_set (NautilusBackground *background)
 	char *default_color;
 	char *default_image;
 	
-	gboolean is_set;
+	gboolean matches;
 	gboolean combine;
 	gboolean default_combine;
 	
@@ -553,18 +757,18 @@ nautilus_file_background_is_set (NautilusBackground *background)
 	nautilus_file_background_get_default_settings (
 		nautilus_file_background_peek_theme_source (background),
 		&default_color, &default_image, &default_placement, &default_combine);
-
-	is_set = (default_color != NULL && nautilus_strcmp (color, default_color) != 0) ||
-		 (default_image != NULL && nautilus_strcmp (image, default_image) != 0) ||
-		 (placement != default_placement) ||
-		 (combine != default_combine);
-
+		 			    
+	matches = !nautilus_file_background_matches_default_settings (color, default_color,
+								      image, default_image,
+								      placement, default_placement,
+								      combine, default_combine);
+	
 	g_free (color);
 	g_free (image);
 	g_free (default_color);
 	g_free (default_image);
 	
-	return is_set;
+	return !matches;
 }
 
 /* handle the background changed signal */
@@ -572,18 +776,18 @@ static void
 background_changed_callback (NautilusBackground *background,
                              NautilusFile       *file)
 {
-  	char *color, *image;
+  	char *color;
+  	char *image;
         
         g_assert (NAUTILUS_IS_BACKGROUND (background));
         g_assert (NAUTILUS_IS_FILE (file));
-        g_assert (gtk_object_get_data (GTK_OBJECT (background), "nautilus_background_file")
-                  == file);
+        g_assert (gtk_object_get_data (GTK_OBJECT (background), "nautilus_background_file") == file);
         
 
 	color = nautilus_background_get_color (background);
 	image = nautilus_background_get_image_uri (background);
 
-	if (nautilus_file_background_is_desktop (background)) {
+	if (nautilus_background_is_desktop (background)) {
 		nautilus_file_background_write_desktop_settings (color, image, nautilus_background_get_image_placement (background), nautilus_background_get_combine_mode (background));
 	} else {
 	        /* Block the other handler while we are writing metadata so it doesn't
@@ -612,8 +816,8 @@ background_changed_callback (NautilusBackground *background,
 	g_free (color);
 	g_free (image);
 	
-	if (nautilus_file_background_is_desktop (background)) {
-		nautilus_file_update_root_window (background);
+	if (nautilus_background_is_desktop (background)) {
+		nautilus_file_update_desktop_pixmaps (background);
 	}
 }
 
@@ -632,7 +836,7 @@ saved_settings_changed_callback (NautilusFile *file,
         g_assert (gtk_object_get_data (GTK_OBJECT (background), "nautilus_background_file")
                   == file);
 
-	if (nautilus_file_background_is_desktop (background)) {
+	if (nautilus_background_is_desktop (background)) {
 		nautilus_file_background_read_desktop_settings (&color, &image, &placement, &combine);
 	} else {
 		color = nautilus_file_get_metadata (file,
@@ -659,8 +863,8 @@ saved_settings_changed_callback (NautilusFile *file,
                                           background_changed_callback,
                                           file);
 
-		nautilus_background_set_color (background, color);     
-		nautilus_background_set_image_uri (background, image);
+	nautilus_background_set_color (background, color);     
+	nautilus_background_set_image_uri (background, image);
         nautilus_background_set_combine_mode (background, combine);
         nautilus_background_set_image_placement (background, placement);
 	
@@ -692,7 +896,7 @@ static void
 background_reset_callback (NautilusBackground *background,
                            NautilusFile       *file)
 {
-	if (nautilus_file_background_is_desktop (background)) {
+	if (nautilus_background_is_desktop (background)) {
 		nautilus_file_background_write_desktop_default_settings ();
 	} else {
 	        /* Block the other handler while we are writing metadata so it doesn't
@@ -720,13 +924,8 @@ background_reset_callback (NautilusBackground *background,
 
 	saved_settings_changed_callback (file, background);
 
-	/* We don't want the default reset handler running.
-	 * It will set color and image_uri  to NULL.
-	 */
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (background), "reset");
-
-	if (nautilus_file_background_is_desktop (background)) {
-		nautilus_file_update_root_window (background);
+	if (nautilus_background_is_desktop (background)) {
+		nautilus_file_update_desktop_pixmaps (background);
 	}
 }
 
@@ -827,22 +1026,6 @@ nautilus_connect_background_to_file_metadata (GtkWidget    *widget,
         saved_settings_changed_callback (file, background);
 }
 
-void
-nautilus_connect_desktop_background_to_file_metadata (GtkWidget *widget,
-                                                      NautilusFile *file)
-{
-	nautilus_file_background_set_desktop (nautilus_get_widget_background (widget));
-
-	/* Strictly speaking, we don't need to know about metadata changes, since
-	 * the desktop setting aren't stored there. But, hooking up to metadata
-	 * changes is actually a small part of what this fn does, and we do need
-	 * the other stuff (hooked up to background & theme changes). Being notified
-	 * of metadata changes on the file is a waste, but won't hurt, so I don't
-	 * think it's worth refactoring the fn at this point.
-	 */
-	nautilus_connect_background_to_file_metadata (widget, file);
-}
-
 void
 nautilus_connect_background_to_file_metadata_by_uri (GtkWidget *widget,
                                                      const char *uri)
