@@ -54,6 +54,7 @@ struct NautilusDirectoryDetails
 	/* The metadata. */
 	gboolean metafile_read;
 	xmlDoc *metafile;
+	GHashTable *metafile_node_hash;
 	GHashTable *metadata_changes;
 
 	/* State for reading and writing metadata. */
@@ -107,6 +108,17 @@ struct NautilusDirectoryDetails
 	GList *file_operations_in_progress; /* list of FileOperation * */
 };
 
+/* A request for information about one or more files. */
+typedef struct {
+	gboolean activation_uri;
+	gboolean deep_count;
+	gboolean directory_count;
+	gboolean file_info;
+	gboolean file_list; /* always FALSE if file != NULL */
+	gboolean metafile;
+	gboolean mime_list;
+	gboolean top_left_text;
+} Request;
 
 NautilusDirectory *nautilus_directory_get_existing                    (const char                *uri);
 
@@ -129,13 +141,15 @@ void               nautilus_directory_cancel_callback_internal        (NautilusD
 void               nautilus_directory_monitor_add_internal            (NautilusDirectory         *directory,
 								       NautilusFile              *file,
 								       gconstpointer              client,
+								       gboolean                   monitor_hidden_files,
+								       gboolean                   monitor_backup_files,
 								       GList                     *attributes);
 void               nautilus_directory_monitor_remove_internal         (NautilusDirectory         *directory,
 								       NautilusFile              *file,
 								       gconstpointer              client);
 void               nautilus_directory_get_info_for_new_files          (NautilusDirectory         *directory,
 								       GList                     *vfs_uris);
-NautilusFile	  *   nautilus_directory_get_existing_corresponding_file (NautilusDirectory         *directory);
+NautilusFile *     nautilus_directory_get_existing_corresponding_file (NautilusDirectory         *directory);
 void               nautilus_directory_invalidate_counts               (NautilusDirectory         *directory);
 gboolean           nautilus_directory_is_file_list_monitored          (NautilusDirectory         *directory);
 gboolean           nautilus_directory_is_anyone_monitoring_file_list  (NautilusDirectory         *directory);
@@ -147,7 +161,11 @@ void               nautilus_directory_stop_monitoring_file_list       (NautilusD
 void               nautilus_directory_cancel                          (NautilusDirectory         *directory);
 void               nautilus_metafile_write_start                      (NautilusDirectory         *directory);
 void               nautilus_async_destroying_file                     (NautilusFile              *file);
-void               nautilus_directory_force_reload                    (NautilusDirectory         *directory);
+void               nautilus_directory_force_reload                    (NautilusDirectory         *directory,
+								       GList                     *file_attributes);
+void               nautilus_directory_cancel_loading_file_attributes  (NautilusDirectory         *directory,
+								       NautilusFile              *file,
+								       GList                     *file_attributes);
 
 /* Calls shared between directory, file, and async. code. */
 void               nautilus_directory_emit_metadata_changed           (NautilusDirectory         *directory);
@@ -158,9 +176,13 @@ void               nautilus_directory_emit_files_changed              (NautilusD
 void               nautilus_directory_emit_change_signals_deep        (NautilusDirectory         *directory,
 								       GList                     *changed_files);
 void               nautilus_directory_emit_done_loading               (NautilusDirectory         *directory);
+void               nautilus_directory_emit_load_error                 (NautilusDirectory         *directory,
+								       GnomeVFSResult             error_result);
 NautilusDirectory *nautilus_directory_get_internal                    (const char                *uri,
 								       gboolean                   create);
-char		  *          nautilus_directory_get_name_for_self_as_new_file   (NautilusDirectory         *directory);
+char *             nautilus_directory_get_name_for_self_as_new_file   (NautilusDirectory         *directory);
+void               nautilus_directory_set_up_request                  (Request                   *request,
+								       GList                     *file_attributes);
 
 /* Interface to the file list. */
 NautilusFile *     nautilus_directory_find_file_by_name               (NautilusDirectory         *directory,
@@ -176,10 +198,8 @@ GList *            nautilus_directory_begin_file_name_change          (NautilusD
 void               nautilus_directory_end_file_name_change            (NautilusDirectory         *directory,
 								       NautilusFile              *file,
 								       GList                     *node);
-
-
-void               nautilus_directory_handle_directory_moved          (NautilusDirectory *directory,
-								       const char        *to_uri);
+void               nautilus_directory_moved                           (const char                *from_uri,
+								       const char                *to_uri);
 
 /* debugging functions */
 int                nautilus_directory_number_outstanding              (void);
