@@ -37,16 +37,15 @@
 #include "nautilus-string.h"
 #include "nautilus-trash-directory.h"
 #include "nautilus-vfs-directory.h"
-#include "nautilus-wait-until-ready.h"
 #include <ctype.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 
-
 enum {
 	FILES_ADDED,
 	FILES_CHANGED,
 	DONE_LOADING,
+	LOAD_ERROR,
 	LAST_SIGNAL
 };
 
@@ -63,20 +62,20 @@ static guint signals[LAST_SIGNAL];
          | GNOME_VFS_PERM_GROUP_ALL \
 	 | GNOME_VFS_PERM_OTHER_ALL)
 
-/* Hack for PR2 to prevent use of too many resources */
-#define NAUTILUS_DIRECTORY_FILE_LIST_HARD_LIMIT     4000
-
 static GHashTable *directories;
 
-static void               nautilus_directory_destroy          (GtkObject   *object);
-static void               nautilus_directory_initialize       (gpointer     object,
-							       gpointer     klass);
+static void               nautilus_directory_destroy          (GtkObject              *object);
+static void               nautilus_directory_initialize       (gpointer                object,
+							       gpointer                klass);
 static void               nautilus_directory_initialize_class (NautilusDirectoryClass *klass);
-static NautilusDirectory *nautilus_directory_new              (const char  *uri);
+static NautilusDirectory *nautilus_directory_new              (const char             *uri);
+static char *             real_get_name_for_self_as_new_file  (NautilusDirectory      *directory);
+static void               set_directory_uri                   (NautilusDirectory      *directory,
+							       const char             *new_uri);
 
-static char		 *real_get_name_for_self_as_new_file  (NautilusDirectory *directory);
-
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusDirectory, nautilus_directory, GTK_TYPE_OBJECT)
+NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusDirectory,
+				   nautilus_directory,
+				   GTK_TYPE_OBJECT)
 
 static void
 nautilus_directory_initialize_class (NautilusDirectoryClass *klass)
@@ -108,7 +107,14 @@ nautilus_directory_initialize_class (NautilusDirectoryClass *klass)
 				GTK_SIGNAL_OFFSET (NautilusDirectoryClass, done_loading),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
-
+	signals[LOAD_ERROR] =
+		gtk_signal_new ("load_error",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (NautilusDirectoryClass, load_error),
+				gtk_marshal_NONE__INT,
+				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+	
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	klass->get_name_for_self_as_new_file = real_get_name_for_self_as_new_file;
@@ -123,6 +129,7 @@ nautilus_directory_initialize (gpointer object, gpointer klass)
 
 	directory->details = g_new0 (NautilusDirectoryDetails, 1);
 	directory->details->file_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	directory->details->metafile_node_hash = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 void
@@ -173,19 +180,21 @@ nautilus_directory_destroy (GtkObject *object)
 		gtk_idle_remove (directory->details->dequeue_pending_idle_id);
 	}
  
+	nautilus_directory_metafile_destroy (directory);
+
 	g_free (directory->details->uri);
-	if (directory->details->private_metafile_vfs_uri != NULL) {
-		gnome_vfs_uri_unref (directory->details->private_metafile_vfs_uri);
-	}
 	if (directory->details->vfs_uri != NULL) {
-	    gnome_vfs_uri_unref (directory->details->vfs_uri);
+		gnome_vfs_uri_unref (directory->details->vfs_uri);
 	}
 	if (directory->details->public_metafile_vfs_uri != NULL) {
 		gnome_vfs_uri_unref (directory->details->public_metafile_vfs_uri);
 	}
+	if (directory->details->private_metafile_vfs_uri != NULL) {
+		gnome_vfs_uri_unref (directory->details->private_metafile_vfs_uri);
+	}
 	g_assert (directory->details->file_list == NULL);
 	g_hash_table_destroy (directory->details->file_hash);
-	nautilus_directory_metafile_destroy (directory);
+	g_hash_table_destroy (directory->details->metafile_node_hash);
 	g_assert (directory->details->directory_load_in_progress == NULL);
 	g_assert (directory->details->count_in_progress == NULL);
 	g_assert (directory->details->dequeue_pending_idle_id == 0);
@@ -406,7 +415,7 @@ real_get_name_for_self_as_new_file (NautilusDirectory *directory)
 	directory_uri = directory->details->uri;
 
 	colon = strchr (directory_uri, ':');
-	if (colon == NULL) {
+	if (colon == NULL || colon == directory_uri) {
 		name = g_strdup (directory_uri);
 	} else {
 		name = g_strndup (directory_uri, colon - directory_uri);
@@ -463,7 +472,6 @@ static NautilusDirectory *
 nautilus_directory_new (const char *uri)
 {
 	NautilusDirectory *directory;
-	GnomeVFSURI *vfs_uri;
 
 	g_assert (uri != NULL);
 
@@ -475,14 +483,7 @@ nautilus_directory_new (const char *uri)
 	gtk_object_ref (GTK_OBJECT (directory));
 	gtk_object_sink (GTK_OBJECT (directory));
 
-	directory->details->uri = g_strdup (uri);
-	directory->details->private_metafile_vfs_uri = construct_private_metafile_vfs_uri (uri);
-
-	vfs_uri = gnome_vfs_uri_new (uri);
-
-	directory->details->vfs_uri = vfs_uri;
-	directory->details->public_metafile_vfs_uri = vfs_uri == NULL ? NULL
-		: gnome_vfs_uri_append_file_name (vfs_uri, METAFILE_NAME);
+	set_directory_uri (directory, uri);
 
 	return directory;
 }
@@ -579,7 +580,7 @@ nautilus_directory_remove_file (NautilusDirectory *directory, NautilusFile *file
 		(directory->details->file_list, node);
 	g_list_free_1 (node);
 
-	if (file->details->unconfirmed == FALSE) {
+	if (!file->details->unconfirmed) {
 		directory->details->confirmed_file_count--;
 	}
 
@@ -595,7 +596,6 @@ nautilus_directory_file_list_length_reached (NautilusDirectory *directory)
 	return directory->details->confirmed_file_count >= NAUTILUS_DIRECTORY_FILE_LIST_HARD_LIMIT;
 }
 
-
 GList *
 nautilus_directory_begin_file_name_change (NautilusDirectory *directory,
 					   NautilusFile *file)
@@ -698,6 +698,16 @@ nautilus_directory_emit_done_loading (NautilusDirectory *directory)
 			 signals[DONE_LOADING]);
 }
 
+void
+nautilus_directory_emit_load_error (NautilusDirectory *directory,
+				    GnomeVFSResult error_result)
+{
+	gtk_signal_emit (GTK_OBJECT (directory),
+			 signals[LOAD_ERROR],
+			 error_result);
+}
+
+
 static char *
 uri_get_directory_part (const char *uri)
 {
@@ -848,10 +858,10 @@ nautilus_directory_notify_files_added (GList *uris)
 	GnomeVFSURI *vfs_uri;
 
 	/* Make a list of added files in each directory. */
-	added_lists = g_hash_table_new (g_direct_hash, g_direct_equal);
+	added_lists = g_hash_table_new (NULL, NULL);
 
 	/* Make a list of parent directories that will need their counts updated. */
-	parent_directories = g_hash_table_new (g_direct_hash, g_direct_equal);
+	parent_directories = g_hash_table_new (NULL, NULL);
 
 	for (p = uris; p != NULL; p = p->next) {
 		uri = (const char *) p->data;
@@ -904,10 +914,10 @@ nautilus_directory_notify_files_removed (GList *uris)
 	NautilusFile *file;
 
 	/* Make a list of changed files in each directory. */
-	changed_lists = g_hash_table_new (g_direct_hash, g_direct_equal);
+	changed_lists = g_hash_table_new (NULL, NULL);
 
 	/* Make a list of parent directories that will need their counts updated. */
-	parent_directories = g_hash_table_new (g_direct_hash, g_direct_equal);
+	parent_directories = g_hash_table_new (NULL, NULL);
 
 	/* Go through all the notifications. */
 	for (p = uris; p != NULL; p = p->next) {
@@ -939,51 +949,174 @@ nautilus_directory_notify_files_removed (GList *uris)
 	g_hash_table_destroy (parent_directories);
 }
 
-void
-nautilus_directory_handle_directory_moved (NautilusDirectory *directory,
-					   const char *to_uri)
+static void
+set_directory_uri (NautilusDirectory *directory,
+		   const char *new_uri)
 {
-	GnomeVFSURI *vfs_uri;
+	GnomeVFSURI *new_vfs_uri;
+	GnomeVFSURI *new_public_metafile_vfs_uri;
+	GnomeVFSURI *new_private_metafile_vfs_uri;
 
-	/* I believe it's impossible for a self-owned file/directory to be
-           moved. But if that did somehow happen, carnage would
-           result. */
-	g_assert (directory->details->as_file == NULL);
-
-	g_hash_table_remove (directories, directory->details->uri);
+	new_vfs_uri = gnome_vfs_uri_new (new_uri);
+	new_public_metafile_vfs_uri = new_vfs_uri == NULL ? NULL
+		: gnome_vfs_uri_append_file_name (new_vfs_uri, METAFILE_NAME);
+	new_private_metafile_vfs_uri = construct_private_metafile_vfs_uri (new_uri);
 
 	g_free (directory->details->uri);
-	directory->details->uri = g_strdup (to_uri);
-
-	if (directory->details->private_metafile_vfs_uri != NULL) {
-		gnome_vfs_uri_unref (directory->details->private_metafile_vfs_uri);
-	}
-
-	directory->details->private_metafile_vfs_uri = construct_private_metafile_vfs_uri (to_uri);
-
-	/* FIXME: we also need to rename the private metafile, if it
-           exists, don't we. */
-
+	directory->details->uri = g_strdup (new_uri);
+	
 	if (directory->details->vfs_uri != NULL) {
 		gnome_vfs_uri_unref (directory->details->vfs_uri);
 	}
+	directory->details->vfs_uri = new_vfs_uri;
 
 	if (directory->details->public_metafile_vfs_uri != NULL) {
 		gnome_vfs_uri_unref (directory->details->public_metafile_vfs_uri);
 	}
+	directory->details->public_metafile_vfs_uri =
+		new_public_metafile_vfs_uri;
 
-	vfs_uri = gnome_vfs_uri_new (to_uri);
+	if (directory->details->private_metafile_vfs_uri != NULL) {
+		gnome_vfs_uri_unref (directory->details->private_metafile_vfs_uri);
+	}
+	directory->details->private_metafile_vfs_uri
+		= new_private_metafile_vfs_uri;
+}
 
-	directory->details->vfs_uri = vfs_uri;
-	directory->details->public_metafile_vfs_uri = vfs_uri == NULL ? NULL
-		: gnome_vfs_uri_append_file_name (vfs_uri, METAFILE_NAME);
+static char *
+get_path_from_vfs_uri (GnomeVFSURI *vfs_uri)
+{
+	char *uri, *path;
 
+	if (vfs_uri == NULL) {
+		return NULL;
+	}
+
+	uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
+	if (uri == NULL) {
+		return NULL;
+	}
+	path = gnome_vfs_get_local_path_from_uri (uri);
+	g_free (uri);
+	return path;
+}
+
+static char *
+get_private_metafile_path (NautilusDirectory *directory)
+{
+	return get_path_from_vfs_uri (directory->details->private_metafile_vfs_uri);
+}
+
+static void
+change_directory_uri (NautilusDirectory *directory,
+		      const char *new_uri)
+{
+	char *old_metafile_path, *new_metafile_path;
+
+	/* I believe it's impossible for a self-owned file/directory
+	 * to be moved. But if that did somehow happen, this function
+	 * wouldn't do enough to handle it.
+	 */
+	g_return_if_fail (directory->details->as_file == NULL);
+
+	old_metafile_path = get_private_metafile_path (directory);
+
+	g_hash_table_remove (directories,
+			     directory->details->uri);
+
+	set_directory_uri (directory, new_uri);
 
 	g_hash_table_insert (directories,
 			     directory->details->uri,
 			     directory);
 
-	/* FIXME: need to handle updating child directories as well */
+	new_metafile_path = get_private_metafile_path (directory);
+
+	if (old_metafile_path != NULL && new_metafile_path != NULL) {
+		rename (old_metafile_path, new_metafile_path);
+	}
+
+	g_free (old_metafile_path);
+	g_free (new_metafile_path);
+}
+
+typedef struct {
+	char *uri_prefix;
+	GList *directories;
+} CollectData;
+
+static void
+collect_directories_by_prefix (gpointer key, gpointer value, gpointer callback_data)
+{
+	const char *uri, *uri_suffix;
+	NautilusDirectory *directory;
+	CollectData *collect_data;
+
+	uri = (const char *) key;
+	directory = NAUTILUS_DIRECTORY (value);
+	collect_data = (CollectData *) callback_data;
+	
+	if (nautilus_str_has_prefix (uri, collect_data->uri_prefix)) {
+		uri_suffix = &uri[strlen (collect_data->uri_prefix)];
+		switch (uri_suffix[0]) {
+		case '\0':
+		case '/':
+			nautilus_directory_ref (directory);
+			collect_data->directories =
+				g_list_prepend (collect_data->directories,
+						directory);
+			break;
+		}
+	}
+}
+
+static char *
+str_replace_prefix (const char *str,
+		    const char *old_prefix,
+		    const char *new_prefix)
+{
+	const char *old_suffix;
+
+	g_return_val_if_fail (nautilus_str_has_prefix (str, old_prefix),
+			      g_strdup (str));
+
+	old_suffix = &str [strlen (old_prefix)];
+	return g_strconcat (new_prefix, old_suffix, NULL);
+}
+
+void
+nautilus_directory_moved (const char *old_uri,
+			  const char *new_uri)
+{
+	char *canonical_old_uri, *canonical_new_uri;
+	CollectData collection;
+	NautilusDirectory *directory;
+	char *new_directory_uri;
+	GList *node;
+
+	canonical_old_uri = nautilus_directory_make_uri_canonical (old_uri);
+	canonical_new_uri = nautilus_directory_make_uri_canonical (new_uri);
+
+	collection.uri_prefix = canonical_old_uri;
+	collection.directories = NULL;
+
+	g_hash_table_foreach (directories, collect_directories_by_prefix, &collection);
+
+	for (node = collection.directories; node != NULL; node = node->next) {
+		directory = NAUTILUS_DIRECTORY (node->data);
+		new_directory_uri = str_replace_prefix (directory->details->uri,
+							canonical_old_uri,
+							canonical_new_uri);
+		change_directory_uri (directory,
+				      new_directory_uri);
+		g_free (new_directory_uri);
+		nautilus_directory_unref (directory);
+	}
+
+	g_list_free (collection.directories);
+
+	g_free (canonical_old_uri);
+	g_free (canonical_new_uri);
 }
 
 void
@@ -993,7 +1126,6 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 	URIPair *pair;
 	NautilusFile *file;
 	NautilusDirectory *old_directory, *new_directory;
-	NautilusDirectory *moved_directory;
 	GHashTable *parent_directories;
 	GList *new_files_list, *unref_list;
 	GHashTable *added_lists, *changed_lists;
@@ -1001,12 +1133,12 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 
 	/* Make a list of added and changed files in each directory. */
 	new_files_list = NULL;
-	added_lists = g_hash_table_new (g_direct_hash, g_direct_equal);
-	changed_lists = g_hash_table_new (g_direct_hash, g_direct_equal);
+	added_lists = g_hash_table_new (NULL, NULL);
+	changed_lists = g_hash_table_new (NULL, NULL);
 	unref_list = NULL;
 
 	/* Make a list of parent directories that will need their counts updated. */
-	parent_directories = g_hash_table_new (g_direct_hash, g_direct_equal);
+	parent_directories = g_hash_table_new (NULL, NULL);
 
 	for (p = uri_pairs; p != NULL; p = p->next) {
 		pair = p->data;
@@ -1024,13 +1156,8 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 						    new_directory);
 		}
 
-		/* rename an existing directory */
-		moved_directory = nautilus_directory_get_existing (pair->from_uri);
-		if (moved_directory != NULL) {
-			nautilus_directory_handle_directory_moved (moved_directory, 
-								   pair->to_uri);
-			nautilus_directory_unref (moved_directory);
-		}
+		/* Update any directory objects that are affected. */
+		nautilus_directory_moved (pair->from_uri, pair->to_uri);
 
 		/* Move an existing file. */
 		file = nautilus_file_get_existing (pair->from_uri);
@@ -1041,50 +1168,33 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 		} else {
 			/* Handle notification in the old directory. */
 			old_directory = file->details->directory;
-			hash_table_list_prepend (changed_lists,
-						 old_directory,
-						 file);
-
 			collect_parent_directories (parent_directories, old_directory);
 
 			/* Locate the new directory. */
 			new_directory = get_parent_directory (pair->to_uri);
-			g_assert (new_directory != NULL);
-
 			collect_parent_directories (parent_directories, new_directory);
-
-			/* If the file is moving between directories,
-			 * there is more to do.
+			/* We can unref now -- new_directory is in the
+			 * parent directories list so it will be
+			 * around until the end of this function
+			 * anyway.
 			 */
-			if (new_directory != old_directory) {
-				/* Update the name. */
-				name = nautilus_uri_get_basename (pair->to_uri);
-				nautilus_file_update_name (file, name);
-				g_free (name);
-				
-				/* Remove from old directory. */
-				nautilus_directory_remove_file (old_directory, file);
+			nautilus_directory_unref (new_directory);
 
-				/* Point the file at the new
-				 * directory. We already have a ref to
-				 * it from the get_parent_directory
-				 * function so we don't have to ref.
-				 */
-				/* FIXME: Doesn't update the link hash table. */
-				file->details->directory = new_directory;
+			/* Update the file's name. */
+			name = nautilus_uri_get_basename (pair->to_uri);
+			nautilus_file_update_name (file, name);
+			g_free (name);
 
-				/* Add to new directory. */
-				nautilus_directory_add_file (new_directory, file);
-				
-				/* Handle notification in the new directory. */
-				hash_table_list_prepend (added_lists,
-							 new_directory,
-							 file);
+			/* Update the file's directory. */
+			nautilus_file_set_directory (file, new_directory);
+			
+			hash_table_list_prepend
+				(changed_lists, old_directory, file);
+			if (old_directory != new_directory) {
+				hash_table_list_prepend
+					(added_lists, new_directory, file);
 			}
 
-			/* Done with the old directory. */
-			nautilus_directory_unref (old_directory);
-
 			/* Unref each file once to balance out nautilus_file_get. */
 			unref_list = g_list_prepend (unref_list, file);
 		}
@@ -1114,7 +1224,7 @@ nautilus_directory_schedule_metadata_copy (GList *uri_pairs)
 	GList *p;
 	URIPair *pair;
 	NautilusDirectory *source_directory, *destination_directory;
-	char *source_file_name, *destination_file_name;
+	const char *source_relative_uri, *destination_relative_uri;
 
 	for (p = uri_pairs; p != NULL; p = p->next) {
 		pair = (URIPair *) p->data;
@@ -1122,16 +1232,13 @@ nautilus_directory_schedule_metadata_copy (GList *uri_pairs)
 		source_directory = get_parent_directory (pair->from_uri);
 		destination_directory = get_parent_directory (pair->to_uri);
 		
-		source_file_name = nautilus_uri_get_basename (pair->from_uri);
-		destination_file_name = nautilus_uri_get_basename (pair->to_uri);
+		source_relative_uri = g_basename (pair->from_uri);
+		destination_relative_uri = g_basename (pair->to_uri);
 		
 		nautilus_directory_copy_file_metadata (source_directory,
-						       source_file_name,
+						       source_relative_uri,
 						       destination_directory,
-						       destination_file_name);
-		
-		g_free (destination_file_name);
-		g_free (source_file_name);
+						       destination_relative_uri);
 		
 		nautilus_directory_unref (source_directory);
 		nautilus_directory_unref (destination_directory);
@@ -1144,7 +1251,7 @@ nautilus_directory_schedule_metadata_move (GList *uri_pairs)
 	GList *p;
 	URIPair *pair;
 	NautilusDirectory *source_directory, *destination_directory;
-	char *source_file_name, *destination_file_name;
+	const char *source_relative_uri, *destination_relative_uri;
 
 	for (p = uri_pairs; p != NULL; p = p->next) {
 		pair = (URIPair *) p->data;
@@ -1152,18 +1259,15 @@ nautilus_directory_schedule_metadata_move (GList *uri_pairs)
 		source_directory = get_parent_directory (pair->from_uri);
 		destination_directory = get_parent_directory (pair->to_uri);
 		
-		source_file_name = nautilus_uri_get_basename (pair->from_uri);
-		destination_file_name = nautilus_uri_get_basename (pair->to_uri);
+		source_relative_uri = g_basename (pair->from_uri);
+		destination_relative_uri = g_basename (pair->to_uri);
 		
 		nautilus_directory_copy_file_metadata (source_directory,
-						       source_file_name,
+						       source_relative_uri,
 						       destination_directory,
-						       destination_file_name);
+						       destination_relative_uri);
 		nautilus_directory_remove_file_metadata (source_directory,
-							 source_file_name);
-		
-		g_free (destination_file_name);
-		g_free (source_file_name);
+							 source_relative_uri);
 		
 		nautilus_directory_unref (source_directory);
 		nautilus_directory_unref (destination_directory);
@@ -1176,18 +1280,17 @@ nautilus_directory_schedule_metadata_remove (GList *uris)
 	GList *p;
 	const char *uri;
 	NautilusDirectory *directory;
-	char *file_name;
+	const char *relative_uri;
 
 	for (p = uris; p != NULL; p = p->next) {
 		uri = (const char *) p->data;
 
 		directory = get_parent_directory (uri);
-		file_name = nautilus_uri_get_basename (uri);
+		relative_uri = g_basename (uri);
 		
 		nautilus_directory_remove_file_metadata (directory,
-							 file_name);
+							 relative_uri);
 		
-		g_free (file_name);
 		nautilus_directory_unref (directory);
 	}
 }
@@ -1234,6 +1337,33 @@ nautilus_directory_contains_file (NautilusDirectory *directory,
 		 contains_file, (directory, file));
 }
 
+char *
+nautilus_directory_get_file_uri (NautilusDirectory *directory,
+				 const char *file_name)
+{
+	GnomeVFSURI *directory_uri, *file_uri;
+	char *result;
+
+	g_return_val_if_fail (NAUTILUS_IS_DIRECTORY (directory), NULL);
+	g_return_val_if_fail (file_name != NULL, NULL);
+
+	result = NULL;
+
+	directory_uri = gnome_vfs_uri_new (directory->details->uri);
+
+	g_assert (directory_uri != NULL);
+
+	file_uri = gnome_vfs_uri_append_string (directory_uri, file_name);
+	gnome_vfs_uri_unref (directory_uri);
+
+	if (file_uri != NULL) {
+		result = gnome_vfs_uri_to_string (file_uri, GNOME_VFS_URI_HIDE_NONE);
+		gnome_vfs_uri_unref (file_uri);
+	}
+
+	return result;
+}
+
 void
 nautilus_directory_call_when_ready (NautilusDirectory *directory,
 				    GList *file_attributes,
@@ -1266,6 +1396,8 @@ nautilus_directory_cancel_callback (NautilusDirectory *directory,
 void
 nautilus_directory_file_monitor_add (NautilusDirectory *directory,
 				     gconstpointer client,
+				     gboolean monitor_hidden_files,
+				     gboolean monitor_backup_files,
 				     GList *file_attributes,
 				     gboolean force_reload)
 {
@@ -1275,6 +1407,8 @@ nautilus_directory_file_monitor_add (NautilusDirectory *directory,
 	NAUTILUS_CALL_VIRTUAL
 		(NAUTILUS_DIRECTORY_CLASS, directory,
 		 file_monitor_add, (directory, client,
+				    monitor_hidden_files,
+				    monitor_backup_files,
 				    file_attributes,
 				    force_reload));
 }
@@ -1350,6 +1484,7 @@ nautilus_self_check_directory (void)
 
 	nautilus_directory_file_monitor_add
 		(directory, &data_dummy,
+		 TRUE, TRUE,
 		 NULL, FALSE);
 
 	got_metadata_flag = FALSE;
@@ -1456,10 +1591,10 @@ nautilus_self_check_directory (void)
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical (""), "file:///");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("file:/"), "file:///");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("file:///"), "file:///");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("TRASH:XXX"), "trash:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("trash:xxx"), "trash:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("GNOME-TRASH:XXX"), "trash:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("gnome-trash:xxx"), "trash:");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("TRASH:XXX"), NAUTILUS_TRASH_URI);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("trash:xxx"), NAUTILUS_TRASH_URI);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("GNOME-TRASH:XXX"), NAUTILUS_TRASH_URI);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("gnome-trash:xxx"), NAUTILUS_TRASH_URI);
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("file:///home/mathieu/"), "file:///home/mathieu");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("file:///home/mathieu"), "file:///home/mathieu");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("ftp://mathieu:password@le-hackeur.org"), "ftp://mathieu:password@le-hackeur.org");
@@ -1468,7 +1603,7 @@ nautilus_self_check_directory (void)
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/"), "http://le-hackeur.org");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/dir"), "http://le-hackeur.org/dir");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/dir/"), "http://le-hackeur.org/dir");
-	/* FIXME: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
+	/* FIXME bugzilla.eazel.com 5068: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("search://[file://]file_name contains stuff"), "search://[file/]file_name contains stuff");
 #ifdef EAZEL_SERVICES
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("eazel-services:/~turtle"), "eazel-services:///~turtle");
