@@ -26,6 +26,7 @@
 #define NAUTILUS_DIRECTORY_H
 
 #include <gtk/gtkobject.h>
+#include <libgnomevfs/gnome-vfs-types.h>
 
 /* NautilusDirectory is a class that manages the model for a directory,
    real or virtual, for Nautilus, mainly the file-manager component. The directory is
@@ -37,9 +38,6 @@
           set of known files changes.
        2) An abstract interface for getting attributes and performing
           operations on files.
-       3) An interface that folds together the cached information that's
-          kept in the metafile with "trustworthy" versions of the same
-          information available from other means.
 */
 
 #define NAUTILUS_TYPE_DIRECTORY \
@@ -59,6 +57,14 @@
 typedef struct NautilusFile NautilusFile;
 #endif
 
+/* FIXME bugzilla.eazel.com 5382:
+ * Increase or remove this limit? 
+ */
+/* FIXME bugzilla.eazel.com 5603:
+ * This limit is not actually "hard", which can lead to some minor UI problems.
+ */
+#define NAUTILUS_DIRECTORY_FILE_LIST_HARD_LIMIT     4000
+
 typedef struct NautilusDirectoryDetails NautilusDirectoryDetails;
 
 typedef struct
@@ -101,6 +107,8 @@ typedef struct
 	 */
 	void     (* done_loading)        (NautilusDirectory         *directory);
 
+	void     (* load_error)          (NautilusDirectory         *directory);
+
 	/*** Virtual functions for subclasses to override. ***/
 	gboolean (* contains_file)       (NautilusDirectory         *directory,
 					  NautilusFile              *file);
@@ -113,6 +121,8 @@ typedef struct
 					  gpointer                   callback_data);
 	void     (* file_monitor_add)    (NautilusDirectory          *directory,
 					  gconstpointer              client,
+					  gboolean                   monitor_hidden_files,
+					  gboolean                   monitor_backup_files,
 					  GList                     *monitor_attributes,
 					  gboolean                   force_reload);
 	void     (* file_monitor_remove) (NautilusDirectory         *directory,
@@ -146,6 +156,10 @@ char *             nautilus_directory_get_uri                  (NautilusDirector
 gboolean           nautilus_directory_contains_file            (NautilusDirectory         *directory,
 								NautilusFile              *file);
 
+/* Get the uri of the file in the directory, NULL if not found */
+char *		   nautilus_directory_get_file_uri	       (NautilusDirectory	  *directory,
+								const char 		  *file_name);
+
 /* Get (and ref) a NautilusFile object for this directory. */
 NautilusFile *     nautilus_directory_get_corresponding_file   (NautilusDirectory         *directory);
 							    
@@ -167,6 +181,8 @@ void               nautilus_directory_cancel_callback          (NautilusDirector
 /* Monitor the files in a directory. */
 void               nautilus_directory_file_monitor_add         (NautilusDirectory         *directory,
 								gconstpointer              client,
+								gboolean                   monitor_hidden_files,
+								gboolean                   monitor_backup_files,
 								GList                     *monitor_attributes,
 								gboolean                   force_reload);
 void               nautilus_directory_file_monitor_remove      (NautilusDirectory         *directory,
