@@ -33,8 +33,9 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomeui/gnome-popup-menu.h>
-#include <string.h>
 #include <stdio.h>
+#include <string.h>
+#include <gtk/gtkmain.h>
 
 #include "nautilus-glib-extensions.h"
 #include "nautilus-link.h"
@@ -54,6 +55,7 @@ nautilus_drag_init (NautilusDragInfo *drag_info,
 	}
 
 	drag_info->drop_occured = FALSE;
+	drag_info->need_to_destroy = FALSE;
 }
 
 void
@@ -295,7 +297,7 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 		return;
 	}
 
-	/* Check for trash: URI.  We do a find_directory for any Trash directory. */
+	/* Check for trash URI.  We do a find_directory for any Trash directory. */
 	if (nautilus_uri_is_trash (target_uri_string)) {
 		result = gnome_vfs_find_directory (NULL, GNOME_VFS_DIRECTORY_KIND_TRASH,
 						   &target_uri, FALSE, FALSE, 0777);
@@ -312,7 +314,7 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 	dropped_uri = gnome_vfs_uri_new (((DragSelectionItem *)items->data)->uri);
 	same_fs = TRUE;
 
-	gnome_vfs_check_same_fs_uris (target_uri, dropped_uri, &same_fs);
+	gnome_vfs_check_same_fs_uris (dropped_uri, target_uri, &same_fs);
 	gnome_vfs_uri_unref (dropped_uri);
 	gnome_vfs_uri_unref (target_uri);
 	
@@ -353,6 +355,44 @@ add_one_uri_list (const char *uri, int x, int y, int w, int h,
 	g_string_append (result, "\r\n");
 }
 
+/* Encode a "_NETSCAPE_URL_" selection.  */
+static void
+add_one_url_list (const char *url, int x, int y, int w, int h, gpointer data)
+{
+	GString *result = (GString *)data;	
+	g_string_append (result, url);
+	g_string_append (result, "\n");
+}
+
+/* Encode a "text/path" selection.  */
+static void
+add_one_path_list (const char *uri, int x, int y, int w, int h, gpointer data)
+{
+	GString *result = (GString *)data;
+	const char *path, *scheme;
+	GnomeVFSURI *vfs_uri;
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	if (vfs_uri == NULL) {
+		return;
+	}
+	
+	/* Only accept the file scheme */
+	scheme = gnome_vfs_uri_get_scheme (vfs_uri);
+	if (strncmp (scheme, "file", strlen ("file") != 0)) {
+		gnome_vfs_uri_unref (vfs_uri);
+		return;
+	}
+	
+	path = gnome_vfs_uri_get_path (vfs_uri);
+	
+	g_string_append (result, path);
+	g_string_append (result, " ");
+	
+	gnome_vfs_uri_unref (vfs_uri);
+}
+
+
 /* Common function for drag_data_get_callback calls.
  * Returns FALSE if it doesn't handle drag data
  */
@@ -367,22 +407,29 @@ nautilus_drag_drag_data_get (GtkWidget *widget,
 {
 	GString *result;
 
-	if (info != NAUTILUS_ICON_DND_GNOME_ICON_LIST && info != NAUTILUS_ICON_DND_URI_LIST) {
-		/* don't know how to handle */
-		return FALSE;
-	}
-	
-	result = g_string_new (NULL);
-	
 	switch (info) {
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
+		result = g_string_new (NULL);
 		each_selected_item_iterator (add_one_gnome_icon_list, container_context, result);
 		break;
+		
 	case NAUTILUS_ICON_DND_URI_LIST:
+		result = g_string_new (NULL);
 		each_selected_item_iterator (add_one_uri_list, container_context, result);
 		break;
+		
+	case NAUTILUS_ICON_DND_TEXT:
+		result = g_string_new (NULL);
+		each_selected_item_iterator (add_one_path_list, container_context, result);
+		break;
+		
+	case NAUTILUS_ICON_DND_URL:
+		result = g_string_new (NULL);
+		each_selected_item_iterator (add_one_url_list, container_context, result);
+		break;
+		
 	default:
-		g_assert_not_reached ();
+		return FALSE;
 	}
 	
 	gtk_selection_data_set (selection_data,
@@ -467,6 +514,19 @@ nautilus_drag_drop_action_ask (GdkDragAction actions)
 	 * edge
 	 */
 
+
+
+gboolean
+nautilus_drag_autoscroll_in_scroll_region (GtkWidget *widget)
+{
+	float x_scroll_delta, y_scroll_delta;
+
+	nautilus_drag_autoscroll_calculate_delta (widget, &x_scroll_delta, &y_scroll_delta);
+
+	return x_scroll_delta != 0 || y_scroll_delta != 0;
+}
+
+
 void
 nautilus_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_delta, float *y_scroll_delta)
 {
@@ -536,15 +596,53 @@ nautilus_drag_autoscroll_calculate_delta (GtkWidget *widget, float *x_scroll_del
 
 }
 
+
+
+void
+nautilus_drag_autoscroll_start (NautilusDragInfo *drag_info,
+				GtkWidget        *widget,
+				GtkFunction       callback,
+				gpointer          user_data)
+{
+	if (nautilus_drag_autoscroll_in_scroll_region (widget)) {
+		if (drag_info->auto_scroll_timeout_id == 0) {
+			drag_info->waiting_to_autoscroll = TRUE;
+			drag_info->start_auto_scroll_in = nautilus_get_system_time() 
+				+ AUTOSCROLL_INITIAL_DELAY;
+			
+			drag_info->auto_scroll_timeout_id = gtk_timeout_add
+				(AUTOSCROLL_TIMEOUT_INTERVAL,
+				 callback,
+			 	 user_data);
+		}
+	} else {
+		if (drag_info->auto_scroll_timeout_id != 0) {
+			gtk_timeout_remove (drag_info->auto_scroll_timeout_id);
+			drag_info->auto_scroll_timeout_id = 0;
+		}
+	}
+}
+
+void
+nautilus_drag_autoscroll_stop (NautilusDragInfo *drag_info)
+{
+	if (drag_info->auto_scroll_timeout_id != 0) {
+		gtk_timeout_remove (drag_info->auto_scroll_timeout_id);
+		drag_info->auto_scroll_timeout_id = 0;
+	}
+}
+
+
 void
 nautilus_drag_file_receive_dropped_keyword (NautilusFile *file, char *keyword)
 {
 	GList *keywords, *word;
 
-	g_assert (keyword != NULL);
+	g_return_if_fail (NAUTILUS_IS_FILE (file));
+	g_return_if_fail (keyword != NULL);
 
 	/* special case the erase emblem */
-	if (!nautilus_strcmp (keyword, ERASE_KEYWORD)) {
+	if (strcmp (keyword, ERASE_KEYWORD) == 0) {
 		keywords = NULL;
 	} else {
 		keywords = nautilus_file_get_keywords (file);
@@ -559,4 +657,5 @@ nautilus_drag_file_receive_dropped_keyword (NautilusFile *file, char *keyword)
 	}
 	
 	nautilus_file_set_keywords (file, keywords);
+	nautilus_g_list_free_deep (keywords);
 }
