@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* nautilus-drag.h - Common Drag & drop handling code shared by the icon container
    and the list view.
 
@@ -35,7 +36,6 @@
 #define AUTOSCROLL_INITIAL_DELAY 750000
 	/* in microseconds */
 
-
 /* Item of the drag selection list */
 typedef struct {
 	char *uri;
@@ -51,7 +51,8 @@ typedef enum {
 	NAUTILUS_ICON_DND_URL,
 	NAUTILUS_ICON_DND_COLOR,
 	NAUTILUS_ICON_DND_BGIMAGE,
-	NAUTILUS_ICON_DND_KEYWORD
+	NAUTILUS_ICON_DND_KEYWORD,
+	NAUTILUS_ICON_DND_TEXT
 } NautilusIconDndTargetType;
 
 /* drag&drop-related information. */
@@ -77,6 +78,9 @@ typedef struct {
         /* has the drop occured ? */
         gboolean drop_occured;
 
+	/* whether or not need to clean up the previous dnd data */
+	gboolean need_to_destroy;
+
 	/* autoscrolling during dragging */
 	int auto_scroll_timeout_id;
 	gboolean waiting_to_autoscroll;
@@ -88,6 +92,7 @@ typedef struct {
 /* Drag & Drop target names. */
 #define NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE "x-special/gnome-icon-list"
 #define NAUTILUS_ICON_DND_URI_LIST_TYPE        "text/uri-list"
+#define NAUTILUS_ICON_DND_TEXT_TYPE            "text/plain"
 #define NAUTILUS_ICON_DND_URL_TYPE	       "_NETSCAPE_URL"
 #define NAUTILUS_ICON_DND_COLOR_TYPE           "application/x-color"
 #define NAUTILUS_ICON_DND_BGIMAGE_TYPE         "property/bgimage"
@@ -138,12 +143,21 @@ int 			nautilus_drag_modifier_based_action 		(int default_action,
 
 GdkDragAction		nautilus_drag_drop_action_ask			(GdkDragAction possible_actions);
 
+gboolean                nautilus_drag_autoscroll_in_scroll_region       (GtkWidget *widget);
 void                    nautilus_drag_autoscroll_calculate_delta        (GtkWidget *widget, 
 									 float *x_scroll_delta, 
 									 float *y_scroll_delta);
+
+void                    nautilus_drag_autoscroll_start                  (NautilusDragInfo *drag_info,
+									 GtkWidget        *widget,
+									 GtkFunction       callback,
+									 gpointer          user_data);
+
+void                    nautilus_drag_autoscroll_stop                   (NautilusDragInfo *drag_info);
+
+
 void                    nautilus_drag_file_receive_dropped_keyword      (NautilusFile *file, 
 									 char *keyword);
 
-
 #endif
 
