@@ -37,6 +37,7 @@
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 
+#include <libnautilus-extensions/nautilus-gdk-extensions.h>
 #include <libnautilus-extensions/nautilus-undo-signal-handlers.h>
 
 #include <orb/orbit.h>
@@ -44,6 +45,7 @@
 
 enum {
 	USER_CHANGED,
+	SELECTION_CHANGED,
 	LAST_SIGNAL
 };
 static guint signals[LAST_SIGNAL];
@@ -53,13 +55,20 @@ static void nautilus_entry_initialize_class (NautilusEntryClass *class);
 static void nautilus_entry_destroy	    (GtkObject 		*object);
 static gint nautilus_entry_key_press 	    (GtkWidget   	*widget,
 		     		      	     GdkEventKey 	*event);
+static gint nautilus_entry_motion_notify    (GtkWidget          *widget,
+					     GdkEventMotion *event);
 static void nautilus_entry_insert_text      (GtkEditable    	*editable,
 			 		     const gchar    	*text,
 			 		     gint            	length,
 			 		     gint           	*position);
+static gboolean nautilus_entry_selection_clear (GtkWidget        *widget,
+					     GdkEventSelection *event);
 static void nautilus_entry_delete_text      (GtkEditable    	*editable,
 			 		     gint            	start_pos,
 			 		     gint            	end_pos);
+static void nautilus_entry_set_selection    (GtkEditable        *editable,
+					     gint               start_pos,
+					     gint               end_pos);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusEntry, nautilus_entry, GTK_TYPE_ENTRY)
 
@@ -75,11 +84,15 @@ nautilus_entry_initialize_class (NautilusEntryClass *class)
 	editable_class = GTK_EDITABLE_CLASS (class);
 		
 	widget_class->key_press_event = nautilus_entry_key_press;
+	widget_class->motion_notify_event = nautilus_entry_motion_notify;
+	widget_class->selection_clear_event = nautilus_entry_selection_clear;
 	
 	object_class->destroy = nautilus_entry_destroy;
 	
 	editable_class->insert_text = nautilus_entry_insert_text;
 	editable_class->delete_text = nautilus_entry_delete_text;
+	editable_class->set_selection = nautilus_entry_set_selection;
+
 
 	/* Set up signals */
 	signals[USER_CHANGED] = gtk_signal_new
@@ -90,16 +103,31 @@ nautilus_entry_initialize_class (NautilusEntryClass *class)
 				    user_changed),
 		 gtk_marshal_NONE__NONE,
 		 GTK_TYPE_NONE, 0);
-				
+	signals[SELECTION_CHANGED] = gtk_signal_new
+		("selection_changed",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusEntryClass,
+				    selection_changed),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
 
 static void
 nautilus_entry_initialize (NautilusEntry *entry)
 {
+	GtkWidget *widget;
+
+	widget = GTK_WIDGET (entry);
+	
 	entry->user_edit = TRUE;
 	entry->special_tab_handling = FALSE;
-	
+	entry->cursor_obscured = FALSE;
+
+	/* Allow pointer motion events so we can expose an obscured cursor if necessary */
+	gtk_widget_set_events (widget, gtk_widget_get_events (widget) | GDK_POINTER_MOTION_MASK);
+
 	nautilus_undo_set_up_nautilus_entry_for_undo (entry);
 }
 
@@ -109,6 +137,18 @@ nautilus_entry_new (void)
 	return gtk_widget_new (NAUTILUS_TYPE_ENTRY, NULL);
 }
 
+GtkWidget *
+nautilus_entry_new_with_max_length (guint16 max)
+{
+	GtkWidget *widget;
+
+	widget = gtk_widget_new (NAUTILUS_TYPE_ENTRY, NULL);
+	GTK_ENTRY (widget)->text_max_length = max;
+
+	return widget;
+}
+
+
 static void 
 nautilus_entry_destroy (GtkObject *object)
 {
@@ -119,12 +159,26 @@ nautilus_entry_destroy (GtkObject *object)
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
 
+
+static void
+obscure_cursor (NautilusEntry *entry)
+{
+	if (entry->cursor_obscured) {
+		return;
+	}
+	
+	entry->cursor_obscured = TRUE;	
+	nautilus_gdk_window_set_invisible_cursor (GTK_ENTRY (entry)->text_area);
+}
+
+
 static gint 
 nautilus_entry_key_press (GtkWidget *widget, GdkEventKey *event)
 {
 	NautilusEntry *entry;
 	GtkEditable *editable;
 	int position;
+	gint return_code;
 	
 	entry = NAUTILUS_ENTRY (widget);
 	editable = GTK_EDITABLE (widget);
@@ -161,9 +215,48 @@ nautilus_entry_key_press (GtkWidget *widget, GdkEventKey *event)
 		break;
 	}
 
-	return NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS,
-					   key_press_event,
-					   (widget, event));
+	obscure_cursor (entry);
+
+	return_code = NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS,
+						  key_press_event,
+						  (widget, event));
+	gtk_signal_emit (GTK_OBJECT (widget), signals[SELECTION_CHANGED]);
+
+	return return_code;
+	
+}
+
+static gint
+nautilus_entry_motion_notify (GtkWidget *widget, GdkEventMotion *event)
+{
+	gint return_code;
+	guint old_start_pos, old_end_pos;
+	GdkCursor *cursor;
+	NautilusEntry *entry;
+
+	/* Reset cursor to I-Beam */
+	entry = NAUTILUS_ENTRY (widget);
+	if (entry->cursor_obscured) {
+		cursor = gdk_cursor_new (GDK_XTERM);
+		gdk_window_set_cursor (GTK_ENTRY (entry)->text_area, cursor);
+		gdk_cursor_destroy (cursor);
+		entry->cursor_obscured = FALSE;
+	}
+
+	if (GTK_ENTRY (widget)->button == 0) {
+    	return FALSE;
+    }
+
+	old_start_pos = GTK_EDITABLE (widget)->selection_start_pos;
+	old_end_pos = GTK_EDITABLE (widget)->selection_end_pos;
+
+	return_code = NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS, motion_notify_event, (widget, event));
+	if (GTK_EDITABLE (widget)->selection_start_pos != old_start_pos ||
+	    GTK_EDITABLE (widget)->selection_end_pos != old_end_pos) {
+		gtk_signal_emit (GTK_OBJECT (widget), signals[SELECTION_CHANGED]);
+	}
+
+	return return_code;
 }
 
 /**
@@ -231,6 +324,19 @@ nautilus_entry_set_text (NautilusEntry *entry, const gchar *text)
 	entry->user_edit = FALSE;
 	gtk_entry_set_text (GTK_ENTRY (entry), text);
 	entry->user_edit = TRUE;
+	
+	gtk_signal_emit (GTK_OBJECT (entry), signals[SELECTION_CHANGED]);
+}
+
+void 
+nautilus_entry_set_selection (GtkEditable *editable,
+			      gint start_pos,
+			      gint end_pos)
+{
+
+	NAUTILUS_CALL_PARENT_CLASS (GTK_EDITABLE_CLASS, set_selection,
+				    (editable, start_pos, end_pos));
+	gtk_signal_emit (GTK_OBJECT (editable), signals[SELECTION_CHANGED]);
 }
 
 static void 
@@ -248,6 +354,8 @@ nautilus_entry_insert_text (GtkEditable *editable, const gchar *text,
 
 	NAUTILUS_CALL_PARENT_CLASS (GTK_EDITABLE_CLASS, insert_text, 
 				   (editable, text, length, position));
+	gtk_signal_emit (GTK_OBJECT (editable), signals[SELECTION_CHANGED]);
+
 }
 
 			 		     
@@ -255,6 +363,7 @@ static void
 nautilus_entry_delete_text (GtkEditable *editable, gint start_pos, gint end_pos)
 {
 	NautilusEntry *entry;
+	
 
 	entry = NAUTILUS_ENTRY(editable);
 
@@ -262,7 +371,33 @@ nautilus_entry_delete_text (GtkEditable *editable, gint start_pos, gint end_pos)
 	if (entry->user_edit) {
 		gtk_signal_emit (GTK_OBJECT (editable), signals[USER_CHANGED]);
 	}
-
+	
 	NAUTILUS_CALL_PARENT_CLASS (GTK_EDITABLE_CLASS, delete_text, 
 				   (editable, start_pos, end_pos));
+
+	gtk_signal_emit (GTK_OBJECT (editable), signals[SELECTION_CHANGED]);
+}
+
+/* Overridden to work around GTK bug. The selection_clear_event is queued
+ * when the selection changes. Changing the selection to NULL and then
+ * back to the original selection owner still sends the event, so the
+ * selection owner then gets the selection ripped away from it. We ran into
+ * this with type-completion behavior in NautilusLocationBar (see bug 5313).
+ * There's a FIXME comment that seems to be about this same issue in
+ * gtk+/gtkselection.c, gtk_selection_clear.
+ */
+static gboolean
+nautilus_entry_selection_clear (GtkWidget         *widget,
+			        GdkEventSelection *event)
+{
+  g_return_val_if_fail (NAUTILUS_IS_ENTRY (widget), FALSE);
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  if (gdk_selection_owner_get (event->selection) == widget->window) {
+	return FALSE;
+  }
+
+  return NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS, 
+  				     selection_clear_event, 
+				     (widget, event));
 }
