@@ -24,25 +24,27 @@
 
 #include <config.h>
 #include "nautilus-file-utilities.h"
+
+#include "nautilus-file.h"
 #include "nautilus-glib-extensions.h"
 #include "nautilus-lib-self-check-functions.h"
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-util.h>
-#include <sys/stat.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include "nautilus-file.h"
 #include "nautilus-link-set.h"
 #include "nautilus-metadata.h"
 #include "nautilus-string.h"
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-async-ops.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-xfer.h>
 #include <ctype.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-xfer.h>
 #include <pthread.h>
+#include <pwd.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #define NAUTILUS_USER_DIRECTORY_NAME ".nautilus"
 #define DEFAULT_NAUTILUS_DIRECTORY_MODE (0755)
@@ -52,8 +54,6 @@
 
 #define NAUTILUS_USER_MAIN_DIRECTORY_NAME "Nautilus"
 
-#define DEFAULT_SCHEME "file://"
-
 #define READ_CHUNK_SIZE 8192
 
 struct NautilusReadFileHandle {
@@ -63,7 +63,7 @@ struct NautilusReadFileHandle {
 	gpointer callback_data;
 	gboolean is_open;
 	char *buffer;
-	int bytes_read;
+	GnomeVFSFileSize bytes_read;
 };
 
 #undef PTHREAD_ASYNC_READ
@@ -85,22 +85,40 @@ static void read_file_read_chunk (NautilusReadFileHandle *handle);
 char *
 nautilus_format_uri_for_display (const char *uri) 
 {
-	gchar *toreturn, *unescaped;
+	char *path;
 
-	g_assert (uri != NULL);
+	g_return_val_if_fail (uri != NULL, g_strdup (""));
 
-	unescaped = gnome_vfs_unescape_string_for_display (uri);
-	
-	/* Remove file:// from the beginning */
-	if (nautilus_istr_has_prefix (uri, DEFAULT_SCHEME)) {
-		toreturn = g_strdup (unescaped + sizeof (DEFAULT_SCHEME) - 1);
-	} else {
-		toreturn = g_strdup (unescaped);
+	path = gnome_vfs_get_local_path_from_uri (uri);
+	if (path != NULL) {
+		return path;
 	}
 	
-	g_free (unescaped);
+	return gnome_vfs_unescape_string_for_display (uri);
+}
 
-	return toreturn;
+static gboolean
+is_valid_scheme_character (char c)
+{
+	return isalnum ((guchar) c) || c == '+' || c == '-' || c == '.';
+}
+
+static gboolean
+has_valid_scheme (const char *uri)
+{
+	const char *p;
+
+	p = uri;
+
+	if (!is_valid_scheme_character (*p)) {
+		return FALSE;
+	}
+
+	do {
+		p++;
+	} while (is_valid_scheme_character (*p));
+
+	return *p == ':';
 }
 
 /**
@@ -117,48 +135,38 @@ nautilus_format_uri_for_display (const char *uri)
 char *
 nautilus_make_uri_from_input (const char *location)
 {
-	char *path, *toreturn, *escaped;
-	const char *no_method;
-	int method_length;
+	char *stripped, *path, *uri;
 
-	if (location[0] == '/') {
-		escaped = gnome_vfs_escape_path_string (location);
-		toreturn = g_strconcat (DEFAULT_SCHEME, escaped, NULL);
-		g_free (escaped);
-	} else if (location[0] == '~') {
-		if (location[1] == '/') {
-			path = g_strconcat (g_get_home_dir (), &location[1], NULL);
-		} else if (location[1]) {
-			path = g_strconcat ("/home/", &location[1], NULL);
-		} else {
-			path = g_strdup (g_get_home_dir ());
-		}
-		escaped = gnome_vfs_escape_path_string (path);
-		toreturn = g_strconcat (DEFAULT_SCHEME, escaped, NULL);
+	g_return_val_if_fail (location != NULL, g_strdup (""));
+
+	/* Strip off leading and trailing spaces.
+	 * This makes copy/paste of URIs less error-prone.
+	 */
+	stripped = g_strstrip (g_strdup (location));
+
+	switch (stripped[0]) {
+	case '\0':
+		uri = g_strdup ("");
+		break;
+	case '/':
+		uri = gnome_vfs_get_uri_from_local_path (stripped);
+		break;
+	case '~':
+		path = gnome_vfs_expand_initial_tilde (stripped);
+		uri = gnome_vfs_get_uri_from_local_path (path);
 		g_free (path);
-		g_free (escaped);
-		
-	} else {
-		no_method = strchr (location, ':');
-		if (no_method == NULL) {
-			no_method = location;
+		break;
+	default:
+		if (has_valid_scheme (stripped)) {
+			uri = g_strdup (stripped);
 		} else {
-			no_method++;
-			if ((no_method[0] == '/') && (no_method[1] == '/')) {
-				no_method += 2;
-			}
+			uri = g_strconcat ("http://", stripped, NULL);
 		}
-
-		method_length = (no_method - location);
-		escaped = gnome_vfs_escape_host_and_path_string (no_method);
-		toreturn = g_new (char, strlen (escaped) + method_length + 1);
-		toreturn[0] = '\0';
-		strncat (toreturn, location, method_length);
-		strcat (toreturn, escaped);
-		g_free (escaped);
 	}
 
-	return toreturn;
+	g_free (stripped);
+
+	return uri;
 }
 
 char *
@@ -291,7 +299,7 @@ nautilus_make_uri_canonical (const char *uri)
 	 * "trash:".
 	 */
 	if (nautilus_uri_is_trash (uri)) {
-		return g_strdup ("trash:");
+		return g_strdup (NAUTILUS_TRASH_URI);
 	}
 
 	/* FIXME bugzilla.eazel.com 648: 
@@ -315,7 +323,8 @@ nautilus_make_uri_canonical (const char *uri)
 		old_uri = canonical_uri;
 
 		if (old_uri[0] != '/') {
-			/* FIXME: bandaid alert. Is this really the right thing to do?
+			/* FIXME bugzilla.eazel.com 5069: 
+			 *  bandaid alert. Is this really the right thing to do?
 			 * 
 			 * We got what really is a relative path. We do a little bit of
 			 * a stretch here and assume it was meant to be a cryptic absolute path,
@@ -433,6 +442,13 @@ nautilus_get_user_directory (void)
 
 	if (!g_file_exists (user_directory)) {
 		mkdir (user_directory, DEFAULT_NAUTILUS_DIRECTORY_MODE);
+		/* FIXME bugzilla.eazel.com 1286: 
+		 * How should we handle the case where this mkdir fails? 
+		 * Note that nautilus_application_startup will refuse to launch if this 
+		 * directory doesn't get created, so that case is OK. But the directory 
+		 * could be deleted after Nautilus was launched, and perhaps
+		 * there is some bad side-effect of not handling that case.
+		 */
 	}
 
 	return user_directory;
@@ -456,6 +472,13 @@ nautilus_get_desktop_directory (void)
 
 	if (!g_file_exists (desktop_directory)) {
 		mkdir (desktop_directory, DEFAULT_DESKTOP_DIRECTORY_MODE);
+		/* FIXME bugzilla.eazel.com 1286: 
+		 * How should we handle the case where this mkdir fails? 
+		 * Note that nautilus_application_startup will refuse to launch if this 
+		 * directory doesn't get created, so that case is OK. But the directory 
+		 * could be deleted after Nautilus was launched, and perhaps
+		 * there is some bad side-effect of not handling that case.
+		 */
 	}
 
 	return desktop_directory;
@@ -525,15 +548,20 @@ nautilus_get_user_main_directory (void)
 					 GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
 					 NULL, NULL);
 		
+		/* FIXME bugzilla.eazel.com 1286: 
+		 * How should we handle error codes returned from gnome_vfs_xfer_uri? 
+		 * Note that nautilus_application_startup will refuse to launch if this 
+		 * directory doesn't get created, so that case is OK. But the directory 
+		 * could be deleted after Nautilus was launched, and perhaps
+		 * there is some bad side-effect of not handling that case.
+		 */
+
 		gnome_vfs_uri_unref (source_uri);
 		gnome_vfs_uri_unref (destination_uri);
-		
-		/* FIXME bugzilla.eazel.com 1286: 
-		 * Is a g_warning good enough here? This seems like a big problem. 
+
+		/* If this fails to create the directory, nautilus_application_startup will
+		 * notice and refuse to launch.
 		 */
-		if (result != GNOME_VFS_OK) {
-			g_warning ("could not install the novice home directory.  Make sure you typed 'make install'");
-		}
 					
 		/* assign a custom image for the directory icon */
 		file_uri = gnome_vfs_get_uri_from_local_path (user_main_directory);
@@ -635,7 +663,7 @@ nautilus_read_entire_file (const char *uri,
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
 	char *buffer;
-	int total_bytes_read;
+	GnomeVFSFileSize total_bytes_read;
 	GnomeVFSFileSize bytes_read;
 
 	*file_size = 0;
@@ -975,7 +1003,7 @@ pthread_nautilus_read_file_thread_entry (void *cast_to_data)
 			}
 
 			buffer = g_realloc (buffer, total_bytes_read + READ_CHUNK_SIZE);
-			/* FIXME:
+			/* FIXME bugzilla.eazel.com 5070:
 			 * For a better cancellation granularity we should use gnome_vfs_read_cancellable
 			 * here, adding a GnomeVFSContext to NautilusAsyncReadFileData.
 			 */
@@ -1057,7 +1085,7 @@ pthread_nautilus_read_file_async(const char *uri, NautilusReadFileCallback callb
 	pthread_attr_init (&thread_attr);
 	pthread_attr_setdetachstate (&thread_attr, PTHREAD_CREATE_DETACHED);
 	if (pthread_create (&thread, &thread_attr, pthread_nautilus_read_file_thread_entry, data) != 0) {
-		/* FIXME:
+		/* FIXME bugzilla.eazel.com 5071:
 		 * Would be cleaner to call through an idle callback here.
 		 */
 		(*callback) (GNOME_VFS_ERROR_INTERNAL, 0, NULL, NULL);
@@ -1132,6 +1160,7 @@ nautilus_read_file_cancel (NautilusReadFileHandle *handle)
 	g_free (handle->buffer);
 	g_free (handle);
 #else
+
 	pthread_nautilus_read_file_async_cancel (handle);
 #endif
 }
@@ -1208,16 +1237,14 @@ nautilus_unique_temporary_file_name (void)
 	return file_name;
 }
 
-#ifdef EAZEL_BUILD_TIMESTAMP
-static const char *BUILD_TIMESTAMP = EAZEL_BUILD_TIMESTAMP;
-#else
-static const char *BUILD_TIMESTAMP = NULL;
-#endif
-
 char *
-nautilus_get_build_timestamp (void)
+nautilus_get_build_time_stamp (void)
 {
-	return BUILD_TIMESTAMP ? g_strdup (BUILD_TIMESTAMP) : NULL;
+#ifdef EAZEL_BUILD_TIMESTAMP
+	return g_strdup (EAZEL_BUILD_TIMESTAMP);
+#else
+	return NULL;
+#endif
 }
 
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
@@ -1225,22 +1252,42 @@ nautilus_get_build_timestamp (void)
 void
 nautilus_self_check_file_utilities (void)
 {
-	/* nautilus_make_uri_from_input */
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (""), "");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (" "), "");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (" / "), "file:///");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (" /"), "file:///");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (" /home\n\n"), "file:///home");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (" \n\t"), "");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("!"), "http://!");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("#"), "http://#");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/ "), "file:///");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/!"), "file:///!");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/#"), "file:///%23");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/%20"), "file:///%2520");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/%25"), "file:///%2525");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/:"), "file:///%3A");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/home"), "file:///home");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("/home/darin"), "file:///home/darin");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (":"), "http://:");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("::"), "http://::");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (":://:://:::::::::::::::::"), "http://:://:://:::::::::::::::::");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:"), "file:");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:///%20"), "file:///%20");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:///%3F"), "file:///%3F");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:///:"), "file:///:");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:///?"), "file:///?");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:///home/joe/some file"), "file:///home/joe/some file");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file://home/joe/some file"), "file://home/joe/some file");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:::::////"), "file:::::////");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("foo://foobar.txt"), "foo://foobar.txt");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("home"), "http://home");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("http://null.stanford.edu"), "http://null.stanford.edu");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("http://null.stanford.edu:80"), "http://null.stanford.edu:80");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("http://seth@null.stanford.edu:80"), "http://seth@null.stanford.edu:80");
-        NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("http://null.stanford.edu/some file"), "http://null.stanford.edu/some%20file");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:///home/joe/some file"), "file:///home/joe/some%20file");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file://home/joe/some file"), "file://home/joe/some%20file");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("foo://foobar.txt"), "foo://foobar.txt");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("::"), "::");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input (":://:://:::::::::::::::::"), ":://%3A%3A//%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A%3A");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("file:::::////"), "file:::::////");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("http:::::::::"), "http:::::::::");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("www.eazel.com"), "http://www.eazel.com");
+        NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_from_input ("http://null.stanford.edu/some file"), "http://null.stanford.edu/some file");
 
-
-	/* nautilus_handle_trailing_slashes */
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_handle_trailing_slashes ("file:///////"), "file:///");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_handle_trailing_slashes ("file://foo/"), "file://foo");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_handle_trailing_slashes ("file://foo"), "file://foo");
@@ -1250,10 +1297,9 @@ nautilus_self_check_file_utilities (void)
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_handle_trailing_slashes ("http://le-hacker.org/dir//////"), "http://le-hacker.org/dir/");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_handle_trailing_slashes ("http://le-hacker.org/////"), "http://le-hacker.org/");
 
-
 	/* nautilus_make_uri_canonical */
 
-	/* FIXME: this is a bizarre result from an empty string */
+	/* FIXME bugzilla.eazel.com 5072: this is a bizarre result from an empty string */
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical (""), "file:///");
 	
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("file:/"), "file:///");
@@ -1267,9 +1313,9 @@ nautilus_self_check_file_utilities (void)
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("http://le-hackeur.org/dir"), "http://le-hackeur.org/dir");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("http://le-hackeur.org/dir/"), "http://le-hackeur.org/dir/");
 
-	/* FIXME: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
+	/* FIXME bugzilla.eazel.com 5068: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("search://[file://]file_name contains stuff"), "search://[file/]file_name contains stuff");
-#ifdef EAZEL_SERVICES	
+#ifdef EAZEL_SERVICES
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("eazel-services:/~turtle"), "eazel-services:///~turtle");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("eazel-services:///~turtle"), "eazel-services:///~turtle");
 #endif
@@ -1359,11 +1405,10 @@ nautilus_self_check_file_utilities (void)
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("file:"), "file:");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("trash"), "file:///trash");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("glorp:"), "glorp:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("TRASH:XXX"), "trash:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("trash:xxx"), "trash:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("GNOME-TRASH:XXX"), "trash:");
-	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("gnome-trash:xxx"), "trash:");
-
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("TRASH:XXX"), NAUTILUS_TRASH_URI);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("trash:xxx"), NAUTILUS_TRASH_URI);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("GNOME-TRASH:XXX"), NAUTILUS_TRASH_URI);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("gnome-trash:xxx"), NAUTILUS_TRASH_URI);
 
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_make_uri_canonical ("pipe:gnome-info2html2 as"), "pipe:gnome-info2html2 as");
 }
