@@ -268,7 +268,8 @@ nautilus_gradient_parse_one_color_spec (const char *spec, int *percent, const ch
 	return result;
 }
 
-/* FIXME anyone using nautilus_gradient_get_start_color_spec or
+/* FIXME bugzilla.eazel.com 5076:
+ * anyone using nautilus_gradient_get_start_color_spec or
  * nautilus_gradient_get_end_color_spec is assuming the gradient
  * is 2 colors which is questionable.
  * 
@@ -496,6 +497,23 @@ nautilus_gdk_rgb_to_color (const guint32 color)
 	return result;
 }
 
+/**
+ * nautilus_gdk_rgb_to_color_spec
+ * @color: a gdk_rgb style value.
+ *
+ * Converts from a gdk_rgb value style to a string color spec.
+ * The gdk_rgb color alpha channel is ignored.
+ * 
+ * Return value: a newly allocated color spec.
+ */
+char *
+nautilus_gdk_rgb_to_color_spec (const guint32 color)
+{
+	return g_strdup_printf("rgb:%04hX/%04hX/%04hX",
+			       NAUTILUS_RGBA_COLOR_GET_R (color) * 65535,
+			       NAUTILUS_RGBA_COLOR_GET_G (color) * 65535,
+			       NAUTILUS_RGBA_COLOR_GET_B (color) * 65535);
+}
 
 static guint32
 nautilus_shift_color_component (guchar component, float shift_by)
@@ -540,71 +558,80 @@ nautilus_rgb_shift_color (guint32 color, float shift_by)
 }
 
 /**
- * nautilus_gdk_font_equal
- * @font_a_null_allowed: A font or NULL.
- * @font_b_null_allowed: A font or NULL.
- *
- * Calls gdk_font_equal, unless one of the fonts is NULL.
+ * nautilus_gdk_color_is_dark:
+ * 
+ * Return true if the given color is `dark'
  */
 gboolean
-nautilus_gdk_font_equal (GdkFont *font_a_null_allowed,
-			 GdkFont *font_b_null_allowed)
+nautilus_gdk_color_is_dark (GdkColor *color)
 {
-	if (font_a_null_allowed == NULL) {
-		return font_b_null_allowed == NULL;
-	}
-	if (font_b_null_allowed == NULL) {
-		return FALSE;
-	}
-	return gdk_font_equal (font_a_null_allowed, font_b_null_allowed);
+	int intensity;
+
+	intensity = (((color->red >> 8) * 77)
+		     + ((color->green >> 8) * 150)
+		     + ((color->blue >> 8) * 28)) >> 8;
+
+	return intensity < 128;
 }
 
-GdkFont *
-nautilus_get_largest_fitting_font (const char *text_to_format, int width, const char *font_template)
+/**
+ * nautilus_gdk_choose_foreground_color:
+ *
+ * Select a foreground color given that BACKGROUND is the background
+ * color. If the PREFERRED color has a high enough contrast with
+ * BACKGROUND, use it, else use one of black or white, depending on
+ * the darkness of BACKGROUND.
+ *
+ * The selected color is stored in PREFERRED.
+ */
+void
+nautilus_gdk_choose_foreground_color (GdkColor *preferred,
+				      GdkColor *background)
 {
-	int font_index, this_width;
-	char *font_name;
-	/*const int font_sizes[5] = { 28, 24, 18, 14, 12 };*/
-	const int font_sizes[4] = { 20, 18, 14, 12 };
-	GdkFont *candidate_font;
-	char *alt_text_to_format = NULL;
-	char *temp_str;
-	char *cr_pos;
+	gboolean preferred_is_dark, background_is_dark;
 
-	temp_str = g_strdup (text_to_format == NULL ? "" : text_to_format);
-	cr_pos = strchr (temp_str, '\n');
-	if (cr_pos != NULL) {
-		*cr_pos = '\0';
-		alt_text_to_format = cr_pos + 1;
-	}
-	
-	candidate_font = NULL;
-	for (font_index = 0; font_index < NAUTILUS_N_ELEMENTS (font_sizes); font_index++) {
-		if (candidate_font != NULL) {
-			gdk_font_unref (candidate_font);
-		}
-		
-		font_name = g_strdup_printf (font_template, font_sizes[font_index]);
-		candidate_font = gdk_fontset_load (font_name);
-		g_free (font_name);
+	preferred_is_dark = nautilus_gdk_color_is_dark (preferred);
+	background_is_dark = nautilus_gdk_color_is_dark (background);
 
-		if (candidate_font != NULL) {
-			this_width = gdk_string_width (candidate_font, temp_str);
-			if (alt_text_to_format != NULL) {
-				int alt_width = gdk_string_width (candidate_font, alt_text_to_format);
-				if (this_width <= width && alt_width <= width) {
-					break;
-				}
-			} else {
-				if (this_width <= width) {
-					break;
-				}
-			}
+	if (preferred_is_dark == background_is_dark) {
+		/* Colors are too similar, so choose a different fg.
+		 * Currently hardcoded to use either white or black.
+		 */
+		if (preferred_is_dark) {
+			preferred->red = 65535;
+			preferred->green = 65535;
+			preferred->blue = 65535;
+		} else {
+			preferred->red = 0;
+			preferred->green = 0;
+			preferred->blue = 0;
 		}
 	}
-	
-	g_free (temp_str);
-	return candidate_font;
+}
+
+/**
+ * nautilus_gdk_gc_choose_foreground_color:
+ *
+ * Use nautilus_gdk_color_choose_foreground_color () to set the
+ * foreground color of GC to something suitable, given that BACKGROUND
+ * will be the background color and PREFERRED is the preferred color.
+ *
+ * Uses GdkRGB to install the color value.
+ */
+void
+nautilus_gdk_gc_choose_foreground_color (GdkGC *gc,
+					 GdkColor *preferred,
+					 GdkColor *background)
+{
+	GdkColor temp;
+	guint32 rgb;
+
+	temp = *preferred;
+	nautilus_gdk_choose_foreground_color (&temp, background);
+	rgb = nautilus_gdk_color_to_rgb (&temp);
+
+	gdk_rgb_init ();
+	gdk_rgb_gc_set_foreground (gc, rgb);
 }
 
 /**
@@ -634,17 +661,41 @@ nautilus_stipple_bitmap ()
 void 
 nautilus_gdk_window_bring_to_front (GdkWindow *window)
 {
-	gdk_window_raise (window);
+	/* This takes care of un-iconifying the window and
+	 * raising it if needed.
+	 */
+	gdk_window_show (window);
 
+	/* If the window was already showing, it would not have
+	 * the focus at this point. Do a little X trickery to
+	 * ensure it is focused.
+	 */
+	nautilus_gdk_window_focus (window, GDK_CURRENT_TIME);
+}
+
+void
+nautilus_gdk_window_focus (GdkWindow *window, guint32 timestamp)
+{
 	gdk_error_trap_push ();
 	XSetInputFocus (GDK_DISPLAY (),
-	 	GDK_WINDOW_XWINDOW (window),
-	    	RevertToPointerRoot,
-	     	GDK_CURRENT_TIME);
+			GDK_WINDOW_XWINDOW (window),
+			RevertToNone,
+			timestamp);
 	gdk_flush();
 	gdk_error_trap_pop ();
 }
 
+void
+nautilus_gdk_window_set_wm_protocols (GdkWindow *window,
+				      GdkAtom *protocols,
+				      int nprotocols)
+{
+	/* This relies on the identity mapping from GdkAtoms to X Atoms */
+	XSetWMProtocols (GDK_WINDOW_XDISPLAY (window),
+			 GDK_WINDOW_XWINDOW (window),
+			 protocols, nprotocols);
+}
+
 void
 nautilus_set_mini_icon (GdkWindow *window,
 			GdkPixmap *pixmap,
@@ -669,80 +720,6 @@ nautilus_set_mini_icon (GdkWindow *window,
                              (guchar *) data, 2);
 }
 
-/**
- * nautilus_gdk_font_get_bold
- * @plain_font: A font.
- * Returns: A bold variant of @plain_font or NULL.
- *
- * Tries to find a bold flavor of a given font. Returns NULL if none is available.
- */
-GdkFont *
-nautilus_gdk_font_get_bold (const GdkFont *plain_font)
-{
-	const char *plain_name;
-	const char *scanner;
-	char *bold_name;
-	int count;
-	GSList *p;
-	GdkFont *result;
-	GdkFontPrivate *private_plain;
-
-	private_plain = (GdkFontPrivate *)plain_font;
-
-	if (private_plain->names == NULL) {
-		return NULL;
-	}
-
-
-	/* -foundry-family-weight-slant-sel_width-add-style-pixels-points-hor_res-ver_res-spacing-average_width-char_set_registry-char_set_encoding */
-
-	bold_name = NULL;
-	for (p = private_plain->names; p != NULL; p = p->next) {
-		plain_name = (const char *)p->data;
-		scanner = plain_name;
-
-		/* skip past foundry and family to weight */
-		for (count = 2; count > 0; count--) {
-			scanner = strchr (scanner + 1, '-');
-			if (!scanner) {
-				break;
-			}
-		}
-
-		if (!scanner) {
-			/* try the other names in the list */
-			continue;
-		}
-		g_assert (*scanner == '-');
-
-		/* copy "-foundry-family-" over */
-		scanner++;
-		bold_name = g_strndup (plain_name, scanner - plain_name);
-
-		/* skip weight */
-		scanner = strchr (scanner, '-');
-		g_assert (scanner != NULL);
-
-		/* FIXME bugzilla.eazel.com 2558:
-		 * some fonts have demibold, etc. instead. We should be able to figure out
-		 * which they are and use them here.
-		 */
-
-		/* add "bold" and copy everything past weight over */
-		bold_name = g_strconcat (bold_name, "bold", scanner, NULL);
-		break;
-	}
-	
-	if (bold_name == NULL) {
-		return NULL;
-	}
-	
-	result = gdk_fontset_load (bold_name);
-	g_free (bold_name);
-
-	return result;
-}
-
 GdkGC *
 nautilus_gdk_create_copy_area_gc (GdkWindow	*window)
 {
@@ -802,6 +779,82 @@ nautilus_gdk_window_set_wm_hints_input (GdkWindow *window, gboolean status)
 	XFree (wm_hints);
 }
 
+void
+nautilus_gdk_window_set_invisible_cursor (GdkWindow *window)
+{
+	XColor foreColor, backColor;
+	GdkWindowPrivate *window_private;
+	Pixmap sourcePixmap, maskPixmap;
+	Cursor xcursor;
+
+	char invisible_cursor_bits[]      = {0x0};
+	char invisible_cursor_mask_bits[] = {0x0};
+
+	foreColor.pixel = 0L;
+	foreColor.red = foreColor.green = foreColor.blue = 0;
+	foreColor.flags = DoRed | DoGreen | DoBlue;
+
+	backColor.pixel = 255L;
+	backColor.red = backColor.green = backColor.blue = 65535;
+	backColor.flags = DoRed | DoGreen | DoBlue;
+
+	window_private = (GdkWindowPrivate *) window;
+  
+	sourcePixmap = XCreateBitmapFromData (window_private->xdisplay, window_private->xwindow,
+					            		  invisible_cursor_bits, 1, 1);
+	g_assert (sourcePixmap != 0);
+
+	maskPixmap = XCreateBitmapFromData (window_private->xdisplay, window_private->xwindow,
+					          			invisible_cursor_mask_bits, 1, 1);
+	g_assert (maskPixmap != 0);
+
+	xcursor = XCreatePixmapCursor (window_private->xdisplay, sourcePixmap, maskPixmap,
+					      		   &foreColor, &backColor, 0, 0);
+
+	XFreePixmap (window_private->xdisplay, sourcePixmap);
+	XFreePixmap (window_private->xdisplay, maskPixmap);
+
+	XDefineCursor (window_private->xdisplay, window_private->xwindow, xcursor);
+}
+
+NautilusGdkGeometryFlags
+nautilus_gdk_parse_geometry (const char *string, int *x_return, int *y_return,
+			     guint *width_return, guint *height_return)
+{
+	int x11_flags;
+	NautilusGdkGeometryFlags gdk_flags;
+
+	g_return_val_if_fail (string != NULL, NAUTILUS_GDK_NO_VALUE);
+	g_return_val_if_fail (x_return != NULL, NAUTILUS_GDK_NO_VALUE);
+	g_return_val_if_fail (y_return != NULL, NAUTILUS_GDK_NO_VALUE);
+	g_return_val_if_fail (width_return != NULL, NAUTILUS_GDK_NO_VALUE);
+	g_return_val_if_fail (height_return != NULL, NAUTILUS_GDK_NO_VALUE);
+
+	x11_flags = XParseGeometry (string, x_return, y_return,
+				    width_return, height_return);
+
+	gdk_flags = NAUTILUS_GDK_NO_VALUE;
+	if (x11_flags & XValue) {
+		gdk_flags |= NAUTILUS_GDK_X_VALUE;
+	}
+	if (x11_flags & YValue) {
+		gdk_flags |= NAUTILUS_GDK_Y_VALUE;
+	}
+	if (x11_flags & WidthValue) {
+		gdk_flags |= NAUTILUS_GDK_WIDTH_VALUE;
+	}
+	if (x11_flags & HeightValue) {
+		gdk_flags |= NAUTILUS_GDK_HEIGHT_VALUE;
+	}
+	if (x11_flags & XNegative) {
+		gdk_flags |= NAUTILUS_GDK_X_NEGATIVE;
+	}
+	if (x11_flags & YNegative) {
+		gdk_flags |= NAUTILUS_GDK_Y_NEGATIVE;
+	}
+
+	return gdk_flags;
+}
 
 #if ! defined (NAUTILUS_OMIT_SELF_CHECK)
 
