@@ -30,8 +30,11 @@
 #include "nautilus-lib-self-check-functions.h"
 #include "nautilus-string.h"
 #include <ctype.h>
-#include <sys/time.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include <stdlib.h>
+#include <math.h>
+#include <sys/time.h>
 
 /* Legal conversion specifiers, as specified in the C standard. */
 #define C_STANDARD_STRFTIME_CHARACTERS "aAbBcdHIjmMpSUwWxXyYZ"
@@ -43,7 +46,7 @@ typedef struct {
 	gboolean keys_known_to_be_strings;
 } HashTableToFree;
 
-GList *hash_tables_to_free_at_exit;
+static GList *hash_tables_to_free_at_exit;
 
 /* We will need this for nautilus_unsetenv if there is no unsetenv. */
 #if !defined (HAVE_SETENV)
@@ -367,57 +370,29 @@ nautilus_g_str_list_equal (GList *list_a, GList *list_b)
 GList *
 nautilus_g_str_list_copy (GList *list)
 {
-	GList *p, *result;
+	GList *node, *result;
 
 	result = NULL;
 	
-	if (list == NULL) {
-		return NULL;
-	}
-
-	for (p = g_list_last (list); p != NULL; p = p->prev) {
-		result = g_list_prepend (result, g_strdup (p->data));
+	for (node = g_list_last (list); node != NULL; node = node->prev) {
+		result = g_list_prepend (result, g_strdup (node->data));
 	}
 	return result;
 }
 
-
 /**
- * nautilus_g_str_list_sort
+ * nautilus_g_str_list_alphabetize
  *
- * Sort a list of strings using strcmp.
+ * Sort a list of strings using locale-sensitive rules.
  *
  * @list: List of strings and/or NULLs.
  * 
  * Return value: @list, sorted.
  **/
 GList *
-nautilus_g_str_list_sort (GList *list)
+nautilus_g_str_list_alphabetize (GList *list)
 {
-	return g_list_sort (list, nautilus_str_compare);
-}
-
-/**
- * nautilus_g_str_list_sort_case_insensitive
- *
- * Sort a list of strings using g_strcasecmp.
- *
- * @list: List of strings and/or NULLs.
- * 
- * Return value: @list, sorted.
- **/
-
-static int
-compare_strings_case_breaks_ties (gconstpointer str_a,
-				  gconstpointer str_b)
-{
-	return nautilus_strcmp_case_breaks_ties (str_a, str_b);
-}
-
-GList *
-nautilus_g_str_list_sort_case_insensitive (GList *list)
-{
-	return g_list_sort (list, compare_strings_case_breaks_ties);
+	return g_list_sort (list, (GCompareFunc) nautilus_strcoll);
 }
 
 /**
@@ -936,8 +911,8 @@ destroy_deep_helper (gpointer key, gpointer value, gpointer data)
 
 void
 nautilus_g_hash_table_destroy_deep_custom (GHashTable *hash_table,
-					GFunc key_free_func, gpointer key_free_data,
-					GFunc value_free_func, gpointer value_free_data)
+					   GFunc key_free_func, gpointer key_free_data,
+					   GFunc value_free_func, gpointer value_free_data)
 {
 	HashTableFreeFuncs free_funcs;
 	
@@ -1022,6 +997,21 @@ nautilus_shell_quote (const char *string)
 	return quoted_str;
 }
 
+int nautilus_g_round (double d)
+{
+	double val = floor (d + .5);
+
+	/* The tests are needed because the result of floating-point to integral
+	 * conversion is undefined if the floating point value is not representable
+	 * in the new type. E.g. the magnititude is too large or a negative
+	 * floating-point value being converted to an unsigned.
+	 */
+	g_return_val_if_fail (val <= INT_MAX, INT_MAX);
+	g_return_val_if_fail (val >= INT_MIN, INT_MIN);
+
+	return val;
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 static void 
@@ -1194,9 +1184,14 @@ nautilus_self_check_glib_extensions (void)
 	NAUTILUS_CHECK_STRING_RESULT (test_strftime (huge_string, 2000, 1, 1, 0, 0, 0), huge_string);
 	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%%", 2000, 1, 1, 1, 0, 0), "%");
 	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%%%%", 2000, 1, 1, 1, 0, 0), "%%");
-	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), "01/01/00, 01:00 AM");
-	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), "1/1/00, 1:00 AM");
-	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), " 1/ 1/00,  1:00 AM");
+	/* localizers: These strings are part of the strftime
+	 * self-check code and must be changed to match what strtfime
+	 * yields -- usually just omitting the AM part is all that's
+	 * needed.
+	 */
+	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), _("01/01/00, 01:00 AM"));
+	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), _("1/1/00, 1:00 AM"));
+	NAUTILUS_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), _(" 1/ 1/00,  1:00 AM"));
 
 	g_free (huge_string);
 
