@@ -29,29 +29,14 @@
 
 BEGIN_GNOME_DECLS
 
-/*
- * The following preferences exist independently of the user level. 
- *
- * Note that the configuration path is fully qualified 
- */
-
 /* Which theme is active */
-#define NAUTILUS_PREFERENCES_THEME				"/apps/nautilus/preferences/theme"
+#define NAUTILUS_PREFERENCES_THEME				"preferences/theme"
 /* Which text attributes appear beneath icon names */
-#define NAUTILUS_PREFERENCES_ICON_CAPTIONS			"/apps/nautilus/icon_view/captions"
+#define NAUTILUS_PREFERENCES_ICON_CAPTIONS			"icon_view/captions"
 /* How wide the sidebar is (or how wide it will be when expanded) */
-#define NAUTILUS_PREFERENCES_SIDEBAR_WIDTH  			"/apps/nautilus/preferences/sidebar_width"
+#define NAUTILUS_PREFERENCES_SIDEBAR_WIDTH  			"preferences/sidebar_width"
 /* Keep track of the sound playing process */
-#define NAUTILUS_PREFERENCES_CURRENT_SOUND_STATE		"/apps/nautilus/preferences/sound_state"
-/* Show the desktop window or not */
-
-/*
- * The following preferences are coupled to the user level.
- *
- * Note that the configuration path does include the nautilus gconf 
- * prefix.  The nautilus_preferences_* api will fill in the missing
- * prefix according to the current user level.
- */
+#define NAUTILUS_PREFERENCES_CURRENT_SOUND_STATE		"preferences/sound_state"
 
 /* Window options */
 #define NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW			"preferences/window_always_new"
@@ -66,6 +51,7 @@ BEGIN_GNOME_DECLS
 #define NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES  		"preferences/show_hidden_files"
 #define NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES  		"preferences/show_backup_files"
 #define NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS			"preferences/show_special_flags"
+#define NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES         "sidebar-panels/tree/show_only_directories"
 
 /* Navigation  */
 #define NAUTILUS_PREFERENCES_HOME_URI                 		"preferences/home_uri"
@@ -92,8 +78,8 @@ BEGIN_GNOME_DECLS
 #define NAUTILUS_PREFERENCES_START_WITH_SIDEBAR		 	"preferences/start_with_sidebar"
 
 /* The sidebar panel preferences are computed from their oafids, which aren't known at
- * compile time. We publish the namespace for all of them so interested parties can
- * monitor changes to all of them collectively, without having to know any oafids. 
+ * compile time. We publish the namespace so that interested parties can monitor changes
+ * to all of them collectively, without having to know the exact oaf iids.
  */
 #define NAUTILUS_PREFERENCES_SIDEBAR_PANELS_NAMESPACE		"sidebar-panels"
 
@@ -123,6 +109,7 @@ typedef enum
 #define NAUTILUS_PREFERENCES_SHOW_TEXT_IN_ICONS		"preferences/show_icon_text"
 #define NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS	"preferences/show_image_thumbnails"
 #define NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA	"preferences/use_public_metadata"
+#define NAUTILUS_PREFERENCES_PREVIEW_SOUND		"preferences/preview_sound"
 
 typedef enum
 {
@@ -134,7 +121,6 @@ void   nautilus_global_preferences_initialize                                 (v
 void   nautilus_global_preferences_show_dialog                                (void);
 void   nautilus_global_preferences_hide_dialog                                (void);
 void   nautilus_global_preferences_set_dialog_title                           (const char *title);
-void   nautilus_global_preferences_dialog_update                              (void);
 
 /* Sidebar */
 GList *nautilus_global_preferences_get_enabled_sidebar_panel_view_identifiers (void);
