@@ -90,58 +90,6 @@ nautilus_gnome_canvas_world_to_canvas_rectangle (GnomeCanvas *canvas,
 			  &canvas_rect->y1);
 }
 
-gboolean
-nautilus_art_irect_contains_irect (const ArtIRect *outer_rect,
-				   const ArtIRect *inner_rect)
-{
-	g_return_val_if_fail (outer_rect != NULL, FALSE);
-	g_return_val_if_fail (inner_rect != NULL, FALSE);
-
-	return outer_rect->x0 <= inner_rect->x0
-		&& outer_rect->y0 <= inner_rect->y0
-		&& outer_rect->x1 >= inner_rect->x1
-		&& outer_rect->y1 >= inner_rect->y1; 
-}
-
-gboolean
-nautilus_art_irect_hits_irect (const ArtIRect *rect_a,
-			       const ArtIRect *rect_b)
-{
-	ArtIRect intersection;
-
-	g_return_val_if_fail (rect_a != NULL, FALSE);
-	g_return_val_if_fail (rect_b != NULL, FALSE);
-
-	art_irect_intersect (&intersection, rect_a, rect_b);
-	return !art_irect_empty (&intersection);
-}
-
-gboolean
-nautilus_art_irect_equal (const ArtIRect *rect_a,
-			  const ArtIRect *rect_b)
-{
-	g_return_val_if_fail (rect_a != NULL, FALSE);
-	g_return_val_if_fail (rect_b != NULL, FALSE);
-
-	return rect_a->x0 == rect_b->x0
-		&& rect_a->y0 == rect_b->y0
-		&& rect_a->x1 == rect_b->x1
-		&& rect_a->y1 == rect_b->y1;
-}
-
-gboolean
-nautilus_art_drect_equal (const ArtDRect *rect_a,
-			  const ArtDRect *rect_b)
-{
-	g_return_val_if_fail (rect_a != NULL, FALSE);
-	g_return_val_if_fail (rect_b != NULL, FALSE);
-
-	return rect_a->x0 == rect_b->x0
-		&& rect_a->y0 == rect_b->y0
-		&& rect_a->x1 == rect_b->x1
-		&& rect_a->y1 == rect_b->y1;
-}
-
 void
 nautilus_gnome_canvas_item_get_current_canvas_bounds (GnomeCanvasItem *item,
 						      ArtIRect *bounds)
@@ -195,8 +143,22 @@ nautilus_gnome_canvas_item_get_world_bounds (GnomeCanvasItem *item,
 	}
 }
 
+void
+nautilus_gnome_canvas_item_get_canvas_bounds (GnomeCanvasItem *item,
+					      ArtIRect *canvas_bounds)
+{
+	ArtDRect world_bounds;
+
+	nautilus_gnome_canvas_item_get_world_bounds
+		(item, &world_bounds);
+	nautilus_gnome_canvas_world_to_canvas_rectangle
+		(item->canvas, &world_bounds, canvas_bounds);
+}
+
 static void
-nautilus_gnome_canvas_draw_pixbuf_helper (art_u8 *dst, int dst_rowstride, const art_u8 *src, int src_rowstride, int copy_width, int copy_height)
+nautilus_gnome_canvas_draw_pixbuf_helper (art_u8 *dst, int dst_rowstride,
+					  const art_u8 *src, int src_rowstride,
+					  int copy_width, int copy_height)
 {
 	art_u8 *dst_limit = dst + copy_height * dst_rowstride;
 	int dst_bytes_per_row = copy_width * 3;
@@ -209,7 +171,9 @@ nautilus_gnome_canvas_draw_pixbuf_helper (art_u8 *dst, int dst_rowstride, const
 }
 
 static void
-nautilus_gnome_canvas_draw_pixbuf_helper_alpha (art_u8 *dst, int dst_rowstride, const art_u8 *src, int src_rowstride, int copy_width, int copy_height)
+nautilus_gnome_canvas_draw_pixbuf_helper_alpha (art_u8 *dst, int dst_rowstride,
+						const art_u8 *src, int src_rowstride,
+						int copy_width, int copy_height)
 {
 	art_u8 *dst_limit = dst + copy_height * dst_rowstride;
 	int dst_bytes_per_row = copy_width * 3;
@@ -478,7 +442,7 @@ max_open_files (void)
 }
 
 static int 
-nautilus_gnome_terminal_shell_execute (const char *shell, const char *command)
+nautilus_gnome_terminal_shell_execute (const char *command)
 {
 	struct sigaction ignore, save_intr, save_quit, save_stop;
 	int status, i;
@@ -530,7 +494,7 @@ nautilus_gnome_terminal_shell_execute (const char *shell, const char *command)
 		
 		pid = fork ();
 		if (pid == 0){
-			execl (shell, shell, "-c", command, (char *) 0);
+			execl ("/bin/sh", "/bin/sh", "-c", command, (char *) 0);
 			/* See note below for why we use _exit () */
 			_exit (127);		/* Exec error */
 		}
@@ -553,24 +517,19 @@ nautilus_gnome_open_terminal (const char *command)
 {
 	char *terminal_path;
 	char *terminal_path_with_flags;
-	char *shell;
 	char *terminal_flags;
-	gboolean quote_all;
+	gboolean using_gnome_terminal;
 	char *command_line;
 
 
-	quote_all = FALSE;
-
-	/* figure out whichever shell we are using */
-    	shell = gnome_util_user_shell ();
+	using_gnome_terminal = FALSE;
 
 	/* Look up a well-known terminal app */
 	terminal_path = gnome_is_program_in_path ("gnome-terminal");
 	if (terminal_path != NULL) {
-		/* apparently gnome-terminal needs it's input nicely quoted and the other
-		 * terminals don't
+		/* gnome-terminal has different command-line options than the others
 		 */
-		quote_all = TRUE;
+		using_gnome_terminal = TRUE;
 	}
 	
 	if (terminal_path == NULL) {
@@ -596,29 +555,29 @@ nautilus_gnome_open_terminal (const char *command)
 	
 	if (terminal_path == NULL){
 		g_message (" Could not start a terminal ");
-	} else if (command){
-		if (quote_all) {
-			command_line = g_strconcat (terminal_path, " -e '", command, "'", NULL);
+	} else if (command) {
+		if (using_gnome_terminal) {
+			command_line = g_strconcat (terminal_path, " -x ", command, NULL);
 		} else {
 			command_line = g_strconcat (terminal_path, " -e ", command, NULL);
 		}
-		nautilus_gnome_terminal_shell_execute (shell, command_line);
+		
+		nautilus_gnome_terminal_shell_execute (command_line);
 		g_free (command_line);
 	} else {
-	        if (quote_all) {
+	        if (using_gnome_terminal) {
 			terminal_flags = " --login";
 			terminal_path_with_flags = g_strconcat (terminal_path, terminal_flags,  NULL);
-		        nautilus_gnome_terminal_shell_execute (shell, terminal_path_with_flags);
+		        nautilus_gnome_terminal_shell_execute (terminal_path_with_flags);
 		        g_free (terminal_path_with_flags);
 		} else {
 			terminal_flags = " -ls";
 			terminal_path_with_flags = g_strconcat (terminal_path, terminal_flags, NULL);
-			nautilus_gnome_terminal_shell_execute (shell, terminal_path_with_flags);
+			nautilus_gnome_terminal_shell_execute (terminal_path_with_flags);
 		}
 	}
 
 
-	g_free (shell);
 	g_free (terminal_path);
 }
 
