@@ -22,14 +22,18 @@
    Boston, MA 02111-1307, USA.
 
    Authors: John Sullivan <sullivan@eazel.com>
+            Ramiro Estrugo <ramiro@eazel.com>
 */
 
 #include <config.h>
 #include "nautilus-gtk-extensions.h"
 #include "nautilus-gdk-extensions.h"
+#include "nautilus-gdk-font-extensions.h"
 
 #include <gtk/gtkselection.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkrc.h>
+#include <libgnomeui/gnome-dialog.h>
 #include <libgnomeui/gnome-geometry.h>
 #include "nautilus-glib-extensions.h"
 #include "nautilus-string.h"
@@ -50,6 +54,11 @@
 #define MINIMUM_ON_SCREEN_WIDTH		100
 #define MINIMUM_ON_SCREEN_HEIGHT	100
 
+/* GTK buttons cram the text too close to the edge of the buttons by default.
+ * This is the standard padding used to make them look non-crammed.
+ */
+#define NAUTILUS_STANDARD_BUTTON_PADDING 1
+
 static gboolean
 finish_button_activation (gpointer data)
 {
@@ -57,10 +66,17 @@ finish_button_activation (gpointer data)
 	
 	button = GTK_BUTTON (data);
 
-	if (!button->in_button) {
+	if (!GTK_OBJECT_DESTROYED (button) && !button->in_button) {
 		gtk_button_clicked (button);
 	}
-	gtk_button_released (button);
+
+	/* Check again--button can be destroyed during call to gtk_button_clicked */
+	if (!GTK_OBJECT_DESTROYED (button)) {
+		gtk_button_released (button);
+	}
+
+	/* this was ref'd in nautilus_gtk_button_auto_click */
+	gtk_object_unref (GTK_OBJECT(button));
 
 	return FALSE;	
 }
@@ -93,6 +109,10 @@ nautilus_gtk_button_auto_click (GtkButton *button)
 	 * In practice the timeout is short enough that this probably
 	 * isn't a problem.
 	 */
+
+	/* This is unref'ed in finish_button_activation */
+	gtk_object_ref (GTK_OBJECT(button));
+
 	g_timeout_add (BUTTON_AUTO_HIGHLIGHT_MILLISECONDS, 
 		       finish_button_activation, button);
 }
@@ -116,6 +136,22 @@ nautilus_gtk_button_set_padding (GtkButton *button, int pad_amount)
 			      pad_amount);
 }
 
+/**
+ * nautilus_gtk_button_set_standard_padding
+ * 
+ * Adds the standard amount of padding around the contained widget in the button 
+ * (typically the label). Use this rather than nautilus_gtk_button_set_padding
+ * unless you have a specific reason to use a non-standard amount.
+ * @button: a GtkButton
+ **/
+void
+nautilus_gtk_button_set_standard_padding (GtkButton *button)
+{
+	g_return_if_fail (GTK_IS_BUTTON (button));
+
+	nautilus_gtk_button_set_padding (button, NAUTILUS_STANDARD_BUTTON_PADDING);
+}
+
 
 /**
  * nautilus_gtk_clist_get_first_selected_row:
@@ -262,35 +298,6 @@ guint nautilus_gtk_signal_connect_free_data (GtkObject *object,
 		(object, name, func, data, g_free);
 }
 
-/**
- * nautilus_gtk_window_hide_retain_geometry:
- * 
- * Hide a GtkWindow such that when reopened it will be in the same
- * place it is now.
- * @window: The GtkWindow to be hidden.
- **/
-static void
-nautilus_gtk_window_hide_retain_geometry (GtkWindow *window) {
-	gchar *geometry_string;
-	int left, top, width, height;
-
-	g_return_if_fail (GTK_IS_WINDOW (window));
-
-	/* Save and restore position to keep it in same position when next shown. */
-
-	geometry_string = gnome_geometry_string (GTK_WIDGET (window)->window);
-    
-	gtk_widget_hide (GTK_WIDGET (window));
-
-	if (gnome_parse_geometry (geometry_string, &left, &top, &width, &height)) 
-	{
-		gtk_window_set_default_size (window, width, height);
-		gtk_widget_set_uposition (GTK_WIDGET (window), left, top);
-	}
-
-	g_free (geometry_string);
-}
-
 /**
  * nautilus_gtk_window_present:
  * 
@@ -298,36 +305,98 @@ nautilus_gtk_window_hide_retain_geometry (GtkWindow *window) {
  * @window: The GtkWindow to be presented to the user.
  **/
 void
-nautilus_gtk_window_present (GtkWindow *window) {
-	g_assert (GTK_IS_WINDOW (window));
+nautilus_gtk_window_present (GtkWindow *window)
+{
+	GdkWindow *gdk_window;
 
-	/* Hide first if already showing, so it will reappear on top.
-	 * This works with iconified windows as well.
+	g_return_if_fail (GTK_IS_WINDOW (window));
+
+	/* If we have no gdk window, then it's OK to just show, since
+	 * the window is new and presumably will show up in front.
 	 */
-	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) 
-	{
-		nautilus_gtk_window_hide_retain_geometry (window);
+	gdk_window = GTK_WIDGET (window)->window;
+	if (gdk_window != NULL) {
+		nautilus_gdk_window_bring_to_front (gdk_window);
 	}
-    
+
 	gtk_widget_show (GTK_WIDGET (window));
 }
 
+static int
+handle_standard_close_accelerator (GtkWindow *window, 
+				   GdkEventKey *event, 
+				   gpointer user_data)
+{
+	g_assert (GTK_IS_WINDOW (window));
+	g_assert (event != NULL);
+	g_assert (user_data == NULL);
+
+	if (nautilus_gtk_window_event_is_close_accelerator (window, event)) {
+		if (GNOME_IS_DIALOG (window)) {
+			gnome_dialog_close (GNOME_DIALOG (window));
+		} else {
+			gtk_widget_hide (GTK_WIDGET (window));
+		}
+		gtk_signal_emit_stop_by_name 
+			(GTK_OBJECT (window), "key_press_event");
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+/**
+ * nautilus_gtk_window_event_is_close_accelerator:
+ * 
+ * Tests whether a key event is the standard window close accelerator.
+ * Not needed for clients that use nautilus_gtk_window_set_up_close_accelerator;
+ * use only if you must set up your own key_event handler for your own reasons.
+ **/
+gboolean
+nautilus_gtk_window_event_is_close_accelerator (GtkWindow *window, GdkEventKey *event)
+{
+	g_return_val_if_fail (GTK_IS_WINDOW (window), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	if (event->state & GDK_CONTROL_MASK) {
+		/* Note: menu item equivalents are case-sensitive, so we will
+		 * be case-sensitive here too.
+		 */		
+		if (event->keyval == NAUTILUS_STANDARD_CLOSE_WINDOW_CONTROL_KEY) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;	
+}
+
+/**
+ * nautilus_gtk_window_set_up_close_accelerator:
+ * 
+ * Sets up the standard keyboard equivalent to close the window.
+ * Call this for windows that don't set up a keyboard equivalent to
+ * close the window some other way, e.g. via a menu item accelerator.
+ * 
+ * @window: The GtkWindow that should be hidden when the standard
+ * keyboard equivalent is typed.
+ **/
+void
+nautilus_gtk_window_set_up_close_accelerator (GtkWindow *window)
+{
+	g_return_if_fail (GTK_IS_WINDOW (window));
+
+	gtk_signal_connect (GTK_OBJECT (window),
+			    "key_press_event",
+			    GTK_SIGNAL_FUNC (handle_standard_close_accelerator),
+			    NULL);
+}
+
 static void
-sanity_check_window_geometry (int *left, int *top, int *width, int *height)
+sanity_check_window_position (int *left, int *top)
 {
 	g_assert (left != NULL);
 	g_assert (top != NULL);
-	g_assert (width != NULL);
-	g_assert (height != NULL);
 
-	/* Pin the size of the window to the screen, so we don't end up in
-	 * a state where the window is so big essential parts of it can't
-	 * be reached (might not be necessary with all window managers,
-	 * but seems reasonable anyway).
-	 */
-	*width = MIN (*width, gdk_screen_width());
-	*height = MIN (*height, gdk_screen_height());
-	
 	/* Make sure the top of the window is on screen, for
 	 * draggability (might not be necessary with all window managers,
 	 * but seems reasonable anyway). Make sure the top of the window
@@ -350,6 +419,20 @@ sanity_check_window_geometry (int *left, int *top, int *width, int *height)
 	*left = CLAMP (*left, 0, gdk_screen_width() - MINIMUM_ON_SCREEN_WIDTH);
 }
 
+static void
+sanity_check_window_dimensions (int *width, int *height)
+{
+	g_assert (width != NULL);
+	g_assert (height != NULL);
+
+	/* Pin the size of the window to the screen, so we don't end up in
+	 * a state where the window is so big essential parts of it can't
+	 * be reached (might not be necessary with all window managers,
+	 * but seems reasonable anyway).
+	 */
+	*width = MIN (*width, gdk_screen_width());
+	*height = MIN (*height, gdk_screen_height());
+}
 
 /**
  * nautilus_gtk_window_set_initial_geometry:
@@ -360,6 +443,8 @@ sanity_check_window_geometry (int *left, int *top, int *width, int *height)
  * some sanity-checking on the passed-in values.
  * 
  * @window: A non-visible GtkWindow
+ * @geometry_flags: A NautilusGdkGeometryFlags value defining which of
+ * the following parameters have defined values
  * @left: pixel coordinate for left of window
  * @top: pixel coordinate for top of window
  * @width: width of window in pixels
@@ -367,13 +452,17 @@ sanity_check_window_geometry (int *left, int *top, int *width, int *height)
  */
 void
 nautilus_gtk_window_set_initial_geometry (GtkWindow *window, 
+					  NautilusGdkGeometryFlags geometry_flags,
 					  int left,
 					  int top,
-					  int width,
-					  int height)
+					  guint width,
+					  guint height)
 {
+	int real_left, real_top;
+
 	g_return_if_fail (GTK_IS_WINDOW (window));
-	g_return_if_fail (width > 0 && height > 0);
+	g_return_if_fail (!(geometry_flags & NAUTILUS_GDK_WIDTH_VALUE) || width > 0);
+	g_return_if_fail (!(geometry_flags & NAUTILUS_GDK_HEIGHT_VALUE) || height > 0);
 
 	/* Setting the default size doesn't work when the window is already showing.
 	 * Someday we could make this move an already-showing window, but we don't
@@ -381,10 +470,31 @@ nautilus_gtk_window_set_initial_geometry (GtkWindow *window,
 	 */
 	g_return_if_fail (!GTK_WIDGET_VISIBLE (window));
 
-	sanity_check_window_geometry (&left, &top, &width, &height);
-			    		
-	gtk_widget_set_uposition (GTK_WIDGET (window), left, top);
-	gtk_window_set_default_size (GTK_WINDOW (window), width, height);
+	if ((geometry_flags & NAUTILUS_GDK_X_VALUE) && (geometry_flags & NAUTILUS_GDK_Y_VALUE)) {
+		real_left = left;
+		real_top = top;
+
+		/* This is sub-optimal. GDK doesn't allow us to set win_gravity
+		 * to South/East types, which should be done if using negative
+		 * positions (so that the right or bottom edge of the window
+		 * appears at the specified position, not the left or top).
+		 * However it does seem to be consistent with other GNOME apps.
+		 */
+		if (geometry_flags & NAUTILUS_GDK_X_NEGATIVE) {
+			real_left = gdk_screen_width () - real_left;
+		}
+		if (geometry_flags & NAUTILUS_GDK_Y_NEGATIVE) {
+			real_top = gdk_screen_height () - real_top;
+		}
+
+		sanity_check_window_position (&real_left, &real_top);
+		gtk_widget_set_uposition (GTK_WIDGET (window), real_left, real_top);
+	}
+
+	if ((geometry_flags & NAUTILUS_GDK_WIDTH_VALUE) && (geometry_flags & NAUTILUS_GDK_HEIGHT_VALUE)) {
+		sanity_check_window_dimensions (&width, &height);
+		gtk_window_set_default_size (GTK_WINDOW (window), width, height);
+	}
 }
 
 /**
@@ -409,8 +519,9 @@ nautilus_gtk_window_set_initial_geometry_from_string (GtkWindow *window,
 					  	      guint minimum_width,
 					  	      guint minimum_height)
 {
-	int left, top, width, height;
-	gboolean parsed;
+	int left, top;
+	guint width, height;
+	NautilusGdkGeometryFlags geometry_flags;
 
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	g_return_if_fail (geometry_string != NULL);
@@ -421,18 +532,19 @@ nautilus_gtk_window_set_initial_geometry_from_string (GtkWindow *window,
 	 */
 	g_return_if_fail (!GTK_WIDGET_VISIBLE (window));
 
-	parsed = gnome_parse_geometry (geometry_string, &left, &top, &width, &height);
-
-	/* Bogus string, dude. */
-	g_return_if_fail (parsed);
+	geometry_flags = nautilus_gdk_parse_geometry (geometry_string, &left, &top, &width, &height);
 
 	/* Make sure the window isn't smaller than makes sense for this window.
 	 * Other sanity checks are performed in set_initial_geometry.
 	 */
-	width = MAX (width, minimum_width);
-	height = MAX (height, minimum_height);
+	if (geometry_flags & NAUTILUS_GDK_WIDTH_VALUE) {
+		width = MAX (width, minimum_width);
+	}
+	if (geometry_flags & NAUTILUS_GDK_HEIGHT_VALUE) {
+		height = MAX (height, minimum_height);
+	}
 
-	nautilus_gtk_window_set_initial_geometry (window, left, top, width, height);
+	nautilus_gtk_window_set_initial_geometry (window, geometry_flags, left, top, width, height);
 }
 
 /**
@@ -526,39 +638,41 @@ nautilus_truncate_text_for_menu_item (const char *text)
  * This function is more of a helper function than a gtk extension,
  * so perhaps it belongs in a different file.
  * 
- * When calling from a callback other than button_press, make sure to pass
- * 0 for button because otherwise the first button click after the button came
- * up would not be handled properly (a subtle fragility of gtk_menu_popup).
- * 
  * @menu: The menu to pop up under the mouse.
  * @offset_x: Number of pixels to displace the popup menu vertically
  * @offset_y: Number of pixels to displace the popup menu horizontally
- * @button: current button if called from button_press.
+ * @event: The event that invoked this popup menu.
  **/
 void 
-nautilus_pop_up_context_menu (GtkMenu	*menu,
-			      gint16	offset_x,
-			      gint16	offset_y,
-			      int	button)
+nautilus_pop_up_context_menu (GtkMenu	     *menu,
+			      gint16	      offset_x,
+			      gint16	      offset_y,
+			      GdkEventButton *event)
 {
 	GdkPoint offset;
+	int button;
 
 	g_return_if_fail (GTK_IS_MENU (menu));
 
 	offset.x = offset_x;
 	offset.y = offset_y;
 
-	/* We pass current time here instead of extracting it from
-	 * the event, for API simplicity. This does not seem to make
-	 * any practical difference. See man XGrabPointer for details.
+	/* The event button needs to be 0 if we're popping up this menu from
+	 * a button release, else a 2nd click outside the menu with any button
+	 * other than the one that invoked the menu will be ignored (instead
+	 * of dismissing the menu). This is a subtle fragility of the GTK menu code.
 	 */
+	button = event->type == GDK_BUTTON_RELEASE
+		? 0
+		: event->button;
+	
 	gtk_menu_popup (menu,					/* menu */
 			NULL,					/* parent_menu_shell */
 			NULL,					/* parent_menu_item */
 			nautilus_popup_menu_position_func,	/* func */
 			&offset,			        /* data */
 			button,					/* button */
-			GDK_CURRENT_TIME);			/* activate_time */
+			event->time);				/* activate_time */
 
 	gtk_object_sink (GTK_OBJECT(menu));
 }
@@ -591,7 +705,7 @@ nautilus_gtk_menu_set_item_visibility (GtkMenu *menu, int index, gboolean visibl
 	g_return_if_fail (GTK_IS_MENU (menu));
 
 	children = gtk_container_children (GTK_CONTAINER (menu));
-	g_return_if_fail (index >= 0 && index < g_list_length (children));
+	g_return_if_fail (index >= 0 && index < (int) g_list_length (children));
 
 	menu_item = GTK_WIDGET (g_list_nth_data (children, index));
 	if (visible) {
@@ -1184,6 +1298,36 @@ nautilus_gtk_container_get_first_child (GtkContainer *container)
 	return first_child;
 }
 
+typedef struct {
+	GtkCallback   callback;
+	gpointer      callback_data;
+} container_foreach_deep_callback_data;
+
+static void
+container_foreach_deep_callback (GtkWidget *child, gpointer data)
+{
+	container_foreach_deep_callback_data *deep_data;
+
+	deep_data = (container_foreach_deep_callback_data *) data;
+
+	deep_data->callback (child, deep_data->callback_data);
+
+	if (GTK_IS_CONTAINER (child)) {
+		gtk_container_foreach (GTK_CONTAINER (child), container_foreach_deep_callback, data);
+	}
+}
+
+void
+nautilus_gtk_container_foreach_deep (GtkContainer *container,
+				     GtkCallback callback,
+				     gpointer callback_data)
+{
+	container_foreach_deep_callback_data deep_data;
+	deep_data.callback = callback;
+	deep_data.callback_data = callback_data;
+	gtk_container_foreach (container, container_foreach_deep_callback, &deep_data);
+}
+
 /* We have to supply a dummy pixmap to avoid the return_if_fail in gtk_pixmap_new. */
 GtkPixmap *
 nautilus_gtk_pixmap_new_empty (void)
@@ -1236,7 +1380,6 @@ nautilus_gtk_adjustment_clamp_value (GtkAdjustment *adjustment)
  * Switches the font of label to a bold equivalent.
  * @label: The label.
  **/
-
 void
 nautilus_gtk_label_make_bold (GtkLabel *label)
 {
@@ -1255,6 +1398,56 @@ nautilus_gtk_label_make_bold (GtkLabel *label)
 	gdk_font_unref (bold_font);
 }
 
+/**
+ * nautilus_gtk_label_make_larger.
+ *
+ * Switches the font of label to a larger version of the font.
+ * @label: The label.
+ **/
+void
+nautilus_gtk_label_make_larger (GtkLabel *label,
+				guint num_steps)
+{
+	GtkStyle *style;
+	GdkFont *larger_font;
+
+	g_return_if_fail (GTK_IS_LABEL (label));
+
+	style = gtk_widget_get_style (GTK_WIDGET (label));
+
+	larger_font = nautilus_gdk_font_get_larger (style->font, num_steps);
+	if (larger_font == NULL) {
+		return;
+	}
+	nautilus_gtk_widget_set_font (GTK_WIDGET (label), larger_font);
+	gdk_font_unref (larger_font);
+}
+
+/**
+ * nautilus_gtk_label_make_smaller.
+ *
+ * Switches the font of label to a smaller version of the font.
+ * @label: The label.
+ **/
+void
+nautilus_gtk_label_make_smaller (GtkLabel *label,
+				 guint num_steps)
+{
+	GtkStyle *style;
+	GdkFont *smaller_font;
+
+	g_return_if_fail (GTK_IS_LABEL (label));
+
+	style = gtk_widget_get_style (GTK_WIDGET (label));
+
+	smaller_font = nautilus_gdk_font_get_smaller (style->font, num_steps);
+	if (smaller_font == NULL) {
+		return;
+	}
+	nautilus_gtk_widget_set_font (GTK_WIDGET (label), smaller_font);
+	gdk_font_unref (smaller_font);
+}
+
 void
 nautilus_gtk_widget_set_background_color (GtkWidget              *widget,
 					  const char		*color_spec)
@@ -1495,3 +1688,41 @@ nautilus_gtk_style_shade (GdkColor *a,
 	b->green = green * 65535.0;
 	b->blue = blue * 65535.0;
 }
+
+/**
+ * nautilus_gtk_class_name_make_like_existing_type:
+ * @class_name: The class name for the custom widget.
+ * @existing_gtk_type: The GtkType of the existing GtkWidget.
+ *
+ * Make the given class name act like an existing GtkType for
+ * gtk theme/style purposes.  This can be used by custom 
+ * widget to emulate the styles of stock Gtk widgets.  For
+ * example:
+ *
+ * nautilus_gtk_class_name_make_like_existing ("NautilusCustomButton",
+ *                                             GTK_TYPE_BUTTON);
+ *
+ *
+ * You should call this function only once from the class_initialize()
+ * method of the custom widget.
+ **/
+void
+nautilus_gtk_class_name_make_like_existing_type (const char *class_name,
+						 GtkType existing_gtk_type)
+{
+	GtkWidget *temporary;
+	GtkStyle *style;
+
+	g_return_if_fail (class_name != NULL);
+
+	temporary = gtk_widget_new (existing_gtk_type, NULL);
+	gtk_widget_ensure_style (temporary);
+		
+	style = gtk_widget_get_style (temporary);
+
+	if (style->rc_style != NULL) {
+		gtk_rc_add_widget_class_style (style->rc_style, class_name);
+	}
+		
+	gtk_widget_destroy (temporary);
+}
