@@ -39,8 +39,10 @@
 #include <libart_lgpl/art_svp_vpath.h>
 #include "nautilus-icon-private.h"
 #include "nautilus-string.h"
+#include "nautilus-art-extensions.h"
 #include "nautilus-glib-extensions.h"
 #include "nautilus-gdk-extensions.h"
+#include "nautilus-gdk-font-extensions.h"
 #include "nautilus-gdk-pixbuf-extensions.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-gtk-macros.h"
@@ -56,13 +58,14 @@
 #define EMBLEM_SPACING 2
 
 /* gap between bottom of icon and start of text box */
-#define LABEL_OFFSET 2
+#define LABEL_OFFSET 1
 
 #define MAX_TEXT_WIDTH_STANDARD 135
 #define MAX_TEXT_WIDTH_TIGHTER 80
 
 /* The list of characters that cause line breaks can be localized. */
-#define LINE_BREAK_CHARACTERS _(" -_,;.?/&")
+static const char untranslated_line_break_characters[] = N_(" -_,;.?/&");
+#define LINE_BREAK_CHARACTERS _(untranslated_line_break_characters)
 
 /* Private part of the NautilusIconCanvasItem structure. */
 struct NautilusIconCanvasItemDetails {
@@ -206,16 +209,8 @@ static void     draw_pixbuf                                (GdkPixbuf
 							    int                            y);
 static gboolean hit_test_stretch_handle                    (NautilusIconCanvasItem        *item,
 							    const ArtIRect                *canvas_rect);
-static void     draw_pixbuf_aa                             (GdkPixbuf                     *pixbuf,
-							    GnomeCanvasBuf                *buf,
-							    double                         affine[6],
-							    int                            x_offset,
-							    int                            y_offset);
 static gboolean icon_canvas_item_is_smooth                 (const NautilusIconCanvasItem  *icon_item);
 
-static void draw_dashed_rectangle_aa 			   (guchar *pixels, int rowstride, gboolean has_alpha,
-		   					    int widt, int height, int left, int right, int top, int bottom);
-
 
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusIconCanvasItem, nautilus_icon_canvas_item, GNOME_TYPE_CANVAS_ITEM)
@@ -282,6 +277,10 @@ nautilus_icon_canvas_item_initialize (NautilusIconCanvasItem *icon_item)
 
 	icon_item->details->is_renaming = FALSE;
 
+	/* invalidate cached text dimensions initially */
+	nautilus_icon_canvas_item_invalidate_label_size (icon_item);
+	
+	/* set up the default font and size */
 	icon_item->details->smooth_font_size = 12;
 	icon_item->details->smooth_font = nautilus_scalable_font_get_default_font ();
 }
@@ -337,14 +336,24 @@ pixbuf_is_acceptable (GdkPixbuf *pixbuf)
 		&& gdk_pixbuf_get_bits_per_sample (pixbuf) == 8;
 }
 
+/* invalidate the text width and height cached in the item details. */
+void
+nautilus_icon_canvas_item_invalidate_label_size (NautilusIconCanvasItem *item)
+{
+	item->details->text_width = -1;
+	item->details->text_height = -1;
+}
+
 /* Set_arg handler for the icon item. */
 static void
 nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
+	NautilusIconCanvasItem *item;
 	NautilusIconCanvasItemDetails *details;
 	GdkFont *font;
 
-	details = NAUTILUS_ICON_CANVAS_ITEM (object)->details;
+	item =  NAUTILUS_ICON_CANVAS_ITEM (object);
+	details = item->details;
 
 	switch (arg_id) {
 
@@ -355,6 +364,8 @@ nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 		g_free (details->editable_text);
 		details->editable_text = g_strdup (GTK_VALUE_STRING (*arg));
+		
+		nautilus_icon_canvas_item_invalidate_label_size (item);		
 		break;
 
 	case ARG_ADDITIONAL_TEXT:
@@ -364,6 +375,8 @@ nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 		g_free (details->additional_text);
 		details->additional_text = g_strdup (GTK_VALUE_STRING (*arg));
+		
+		nautilus_icon_canvas_item_invalidate_label_size (item);		
 		break;
 
 	case ARG_FONT:
@@ -379,6 +392,8 @@ nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 			gdk_font_unref (details->font);
 		}
 		details->font = font;
+		
+		nautilus_icon_canvas_item_invalidate_label_size (item);		
 		break;
 
 	case ARG_HIGHLIGHTED_FOR_SELECTION:
@@ -405,6 +420,7 @@ nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
         case ARG_SMOOTH_FONT:
 		nautilus_icon_canvas_item_set_smooth_font (NAUTILUS_ICON_CANVAS_ITEM (object),
 							   NAUTILUS_SCALABLE_FONT (GTK_VALUE_OBJECT (*arg)));
+		nautilus_icon_canvas_item_invalidate_label_size (item);				
 		break;
 
         case ARG_SMOOTH_FONT_SIZE:
@@ -655,12 +671,16 @@ static gboolean
 in_single_click_mode ()
 {
 	/* Perhaps this should be computed elsewhere and passed in. */
-	return nautilus_preferences_get_enum 
-		(NAUTILUS_PREFERENCES_CLICK_POLICY, NAUTILUS_CLICK_POLICY_DOUBLE) == 
-			NAUTILUS_CLICK_POLICY_SINGLE;
-
+	return nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE;
 }
 
+/* Keep these for a bit while we work on performance of draw_or_measure_label_text. */
+
+/*
+#define PERFORMANCE_TEST_DRAW_DISABLE
+#define PERFORMANCE_TEST_MEASURE_DISABLE
+*/
+
 /* Draw the text in a box, using gnomelib routines. */
 static void
 draw_or_measure_label_text (NautilusIconCanvasItem *item,
@@ -669,7 +689,7 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 			    int icon_bottom)
 {
 	NautilusIconCanvasItemDetails *details;
-	int width_so_far, height_so_far;
+	guint width_so_far, height_so_far;
 	GdkGC* gc;
 	GdkGCValues save_gc;
 	guint32 label_color;
@@ -694,6 +714,7 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 	have_additional = details->additional_text != NULL
 		&& details->additional_text[0] != '\0';
 
+
 	/* No font or no text, then do no work. */
 	if (details->font == NULL || (!have_editable && !have_additional)) {
 		details->text_height = 0;
@@ -701,6 +722,28 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		return;
 	}
 
+#if (defined PERFORMANCE_TEST_MEASURE_DISABLE && defined PERFORMANCE_TEST_DRAW_DISABLE)
+	/* don't do any drawing and fake out the width */
+	details->text_width = 80;
+	details->text_height = 20;
+	return;
+#endif
+
+#ifdef PERFORMANCE_TEST_MEASURE_DISABLE
+	if (drawable == NULL) {
+		/* fake out the width */
+		details->text_width = 80;
+		details->text_height = 20;
+		return;
+	}
+#endif
+
+#ifdef PERFORMANCE_TEST_DRAW_DISABLE
+	if (drawable != NULL) {
+		return;
+	}
+#endif
+
 	/* Combine editable and additional text for processing */
 	combined_text = g_strconcat
 		(have_editable ? details->editable_text : "",
@@ -787,17 +830,15 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 			}
 		}
 		
-		width_so_far = MAX (width_so_far, icon_text_info->width);
+		width_so_far = MAX (width_so_far, (guint) icon_text_info->width);
 		height_so_far += icon_text_info->height;
 		
 		gnome_icon_text_info_free (icon_text_info);
 	}
 	g_strfreev (pieces);
 	
-	if (needs_highlight) {
-		height_so_far += 2; /* extra slop for nicer highlighting */	
-		width_so_far += 4; /* account for emboldening, plus extra to make it look nicer */
-	}	
+	/* add slop used for highlighting, even if we're not highlighting now */
+	width_so_far += 4;
 	
 	if (drawable != NULL) {
 		/* Current calculations should match what we measured before drawing.
@@ -805,8 +846,10 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		 * before the call to draw. We might later decide to use this function
 		 * differently and change these asserts.
 		 */
-		g_assert (height_so_far == details->text_height);
-		g_assert (width_so_far == details->text_width);
+#if (defined PERFORMANCE_TEST_MEASURE_DISABLE || defined PERFORMANCE_TEST_DRAW_DISABLE)
+		g_assert ((int) height_so_far == details->text_height);
+		g_assert ((int) width_so_far == details->text_width);
+#endif
 
 		gdk_gc_set_foreground (gc, &save_gc.foreground);
 	
@@ -835,6 +878,14 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 static void
 measure_label_text (NautilusIconCanvasItem *item)
 {
+	/* check to see if the cached values are still valid; if so, there's
+	 * no work necessary
+	 */
+	
+	if (item->details->text_width >= 0 && item->details->text_height >= 0) {
+		return;
+	}
+	
 	if (icon_canvas_item_is_smooth (item)) {
 		draw_or_measure_label_text_aa (item, NULL, 0, 0);
 	}
@@ -893,9 +944,66 @@ draw_stretch_handles (NautilusIconCanvasItem *item, GdkDrawable *drawable,
 	gdk_gc_unref (gc);
 }
 
+/* utility routine that uses libart to draw an outlined rectangle */
+static void
+draw_outline_rectangle_aa (GnomeCanvasBuf *buf, int x0, int y0, int x1, int y1, guint outline_color)
+{
+	ArtVpath vpath[12];
+	ArtSVP *path;
+	int halfwidth;
+	
+	halfwidth = 1.0;
+
+	vpath[0].code = ART_MOVETO;
+	vpath[0].x = x0 - halfwidth;
+	vpath[0].y = y0 - halfwidth;
+
+	vpath[1].code = ART_LINETO;
+	vpath[1].x = x0 - halfwidth;
+	vpath[1].y = y1 + halfwidth;
+
+	vpath[2].code = ART_LINETO;
+	vpath[2].x = x1 + halfwidth;
+	vpath[2].y = y1 + halfwidth;
+
+	vpath[3].code = ART_LINETO;
+	vpath[3].x = x1 + halfwidth;
+	vpath[3].y = y0 - halfwidth;
+
+	vpath[4].code = ART_LINETO;
+	vpath[4].x = x0 - halfwidth;
+	vpath[4].y = y0 - halfwidth;
+
+	vpath[5].code = ART_MOVETO;
+	vpath[5].x = x0 + halfwidth;
+	vpath[5].y = y0 + halfwidth;
+
+	vpath[6].code = ART_LINETO;
+	vpath[6].x = x1 - halfwidth;
+	vpath[6].y = y0 + halfwidth;
+
+	vpath[7].code = ART_LINETO;
+	vpath[7].x = x1 - halfwidth;
+	vpath[7].y = y1 - halfwidth;
+
+	vpath[8].code = ART_LINETO;
+	vpath[8].x = x0 + halfwidth;
+	vpath[8].y = y1 - halfwidth;
+
+	vpath[9].code = ART_LINETO;
+	vpath[9].x = x0 + halfwidth;
+	vpath[9].y = y0 + halfwidth;
+
+	vpath[10].code = ART_END;
+	vpath[10].x = 0;
+	vpath[10].y = 0;
+	
+	path = art_svp_from_vpath(vpath);
+	gnome_canvas_render_svp(buf, path, outline_color);
+	art_svp_free(path);	
+}
 
 /* draw the stretch handles in the anti-aliased canvas */
-
 static void
 draw_stretch_handles_aa (NautilusIconCanvasItem *item, GnomeCanvasBuf *buf,
 		      const ArtIRect *rect)
@@ -904,8 +1012,6 @@ draw_stretch_handles_aa (NautilusIconCanvasItem *item, GnomeCanvasBuf *buf,
 	GnomeCanvasItem *canvas_item;
 	char *knob_filename;
 	GdkPixbuf *knob_pixbuf;
-	ArtIRect box_rect, draw_rect;
-	double affine[6];
 	
 	if (!item->details->show_stretch_handles) {
 		return;
@@ -917,35 +1023,18 @@ draw_stretch_handles_aa (NautilusIconCanvasItem *item, GnomeCanvasBuf *buf,
 	knob_pixbuf = gdk_pixbuf_new_from_file (knob_filename);
 	knob_width = gdk_pixbuf_get_width (knob_pixbuf);
 	knob_height = gdk_pixbuf_get_height (knob_pixbuf);
-	
-	art_affine_identity (affine);
-	
-
+		
 	/* draw a box to connect the dots */
- 	
- 	box_rect.x0 = rect->x0;
-  	box_rect.y0 = rect->y0;
- 	box_rect.x1 = rect->x1;
- 	box_rect.y1 = rect->y1;
-
-	art_irect_intersect (&draw_rect, &box_rect, &buf->rect);
-	if (!art_irect_empty (&draw_rect)) {
- 		draw_dashed_rectangle_aa (buf->buf, buf->buf_rowstride, FALSE, 
-				  buf->rect.x1 - buf->rect.x0,
-				  buf->rect.y1 - buf->rect.y0,
-				  box_rect.x0 -  buf->rect.x0,
-				  box_rect.y0 -  buf->rect.y0,
-				  box_rect.x1 -  buf->rect.x0,
-				  box_rect.y1 -  buf->rect.y0);
-	}
-	
+	draw_outline_rectangle_aa (buf, rect->x0 + 1, rect->y0 + 1,
+				   rect->x1 - 1, rect->y1 - 1,
+				   NAUTILUS_RGBA_COLOR_PACK (153, 153, 153, 127));	
 
 	/* now draw the stretch handles themselves  */
-	draw_pixbuf_aa (knob_pixbuf, buf, affine, rect->x0, rect->y0);
-	draw_pixbuf_aa (knob_pixbuf, buf, affine, rect->x0,  rect->y1 - knob_height);
-	draw_pixbuf_aa (knob_pixbuf, buf, affine, rect->x1 - knob_width, rect->y0);
-	draw_pixbuf_aa (knob_pixbuf, buf, affine, rect->x1 - knob_width, rect->y1 - knob_height);
-		
+	nautilus_gnome_canvas_draw_pixbuf (buf, knob_pixbuf, rect->x0, rect->y0);
+	nautilus_gnome_canvas_draw_pixbuf (buf, knob_pixbuf, rect->x0, rect->y1 - knob_height);
+	nautilus_gnome_canvas_draw_pixbuf (buf, knob_pixbuf, rect->x1 - knob_width, rect->y0);
+	nautilus_gnome_canvas_draw_pixbuf (buf, knob_pixbuf, rect->x1 - knob_width, rect->y1 - knob_height);
+			
 	g_free(knob_filename);
 	gdk_pixbuf_unref(knob_pixbuf);	
 }
@@ -1109,40 +1198,6 @@ draw_pixbuf (GdkPixbuf *pixbuf, GdkDrawable *drawable, int x, int y)
 					     0, 0);
 }
 
-static void
-draw_pixbuf_aa (GdkPixbuf *pixbuf, GnomeCanvasBuf *buf, double affine[6], int x_offset, int y_offset)
-{
-	void (* affine_function)
-		(art_u8 *dst, int x0, int y0, int x1, int y1, int dst_rowstride,
-		 const art_u8 *src, int src_width, int src_height, int src_rowstride,
-		 const double affine[6],
-		 ArtFilterLevel level,
-		 ArtAlphaGamma *alpha_gamma);
-
-	affine[4] += x_offset;
-	affine[5] += y_offset;
-
-	affine_function = gdk_pixbuf_get_has_alpha (pixbuf)
-		? art_rgb_rgba_affine
-		: art_rgb_affine;
-	
-	(* affine_function)
-		(buf->buf,
-		 buf->rect.x0, buf->rect.y0,
-		 buf->rect.x1, buf->rect.y1,
-		 buf->buf_rowstride,
-		 gdk_pixbuf_get_pixels (pixbuf),
-		 gdk_pixbuf_get_width (pixbuf),
-		 gdk_pixbuf_get_height (pixbuf),
-		 gdk_pixbuf_get_rowstride (pixbuf),
-		 affine,
-		 ART_FILTER_NEAREST,
-		 NULL);
-
-	affine[4] -= x_offset;
-	affine[5] -= y_offset;
-}
-
 /* shared code to highlight or dim the passed-in pixbuf */
 static GdkPixbuf *
 map_pixbuf (NautilusIconCanvasItem *icon_item)
@@ -1254,200 +1309,6 @@ nautilus_icon_canvas_item_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	draw_label_text (icon_item, drawable, icon_rect.x0, icon_rect.y1);
 }
 
-static void draw_dashed_rectangle_aa (guchar *pixels, int rowstride, gboolean has_alpha,
-		   int width, int height, int left, int top, int right, int bottom)
-{
-	guchar		red, green, blue, alpha, dash_color, dash_alpha;
-	guint		pixel_offset;
-	int		x, y, x1, y1, x2, y2;
-	guchar		*row_offset, *offset;
-	int 		index, dash_width;
-	
-	dash_width = 1;
-	pixel_offset = has_alpha ? 4 : 3;
-
-	/* main color is black */
-	red = 0;
-	green = 0;
-	blue = 0;
-	alpha = 255;
-	
-	/* dask color is white/transparent */
-	dash_color = 255;
-	dash_alpha = 0;
-	
-	/* clip to the buffer bounds */	
-	if (left < 0)
-		x1 = 0;
-	else
-		x1 = left;
-	if (top < 0)
-		y1 = 0;
-	else
-		y1 = top;
-			
-	if (right < width) {
-		x2 = right;
-	} else {
-		x2 = width - 1;
-	}
-
-	if (bottom < height) {
-		y2 = bottom;
-	} else {
-		y2 = height - 1;
-	}
-	
-	/* Draw top line */
-	if (top >= 0) {
-		row_offset = pixels + y1 * rowstride;
-		offset = row_offset + (x1 * pixel_offset);
-		index = 0;
-		for (x = x1; x < x2; x++)
-		{
-			if (index < dash_width) {
-				*(offset++) = red;
-				*(offset++) = green;
-				*(offset++) = blue;
-			} else {
-				*(offset++) = dash_color;
-				*(offset++) = dash_color;
-				*(offset++) = dash_color;
-			}		
-		
-			if (has_alpha) {
-				if (index < dash_width) {
-					*(offset++) = alpha;
-				} else {
-					*(offset++) = dash_alpha;
-				}				
-			}
-
-			index++;
-			if (index > dash_width) {
-				index = 0;
-			}
-		}
-	}
-	
-	/* Draw bottom line */
-	if (bottom < height) {
-		row_offset = pixels + (y2 -1) * rowstride;
-		offset = row_offset + (x1 * pixel_offset);
-		index = 0;
-		for (x = x1; x < x2; x++)
-		{
-			if (index < dash_width) {
-				*(offset++) = red;
-				*(offset++) = green;
-				*(offset++) = blue;
-			} else {
-				*(offset++) = dash_color;
-				*(offset++) = dash_color;
-				*(offset++) = dash_color;
-			}
-		
-			if (has_alpha) {
-				if (index < dash_width) {
-					*(offset++) = alpha;
-				} else {
-					*(offset++) = dash_alpha;
-				}				
-			}
-
-			index++;
-			if (index > dash_width) {
-				index = 0;
-			}
-		}
-	}
-	
-	/* Draw left line */
-	
-	row_offset = pixels + y1 * rowstride;
-	index = 0;
-	for (y = y1; y < y2; y++)
-	{
-		offset = row_offset + (x1 * pixel_offset);
-
-		if (index < dash_width) {
-			*(offset++) = red;
-			*(offset++) = green;
-			*(offset++) = blue;
-		} else {
-			*(offset++) = dash_color;
-			*(offset++) = dash_color;
-			*(offset++) = dash_color;
-		}		
-
-		if (has_alpha) {
-			if (index < dash_width) {
-				*offset = alpha;
-			} else {
-				*offset = dash_alpha;
-			}				
-		}
-		row_offset += rowstride;
-
-		index++;
-		if (index > dash_width) {
-			index = 0;
-		}
-	}
-
-	/* Draw right line */
-	
-	row_offset = pixels + y1 * rowstride;
-	index = 0;
-	for (y = y1; y < y2; y++)
-	{
-		offset = row_offset + ((x2 -1) * pixel_offset);
-
-		if (index < dash_width) {
-			*(offset++) = red;
-			*(offset++) = green;
-			*(offset++) = blue;
-		} else {
-			*(offset++) = dash_color;
-			*(offset++) = dash_color;
-			*(offset++) = dash_color;
-		}
-						
-		if (has_alpha) {
-			if (index < dash_width) {
-				*offset = alpha;
-			} else {
-				*offset = dash_alpha;
-			}				
-		}
-		row_offset += rowstride;
-
-		index++;
-		if (index > dash_width) {
-			index = 0;
-		}
-	}
-}
-
-static void
-draw_focus_rect (GdkPixbuf *pixbuf, int left, int top, int right, int bottom)
-{
-	int rowstride, width, height;
-	guchar *pixels;
-	gboolean has_alpha;
-
-	g_return_if_fail (pixbuf != NULL);
-	
-	pixels = gdk_pixbuf_get_pixels (pixbuf);
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
-	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
-
-	draw_dashed_rectangle_aa (pixels, rowstride, has_alpha,
-			   width, height, left, top, right, bottom);
-}
-
 static void
 draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 			       GdkPixbuf *destination_pixbuf,
@@ -1455,7 +1316,7 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 			       int icon_bottom)
 {
 	NautilusIconCanvasItemDetails *details;
-	int width_so_far, height_so_far;
+	guint width_so_far, height_so_far;
 	guint32 label_name_color;
 	guint32 label_info_color;
 	GnomeCanvasItem *canvas_item;
@@ -1481,6 +1342,28 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 		return;
 	}
 	
+#if (defined PERFORMANCE_TEST_MEASURE_DISABLE && defined PERFORMANCE_TEST_DRAW_DISABLE)
+	/* don't do any drawing and fake out the width */
+	details->text_width = 80;
+	details->text_height = 20;
+	return;
+#endif
+
+#ifdef PERFORMANCE_TEST_MEASURE_DISABLE
+	if (destination_pixbuf == NULL) {
+		/* fake out the width */
+		details->text_width = 80;
+		details->text_height = 20;
+		return;
+	}
+#endif
+
+#ifdef PERFORMANCE_TEST_DRAW_DISABLE
+	if (destination_pixbuf != NULL) {
+		return;
+	}
+#endif
+
 	/* Combine editable and additional text for processing */
 	combined_text = g_strconcat
 		(have_editable ? details->editable_text : "",
@@ -1549,15 +1432,14 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 			
 			/* draw the shadow in black */
 			if (needs_highlight) {
-				icon_bottom += 3; /* leave some space for selection frame */
-				text_left   -= 1;
+				icon_bottom += 1; /* leave some space for selection frame */
+				text_left -= 1;
 				nautilus_text_layout_paint (icon_text_layout, 
 							    destination_pixbuf, 
 							    text_left + 2, 
 							    icon_bottom + height_so_far + 1,
 							    GTK_JUSTIFY_CENTER,
 							    NAUTILUS_RGB_COLOR_BLACK,
-							    FALSE,
 							    underlined);
 			}
 						
@@ -1568,7 +1450,6 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 						    icon_bottom + height_so_far,
 						    GTK_JUSTIFY_CENTER,
 						    label_color,
-						    FALSE,
 						    underlined);
 			
 			/* if it's highlighted, embolden by drawing twice */
@@ -1579,24 +1460,21 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 							    icon_bottom + height_so_far,
 							    GTK_JUSTIFY_CENTER,
 							    label_color,
-							    FALSE,
 							    underlined);
 			}
 			
 		}
 
-		width_so_far = MAX (width_so_far, icon_text_layout->width);
+		width_so_far = MAX (width_so_far, (guint) icon_text_layout->width);
 		height_so_far += icon_text_layout->height;
 		
 		nautilus_text_layout_free (icon_text_layout);
 	}
 	g_strfreev (pieces);
 	
-	/* add some extra space for highlighting */
-	if (needs_highlight) {
-		height_so_far += 8; /* extra slop for nicer highlighting */	
-		width_so_far += 15; /* account for emboldening, plus extra to make it look nicer */
-	}	
+	/* add some extra space for highlighting, even when we don't highlight so things wont move */
+	height_so_far += 2; /* extra slop for nicer highlighting */	
+	width_so_far += 8;  /* account for emboldening, plus extra to make it look nicer */
 	
 	if (destination_pixbuf != NULL) {
 		/* Current calculations should match what we measured before drawing.
@@ -1604,15 +1482,13 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 		 * before the call to draw. We might later decide to use this function
 		 * differently and change these asserts.
 		 */
-		g_assert (height_so_far == details->text_height);
-		g_assert (width_so_far == details->text_width);
+#if (defined PERFORMANCE_TEST_MEASURE_DISABLE || defined PERFORMANCE_TEST_DRAW_DISABLE)
+		g_assert ((int) height_so_far == details->text_height);
+		g_assert ((int) width_so_far == details->text_width);
+#endif
 	
 		box_left = icon_left + (icon_width - width_so_far) / 2;
 
-		/* Indicate keyboard selection by framing the text with a gray-stippled rectangle */
-		if (details->is_highlighted_as_keyboard_focus) {
-			draw_focus_rect (destination_pixbuf, 0, 0, width_so_far - 1, height_so_far - 1);
-		}
 	} else {
 		/* If measuring, remember the width & height. */
 		details->text_width = width_so_far;
@@ -1664,7 +1540,7 @@ clear_rounded_corners (GdkPixbuf *destination_pixbuf, GdkPixbuf *corner_pixbuf,
 }
 
 static void
-draw_label_text_aa (NautilusIconCanvasItem *icon_item, GnomeCanvasBuf *buf, double i2c[6], int x_delta)
+draw_label_text_aa (NautilusIconCanvasItem *icon_item, GnomeCanvasBuf *buf, int x, int y, int x_delta)
 {
 	GdkPixbuf *text_pixbuf;
 	NautilusIconContainer *container;
@@ -1673,8 +1549,7 @@ draw_label_text_aa (NautilusIconCanvasItem *icon_item, GnomeCanvasBuf *buf, doub
 	gboolean have_editable;
 	gboolean have_additional;
 	
-	/* make sure this is really necessary */
-	
+	/* make sure this is really necessary */	
 	have_editable = icon_item->details->editable_text != NULL
 		&& icon_item->details->editable_text[0] != '\0';
 	have_additional = icon_item->details->additional_text != NULL
@@ -1689,10 +1564,10 @@ draw_label_text_aa (NautilusIconCanvasItem *icon_item, GnomeCanvasBuf *buf, doub
 	}
 	
 	if (icon_item->details->is_renaming) {
-		/* FIXME bugzilla.eazel.com 2472: 
-		 * Why is it OK to leave text_height and
-		 * text_width alone in this code path?
-		 */
+		/* Exit if we are renaming. We don't need to set the text
+		* width and height to 0 because there is text, it just is not
+		* drawn to the canvas while the renaming widget is dispalyed.
+		*/
 		return;
 	}
 		
@@ -1700,12 +1575,16 @@ draw_label_text_aa (NautilusIconCanvasItem *icon_item, GnomeCanvasBuf *buf, doub
 	needs_highlight = icon_item->details->is_highlighted_for_selection
 		|| icon_item->details->is_highlighted_for_drop;
 
+	/* Optimizing out the allocation of this pixbuf on every call is a
+	 * measureable speed improvement, but only by around 5%.
+	 * draw_or_measure_label_text_aa accounts for about 90% of the time.
+	 */
 	text_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 				TRUE,
 				8,
 				icon_item->details->text_width,
 				icon_item->details->text_height);
-	
+
 	if (needs_highlight) {
 		container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (icon_item)->canvas);	
 		nautilus_gdk_pixbuf_fill_rectangle_with_color (text_pixbuf, NULL,
@@ -1720,14 +1599,21 @@ draw_label_text_aa (NautilusIconCanvasItem *icon_item, GnomeCanvasBuf *buf, doub
 	draw_or_measure_label_text_aa (icon_item, text_pixbuf, x_delta, 0);
 	
 	/* Draw the pixbuf containing the label. */
-	i2c[4] -= x_delta;
-	draw_pixbuf_aa (text_pixbuf, buf, i2c, 0, LABEL_OFFSET);
-	i2c[4] += x_delta;
-	gdk_pixbuf_unref (text_pixbuf);
+
+	nautilus_gnome_canvas_draw_pixbuf (buf, text_pixbuf, x - x_delta, y + LABEL_OFFSET);
+
+	gdk_pixbuf_unref (text_pixbuf);	
+
+	/* draw the keyboard selection focus indicator if necessary */
+	if (icon_item->details->is_highlighted_as_keyboard_focus) {
+		draw_outline_rectangle_aa (buf, x - x_delta + 1, y + 1,
+					   x - x_delta + icon_item->details->text_width,
+					   y + icon_item->details->text_height,
+					   NAUTILUS_RGBA_COLOR_PACK (153, 153, 153, 127));
+	}
 }
 
 /* draw the item for anti-aliased mode */
-
 static void
 nautilus_icon_canvas_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 {
@@ -1735,7 +1621,6 @@ nautilus_icon_canvas_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	EmblemLayout emblem_layout;
 	GdkPixbuf *emblem_pixbuf, *temp_pixbuf;
 	NautilusIconCanvasItem *icon_item;
-	double i2c[6];
 	int x_delta;
 
 	icon_item = NAUTILUS_ICON_CANVAS_ITEM (item);
@@ -1743,45 +1628,32 @@ nautilus_icon_canvas_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	/* map the pixbuf for selection or other effects */
 	temp_pixbuf = map_pixbuf (icon_item);
 
-	/* Compute the affine transform, but force the scale to 1.0
-	 * because the icon factory does the scaling for us.
-	 */
-	gnome_canvas_item_i2c_affine (item, i2c);
-	i2c[0] = 1.0;
-	i2c[3] = 1.0;
-	
-	/* force it to integer pixel boundaries to avoid losing some of the left edge */
-	i2c[4] = floor (i2c[4]);
-	i2c[5] = floor (i2c[5]);
+	get_icon_canvas_rectangle (icon_item, &icon_rect);
 	
 	if (buf->is_bg) {
 		gnome_canvas_buf_ensure_buf (buf);
 		buf->is_bg = FALSE;
 	}
-	
+
 	/* draw the icon */
-	draw_pixbuf_aa (temp_pixbuf, buf, i2c, 0, 0);
-	
+	nautilus_gnome_canvas_draw_pixbuf (buf, temp_pixbuf, icon_rect.x0, icon_rect.y0);
+
 	if (temp_pixbuf != icon_item->details->pixbuf) {
 		gdk_pixbuf_unref (temp_pixbuf);
 	}
 
-	/* draw the emblems */
-	get_icon_canvas_rectangle (icon_item, &icon_rect);
-	
+	/* draw the emblems */	
 	emblem_layout_reset (&emblem_layout, icon_item, &icon_rect);
 	while (emblem_layout_next (&emblem_layout, &emblem_pixbuf, &emblem_rect)) {
-		draw_pixbuf_aa (emblem_pixbuf, buf, i2c, emblem_rect.x0 - icon_rect.x0, 
-				emblem_rect.y0 - icon_rect.y0);
+		nautilus_gnome_canvas_draw_pixbuf (buf, emblem_pixbuf, emblem_rect.x0, emblem_rect.y0);
 	}
 
 	/* draw the stretch handles */
 	draw_stretch_handles_aa (icon_item, buf, &icon_rect);
 		
 	/* draw the text */
-	i2c[5] += icon_rect.y1 - icon_rect.y0;
 	x_delta = (icon_item->details->text_width - (icon_rect.x1 - icon_rect.x0)) / 2;
-	draw_label_text_aa (icon_item, buf, i2c, x_delta);
+	draw_label_text_aa (icon_item, buf, icon_rect.x0, icon_rect.y1, x_delta);
 }
 
 
@@ -1898,7 +1770,7 @@ hit_test_pixbuf (GdkPixbuf *pixbuf, const ArtIRect *pixbuf_location, const ArtIR
 			pixel = gdk_pixbuf_get_pixels (pixbuf)
 				+ y * gdk_pixbuf_get_rowstride (pixbuf)
 				+ x * 4;
-			if (pixel[3] >= 128) {
+			if (pixel[3] > 1) {
 				return TRUE;
 			}
 		}
@@ -1921,10 +1793,16 @@ hit_test (NautilusIconCanvasItem *icon_item, const ArtIRect *canvas_rect)
 		return TRUE;
 	}
 	
-	/* Check for hit in the icon. */
+	/* Check for hit in the icon. If we're highlighted for dropping, anywhere in the rect is OK */
 	get_icon_canvas_rectangle (icon_item, &icon_rect);
-	if (hit_test_pixbuf (details->pixbuf, &icon_rect, canvas_rect)) {
-		return TRUE;
+	if (icon_item->details->is_highlighted_for_drop) {
+		if (nautilus_art_irect_hits_irect (&icon_rect, canvas_rect)) {
+			return TRUE;
+		}
+	} else {
+		if (hit_test_pixbuf (details->pixbuf, &icon_rect, canvas_rect)) {
+			return TRUE;
+		}
 	}
 
 	/* Check for hit in the text. */
