@@ -71,14 +71,16 @@ typedef struct {
 	/* Operations on the container. */
 	int          (* button_press) 	          (NautilusIconContainer *container,
 						   GdkEventButton *event);
-	void         (* context_click_background) (NautilusIconContainer *container);
+	void         (* context_click_background) (NautilusIconContainer *container,
+						   GdkEventButton *event);
 	void         (* middle_click) 		  (NautilusIconContainer *container,
 						   GdkEventButton *event);
 
 	/* Operations on icons. */
 	void         (* activate)	  	  (NautilusIconContainer *container,
 						   NautilusIconData *data);
-	void         (* context_click_selection)  (NautilusIconContainer *container);
+	void         (* context_click_selection)  (NautilusIconContainer *container,
+						   GdkEventButton *event);
 	void	     (* move_copy_items)	  (NautilusIconContainer *container,
 						   GList *item_uris,
 						   GdkPoint *relative_item_points,
@@ -135,6 +137,12 @@ typedef struct {
 	void         (* icon_text_changed)        (NautilusIconContainer *container,
 						   NautilusIconData *data,
 						   const char *text);
+	void         (* renaming_icon)            (NautilusIconContainer *container,
+						   GtkWidget *renaming_widget);
+	void	     (* icon_stretch_started)     (NautilusIconContainer *container,
+						   NautilusIconData *data);
+	void	     (* icon_stretch_ended)       (NautilusIconContainer *container,
+						   NautilusIconData *data);
 	int	     (* preview)		  (NautilusIconContainer *container,
 						   NautilusIconData *data,
 						   gboolean start_flag);
@@ -183,6 +191,7 @@ void       nautilus_icon_container_select_all                    (NautilusIconCo
 GList     *nautilus_icon_container_get_selection                 (NautilusIconContainer  *view);
 void       nautilus_icon_container_set_selection                 (NautilusIconContainer  *view,
 								  GList                  *selection);
+GArray    *nautilus_icon_container_get_selected_icon_locations	 (NautilusIconContainer  *view);
 gboolean   nautilus_icon_container_has_stretch_handles           (NautilusIconContainer  *container);
 gboolean   nautilus_icon_container_is_stretched                  (NautilusIconContainer  *container);
 void       nautilus_icon_container_show_stretch_handles          (NautilusIconContainer  *container);
@@ -190,7 +199,7 @@ void       nautilus_icon_container_unstretch                     (NautilusIconCo
 void       nautilus_icon_container_start_renaming_selected_item  (NautilusIconContainer  *container);
 
 /* options */
-int        nautilus_icon_container_get_zoom_level                (NautilusIconContainer  *view);
+NautilusZoomLevel nautilus_icon_container_get_zoom_level         (NautilusIconContainer  *view);
 void       nautilus_icon_container_set_zoom_level                (NautilusIconContainer  *view,
 								  int                     new_zoom_level);
 void       nautilus_icon_container_set_single_click_mode         (NautilusIconContainer  *container,
@@ -209,4 +218,5 @@ gboolean   nautilus_icon_container_get_is_fixed_size             (NautilusIconCo
 void       nautilus_icon_container_set_is_fixed_size             (NautilusIconContainer  *container,
 								  gboolean                is_fixed_size);
 
+void	   nautilus_icon_container_reset_scroll_region	 	 (NautilusIconContainer	 *container);
 #endif /* NAUTILUS_ICON_CONTAINER_H */
