@@ -94,7 +94,8 @@ static void     set_drop_target                                  (NautilusIconCo
 static GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL }
+	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
+	{ NAUTILUS_ICON_DND_TEXT_TYPE, 0, NAUTILUS_ICON_DND_TEXT }
 };
 
 static GtkTargetEntry drop_types [] = {
@@ -107,9 +108,6 @@ static GtkTargetEntry drop_types [] = {
 };
 
 
-/* special reserved name for the erase emblem */
-#define ERASE_KEYWORD "erase"
-
 static GnomeCanvasItem *
 create_selection_shadow (NautilusIconContainer *container,
 			 GList *list)
@@ -140,9 +138,6 @@ create_selection_shadow (NautilusIconContainer *container,
            we try to be smart and only create the maximum number of rectangles
            that we will need, in the vertical/horizontal directions.  */
 
-	/* FIXME bugzilla.eazel.com 624: 
-	 * Does this work properly if the window is scrolled? 
-	 */
 	max_x = GTK_WIDGET (container)->allocation.width;
 	min_x = -max_x;
 
@@ -597,7 +592,7 @@ receive_dropped_keyword (NautilusIconContainer *container, char* keyword, int x,
 static void
 receive_dropped_uri_list (NautilusIconContainer *container, char *uri_list, int x, int y)
 {
-	/* FIXME:
+	/* FIXME bugzilla.eazel.com 5080:
 	 * this needs a better name - it's link/desktop specific 
 	 */
 	GList *li, *files;
@@ -605,7 +600,9 @@ receive_dropped_uri_list (NautilusIconContainer *container, char *uri_list, int
 	char **argv;
 	int i;
 	
-	g_assert (uri_list != NULL);
+	if (uri_list == NULL) {
+		return;
+	}
 
 	files = gnome_uri_list_extract_filenames (uri_list);
 	argc = g_list_length (files);
@@ -637,13 +634,13 @@ auto_scroll_timeout_callback (gpointer data)
 	widget = GTK_WIDGET (data);
 	container = NAUTILUS_ICON_CONTAINER (widget);
 
-	if (container->details->waiting_to_autoscroll
-		&& container->details->start_auto_scroll_in < nautilus_get_system_time()) {
+	if (container->details->dnd_info->drag_info.waiting_to_autoscroll
+	    && container->details->dnd_info->drag_info.start_auto_scroll_in > nautilus_get_system_time()) {
 		/* not yet */
 		return TRUE;
 	}
 
-	container->details->waiting_to_autoscroll = FALSE;
+	container->details->dnd_info->drag_info.waiting_to_autoscroll = FALSE;
 
 	nautilus_drag_autoscroll_calculate_delta (widget, &x_scroll_delta, &y_scroll_delta);
 	if (x_scroll_delta == 0 && y_scroll_delta == 0) {
@@ -695,24 +692,16 @@ auto_scroll_timeout_callback (gpointer data)
 static void
 set_up_auto_scroll_if_needed (NautilusIconContainer *container)
 {
-	if (container->details->auto_scroll_timeout_id == 0) {
-		container->details->waiting_to_autoscroll = TRUE;
-		container->details->start_auto_scroll_in = nautilus_get_system_time() 
-			+ AUTOSCROLL_INITIAL_DELAY;
-		container->details->auto_scroll_timeout_id = gtk_timeout_add
-				(AUTOSCROLL_TIMEOUT_INTERVAL,
-				 auto_scroll_timeout_callback,
-			 	 container);
-	}
+	nautilus_drag_autoscroll_start (&container->details->dnd_info->drag_info,
+					GTK_WIDGET (container),
+					auto_scroll_timeout_callback,
+					container);
 }
 
 static void
 stop_auto_scroll (NautilusIconContainer *container)
 {
-	if (container->details->auto_scroll_timeout_id) {
-		gtk_timeout_remove (container->details->auto_scroll_timeout_id);
-		container->details->auto_scroll_timeout_id = 0;
-	}
+	nautilus_drag_autoscroll_stop (&container->details->dnd_info->drag_info);
 }
 
 static gboolean
@@ -788,11 +777,8 @@ handle_local_move (NautilusIconContainer *container,
 	}		
 	nautilus_icon_container_select_list_unselect_others
 		(container, moved_icons);
-	/* Might have been moved in a way that requires adjusting scroll region.
-	 * Include visible area so coordinate-adjustment "auto-scrolling" doesn't
-	 * occur after the drop. (see bugzilla.eazel.com 2068)
-	 */
-	nautilus_icon_container_update_scroll_region_include_visible_area (container);
+	/* Might have been moved in a way that requires adjusting scroll region. */
+	nautilus_icon_container_update_scroll_region (container);
 	g_list_free (moved_icons);
 }
 
@@ -804,8 +790,8 @@ handle_nonlocal_move (NautilusIconContainer *container,
 		      gboolean icon_hit)
 {
 	GList *source_uris, *p;
-	GdkPoint *source_item_locations;
-	int i;
+	GArray *source_item_locations;
+	int index;
 
 	if (container->details->dnd_info->drag_info.selection_list == NULL) {
 		return;
@@ -818,16 +804,20 @@ handle_nonlocal_move (NautilusIconContainer *container,
 	}
 	source_uris = g_list_reverse (source_uris);
 	
-	source_item_locations = NULL;
+	source_item_locations = g_array_new (FALSE, TRUE, sizeof (GdkPoint));
 	if (!icon_hit) {
 		/* Drop onto a container. Pass along the item points to allow placing
 		 * the items in their same relative positions in the new container.
 		 */
-		source_item_locations = g_new (GdkPoint, g_list_length (source_uris));
-		for (i = 0, p = container->details->dnd_info->drag_info.selection_list;
-		     p != NULL; i++, p = p->next) {
-			source_item_locations[i].x = ((DragSelectionItem *)p->data)->icon_x;
-			source_item_locations[i].y = ((DragSelectionItem *)p->data)->icon_y;
+		source_item_locations = g_array_set_size (source_item_locations,
+			g_list_length (container->details->dnd_info->drag_info.selection_list));
+			
+		for (index = 0, p = container->details->dnd_info->drag_info.selection_list;
+			p != NULL; index++, p = p->next) {
+		     	g_array_index (source_item_locations, GdkPoint, index).x =
+		     		((DragSelectionItem *)p->data)->icon_x;
+		     	g_array_index (source_item_locations, GdkPoint, index).y =
+				((DragSelectionItem *)p->data)->icon_y;
 		}
 	}
 		
@@ -838,8 +828,9 @@ handle_nonlocal_move (NautilusIconContainer *container,
 				 target_uri,
 				 context->action,
 				 x, y);
+
 	g_list_free (source_uris);
-	g_free (source_item_locations);
+	g_array_free (source_item_locations, TRUE);
 }
 
 static char *
@@ -853,13 +844,14 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 	NautilusFile *file;
 	char *icon_uri;
 
+	*icon_hit = FALSE;
 	if (container->details->dnd_info->drag_info.selection_list == NULL) {
 		return NULL;
 	}
 
   	gnome_canvas_window_to_world (GNOME_CANVAS (container),
 				      x, y, &world_x, &world_y);
-
+	
 	/* FIXME bugzilla.eazel.com 2485: 
 	 * These "can_accept_items" tests need to be done by
 	 * the icon view, not here. This file is not supposed to know
@@ -873,7 +865,7 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 		if (icon_uri != NULL) {
 			file = nautilus_file_get (icon_uri);
 
-			if ( !nautilus_drag_can_accept_items (file, 
+			if (!nautilus_drag_can_accept_items (file, 
 					container->details->dnd_info->drag_info.selection_list)) {
 			 	/* the item we dropped our selection on cannot accept the items,
 			 	 * do the same thing as if we just dropped the items on the canvas
@@ -892,7 +884,6 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 	}
 	
 	*icon_hit = TRUE;
-
 	return nautilus_icon_container_get_icon_drop_target_uri (container, drop_target_icon);
 }
 
@@ -990,12 +981,22 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 {
 	char *drop_target;
 	gboolean icon_hit;
-
+	NautilusIcon *icon;
+	double world_x, world_y;
+	
+	icon_hit = FALSE;
 	if (!container->details->dnd_info->drag_info.got_drop_data_type) {
 		/* drag_data_received_callback didn't get called yet */
 		return;
 	}
 
+	/* find out if we're over an icon */
+  	gnome_canvas_window_to_world (GNOME_CANVAS (container),
+				      x, y, &world_x, &world_y);
+	
+	icon = nautilus_icon_container_item_at (container, world_x, world_y);
+
+	/* case out on the type of object being dragged */
 	switch (container->details->dnd_info->drag_info.data_type) {
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
 		if (container->details->dnd_info->drag_info.selection_list == NULL) {
@@ -1013,11 +1014,34 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 		nautilus_drag_default_drop_action_for_icons (context, drop_target, 
 			container->details->dnd_info->drag_info.selection_list, 
 			default_action, non_default_action);
+		g_free (drop_target);
 		break;
 
+	/* handle emblems by setting the action if we're over an object */
+	case NAUTILUS_ICON_DND_KEYWORD:
+		if (icon == NULL) {
+			*default_action = 0;
+			*non_default_action = 0;
+		} else {
+			*default_action = context->suggested_action;
+			*non_default_action = context->suggested_action;
+		}
+		
+		break;
+	
+	/* handle colors and backgrounds by setting the action if we're over the background */		
 	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:
-	case NAUTILUS_ICON_DND_KEYWORD:
+		if (icon == NULL) {
+			*default_action = context->suggested_action;
+			*non_default_action = context->suggested_action;
+		} else {
+			*default_action = 0;
+			*non_default_action = 0;
+		}
+	
+		break;
+	
 	case NAUTILUS_ICON_DND_URI_LIST:
 		*default_action = context->suggested_action;
 		*non_default_action = context->suggested_action;
