@@ -44,6 +44,7 @@
 #include "nautilus-string.h"
 #include "nautilus-theme.h"
 #include "nautilus-thumbnails.h"
+#include "nautilus-trash-monitor.h"
 #include "nautilus-xml-extensions.h"
 #include <gnome-xml/parser.h>
 #include <gnome-xml/xmlmemory.h>
@@ -52,6 +53,8 @@
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-mime-info.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnomevfs/gnome-vfs-mime-monitor.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
 #include <librsvg/rsvg.h>
@@ -68,28 +71,30 @@ static const char *icon_file_name_suffixes[] =
 	".PNG",
 	".gif",
 	".GIF",
+	".jpg",
+	".JPG",
 	".xpm",
 	".XPM"
 };
 
+#define ICON_NAME_BLOCK_DEVICE          "i-blockdev"
+#define ICON_NAME_BROKEN_SYMBOLIC_LINK  "i-symlink"
+#define ICON_NAME_CHARACTER_DEVICE      "i-chardev"
 #define ICON_NAME_DIRECTORY             "i-directory"
 #define ICON_NAME_DIRECTORY_CLOSED      "i-dirclosed"
 #define ICON_NAME_EXECUTABLE            "i-executable"
+#define ICON_NAME_FIFO                  "i-fifo"
+#define ICON_NAME_INSTALL		"services-rpm"
 #define ICON_NAME_REGULAR               "i-regular"
 #define ICON_NAME_SEARCH_RESULTS        "i-search"
-#define ICON_NAME_WEB			"i-web"
 #define ICON_NAME_SOCKET                "i-sock"
-#define ICON_NAME_FIFO                  "i-fifo"
-#define ICON_NAME_CHARACTER_DEVICE      "i-chardev"
-#define ICON_NAME_BLOCK_DEVICE          "i-blockdev"
-#define ICON_NAME_BROKEN_SYMBOLIC_LINK  "i-symlink"
-#define ICON_NAME_INSTALL		"services-rpm"
-
 #define ICON_NAME_THUMBNAIL_LOADING     "loading"
+#define ICON_NAME_TRASH_EMPTY           "trash-empty"
+#define ICON_NAME_TRASH_NOT_EMPTY       "trash-full"
+#define ICON_NAME_WEB			"i-web"
 
 #define EMBLEM_NAME_PREFIX              "emblem-"
-
-#define DEFAULT_ICON_THEME		"default"
+#define EMBLEM_SCALE_FACTOR		0.75
 
 /* This used to be called ICON_CACHE_MAX_ENTRIES, but it's misleading
  * to call it that, since we can have any number of entries in the
@@ -103,41 +108,50 @@ static const char *icon_file_name_suffixes[] =
  */
 #define ICON_CACHE_SWEEP_TIMEOUT        (10 * 1000)
 
-/* For now, images are used themselves as thumbnails when they are
- * below this threshold size. Later we might have to have a more
+/* Images are used themselves as thumbnails when they are below this
+ * threshold size.
+ */
+/* FIXME bugzilla.eazel.com 5081: Later we might have to have a more
  * complex rule about when to use an image for itself.
  */
 #define SELF_THUMBNAIL_SIZE_THRESHOLD   16384
 
-/* extremely large images can eat up hundreds of megabytes of memory, so we
- * shouldn't automatically thumbnail when larges are too large.  Eventually,
- * we want this threshold to be user-settable, but for now it's hard-wired.
+/* Extremely large images can eat up hundreds of megabytes of memory,
+ * so we shouldn't automatically thumbnail when larges are too large.
  */
- #define INHIBIT_THUMBNAIL_SIZE_THRESHOLD 1048576
+/* FIXME bugzilla.eazel.com 5082: Eventually, we want this threshold
+ * to be user-settable, but for now it's hard-wired.
+ */
+#define INHIBIT_THUMBNAIL_SIZE_THRESHOLD (1024 * 1024)
  
-/* This circular doubly-linked list structure is used to keep a list
- * of the most recently used items in the cache.
- */
-typedef struct NautilusCircularList NautilusCircularList;
-struct NautilusCircularList {
-	NautilusCircularList *next;
-	NautilusCircularList *prev;
-};
-
-/* maximum size for either dimension at the standard zoom level */
-#define MAXIMUM_ICON_SIZE 96
+/* Maximum size for either dimension at the standard zoom level. */
+#define MAXIMUM_ICON_SIZE                96
 
 /* FIXME bugzilla.eazel.com 1102: Embedded text should use preferences
- * to determine what font it uses.
+ * to determine what font it uses instead of this set of constants.
  */
-#define EMBEDDED_TEXT_FONT_FAMILY       _("helvetica")
-#define EMBEDDED_TEXT_FONT_WEIGHT       _("medium")
+static const char untranslated_embedded_text_font_family[] = N_("helvetica");
+static const char untranslated_embedded_text_font_weight[] = N_("medium");
+#define EMBEDDED_TEXT_FONT_FAMILY       _(untranslated_embedded_text_font_family)
+#define EMBEDDED_TEXT_FONT_WEIGHT       _(untranslated_embedded_text_font_weight)
 #define EMBEDDED_TEXT_FONT_SLANT        NULL
 #define EMBEDDED_TEXT_FONT_SET_WIDTH    NULL
 #define EMBEDDED_TEXT_FONT_SIZE         9
 #define EMBEDDED_TEXT_LINE_OFFSET       1
 #define EMBEDDED_TEXT_EMPTY_LINE_HEIGHT 4
 
+#define MINIMUM_EMBEDDED_TEXT_RECT_WIDTH	20
+#define MINIMUM_EMBEDDED_TEXT_RECT_HEIGHT	20
+
+/* This circular doubly-linked list structure is used to keep a list
+ * of the most recently used items in the cache.
+ */
+typedef struct CircularList CircularList;
+struct CircularList {
+	CircularList *next;
+	CircularList *prev;
+};
+
 /* The icon factory.
  * These are just globals, but they're in an object so we can
  * connect signals and have multiple icon factories some day
@@ -172,7 +186,7 @@ typedef struct {
 	 * let them go when we sweep the cache.
 	 */
 	GHashTable *icon_cache;
-	NautilusCircularList recently_used_dummy_head;
+	CircularList recently_used_dummy_head;
 	guint recently_used_count;
         guint sweep_timer;
 } NautilusIconFactory;
@@ -234,7 +248,7 @@ typedef struct {
 	guint internal_ref_count;
 
 	/* Used to decide when to kick icons out of the cache. */
-	NautilusCircularList recently_used_node;
+	CircularList recently_used_node;
 
 	/* Used to know when to make a new thumbnail. */
 	time_t cache_time;
@@ -242,11 +256,9 @@ typedef struct {
 	/* Type of icon. */
 	gboolean custom;
 	gboolean scaled;
+	gboolean is_fallback;
 } CacheIcon;
 
-#define MINIMUM_EMBEDDED_TEXT_RECT_WIDTH	20.0
-#define MINIMUM_EMBEDDED_TEXT_RECT_HEIGHT	20.0
-
 static CacheIcon *fallback_icon;
 
 /* forward declarations */
@@ -256,6 +268,8 @@ static void       nautilus_icon_factory_initialize_class (NautilusIconFactoryCla
 static void       nautilus_icon_factory_initialize       (NautilusIconFactory      *factory);
 static void       nautilus_icon_factory_destroy          (GtkObject                *object);
 static void       icon_theme_changed_callback            (gpointer                  user_data);
+static void       mime_type_data_changed_callback        (GnomeVFSMIMEMonitor	   *monitor,
+							  gpointer                  user_data);
 static guint      nautilus_scalable_icon_hash            (gconstpointer             p);
 static gboolean   nautilus_scalable_icon_equal           (gconstpointer             a,
 							  gconstpointer             b);
@@ -300,6 +314,11 @@ get_icon_factory (void)
 						   icon_theme_changed_callback,
 						   NULL);
 
+		gtk_signal_connect (GTK_OBJECT (gnome_vfs_mime_monitor_get ()),
+				    "data_changed",
+				    mime_type_data_changed_callback,
+				    NULL);
+
 		g_atexit (destroy_icon_factory);
         }
         return global_icon_factory;
@@ -315,8 +334,8 @@ static void
 check_recently_used_list (void)
 {
 	NautilusIconFactory *factory;
-	NautilusCircularList *head, *node, *next;
-	int count;
+	CircularList *head, *node, *next;
+	guint count;
 
 	factory = get_icon_factory ();
 
@@ -347,8 +366,7 @@ nautilus_icon_factory_initialize (NautilusIconFactory *factory)
 {
 	factory->scalable_icons = g_hash_table_new (nautilus_scalable_icon_hash,
 						    nautilus_scalable_icon_equal);
-	factory->cache_icons = g_hash_table_new (g_direct_hash,
-						 g_direct_equal);
+	factory->cache_icons = g_hash_table_new (NULL, NULL);
         factory->icon_cache = g_hash_table_new (cache_key_hash,
 						cache_key_equal);
 	
@@ -427,6 +445,8 @@ cache_icon_new (GdkPixbuf *pixbuf,
 	icon->internal_ref_count = 1;
 	icon->custom = custom;
 	icon->scaled = scaled;
+	icon->is_fallback = FALSE;
+	
 	if (details != NULL) {
 		icon->details = *details;
 	}
@@ -455,7 +475,7 @@ static void
 cache_icon_unref (CacheIcon *icon)
 {
 	NautilusIconFactory *factory;
-	NautilusCircularList *node;
+	CircularList *node;
 
 	factory = get_icon_factory ();
 
@@ -530,7 +550,7 @@ static void
 nautilus_icon_factory_clear (void)
 {
 	NautilusIconFactory *factory;
-	NautilusCircularList *head;
+	CircularList *head;
 
 	factory = get_icon_factory ();
 
@@ -724,40 +744,38 @@ set_theme (const char *theme_name)
 static const char *
 nautilus_icon_factory_get_icon_name_for_regular_file (NautilusFile *file)
 {
-	char *mime_type, *uri;
 	const char *icon_name;
-	gboolean is_text_file;
+	char *mime_type, *uri;
 	
-	/* force plain text files to use the generic document icon so we can have the text-in-icons feature;
-		eventually, we want to force other types of text files as well */
-        
 	mime_type = nautilus_file_get_mime_type (file);
-	is_text_file = mime_type != NULL && !nautilus_strcasecmp (mime_type, "text/plain");
-	
-	if (mime_type != NULL && !is_text_file) {
-		icon_name = gnome_vfs_mime_get_value (mime_type, "icon-filename");
+	if (mime_type != NULL) {
+		icon_name = gnome_vfs_mime_get_icon (mime_type);
 		if (icon_name != NULL) {
 			g_free (mime_type);
 			return icon_name;
 		}
 	}
 
-	/* gnome_vfs_mime didn't give us an icon name, so we have to fall back on default icons. */
-	if (nautilus_file_is_executable (file) & !is_text_file) {
-		g_free (mime_type);		
-		return ICON_NAME_EXECUTABLE;
-	}
-
-	/* special-case icons based on the uri scheme.  eventually we should generalize this. */
 	uri = nautilus_file_get_uri (file);
-	if (nautilus_istr_has_prefix (uri, "http:") && !nautilus_strcmp (mime_type, "text/html")) {
+
+	/* Special-case icons based on the uri scheme. Eventually we
+	 * should generalize this or at least have a way for others to
+	 * extend it.
+	 */
+	if (nautilus_istr_has_prefix (uri, "http:")
+	    && nautilus_strcmp (mime_type, "text/html") == 0) {
 		icon_name = ICON_NAME_WEB;
 	} else if (nautilus_istr_has_prefix (uri, "eazel-install:")) {
 		icon_name = ICON_NAME_INSTALL;
 	} else {
-		icon_name = ICON_NAME_REGULAR;
+		if (nautilus_file_is_executable (file)
+		    && nautilus_strcasecmp (mime_type, "text/plain") != 0) {
+			icon_name = ICON_NAME_EXECUTABLE;
+		} else {
+			icon_name = ICON_NAME_REGULAR;
+		}
 	}
-
+	
         g_free (uri);
         g_free (mime_type);	
 	
@@ -768,18 +786,20 @@ nautilus_icon_factory_get_icon_name_for_regular_file (NautilusFile *file)
 static const char *
 nautilus_icon_factory_get_icon_name_for_directory (NautilusFile *file)
 {
+	const char *icon_name;
 	char *mime_type;
-	
+
 	mime_type = nautilus_file_get_mime_type (file);
 	
-	if (mime_type != NULL && !nautilus_strcasecmp (mime_type, "x-directory/search")) {
-		g_free (mime_type);
-		return ICON_NAME_SEARCH_RESULTS;
-	}
-	else {
-		g_free (mime_type);
-		return ICON_NAME_DIRECTORY;
+	if (nautilus_strcasecmp (mime_type, "x-directory/search") == 0) {
+		icon_name = ICON_NAME_SEARCH_RESULTS;
+	} else {
+		icon_name = ICON_NAME_DIRECTORY;
 	}
+	
+	g_free (mime_type);
+
+	return icon_name;
 }
 
 
@@ -787,6 +807,16 @@ nautilus_icon_factory_get_icon_name_for_directory (NautilusFile *file)
 static const char *
 nautilus_icon_factory_get_icon_name_for_file (NautilusFile *file)
 {	
+	char *uri;
+	
+	uri = nautilus_file_get_uri (file);
+	if (strcmp (uri, NAUTILUS_TRASH_URI) == 0) {
+		g_free (uri);
+		return nautilus_trash_monitor_is_empty ()
+			? ICON_NAME_TRASH_EMPTY : ICON_NAME_TRASH_NOT_EMPTY;
+	}
+	g_free (uri);
+
 	/* Get an icon name based on the file's type. */
         switch (nautilus_file_get_file_type (file)) {
         case GNOME_VFS_FILE_TYPE_DIRECTORY:
@@ -875,7 +905,7 @@ get_themed_icon_file_path (const char *theme_name,
 			   gboolean aa_mode,
 			   IconDetails *details)
 {
-	int i;
+	guint i;
 	gboolean include_size;
 	char *themed_icon_name, *partial_path, *path, *aa_path, *xml_path;
 	xmlDocPtr doc;
@@ -976,7 +1006,7 @@ get_themed_icon_file_path (const char *theme_name,
 	 * check out the user's home directory, since it might be an emblem
 	 * that they've added there.
 	 */
-	if (path == NULL && nautilus_str_has_prefix (icon_name, "emblem-")) {
+	if (path == NULL && icon_size == NAUTILUS_ICON_SIZE_STANDARD && nautilus_str_has_prefix (icon_name, EMBLEM_NAME_PREFIX)) {
 		for (i = 0; i < NAUTILUS_N_ELEMENTS (icon_file_name_suffixes); i++) {
 			user_directory = nautilus_get_user_directory ();
 			path = g_strdup_printf ("%s/emblems/%s%s", 
@@ -995,6 +1025,10 @@ get_themed_icon_file_path (const char *theme_name,
 	}
 	g_free (themed_icon_name);
 
+	if (path == NULL) {
+		path = gnome_vfs_icon_path_from_filename (icon_name);
+	}
+
 	return path;
 }
 
@@ -1038,10 +1072,10 @@ get_icon_file_path (const char *name,
 			g_free (path);
 		} else if (icon_factory->default_theme_name != NULL) {
 			path = get_themed_icon_file_path (icon_factory->default_theme_name,
-						  name,
-						  NAUTILUS_ICON_SIZE_STANDARD,
-						  aa_mode,
-						  details);		
+							  name,
+							  NAUTILUS_ICON_SIZE_STANDARD,
+							  aa_mode,
+							  details);
 			if (path != NULL) {
 				theme_to_use = icon_factory->default_theme_name;
 				g_free (path);
@@ -1080,8 +1114,7 @@ icon_theme_changed_callback (gpointer user_data)
 	/* Consult the user preference and the Nautilus theme. In the
 	 * long run, we sould just get rid of the user preference.
 	 */
-	theme_preference = nautilus_preferences_get
-		(NAUTILUS_PREFERENCES_THEME, DEFAULT_ICON_THEME);
+	theme_preference = nautilus_preferences_get (NAUTILUS_PREFERENCES_THEME);
 	icon_theme = nautilus_theme_get_theme_data ("icons", "ICON_THEME");
 	
 	set_theme (icon_theme == NULL ? theme_preference : icon_theme);
@@ -1090,6 +1123,19 @@ icon_theme_changed_callback (gpointer user_data)
 	g_free (icon_theme);
 }
 
+static void       
+mime_type_data_changed_callback (GnomeVFSMIMEMonitor *monitor, gpointer user_data)
+{
+	g_assert (monitor != NULL);
+	g_assert (user_data == NULL);
+
+	/* We don't know which data changed, so we have to assume that
+	 * any or all icons might have changed.
+	 */
+	gtk_signal_emit (GTK_OBJECT (get_icon_factory ()), 
+			 signals[ICONS_CHANGED]);
+}				 
+
 /* Decompose a scalable icon into its text pieces. */
 void
 nautilus_scalable_icon_get_text_pieces (NautilusScalableIcon *icon,
@@ -1142,7 +1188,7 @@ nautilus_scalable_icon_new_from_text_pieces (const char *uri,
 	}
 
 	/* Get at the hash table. */
-	hash_table = get_icon_factory ()->scalable_icons;
+	hash_table = factory->scalable_icons;
 
 	/* Check to see if it's already in the table. */
 	cache_key.uri = (char *) uri;
@@ -1251,14 +1297,24 @@ nautilus_scalable_icon_equal (gconstpointer a,
 
 
 static gboolean
-should_display_image_file_as_itself (NautilusFile *file)
+should_display_image_file_as_itself (NautilusFile *file, gboolean anti_aliased)
 {
 	NautilusSpeedTradeoffValue preference_value;
 	
-	preference_value = nautilus_preferences_get_enum
-		(NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS, 
-		 NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY);
+	preference_value = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS);
 
+	/* see if there's a proxy thumbnail to indicate that thumbnailing
+	 * failed, in which case we shouldn't use the thumbnail.
+	 */
+	if (nautilus_thumbnail_has_invalid_thumbnail (file, anti_aliased)) {
+		return FALSE;
+	}
+	
+	/* if we don't have read permissions for the file, don't show as itself for security reasons  */
+	if (!nautilus_file_can_read (file)) {
+		return FALSE;
+	}
+	
 	if (preference_value == NAUTILUS_SPEED_TRADEOFF_ALWAYS) {
 		return TRUE;
 	}
@@ -1273,7 +1329,7 @@ should_display_image_file_as_itself (NautilusFile *file)
 
 /* key routine to get the scalable icon for a file */
 NautilusScalableIcon *
-nautilus_icon_factory_get_icon_for_file (NautilusFile *file, const char* modifier, gboolean anti_aliased)
+nautilus_icon_factory_get_icon_for_file (NautilusFile *file, const char *modifier, gboolean anti_aliased)
 {
 	char *uri, *file_uri, *file_path, *image_uri, *icon_name, *mime_type, *top_left_text;
  	gboolean is_local;
@@ -1299,7 +1355,8 @@ nautilus_icon_factory_get_icon_for_file (NautilusFile *file, const char* modifie
 		mime_type = nautilus_file_get_mime_type (file);
 		file_size = nautilus_file_get_size (file);
 		
-		if (nautilus_istr_has_prefix (mime_type, "image/") && should_display_image_file_as_itself (file)) {
+		if (nautilus_istr_has_prefix (mime_type, "image/") &&
+				should_display_image_file_as_itself (file, anti_aliased)) {
 			if (file_size < SELF_THUMBNAIL_SIZE_THRESHOLD && is_local) {
 				uri = nautilus_file_get_uri (file);				
 			} else if (strstr (file_uri, "/.thumbnails/") == NULL && file_size < INHIBIT_THUMBNAIL_SIZE_THRESHOLD) {
@@ -1336,7 +1393,7 @@ nautilus_icon_factory_get_icon_for_file (NautilusFile *file, const char* modifie
 			g_free (file_path);
 		}
 	}
-			
+	
 	/* handle SVG files */
 	if (uri == NULL && nautilus_file_is_mime_type (file, "image/svg")) {
 		uri = g_strdup (file_uri);
@@ -1417,19 +1474,35 @@ nautilus_icon_factory_get_emblem_icon_by_name (const char *emblem_name, gboolean
 }
 
 GList *
-nautilus_icon_factory_get_emblem_icons_for_file (NautilusFile *file, gboolean anti_aliased, NautilusStringList *exclude)
+nautilus_icon_factory_get_emblem_icons_for_file (NautilusFile *file,
+						 gboolean anti_aliased,
+						 NautilusStringList *exclude)
 {
-	GList *icons, *emblem_names, *p;
+	GList *icons, *emblem_names, *node;
+	char *uri, *name;
 	NautilusScalableIcon *icon;
+	gboolean file_is_trash;
+
+	/* Leave out the trash emblem for the trash itself, since
+	 * putting a trash emblem on a trash icon is gilding the
+	 * lily.
+	 */
+	uri = nautilus_file_get_uri (file);
+	file_is_trash = strcmp (uri, NAUTILUS_TRASH_URI) == 0;
+	g_free (uri);
 
 	icons = NULL;
 
 	emblem_names = nautilus_file_get_emblem_names (file);
-	for (p = emblem_names; p != NULL; p = p->next) {
-		if (nautilus_string_list_contains (exclude, p->data)) {
+	for (node = emblem_names; node != NULL; node = node->next) {
+		name = node->data;
+		if (file_is_trash && strcmp (name, NAUTILUS_FILE_EMBLEM_NAME_TRASH) == 0) {
 			continue;
 		}
-		icon = nautilus_icon_factory_get_emblem_icon_by_name (p->data, anti_aliased);
+		if (nautilus_string_list_contains (exclude, name)) {
+			continue;
+		}
+		icon = nautilus_icon_factory_get_emblem_icon_by_name (name, anti_aliased);
 		icons = g_list_prepend (icons, icon);
 	}
 	nautilus_g_list_free_deep (emblem_names);
@@ -1524,16 +1597,28 @@ get_next_icon_size_to_try (guint target_size, guint *current_size)
 
 /* This loads an SVG image, scaling it to the appropriate size. */
 static GdkPixbuf *
-load_pixbuf_svg (const char *path, guint size_in_pixels)
+load_pixbuf_svg (const char *path, const char *name, guint size_in_pixels)
 {
 	FILE *f;
 	GdkPixbuf *pixbuf;
+	double actual_size_in_pixels;
 	
 	f = fopen (path, "rb");
 	if (f == NULL) {
 		return NULL;
 	}
-	pixbuf = rsvg_render_file (f, ((double) size_in_pixels) / NAUTILUS_ICON_SIZE_STANDARD);
+	
+	/* FIXME: the nominal size of .svg emblems is too large, so we scale it
+	 * down here if the file is an emblem.  This code should be removed
+	 * eventually when we scale all the emblems properly.
+	 */
+	if (nautilus_str_has_prefix (name, EMBLEM_NAME_PREFIX)) {
+		actual_size_in_pixels = size_in_pixels * EMBLEM_SCALE_FACTOR;
+	} else {
+		actual_size_in_pixels = size_in_pixels;
+	}
+	
+	pixbuf = rsvg_render_file (f, (double) (actual_size_in_pixels / NAUTILUS_ICON_SIZE_STANDARD));
 	fclose (f);
 	
 	return pixbuf;
@@ -1561,6 +1646,41 @@ path_represents_svg_image (const char *path)
 	return is_svg;
 }
 
+/* Returns GNOME_VFS_ERROR_NOT_SUPPORTED for icons that are not files. */
+static GnomeVFSResult
+get_cache_time (const char *file_uri, time_t *cache_time)
+{
+	GnomeVFSURI *vfs_uri;
+	GnomeVFSFileInfo file_info;
+	GnomeVFSResult result;
+	gboolean is_local;
+
+	/* If there's no specific file, simply return. */
+	if (file_uri == NULL) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+		
+	/* FIXME bugzilla.eazel.com 2566: if the URI is remote, assume
+	 * it's valid to match logic below.
+	 */
+	vfs_uri = gnome_vfs_uri_new (file_uri);
+	is_local = gnome_vfs_uri_is_local (vfs_uri);
+	gnome_vfs_uri_unref (vfs_uri);
+	if (!is_local) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+	
+	/* Gather the info and then compare modification times. */
+	gnome_vfs_file_info_init (&file_info);
+	result = gnome_vfs_get_file_info (file_uri, &file_info, GNOME_VFS_FILE_INFO_DEFAULT);
+	if (result == GNOME_VFS_OK) {
+		*cache_time = file_info.mtime;
+	}
+	gnome_vfs_file_info_clear (&file_info);
+
+	return result;
+}
+
 /* This load function returns NULL if the icon is not available at
  * this size.
  */
@@ -1592,7 +1712,7 @@ load_specific_icon (NautilusScalableIcon *scalable_icon,
 	/* Get the icon. */
 	if (path != NULL) {
 		if (path_represents_svg_image (path)) {
-			pixbuf = load_pixbuf_svg (path, size_in_pixels);
+			pixbuf = load_pixbuf_svg (path, scalable_icon->name, size_in_pixels);
 		} else {
 			/* Custom non-svg icons exist at one size.
 			 * Non-custom icons have their size encoded in their path.
@@ -1612,7 +1732,10 @@ load_specific_icon (NautilusScalableIcon *scalable_icon,
 
 	/* Since we got something, we can create a cache icon. */
 	icon = cache_icon_new (pixbuf, custom, FALSE, &details);
+	get_cache_time (scalable_icon->uri,
+			&icon->cache_time);
 	gdk_pixbuf_unref (pixbuf);
+
 	return icon;
 }
 
@@ -1685,6 +1808,7 @@ load_icon_for_scaling (NautilusScalableIcon *scalable_icon,
 			 NULL, /* don't destroy data */
 			 NULL);
 		fallback_icon = cache_icon_new (pixbuf, FALSE, FALSE, NULL);
+		fallback_icon->is_fallback = TRUE;
 		g_atexit (destroy_fallback_icon);
 	}
 
@@ -1749,6 +1873,8 @@ scale_icon (CacheIcon *icon,
 				      icon->custom,
 				      TRUE,
 				      &scaled_details);
+	scaled_icon->is_fallback = icon->is_fallback;
+	scaled_icon->cache_time = icon->cache_time;
 	gdk_pixbuf_unref (scaled_pixbuf);
 	return scaled_icon;
 }
@@ -1759,14 +1885,14 @@ revise_scale_factors_if_too_big (GdkPixbuf *pixbuf,
 				 double *scale_x,
 				 double *scale_y)
 {
-	int width, height;
+	guint width, height;
 	double y_distortion;
 
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
 
-	if ((int) (width * *scale_x) <= size->maximum_width
-	    && (int) (height * *scale_y) <= size->maximum_height) {
+	if ((int) (width * *scale_x) <= (int) size->maximum_width
+	    && (int) (height * *scale_y) <= (int) size->maximum_height) {
 		return;
 	}
 
@@ -1823,10 +1949,10 @@ load_icon_scale_if_necessary (NautilusScalableIcon *scalable_icon,
  * bumping the last item off that list if necessary.
  */
 static void
-mark_recently_used (NautilusCircularList *node)
+mark_recently_used (CircularList *node)
 {
 	NautilusIconFactory *factory;
-	NautilusCircularList *head, *last_node;
+	CircularList *head, *last_node;
 
 	check_recently_used_list ();
 
@@ -1870,46 +1996,32 @@ mark_recently_used (NautilusCircularList *node)
 	check_recently_used_list ();
 }
 
-/* Utility routine that checks if a cached icon has changed since it
- * was cached. It returns TRUE if the icon is still valid, and
- * removes it from the cache if it's not.
+/* Utility routine that checks if a cached thubmnail-type icon has
+ * changed since it was cached. Returns TRUE after removing the icon
+ * from the cache if the icon has changed.
  */
 static gboolean
-cached_icon_still_valid (const char *file_uri, time_t cached_time)
+remove_icons_if_file_changed (const char *file_uri, time_t cached_time)
 {
-	GnomeVFSURI *vfs_uri;
-	GnomeVFSFileInfo file_info;
 	GnomeVFSResult result;
-	gboolean is_local, is_valid;
+	time_t new_time;
 
-	/* If there's no specific file, simply return TRUE. */
-	if (file_uri == NULL) {
-		return TRUE;
-	}
-		
-	/* FIXME bugzilla.eazel.com 2566: if the URI is remote, assume
-	 * it's valid to avoid delay of testing. Eventually we'll have
-	 * to make this async to fix this.
-	 */
-	vfs_uri = gnome_vfs_uri_new (file_uri);
-	is_local = gnome_vfs_uri_is_local (vfs_uri);
-	gnome_vfs_uri_unref (vfs_uri);	
-	if (!is_local) {
-		return TRUE;
-	}
-	
-	/* Gather the info and then compare modification times. */
-	gnome_vfs_file_info_init (&file_info);
-	result = gnome_vfs_get_file_info (file_uri, &file_info, GNOME_VFS_FILE_INFO_DEFAULT);
-	is_valid = result == GNOME_VFS_OK && file_info.mtime <= cached_time;
-	gnome_vfs_file_info_clear (&file_info);
+	/* Get the time from the file. */
+	result = get_cache_time (file_uri, &new_time);
 
-	/* if it's not valid, remove it from the cache */
-	if (!is_valid) {
-		nautilus_icon_factory_remove_by_uri (file_uri);	
+	/* Do nothing for cases where a time doesn't apply. */
+	if (result == GNOME_VFS_ERROR_NOT_SUPPORTED) {
+		return FALSE;
 	}
-	
-	return is_valid;
+
+	/* Do nothing if the file is still the same as before. */
+	if (result == GNOME_VFS_OK && new_time == cached_time) {
+		return FALSE;
+	}
+
+	/* Remove the icon from the cache and inform the caller. */
+	nautilus_icon_factory_remove_by_uri (file_uri);	
+	return TRUE;
 }
 
 /* Get the icon, handling the caching.
@@ -1957,8 +2069,8 @@ get_icon_from_cache (NautilusScalableIcon *scalable_icon,
 		}
 
 		/* Check if the cached image is good before using it. */
-		if (!cached_icon_still_valid (scalable_icon->uri,
-					      icon->cache_time)) {
+		if (remove_icons_if_file_changed (scalable_icon->uri,
+						  icon->cache_time)) {
 			icon = NULL;
 		}
 	}
@@ -1996,6 +2108,7 @@ get_icon_from_cache (NautilusScalableIcon *scalable_icon,
 				icon = scaled_icon;
 			}
 		} else {
+			
 			if (scalable_icon->embedded_text != NULL) {
 				icon = load_icon_with_embedded_text (scalable_icon, size);
 			} else {
@@ -2033,7 +2146,8 @@ nautilus_icon_factory_get_pixbuf_for_icon (NautilusScalableIcon *scalable_icon,
 					   guint nominal_height,
 					   guint maximum_width,
 					   guint maximum_height,
-					   NautilusEmblemAttachPoints *attach_points)
+					   NautilusEmblemAttachPoints *attach_points,
+					   gboolean wants_default)
 {
 	IconSizeRequest size;
 	CacheIcon *icon;
@@ -2050,6 +2164,12 @@ nautilus_icon_factory_get_pixbuf_for_icon (NautilusScalableIcon *scalable_icon,
 		*attach_points = icon->details.attach_points;
 	}
 	
+	/* if we don't want a default icon and one is returned, return NULL instead */
+	if (!wants_default && icon->is_fallback) {
+		cache_icon_unref (icon);
+		return NULL;
+	}
+	
 	/* The first time we hand out an icon we just leave it with a
 	 * single ref (we'll get called back for the unref), but
 	 * subsequent times we add additional refs.
@@ -2143,7 +2263,8 @@ nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
 	pixbuf = nautilus_icon_factory_get_pixbuf_for_icon
 		(icon,
 		 size_in_pixels, size_in_pixels,
-		 size_in_pixels, size_in_pixels, NULL);
+		 size_in_pixels, size_in_pixels,
+		 NULL, TRUE);
 	nautilus_scalable_icon_unref (icon);
 
 	return pixbuf;
@@ -2176,6 +2297,24 @@ nautilus_icon_factory_get_pixmap_and_mask_for_file (NautilusFile *file,
 	gdk_pixbuf_unref (pixbuf);
 }
 
+/* Convenience routine for getting a pixbuf from an icon name
+ */
+GdkPixbuf * nautilus_icon_factory_get_pixbuf_from_name (const char *icon_name,
+						       const char *modifier,
+						       guint size_in_pixels,
+						       gboolean anti_aliased)
+{
+	GdkPixbuf *pixbuf;
+	NautilusScalableIcon *icon;
+	
+	icon = nautilus_scalable_icon_new_from_text_pieces (NULL, icon_name, modifier, NULL, anti_aliased);
+	pixbuf = nautilus_icon_factory_get_pixbuf_for_icon (icon, size_in_pixels, size_in_pixels,
+							    size_in_pixels, size_in_pixels, NULL, TRUE); 	
+	nautilus_scalable_icon_unref (icon);	
+	return pixbuf;
+}
+									  
+
 static gboolean
 embedded_text_rect_usable (const ArtIRect *embedded_text_rect)
 {
@@ -2193,18 +2332,12 @@ embedded_text_rect_usable (const ArtIRect *embedded_text_rect)
 	return TRUE;
 }
 
-/* FIXME bugzilla.eazel.com 2783: Icon factory needs support for speciiy whether
- * embedded text Embedded text should be smoother or not.
- */
-static gboolean smooth_graphics = TRUE;
-
 static GdkPixbuf *
 embed_text (GdkPixbuf *pixbuf_without_text,
 	    const ArtIRect *embedded_text_rect,
 	    const char *text)
 {
 	NautilusScalableFont *smooth_font;
-	static GdkFont *font;
 	GdkPixbuf *pixbuf_with_text;
 	
 	g_return_val_if_fail (pixbuf_without_text != NULL, NULL);
@@ -2218,46 +2351,37 @@ embed_text (GdkPixbuf *pixbuf_without_text,
 	}
 		
 	pixbuf_with_text = gdk_pixbuf_copy (pixbuf_without_text);
-		
-	if (smooth_graphics) {
-		smooth_font = NAUTILUS_SCALABLE_FONT
-			(nautilus_scalable_font_new
-			 (EMBEDDED_TEXT_FONT_FAMILY,
-			  EMBEDDED_TEXT_FONT_WEIGHT,
-			  EMBEDDED_TEXT_FONT_SLANT,
-			  EMBEDDED_TEXT_FONT_SET_WIDTH));
+	
+	/* FIXME bugzilla.eazel.com 1102: Embedded text should use preferences to determine
+	 * the font it uses
+	 */
 
-		nautilus_scalable_font_draw_text_lines
-			(smooth_font,
-			 pixbuf_with_text,
-			 embedded_text_rect->x0,
-			 embedded_text_rect->y0,
-			 embedded_text_rect,
-			 EMBEDDED_TEXT_FONT_SIZE,
-			 EMBEDDED_TEXT_FONT_SIZE,
-			 text,
-			 GTK_JUSTIFY_LEFT,
-			 EMBEDDED_TEXT_LINE_OFFSET,
-			 EMBEDDED_TEXT_EMPTY_LINE_HEIGHT,
-			 NAUTILUS_RGB_COLOR_BLACK,
-			 255,
-			 FALSE);
-		
-		gtk_object_unref (GTK_OBJECT (smooth_font));
-	} else {
-		/* Get the font the first time through. */
-		if (font == NULL) {
-			/* FIXME bugzilla.eazel.com 1102: Embedded text should use preferences to determine
-			 * the font it uses
-			 */
-			font = gdk_fontset_load (_("-*-helvetica-medium-r-normal-*-10-*-*-*-*-*-*-*"));
-			g_return_val_if_fail (font != NULL, NULL);
-		}
-		
-		nautilus_gdk_pixbuf_draw_text
-			(pixbuf_with_text, font, 1.0, embedded_text_rect, 
-			 text, NAUTILUS_RGB_COLOR_BLACK, 0xFF);
-	}
+	/* FIXME bugzilla.eazel.com 2783: It would be nice if embedded didnt always draw
+	 * anti aliased based on the user's smooth graphics preference.  It is however
+	 * a questionable improvement since at that tiny font size, anti aliased fonts
+	 * probably give a better preview of the content than non anti-aliased fonts.
+	 */
+	smooth_font = nautilus_scalable_font_new (EMBEDDED_TEXT_FONT_FAMILY,
+						  EMBEDDED_TEXT_FONT_WEIGHT,
+						  EMBEDDED_TEXT_FONT_SLANT,
+						  EMBEDDED_TEXT_FONT_SET_WIDTH);
+	
+	nautilus_scalable_font_draw_text_lines
+		(smooth_font,
+		 pixbuf_with_text,
+		 embedded_text_rect->x0,
+		 embedded_text_rect->y0,
+		 embedded_text_rect,
+		 EMBEDDED_TEXT_FONT_SIZE,
+		 EMBEDDED_TEXT_FONT_SIZE,
+		 text,
+		 GTK_JUSTIFY_LEFT,
+		 EMBEDDED_TEXT_LINE_OFFSET,
+		 EMBEDDED_TEXT_EMPTY_LINE_HEIGHT,
+		 NAUTILUS_RGB_COLOR_BLACK,
+		 NAUTILUS_OPACITY_FULLY_OPAQUE);
+	
+	gtk_object_unref (GTK_OBJECT (smooth_font));
 
 	return pixbuf_with_text;
 }
@@ -2300,6 +2424,7 @@ load_icon_with_embedded_text (NautilusScalableIcon *scalable_icon,
 					 icon_without_text->custom,
 					 icon_without_text->scaled,
 					 &details);
+	icon_with_text->cache_time = icon_without_text->cache_time;
 	cache_icon_unref (icon_without_text);
 	gdk_pixbuf_unref (pixbuf_with_text);
 
