@@ -25,6 +25,7 @@
 #ifndef NAUTILUS_ICON_CONTAINER_PRIVATE_H
 #define NAUTILUS_ICON_CONTAINER_PRIVATE_H
 
+#include "nautilus-entry.h"
 #include "nautilus-glib-extensions.h"
 #include "nautilus-icon-container.h"
 #include "nautilus-icon-dnd.h"
@@ -107,7 +108,8 @@ struct NautilusIconContainerDetails {
 
 	/* Current icon with stretch handles, so we have only one. */
 	NautilusIcon *stretch_icon;
-	int initial_stretch_size;
+	double stretch_initial_x, stretch_initial_y;
+	guint stretch_initial_size;
 	
 	/* Last highlighted drop target. */
 	NautilusIcon *drop_target;
@@ -143,14 +145,11 @@ struct NautilusIconContainerDetails {
 	gboolean drag_started;
 	StretchState stretch_start;
 	int context_menu_timeout_id;
-	int auto_scroll_timeout_id;
-	gboolean waiting_to_autoscroll;
-	gint64 start_auto_scroll_in;
 
 	/* Renaming Details */
 	gboolean renaming;
 	NautilusIconTextItem *rename_widget;	/* Editable text item */
-	gchar *original_text;			/* Copy of editable text for later compare */
+	char *original_text;			/* Copy of editable text for later compare */
 
 	/* typeahead selection state */
 	TypeSelectState *type_select_state;
@@ -198,9 +197,15 @@ struct NautilusIconContainerDetails {
 	
 	/* Layout mode */
 	NautilusIconLayoutMode layout_mode;
+
+	/* Set to TRUE after first allocation has been done */
+	gboolean has_been_allocated;
 	
 	/* Is the container fixed or resizable */
 	gboolean is_fixed_size;
+
+	/* Ignore the visible area the next time the scroll region is recomputed */
+	gboolean reset_scroll_region_trigger;
 };
 
 /* Private functions shared by mutiple files. */
@@ -231,8 +236,6 @@ gboolean      nautilus_icon_container_scroll                      (NautilusIconC
 								   int                    delta_x,
 								   int                    delta_y);
 void          nautilus_icon_container_update_scroll_region        (NautilusIconContainer *container);
-void          nautilus_icon_container_update_scroll_region_include_visible_area        
-								  (NautilusIconContainer *container);
 
 /* label color for items */
 guint32       nautilus_icon_container_get_label_color             (NautilusIconContainer *container,
