@@ -29,6 +29,7 @@
 #include "nautilus-directory.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-file.h"
+#include "nautilus-file-attributes.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-metadata.h"
 #include "nautilus-preferences.h"
@@ -107,6 +108,11 @@ nautilus_link_local_create (const char *directory_path,
 	char *uri;
 	GList dummy_list;
 	
+	g_return_val_if_fail (directory_path != NULL, FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (image != NULL, FALSE);
+	g_return_val_if_fail (target_uri != NULL, FALSE);
+	
 	/* create a new xml document */
 	output_document = xmlNewDoc ("1.0");
 	
@@ -245,6 +251,17 @@ nautilus_link_local_set_icon (const char *path, const char *icon_name)
 					NULL);
 }
 
+
+static void
+forget_file_activation_uri (NautilusFile *file)
+{
+	GList *attributes;
+
+	attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI);
+	nautilus_file_invalidate_attributes (file, attributes);
+	g_list_free (attributes);
+}
+
 /* Set the link uri for a link file. This can only be called on local
  * paths, and only on files known to be link files.
  */
@@ -254,7 +271,7 @@ nautilus_link_local_set_link_uri (const char *path, const char *link_uri)
 	return local_set_root_property (path,
 					"LINK",
 					link_uri,
-					nautilus_file_forget_activation_uri);
+					forget_file_activation_uri);
 }
 
 gboolean
@@ -280,6 +297,8 @@ nautilus_link_local_get_additional_text (const char *path)
 static char *
 make_local_path (const char *image_uri)
 {
+	GnomeVFSResult result;
+	
 	char *escaped_uri, *local_directory_path, *local_directory_uri, *local_file_path;
 	
 	escaped_uri = gnome_vfs_escape_slashes (image_uri);		
@@ -291,11 +310,16 @@ make_local_path (const char *image_uri)
 
 	/* We must create the directory if it doesn't exist. */
 	local_directory_uri = gnome_vfs_get_uri_from_local_path (local_directory_path);
-	/* FIXME bugzilla.eazel.com 2494: Is it OK to ignore the error here? */
-	gnome_vfs_make_directory (local_directory_uri, REMOTE_ICON_DIR_PERMISSIONS);
-	g_free (local_directory_uri);
-
+	result = gnome_vfs_make_directory (local_directory_uri, REMOTE_ICON_DIR_PERMISSIONS);
+	if (result != GNOME_VFS_OK) {
+		g_free (local_directory_uri);
+		g_free (escaped_uri);
+		g_free (local_directory_path);
+		return NULL;
+	}
+			
 	local_file_path = nautilus_make_path (local_directory_path, escaped_uri);
+	g_free (local_directory_uri);
 	g_free (escaped_uri);
 	g_free (local_directory_path);
 
@@ -375,6 +399,10 @@ nautilus_link_local_get_image_uri (const char *path)
 	/* if the image is remote, see if we can find it in our local cache */
 	if (nautilus_is_remote_uri (icon_uri)) {
 		local_path = make_local_path (icon_uri);
+		if (local_path == NULL) {
+			g_free (icon_uri);
+			return NULL;
+		}
 		if (g_file_exists (local_path)) {
 			g_free (icon_uri);			
 			local_uri = gnome_vfs_get_uri_from_local_path (local_path);
