@@ -33,11 +33,12 @@
 
 #include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkclist.h>
 #include <gtk/gtkmain.h>
 
 #include <libgnomeui/gnome-pixmap.h>
 
+#include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
+
 #include <string.h>
 
 /* these are from GtkCList, for now we need to copy them here 
@@ -153,7 +154,8 @@ nautilus_list_column_title_initialize_class (gpointer klass)
 NautilusListColumnTitle *
 nautilus_list_column_title_new (void)
 {
-	return gtk_type_new (nautilus_list_column_title_get_type ());
+	return NAUTILUS_LIST_COLUMN_TITLE
+		(gtk_widget_new (nautilus_list_column_title_get_type (), NULL));
 }
 
 static void
@@ -232,16 +234,16 @@ nautilus_list_column_title_finalize (GtkObject *object)
 	if (column_title->details->up_indicator_pixmap != NULL) {
 		gdk_pixmap_unref (column_title->details->up_indicator_pixmap);
 		column_title->details->up_indicator_pixmap = NULL;
-
-		g_assert (column_title->details->up_indicator_mask != NULL);
+	}
+	if (column_title->details->up_indicator_mask != NULL) {
 		gdk_bitmap_unref (column_title->details->up_indicator_mask);
 		column_title->details->up_indicator_mask = NULL;
 	}
 	if (column_title->details->down_indicator_pixmap != NULL) {
 		gdk_pixmap_unref (column_title->details->down_indicator_pixmap);
 		column_title->details->down_indicator_pixmap = NULL;
-
-		g_assert (column_title->details->down_indicator_mask != NULL);
+	}
+	if (column_title->details->down_indicator_mask != NULL) {
 		gdk_bitmap_unref (column_title->details->down_indicator_mask);
 		column_title->details->down_indicator_mask = NULL;
 	}
@@ -289,9 +291,9 @@ nautilus_list_column_title_request (GtkWidget *widget, GtkRequisition *requisiti
 static const char *
 get_column_label_at (GtkWidget *column_title, int index)
 {
-	GtkCList *parent_clist;
+	NautilusCList *parent_clist;
 
-	parent_clist = GTK_CLIST (column_title->parent);
+	parent_clist = NAUTILUS_CLIST (column_title->parent);
 
 	return parent_clist->column[index].title;
 }
@@ -299,8 +301,8 @@ get_column_label_at (GtkWidget *column_title, int index)
 static void
 get_column_frame_at(GtkWidget *column_title, int index, GdkRectangle *result)
 {
-	GtkCList *parent_clist;
-	parent_clist = GTK_CLIST (column_title->parent);
+	NautilusCList *parent_clist;
+	parent_clist = NAUTILUS_CLIST (column_title->parent);
 
 	*result = parent_clist->column_title_area;
 	result->x = parent_clist->column[index].area.x - COLUMN_INSET;
@@ -328,7 +330,7 @@ load_up_indicator (const char **xpm_data,
 		return;
 	}
 
-	gdk_pixbuf_render_pixmap_and_mask (pixbuf, indicator_pixmap, indicator_mask, 127);
+	gdk_pixbuf_render_pixmap_and_mask (pixbuf, indicator_pixmap, indicator_mask, NAUTILUS_STANDARD_ALPHA_THRESHHOLD);
 
 	gdk_pixbuf_unref (pixbuf);
 }
@@ -433,13 +435,13 @@ nautilus_list_column_title_paint (GtkWidget *widget, GtkWidget *draw_target,
 				  GdkDrawable *target_drawable, GdkRectangle *area)
 {
 	NautilusListColumnTitle *column_title;
-	GtkCList *parent_clist;
+	NautilusCList *parent_clist;
 	int index;
 
-	g_assert (GTK_CLIST (widget->parent) != NULL);
+	g_assert (NAUTILUS_CLIST (widget->parent) != NULL);
 
 	column_title = NAUTILUS_LIST_COLUMN_TITLE(widget);
-	parent_clist = GTK_CLIST (widget->parent);
+	parent_clist = NAUTILUS_CLIST (widget->parent);
 
 	for (index = 0; index < parent_clist->columns; index++) {
 		GdkRectangle cell_rectangle;
@@ -691,10 +693,10 @@ in_column_rect (GtkWidget *widget, int x, int y)
 {
 	/* return the index of the column we hit or -1 */
 
-	GtkCList *parent_clist;
+	NautilusCList *parent_clist;
 	int index;
 
-	parent_clist = GTK_CLIST (widget->parent);
+	parent_clist = NAUTILUS_CLIST (widget->parent);
 	
 	for (index = 0; index < parent_clist->columns; index++) {
 		/* hit testing for column resizing */
@@ -721,10 +723,10 @@ in_resize_rect (GtkWidget *widget, int x, int y)
 {
 	/* return the index of the resize rect of a column we hit or -1 */
 
-	GtkCList *parent_clist;
+	NautilusCList *parent_clist;
 	int index;
 
-	parent_clist = GTK_CLIST (widget->parent);
+	parent_clist = NAUTILUS_CLIST (widget->parent);
 	
 	for (index = 0; index < parent_clist->columns; index++) {
 		/* hit testing for column resizing */
