@@ -43,6 +43,7 @@
 
 /* pointer casting for cells */
 #define NAUTILUS_CELL_PIXBUF_LIST(cell)	((NautilusCellPixbufList *) &(cell))
+#define NAUTILUS_CELL_PIXBUF(cell)	((GdkPixbuf *) ((cell).u.text))
 /* no #define for NAUTILUS_CELL_LINK_TEXT, use GTK_CELL_TEXT instead */
 
 /* returns the GList item for the nth row */
@@ -88,10 +89,12 @@ struct NautilusListClass {
 	NautilusCListClass parent_class;
 
 	/* Signal: invoke the popup menu for selected items */
-	void (* context_click_selection) (NautilusList *list, int row);
+	void (* context_click_selection) (NautilusList *list, 
+					  GdkEventButton *event);
 
 	/* Signal: invoke the popup menu for empty areas */
-	void (* context_click_background) (NautilusList *list);
+	void (* context_click_background) (NautilusList *list,
+					   GdkEventButton *event);
 
 	/* Signal: announce that one or more items have been activated. */
 	void (* activate) (NautilusList *list, GList *row_data_list);
@@ -149,6 +152,13 @@ void         nautilus_list_set_pixbuf_list       (NautilusList            *list,
 						  gint                     row,
 						  gint                     column,
 						  GList                   *pixbufs);
+void	     nautilus_list_set_pixbuf		 (NautilusList		  *list,
+						  int			   row_index,
+						  int			   column_index,
+						  GdkPixbuf		  *pixbuf);
+GdkPixbuf   *nautilus_list_get_pixbuf		 (NautilusList		  *list,
+						  int			   row_index,
+						  int			   column_index);
 void         nautilus_list_mark_cell_as_link     (NautilusList            *list,
 						  gint                     row,
 						  gint                     column);
