@@ -34,6 +34,7 @@
 
 struct NautilusMergedDirectoryDetails {
 	GList *directories;
+	GList *directories_not_done_loading;
 	GHashTable *callbacks;
 	GHashTable *monitors;
 };
@@ -53,10 +54,20 @@ typedef struct {
 typedef struct {
 	NautilusMergedDirectory *merged;
 
+	gboolean monitor_hidden_files;
+	gboolean monitor_backup_files;
 	GList *monitor_attributes;
 	gboolean force_reload;
 } MergedMonitor;
 
+enum {
+	ADD_REAL_DIRECTORY,
+	REMOVE_REAL_DIRECTORY,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 static void     nautilus_merged_directory_initialize       (gpointer                 object,
 							    gpointer                 klass);
 static void     nautilus_merged_directory_initialize_class (gpointer                 klass);
@@ -79,8 +90,7 @@ nautilus_merged_directory_initialize (gpointer object, gpointer klass)
 	merged->details = g_new0 (NautilusMergedDirectoryDetails, 1);
 	merged->details->callbacks = g_hash_table_new
 		(merged_callback_hash, merged_callback_equal);
-	merged->details->monitors = g_hash_table_new
-		(g_direct_hash, g_direct_equal);
+	merged->details->monitors = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -129,24 +139,6 @@ merged_callback_equal (gconstpointer merged_callback_as_pointer,
 		&& merged_callback->callback_data == merged_callback_2->callback_data;
 }
 
-/* Return true if any directory in the list does. */
-static gboolean
-merged_contains_file (NautilusDirectory *directory,
-		      NautilusFile *file)
-{
-	NautilusMergedDirectory *merged;
-	GList *p;
-
-	merged = NAUTILUS_MERGED_DIRECTORY (directory);
-
-	for (p = merged->details->directories; p != NULL; p = p->next) {
-		if (nautilus_directory_contains_file (p->data, file)) {
-			return TRUE;
-		}
-	}
-	return FALSE;
-}
-
 static void
 merged_callback_destroy (MergedCallback *merged_callback)
 {
@@ -186,8 +178,6 @@ merged_callback_remove_directory (MergedCallback *merged_callback,
 	merged_callback->non_ready_directories = g_list_remove
 		(merged_callback->non_ready_directories,
 		 directory);
-
-	/* Check if we are ready. */
 	merged_callback_check_done (merged_callback);
 }
 
@@ -210,18 +200,7 @@ directory_ready_callback (NautilusDirectory *directory,
 		 nautilus_file_list_copy (files));
 
 	/* Check if we are ready. */
-	merged_callback_remove_directory (merged_callback,
-					  directory);
-}
-
-static void
-merged_callback_connect_directory (MergedCallback *merged_callback,
-				   NautilusDirectory *real_merged)
-{
-	nautilus_directory_call_when_ready
-		(real_merged,
-		 merged_callback->wait_for_attributes,
-		 directory_ready_callback, merged_callback);
+	merged_callback_remove_directory (merged_callback, directory);
 }
 
 static void
@@ -232,7 +211,7 @@ merged_call_when_ready (NautilusDirectory *directory,
 {
 	NautilusMergedDirectory *merged;
 	MergedCallback search_key, *merged_callback;
-	GList *p;
+	GList *node;
 
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 
@@ -250,9 +229,9 @@ merged_call_when_ready (NautilusDirectory *directory,
 	merged_callback->callback = callback;
 	merged_callback->callback_data = callback_data;
 	merged_callback->wait_for_attributes = nautilus_g_str_list_copy (file_attributes);
-	for (p = merged->details->directories; p != NULL; p = p->next) {
+	for (node = merged->details->directories; node != NULL; node = node->next) {
 		merged_callback->non_ready_directories = g_list_prepend
-			(merged_callback->non_ready_directories, p->data);
+			(merged_callback->non_ready_directories, node->data);
 	}
 
 	/* Put it in the hash table. */
@@ -265,8 +244,11 @@ merged_call_when_ready (NautilusDirectory *directory,
 	}
 
 	/* Now tell all the directories about it. */
-	for (p = merged->details->directories; p != NULL; p = p->next) {
-		merged_callback_connect_directory (merged_callback, p->data);
+	for (node = merged->details->directories; node != NULL; node = node->next) {
+		nautilus_directory_call_when_ready
+			(node->data,
+			 merged_callback->wait_for_attributes,
+			 directory_ready_callback, merged_callback);
 	}
 }
 
@@ -277,7 +259,7 @@ merged_cancel_callback (NautilusDirectory *directory,
 {
 	NautilusMergedDirectory *merged;
 	MergedCallback search_key, *merged_callback;
-	GList *p;
+	GList *node;
 
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 
@@ -293,9 +275,9 @@ merged_cancel_callback (NautilusDirectory *directory,
 	g_hash_table_remove (merged_callback->merged->details->callbacks, merged_callback);
 
 	/* Tell all the directories to cancel the call. */
-	for (p = merged_callback->non_ready_directories; p != NULL; p = p->next) {
+	for (node = merged_callback->non_ready_directories; node != NULL; node = node->next) {
 		nautilus_directory_cancel_callback
-			(p->data,
+			(node->data,
 			 directory_ready_callback, merged_callback);
 	}
 	merged_callback_destroy (merged_callback);
@@ -305,12 +287,14 @@ merged_cancel_callback (NautilusDirectory *directory,
 static void
 merged_file_monitor_add (NautilusDirectory *directory,
 			 gconstpointer client,
+			 gboolean monitor_hidden_files,
+			 gboolean monitor_backup_files,
 			 GList *file_attributes,
 			 gboolean force_reload)
 {
 	NautilusMergedDirectory *merged;
 	MergedMonitor *monitor;
-	GList *p;
+	GList *node;
 
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 
@@ -327,13 +311,16 @@ merged_file_monitor_add (NautilusDirectory *directory,
 		g_hash_table_insert (merged->details->monitors,
 				     (gpointer) client, monitor);
 	}
+	monitor->monitor_hidden_files = monitor_hidden_files;
+	monitor->monitor_backup_files = monitor_backup_files;
 	monitor->monitor_attributes = nautilus_g_str_list_copy (file_attributes);
 	monitor->force_reload = force_reload;
 	
 	/* Call through to the real directory add calls. */
-	for (p = merged->details->directories; p != NULL; p = p->next) {
+	for (node = merged->details->directories; node != NULL; node = node->next) {
 		nautilus_directory_file_monitor_add
-			(p->data, monitor,
+			(node->data, monitor,
+			 monitor_hidden_files, monitor_backup_files,
 			 file_attributes, force_reload);
 	}
 }
@@ -345,7 +332,7 @@ merged_file_monitor_remove (NautilusDirectory *directory,
 {
 	NautilusMergedDirectory *merged;
 	MergedMonitor *monitor;
-	GList *p;
+	GList *node;
 	
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 	
@@ -357,26 +344,44 @@ merged_file_monitor_remove (NautilusDirectory *directory,
 	g_hash_table_remove (merged->details->monitors, client);
 
 	/* Call through to the real directory remove calls. */
-	for (p = merged->details->directories; p != NULL; p = p->next) {
+	for (node = merged->details->directories; node != NULL; node = node->next) {
 		nautilus_directory_file_monitor_remove
-			(p->data, monitor);
+			(node->data, monitor);
 	}
 
 	nautilus_g_list_free_deep (monitor->monitor_attributes);
 	g_free (monitor);
 }
 
+/* Return true if any directory in the list does. */
+static gboolean
+merged_contains_file (NautilusDirectory *directory,
+		      NautilusFile *file)
+{
+	NautilusMergedDirectory *merged;
+	GList *node;
+
+	merged = NAUTILUS_MERGED_DIRECTORY (directory);
+
+	for (node = merged->details->directories; node != NULL; node = node->next) {
+		if (nautilus_directory_contains_file (node->data, file)) {
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 /* Return true only if all directories in the list do. */
 static gboolean
 merged_are_all_files_seen (NautilusDirectory *directory)
 {
 	NautilusMergedDirectory *merged;
-	GList *p;
+	GList *node;
 
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 
-	for (p = merged->details->directories; p != NULL; p = p->next) {
-		if (!nautilus_directory_are_all_files_seen (p->data)) {
+	for (node = merged->details->directories; node != NULL; node = node->next) {
+		if (!nautilus_directory_are_all_files_seen (node->data)) {
 			return FALSE;
 		}
 	}
@@ -388,38 +393,18 @@ static gboolean
 merged_is_not_empty (NautilusDirectory *directory)
 {
 	NautilusMergedDirectory *merged;
-	GList *p;
+	GList *node;
 
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 
-	for (p = merged->details->directories; p != NULL; p = p->next) {
-		if (nautilus_directory_is_not_empty (p->data)) {
+	for (node = merged->details->directories; node != NULL; node = node->next) {
+		if (nautilus_directory_is_not_empty (node->data)) {
 			return TRUE;
 		}
 	}
 	return FALSE;
 }
 
-static void
-nautilus_merged_directory_initialize_class (gpointer klass)
-{
-	GtkObjectClass *object_class;
-	NautilusDirectoryClass *directory_class;
-
-	object_class = GTK_OBJECT_CLASS (klass);
-	directory_class = NAUTILUS_DIRECTORY_CLASS (klass);
-	
-	object_class->destroy = merged_destroy;
-
-	directory_class->contains_file = merged_contains_file;
-	directory_class->call_when_ready = merged_call_when_ready;
-	directory_class->cancel_callback = merged_cancel_callback;
-	directory_class->file_monitor_add = merged_file_monitor_add;
-	directory_class->file_monitor_remove = merged_file_monitor_remove;
- 	directory_class->are_all_files_seen = merged_are_all_files_seen;
-	directory_class->is_not_empty = merged_is_not_empty;
-}
-
 static void
 forward_files_added_cover (NautilusDirectory *real_directory,
 			   GList *files,
@@ -436,6 +421,17 @@ forward_files_changed_cover (NautilusDirectory *real_directory,
 	nautilus_directory_emit_files_changed (NAUTILUS_DIRECTORY (merged), files);
 }
 
+static void
+done_loading_callback (NautilusDirectory *real_directory,
+		       NautilusMergedDirectory *merged)
+{
+	merged->details->directories_not_done_loading = g_list_remove
+		(merged->details->directories_not_done_loading, real_directory);
+	if (merged->details->directories_not_done_loading == NULL) {
+		nautilus_directory_emit_done_loading (NAUTILUS_DIRECTORY (merged));
+	}
+}
+
 static void
 monitor_add_directory (gpointer key,
 		       gpointer value,
@@ -446,10 +442,53 @@ monitor_add_directory (gpointer key,
 	monitor = value;
 	nautilus_directory_file_monitor_add
 		(NAUTILUS_DIRECTORY (callback_data), monitor,
+		 monitor->monitor_hidden_files,
+		 monitor->monitor_backup_files,
 		 monitor->monitor_attributes,
 		 monitor->force_reload);
 }
 
+static void
+merged_add_real_directory (NautilusMergedDirectory *merged,
+			   NautilusDirectory *real_directory)
+{
+	g_return_if_fail (NAUTILUS_IS_MERGED_DIRECTORY (merged));
+	g_return_if_fail (NAUTILUS_IS_DIRECTORY (real_directory));
+	g_return_if_fail (!NAUTILUS_IS_MERGED_DIRECTORY (real_directory));
+	g_return_if_fail (g_list_find (merged->details->directories, real_directory) == NULL);
+
+	/* Add to our list of directories. */
+	nautilus_directory_ref (real_directory);
+	merged->details->directories = g_list_prepend
+		(merged->details->directories, real_directory);
+	merged->details->directories_not_done_loading = g_list_prepend
+		(merged->details->directories_not_done_loading, real_directory);
+
+	/* Connect signals. */
+	gtk_signal_connect (GTK_OBJECT (real_directory),
+			    "files_added",
+			    forward_files_added_cover,
+			    merged);
+	gtk_signal_connect (GTK_OBJECT (real_directory),
+			    "files_changed",
+			    forward_files_changed_cover,
+			    merged);
+	gtk_signal_connect (GTK_OBJECT (real_directory),
+			    "done_loading",
+			    done_loading_callback,
+			    merged);
+
+	/* FIXME bugzilla.eazel.com 5084: The done_loading part won't work for the case where
+         * we have no directories in our list.
+	 */
+
+	/* Add the directory to any extant monitors. */
+	g_hash_table_foreach (merged->details->monitors,
+			      monitor_add_directory,
+			      real_directory);
+	/* FIXME bugzilla.eazel.com 2541: Do we need to add the directory to callbacks too? */
+}
+
 void
 nautilus_merged_directory_add_real_directory (NautilusMergedDirectory *merged,
 					      NautilusDirectory *real_directory)
@@ -463,27 +502,15 @@ nautilus_merged_directory_add_real_directory (NautilusMergedDirectory *merged,
 		return;
 	}
 
-	/* Add to our list of directories. */
-	nautilus_directory_ref (real_directory);
-	merged->details->directories = g_list_prepend
-		(merged->details->directories, real_directory);
+	gtk_signal_emit (GTK_OBJECT (merged),
+			 signals[ADD_REAL_DIRECTORY],
+			 real_directory);
+}
 
-	/* Connect signals. */
-	gtk_signal_connect (GTK_OBJECT (real_directory),
-			    "files_added",
-			    forward_files_added_cover,
-			    merged);
-	gtk_signal_connect (GTK_OBJECT (real_directory),
-			    "files_changed",
-			    forward_files_changed_cover,
-			    merged);
-	/* FIXME bugzilla.eazel.com 2540: Hook up the done_loading signal too? */
-
-	/* Add the directory to any extant monitors. */
-	g_hash_table_foreach (merged->details->monitors,
-			      monitor_add_directory,
-			      real_directory);
-	/* FIXME bugzilla.eazel.com 2541: Do we need to add the directory to callbacks too? */
+GList *
+nautilus_merged_directory_get_real_directories (NautilusMergedDirectory *merged)
+{
+	return g_list_copy (merged->details->directories);
 }
 
 static void
@@ -504,17 +531,13 @@ monitor_remove_directory (gpointer key,
 		(NAUTILUS_DIRECTORY (callback_data), value);
 }
 
-void
-nautilus_merged_directory_remove_real_directory (NautilusMergedDirectory *merged,
-						 NautilusDirectory *real_directory)
+static void
+merged_remove_real_directory (NautilusMergedDirectory *merged,
+			      NautilusDirectory *real_directory)
 {
 	g_return_if_fail (NAUTILUS_IS_MERGED_DIRECTORY (merged));
 	g_return_if_fail (NAUTILUS_IS_DIRECTORY (real_directory));
-
-	/* Quietly do nothing if asked to remove something that's not there. */
-	if (g_list_find (merged->details->directories, real_directory) == NULL) {
-		return;
-	}
+	g_return_if_fail (g_list_find (merged->details->directories, real_directory) != NULL);
 
 	/* Remove this directory from callbacks and monitors. */
 	nautilus_g_hash_table_safe_for_each
@@ -532,9 +555,27 @@ nautilus_merged_directory_remove_real_directory (NautilusMergedDirectory *merged
 	/* Remove from our list of directories. */
 	merged->details->directories = g_list_remove
 		(merged->details->directories, real_directory);
+	merged->details->directories_not_done_loading = g_list_remove
+		(merged->details->directories_not_done_loading, real_directory);
 	nautilus_directory_unref (real_directory);
 }
 
+void
+nautilus_merged_directory_remove_real_directory (NautilusMergedDirectory *merged,
+						 NautilusDirectory *real_directory)
+{
+	g_return_if_fail (NAUTILUS_IS_MERGED_DIRECTORY (merged));
+
+	/* Quietly do nothing if asked to remove something that's not there. */
+	if (g_list_find (merged->details->directories, real_directory) == NULL) {
+		return;
+	}
+
+	gtk_signal_emit (GTK_OBJECT (merged),
+			 signals[REMOVE_REAL_DIRECTORY],
+			 real_directory);
+}
+
 static void
 remove_all_real_directories (NautilusMergedDirectory *merged)
 {
@@ -543,3 +584,47 @@ remove_all_real_directories (NautilusMergedDirectory *merged)
 			(merged, merged->details->directories->data);
 	}
 }
+
+static void
+nautilus_merged_directory_initialize_class (gpointer klass)
+{
+	GtkObjectClass *object_class;
+	NautilusDirectoryClass *directory_class;
+	NautilusMergedDirectoryClass *merged_directory_class;
+
+	object_class = GTK_OBJECT_CLASS (klass);
+	directory_class = NAUTILUS_DIRECTORY_CLASS (klass);
+	merged_directory_class = NAUTILUS_MERGED_DIRECTORY_CLASS (klass);
+	
+	object_class->destroy = merged_destroy;
+
+	directory_class->contains_file = merged_contains_file;
+	directory_class->call_when_ready = merged_call_when_ready;
+	directory_class->cancel_callback = merged_cancel_callback;
+	directory_class->file_monitor_add = merged_file_monitor_add;
+	directory_class->file_monitor_remove = merged_file_monitor_remove;
+ 	directory_class->are_all_files_seen = merged_are_all_files_seen;
+	directory_class->is_not_empty = merged_is_not_empty;
+
+	merged_directory_class->add_real_directory = merged_add_real_directory;
+	merged_directory_class->remove_real_directory = merged_remove_real_directory;
+
+	signals[ADD_REAL_DIRECTORY] 
+		= gtk_signal_new ("add_real_directory",
+				  GTK_RUN_LAST,
+				  object_class->type,
+				  GTK_SIGNAL_OFFSET (NautilusMergedDirectoryClass, 
+						     add_real_directory),
+				  gtk_marshal_NONE__POINTER,
+				  GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+	signals[REMOVE_REAL_DIRECTORY] 
+		= gtk_signal_new ("remove_real_directory",
+				  GTK_RUN_LAST,
+				  object_class->type,
+				  GTK_SIGNAL_OFFSET (NautilusMergedDirectoryClass, 
+						     remove_real_directory),
+				  gtk_marshal_NONE__POINTER,
+				  GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
+	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);				  
+}
