@@ -25,63 +25,40 @@
 #include <config.h>
 #include "nautilus-mime-actions.h"
 
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-mime-info.h>
-#include <libgnomevfs/gnome-vfs-application-registry.h>
-#include "nautilus-lib-self-check-functions.h"
-#include "nautilus-file.h"
 #include "nautilus-file-attributes.h"
+#include "nautilus-file.h"
 #include "nautilus-glib-extensions.h"
+#include "nautilus-lib-self-check-functions.h"
 #include "nautilus-metadata.h"
-#include "nautilus-wait-until-ready.h"
-
+#include <libgnomevfs/gnome-vfs-application-registry.h>
+#include <libgnomevfs/gnome-vfs-mime-info.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <stdio.h>
-
-
-static gint        gnome_vfs_mime_application_has_id             (GnomeVFSMimeApplication *application, 
-								  const char              *id);
-static gint        gnome_vfs_mime_id_matches_application         (const char              *id, 
-								  GnomeVFSMimeApplication *application);
-static gboolean    gnome_vfs_mime_application_has_id_not_in_list (GnomeVFSMimeApplication *application, 
-								  GList                   *ids);
-static gboolean    string_not_in_list                            (const char              *str, 
-								  GList                   *list);
-static char       *extract_prefix_add_suffix                     (const char              *string, 
-								  const char              *separator, 
-								  const char              *suffix);
-static char       *mime_type_get_supertype                       (const char              *mime_type);
-static GList      *get_explicit_content_view_iids_from_metafile  (NautilusFile            *file);
-static char       *make_oaf_query_for_explicit_content_view_iids (GList                   *view_iids);
-static char       *make_oaf_query_with_known_mime_type           (const char              *mime_type, 
-								  const char              *uri_scheme, 
-								  GList                   *explicit_iids, 
-								  const char              *extra_requirements);
-static char       *make_oaf_query_with_uri_scheme_only           (const char              *uri_scheme, 
-								  GList                   *explicit_iids, 
-								  const char              *extra_requirements);
-static GHashTable *mime_type_list_to_hash_table                  (GList                   *files);
-static void        free_key                                      (gpointer                 key, 
-								  gpointer                 value, 
-								  gpointer                 user_data);
-static void        mime_type_hash_table_destroy                  (GHashTable              *table);
-static gboolean    server_has_content_requirements               (OAF_ServerInfo          *server);
-static gboolean    server_matches_content_requirements           (OAF_ServerInfo          *server, 
-								  GHashTable              *type_table, 
-								  GList                   *explicit_iids);
-static GList      *nautilus_do_component_query                   (const char              *mime_type, 
-								  const char              *uri_scheme, 
-								  GList                   *content_mime_types,
-								  gboolean                 ignore_content_mime_types,
-								  GList                   *explicit_iids,
-								  char                   **extra_sort_criteria,
-								  char                    *extra_requirements,
-								  CORBA_Environment       *ev);
-static GList      *str_list_difference                           (GList                   *a, 
-								  GList                   *b);
-static int         strv_length                                   (char                   **a);
-static char      **strv_concat                                   (char                   **a, 
-								  char                   **b);
+ 
+static int         gnome_vfs_mime_application_has_id             (GnomeVFSMimeApplication  *application,
+								  const char               *id);
+static int         gnome_vfs_mime_id_matches_application         (const char               *id,
+								  GnomeVFSMimeApplication  *application);
+static gboolean    gnome_vfs_mime_application_has_id_not_in_list (GnomeVFSMimeApplication  *application,
+								  GList                    *ids);
+static gboolean    string_not_in_list                            (const char               *str,
+								  GList                    *list);
+static char       *mime_type_get_supertype                       (const char               *mime_type);
+static GList      *get_explicit_content_view_iids_from_metafile  (NautilusFile             *file);
+static gboolean    server_has_content_requirements               (OAF_ServerInfo           *server);
+static GList      *nautilus_do_component_query                   (const char               *mime_type,
+								  const char               *uri_scheme,
+								  GList                    *content_mime_types,
+								  gboolean                  ignore_content_mime_types,
+								  GList                    *explicit_iids,
+								  char                    **extra_sort_criteria,
+								  char                     *extra_requirements,
+								  CORBA_Environment        *ev);
+static GList      *str_list_difference                           (GList                    *a,
+								  GList                    *b);
+static char      **strv_concat                                   (char                    **a,
+								  char                    **b);
 
 static gboolean
 is_known_mime_type (const char *mime_type)
@@ -90,15 +67,13 @@ is_known_mime_type (const char *mime_type)
 		return FALSE;
 	}
 	
-	if (strcasecmp (mime_type, "application/octet-stream") == 0) {
+	if (strcasecmp (mime_type, "GNOME_VFS_MIME_TYPE_UNKNOWN") == 0) {
 		return FALSE;
 	}
 	
 	return TRUE;
 }
 
-
-
 static gboolean
 nautilus_mime_actions_check_if_minimum_attributes_ready (NautilusFile *file)
 {
@@ -127,7 +102,7 @@ nautilus_mime_actions_check_if_full_attributes_ready (NautilusFile *file)
 
 
 GList *
-nautilus_mime_actions_get_minimum_file_attributes ()
+nautilus_mime_actions_get_minimum_file_attributes (void)
 {
 	GList *attributes;
 
@@ -140,7 +115,7 @@ nautilus_mime_actions_get_minimum_file_attributes ()
 
 
 GList *
-nautilus_mime_actions_get_full_file_attributes ()
+nautilus_mime_actions_get_full_file_attributes (void)
 {
 	GList *attributes;
 
@@ -152,19 +127,6 @@ nautilus_mime_actions_get_full_file_attributes ()
 
 
 
-void
-nautilus_mime_actions_wait_for_full_file_attributes (NautilusFile *file)
-{
-	GList *attributes;
-	
-	attributes = nautilus_mime_actions_get_full_file_attributes ();
-	
-	nautilus_file_wait_until_ready (file, attributes);
-	
-	g_list_free (attributes);
-}
-
-
 GnomeVFSMimeActionType
 nautilus_mime_get_default_action_type_for_file (NautilusFile *file)
 {
@@ -172,8 +134,9 @@ nautilus_mime_get_default_action_type_for_file (NautilusFile *file)
 	char *action_type_string;
 	GnomeVFSMimeActionType action_type;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      GNOME_VFS_MIME_ACTION_TYPE_NONE);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return GNOME_VFS_MIME_ACTION_TYPE_NONE;
+	}
 
 	action_type_string = nautilus_file_get_metadata
 		(file, NAUTILUS_METADATA_KEY_DEFAULT_ACTION_TYPE, NULL);
@@ -195,12 +158,13 @@ nautilus_mime_get_default_action_type_for_file (NautilusFile *file)
 }
 
 GnomeVFSMimeAction *
-nautilus_mime_get_default_action_for_file (NautilusFile      *file)
+nautilus_mime_get_default_action_for_file (NautilusFile *file)
 {
 	GnomeVFSMimeAction *action;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	action = g_new0 (GnomeVFSMimeAction, 1);
 
@@ -243,15 +207,17 @@ nautilus_mime_get_default_application_for_file_internal (NautilusFile *file,
 	char *default_application_string;
 	gboolean used_user_chosen_info;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	used_user_chosen_info = TRUE;
 
 	default_application_string = nautilus_file_get_metadata 
 		(file, NAUTILUS_METADATA_KEY_DEFAULT_APPLICATION, NULL);
 
-	/* FIXME: should fall back to normal default if user-specified default is bogus */
+	/* FIXME bugzilla.eazel.com 5085: should fall back to normal default 
+	   if user-specified default is bogus */
 
 	if (default_application_string == NULL) {
 		mime_type = nautilus_file_get_mime_type (file);
@@ -387,8 +353,9 @@ nautilus_mime_get_default_component_for_file_internal (NautilusFile *file,
 	gboolean used_user_chosen_info;
 	gboolean metadata_default;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	used_user_chosen_info = TRUE;
 
@@ -428,12 +395,6 @@ nautilus_mime_get_default_component_for_file_internal (NautilusFile *file,
 		metadata_default = TRUE;
 	}
 	
-	/* FIXME bugzilla.eazel.com 1264: 
-	 * should return NULL for the following cases:
-	 * - Known URI scheme pointing to nonexistent location
-	 * - Syntactically invalid URI
-	 */
-	
 	sort_conditions = nautilus_mime_get_default_component_sort_conditions (file, default_component_string);
 
 	/* If the default is specified in the per-uri metadata,
@@ -517,8 +478,9 @@ nautilus_mime_get_short_list_applications_for_file (NautilusFile      *file)
 	GnomeVFSMimeApplication *application;
 	CORBA_Environment ev;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	CORBA_exception_init (&ev);
 
@@ -579,8 +541,9 @@ nautilus_mime_get_short_list_components_for_file (NautilusFile      *file)
 	char *extra_requirements;
 	char *prev;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	CORBA_exception_init (&ev);
 
@@ -661,7 +624,7 @@ nautilus_mime_get_short_list_components_for_file (NautilusFile      *file)
 	return result;
 }
 
-/* FIXME: we should disable this for 1.0 I think */
+/* FIXME bugzilla.eazel.com 5086: we should disable this for 1.0 I think */
 
 char *
 nautilus_mime_get_short_list_methods_for_file (NautilusFile      *file)
@@ -669,11 +632,12 @@ nautilus_mime_get_short_list_methods_for_file (NautilusFile      *file)
 	char *mime_type;
 	const char *method;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	mime_type = nautilus_file_get_mime_type (file);
-	method = gnome_vfs_mime_get_value (mime_type, "vfs-method");
+	method = gnome_vfs_mime_get_value (mime_type, "vfs_method");
 	g_free (mime_type);
 	return g_strdup (method);
 }
@@ -687,8 +651,9 @@ nautilus_mime_get_all_applications_for_file (NautilusFile      *file)
 	GList *p;
 	GnomeVFSMimeApplication *application;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	metadata_application_ids = nautilus_file_get_metadata_list 
 		(file,
@@ -783,8 +748,9 @@ nautilus_mime_get_all_components_for_file (NautilusFile *file)
 	GList *explicit_iids;
 	CORBA_Environment ev;
 
-	g_return_val_if_fail (nautilus_mime_actions_check_if_minimum_attributes_ready (file), 
-			      NULL);
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
 
 	CORBA_exception_init (&ev);
 
@@ -1144,15 +1110,15 @@ nautilus_mime_remove_from_all_applications_for_file (NautilusFile *file,
 	return GNOME_VFS_OK;
 }
 
-static gint
+static int
 gnome_vfs_mime_application_has_id (GnomeVFSMimeApplication *application, 
-				   const char              *id)
+				   const char *id)
 {
 	return strcmp (application->id, id);
 }
 
-static gint
-gnome_vfs_mime_id_matches_application (const char              *id, 
+static int
+gnome_vfs_mime_id_matches_application (const char *id, 
 				       GnomeVFSMimeApplication *application)
 {
 	return gnome_vfs_mime_application_has_id (application, id);
@@ -1184,7 +1150,7 @@ extract_prefix_add_suffix (const char *string,
 
         separator_position = strstr (string, separator);
         prefix_length = separator_position == NULL
-                ? strlen (string)
+                ? (int) strlen (string)
                 : separator_position - string;
 
         result = g_malloc (prefix_length + strlen(suffix) + 1);
@@ -1500,7 +1466,7 @@ server_matches_content_requirements (OAF_ServerInfo *server,
 {
         OAF_Property *prop;
         GNOME_stringlist types;
-        int i;
+        guint i;
 
         /* Components explicitly requested in the metafile are not capability tested. */
         if (g_list_find_custom (explicit_iids, (gpointer) server->iid, (GCompareFunc) strcmp) != NULL) {
@@ -1510,7 +1476,7 @@ server_matches_content_requirements (OAF_ServerInfo *server,
         if (!server_has_content_requirements (server)) {
                 return TRUE;
         } else {
-        prop = oaf_server_info_prop_find (server, "nautilus:required_directory_content_mime_types");
+        	prop = oaf_server_info_prop_find (server, "nautilus:required_directory_content_mime_types");
 
                 types = prop->v._u.value_stringv;
 
@@ -1575,7 +1541,7 @@ nautilus_do_component_query (const char        *mime_type,
 
         if (ev->_major == CORBA_NO_EXCEPTION && oaf_result != NULL && oaf_result->_length > 0) {
                 GHashTable *content_types;
-                int i;
+                guint i;
            
                 content_types = mime_type_list_to_hash_table (item_mime_types);
                 
