@@ -24,6 +24,7 @@
 
 #include <config.h>
 #include "nautilus-preferences-dialog.h"
+#include "nautilus-gtk-extensions.h"
 #include "nautilus-gtk-macros.h"
 
 /* #include "caption-table.h" */
@@ -57,7 +58,7 @@ static const gint DEFAULT_BUTTON = 0;
 static const guint DEFAULT_BORDER_WIDTH = 0;
 
 static const guint PREFS_DIALOG_DEFAULT_WIDTH = 500;
-static const guint PREFS_DIALOG_DEFAULT_HEIGHT = 375;
+static const guint PREFS_DIALOG_DEFAULT_HEIGHT = 403;
 
 enum 
 {
@@ -83,6 +84,7 @@ static void dialog_destroy                         (GtkWidget                *wi
 /* Misc private stuff */
 static void nautilus_preferences_dialog_construct        (NautilusPreferencesDialog      *prefs_dialog,
 						    const gchar              *dialog_title);
+static void user_level_changed_callback (gpointer callback_data);
 
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusPreferencesDialog, 
@@ -113,6 +115,8 @@ nautilus_preferences_dialog_initialize (NautilusPreferencesDialog * prefs_dialog
 	prefs_dialog->details = g_new (NautilusPreferencesDialogDetails, 1);
 
 	prefs_dialog->details->prefs_box = NULL;
+
+	nautilus_preferences_add_callback ("user_level", user_level_changed_callback, prefs_dialog);
 }
 
 static void
@@ -180,6 +184,8 @@ nautilus_preferences_dialog_construct (NautilusPreferencesDialog *prefs_dialog,
 	gnome_dialog_set_default (GNOME_DIALOG(prefs_dialog), 
 				  DEFAULT_BUTTON);
 
+	nautilus_gtk_window_set_up_close_accelerator (GTK_WINDOW (prefs_dialog));
+
 	gtk_signal_connect (GTK_OBJECT (prefs_dialog),
 			    "clicked",
 			    GTK_SIGNAL_FUNC (dialog_clicked),
@@ -235,6 +241,8 @@ nautilus_preferences_dialog_destroy(GtkObject* object)
 	
 	prefs_dialog = NAUTILUS_PREFERENCES_DIALOG(object);
 
+	nautilus_preferences_remove_callback ("user_level", user_level_changed_callback, prefs_dialog);
+
 	g_free (prefs_dialog->details);
 
 	/* Chain */
@@ -250,3 +258,19 @@ nautilus_preferences_dialog_get_prefs_box (NautilusPreferencesDialog *prefs_dial
 
 	return prefs_dialog->details->prefs_box;
 }
+
+void
+nautilus_preferences_dialog_update (NautilusPreferencesDialog *preferences_dialog)
+{
+	g_return_if_fail (NAUTILUS_IS_PREFS_DIALOG (preferences_dialog));
+
+	nautilus_preferences_box_update (NAUTILUS_PREFERENCES_BOX (preferences_dialog->details->prefs_box));
+}
+
+static void
+user_level_changed_callback (gpointer callback_data)
+{
+	g_return_if_fail (NAUTILUS_IS_PREFS_DIALOG (callback_data));
+
+	nautilus_preferences_dialog_update (NAUTILUS_PREFERENCES_DIALOG (callback_data));
+}
