@@ -69,12 +69,13 @@ typedef enum
 	NAUTILUS_PREFERENCE_ITEM_FONT_FAMILY,
 	NAUTILUS_PREFERENCE_ITEM_THEME,
 	NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING,
+	NAUTILUS_PREFERENCE_ITEM_INTEGER
 } NautilusPreferencesItemType;
 
 GtkType    nautilus_preferences_item_get_type (void);
-GtkWidget* nautilus_preferences_item_new      (const gchar                 *preference_name,
-					       NautilusPreferencesItemType  item_type);
-
+GtkWidget* nautilus_preferences_item_new      (const gchar                   *preference_name,
+					       NautilusPreferencesItemType    item_type);
+char *     nautilus_preferences_item_get_name (const NautilusPreferencesItem *preferences_item);
 
 END_GNOME_DECLS
 
