@@ -27,21 +27,19 @@
 #include "nautilus-program-chooser.h"
 
 #include "nautilus-gnome-extensions.h"
-#include "nautilus-stock-dialogs.h"
 #include "nautilus-gtk-extensions.h"
-#include "nautilus-program-choosing.h"
-#include "nautilus-view-identifier.h"
 #include "nautilus-mime-actions.h"
-
-#include <gtk/gtkradiobutton.h>
+#include "nautilus-program-choosing.h"
+#include "nautilus-stock-dialogs.h"
+#include "nautilus-view-identifier.h"
 #include <gtk/gtkclist.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkradiobutton.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkvbox.h>
-
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-stock.h>
@@ -315,8 +313,6 @@ repopulate_program_list (GnomeDialog *program_chooser,
 	int new_row;
 	GnomeVFSMimeActionType type;
 
-	nautilus_mime_actions_wait_for_full_file_attributes (file);
-
 	type = nautilus_program_chooser_get_type (program_chooser);
 
 	g_assert (type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT
@@ -504,8 +500,6 @@ is_component_default_for_file (NautilusViewIdentifier *identifier, NautilusFile
 
 	g_assert (identifier != NULL);
 
-	nautilus_mime_actions_wait_for_full_file_attributes (file);
-
 	default_component = nautilus_mime_get_default_component_for_file (file);
 	result = (default_component != NULL && strcmp (default_component->iid, identifier->iid) == 0);
 
@@ -613,8 +607,6 @@ program_file_pair_is_default_for_file (ProgramFilePair *pair)
 	g_assert (pair != NULL);
 	g_assert (NAUTILUS_IS_FILE (pair->file));
 
-	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
-
 	if (pair->action_type != nautilus_mime_get_default_action_type_for_file (pair->file)) {
 		return FALSE;
 	}
@@ -658,8 +650,6 @@ program_file_pair_is_in_short_list_for_file (ProgramFilePair *pair)
 	g_assert (pair != NULL);
 	g_assert (NAUTILUS_IS_FILE (pair->file));
 
-	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
-
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
 		result = is_component_in_short_list_for_file (pair->view_identifier, pair->file);
 	} else {
@@ -775,8 +765,6 @@ pack_radio_button (GtkBox *box, const char *label_text, GtkRadioButton *group)
 static void
 add_to_short_list_for_file (ProgramFilePair *pair)
 {
-	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
-
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
 		nautilus_mime_add_application_to_short_list_for_file (pair->file, pair->application->id);
 	} else {
@@ -787,8 +775,6 @@ add_to_short_list_for_file (ProgramFilePair *pair)
 static void
 remove_from_short_list_for_file (ProgramFilePair *pair)
 {
-	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
-
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
 		nautilus_mime_remove_application_from_short_list_for_file (pair->file, pair->application->id);
 	} else {
@@ -851,8 +837,6 @@ remove_default_for_type (ProgramFilePair *pair)
 static void
 remove_default_for_item (ProgramFilePair *pair)
 {
-	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
-
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
 	    	/* If the default is just falling through to the default for this type,
 	    	 * don't do anything here.
@@ -895,8 +879,6 @@ set_default_for_type (ProgramFilePair *pair)
 static void
 set_default_for_item (ProgramFilePair *pair)
 {
-	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
-
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
 		nautilus_mime_set_default_application_for_file (pair->file, pair->application->id);
 	} else {
@@ -911,7 +893,7 @@ launch_mime_capplet (GtkWidget *button, gpointer ignored)
 {
 	g_assert (GTK_IS_WIDGET (button));
 
-	nautilus_launch_application_from_command ("nautilus-mime-type-capplet", NULL);
+	nautilus_launch_application_from_command ("nautilus-mime-type-capplet", NULL, FALSE);
 }
 
 static void
@@ -971,6 +953,7 @@ run_program_configurator_callback (GtkWidget *button, gpointer callback_data)
 				   GNOME_STOCK_BUTTON_CANCEL,
 				   NULL);
 	g_free (title);
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), "program_chooser", "Nautilus");
 
 	/* Labeled frame to avoid repeating text in each radio button,
 	 * and to look nice.
@@ -1220,7 +1203,7 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
 	GtkWidget *change_button_holder;
 	GtkWidget *change_button;
 	GtkWidget *capplet_button_frame, *capplet_hbox;
-	GtkWidget *capplet_button, *capption;
+	GtkWidget *capplet_button, *capption, *capplet_button_vbox;
 	char *file_name, *prompt;
 	const char *title;
 
@@ -1255,6 +1238,7 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
 	gtk_window_set_default_size (GTK_WINDOW (window), 
 				     NO_DEFAULT_MAGIC_NUMBER,
 				     PROGRAM_CHOOSER_DEFAULT_HEIGHT);
+	gtk_window_set_wmclass (GTK_WINDOW (window), "program_chooser", "Nautilus");
 
 	gtk_object_set_data (GTK_OBJECT (window), "type", GINT_TO_POINTER (action_type));
 
@@ -1308,7 +1292,7 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
   	gtk_box_pack_end (GTK_BOX (framed_hbox), change_button_holder, FALSE, FALSE, 0);
 
   	change_button = gtk_button_new_with_label(_("Modify..."));
-	nautilus_gtk_button_set_padding (GTK_BUTTON (change_button), GNOME_PAD_SMALL);
+	nautilus_gtk_button_set_standard_padding (GTK_BUTTON (change_button));
   	gtk_widget_show (change_button);
   	gtk_box_pack_end (GTK_BOX (change_button_holder), change_button, TRUE, FALSE, 0);
 
@@ -1327,14 +1311,17 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
   	gtk_container_add (GTK_CONTAINER (capplet_button_frame), capplet_hbox);
   	gtk_container_set_border_width (GTK_CONTAINER (capplet_hbox), GNOME_PAD);
 
+	capplet_button_vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (capplet_button_vbox);
+	gtk_box_pack_end (GTK_BOX (capplet_hbox), capplet_button_vbox, FALSE, FALSE, 0);
 	capplet_button = gtk_button_new_with_label (_("Go There"));	 
-	nautilus_gtk_button_set_padding (GTK_BUTTON (capplet_button), GNOME_PAD_SMALL);
+	nautilus_gtk_button_set_standard_padding (GTK_BUTTON (capplet_button));
 	gtk_signal_connect (GTK_OBJECT (capplet_button),
 			    "clicked",
 			    launch_mime_capplet_and_close_dialog,
 			    window);
 	gtk_widget_show (capplet_button);
-	gtk_box_pack_end (GTK_BOX (capplet_hbox), capplet_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (capplet_button_vbox), capplet_button, TRUE, FALSE, 0);
 
 	capption = gtk_label_new (_("You can configure which programs are offered "
 				    "for which file types in the GNOME Control Center."));
