@@ -23,7 +23,7 @@
 */
 
 #include <config.h>
-#include "nautilus-scalable-font.h"
+#include "nautilus-scalable-font-private.h"
 
 #include "nautilus-gtk-macros.h"
 #include "nautilus-gdk-extensions.h"
@@ -49,24 +49,25 @@
 
 /* FontEntry */
 typedef struct {
-	char			*weight;
-	char			*slant;
-	char			*set_width;
-	RsvgFTFontHandle	font_handle;
-	char			*path;
+	char *weight;
+	char *slant;
+	char *set_width;
+	RsvgFTFontHandle font_handle;
+	char *path;
 } FontEntry;
 
 /* FontFamilyEntry */
 typedef struct {
-	char			*family;
-	GList			*fonts;
+	char *family;
+	GList *fonts;
 } FontFamilyEntry;
 
-
 /* Detail member struct */
 struct _NautilusScalableFontDetail
 {
-	RsvgFTFontHandle	font_handle;
+	RsvgFTFontHandle font_handle;
+	const FontFamilyEntry *font_family_entry;
+	const FontEntry *font_entry;
 };
 
 /* Global things */
@@ -75,8 +76,6 @@ static RsvgFTCtx		*global_rsvg_ft_context = NULL;
 static NautilusScalableFont	*global_default_font = NULL;
 static NautilusStringMap	*global_family_string_map = NULL;
 
-static const RsvgFTFontHandle UNDEFINED_FONT_HANDLE = -1;
-
 /* GtkObjectClass methods */
 static void nautilus_scalable_font_initialize_class   (NautilusScalableFontClass *font_class);
 static void nautilus_scalable_font_initialize         (NautilusScalableFont      *font);
@@ -106,9 +105,9 @@ nautilus_scalable_font_initialize_class (NautilusScalableFontClass *font_class)
 void
 nautilus_scalable_font_initialize (NautilusScalableFont *font)
 {
-	font->detail = g_new (NautilusScalableFontDetail, 1);
+	font->detail = g_new0 (NautilusScalableFontDetail, 1);
 
-	font->detail->font_handle = UNDEFINED_FONT_HANDLE;
+	font->detail->font_handle = NAUTILUS_SCALABLE_FONT_UNDEFINED_HANDLE;
 }
 
 /* GtkObjectClass methods */
@@ -159,7 +158,7 @@ file_as_string (const char *file_name)
 
 	fclose (stream);
 
-	if (num_read != stat_info.st_size) {
+	if ((ssize_t)num_read != stat_info.st_size) {
 		g_free (result);
 		return NULL;
 	}
@@ -178,7 +177,7 @@ parse_font_description_file (const char		*directory,
 	char			*fonts_dir_content;
 	NautilusStringList	*tokenized_list;
 	guint			i;
-	gint			count;
+	guint			count;
 
 	g_return_val_if_fail (directory != NULL, FALSE);
 	g_return_val_if_fail (g_file_exists (directory), FALSE);
@@ -281,7 +280,7 @@ font_entry_new (const char *weight,
 	entry->slant = g_strdup (slant);
 	entry->set_width = g_strdup (set_width);
 	entry->path = g_strdup (path);
-	entry->font_handle = UNDEFINED_FONT_HANDLE;
+	entry->font_handle = NAUTILUS_SCALABLE_FONT_UNDEFINED_HANDLE;
 
 	return entry;
 }
@@ -297,7 +296,7 @@ font_entry_free (FontEntry* entry)
 	g_free (entry->path);
 
 	/* These fonts arent refcounted because they are catched internally by librsvg */
-	entry->font_handle = UNDEFINED_FONT_HANDLE;
+	entry->font_handle = NAUTILUS_SCALABLE_FONT_UNDEFINED_HANDLE;
 	
 	g_free (entry);
 }
@@ -541,35 +540,46 @@ font_family_string_map_new (void)
 }
 
 /* Public NautilusScalableFont methods */
-GtkObject*
+
+/**
+ * nautilus_scalable_font_new:
+ * @family: The desired font family.
+ * @weight: The desired font weight.
+ * @slant: The desired font slant.
+ * @set_width: The desired font set_width.
+ *
+ * Returns a font for the desired components.  Returns NULL if the font
+ * does not exist.
+ *
+ */
+NautilusScalableFont *
 nautilus_scalable_font_new (const char	*family,
 			    const char	*weight,
 			    const char	*slant,
 			    const char	*set_width)
 {
 	/* const char	*foundry = "URW"; */
-	FontFamilyEntry		 *family_entry;
-	FontEntry		 *font_entry;
-	NautilusScalableFont	 *font;
+	FontFamilyEntry *font_family_entry;
+	FontEntry *font_entry;
+	NautilusScalableFont *font;
 
 	initialize_global_stuff_if_needed ();
 
-	family_entry = font_family_lookup (global_font_family_table, family);
+	font_family_entry = font_family_lookup (global_font_family_table, family);
 
 	/* If the family entry was not found, try a mapped family name */
-	if (family_entry == NULL) {
+	if (font_family_entry == NULL) {
 		char *mapped_family;
 
 		mapped_family = nautilus_string_map_lookup (global_family_string_map, family);
 
 		if (mapped_family != NULL) {
-			family_entry = font_family_lookup (global_font_family_table, mapped_family);
+			font_family_entry = font_family_lookup (global_font_family_table, mapped_family);
 			g_free (mapped_family);
 		}
 	}
 
-	if (family_entry == NULL) {
-		g_warning ("There is no such font: %s", family);
+	if (font_family_entry == NULL) {
 		return NULL;
 	}
 
@@ -577,15 +587,14 @@ nautilus_scalable_font_new (const char	*family,
 	slant = slant ? slant : "r";
 	set_width = set_width ? set_width : "normal";
 
-	font_entry = font_family_find_font (family_entry, weight, slant, set_width);
+	font_entry = font_family_find_font (font_family_entry, weight, slant, set_width);
 
 	if (font_entry == NULL) {
-		g_warning ("There is no such font '%s-%s-%s-%s'", family, weight, slant, set_width);
 		return NULL;
 	}
 
 	/* 'Intern' the rsvg font handle if needed */
-	if (font_entry->font_handle == UNDEFINED_FONT_HANDLE) {
+	if (font_entry->font_handle == NAUTILUS_SCALABLE_FONT_UNDEFINED_HANDLE) {
 		font_entry->font_handle = rsvg_ft_intern (global_rsvg_ft_context, font_entry->path);
 
 	}
@@ -600,8 +609,43 @@ nautilus_scalable_font_new (const char	*family,
 	gtk_object_sink (GTK_OBJECT (font));
 	
 	font->detail->font_handle = font_entry->font_handle;
+	font->detail->font_family_entry = font_family_entry;
+	font->detail->font_entry = font_entry;
 
-	return GTK_OBJECT (font);
+	return font;
+}
+
+NautilusScalableFont *
+nautilus_scalable_font_make_bold (NautilusScalableFont *font)
+{
+	NautilusScalableFont *bold_font = NULL;
+
+	g_return_val_if_fail (NAUTILUS_IS_SCALABLE_FONT (font), NULL);
+
+	/* FIXME bugzilla.eazel.com xxxx:
+	 * To be 100% correc we need to query for the weights.
+	 * It just so happens that the limited number of Type1
+	 * font we have dealt with all have a "bold" weight.
+	 */
+	
+	/* If the font is already bold, then do nothing */
+ 	if (nautilus_str_is_equal (font->detail->font_entry->weight, "bold")) {
+		gtk_object_ref (GTK_OBJECT (font));
+		return font;
+	}
+
+	bold_font = nautilus_scalable_font_new (font->detail->font_family_entry->family,
+						"bold",
+						font->detail->font_entry->slant,
+						font->detail->font_entry->set_width);
+
+	/* If the bold font does not exist, then return the source font */
+	if (bold_font == NULL) {
+		gtk_object_ref (GTK_OBJECT (font));
+		bold_font = font;
+	}
+
+	return bold_font;
 }
 
 void
@@ -679,53 +723,18 @@ nautilus_scalable_font_text_width (const NautilusScalableFont  *font,
 	return text_width;
 }
 
-/* Lifted from Raph's test-ft-gtk.c sample program */
-static void
-invert_glyph (guchar *buf, int rowstride, int width, int height)
-{
-	int x, y;
-	int first;
-	int n_words;
-	int last;
-	guint32 *middle;
-	
-	if (width >= 8 && ((rowstride & 3) == 0)) {
-		first = (-(long)buf) & 3;
-		n_words = (width - first) >> 2;
-		last = first + (n_words << 2);
-		
-		for (y = 0; y < height; y++) {
-			middle = (guint32 *)(buf + first);
-			for (x = 0; x < first; x++)
-				buf[x] = ~buf[x];
-			for (x = 0; x < n_words; x++)
-				middle[x] = ~middle[x];
-			for (x = last; x < width; x++)
-				buf[x] = ~buf[x];
-			buf += rowstride;
-		}
-	} else {
-		for (y = 0; y < height; y++) {
-			for (x = 0; x < width; x++)
-				buf[x] = ~buf[x];
-			buf += rowstride;
-		}
-	}
-}
-
 void
 nautilus_scalable_font_draw_text (const NautilusScalableFont *font,
-				  GdkPixbuf		     *destination_pixbuf,
-				  int			     x,
-				  int			     y,
-				  const ArtIRect              *clip_area,
-				  guint			     font_width,
-				  guint			     font_height,
-				  const char		     *text,
-				  guint			     text_length,
-				  guint32		     color,
-				  guchar		     overall_alpha,
-				  gboolean		     inverted)
+				  GdkPixbuf *destination_pixbuf,
+				  int x,
+				  int y,
+				  const ArtIRect *clip_area,
+				  guint font_width,
+				  guint font_height,
+				  const char *text,
+				  guint text_length,
+				  guint32 color,
+				  int opacity)
 {
 	RsvgFTGlyph	*glyph;
  	double		affine[6];
@@ -770,37 +779,6 @@ nautilus_scalable_font_draw_text (const NautilusScalableFont *font,
 	glyph_area.x1 = glyph_area.x0 + glyph->width;
 	glyph_area.y1 = glyph_area.y0 + glyph->height;
 	
-	/* Invert the glyph if needed */
-	if (inverted) {
-		ArtIRect invert_area;
-		
-		if (clip_area != NULL) {
-			art_irect_intersect (&invert_area, &glyph_area, clip_area);
-		}
-		else {
-			invert_area = glyph_area;
-		}
-
-		if (!art_irect_empty (&invert_area)) {
-			guchar *glyph_pixels;
-			guint glyph_rowstride;
-
-			glyph_rowstride = glyph->rowstride;
-			
-			glyph_pixels = glyph->buf;
-
-			glyph_pixels = 
-				glyph->buf + 
-				(invert_area.x0 - glyph_area.x0) + 
-				glyph_rowstride * (invert_area.y0 - glyph_area.y0);
-
-			invert_glyph (glyph_pixels, 
-				      glyph_rowstride,
-				      invert_area.x1 - invert_area.x0,
-				      invert_area.y1 - invert_area.y0);
-		}
-	}
-
 	/* Translate the glyph area to the (x,y) where its to be rendered */
 	glyph_area.x0 += x;
 	glyph_area.y0 += y;
@@ -850,8 +828,9 @@ nautilus_scalable_font_draw_text (const NautilusScalableFont *font,
 		art_color_array[1] = ART_PIX_MAX_FROM_8 (NAUTILUS_RGBA_COLOR_GET_G (color));
 		art_color_array[2] = ART_PIX_MAX_FROM_8 (NAUTILUS_RGBA_COLOR_GET_B (color));
 		
+		art_render_mask_solid (art_render, (opacity << 8) + opacity + (opacity >> 7));
 		art_render_image_solid (art_render, art_color_array);
-		
+
 		art_render_mask (art_render,
 				 render_area.x0,
 				 render_area.y0,
@@ -859,7 +838,6 @@ nautilus_scalable_font_draw_text (const NautilusScalableFont *font,
 				 render_area.y1,
 				 glyph->buf, 
 				 glyph->rowstride);
-		
 		art_render_invoke (art_render);
 	}
 	
@@ -970,8 +948,7 @@ nautilus_scalable_font_draw_text_lines_with_dimensions (const NautilusScalableFo
 							guint                        line_offset,
 							double			     empty_line_height,
 							guint32                      color,
-							guchar                       overall_alpha,
-							gboolean		     inverted)
+							int                          opacity)
 {
 	guint		i;
 	const char	*line;
@@ -1064,8 +1041,7 @@ nautilus_scalable_font_draw_text_lines_with_dimensions (const NautilusScalableFo
 							  line,
 							  length,
 							  color,
-							  overall_alpha,
-							  inverted);
+							  opacity);
 			
 			y += (line_offset + text_line_heights[i]);
 		}
@@ -1092,8 +1068,7 @@ nautilus_scalable_font_draw_text_lines (const NautilusScalableFont  *font,
 					guint                        line_offset,
 					double			     empty_line_height,
 					guint32                      color,
-					guchar                       overall_alpha,
-					gboolean		     inverted)
+					int                       opacity)
 {
 	guint	num_text_lines;
 	guint	*text_line_widths;
@@ -1147,8 +1122,7 @@ nautilus_scalable_font_draw_text_lines (const NautilusScalableFont  *font,
 								line_offset,
 								empty_line_height,
 								color,
-								overall_alpha,
-								inverted);
+								opacity);
 	
 	g_free (text_line_widths);
 	g_free (text_line_heights);
@@ -1207,7 +1181,7 @@ NautilusScalableFont *
 nautilus_scalable_font_get_default_font (void)
 {
 	if (global_default_font == NULL) {
-		global_default_font = NAUTILUS_SCALABLE_FONT (nautilus_scalable_font_new ("helvetica", NULL, NULL, NULL));
+		global_default_font = nautilus_scalable_font_new ("helvetica", NULL, NULL, NULL);
 		g_assert (global_default_font != NULL);
 		g_atexit (default_font_at_exit_destructor);
 	}
@@ -1404,6 +1378,25 @@ initialize_global_stuff_if_needed (void)
 	}
 }
 
+/* Private NautilusScalableFont things */
+int
+nautilus_scalable_font_get_rsvg_handle (const NautilusScalableFont *font)
+{
+	g_return_val_if_fail (NAUTILUS_IS_SCALABLE_FONT (font), NAUTILUS_SCALABLE_FONT_UNDEFINED_HANDLE);
+
+	return font->detail->font_handle;
+}
+
+struct _RsvgFTCtx *
+nautilus_scalable_font_get_rsvg_context (const NautilusScalableFont *font)
+{
+	g_return_val_if_fail (NAUTILUS_IS_SCALABLE_FONT (font), NULL);
+
+	initialize_global_stuff_if_needed ();
+
+	return global_rsvg_ft_context;
+}
+
 /*
  * The following text_layout stuff was shamelessly plundered
  * from libgnomeui/gnome-icon-text.[ch] by Federico Mena.
@@ -1470,7 +1463,7 @@ nautilus_text_layout_new (const NautilusScalableFont *font,
 			  guint font_size,
 			  const char *text,
 			  const char *separators,
-			  int max_width,
+			  guint max_width,
 			  gboolean confine)
 {
 	NautilusTextLayout *text_layout;
@@ -1478,7 +1471,8 @@ nautilus_text_layout_new (const NautilusScalableFont *font,
 	const char *row_end;
 	const char *s, *word_start, *word_end, *old_word_end;
 	char *sub_text;
-	int i, w_len, w;
+	int i, w_len;
+	guint w;
 	const char *text_iter;
 	int text_len, separators_len;
 
@@ -1644,7 +1638,6 @@ nautilus_text_layout_paint (const NautilusTextLayout	*text_layout,
 			    int				y, 
 			    GtkJustification		justification,
 			    guint32			color,
-			    gboolean			inverted,
 			    gboolean			underlined)
 {
 	GList *item;
@@ -1655,7 +1648,7 @@ nautilus_text_layout_paint (const NautilusTextLayout	*text_layout,
 	g_return_if_fail (destination_pixbuf != NULL);
 	g_return_if_fail (justification >= GTK_JUSTIFY_LEFT && justification <= GTK_JUSTIFY_FILL);
 	
-	/* FIXME: Make sure the color we are fed is opaque.  The real solution is 
+	/* FIXME bugzilla.eazel.com 5087: Make sure the color we are fed is opaque.  The real solution is 
 	 * to fix the callers.
 	 */
 	color = NAUTILUS_RGBA_COLOR_PACK (NAUTILUS_RGBA_COLOR_GET_R (color),
@@ -1697,87 +1690,33 @@ nautilus_text_layout_paint (const NautilusTextLayout	*text_layout,
 							  row->text,
 							  row->text_length,
 							  color,
-							  255,
-							  inverted);
+							  255);
 
 			/* Underline the text if needed */
 			if (underlined) {
-				GdkRectangle underline_rect;
+				ArtIRect underline_rect;
 
 				/* FIXME bugzilla.eazel.com 2865: This underlining code should
 				 * take into account the baseline for the rendered string rather
 				 * that doing the '-2' nonsense.
 				 */
-				underline_rect.x = x + xpos;
-				underline_rect.y = y + text_layout->font_size - 2;
-				underline_rect.width = row->width;
-				underline_rect.height = 1;
+				nautilus_art_irect_assign (&underline_rect,
+							   x + xpos,
+							   y + text_layout->font_size - 2,
+							   row->width,
+							   1);
 
 				nautilus_gdk_pixbuf_fill_rectangle_with_color (destination_pixbuf,
 									       &underline_rect,
 									       color);
 			}
-
+			
 			y += text_layout->baseline_skip;
 		} else
 			y += text_layout->baseline_skip / 2;
 	}
 }
 
-GdkPixbuf *
-nautilus_gdk_pixbuf_new_from_text (const NautilusScalableFont  *font,
-				   guint                        font_width,
-				   guint                        font_height,
-				   const char                  *text,
-				   guint                        text_length,
-				   guint32                      color,
-				   guchar                       overall_alpha,
-				   gboolean			inverted)
-{
-	GdkPixbuf *pixbuf;
-	guint text_width;
-	guint text_height;
-
-	g_return_val_if_fail (NAUTILUS_IS_SCALABLE_FONT (font), NULL);
-	g_return_val_if_fail (font_width > 0, NULL);
-	g_return_val_if_fail (font_height > 0, NULL);
-
-	if (text == NULL || text[0] == '\0' || text_length == 0) {
-		return NULL;
-	}
-
-	g_return_val_if_fail (text_length <= strlen (text), NULL);
-
-	nautilus_scalable_font_measure_text (font,
-					     font_width,
-					     font_height,
-					     text,
-					     strlen (text),
-					     &text_width,
-					     &text_height);
-	g_assert (text_width > 0);
-	g_assert (text_height > 0);
-
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, text_width, text_height);
-
-	nautilus_gdk_pixbuf_fill_rectangle_with_color (pixbuf, NULL, NAUTILUS_RGBA_COLOR_PACK (0, 0, 0, 0));
-	
-	nautilus_scalable_font_draw_text (font,
-					  pixbuf,
-					  0,
-					  0,
-					  NULL,
-					  font_width,
-					  font_height,
-					  text,
-					  strlen (text),
-					  color,
-					  overall_alpha,
-					  inverted);
-
-	return pixbuf;
-}
-
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 void
