@@ -37,8 +37,6 @@ static GSList *           tokenize_uri                (const char *string);
 static char *             get_translated_criterion    (const GSList *criterion);
 static char *             get_first_criterion_prefix  (GSList *criterion);
 static char *             get_nth_criterion_prefix    (GSList *criterion);
-static char *             get_nth_criterion_suffix    (GSList *criterion);
-static char *             get_first_criterion_suffix  (GSList *criterion);
 static char *             parse_uri                   (const char *search_uri);
 static void               free_tokenized_uri          (GSList *list);
 
@@ -192,7 +190,7 @@ struct _value_criterion_item {
 	char *translation;
         /* this field is there only to make the 3 structures similar enough 
            so that you can safely cast between them it is a kind of evil hack 
-           but i like it. It is waranteed to be always NULL. */
+           but i like it. It is guaranteed to be always NULL. */
 	value_criterion_table items; 
 };
 
@@ -203,23 +201,36 @@ struct _value_criterion_item {
 */
 
 static operand_criterion_item file_name2_table [] = {
-        {"contains", 
-         N_("that have \"%s\" in the name"),
+        {"contains",  
+         /* Part of a window title for search results: a
+            description of the set of files that contains a string */
+         N_("containing \"%s\" in their names"),
          NULL},
         {"starts_with",
-         N_("that start with \"%s\""),
+         /* Part of a window title for search results: a
+            description of the set of files that start with a string */
+         N_("starting with \"%s\""),
          NULL},
         {"ends_with",
-         N_("that end with %s"),
+         /* Part of a window title for search results: a
+            description of the set of files that end with a string */
+         N_("ending with %s"),
          NULL},
         {"does_not_contain",
-         N_("that don't contain \"%s\""),
+         /* Part of a window title for search results: a
+            description of the set of files that end with a string */
+         N_("containing \"%s\" in their names"),
          NULL},
         {"regexp_matches",
-         N_("that match the regular expression \"%s\""),
+         /* Part of a window title for search results: a
+            description of the set of files that match a regular
+            expression */
+         N_("matching the regular expression \"%s\""),
          NULL},
         {"matches",
-         N_("that match the file pattern \"%s\""),
+         /* Part of a window title for search results: a
+            description of the set of files that match a glob pattern */
+         N_("matching the file pattern \"%s\""),
          NULL},
         {NULL, NULL, NULL}
         
@@ -232,30 +243,51 @@ static operand_criterion_item file_name2_table [] = {
 */
 static value_criterion_item file_type_options3_table [] = {
         {"file",
+         /* Part of a window title for search results: a
+            description of the set of files that are not directories,
+            sockets, etc. */
          N_("regular files"),
          NULL},
         {"text_file",
+         /* Part of a window title for search results: a
+            description of the set of files that are stored as plain
+            text */
          N_("text files"),
          NULL},
         {"application",
+         /* Part of a window title for search results: a
+            description of the set of files that are binary files or
+            applications you can run */
          N_("applications"),
          NULL},
         {"directory",
-         N_("directories"),
+         /* Part of a window title for search results: a
+            description of the set of items that are folders
+            (directories) */
+         N_("folders"),
          NULL},
         {"music",
+         /* Part of a window title for search results: a
+            description of the set of items that are music files
+            (mp3s, wavs) */
          N_("music"),
          NULL},
         {NULL, NULL, NULL}
 };
 static operand_criterion_item file_type2_table [] = {
-        /* this one is not yet implemented in medusa */
         {"is_not",  
-         N_("are not %s"),
+         /* Part of a window title for search results: way of
+            describing the fact that files are not of a certain type
+            (folder, music, etc).  Context is "files that are [folder,
+            music]" */
+         N_("that are %s"),
          file_type_options3_table},
-        /* this one is implemented */
         {"is", 
-         N_("are %s"),
+         /* Part of a window title for search results: way of
+            describing the fact that files are of a certain type
+            (folder, music, etc).  Context is "files that are not
+            [folder, music]" */
+         N_("that are %s"),
          file_type_options3_table},
 };
 	
@@ -266,17 +298,29 @@ static operand_criterion_item file_type2_table [] = {
 */
 static operand_criterion_item owner2_table [] = {
         {"is_not",
-         N_("are not owned by \"%s\""),
+         /* Part of a window title for search results: way of
+            describing the fact that files are not owned by someone,
+            Context is files not ownd by xxx */
+         N_("not owned by \"%s\""),
          NULL},
         {"is",
-         N_("are owned by \"%s\""),
+         /* Part of a window title for search results: way of
+            describing the fact that files are owned by someone.
+            Context is files owned by root */
+         N_("owned by \"%s\""),
          NULL},
-        /* folowing ones are not supported by Nautilus UI */
         {"has_uid",
-         N_("have owner UID \"%s\""),
+         /* Part of a window title for search results: way of
+            describing the fact that files have a specific owner uid.
+            Context is "files with owner UID xx" */
+         N_("with owner UID \"%s\""),
          NULL},
         {"does_not_have_uid",
-         N_("have owner UID other than \"%s\""),
+         /* Part of a window title for search results: way of
+            describing the fact that files have a specific owner uid
+            other than this one.  Context is "files with owner uid
+            other than xx" */
+         N_("with owner UID other than \"%s\""),
          NULL},
         {NULL, NULL, NULL}
 };
@@ -287,13 +331,22 @@ static operand_criterion_item owner2_table [] = {
 */
 static operand_criterion_item size2_table [] = {
         {"larger_than",
-         N_("that are larger than %s bytes"),
+         /* Part of a window title for search results: way of
+            describing that the files are larger than a certain size.
+            Context is "files larger than %s bytes" */
+         N_("larger than %s bytes"),
          NULL},
         {"smaller_than",
-         N_("that are smaller than %s bytes"),
+         /* Part of a window title for search results: way of
+            describing that the files are larger than a certain size.
+            Context is "files smaller than %s bytes" */
+         N_("smaller than %s bytes"),
          NULL},
         {"is",
-         N_("that are %s bytes"),
+         /* Part of a window title for search results: way of
+            describing that the files are larger than a certain size.
+            Context is "files of %s bytes" */
+         N_("of %s bytes"),
          NULL},
         {NULL, NULL, NULL}
 };
@@ -304,27 +357,57 @@ static operand_criterion_item size2_table [] = {
 */
 static operand_criterion_item mod_time2_table [] = {
         {"is today", 
+         /* Part of a window title for search results: way of
+            describing that the files were modified since midnight of
+            the current day.  Context is "files modified today" */
          N_("modified today"), 
          NULL},
         {"is yesterday",
+         /* Part of a window title for search results: way of
+            describing that the files were modified since midnight of
+            the previous day.  Context is "files modified yesterday" */
          N_("modified yesterday"),
          NULL},
         {"is",
+         /* Part of a window title for search results: way of
+            describing that the files were modified since midnight of
+            the date that will replace the %s.  Context is "files
+            modified on 11/20/00" */
          N_("modified on %s"),
          NULL},
         {"is_not", 
+         /* Part of a window title for search results: way of
+            describing that the files were not modified since midnight
+            of the date that will replace the %s.  Context is "files
+            not modified on 11/20/00" */
          N_("not modified on %s"), 
          NULL},
         {"is_before",
+         /* Part of a window title for search results: way of
+            describing that the files were not modified since midnight
+            of the date that will replace the %s. Context is "files
+            modified before 11/20/00" */
          N_("modified before %s"),
          NULL},
         {"is_after",
+         /* Part of a window title for search results: way of
+            describing that the files were modified after midnight of
+            the date field that will replace the %s.  Context is
+            "files modified after 11/20/00" */
          N_("modified after %s"),
          NULL},
         {"is_within_a_week_of",
+         /* Part of a window title for search results: way of
+            describing that the files were modified within a week of 
+            the date field that will replace the %s.  Context is
+            "files within a week of 11/20/00" */
          N_("modified within a week of %s"),
          NULL},
         {"is_within_a_month_of",
+         /* Part of a window title for search results: way of
+            describing that the files were modified within a week of 
+            the date field that will replace the %s.  Context is
+            "files within a month of 11/20/00" */
          N_("modified within a month of %s"),
          NULL},
         {NULL, NULL, NULL}
@@ -337,9 +420,17 @@ static operand_criterion_item mod_time2_table [] = {
 
 static operand_criterion_item emblem2_table [] = {
         { "include",
+         /* Part of a window title for search results: way of
+            describing that the files have emblems attached that include
+            the one that will replace the %s.  Context is "files
+            marked with Important" */
           N_("marked with \"%s\""),
           NULL},
         { "do_not_include",
+         /* Part of a window title for search results: way of
+            describing that the files have emblems attached that don't
+            include the one that will replace the %s.  Context is
+            "files not marked with Important" */
           N_("not marked with \"%s\""),
           NULL},
         {NULL, NULL, NULL}
@@ -354,15 +445,31 @@ static operand_criterion_item emblem2_table [] = {
 
 static operand_criterion_item contains2_table [] = {
         {"includes_all_of",
+         /* Part of a window title for search results: way of
+            describing that the (text) files searched for contain the
+            word or words that will replace the %s.  Context is "files
+            with all the word "bob frank" */
          N_("with all the words \"%s\""),
          NULL},
         {"includes_any_of",
+         /* Part of a window title for search results: way of
+            describing that the (text) files searched for contain the
+            word or one or more of the words that will replace the %s.
+            Context is "files containing one of the words "bob frank" */
          N_("containing one of the words \"%s\""),
          NULL},
         {"does_not_include_all_of",
+         /* Part of a window title for search results: way of
+            describing that the (text) files searched for don't
+            contain the word or all of the words that will replace the
+            %s.  Context is "files without all the word "bob frank" */
          N_("without all the words \"%s\""),
          NULL},
         {"does_not_include_any_of",
+         /* Part of a window title for search results: way of
+            describing that the (text) files searched for contain none of 
+            the word or words that will replace the %s.  Context is "files
+            with none of the words "bob frank" */
          N_("without any of the words \"%s\""),
          NULL},
         {NULL, NULL, NULL},
@@ -372,29 +479,61 @@ static operand_criterion_item contains2_table [] = {
 
 /* -------------------------------------------------------
    -       main table                                    -
-   -------------------------------------------------------
-*/
+   ------------------------------------------------------- */
 
 static field_criterion_item main_table[] = {
         {"file_name",
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the file
+            name attribute matched.  Context is after "files" and
+            before the translation for "containing xx in the name" */
          N_(""),
          file_name2_table},
         {"file_type",
-         N_("that"),
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the file
+            type attribute matched.  Context is after "files" and
+            before the translation for "that are music" */
+         N_(""),
          file_type2_table},
         {"owner",
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the
+            properties of the file's owner that the search matched.
+            Context is after "files" and before the translation for
+            "owned by xx" */
          N_(""),
          owner2_table},
         {"size",
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the
+            properties of the file's size that the search matched.
+            Context is after "files" and before the translation for
+            "larger than 500 bytes" */
          N_(""),
          size2_table},
         {"content",
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the
+            properties of the file's content that the search matched.
+            Context is after "files" and before the translation for
+            "containing all the words" */
          N_(""),
          contains2_table},
         {"modified",
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the
+            properties of the file's modification date that the search
+            matched.  Context is after "files" and before the
+            translation for "modified today" */
          N_(""),
          mod_time2_table},
         {"keywords",
+         /* Part of a window title for search results: Optional
+            preposition that precedes the clause describing the
+            properties of the file's attached emblems that the search
+            matched.  Context is after "files" and before the
+            translation for "marked with Important" */
          N_(""),
          emblem2_table},
         {NULL, NULL, NULL}
@@ -411,8 +550,7 @@ static field_criterion_item main_table[] = {
  * into the @current_table.
  * it returns -1 if it could not find it.
  * Yes, I know it is wrong to use the normal function return value
- * to pass error status.
- */
+ * to pass error status.  */
 static int 
 get_item_number (field_criterion_item *current_table, char *item) 
 {
@@ -500,20 +638,14 @@ get_translated_criterion (const GSList *criterion)
                 return ret_val;
         }
 
-        return g_strdup (_("are directories"));
+        return g_strdup (_("are folders"));
 }
 
 /**
  * get_first_criterion_prefix:
  * @criterion: The GSList whose data field points to the criterion GSList.
  *
- * calculates the "whose", "who" or "which" prefix for a given criterion.
- * FIXME bugzilla.eazel.com 2440: it is an ugly hack I added after arlo asked me to:
- * it is likely to be a pain for translations. I need to modify the data struct 
- * for  this to work cleanly.
- *
- * return value: the translated prefix.
- */
+ * calculates the prefix for a given criterion */
 static char *
 get_first_criterion_prefix (GSList *criterion) 
 {
@@ -549,39 +681,6 @@ get_nth_criterion_prefix (GSList *criterion)
         return g_strdup (", ");
 }
 
-/**
- * get_nth_criterion_suffix:
- * @criterion: The GSList whose data field points to the criterion GSList.
- *
- * calculates the "." suffix for any criterion.
- *
- * return value: the translated suffix.
- */
-static char *
-get_nth_criterion_suffix (GSList *criterion)
-{
-        /* if we are the last criterion, put it here. */
-
-        if (criterion->next == NULL) {
-                return g_strdup (".");
-        }
-        return g_strdup ("");
-}
-
-/**
- * get_first_criterion_suffix:
- * @criterion: The GSList whose data field points to the criterion GSList.
- *
- * calculates the "." suffix for any criterion.
- *
- * return value: the translated suffix.
- */
-static char *
-get_first_criterion_suffix (GSList *criterion)
-{
-        return get_nth_criterion_suffix (criterion);
-}
-
 /**
  * parse_uri:
  * @search_uri: uri to translate.
@@ -592,7 +691,7 @@ static char *
 parse_uri (const char *search_uri)
 {
         GSList *criteria, *criterion;
-        char *translated_criterion, *translated_prefix, *translated_suffix;
+        char *translated_criterion, *translated_prefix;
         char *ret_val, *temp;
         
         criteria = tokenize_uri (search_uri);
@@ -607,15 +706,13 @@ parse_uri (const char *search_uri)
                 return NULL;
         }
         translated_prefix = get_first_criterion_prefix (criteria);
-        translated_suffix = get_first_criterion_suffix (criteria);
         if (strcmp (translated_prefix, "") == 0) {
-                ret_val = g_strdup_printf (_("Items %s%s"), 
-                                           translated_criterion, translated_suffix);
+                ret_val = g_strdup_printf (_("Items %s"),
+                                           translated_criterion);
         } else {
-                ret_val = g_strdup_printf (_("Items %s %s%s"), 
-                                           translated_prefix, translated_criterion, translated_suffix);
+                ret_val = g_strdup_printf (_("Items %s %s"),
+                                           translated_prefix, translated_criterion);
         }
-        g_free (translated_suffix);
         g_free (translated_criterion);
         g_free (translated_prefix);
         
@@ -628,13 +725,11 @@ parse_uri (const char *search_uri)
                         return NULL;
                 }
                 translated_prefix = get_nth_criterion_prefix (criterion);
-                translated_suffix = get_nth_criterion_suffix (criterion);
                 temp = g_strconcat (ret_val, translated_prefix, 
-                                    translated_criterion, translated_suffix, NULL);
+                                    translated_criterion, NULL);
                 g_free (ret_val);
                 ret_val = temp;
                 g_free (translated_criterion);
-                g_free (translated_suffix);
                 g_free (translated_prefix);
         }
         
@@ -668,6 +763,12 @@ nautilus_search_uri_to_human (const char *search_uri)
         return human;
 }
 
+char *
+nautilus_get_target_uri_from_search_result_name (const char *search_result_name)
+{
+	return gnome_vfs_unescape_string (search_result_name, NULL);
+}
+
 gboolean
 nautilus_is_search_uri (const char *uri)
 {
@@ -721,16 +822,19 @@ nautilus_self_check_search_uri (void)
 
         /* make sure all the code paths work */
         NAUTILUS_CHECK_STRING_RESULT (nautilus_search_uri_to_human ("search:[][]file_name contains stuff"), 
-                                      _("Items that have \"stuff\" in the name."));
+                                      _("Items containing \"stuff\" in their names"));
+        /* FIXME bugzilla.eazel.com 5088: This may be what the function calls "human", but it's bad grammar. */
         NAUTILUS_CHECK_STRING_RESULT (nautilus_search_uri_to_human ("search:[][]file_name contains stuff & file_type is file"), 
-                                      _("Items that have \"stuff\" in the name and are regular files."));
+                                      _("Items containing \"stuff\" in their names and that are regular files"));
+        /* FIXME bugzilla.eazel.com 5088: This may be what the function calls "human", but it's bad grammar. */
         NAUTILUS_CHECK_STRING_RESULT (nautilus_search_uri_to_human ("search:[][]file_name contains stuff & file_type is file"
                                                                     " & size smaller_than 2000"), 
-                                      _("Items that have \"stuff\" in the name, are regular files and that are "
-                                        "smaller than 2000 bytes."));
+                                      _("Items containing \"stuff\" in their names, that are regular files and "
+                                        "smaller than 2000 bytes"));
+        /* FIXME bugzilla.eazel.com 5088: This may be what the function calls "human", but it's bad grammar. */
         NAUTILUS_CHECK_STRING_RESULT (nautilus_search_uri_to_human ("search:[][]file_name contains medusa & file_type is directory"), 
-                                      _("Items that have \"medusa\" in the name and are "
-                                        "directories."));
+                                      _("Items containing \"medusa\" in their names and that are "
+                                        "folders"));
         
         /* is_search_uri */
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_is_search_uri (""), FALSE);
