@@ -53,13 +53,13 @@ static void
 nautilus_report_check_failure (char *result, char *expected)
 {
 	if (!failed) {
-		printf ("\n");
+		fprintf (stderr, "\n");
 	}
 
-	printf ("FAIL: check failed in %s, line %d\n", current_file_name, current_line_number);
-	printf ("      evaluated: %s\n", current_expression);
-	printf ("       expected: %s\n", expected == NULL ? "NULL" : expected);
-	printf ("            got: %s\n", result == NULL ? "NULL" : result);
+	fprintf (stderr, "FAIL: check failed in %s, line %d\n", current_file_name, current_line_number);
+	fprintf (stderr, "      evaluated: %s\n", current_expression);
+	fprintf (stderr, "       expected: %s\n", expected == NULL ? "NULL" : expected);
+	fprintf (stderr, "            got: %s\n", result == NULL ? "NULL" : result);
 	
 	failed = TRUE;
 
@@ -137,4 +137,15 @@ nautilus_check_string_result (char *result, const char *expected)
 	nautilus_after_check ();
 }
 
+void
+nautilus_before_check_function (const char *name)
+{
+	fprintf (stderr, "running %s\n", name);
+}
+
+void
+nautilus_after_check_function (void)
+{
+}
+
 #endif /* ! NAUTILUS_OMIT_SELF_CHECK */
