@@ -27,6 +27,7 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <locale.h>
 #include <stdlib.h>
 
 #include "nautilus-lib-self-check-functions.h"
@@ -46,6 +47,11 @@ nautilus_strchr (const char *haystack, char needle)
 int
 nautilus_strcmp (const char *string_a, const char *string_b)
 {
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	return strcmp (string_a == NULL ? "" : string_a,
 		       string_b == NULL ? "" : string_b);
 }
@@ -53,6 +59,11 @@ nautilus_strcmp (const char *string_a, const char *string_b)
 int
 nautilus_strcasecmp (const char *string_a, const char *string_b)
 {
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	return g_strcasecmp (string_a == NULL ? "" : string_a,
 		             string_b == NULL ? "" : string_b);
 }
@@ -62,6 +73,11 @@ nautilus_strcmp_case_breaks_ties (const char *string_a, const char *string_b)
 {
 	int casecmp_result;
 
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	casecmp_result = nautilus_strcasecmp (string_a, string_b);
 	if (casecmp_result != 0) {
 		return casecmp_result;
@@ -69,6 +85,34 @@ nautilus_strcmp_case_breaks_ties (const char *string_a, const char *string_b)
 	return nautilus_strcmp (string_a, string_b);
 }
 
+int
+nautilus_strcoll (const char *string_a, const char *string_b)
+{
+	const char *locale;
+	int result;
+	
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
+
+	locale = setlocale (LC_COLLATE, NULL);
+	
+	if (locale == NULL || strcmp (locale, "C") == 0 || strcmp (locale, "POSIX") == 0) {
+		/* If locale is NULL or default "C" or "POSIX" use nautilus sorting */
+		return nautilus_strcmp_case_breaks_ties (string_a, string_b);
+	} else {
+		/* Use locale-specific collated sorting */
+		result = strcoll (string_a == NULL ? "" : string_a,
+				  string_b == NULL ? "" : string_b);
+		if (result != 0) {
+			return result;
+		}
+		return nautilus_strcmp (string_a, string_b);
+	}
+}
+
 gboolean
 nautilus_str_is_empty (const char *string_or_null)
 {
@@ -78,25 +122,57 @@ nautilus_str_is_empty (const char *string_or_null)
 gboolean
 nautilus_str_is_equal (const char *string_a, const char *string_b)
 {
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL != ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	return nautilus_strcmp (string_a, string_b) == 0;
 }
 
 gboolean
 nautilus_istr_is_equal (const char *string_a, const char *string_b)
 {
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL != ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	return nautilus_strcasecmp (string_a, string_b) == 0;
 }
 
 int
-nautilus_str_compare (gconstpointer string_a, gconstpointer string_b)
+nautilus_strcmp_compare_func (gconstpointer string_a, gconstpointer string_b)
 {
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	return nautilus_strcmp ((const char *) string_a,
 				(const char *) string_b);
 }
 
 int
-nautilus_istr_compare (gconstpointer string_a, gconstpointer string_b)
+nautilus_strcoll_compare_func (gconstpointer string_a, gconstpointer string_b)
 {
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
+	return nautilus_strcoll ((const char *) string_a,
+				 (const char *) string_b);
+}
+
+int
+nautilus_strcasecmp_compare_func (gconstpointer string_a, gconstpointer string_b)
+{
+	/* FIXME bugzilla.eazel.com 5450: Maybe we need to make this
+	 * treat 'NULL < ""', or have a flavor that does that. If we
+	 * didn't have code that already relies on 'NULL == ""', I
+	 * would change it right now.
+	 */
 	return nautilus_strcasecmp ((const char *) string_a,
 				    (const char *) string_b);
 }
@@ -164,12 +240,8 @@ nautilus_istr_has_prefix (const char *haystack, const char *needle)
 		}
 		hc = *h++;
 		nc = *n++;
-		if (isupper (hc)) {
-			hc = tolower (hc);
-		}
-		if (isupper (nc)) {
-			nc = tolower (nc);
-		}
+		hc = tolower ((guchar) hc);
+		nc = tolower ((guchar) nc);
 	} while (hc == nc);
 	return FALSE;
 }
@@ -188,8 +260,8 @@ nautilus_istr_has_suffix (const char *haystack, const char *needle)
 	}
 		
 	/* Eat one character at a time. */
-	h = haystack + strlen(haystack);
-	n = needle + strlen(needle);
+	h = haystack + strlen (haystack);
+	n = needle + strlen (needle);
 	do {
 		if (n == needle) {
 			return TRUE;
@@ -199,12 +271,8 @@ nautilus_istr_has_suffix (const char *haystack, const char *needle)
 		}
 		hc = *--h;
 		nc = *--n;
-		if (isupper (hc)) {
-			hc = tolower (hc);
-		}
-		if (isupper (nc)) {
-			nc = tolower (nc);
-		}
+		hc = tolower ((guchar) hc);
+		nc = tolower ((guchar) nc);
 	} while (hc == nc);
 	return FALSE;
 }
@@ -454,13 +522,19 @@ nautilus_str_capitalize (const char *string)
 
 	capitalized = g_strdup (string);
 
-	if (islower (capitalized[0])) {
-		capitalized[0] = toupper (capitalized[0]);
-	}
+	capitalized[0] = toupper ((guchar) capitalized[0]);
 
 	return capitalized;
 }
 
+/* Note: nautilus_string_ellipsize_* that use a length in pixels
+ * rather than characters can be found in nautilus_gdk_extensions.h
+ * 
+ * FIXME bugzilla.eazel.com 5089: 
+ * we should coordinate the names of nautilus_string_ellipsize_*
+ * and nautilus_str_*_truncate so that they match better and reflect
+ * their different behavior.
+ */
 char *
 nautilus_str_middle_truncate (const char *string,
 			      guint truncate_length)
@@ -479,7 +553,7 @@ nautilus_str_middle_truncate (const char *string,
 	}
 
 	/* It doesnt make sense to truncate strings to less than
-	 * the size of the delimter plus 2 characters (one on each
+	 * the size of the delimiter plus 2 characters (one on each
 	 * side)
 	 */
 	if (truncate_length < min_truncate_length) {
@@ -532,6 +606,78 @@ nautilus_str_count_characters (const char	*string,
 	return count;
 }
 
+char *
+nautilus_str_strip_substring_and_after (const char *string,
+					const char *substring)
+{
+	const char *substring_position;
+
+	g_return_val_if_fail (substring != NULL, g_strdup (string));
+	g_return_val_if_fail (substring[0] != '\0', g_strdup (string));
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	substring_position = strstr (string, substring);
+	if (substring_position == NULL) {
+		return g_strdup (string);
+	}
+
+	return g_strndup (string,
+			  substring_position - string);
+}
+
+char *
+nautilus_str_replace_substring (const char *string,
+				const char *substring,
+				const char *replacement)
+{
+	int substring_length, replacement_length, result_length, remaining_length;
+	const char *p, *substring_position;
+	char *result, *result_position;
+
+	g_return_val_if_fail (substring != NULL, g_strdup (string));
+	g_return_val_if_fail (substring[0] != '\0', g_strdup (string));
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	substring_length = strlen (substring);
+	replacement_length = nautilus_strlen (replacement);
+
+	result_length = strlen (string);
+	for (p = string; ; p = substring_position + substring_length) {
+		substring_position = strstr (p, substring);
+		if (substring_position == NULL) {
+			break;
+		}
+		result_length += replacement_length - substring_length;
+	}
+
+	result = g_malloc (result_length + 1);
+
+	result_position = result;
+	for (p = string; ; p = substring_position + substring_length) {
+		substring_position = strstr (p, substring);
+		if (substring_position == NULL) {
+			remaining_length = strlen (p);
+			memcpy (result_position, p, remaining_length);
+			result_position += remaining_length;
+			break;
+		}
+		memcpy (result_position, p, substring_position - p);
+		result_position += substring_position - p;
+		memcpy (result_position, replacement, replacement_length);
+		result_position += replacement_length;
+	}
+	g_assert (result_position - result == result_length);
+	result_position[0] = '\0';
+
+	return result;
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 static int
@@ -701,7 +847,6 @@ nautilus_self_check_string (void)
 		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_eat_str_to_int (g_strdup (string), &integer), boolean_result); \
 		NAUTILUS_CHECK_INTEGER_RESULT (call_eat_str_to_int (g_strdup (string)), integer_result);
 
-	
 	TEST_INTEGER_CONVERSION_FUNCTIONS (NULL, FALSE, 9999)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("", FALSE, 9999)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("a", FALSE, 9999)
@@ -723,7 +868,6 @@ nautilus_self_check_string (void)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("+21474836470", FALSE, 9999)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("-21474836480", FALSE, 9999)
 
-	/* nautilus_str_is_equal */
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal (NULL, NULL), TRUE);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal (NULL, ""), TRUE);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("", ""), TRUE);
@@ -732,7 +876,6 @@ nautilus_self_check_string (void)
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("foo", "foo"), TRUE);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("foo", "bar"), FALSE);
 
-	/* nautilus_istr_is_equal */
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal (NULL, NULL), TRUE);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal (NULL, ""), TRUE);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("", ""), TRUE);
@@ -743,7 +886,6 @@ nautilus_self_check_string (void)
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("Foo", "foo"), TRUE);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("foo", "Foo"), TRUE);
 
-	/* nautilus_str_count_characters */
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters (NULL, 'x'), 0);
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("", 'x'), 0);
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters (NULL, '\0'), 0);
@@ -752,6 +894,24 @@ nautilus_self_check_string (void)
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("foo", 'f'), 1);
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("foo", 'o'), 2);
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("xxxx", 'x'), 4);
+
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_strip_substring_and_after (NULL, "bar"), NULL);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_strip_substring_and_after ("", "bar"), "");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_strip_substring_and_after ("foo", "bar"), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_strip_substring_and_after ("foo bar", "bar"), "foo ");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_strip_substring_and_after ("foo bar xxx", "bar"), "foo ");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_strip_substring_and_after ("bar", "bar"), "");
+
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring (NULL, "foo", NULL), NULL);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring (NULL, "foo", "bar"), NULL);
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("bar", "foo", NULL), "bar");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("", "foo", ""), "");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("", "foo", "bar"), "");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("bar", "foo", ""), "bar");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("xxx", "x", "foo"), "foofoofoo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("fff", "f", "foo"), "foofoofoo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("foofoofoo", "foo", "f"), "fff");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_replace_substring ("foofoofoo", "f", ""), "oooooo");
 }
 
 #endif /* !NAUTILUS_OMIT_SELF_CHECK */
