@@ -48,11 +48,33 @@ static char	 *last_theme_name = NULL;
 static xmlDocPtr last_theme_document = NULL;
 static xmlDocPtr default_theme_document = NULL;
 
+static char *theme_from_preferences = NULL;
+
+static void
+theme_changed_callback (gpointer callback_data)
+{
+	g_free (theme_from_preferences);
+	theme_from_preferences = nautilus_preferences_get (NAUTILUS_PREFERENCES_THEME);
+}
+
 /* return the current theme by asking the preferences machinery */
 char *
 nautilus_theme_get_theme (void)
 {
-	return nautilus_preferences_get (NAUTILUS_PREFERENCES_THEME, "default");
+	static gboolean theme_changed_callback_installed = FALSE;
+
+	/* Add the callback once for the life of our process */
+	if (theme_changed_callback_installed == FALSE) {
+		nautilus_preferences_add_callback_while_process_is_running (NAUTILUS_PREFERENCES_THEME,
+									    theme_changed_callback,
+									    NULL);
+		theme_changed_callback_installed = TRUE;
+		
+		/* Peek for the first time */
+		theme_changed_callback (NULL);
+	}
+	
+	return g_strdup (theme_from_preferences);
 }
 
 /* set the current theme */
@@ -129,11 +151,12 @@ free_default_theme (void)
 	xmlFreeDoc (default_theme_document);
 }
 
-/* fetch data from the current theme.  Cache the last theme file as a parsed xml document */
+/* Fetch data from the specified theme.  Cache the last theme file as a parsed xml document
+ */
 char *
-nautilus_theme_get_theme_data (const char *resource_name, const char *property_name)
+nautilus_theme_get_theme_data_from_theme (const char *resource_name, const char *property_name, const char* theme_name)
 {
-	char *theme_name, *temp_str;
+	char *temp_str;
 	char *theme_data;
 	xmlDocPtr theme_document;
 	xmlNodePtr resource_node;
@@ -142,7 +165,6 @@ nautilus_theme_get_theme_data (const char *resource_name, const char *property_n
 	/* fetch the current theme name */
 	theme_data = NULL;
 	
-	theme_name = nautilus_preferences_get (NAUTILUS_PREFERENCES_THEME, "default");
 	if (nautilus_strcmp (theme_name, last_theme_name) == 0) {
 		theme_document = last_theme_document;
 	} else {
@@ -187,11 +209,23 @@ nautilus_theme_get_theme_data (const char *resource_name, const char *property_n
 		}
 
 	}
-	g_free (theme_name);
 
 	return theme_data;
 }
 
+/* Fetch data from the current theme.
+ */
+char *
+nautilus_theme_get_theme_data (const char *resource_name, const char *property_name)
+{
+	char *result;
+	char *theme_name;
+	theme_name = nautilus_theme_get_theme ();
+	result = nautilus_theme_get_theme_data_from_theme (resource_name, property_name, theme_name);
+	g_free (theme_name);
+	return result;
+}
+
 /* utility routine to return the full path to a themed image that
    searches the local themes if it can't find it in the shared space */
 static char *
@@ -278,7 +312,7 @@ nautilus_theme_get_image_path (const char *image_name)
 {
 	char *theme_name, *image_path;
 	
-	theme_name = nautilus_preferences_get (NAUTILUS_PREFERENCES_THEME, "default");	
+	theme_name = nautilus_theme_get_theme ();
 	image_path = nautilus_theme_get_image_path_from_theme (image_name, theme_name);	
 	g_free (theme_name);
 	
