@@ -25,11 +25,6 @@
 #ifndef NAUTILUS_VIEW_IDENTIFIER_H
 #define NAUTILUS_VIEW_IDENTIFIER_H
 
-/* FIXME bugzilla.eazel.com 1291: 
- * Without this define liboaf doesn't include popt.h, causing trouble
- * later; this mechanism should be fixed. 
- */
-#define HAVE_POPT_H
 #include <liboaf/liboaf.h>
 
 typedef struct {
