@@ -20,7 +20,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Author: Elliot Lee <sopwith@redhat.com>
+ *  Authors: Elliot Lee <sopwith@redhat.com>
+ *           Maciej Stachowiak <mjs@eazel.com>
  *
  */
 
@@ -39,9 +40,10 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 
 enum {
+	HISTORY_CHANGED,
 	LOAD_LOCATION,
-	STOP_LOADING,
 	SELECTION_CHANGED,
+	STOP_LOADING,
 	TITLE_CHANGED,
 	LAST_SIGNAL
 };
@@ -68,19 +70,23 @@ static void impl_Nautilus_View_selection_changed (PortableServer_Servant  servan
 static void impl_Nautilus_View_title_changed     (PortableServer_Servant  servant,
 						  const CORBA_char       *title,
 						  CORBA_Environment      *ev);
+static void impl_Nautilus_View_history_changed   (PortableServer_Servant  servant,
+						  const Nautilus_History *history,
+						  CORBA_Environment      *ev);
 static void nautilus_view_initialize             (NautilusView           *view);
 static void nautilus_view_destroy                (GtkObject              *object);
 static void nautilus_view_initialize_class       (NautilusViewClass      *klass);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusView, nautilus_view, BONOBO_OBJECT_TYPE)
 
-POA_Nautilus_View__epv libnautilus_Nautilus_View_epv =
+static POA_Nautilus_View__epv libnautilus_Nautilus_View_epv =
 {
 	NULL,
-	&impl_Nautilus_View_load_location,
-	&impl_Nautilus_View_stop_loading,
-	&impl_Nautilus_View_selection_changed,
-	&impl_Nautilus_View_title_changed
+	impl_Nautilus_View_load_location,
+	impl_Nautilus_View_stop_loading,
+	impl_Nautilus_View_selection_changed,
+	impl_Nautilus_View_title_changed,
+	impl_Nautilus_View_history_changed
 };
 
 static PortableServer_ServantBase__epv base_epv;
@@ -98,7 +104,7 @@ GList *
 nautilus_shallow_g_list_from_uri_list (const Nautilus_URIList *uri_list)
 {
 	GList *list;
-	int i;
+	guint i;
 
 	list = NULL;
 	for (i = 0; i < uri_list->_length; i++) {
@@ -178,6 +184,15 @@ impl_Nautilus_View_title_changed (PortableServer_Servant servant,
 			 title);
 }
 
+static void 
+impl_Nautilus_View_history_changed (PortableServer_Servant servant,
+				    const Nautilus_History *history,
+				    CORBA_Environment *ev)
+{
+	gtk_signal_emit (GTK_OBJECT (((impl_POA_Nautilus_View *) servant)->bonobo_object),
+			 signals[HISTORY_CHANGED],
+			 history);
+}
 
 static void
 impl_Nautilus_View__destroy (BonoboObject *object, PortableServer_Servant servant)
@@ -255,6 +270,13 @@ nautilus_view_initialize_class (NautilusViewClass *klass)
 			       GTK_SIGNAL_OFFSET (NautilusViewClass, title_changed),
 			       gtk_marshal_NONE__STRING,
 			       GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
+	signals[HISTORY_CHANGED] =
+		gtk_signal_new ("history_changed",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET (NautilusViewClass, history_changed),
+			       gtk_marshal_NONE__POINTER,
+			       GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
 	
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
@@ -285,9 +307,27 @@ nautilus_view_new (GtkWidget *widget)
 NautilusView *
 nautilus_view_new_from_bonobo_control (BonoboControl *control)
 {
-	NautilusView *view;
-	
-	view = NAUTILUS_VIEW (gtk_object_new (NAUTILUS_TYPE_VIEW, NULL));
+	return nautilus_view_construct_from_bonobo_control 
+		(NAUTILUS_VIEW (gtk_object_new (NAUTILUS_TYPE_VIEW, NULL)), 
+		 control);
+}
+
+NautilusView *
+nautilus_view_construct (NautilusView   *view,
+			 GtkWidget      *widget)
+{
+	return nautilus_view_construct_from_bonobo_control
+		(view, bonobo_control_new (widget));
+}
+
+
+NautilusView *
+nautilus_view_construct_from_bonobo_control (NautilusView   *view,
+					     BonoboControl  *control)
+{
+	g_return_val_if_fail (NAUTILUS_IS_VIEW (view), NULL);
+	g_return_val_if_fail (BONOBO_IS_CONTROL (control), NULL);
+
 	view->details->control = control;
 	bonobo_object_add_interface (BONOBO_OBJECT (view), BONOBO_OBJECT (control));
 	nautilus_undo_set_up_bonobo_control (control);
@@ -295,6 +335,8 @@ nautilus_view_new_from_bonobo_control (BonoboControl *control)
 	return view;
 }
 
+
+
 static void
 nautilus_view_destroy (GtkObject *object)
 {
@@ -312,7 +354,7 @@ view_frame_call_begin (NautilusView *view, CORBA_Environment *ev)
 	
 	CORBA_exception_init (ev);
 
-	view_frame = Bonobo_Unknown_query_interface 
+	view_frame = Bonobo_Unknown_queryInterface 
 		(bonobo_control_get_control_frame (nautilus_view_get_bonobo_control (view)),
 		 "IDL:Nautilus/ViewFrame:1.0", ev);
 
@@ -334,23 +376,39 @@ view_frame_call_end (Nautilus_ViewFrame frame, CORBA_Environment *ev)
 }
 
 void
-nautilus_view_open_location (NautilusView *view,
-			     const char *location)
+nautilus_view_open_location_in_this_window (NautilusView *view,
+					    const char *location)
 {
 	CORBA_Environment ev;
 	Nautilus_ViewFrame view_frame;
 	
 	view_frame = view_frame_call_begin (view, &ev);
 	if (view_frame != CORBA_OBJECT_NIL) {
-		Nautilus_ViewFrame_open_location (view_frame, (CORBA_char *) location, &ev);
+		Nautilus_ViewFrame_open_location_in_this_window
+			(view_frame, (CORBA_char *) location, &ev);
 	}
 	view_frame_call_end (view_frame, &ev);
 }
 
 void
-nautilus_view_open_location_in_new_window (NautilusView *view,
-					   const char *location,
-					   GList *selection)
+nautilus_view_open_location_prefer_existing_window (NautilusView *view,
+						    const char *location)
+{
+	CORBA_Environment ev;
+	Nautilus_ViewFrame view_frame;
+	
+	view_frame = view_frame_call_begin (view, &ev);
+	if (view_frame != CORBA_OBJECT_NIL) {
+		Nautilus_ViewFrame_open_location_prefer_existing_window
+			(view_frame, (CORBA_char *) location, &ev);
+	}
+	view_frame_call_end (view_frame, &ev);
+}
+
+void
+nautilus_view_open_location_force_new_window (NautilusView *view,
+					      const char *location,
+					      GList *selection)
 {
 	CORBA_Environment ev;
 	Nautilus_ViewFrame view_frame;
@@ -359,7 +417,8 @@ nautilus_view_open_location_in_new_window (NautilusView *view,
 	view_frame = view_frame_call_begin (view, &ev);
 	if (view_frame != CORBA_OBJECT_NIL) {
 		uri_list = nautilus_uri_list_from_g_list (selection);
-		Nautilus_ViewFrame_open_location_in_new_window (view_frame, (CORBA_char *) location, uri_list, &ev);
+		Nautilus_ViewFrame_open_location_force_new_window
+			(view_frame, (CORBA_char *) location, uri_list, &ev);
 		CORBA_free (uri_list);
 	}
 	view_frame_call_end (view_frame, &ev);
