@@ -4,7 +4,7 @@ typedef struct _RsvgFTGlyph RsvgFTGlyph;
 struct _RsvgFTGlyph {
 	int refcnt;
 	int width, height;
-	double xpen, ypen; /* location of pen after the glyph */
+	double xpen, ypen; /* relative location of pen after the glyph */
 	int rowstride;
 	guchar *buf;
 };
@@ -22,7 +22,7 @@ rsvg_ft_intern (RsvgFTCtx *ctx, const char *font_file_name);
 
 void
 rsvg_ft_font_attach (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
-		     char *font_file_name);
+		     const char *font_file_name);
 
 #if 0
 void
@@ -34,8 +34,20 @@ rsvg_ft_font_unref (RsvgFTFont *font);
 
 RsvgFTGlyph *
 rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
-		       const char *str, double sx, double sy,
+		       const char *str, 
+		       unsigned int length,
+		       double sx, double sy,
 		       const double affine[6], int xy[2]);
 
+void
+rsvg_ft_measure_string (RsvgFTCtx *ctx,
+			RsvgFTFontHandle fh,
+			const char *str, 
+			unsigned int length,
+			double sx, double sy,
+			const double affine[6], 
+			int xy[2],
+			unsigned int dimensions[2]);
+
 void
 rsvg_ft_glyph_unref (RsvgFTGlyph *glyph);
