@@ -93,6 +93,8 @@ nautilus-service-install-view \
 nautilus-service-startup-view \
 nautilus-sidebar-loser \
 nautilus-summary-view \
+nautilus-text-view \
+nautilus-throbber \
 nautilus-tree-view \
 trilobite-eazel-install-service  \
 trilobite-eazel-time-view \
