@@ -28,6 +28,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtkwindow.h>
 #include <libnautilus-extensions/nautilus-background.h>
 #include <libnautilus-extensions/nautilus-directory.h>
 #include <libnautilus-extensions/nautilus-file.h>
@@ -116,27 +117,21 @@ struct FMDirectoryViewClass {
 	 */
 	void 	(* begin_loading) 	 (FMDirectoryView *view);
 
-	/* The 'create_selection_context_menu_items' signal is emitted 
-	 * when creating a context menu for the selected items. @files is
-	 * the list of selected files; it isn't destroyed until the menu
-	 * is destroyed so it can be used in callbacks.
-	 * Subclasses might want to override this function to
-	 * modify the menu contents.
+	/* The 'end_loading' signal is emitted after all of the contents
+	 * of a directory are added to the view. It can be replaced by a 
+	 * subclass to do any necessary clean-up. The default implementation 
+	 * does nothing.
 	 */
-	void 	(* create_selection_context_menu_items) 	 
-					 (FMDirectoryView *view,
-					  GtkMenu *menu,
-					  GList *files);
+	void 	(* end_loading) 	 (FMDirectoryView *view);
+
+	/* The 'load_error' signal is emitted when the directory model
+	   reports an error in the process of monitoring the directory's
+	   contents.  The load error indicates that the process of 
+	   load the contents has ended, but the directory is still
+	   being monitored*/
+	void    (* load_error)           (FMDirectoryView *view,
+					  GnomeVFSResult result);
 
-	/* The 'create_background_context_menu_items' signal is emitted 
-	 * when creating a context menu, either an item-specific one or
-	 * a background one. Subclasses might want to override this to
-	 * modify the menu contents.
-	 */
-	void 	(* create_background_context_menu_items) 	 
-					 (FMDirectoryView *view,
-					  GtkMenu *menu);
-	 
 	/* Function pointers that don't have corresponding signals */
 
 	/* get_selection is not a signal; it is just a function pointer for
@@ -146,6 +141,20 @@ struct FMDirectoryViewClass {
 	 */
 	GList *	(* get_selection) 	 	(FMDirectoryView *view);
 	
+        /* select_all is a function pointer that subclasses must override to
+         * select all of the items in the view */
+        void     (* select_all)	         	(FMDirectoryView *view);
+
+        /* set_selection is a function pointer that subclasses must
+         * override to select the specified items (and unselect all
+         * others). The argument is a list of NautilusFiles. */
+
+        void     (* set_selection)	 	(FMDirectoryView *view, 
+        					 GList *selection);
+
+	/* Return an array of locations of selected icons in their view. */
+	GArray * (* get_selected_icon_locations) (FMDirectoryView *view);
+
         /* bump_zoom_level is a function pointer that subclasses must override
          * to change the zoom level of an object. */
         void    (* bump_zoom_level)      	(FMDirectoryView *view,
@@ -168,17 +177,6 @@ struct FMDirectoryViewClass {
          * return whether the view is at minimum size (furthest-out zoom level) */
         gboolean (* can_zoom_out)	 	(FMDirectoryView *view);
         
-        /* select_all is a function pointer that subclasses must override to
-         * select all of the items in the view */
-        void     (* select_all)	         	(FMDirectoryView *view);
-
-        /* set_selection is a function pointer that subclasses must
-         * override to select the specified items (and unselect all
-         * others). The argument is a list of NautilusFiles. */
-
-        void     (* set_selection)	 	(FMDirectoryView *view, 
-        					 GList *selection);
-
         /* reveal_selection is a function pointer that subclasses may
          * override to make sure the selected items are sufficiently
          * apparent to the user (e.g., scrolled into view). By default,
@@ -211,6 +209,15 @@ struct FMDirectoryViewClass {
 	 */
 	NautilusStringList * (* get_emblem_names_to_exclude)	(FMDirectoryView *view);
 
+	/* file_limit_reached is a function pointer that subclasses may
+	 * override to control what happens when a directory is loaded
+	 * that has more files than Nautilus can handle. The default
+	 * implmentation puts up a dialog box that is appropriate to
+	 * display when the user explicitly tried to visit a location that
+	 * they would think of as a normal directory.
+	 */
+	void (* file_limit_reached)		(FMDirectoryView *view);
+
 	/* supports_properties is a function pointer that subclasses may
 	 * override to control whether the "Show Properties" menu item
 	 * should be enabled for selected items. The default implementation 
@@ -268,85 +275,85 @@ struct FMDirectoryViewClass {
 };
 
 /* GtkObject support */
-GtkType            fm_directory_view_get_type                       (void);
+GtkType             fm_directory_view_get_type                         (void);
 
 /* Component embedding support */
-NautilusView *     fm_directory_view_get_nautilus_view              (FMDirectoryView       *view);
+NautilusView *      fm_directory_view_get_nautilus_view                (FMDirectoryView  *view);
 
 /* Functions callable from the user interface and elsewhere. */
-char *             fm_directory_view_get_uri		            (FMDirectoryView       *view);
-gboolean           fm_directory_view_can_accept_item                (NautilusFile          *target_item,
-								     const char            *item_uri,
-								     FMDirectoryView       *view);
-void		   fm_directory_view_display_selection_info	    (FMDirectoryView	   *view);
-GList *            fm_directory_view_get_selection                  (FMDirectoryView       *view);
-void               fm_directory_view_stop                           (FMDirectoryView       *view);
-gboolean           fm_directory_view_can_zoom_in                    (FMDirectoryView       *view);
-gboolean           fm_directory_view_can_zoom_out                   (FMDirectoryView       *view);
-GtkWidget *	   fm_directory_view_get_background_widget 	    (FMDirectoryView	   *view);
-void               fm_directory_view_bump_zoom_level                (FMDirectoryView       *view,
-								     int                    zoom_increment);
-void               fm_directory_view_zoom_to_level                  (FMDirectoryView        *view,
-								     int                    zoom_level);
-void               fm_directory_view_set_zoom_level		    (FMDirectoryView        *view,
-								     int                    zoom_level);
-void               fm_directory_view_restore_default_zoom_level     (FMDirectoryView       *view);
-void               fm_directory_view_select_all                     (FMDirectoryView       *view);
-void               fm_directory_view_set_selection                  (FMDirectoryView       *view,
-								     GList                 *selection);
-void               fm_directory_view_reveal_selection               (FMDirectoryView       *view);
-gboolean	   fm_directory_view_is_empty			    (FMDirectoryView	   *view);
-gboolean	   fm_directory_view_is_read_only		    (FMDirectoryView	   *view);
-gboolean	   fm_directory_view_supports_creating_files	    (FMDirectoryView	   *view);
-gboolean	   fm_directory_view_accepts_dragged_files	    (FMDirectoryView	   *view);
-gboolean	   fm_directory_view_supports_properties	    (FMDirectoryView	   *view);
-gboolean	   fm_directory_view_supports_zooming	    	    (FMDirectoryView	   *view);
-void               fm_directory_view_move_copy_items                (const GList           *item_uris,
-								     GdkPoint              *relative_item_points,
-								     const char            *target_uri,
-								     int                    copy_action,
-								     int                    x,
-								     int                    y,
-								     FMDirectoryView       *view);
-gint               fm_directory_view_get_context_menu_index         (GtkMenu 		   *menu,
-								     const char            *verb_path);
-gboolean	   fm_directory_link_type_in_selection 		    (FMDirectoryView 	   *view, 
-								     NautilusLinkType 	   link_type);
+char *              fm_directory_view_get_uri                          (FMDirectoryView  *view);
+gboolean            fm_directory_view_can_accept_item                  (NautilusFile     *target_item,
+									const char       *item_uri,
+									FMDirectoryView  *view);
+void                fm_directory_view_display_selection_info           (FMDirectoryView  *view);
+GList *             fm_directory_view_get_selection                    (FMDirectoryView  *view);
+void                fm_directory_view_stop                             (FMDirectoryView  *view);
+gboolean            fm_directory_view_can_zoom_in                      (FMDirectoryView  *view);
+gboolean            fm_directory_view_can_zoom_out                     (FMDirectoryView  *view);
+GtkWidget *         fm_directory_view_get_background_widget            (FMDirectoryView  *view);
+void                fm_directory_view_bump_zoom_level                  (FMDirectoryView  *view,
+									int               zoom_increment);
+void                fm_directory_view_zoom_to_level                    (FMDirectoryView  *view,
+									int               zoom_level);
+void                fm_directory_view_set_zoom_level                   (FMDirectoryView  *view,
+									int               zoom_level);
+void                fm_directory_view_restore_default_zoom_level       (FMDirectoryView  *view);
+void                fm_directory_view_select_all                       (FMDirectoryView  *view);
+void                fm_directory_view_set_selection                    (FMDirectoryView  *view,
+									GList            *selection);
+GArray *            fm_directory_get_selected_icon_locations           (FMDirectoryView  *view);
+void                fm_directory_view_reveal_selection                 (FMDirectoryView  *view);
+gboolean            fm_directory_view_is_empty                         (FMDirectoryView  *view);
+gboolean            fm_directory_view_is_read_only                     (FMDirectoryView  *view);
+gboolean            fm_directory_view_supports_creating_files          (FMDirectoryView  *view);
+gboolean            fm_directory_view_accepts_dragged_files            (FMDirectoryView  *view);
+gboolean            fm_directory_view_supports_properties              (FMDirectoryView  *view);
+gboolean            fm_directory_view_supports_zooming                 (FMDirectoryView  *view);
+void                fm_directory_view_move_copy_items                  (const GList      *item_uris,
+									GArray           *relative_item_points,
+									const char       *target_uri,
+									int               copy_action,
+									int               x,
+									int               y,
+									FMDirectoryView  *view);
+gboolean            fm_directory_link_type_in_selection                (FMDirectoryView  *view,
+									NautilusLinkType  link_type);
 
 /* Wrappers for signal emitters. These are normally called 
  * only by FMDirectoryView itself. They have corresponding signals
  * that observers might want to connect with.
  */
-void               fm_directory_view_clear                          (FMDirectoryView       *view);
-void               fm_directory_view_begin_loading                  (FMDirectoryView       *view);
+void                fm_directory_view_clear                            (FMDirectoryView  *view);
+void                fm_directory_view_begin_loading                    (FMDirectoryView  *view);
+void                fm_directory_view_end_loading                      (FMDirectoryView  *view);
 
 /* Hooks for subclasses to call. These are normally called only by 
  * FMDirectoryView and its subclasses 
  */
-void               fm_directory_view_activate_files                 (FMDirectoryView       *view,
-								     GList          	   *files);								     
-void		   fm_directory_view_start_batching_selection_changes (FMDirectoryView	   *view);
-void		   fm_directory_view_stop_batching_selection_changes (FMDirectoryView	   *view);
-gboolean	   fm_directory_view_confirm_multiple_windows	    (FMDirectoryView	   *view,
-								     int		    window_count);
-void		   fm_directory_view_queue_file_change	    	    (FMDirectoryView	   *view,
-								     NautilusFile 	   *file);
-void               fm_directory_view_notify_selection_changed       (FMDirectoryView       *view);
-Bonobo_UIContainer fm_directory_view_get_bonobo_ui_container        (FMDirectoryView       *view);
-NautilusStringList *fm_directory_view_get_emblem_names_to_exclude   (FMDirectoryView	   *view);
-NautilusDirectory  *fm_directory_view_get_model                     (FMDirectoryView       *view);
-NautilusFile       *fm_directory_view_get_directory_as_file         (FMDirectoryView       *view);
-NautilusBackground *fm_directory_view_get_background		    (FMDirectoryView 	   *view);
-void               fm_directory_view_pop_up_background_context_menu (FMDirectoryView       *view);
-void               fm_directory_view_pop_up_selection_context_menu  (FMDirectoryView       *view); 
-void               fm_directory_view_update_menus                   (FMDirectoryView       *view);
-GtkMenuItem 	  *fm_directory_view_insert_context_menu_item	    (FMDirectoryView 	   *view, 
-								     GtkMenu 		   *menu, 
-								     const char 	   *label,
-								     const char 	   *identifier,
-								     int		    position,
-								     void 		  (* callback) (GtkMenuItem *, FMDirectoryView *),
-								     gboolean 		   sensitive);
-void		   fm_directory_view_new_folder			    (FMDirectoryView       *view);
+void                fm_directory_view_activate_files                   (FMDirectoryView  *view,
+									GList            *files);
+void                fm_directory_view_start_batching_selection_changes (FMDirectoryView  *view);
+void                fm_directory_view_stop_batching_selection_changes  (FMDirectoryView  *view);
+gboolean            fm_directory_view_confirm_multiple_windows         (FMDirectoryView  *view,
+									int               window_count);
+void                fm_directory_view_queue_file_change                (FMDirectoryView  *view,
+									NautilusFile     *file);
+void                fm_directory_view_notify_selection_changed         (FMDirectoryView  *view);
+Bonobo_UIContainer  fm_directory_view_get_bonobo_ui_container          (FMDirectoryView  *view);
+BonoboControl *     fm_directory_view_get_bonobo_control               (FMDirectoryView  *view);
+NautilusStringList *fm_directory_view_get_emblem_names_to_exclude      (FMDirectoryView  *view);
+NautilusDirectory  *fm_directory_view_get_model                        (FMDirectoryView  *view);
+GtkWindow	   *fm_directory_view_get_containing_window	       (FMDirectoryView  *view);
+NautilusFile       *fm_directory_view_get_directory_as_file            (FMDirectoryView  *view);
+NautilusBackground *fm_directory_view_get_background                   (FMDirectoryView  *view);
+void                fm_directory_view_pop_up_background_context_menu   (FMDirectoryView  *view,
+									GdkEventButton   *event);
+void                fm_directory_view_pop_up_selection_context_menu    (FMDirectoryView  *view,
+									GdkEventButton   *event); 
+gboolean            fm_directory_view_should_show_file                 (FMDirectoryView  *view,
+									NautilusFile     *file);
+void                fm_directory_view_update_menus                     (FMDirectoryView  *view);
+void                fm_directory_view_new_folder                       (FMDirectoryView  *view);
+void                fm_directory_view_ignore_hidden_file_preferences   (FMDirectoryView  *view);
 
 #endif /* FM_DIRECTORY_VIEW_H */
