@@ -26,6 +26,7 @@
 #include <config.h>
 #include "fm-error-reporting.h"
 
+#include <string.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <libnautilus-extensions/nautilus-file.h>
@@ -43,29 +44,52 @@ fm_report_error_renaming_file (NautilusFile *file,
 	char *original_name;
 	char *message;
 
-	switch (error) {
-	case GNOME_VFS_OK:
+	if (error == GNOME_VFS_OK) {
 		return;
+	}
+
+	original_name = nautilus_file_get_name (file);
+	
+	switch (error) {
 	case GNOME_VFS_ERROR_FILE_EXISTS:
-		message = g_strdup_printf (_("The name \"%s\" is already used in this folder.\nPlease use a different name."), 
+		message = g_strdup_printf (_("The name \"%s\" is already used in this folder. "
+					     "Please use a different name."), 
 					   new_name);
 		break;
+	case GNOME_VFS_ERROR_NOT_FOUND:
+		message = g_strdup_printf (_("There is no \"%s\" in this folder. "
+					     "Perhaps it was just moved or deleted?"), 
+					   original_name);
+		break;
 	case GNOME_VFS_ERROR_ACCESS_DENIED:
-		original_name = nautilus_file_get_name (file);
-		message = g_strdup_printf (_("You do not have the permissions necessary to rename \"%s.\""),
+		message = g_strdup_printf (_("You do not have the permissions necessary to rename \"%s\"."),
+					   original_name);
+		break;
+	case GNOME_VFS_ERROR_NOT_PERMITTED:
+		if (strchr (new_name, '/') != NULL) {
+			message = g_strdup_printf (_("The name \"%s\" is not valid because it contains the character \"/\". "
+						     "Please use a different name."),
+						   new_name);
+		} else {
+			message = g_strdup_printf (_("The name \"%s\" is not valid. "
+						     "Please use a different name."),
+						   new_name);
+		}
+		break;
+	case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+		message = g_strdup_printf (_("Couldn't change the name of \"%s\" because it is on a read-only disk"),
 					   original_name);
-		g_free (original_name);
 		break;
 	default:
 		/* We should invent decent error messages for every case we actually experience. */
 		g_warning ("Hit unhandled case %d in fm_report_error_renaming_file, tell sullivan@eazel.com", error);
 		/* fall through */
-		original_name = nautilus_file_get_name (file);
 		message = g_strdup_printf (_("Sorry, couldn't rename \"%s\" to \"%s\"."),
 					   original_name, new_name);
-		g_free (original_name);
 	}
 
+	g_free (original_name);
+
 	nautilus_error_dialog (message, _("Renaming Error"), NULL);
 	g_free (message);
 }
@@ -77,14 +101,20 @@ fm_report_error_setting_group (NautilusFile *file,
 	char *file_name;
 	char *message;
 
-	switch (error) {
-	case GNOME_VFS_OK:
+	if (error == GNOME_VFS_OK) {
 		return;
+	}
+
+	file_name = nautilus_file_get_name (file);
+
+	switch (error) {
 	case GNOME_VFS_ERROR_NOT_PERMITTED:
-		file_name = nautilus_file_get_name (file);
 		message = g_strdup_printf (_("You do not have the permissions necessary to change the group of \"%s\"."),
 					   file_name);
-		g_free (file_name);
+		break;
+	case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+		message = g_strdup_printf (_("Couldn't change the group of \"%s\" because it is on a read-only disk"),
+					   file_name);
 		break;
 	default:
 		/* We should invent decent error messages for every case we actually experience. */
@@ -95,6 +125,8 @@ fm_report_error_setting_group (NautilusFile *file,
 	}
 
 	nautilus_error_dialog (message, _("Error Setting Group"), NULL);
+
+	g_free (file_name);
 	g_free (message);
 }		
 
@@ -105,18 +137,26 @@ fm_report_error_setting_owner (NautilusFile *file,
 	char *file_name;
 	char *message;
 
-	switch (error) {
-	case GNOME_VFS_OK:
+	if (error == GNOME_VFS_OK) {
 		return;
+	}
+
+	file_name = nautilus_file_get_name (file);
+
+	switch (error) {
+	case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+		message = g_strdup_printf (_("Couldn't change the owner of \"%s\" because it is on a read-only disk"),
+					   file_name);
+		break;
 	default:
 		/* We should invent decent error messages for every case we actually experience. */
 		g_warning ("Hit unhandled case %d in fm_report_error_setting_owner, tell sullivan@eazel.com", error);
-		file_name = nautilus_file_get_name (file);
 		message = g_strdup_printf (_("Sorry, couldn't change the owner of \"%s\"."), file_name);
-		g_free (file_name);
 	}
 
 	nautilus_error_dialog (message, _("Error Setting Owner"), NULL);
+
+	g_free (file_name);
 	g_free (message);
 }		
 
@@ -127,18 +167,26 @@ fm_report_error_setting_permissions (NautilusFile *file,
 	char *file_name;
 	char *message;
 
-	switch (error) {
-	case GNOME_VFS_OK:
+	if (error == GNOME_VFS_OK) {
 		return;
+	}
+
+	file_name = nautilus_file_get_name (file);
+
+	switch (error) {
+	case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+		message = g_strdup_printf (_("Couldn't change the permissions of \"%s\" because it is on a read-only disk"),
+					   file_name);
+		break;
 	default:
 		/* We should invent decent error messages for every case we actually experience. */
 		g_warning ("Hit unhandled case %d in fm_report_error_setting_permissions, tell sullivan@eazel.com", error);
-		file_name = nautilus_file_get_name (file);
 		message = g_strdup_printf (_("Sorry, couldn't change the permissions of \"%s\"."), file_name);
-		g_free (file_name);
 	}
 
 	nautilus_error_dialog (message, _("Error Setting Permissions"), NULL);
+
+	g_free (file_name);
 	g_free (message);
 }		
 
@@ -202,7 +250,7 @@ fm_rename_file (NautilusFile *file,
 
 	/* Start the timed wait to cancel the rename. */
 	old_name = nautilus_file_get_name (file);
-	wait_message = g_strdup_printf (_("Renaming %s to %s."),
+	wait_message = g_strdup_printf (_("Renaming \"%s\" to \"%s\"."),
 					old_name,
 					new_name);
 	g_free (old_name);
