@@ -42,6 +42,7 @@
 #include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
+#include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-string.h>
 
 static void     ensure_unique_attributes                  (int        menu_index);
@@ -193,7 +194,7 @@ changed_attributes_option_menu_callback (GtkMenuItem *menu_item, gpointer user_d
 	attribute_names_string = g_strjoinv ("|", attribute_names_array);
 
 	nautilus_preferences_set (NAUTILUS_PREFERENCES_ICON_CAPTIONS,
-				  attribute_names_string);
+					 attribute_names_string);
 
 	g_free (attribute_names_string);
 	g_strfreev (attribute_names_array);
@@ -257,8 +258,10 @@ create_icon_text_window (void)
   	int index;
 
   	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	nautilus_gtk_window_set_up_close_accelerator (GTK_WINDOW (window));
   	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
   	gtk_window_set_title (GTK_WINDOW (window), _("Icon Captions"));
+	gtk_window_set_wmclass (GTK_WINDOW (window), "icon_captions", "Nautilus");
   	gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);
 
   	contents_vbox = gtk_vbox_new (FALSE, 0);
@@ -391,17 +394,43 @@ fm_icon_text_window_get_or_create (void)
 	return GTK_WINDOW (icon_text_window);
 }
 
+/* This string will leak at exit time */
+static char *icon_captions = NULL;
+
+static void
+icon_captions_changed_callback (gpointer callback_data)
+{
+	char *new_icon_captions = NULL;
+	
+	g_free (icon_captions);
+	
+	new_icon_captions = nautilus_preferences_get (NAUTILUS_PREFERENCES_ICON_CAPTIONS);
+	
+	if (attribute_names_string_is_good (new_icon_captions)) {
+		icon_captions = new_icon_captions;
+		return;
+	}
+
+	g_free (new_icon_captions);
+
+	icon_captions = g_strdup (DEFAULT_ATTRIBUTE_NAMES);
+}
+
 char *
 fm_get_text_attribute_names_preference_or_default (void)
 {
-	char *preference;
+	static gboolean icon_captions_callback_added = FALSE;
 
-	preference = nautilus_preferences_get (NAUTILUS_PREFERENCES_ICON_CAPTIONS,
-					       DEFAULT_ATTRIBUTE_NAMES);
-	if (preference && attribute_names_string_is_good (preference)) {
-		return preference;
+	/* Add the callback once for the life of our process */
+	if (icon_captions_callback_added == FALSE) {
+		nautilus_preferences_add_callback_while_process_is_running (NAUTILUS_PREFERENCES_ICON_CAPTIONS,
+									    icon_captions_changed_callback,
+									    NULL);
+		icon_captions_callback_added = TRUE;
+
+		/* Peek for the first time */
+		icon_captions_changed_callback (NULL);
 	}
-	if (preference)
-		g_free (preference);
-	return g_strdup (DEFAULT_ATTRIBUTE_NAMES);
+
+	return g_strdup (icon_captions);
 }
