@@ -60,6 +60,7 @@
 #include <libnautilus-extensions/nautilus-sound.h>
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus/nautilus-bonobo-ui.h>
+#include <libnautilus/nautilus-clipboard.h>
 #include <locale.h>
 #include <signal.h>
 #include <stdio.h>
@@ -78,6 +79,8 @@
 #define MENU_PATH_SORT_REVERSED			"/menu/View/View Items Placeholder/Lay Out/Reversed Order"
 #define MENU_PATH_CLEAN_UP			"/menu/View/View Items Placeholder/Clean Up"
 
+#define POPUP_PATH_LAY_OUT			"/popups/background/Before Zoom Items/View Items/Lay Out"
+
 #define COMMAND_PREFIX                          "/commands/"
 #define COMMAND_RENAME 				"/commands/Rename"
 #define COMMAND_STRETCH_ICON 			"/commands/Stretch"
@@ -95,6 +98,7 @@
 static void     create_icon_container                     (FMIconView        *icon_view);
 static void     fm_icon_view_initialize                   (FMIconView        *icon_view);
 static void     fm_icon_view_initialize_class             (FMIconViewClass   *klass);
+static gboolean fm_icon_view_is_empty 			  (FMDirectoryView *view);
 static void     fm_icon_view_set_directory_sort_by        (FMIconView        *icon_view,
 							   NautilusFile      *file,
 							   const char        *sort_by);
@@ -194,6 +198,8 @@ struct FMIconViewDetails
 	guint react_to_icon_change_idle_id;
 	gboolean menus_ready;
 
+	gboolean loading;
+
 	const SortCriterion *sort;
 	gboolean sort_reversed;
 
@@ -315,36 +321,22 @@ set_sort_criterion (FMIconView *icon_view, const SortCriterion *sort)
 	return TRUE;
 }
 
-/**
- * Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
 static void
-show_stretch_handles_callback (gpointer ignored, gpointer view)
+show_stretch_handles_callback (BonoboUIComponent *component, gpointer callback_data, const char *verb)
 {
-	g_assert (FM_IS_ICON_VIEW (view));
+	g_assert (FM_IS_ICON_VIEW (callback_data));
 
 	nautilus_icon_container_show_stretch_handles
-		(get_icon_container (FM_ICON_VIEW (view)));
-
-        /* Update menus because Stretch and Unstretch items have changed state */
-	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+		(get_icon_container (FM_ICON_VIEW (callback_data)));
 }
 
-/**
- * Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
 static void
-unstretch_icons_callback (gpointer ignored, gpointer view)
+unstretch_icons_callback (BonoboUIComponent *component, gpointer callback_data, const char *verb)
 {
-	g_assert (FM_IS_ICON_VIEW (view));
+	g_assert (FM_IS_ICON_VIEW (callback_data));
 
 	nautilus_icon_container_unstretch
-		(get_icon_container (FM_ICON_VIEW (view)));
-
-        /* Update menus because Stretch and Unstretch items have changed state */
-	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+		(get_icon_container (FM_ICON_VIEW (callback_data)));
 }
 
 static void
@@ -366,38 +358,30 @@ fm_icon_view_real_clean_up (FMIconView *icon_view)
 	
 	set_sort_reversed (icon_view, FALSE);
 	set_sort_criterion (icon_view, &sort_criteria[0]);
-	
+
 	nautilus_icon_container_sort (icon_container);
 	nautilus_icon_container_freeze_icon_positions (icon_container);
 
 	set_sort_reversed (icon_view, saved_sort_reversed);
 }
 
-/**
- * Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
 static void
-clean_up_callback (gpointer ignored, gpointer view)
+clean_up_callback (BonoboUIComponent *component, gpointer callback_data, const char *verb)
 {
-	fm_icon_view_clean_up (FM_ICON_VIEW (view));
+	fm_icon_view_clean_up (FM_ICON_VIEW (callback_data));
 }
 
 	
 
-/**
- * Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
 static void
-rename_icon_callback (gpointer ignored, gpointer view)
+rename_icon_callback (BonoboUIComponent *component, gpointer callback_data, const char *verb)
 {
-	g_assert (FM_IS_ICON_VIEW (view));
+	g_assert (FM_IS_ICON_VIEW (callback_data));
   		
 	nautilus_icon_container_start_renaming_selected_item
-		(get_icon_container (FM_ICON_VIEW (view)));
+		(get_icon_container (FM_ICON_VIEW (callback_data)));
 
-	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (callback_data));
 }
 
 static void
@@ -411,40 +395,6 @@ set_tighter_layout (FMIconView *icon_view, gboolean new_value)
 						    new_value);	
 }
 
-static void
-gtk_tighter_layout_callback (GtkWidget *menu_item, gpointer user_data)
-{
-	FMIconView *icon_view;
-
-	icon_view = FM_ICON_VIEW (user_data);
-
-	/* This callback is called only by the context menu. It will
-	 * go away when we switch the context menus to use Bonobo.
-	 * This callback is only called when the user has actually chosen
-	 * the menu item, so we can be confident that the desired new
-	 * value is the opposite of the current value.
-	 */
-	set_tighter_layout (icon_view, !fm_icon_view_using_tighter_layout (icon_view));
-}
-
-static void
-gtk_sort_reversed_callback (GtkWidget *menu_item, gpointer user_data)
-{
-	FMIconView *icon_view;
-
-	icon_view = FM_ICON_VIEW (user_data);
-
-	/* This callback is called only by the context menu. It will
-	 * go away when we switch the context menus to use Bonobo.
-	 * This callback is only called when the user has actually chosen
-	 * the menu item, so we can be confident that the desired new
-	 * value is the opposite of the current value.
-	 */
-	if (set_sort_reversed (icon_view, !icon_view->details->sort_reversed)) {
-		nautilus_icon_container_sort (get_icon_container (icon_view));
-	}
-}
-
 static void
 tighter_layout_state_changed_callback (BonoboUIComponent   *component,
 				       const char          *path,
@@ -478,89 +428,26 @@ fm_icon_view_using_tighter_layout (FMIconView *icon_view)
 		(get_icon_container (icon_view));
 }
 
-static void
-compute_menu_item_info (FMIconView *view, 
-			GList *selection, 
-			const char *menu_path,
-			gboolean include_accelerator_underbars,
-			char **return_name,
-			gboolean *sensitive_return,
-			MenuItemType *menu_item_type_return)
+/* special_link_in_selection
+ * 
+ * Return TRUE is one of our special links is the selection.
+ * Special links include the following: 
+ *	 NAUTILUS_LINK_TRASH, NAUTILUS_LINK_HOME, NAUTILUS_LINK_MOUNT
+ */
+ 
+static gboolean
+special_link_in_selection (FMIconView *view)
 {
-	NautilusIconContainer *icon_container;
-	char *name, *stripped;
-	gboolean sensitive;
-	MenuItemType type;
-
-	g_assert (FM_IS_ICON_VIEW (view));
-	g_assert (menu_path != NULL);
-	g_assert (return_name != NULL);
-
-	icon_container = get_icon_container (view);
-	sensitive = TRUE;
-	type = MENU_ITEM_TYPE_STANDARD;
-	
-	if (strcmp (MENU_PATH_STRETCH_ICON, menu_path) == 0) {
-                name = g_strdup (_("_Stretch Icon"));
-		/* Current stretching UI only works on one item at a time, so we'll
-		 * desensitize the menu item if that's not the case.
-		 */
-        	sensitive = nautilus_g_list_exactly_one_item (selection)
-			&& (icon_container != NULL
-			    && !nautilus_icon_container_has_stretch_handles (icon_container));
-	} else if (strcmp (MENU_PATH_UNSTRETCH_ICONS, menu_path) == 0) {
-                if (nautilus_g_list_more_than_one_item (selection)) {
-                        name = g_strdup (_("_Restore Icons' Original Sizes"));
-                } else {
-                        name = g_strdup (_("_Restore Icon's Original Size"));
-                }
-        	sensitive = icon_container != NULL
-			&& nautilus_icon_container_is_stretched (icon_container);
-	} else if (strcmp (MENU_PATH_CUSTOMIZE_ICON_TEXT, menu_path) == 0) {
-                name = g_strdup (_("_Icon Captions..."));
-	} else if (strcmp (MENU_PATH_RENAME, menu_path) == 0) {
-		/* Modify file name. We only allow this on a single file selection. */
-		name = g_strdup (_("_Rename"));
-		sensitive = nautilus_g_list_exactly_one_item (selection)
-			&& nautilus_file_can_rename (selection->data);
-	} else if (strcmp (MENU_PATH_CLEAN_UP, menu_path) == 0) {
-		name = g_strdup (_("_Clean Up by Name"));
-		sensitive = !fm_icon_view_using_auto_layout (view);
-	} else if (strcmp (MENU_PATH_TIGHTER_LAYOUT, menu_path) == 0) {
-		name = g_strdup (_("_Use Tighter Layout"));
-		sensitive = fm_icon_view_using_auto_layout (view);
-		type = MENU_ITEM_TYPE_CHECK;
-	} else if (strcmp (MENU_PATH_SORT_REVERSED, menu_path) == 0) {
-		name = g_strdup (_("Re_versed Order"));
-		sensitive = fm_icon_view_using_auto_layout (view);
-		type = MENU_ITEM_TYPE_CHECK;
-	} else if (strcmp (MENU_PATH_LAY_OUT, menu_path) == 0) {
-		name = g_strdup (_("_Lay out items"));
-		type = MENU_ITEM_TYPE_TREE;
-	} else if (strcmp (MENU_PATH_MANUAL_LAYOUT, menu_path) == 0) {
-		name = g_strdup (_("_manually"));
-		type = MENU_ITEM_TYPE_RADIO;
-	} else {
-		name = NULL;
-		g_message ("Unknown menu path\"%s\" in compute_menu_item_info", menu_path);
+	if (fm_directory_link_type_in_selection (FM_DIRECTORY_VIEW (view), NAUTILUS_LINK_TRASH)) {
+		return TRUE;
 	}
 
-	if (!include_accelerator_underbars) {
-                stripped = nautilus_str_strip_chr (name, '_');
-		g_free (name);
-		name = stripped;
-        }
+	if (fm_directory_link_type_in_selection (FM_DIRECTORY_VIEW (view), NAUTILUS_LINK_MOUNT)) {
+		return TRUE;
+	}
 
-        if (sensitive_return != NULL) {
-		*sensitive_return = sensitive;
-        }
-
-        if (menu_item_type_return != NULL) {
-		*menu_item_type_return = type;
-        }
-
-	*return_name = name;
-}           
+	return FALSE;
+}
 
 static void
 handle_radio_item (FMIconView *view,
@@ -574,286 +461,11 @@ handle_radio_item (FMIconView *view,
 }
 
 static void
-context_menu_layout_radio_item_callback (GtkWidget *menu_item, gpointer user_data)
-{
-	FMIconView *icon_view;
-	const char *id;
-
-	icon_view = FM_ICON_VIEW (user_data);
-
-	id = (const char *) gtk_object_get_data (GTK_OBJECT (menu_item), "id");
-	g_assert (id != NULL);
-	handle_radio_item (icon_view, id);
-}
-
-static GtkMenuItem *
-append_one_context_menu_layout_item (FMIconView *view,
-                              	     GtkMenu *menu,
-                              	     GtkRadioMenuItem *item_in_group,
-                              	     const char *id,
-                              	     const char *menu_label)
-{
-	GtkWidget *menu_item;
-	char *label_for_display;
-
-	label_for_display = nautilus_str_strip_chr (menu_label, '_');
-        menu_item = gtk_radio_menu_item_new_with_label (
-        	item_in_group ? gtk_radio_menu_item_group (item_in_group) : NULL,
-        	label_for_display);
-        g_free (label_for_display);
-
-	/* Match appearance of Bonobo menu items, where unchosen items are still marked. */
-       	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (menu_item), TRUE);
-
-        /* Attach id to menu item so we can check it in the callback. */
-        gtk_object_set_data_full (GTK_OBJECT (menu_item), "id",
-				  g_strdup (id), g_free);
-        
-	gtk_widget_show (menu_item);
-	gtk_signal_connect (GTK_OBJECT (menu_item), "activate", context_menu_layout_radio_item_callback, view);
-	gtk_menu_append (menu, menu_item);
-
-	return GTK_MENU_ITEM (menu_item);
-}
-
-static GtkMenuItem *
-insert_one_context_menu_item (FMIconView *view,
-                              GtkMenu *menu,
-                              GList *selection,
-                              const char *menu_path,
-                              gint position,
-                              GtkSignalFunc callback,
-                              gboolean initial_state)
-{
-	GtkWidget *menu_item;
-	char *label;
-	gboolean sensitive;
-	MenuItemType type;
-        
-        compute_menu_item_info (view, selection, menu_path, FALSE, &label, &sensitive, &type);
-
-        switch (type) {
-        case MENU_ITEM_TYPE_CHECK:
-        	menu_item = gtk_check_menu_item_new_with_label (label);
-        	/* Match appearance of Bonobo toggle items, where inactive
-        	 * check items are still marked.
-        	 */
-        	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (menu_item), TRUE);
-		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), 
-					   	initial_state); 
-        	break;
-
-        case MENU_ITEM_TYPE_TREE:
-        case MENU_ITEM_TYPE_STANDARD:
-		menu_item = gtk_menu_item_new_with_label (label);
-		break;
-
-	case MENU_ITEM_TYPE_RADIO:	/* Not handled here due to required group parameter */
-        default:
-        	g_assert_not_reached ();
-		menu_item = NULL;
-        }
-
-        g_free (label);
-        gtk_widget_set_sensitive (menu_item, sensitive);
-	gtk_widget_show (menu_item);
-	gtk_signal_connect (GTK_OBJECT (menu_item), "activate", callback, view);
-	gtk_menu_insert (menu, menu_item, position);
-
-	return GTK_MENU_ITEM (menu_item);
-}
-
-static GtkMenuItem *
-append_one_context_menu_item (FMIconView *view,
-                              GtkMenu *menu,
-                              GList *selection,
-                              const char *menu_path,
-                              GtkSignalFunc callback)
-{
-	return insert_one_context_menu_item (view, menu, selection, menu_path, -1, callback, FALSE);
-}
-
-static GtkMenuItem *
-append_one_toggle_context_menu_item (FMIconView *view,
-                                     GtkMenu *menu,
-                                     GList *selection,
-                                     const char *menu_path,
-                                     GtkSignalFunc callback,
-                                     gboolean initial_state)
-{
-	return insert_one_context_menu_item (view, menu, selection, menu_path, -1, callback, initial_state);
-}
-
-/* special_link_in_selection
- * 
- * Return TRUE is one of our special links is the selection.
- * Special links include the following: 
- *	 NAUTILUS_LINK_TRASH, NAUTILUS_LINK_HOME, NAUTILUS_LINK_MOUNT
- */
- 
-static gboolean
-special_link_in_selection (FMDirectoryView *view)
-{
-	if (fm_directory_link_type_in_selection (view, NAUTILUS_LINK_TRASH)) {
-		return TRUE;
-	}
-
-	if (fm_directory_link_type_in_selection (view, NAUTILUS_LINK_HOME)) {
-		return TRUE;
-	}
-
-	if (fm_directory_link_type_in_selection (view, NAUTILUS_LINK_MOUNT)) {
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-static void
-fm_icon_view_create_selection_context_menu_items (FMDirectoryView *view,
-						  GtkMenu *menu,
-						  GList *selection)
-{
-	gint position;
-
-	g_assert (FM_IS_ICON_VIEW (view));
-	g_assert (GTK_IS_MENU (menu));
-
-	NAUTILUS_CALL_PARENT_CLASS
-		(FM_DIRECTORY_VIEW_CLASS, 
-		 create_selection_context_menu_items,
-		 (view, menu, selection));
-
-        append_one_context_menu_item
-		(FM_ICON_VIEW (view), menu, selection,
-		 MENU_PATH_STRETCH_ICON,
-		 GTK_SIGNAL_FUNC (show_stretch_handles_callback));
-        append_one_context_menu_item
-		(FM_ICON_VIEW (view), menu, selection,
-		 MENU_PATH_UNSTRETCH_ICONS,
-		 GTK_SIGNAL_FUNC (unstretch_icons_callback));
-	
-	/* The Rename item is inserted directly after the
-	 * Duplicate item created by the FMDirectoryView.
-	 */
-	if (!special_link_in_selection (view)) {
-		position = fm_directory_view_get_context_menu_index
-				(menu, FM_DIRECTORY_VIEW_COMMAND_DUPLICATE) + 1;
-     		insert_one_context_menu_item
-				(FM_ICON_VIEW (view), menu, selection, 
-		 		 MENU_PATH_RENAME, position,
-		 		 GTK_SIGNAL_FUNC (rename_icon_callback),
-		 		 FALSE);
-	}
-}
-
-/* Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
-static void
-customize_icon_text_callback (gpointer ignored1, gpointer ignored2)
+customize_icon_text_callback (BonoboUIComponent *component, gpointer callback_data, const char *verb)
 {
 	nautilus_gtk_window_present (fm_icon_text_window_get_or_create ());
 }
 
-static void
-fm_icon_view_create_background_context_menu_items (FMDirectoryView *view,
-						   GtkMenu *menu)
-{
-	FMIconView *icon_view;
-	char *manual_item_label;
-	GtkMenuItem *lay_out_item;
-	GtkMenuItem *toggle_item;
-	GtkRadioMenuItem *manual_item;
-	GtkMenu *layout_submenu;
-	gboolean is_auto_layout;
-	int position;
-	int i;
-
-	g_assert (GTK_IS_MENU (menu));
-
-	icon_view = FM_ICON_VIEW (view);
-
-	NAUTILUS_CALL_PARENT_CLASS
-		(FM_DIRECTORY_VIEW_CLASS, 
-		 create_background_context_menu_items, 
-		 (view, menu));
-
-	position = fm_directory_view_get_context_menu_index
-		(menu, FM_DIRECTORY_VIEW_COMMAND_NEW_FOLDER) + 1;
-
-	nautilus_gtk_menu_insert_separator (menu, position++);
-
-	if (fm_icon_view_supports_auto_layout (icon_view)) {
-
-		is_auto_layout = fm_icon_view_using_auto_layout (icon_view);
-
-	     	lay_out_item = GTK_MENU_ITEM (insert_one_context_menu_item
-			(icon_view, menu, NULL, 
-			 MENU_PATH_LAY_OUT, position++,
-			 NULL,
-			 FALSE));
-		layout_submenu = GTK_MENU (gtk_menu_new ());
-
-		/* Compute label the standard bonobo_or_gtk way, to avoid
-		 * duplicating string constant.
-		 */
-		compute_menu_item_info (icon_view, 
-					NULL, 
-					MENU_PATH_MANUAL_LAYOUT,
-					FALSE, 
-					&manual_item_label, 
-					NULL, 
-					NULL);
-		manual_item = GTK_RADIO_MENU_ITEM (append_one_context_menu_layout_item 
-			(icon_view, layout_submenu, NULL, 
-			 ID_MANUAL_LAYOUT, manual_item_label));
-		g_free (manual_item_label);
-		if (!is_auto_layout) {
-			gtk_check_menu_item_set_active 
-				(GTK_CHECK_MENU_ITEM (manual_item), TRUE);
-		}
-
-		nautilus_gtk_menu_append_separator (layout_submenu);
-
-		for (i = 0; i < NAUTILUS_N_ELEMENTS (sort_criteria); i++) {
-			toggle_item = append_one_context_menu_layout_item 
-				(icon_view, layout_submenu, manual_item, 
-			 	sort_criteria[i].id,
-			 	sort_criteria[i].menu_label);		
-			if (is_auto_layout && strcmp (icon_view->details->sort->id, 
-						      sort_criteria[i].id) == 0) {
-				gtk_check_menu_item_set_active 
-					(GTK_CHECK_MENU_ITEM (toggle_item), TRUE);
-			}
-		}
-	 
-		nautilus_gtk_menu_append_separator (layout_submenu);
-
-		/* No callback here, since that is handled by the bonobo "id" field */
-	     	toggle_item = append_one_toggle_context_menu_item
-			(icon_view, layout_submenu, NULL, 
-			 MENU_PATH_TIGHTER_LAYOUT,
-			 gtk_tighter_layout_callback,
-			 fm_icon_view_using_tighter_layout (icon_view));
-		
-
-	     	toggle_item = append_one_toggle_context_menu_item
-			(icon_view, layout_submenu, NULL, 
-			 MENU_PATH_SORT_REVERSED,
-			 gtk_sort_reversed_callback,
-			 icon_view->details->sort_reversed);
-		
-		gtk_menu_item_set_submenu (lay_out_item, GTK_WIDGET (layout_submenu));
-	}
-
-     	insert_one_context_menu_item
-		(icon_view, menu, NULL, 
-		 MENU_PATH_CLEAN_UP, position++,
-		 GTK_SIGNAL_FUNC (clean_up_callback),
-		 FALSE);
-}
-
 static void
 unref_cover (NautilusIconData *data, gpointer callback_data)
 {
@@ -877,9 +489,19 @@ fm_icon_view_clear (FMDirectoryView *view)
 static void
 fm_icon_view_add_file (FMDirectoryView *view, NautilusFile *file)
 {
+	FMIconView *icon_view;
+
 	g_assert (NAUTILUS_IS_FILE (file));
 
-	if (nautilus_icon_container_add (get_icon_container (FM_ICON_VIEW (view)),
+	icon_view = FM_ICON_VIEW (view);
+
+	/* Reset scroll region for the first icon added when loading a directory. */
+	if (icon_view->details->loading
+	    && fm_icon_view_is_empty (FM_DIRECTORY_VIEW (icon_view))) {
+		nautilus_icon_container_reset_scroll_region (get_icon_container (icon_view));
+	}
+
+	if (nautilus_icon_container_add (get_icon_container (icon_view),
 					 NAUTILUS_ICON_CONTAINER_ICON_DATA (file))) {
 		nautilus_file_ref (file);
 	}
@@ -918,6 +540,8 @@ update_layout_menus (FMIconView *view)
 
 	is_auto_layout = fm_icon_view_using_auto_layout (view);
 
+	bonobo_ui_component_freeze (view->details->ui, NULL);
+
 	if (fm_icon_view_supports_auto_layout (view)) {
 		/* Mark sort criterion. */
 		path = g_strconcat (COMMAND_PREFIX,
@@ -946,6 +570,8 @@ update_layout_menus (FMIconView *view)
 	/* Clean Up is only relevant for manual layout */
 	nautilus_bonobo_set_sensitive
 		(view->details->ui, COMMAND_CLEAN_UP, !is_auto_layout);	
+
+	bonobo_ui_component_thaw (view->details->ui, NULL);
 }
 
 
@@ -1157,7 +783,7 @@ set_sort_reversed (FMIconView *icon_view, gboolean new_value)
 static const SortCriterion *
 get_sort_criterion_by_metadata_text (const char *metadata_text)
 {
-	int i;
+	guint i;
 
 	/* Figure out what the new sort setting should be. */
 	for (i = 0; i < NAUTILUS_N_ELEMENTS (sort_criteria); i++) {
@@ -1171,7 +797,7 @@ get_sort_criterion_by_metadata_text (const char *metadata_text)
 static const SortCriterion *
 get_sort_criterion_by_id (const char *id)
 {
-	int i;
+	guint i;
 
 	/* Figure out what the new sort setting should be. */
 	for (i = 0; i < NAUTILUS_N_ELEMENTS (sort_criteria); i++) {
@@ -1197,14 +823,16 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	file = fm_directory_view_get_directory_as_file (view);
 	icon_container = GTK_WIDGET (get_icon_container (icon_view));
 
+	icon_view->details->loading = TRUE;
+
 	/* kill any sound preview process that is ongoing */
 	preview_sound (NULL, FALSE);
 	
-	/* FIXME: this should be done with a virtual function, not
-           hardcoding knowledge about a subclass into the parent class. */
-
+	/* FIXME bugzilla.eazel.com 5060: Should use methods instead
+	 * of hardcoding desktop knowledge in here.
+	 */
 	if (FM_IS_DESKTOP_ICON_VIEW (view)) {
-		nautilus_connect_desktop_background_to_file_metadata (icon_container, file);
+		nautilus_connect_desktop_background_to_file_metadata (NAUTILUS_ICON_CONTAINER (icon_container), file);
 	} else {
 		nautilus_connect_background_to_file_metadata (icon_container, file);
 	}
@@ -1238,7 +866,16 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	nautilus_icon_container_set_tighter_layout
 		(get_icon_container (icon_view), 
 		 fm_icon_view_get_directory_tighter_layout (icon_view, file));
+}
 
+static void
+fm_icon_view_end_loading (FMDirectoryView *view)
+{
+	FMIconView *icon_view;
+
+	icon_view = FM_ICON_VIEW (view);
+
+	icon_view->details->loading = FALSE;
 }
 
 static NautilusZoomLevel
@@ -1417,17 +1054,22 @@ fm_icon_view_get_selection (FMDirectoryView *view)
 static void
 set_sort_criterion_by_id (FMIconView *icon_view, const char *id)
 {
+	const SortCriterion *sort;
+
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 	g_assert (id != NULL);
 
-	set_sort_criterion (icon_view, get_sort_criterion_by_id (id));
+	sort = get_sort_criterion_by_id (id);
+
+	if (sort == icon_view->details->sort
+	    && fm_icon_view_using_auto_layout (icon_view)) {
+		return;
+	}
+
+	set_sort_criterion (icon_view, sort);
 	nautilus_icon_container_sort (get_icon_container (icon_view));
 }
 
-/**
- * Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
 static void
 sort_reversed_state_changed_callback (BonoboUIComponent *component,
 				      const char        *path,
@@ -1454,7 +1096,12 @@ sort_reversed_state_changed_callback (BonoboUIComponent *component,
 static void
 switch_to_manual_layout (FMIconView *icon_view)
 {
+	if (!fm_icon_view_using_auto_layout (icon_view)) {
+		return;
+	}
+
 	icon_view->details->sort = &sort_criteria[0];
+	
 	nautilus_icon_container_set_auto_layout
 		(get_icon_container (icon_view), FALSE);
 }
@@ -1512,11 +1159,11 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 {
 	FMIconView *icon_view;
 	BonoboUIVerb verbs [] = {
-		BONOBO_UI_VERB ("Rename", (BonoboUIVerbFn)rename_icon_callback),
-		BONOBO_UI_VERB ("Icon Text", (BonoboUIVerbFn)customize_icon_text_callback),
-		BONOBO_UI_VERB ("Stretch", (BonoboUIVerbFn)show_stretch_handles_callback),
-		BONOBO_UI_VERB ("Unstretch", (BonoboUIVerbFn)unstretch_icons_callback),
-		BONOBO_UI_VERB ("Clean Up", (BonoboUIVerbFn)clean_up_callback),
+		BONOBO_UI_VERB ("Rename", rename_icon_callback),
+		BONOBO_UI_VERB ("Icon Text", customize_icon_text_callback),
+		BONOBO_UI_VERB ("Stretch", show_stretch_handles_callback),
+		BONOBO_UI_VERB ("Unstretch", unstretch_icons_callback),
+		BONOBO_UI_VERB ("Clean Up", clean_up_callback),
 		BONOBO_UI_VERB_END
 	};
 	
@@ -1542,48 +1189,69 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 	bonobo_ui_component_add_listener (icon_view->details->ui, ID_SORT_REVERSED, sort_reversed_state_changed_callback, view);
 	icon_view->details->menus_ready = TRUE;
 
-	update_layout_menus (icon_view);
-}
-
-static void
-update_one_menu_item (FMIconView *view, 
-		      GList *selection,
-		      const char *menu_path,
-		      const char *verb_path)
-{
-	char *label;
-	gboolean sensitive;
+	bonobo_ui_component_freeze (icon_view->details->ui, NULL);
 	
-	compute_menu_item_info (view, selection, menu_path, TRUE, &label, &sensitive, NULL);
+	/* Do one-time state-setting here; context-dependent state-setting
+	 * is done in update_menus.
+	 */
+	if (!fm_icon_view_supports_auto_layout (icon_view)) {
+		nautilus_bonobo_set_hidden 
+			(icon_view->details->ui, POPUP_PATH_LAY_OUT, TRUE);
+	}
 
-	nautilus_bonobo_set_sensitive (view->details->ui, verb_path, sensitive);
-	nautilus_bonobo_set_label (view->details->ui, menu_path, label);
-	g_free (label);
+	update_layout_menus (icon_view);
+
+	bonobo_ui_component_thaw (icon_view->details->ui, NULL);
 }
 
 static void
 fm_icon_view_update_menus (FMDirectoryView *view)
 {
+	FMIconView *icon_view;
         GList *selection;
+        int selection_count;
+        NautilusIconContainer *icon_container;
+
+        icon_view = FM_ICON_VIEW (view);
 
 	/* don't update if the menus aren't ready */
-	if (!FM_ICON_VIEW (view)->details->menus_ready) {
+	if (!icon_view->details->menus_ready) {
 		return;
 	}
 	
 	NAUTILUS_CALL_PARENT_CLASS (FM_DIRECTORY_VIEW_CLASS, update_menus, (view));
 
         selection = fm_directory_view_get_selection (view);
+        selection_count = g_list_length (selection);
+        icon_container = get_icon_container (icon_view);
 
-	update_one_menu_item (FM_ICON_VIEW (view), selection, 
-			      MENU_PATH_STRETCH_ICON,
-			      COMMAND_STRETCH_ICON);
-        update_one_menu_item (FM_ICON_VIEW (view), selection, 
-			      MENU_PATH_UNSTRETCH_ICONS,
-			      COMMAND_UNSTRETCH_ICONS);
-        update_one_menu_item (FM_ICON_VIEW (view), selection, 
-			      MENU_PATH_RENAME,
-			      COMMAND_RENAME);
+	bonobo_ui_component_freeze (icon_view->details->ui, NULL);
+
+	nautilus_bonobo_set_sensitive (icon_view->details->ui, 
+				       COMMAND_STRETCH_ICON,
+				       selection_count == 1
+				       && icon_container != NULL
+			    	       && !nautilus_icon_container_has_stretch_handles (icon_container));
+
+	nautilus_bonobo_set_label_for_menu_item_and_command 
+		(icon_view->details->ui,
+		 MENU_PATH_UNSTRETCH_ICONS,
+		 COMMAND_UNSTRETCH_ICONS,
+		 nautilus_g_list_more_than_one_item (selection)
+		 	? _("_Restore Icons' Original Sizes")
+		 	: _("_Restore Icon's Original Size"));
+	nautilus_bonobo_set_sensitive (icon_view->details->ui, 
+				       COMMAND_UNSTRETCH_ICONS,
+				       icon_container != NULL
+			    	       && nautilus_icon_container_is_stretched (icon_container));
+
+	nautilus_bonobo_set_sensitive (icon_view->details->ui, 
+				       COMMAND_RENAME,
+				       selection_count == 1
+				       && nautilus_file_can_rename (selection->data)
+				       && !special_link_in_selection (icon_view));
+
+	bonobo_ui_component_thaw (icon_view->details->ui, NULL);
 	
 	nautilus_file_list_free (selection);
 }
@@ -1618,15 +1286,23 @@ fm_icon_view_reveal_selection (FMDirectoryView *view)
         nautilus_file_list_free (selection);
 }
 
+static GArray *
+fm_icon_view_get_selected_icon_locations (FMDirectoryView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), NULL);
+
+	return nautilus_icon_container_get_selected_icon_locations
+		(get_icon_container (FM_ICON_VIEW (view)));
+}
+
+
 static void
 fm_icon_view_set_selection (FMDirectoryView *view, GList *selection)
 {
-	NautilusIconContainer *icon_container;
-
 	g_return_if_fail (FM_IS_ICON_VIEW (view));
 
-	icon_container = get_icon_container (FM_ICON_VIEW (view));
-	nautilus_icon_container_set_selection (icon_container, selection);
+	nautilus_icon_container_set_selection
+		(get_icon_container (FM_ICON_VIEW (view)), selection);
 }
 
 static void
@@ -1660,7 +1336,7 @@ band_select_ended_callback (NautilusIconContainer *container,
 	fm_directory_view_stop_batching_selection_changes (FM_DIRECTORY_VIEW (icon_view));
 }
 
-/* handle the preview signal by inspecting the mime type.  For now, we only preview sound files. */
+/* handle the preview signal by inspecting the mime type.  For now, we only preview local sound files. */
 
 /* here's the timer task that actually plays the file using mpg123. */
 /* FIXME bugzilla.eazel.com 1258: we should get the application from our mime-type stuff */
@@ -1728,6 +1404,26 @@ preview_sound (NautilusFile *file, gboolean start_flag)
 	}
 }
 
+
+static gboolean
+should_preview_sound (NautilusFile *file) {
+	int preview_mode;
+	
+	preview_mode = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_PREVIEW_SOUND);
+
+	if (preview_mode == NAUTILUS_SPEED_TRADEOFF_NEVER) {
+		return FALSE;
+	}
+	/* the followinf is disabled until we can preview remote sounds, which we currently can't do */
+	/*
+	if (preview_mode == NAUTILUS_SPEED_TRADEOFF_ALWAYS) {
+		return TRUE;
+	}
+	*/
+	
+	return nautilus_file_is_local (file);
+}
+
 static int
 icon_container_preview_callback (NautilusIconContainer *container,
 				 NautilusFile *file,
@@ -1741,14 +1437,18 @@ icon_container_preview_callback (NautilusIconContainer *container,
 	
 	/* preview files based on the mime_type. */
 	/* at first, we just handle sounds */
-	mime_type = nautilus_file_get_mime_type (file);
-	if (nautilus_istr_has_prefix (mime_type, "audio/")) {   	
-		if (nautilus_sound_can_play_sound ()) {
-			result = 1;
-			preview_sound (file, start_flag);
-		}
-	}	
-	g_free (mime_type);
+
+	if (should_preview_sound (file)) {
+		mime_type = nautilus_file_get_mime_type (file);
+		if (nautilus_istr_has_prefix (mime_type, "audio/") &&
+				nautilus_strcasecmp (mime_type, "audio/x-pn-realaudio") != 0) {
+			if (nautilus_sound_can_play_sound ()) {
+				result = 1;
+				preview_sound (file, start_flag);
+			}
+		}	
+		g_free (mime_type);
+	}
 	
 	/* display file name in status area at low zoom levels, since the name is not displayed or hard to read */
 	if (fm_icon_view_get_zoom_level (icon_view) <= NAUTILUS_ZOOM_LEVEL_SMALLER) {
@@ -1768,6 +1468,25 @@ icon_container_preview_callback (NautilusIconContainer *container,
 	return result;
 }
 
+static void
+renaming_icon_callback (NautilusIconContainer *container,
+			gpointer editable_data,
+			gpointer callback_data)
+{
+	FMDirectoryView *directory_view;
+
+	directory_view = FM_DIRECTORY_VIEW (callback_data);
+	nautilus_clipboard_set_up_editable_in_control (GTK_EDITABLE (editable_data),
+						       fm_directory_view_get_bonobo_control (directory_view),
+						       TRUE);
+	/* Focus the editable in so the clipboard items will get turned on
+	   while we're renaming;  the hack that the nautilus entry is a 
+	   virtual widget prevents this from happening normally */
+	gtk_signal_emit_by_name (GTK_OBJECT (editable_data), "grab_focus");
+
+	
+}
+
 static int
 icon_container_compare_icons_callback (NautilusIconContainer *container,
 				       NautilusFile *file_a,
@@ -1803,22 +1522,26 @@ selection_changed_callback (NautilusIconContainer *container,
 
 static void
 icon_container_context_click_selection_callback (NautilusIconContainer *container,
+						 GdkEventButton *event,
 						 FMIconView *icon_view)
 {
 	g_assert (NAUTILUS_IS_ICON_CONTAINER (container));
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 
-	fm_directory_view_pop_up_selection_context_menu (FM_DIRECTORY_VIEW (icon_view));
+	fm_directory_view_pop_up_selection_context_menu 
+		(FM_DIRECTORY_VIEW (icon_view), event);
 }
 
 static void
 icon_container_context_click_background_callback (NautilusIconContainer *container,
+						  GdkEventButton *event,
 						  FMIconView *icon_view)
 {
 	g_assert (NAUTILUS_IS_ICON_CONTAINER (container));
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 
-	fm_directory_view_pop_up_background_context_menu (FM_DIRECTORY_VIEW (icon_view));
+	fm_directory_view_pop_up_background_context_menu 
+		(FM_DIRECTORY_VIEW (icon_view), event);
 }
 
 static gboolean
@@ -1959,7 +1682,6 @@ get_icon_uri_callback (NautilusIconContainer *container,
 	g_assert (NAUTILUS_IS_FILE (file));
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 
-	
 	return nautilus_file_get_uri (file);
 }
 
@@ -2137,6 +1859,7 @@ fm_icon_view_initialize_class (FMIconViewClass *klass)
 	
 	fm_directory_view_class->add_file = fm_icon_view_add_file;
 	fm_directory_view_class->begin_loading = fm_icon_view_begin_loading;
+	fm_directory_view_class->end_loading = fm_icon_view_end_loading;
 	fm_directory_view_class->bump_zoom_level = fm_icon_view_bump_zoom_level;
 	fm_directory_view_class->zoom_to_level = fm_icon_view_zoom_to_level;
 	fm_directory_view_class->restore_default_zoom_level = fm_icon_view_restore_default_zoom_level;
@@ -2150,10 +1873,7 @@ fm_icon_view_initialize_class (FMIconViewClass *klass)
 	fm_directory_view_class->select_all = fm_icon_view_select_all;
 	fm_directory_view_class->set_selection = fm_icon_view_set_selection;
 	fm_directory_view_class->reveal_selection = fm_icon_view_reveal_selection;
-        fm_directory_view_class->create_background_context_menu_items =
-		fm_icon_view_create_background_context_menu_items;
-        fm_directory_view_class->create_selection_context_menu_items =
-		fm_icon_view_create_selection_context_menu_items;
+	fm_directory_view_class->get_selected_icon_locations = fm_icon_view_get_selected_icon_locations;
         fm_directory_view_class->merge_menus = fm_icon_view_merge_menus;
         fm_directory_view_class->update_menus = fm_icon_view_update_menus;
         fm_directory_view_class->start_renaming_item = fm_icon_view_start_renaming_item;
@@ -2208,7 +1928,7 @@ icon_view_get_container_uri (NautilusIconContainer *container,
 static void
 icon_view_move_copy_items (NautilusIconContainer *container,
 			   const GList *item_uris,
-			   GdkPoint *relative_item_points,
+			   GArray *relative_item_points,
 			   const char *target_dir,
 			   int copy_action,
 			   int x, int y,
@@ -2251,8 +1971,7 @@ fm_icon_view_update_click_mode (FMIconView *icon_view)
 	icon_container = get_icon_container (icon_view);
 	g_assert (icon_container != NULL);
 
-	click_mode = nautilus_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY,
-						    NAUTILUS_CLICK_POLICY_DOUBLE);
+	click_mode = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_CLICK_POLICY);
 
 
 	nautilus_icon_container_set_single_click_mode (icon_container,
@@ -2268,9 +1987,8 @@ fm_icon_view_update_smooth_graphics_mode (FMIconView *icon_view)
 	icon_container = get_icon_container (icon_view);
 	g_assert (icon_container != NULL);
 
-	smooth_graphics_mode = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
-								 TRUE);
-
+	smooth_graphics_mode = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE);
+	
 	nautilus_icon_container_set_anti_aliased_mode (icon_container, smooth_graphics_mode);
 }
 
@@ -2361,6 +2079,18 @@ create_icon_container (FMIconView *icon_view)
 			    "preview",
 			    GTK_SIGNAL_FUNC (icon_container_preview_callback),
 			    icon_view);
+	gtk_signal_connect (GTK_OBJECT (icon_container),
+			    "renaming_icon",
+			    renaming_icon_callback,
+			    directory_view);
+	gtk_signal_connect_object (GTK_OBJECT (icon_container),
+			           "icon_stretch_started",
+			           fm_directory_view_update_menus,
+			           GTK_OBJECT (directory_view));
+	gtk_signal_connect_object (GTK_OBJECT (icon_container),
+			           "icon_stretch_ended",
+			           fm_directory_view_update_menus,
+			           GTK_OBJECT (directory_view));
 
 	gtk_container_add (GTK_CONTAINER (icon_view),
 			   GTK_WIDGET (icon_container));
