@@ -26,6 +26,7 @@
 #include "fm-list-view.h"
 
 #include "fm-list-view-private.h"
+#include "fm-properties-window.h"
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
@@ -37,6 +38,7 @@
 #include <libnautilus-extensions/nautilus-font-factory.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
+#include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-icon-factory.h>
@@ -56,7 +58,7 @@ struct FMListViewDetails {
  * Emblems should never get so small that they're illegible,
  * so we semi-arbitrarily choose a minimum size.
  */
-#define LIST_VIEW_MINIMUM_EMBLEM_SIZE	NAUTILUS_ICON_SIZE_SMALL
+#define LIST_VIEW_MINIMUM_EMBLEM_SIZE	NAUTILUS_ICON_SIZE_SMALLER
 
 /*
  * The row height should be large enough to not clip emblems.
@@ -122,6 +124,7 @@ static void                 fm_list_view_font_family_changed          (FMDirecto
 static void                 fm_list_view_set_selection                (FMDirectoryView   *view,
 								       GList             *selection);
 static void                 fm_list_view_reveal_selection             (FMDirectoryView   *view);
+static GArray * 	    fm_list_view_get_selected_icon_locations  (FMDirectoryView   *view);
 static void                 fm_list_view_set_zoom_level               (FMListView        *list_view,
 								       NautilusZoomLevel  new_level,
 								       gboolean           always_set_level);
@@ -164,6 +167,8 @@ static void                 real_get_column_specification             (FMListVie
 								       int                column_number,
 								       FMListViewColumn  *specification);
 static gboolean		    real_is_empty			      (FMDirectoryView	 *view);
+static void		    real_start_renaming_item  		      (FMDirectoryView   *view, 
+								       const char 	 *uri);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (FMListView,
 				   fm_list_view,
@@ -201,6 +206,8 @@ fm_list_view_initialize_class (gpointer klass)
 	fm_directory_view_class->select_all = fm_list_view_select_all;
 	fm_directory_view_class->set_selection = fm_list_view_set_selection;
 	fm_directory_view_class->reveal_selection = fm_list_view_reveal_selection;
+	fm_directory_view_class->start_renaming_item = real_start_renaming_item;
+	fm_directory_view_class->get_selected_icon_locations = fm_list_view_get_selected_icon_locations;
         fm_directory_view_class->click_policy_changed = fm_list_view_update_click_mode;
         fm_directory_view_class->embedded_text_policy_changed = fm_list_view_embedded_text_policy_changed;
         fm_directory_view_class->image_display_policy_changed = fm_list_view_image_display_policy_changed;
@@ -353,20 +360,26 @@ match_row_name (gconstpointer a, void *callback_data)
 }
 
 static void 
-context_click_selection_callback (NautilusCList *clist, FMListView *list_view)
+context_click_selection_callback (NautilusCList *clist, 
+				  GdkEventButton *event,
+				  FMListView *list_view)
 {
 	g_assert (NAUTILUS_IS_CLIST (clist));
 	g_assert (FM_IS_LIST_VIEW (list_view));
 
-	fm_directory_view_pop_up_selection_context_menu (FM_DIRECTORY_VIEW (list_view));
+	fm_directory_view_pop_up_selection_context_menu 
+		(FM_DIRECTORY_VIEW (list_view), event);
 }
 
 static void 
-context_click_background_callback (NautilusCList *clist, FMListView *list_view)
+context_click_background_callback (NautilusCList *clist,
+				   GdkEventButton *event,
+				   FMListView *list_view)
 {
 	g_assert (FM_IS_LIST_VIEW (list_view));
 
-	fm_directory_view_pop_up_background_context_menu (FM_DIRECTORY_VIEW (list_view));
+	fm_directory_view_pop_up_background_context_menu 
+		(FM_DIRECTORY_VIEW (list_view), event);
 }
 
 static GPtrArray *
@@ -388,15 +401,16 @@ make_sorted_row_array (GtkWidget *widget)
 }
 
 static void
-select_row_common (GtkWidget *widget, const GPtrArray *array, int array_row_index)
+select_row_common (GtkWidget *widget, const GPtrArray *array, guint array_row_index)
 {
 	NautilusCListRow *row;
 	int list_row_index;
 
-	if (array_row_index >= array->len)
+	if (array_row_index >= array->len) {
+		g_assert (array->len >= 1);
 		array_row_index = array->len - 1;
+	}
 
-	g_assert (array_row_index >= 0);
 	row = g_ptr_array_index (array, array_row_index);
 
 	g_assert (row != NULL);
@@ -429,6 +443,7 @@ select_matching_name_callback (GtkWidget *widget, const char *pattern, FMListVie
 	array_row_index = nautilus_g_ptr_array_search (array, match_row_name, 
 						       (char *) pattern, FALSE);
 
+	g_assert (array_row_index >= 0);
 	select_row_common (widget, array, array_row_index);
 
 	g_ptr_array_free (array, TRUE);
@@ -439,7 +454,7 @@ select_previous_next_common (GtkWidget *widget, FMListView *list_view, gboolean
 {
 	GPtrArray *array;
 	int array_row_index;
-	int index;
+	guint index;
 	int first_selected_row;
 	int last_selected_row;
 
@@ -493,7 +508,7 @@ select_previous_next_common (GtkWidget *widget, FMListView *list_view, gboolean
  		array_row_index = 0;
  	}
 
- 	if (array_row_index >= array->len) {
+ 	if (array_row_index >= (int) array->len) {
  		array_row_index = array->len - 1;
  	}
 
@@ -538,6 +553,7 @@ fm_list_receive_dropped_icons (NautilusList *list,
 	char *target_item_uri;
 	NautilusFile *target_item;
 	GList *source_uris, *p;
+	GArray *dummy;
 
 	target_item_uri = NULL;
 	source_uris = NULL;
@@ -563,8 +579,10 @@ fm_list_receive_dropped_icons (NautilusList *list,
 			target_item_uri = nautilus_file_get_uri (target_item);
 		}
 
-		if (target_item_uri != NULL) {
-
+		if (target_item_uri != NULL
+			&& (action != GDK_ACTION_MOVE
+				/* don't actually move the items if they are in the same directory */
+				|| !nautilus_drag_items_local (target_item_uri, drop_data))) {
 			/* build a list of URIs to copy */
 			for (p = drop_data; p != NULL; p = p->next) {
 				/* do a shallow copy of all the uri strings of the copied files */
@@ -573,9 +591,15 @@ fm_list_receive_dropped_icons (NautilusList *list,
 			}
 			source_uris = g_list_reverse (source_uris);
 			
+			/* pass in a 0-item array of icon positions */
+			dummy = g_array_new (FALSE, TRUE, sizeof (GdkPoint));
 			/* start the copy */
-			fm_directory_view_move_copy_items (source_uris, NULL,
-							   target_item_uri, action, x, y, directory_view);
+			fm_directory_view_move_copy_items (source_uris, dummy,
+							   target_item_uri,
+							   action,
+							   x, y,
+							   directory_view);
+		        g_array_free (dummy, TRUE);
 		}
 	}
 	
@@ -680,35 +704,37 @@ fm_list_get_default_action (NautilusList *list,
 
 	switch (info) {
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-
-		drop_target = nautilus_list_find_icon_list_drop_target (list, x, y,
-									drop_data,
-									list_view);
-		if (!drop_target) {
+		if (drop_data == NULL) {
 			*default_action = 0;
 			*non_default_action = 0;
 			return;
 		}
+
+		drop_target = nautilus_list_find_icon_list_drop_target (list, x, y,
+									drop_data,
+									list_view);
+		if (drop_target == NULL) {
+			*default_action = 0;
+			*non_default_action = 0;
+			return;
+		}
+
 		nautilus_drag_default_drop_action_for_icons (context, drop_target, 
 							     drop_data, 
 							     default_action, non_default_action);
 		break;
-
+		
 	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:
 		*default_action = context->suggested_action;
 		*non_default_action = context->suggested_action;
 		break;
-
+		
 	case NAUTILUS_ICON_DND_KEYWORD:
-		/* FIXME bugzilla.eazel.com 2572: Doesn't handle dropped keywords in 
-		   list view? Do we need to support this? */
 		*default_action = context->suggested_action;
 		*non_default_action = context->suggested_action;
 		break;
-	default:
 	}
-
 }
 
 
@@ -812,21 +838,17 @@ static void
 fm_list_get_drag_pixmap (GtkWidget *widget, int row_index, GdkPixmap **pixmap, 
 			 GdkBitmap **mask, FMListView *list_view)
 {
-	NautilusCList *clist;
-	NautilusCListRow *row;
+	GdkPixbuf *pixbuf;
 
 	g_assert (NAUTILUS_IS_LIST (widget));
 
-	clist = NAUTILUS_CLIST (widget);
-	row = ROW_ELEMENT (clist, row_index)->data;
+	pixbuf = nautilus_list_get_pixbuf (NAUTILUS_LIST (widget),
+					   row_index, LIST_VIEW_COLUMN_ICON);
 
-	g_assert (row != NULL);
+	g_assert (pixbuf != NULL);
 
-	*pixmap = gdk_pixmap_ref (GTK_CELL_PIXMAP (row->cell[LIST_VIEW_COLUMN_ICON])->pixmap);
-	*mask = NULL;
-	if (GTK_CELL_PIXMAP(row->cell[LIST_VIEW_COLUMN_ICON])->mask != NULL) {
-		*mask = gdk_bitmap_ref (GTK_CELL_PIXMAP (row->cell[LIST_VIEW_COLUMN_ICON])->mask);
-	}
+	gdk_pixbuf_render_pixmap_and_mask (pixbuf, pixmap, mask,
+					   NAUTILUS_STANDARD_ALPHA_THRESHHOLD);
 }
 
 static int
@@ -1095,7 +1117,7 @@ fm_list_view_bump_zoom_level (FMDirectoryView *view, int zoom_increment)
 	list_view = FM_LIST_VIEW (view);
 	old_level = fm_list_view_get_zoom_level (list_view);
 
-	if (zoom_increment < 0 && 0 - zoom_increment > old_level) {
+	if (zoom_increment < 0 && 0 - zoom_increment > (int) old_level) {
 		new_level = NAUTILUS_ZOOM_LEVEL_SMALLEST;
 	} else {
 		new_level = MIN (old_level + zoom_increment,
@@ -1373,6 +1395,19 @@ real_is_empty (FMDirectoryView *view)
 	return NAUTILUS_CLIST (get_list (FM_LIST_VIEW (view)))->rows == 0;
 }
 
+static void
+real_start_renaming_item  (FMDirectoryView *view, const char *uri)
+{
+	NautilusFile *file;
+
+	/* call parent class to make sure the right icon is selected */
+	NAUTILUS_CALL_PARENT_CLASS (FM_DIRECTORY_VIEW_CLASS, start_renaming_item, (view, uri));
+	/* Show properties window since we don't do in-place renaming here */
+	file = nautilus_file_get (uri);
+	fm_properties_window_present (file, view);
+	nautilus_file_unref (file);
+}
+
 static GList *
 fm_list_view_get_selection (FMDirectoryView *view)
 {
@@ -1508,6 +1543,16 @@ fm_list_view_reveal_selection (FMDirectoryView *view)
         nautilus_file_list_free (selection);
 }
 
+static GArray *
+fm_list_view_get_selected_icon_locations (FMDirectoryView *view)
+{
+	g_return_val_if_fail (FM_IS_LIST_VIEW (view), NULL);
+
+	/* icon locations are not stored in the list view,
+	 * just return an empty list
+	 */
+	return g_array_new (FALSE, TRUE, sizeof (GdkPoint));
+}
 
 static void
 fm_list_view_sort_items (FMListView *list_view, 
@@ -1523,8 +1568,7 @@ fm_list_view_sort_items (FMListView *list_view,
 	g_return_if_fail (column < NAUTILUS_CLIST (get_list (list_view))->columns);
 
 	if (column == list_view->details->sort_column &&
-	    reversed == list_view->details->sort_reversed)
-	{
+	    reversed == list_view->details->sort_reversed) {
 		return;
 	}
 
@@ -1545,8 +1589,7 @@ fm_list_view_sort_items (FMListView *list_view,
 
 	list_view->details->sort_column = column;
 
-	if (reversed != list_view->details->sort_reversed)
-	{
+	if (reversed != list_view->details->sort_reversed) {
 		nautilus_clist_set_sort_type (clist, reversed
 					 ? GTK_SORT_DESCENDING
 					 : GTK_SORT_ASCENDING);
@@ -1630,7 +1673,7 @@ fm_list_view_get_emblem_pixbufs_for_file (FMListView *list_view,
 			(p->data, 
 			 emblem_size, emblem_size,
 			 emblem_size, emblem_size,
-			 NULL);
+			 NULL, FALSE);
 		if (emblem_pixbuf != NULL) {
 			emblem_pixbufs = g_list_prepend
 				(emblem_pixbufs, emblem_pixbuf);
@@ -1657,8 +1700,7 @@ install_row_images (FMListView *list_view, guint row)
 	NautilusList *list;
 	NautilusCList *clist;
 	NautilusFile *file;
-	GdkPixmap *pixmap;
-	GdkBitmap *bitmap;
+	GdkPixbuf *icon;
 
 	g_return_if_fail (FM_IS_LIST_VIEW (list_view));
 
@@ -1669,16 +1711,9 @@ install_row_images (FMListView *list_view, guint row)
 	g_return_if_fail (file != NULL);
 
 	/* Install the icon for this file. */
-	nautilus_icon_factory_get_pixmap_and_mask_for_file
-		(file,
-		 NULL,
-		 fm_list_view_get_icon_size (list_view),
-		 &pixmap, &bitmap);
-	nautilus_clist_set_pixmap (clist, row, LIST_VIEW_COLUMN_ICON, pixmap, bitmap);
-	gdk_pixmap_unref (pixmap);
-	if (bitmap != NULL) {
-		gdk_bitmap_unref (bitmap);
-	}
+	icon = (nautilus_icon_factory_get_pixbuf_for_file
+		(file, NULL, fm_list_view_get_icon_size (list_view), TRUE));
+	nautilus_list_set_pixbuf (list, row, LIST_VIEW_COLUMN_ICON, icon);
 
 	/* Install any emblems for this file. */
 	nautilus_list_set_pixbuf_list (list, row, LIST_VIEW_COLUMN_EMBLEMS, 
@@ -1706,8 +1741,7 @@ fm_list_view_update_click_mode (FMDirectoryView *view)
 	g_assert (FM_IS_LIST_VIEW (view));
 	g_assert (get_list (FM_LIST_VIEW (view)) != NULL);
 
-	click_mode = nautilus_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY,
-						    NAUTILUS_CLICK_POLICY_DOUBLE);
+	click_mode = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_CLICK_POLICY);
 
 	nautilus_list_set_single_click_mode (get_list (FM_LIST_VIEW (view)), 
 		click_mode == NAUTILUS_CLICK_POLICY_SINGLE);
@@ -1883,7 +1917,7 @@ real_get_column_specification (FMListView *view,
 		fm_list_view_column_set (specification,
 					 "name", _("Name"),
 					 NAUTILUS_FILE_SORT_BY_NAME,
-					 30, 130, 300, FALSE);
+					 30, 170, 300, FALSE);
 		break;
 	case 3:
 		fm_list_view_column_set (specification,
@@ -1895,7 +1929,7 @@ real_get_column_specification (FMListView *view,
 		fm_list_view_column_set (specification,
 					 "type", _("Type"),
 					 NAUTILUS_FILE_SORT_BY_TYPE,
-					 20, 95, 200, FALSE);
+					 20, 135, 200, FALSE);
 		break;
 	case 5:
 		fm_list_view_column_set (specification,
