@@ -32,13 +32,22 @@
 #include <bonobo/bonobo-ui-util.h>
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-uidefs.h>
+#include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#ifdef HAVE_MEDUSA
+#include <libmedusa/medusa-indexed-search.h>
+#include <libmedusa/medusa-unindexed-search.h>
+#include <libmedusa/medusa-index-service.h>
+#endif
 #include <libnautilus-extensions/nautilus-bonobo-extensions.h>
 #include <libnautilus-extensions/nautilus-file-attributes.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
+#include <libnautilus-extensions/nautilus-gnome-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-search-uri.h>
+#include <libnautilus-extensions/nautilus-stock-dialogs.h>
 #include <libnautilus-extensions/nautilus-string.h>
 
 /* FIXME bugzilla.eazel.com 2815: This code uses part of the
@@ -66,9 +75,6 @@ static void 	fm_search_list_view_initialize_class 	 (gpointer          klass);
 static void     real_destroy                             (GtkObject        *object);
 static void 	real_add_file				 (FMDirectoryView  *view,
 							  NautilusFile 	   *file);
-static void	real_create_selection_context_menu_items (FMDirectoryView  *view,
-					  		  GtkMenu 	   *menu,
-					  		  GList 	   *selection);
 static void	real_adding_file 			 (FMListView 	   *view, 
 							  NautilusFile 	   *file);
 static void	real_removing_file 			 (FMListView 	   *view, 
@@ -82,6 +88,7 @@ static void 	real_get_column_specification        	 (FMListView       *list_view
 						      	  int               column_number,
 						      	  FMListViewColumn *specification);
 static NautilusStringList * real_get_emblem_names_to_exclude         (FMDirectoryView  *view);
+static void	real_file_limit_reached 		 (FMDirectoryView  *view);
 static void	real_merge_menus 		     	 (FMDirectoryView  *view);
 static gboolean real_supports_creating_files		 (FMDirectoryView  *view);
 static gboolean real_accepts_dragged_files		 (FMDirectoryView  *view);
@@ -89,8 +96,12 @@ static gboolean real_supports_properties 	     	 (FMDirectoryView  *view);
 static void 	load_location_callback               	 (NautilusView 	   *nautilus_view, 
 						      	  char 		   *location);
 static void	real_update_menus 		     	 (FMDirectoryView  *view);
-static void	reveal_selected_items_callback 		 (gpointer 	    ignored, 
-							  gpointer 	    user_data);
+#ifdef HAVE_MEDUSA
+static void     display_indexed_search_problems_dialog   (gboolean unindexed_search_is_available);
+#endif
+static void	reveal_selected_items_callback 		 (BonoboUIComponent *component, 
+							  gpointer 	    user_data, 
+							  const char 	   *verb);
 
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (FMSearchListView,
@@ -100,16 +111,223 @@ NAUTILUS_DEFINE_CLASS_BOILERPLATE (FMSearchListView,
 static void
 load_location_callback (NautilusView *nautilus_view, char *location)
 {
-	char *human_string;
+#ifdef HAVE_MEDUSA
+	char *last_indexing_time, *status_string;
+	char *unescaped_location;
+	gboolean unindexed_search_is_available_for_uri;
+	gboolean indexed_search_is_available;
+
+	indexed_search_is_available = medusa_indexed_search_is_available () == GNOME_VFS_OK;
+	if (indexed_search_is_available) {
+		last_indexing_time = nautilus_indexing_info_get_last_index_time ();
+		if (last_indexing_time) {
+			status_string = g_strdup_printf (_("Search results may not include items modified after %s, when your drive was last indexed."),
+							 last_indexing_time);
+			
+			g_free (last_indexing_time);
+			nautilus_view_report_status (nautilus_view, status_string);
+			g_free (status_string);
+		}
+	}
+	if (!indexed_search_is_available) {
+		unescaped_location = gnome_vfs_unescape_string (location, NULL);
+		unindexed_search_is_available_for_uri = (medusa_unindexed_search_is_available_for_uri (unescaped_location) == GNOME_VFS_OK);
+		g_free (unescaped_location);
+		display_indexed_search_problems_dialog (unindexed_search_is_available_for_uri);
+	}
+#else
+	nautilus_error_dialog (_("Sorry, but the Medusa search service is not available because it is not installed."),
+			       _("Search Service Not Available"),
+			       NULL);
+#endif
+
+	nautilus_view_set_title (nautilus_view, _("Search Results"));
+}
+
+/* FIXME bugzilla.eazel.com 5057: GnomeVFSResults may not be the
+   best way to communicate an error code to
+   a view */
+static void
+load_error_callback (FMDirectoryView *nautilus_view, 
+		     GnomeVFSResult result,
+		     gpointer callback_data)
+{
+	GnomeDialog *load_error_dialog;
+	char *error_string;
 	
-	human_string = nautilus_search_uri_to_human (location);
+	switch (result) {
+	case GNOME_VFS_ERROR_SERVICE_OBSOLETE:
+		/* FIXME bugzilla.eazel.com 5058: Should be two messages, one for each of whether
+		   "slow complete search" turned on or not */
+		load_error_dialog = nautilus_yes_no_dialog (_("The search you have selected "
+							      "is newer than the index on your "
+							      "system.  The search will return "
+							      "no results right now.  Would you like "
+							      "to create a new index now?"),
+							    _("Search for items that are too new"),
+							    _("Create a new index"),
+							    _("Don't create index"),
+							    NULL);
+		gtk_signal_connect (GTK_OBJECT (nautilus_gnome_dialog_get_button_by_index
+						(load_error_dialog, GNOME_OK)),
+				    "clicked",
+				    nautilus_indexing_info_request_reindex,
+				    NULL);
+		break;
+	case GNOME_VFS_ERROR_TOO_BIG:
+		nautilus_error_dialog (_("Every indexed file on your computer "
+					 "matches the criteria you selected. "
+					 "You can check the spelling on your selections "
+					 "or add more criteria to narrow your results."),
+				       _("Error during search"),
+				       NULL);
+		break;
+	case GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE:
+		/* We've handled this case in load_location_callback */
+		break;
+	default:
+		error_string = g_strdup_printf (_("An error occurred while loading "
+							  "this search's contents: "
+							  "%s"),
+							gnome_vfs_result_to_string (result));
+		nautilus_error_dialog (error_string,
+				       _("Error during search"),
+				       NULL);
+		g_free (error_string);
+		
+	}
+}
 
-	nautilus_view_set_title (nautilus_view, human_string);
 
-	g_free (human_string);
+#ifdef HAVE_MEDUSA
+static void
+display_indexed_search_problems_dialog (gboolean backup_search_is_available)
+{
+
+	GnomeDialog *index_problem_dialog;
+	const char *error_string, *title_string;
+	MedusaIndexingRequestResult index_service_availability;
+	GtkButton *create_index_button;
+
+	if (medusa_indexed_search_system_index_files_look_available ()) {
+		/* There is an index on the system, but there is no
+		   way to run a search anyways.  The system is
+		   confused.  Tell the user this */
+		error_string = backup_search_is_available 
+			? N_("To do a fast search, Find requires an index "
+			     "of the files on your system.  "
+			     "Find can't access your index right now "
+			     "so a slower search will be performed that "
+			     "doesn't use the index.") 
+			: N_("To do a content search, Find requires an index "
+			     "of thie files on your system.  "
+			     "Find can't access your index right now. ");
+		title_string = backup_search_is_available 
+			?  N_("Fast searches are not available")
+			:  N_("Content searches are not available");
+		nautilus_error_dialog_with_details (error_string,
+						    title_string,
+						    _("Your index files are available "
+						      "but the Medusa search daemon, which handles "
+						      "index requests, isn't running.  "
+						      "To start this program, log in as root and "
+						      "enter this command at the command line: "
+						      "medusa-searchd"),
+						    NULL);
+	}
+	else {
+		/* There is currently no index available, so we need to explain
+		   whether a new index needs to be made, and if nautilus can direct it
+		   to be made correctly. 
+		   There are three possibilities here:
+		   1.  There is no index on this system,  and one is not currently being made.
+		   2.  There is no index on this system, but one is currently being made.
+		   3.  We can't contact the indexing service. */
+		index_service_availability = medusa_index_service_is_available ();
+		switch (index_service_availability) {
+		case MEDUSA_INDEXER_READY_TO_INDEX:
+			error_string = backup_search_is_available
+				? N_("To do a fast search, Find requires an index "
+				     "of the files on your system.  "
+				     "Your computer does not have an index "
+				     "right now.  Because Find cannot use an "
+				     "index, this search may take several "
+				     "minutes.  "
+				     "Would you like to create an index? "
+				     "Creating an index will be done while "
+				     "you are not actively using your computer.")
+				: N_("To do a content search, Find requires an index "
+				     "of the content on your system.  "
+				     "Your computer does not have an index "
+				     "right now.  Would you like to create an index?  "
+				     "Creating an index will be done while you are not "
+				     "actively using your computer.");
+			title_string = backup_search_is_available
+				? N_("Indexed searches are not available")
+				: N_("Content searches are not available");
+			index_problem_dialog = nautilus_yes_no_dialog (error_string,
+								       title_string,
+								       _("Create an Index"),
+								       _("Don't Create an Index Now"),
+								       NULL);
+			create_index_button = nautilus_gnome_dialog_get_button_by_index (index_problem_dialog,
+											  GNOME_OK);
+			gtk_signal_connect (GTK_OBJECT (create_index_button),
+					    "clicked",
+					    nautilus_indexing_info_request_reindex,
+					    NULL);
+			break;
+		case MEDUSA_INDEXER_ALREADY_INDEXING:
+			error_string = backup_search_is_available 
+				? N_("To do a fast search, Find requires an index "
+				    "of the files on your system.  "
+				    "Your computer is currently creating that "
+				     "index.   Because Find cannot use an index, "
+				     "this search may take several "
+				     "minutes.")
+				: N_("To do a content search, Find requires an index "
+				     "of the content on your system.  "
+				     "Your computer is currently creating that "
+				     "index.  Content searches will be available "
+				     "when the index is complete.");
+			title_string = backup_search_is_available
+				? N_("Indexed searches are not available")
+				: N_("Content searches are not available");
+			nautilus_error_dialog (error_string,
+					       title_string,
+					       NULL);
+			break;
+		default:
+			error_string = backup_search_is_available
+				? N_("To do a fast search, Find requires an index "
+				     "of the files on your system.  "
+				     "An index can't be created "
+				     "right now. When an index is not "
+				     "available, searches will "
+				     "take several minutes.")
+				: N_("To do a content search, Find requires an index "
+				     "of the content on your system.  "
+				     "An index can't be created "
+				     "right now.");
+			title_string = backup_search_is_available
+				? N_("Indexed searches are not available")
+				: N_("Content searches are not available");
+			nautilus_error_dialog_with_details (error_string,
+							    title_string,
+							    _("The program that creates an index "
+							      "is not set up correctly.  You can "
+							      "create an index by hand by "
+							      "running \"medusa-indexd\" as root "
+							      "on the command line."),
+							    NULL);
+			break;
+		}
+	}
 
 }
 
+#endif	
+
 static void
 fm_search_list_view_initialize_class (gpointer klass)
 {
@@ -124,10 +342,9 @@ fm_search_list_view_initialize_class (gpointer klass)
 	object_class->destroy = real_destroy;
 
   	fm_directory_view_class->add_file = real_add_file;
-	fm_directory_view_class->create_selection_context_menu_items = 
-		real_create_selection_context_menu_items;
 	fm_directory_view_class->get_emblem_names_to_exclude = 
 		real_get_emblem_names_to_exclude;
+	fm_directory_view_class->file_limit_reached = real_file_limit_reached;
   	fm_directory_view_class->merge_menus = real_merge_menus;
 	fm_directory_view_class->supports_creating_files = 
 		real_supports_creating_files;
@@ -163,11 +380,14 @@ fm_search_list_view_initialize (gpointer object,
 
 	nautilus_view = fm_directory_view_get_nautilus_view (directory_view);
 
-	gtk_signal_connect (GTK_OBJECT(nautilus_view),
+	gtk_signal_connect (GTK_OBJECT (nautilus_view),
 			    "load_location",
 			    GTK_SIGNAL_FUNC (load_location_callback),
 			    NULL);
-
+	gtk_signal_connect (GTK_OBJECT (directory_view),
+			    "load_error",
+			    GTK_SIGNAL_FUNC (load_error_callback),
+			    NULL);
 }
 
 static void
@@ -275,7 +495,7 @@ real_get_column_specification (FMListView *view,
 		break;
 	case 3:
 		fm_list_view_column_set (specification,
-					 "parent_uri", _("Where"),
+					 "where", _("Where"),
 					 NAUTILUS_FILE_SORT_BY_DIRECTORY,
 					 30, 120, 500, FALSE);
 		break;
@@ -310,15 +530,14 @@ indexing_info_callback (BonoboUIComponent *component, gpointer data, const char
 
 static void
 compute_reveal_item_name_and_sensitivity (GList *selected_files,
-					  gboolean include_accelerator_underbars,
-					  char **return_name,
+					  char **return_name_with_underscore,
+					  char **return_name_no_underscore,
 					  gboolean *return_sensitivity)
 {
-	char *name;
-	char *stripped;
+	char *name_with_underscore;
 	int count;
 
-	g_assert (*return_name != NULL);
+	g_assert (return_name_with_underscore != NULL || return_name_no_underscore != NULL);
 	g_assert (return_sensitivity != NULL);
 
 	count = g_list_length (selected_files);
@@ -328,7 +547,7 @@ compute_reveal_item_name_and_sensitivity (GList *selected_files,
 		 * and scroll as necessary to make that item visible (this comment
 		 * is to inform translators of this tricky concept).
 		 */
-		name = g_strdup (_("_Reveal in New Window"));
+		name_with_underscore = g_strdup (_("_Reveal in New Window"));
 	} else {
 		/* "Reveal in n New Windows" means open the parent folder for each
 		 * selected item in a separate new window, select each selected
@@ -336,18 +555,20 @@ compute_reveal_item_name_and_sensitivity (GList *selected_files,
 		 * items visible (this comment is to inform translators of this 
 		 * tricky concept).
 		 */
-		name = g_strdup_printf (_("Reveal in %d _New Windows"), count);
+		name_with_underscore = g_strdup_printf (_("Reveal in %d _New Windows"), count);
 	}
 
 	*return_sensitivity = selected_files != NULL;
 	
-	if (!include_accelerator_underbars) {
-		stripped = nautilus_str_strip_chr (name, '_');
-		g_free (name);
-		name = stripped;
-	}
+        if (return_name_no_underscore != NULL) {
+        	*return_name_no_underscore = nautilus_str_strip_chr (name_with_underscore, '_');
+        }
 
-	*return_name = name;
+        if (return_name_with_underscore != NULL) {
+		*return_name_with_underscore = name_with_underscore;
+        } else {
+		g_free (name_with_underscore);
+        }
 }
 
 static void
@@ -364,53 +585,25 @@ real_add_file (FMDirectoryView *view, NautilusFile *file)
 	 * search results virtual file.
 	 */
 	fake_file_name = nautilus_file_get_name (file);
-	real_file_uri = gnome_vfs_unescape_string (fake_file_name, NULL);
+	real_file_uri = nautilus_get_target_uri_from_search_result_name (fake_file_name);
 	real_file = nautilus_file_get (real_file_uri);
 
-	/* Tell the normal list-view code to add this file. It will add
-	 * and ref it only if it's not already in the list.
-	 */ 
-	NAUTILUS_CALL_PARENT_CLASS 
-		(FM_DIRECTORY_VIEW_CLASS, add_file, (view, real_file));
+	/* We don't benefit from the filtering done in queue_pending_files,
+	 * since that's done on the fake file, so we do our own filtering here.
+	 */
+	if (fm_directory_view_should_show_file (view, real_file)) {
+		/* Tell the normal list-view code to add this file. It will add
+		 * and ref it only if it's not already in the list.
+		 */ 
+		NAUTILUS_CALL_PARENT_CLASS 
+			(FM_DIRECTORY_VIEW_CLASS, add_file, (view, real_file));
+	}
 
 	g_free (fake_file_name);
 	g_free (real_file_uri);
 	nautilus_file_unref (real_file);
 }
 
-static void
-real_create_selection_context_menu_items (FMDirectoryView *view,
-					  GtkMenu *menu,
-					  GList *selection)
-{
-	GtkWidget *menu_item;
-	char *name;
-	gboolean sensitive;
-	gint position;
-	
-	g_assert (FM_IS_SEARCH_LIST_VIEW (view));
-	g_assert (GTK_IS_MENU (menu));
-
-	NAUTILUS_CALL_PARENT_CLASS
-		(FM_DIRECTORY_VIEW_CLASS, 
-		 create_selection_context_menu_items,
-		 (view, menu, selection));
-
-	/* The Reveal item is inserted directly after the
-	 * existing Open With item.
-	 */
-	position = fm_directory_view_get_context_menu_index
-		(menu, FM_DIRECTORY_VIEW_COMMAND_OPEN_WITH) + 1;
-	compute_reveal_item_name_and_sensitivity (selection, FALSE, &name, &sensitive);
-        menu_item = gtk_menu_item_new_with_label (name);
-        g_free (name);
-        gtk_widget_set_sensitive (menu_item, sensitive);
-	gtk_widget_show (menu_item);
-
-	gtk_signal_connect (GTK_OBJECT (menu_item), "activate", reveal_selected_items_callback, view);
-	gtk_menu_insert (menu, menu_item, position);
-}
-
 static void
 real_adding_file (FMListView *view, NautilusFile *file)
 {
@@ -421,7 +614,7 @@ real_adding_file (FMListView *view, NautilusFile *file)
 
 	NAUTILUS_CALL_PARENT_CLASS (FM_LIST_VIEW_CLASS, adding_file, (view, file));
 
-	/* FIXME: this implies that positioning, custom icon, icon
+	/* FIXME bugzilla.eazel.com 5059: this implies that positioning, custom icon, icon
 	 * stretching, etc, will be based on the real directory the file is in,
 	 * and won't be specific to the search directory. Is that OK? 
 	 */
@@ -465,11 +658,26 @@ real_file_still_belongs (FMListView *view, NautilusFile *file)
 	return !nautilus_file_is_gone (file);
 }
 
+static void
+real_file_limit_reached (FMDirectoryView *view)
+{
+	g_assert (FM_IS_SEARCH_LIST_VIEW (view));
+	
+	/* Note that the number of items actually displayed varies somewhat due
+	 * to the way files are collected in batches. So you can't assume that
+	 * no more than the constant limit are displayed.
+	 */
+	nautilus_warning_dialog (_("Nautilus found more search results than it can display. "
+				   "Some matching items will not be displayed. "), 
+				 _("Too Many Matches"),
+				 fm_directory_view_get_containing_window (view));
+}
+
 static void
 update_reveal_item (FMSearchListView *view)
 {
 	GList *selected_files;
-	char *name;
+	char *label_with_underscore, *label_no_underscore;
 	gboolean sensitive;
 
 	g_assert (FM_IS_DIRECTORY_VIEW (view));
@@ -477,15 +685,16 @@ update_reveal_item (FMSearchListView *view)
 	selected_files = fm_directory_view_get_selection (FM_DIRECTORY_VIEW (view));
 
 	compute_reveal_item_name_and_sensitivity 
-		(selected_files, TRUE, &name, &sensitive);
+		(selected_files, &label_with_underscore, &label_no_underscore, &sensitive);
 
 	nautilus_bonobo_set_sensitive (view->details->ui, 
 				       COMMAND_REVEAL_IN_NEW_WINDOW, 
 				       sensitive);
-					       
-	nautilus_bonobo_set_label (view->details->ui, MENU_PATH_REVEAL_IN_NEW_WINDOW, name);
+	nautilus_bonobo_set_label (view->details->ui, COMMAND_REVEAL_IN_NEW_WINDOW, label_no_underscore);					       
+	nautilus_bonobo_set_label (view->details->ui, MENU_PATH_REVEAL_IN_NEW_WINDOW, label_with_underscore);
 
-	g_free (name);
+	g_free (label_with_underscore);
+	g_free (label_no_underscore);
 
         nautilus_file_list_free (selected_files);
 }
@@ -496,7 +705,7 @@ real_merge_menus (FMDirectoryView *view)
 	FMSearchListView *search_view;
 	BonoboUIVerb verbs [] = {
 		BONOBO_UI_VERB ("Indexing Info", indexing_info_callback),
-		BONOBO_UI_VERB ("Reveal", (BonoboUIVerbFn)reveal_selected_items_callback),
+		BONOBO_UI_VERB ("Reveal", reveal_selected_items_callback),
 		BONOBO_UI_VERB_END
 	};
 
@@ -554,12 +763,8 @@ real_update_menus (FMDirectoryView *view)
 	update_reveal_item (FM_SEARCH_LIST_VIEW (view));
 }
 
-/**
- * Note that this is used both as a Bonobo menu callback and a signal callback.
- * The first parameter is different in these cases, but we just ignore it anyway.
- */
 static void
-reveal_selected_items_callback (gpointer ignored, gpointer user_data)
+reveal_selected_items_callback (BonoboUIComponent *component, gpointer user_data, const char *verb)
 {
 	FMDirectoryView *directory_view;
 	char *parent_uri;
@@ -580,7 +785,7 @@ reveal_selected_items_callback (gpointer ignored, gpointer user_data)
 			parent_uri = nautilus_file_get_parent_uri (file);
 			if (parent_uri != NULL) {
 				file_as_list = g_list_prepend (NULL, nautilus_file_get_uri (file));
-				nautilus_view_open_location_in_new_window
+				nautilus_view_open_location_force_new_window
 					(fm_directory_view_get_nautilus_view (directory_view), 
 					 parent_uri,
 					 file_as_list);
