@@ -36,6 +36,7 @@
 #include "nautilus-main.h"
 #include "nautilus-shell-interface.h"
 #include "nautilus-shell.h"
+#include "nautilus-window-private.h"
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
 #include <gtk/gtksignal.h>
@@ -43,6 +44,7 @@
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-stock.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
@@ -51,6 +53,7 @@
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
 #include <libnautilus-extensions/nautilus-string-list.h>
 #include <libnautilus-extensions/nautilus-undo-manager.h>
+#include <libnautilus-extensions/nautilus-volume-monitor.h>
 #include <liboaf/liboaf.h>
 
 #define FACTORY_IID	"OAFIID:nautilus_factory:bd1e1862-92d7-4391-963e-37583f0daef3"
@@ -69,10 +72,16 @@ static CORBA_Object  create_object                               (PortableServer
 static void          nautilus_application_initialize             (NautilusApplication      *application);
 static void          nautilus_application_initialize_class       (NautilusApplicationClass *klass);
 static void          nautilus_application_destroy                (GtkObject                *object);
-static void          nautilus_application_check_user_directories (NautilusApplication      *application);
 static gboolean	     check_for_and_run_as_super_user 		 (void);
 static gboolean	     need_to_show_first_time_druid		 (void);
 static void	     desktop_changed_callback 		         (gpointer 		   user_data);
+static void	     volume_mounted_callback 			 (NautilusVolumeMonitor    *monitor,
+								  NautilusVolume 	   *volume,
+			  	 				  NautilusApplication 	   *application);
+static void	     volume_unmounted_callback 			 (NautilusVolumeMonitor    *monitor,
+								  NautilusVolume 	   *volume,
+			  	 				  NautilusApplication 	   *application);
+
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusApplication, nautilus_application, BONOBO_OBJECT_TYPE)
 
@@ -179,10 +188,17 @@ nautilus_application_initialize (NautilusApplication *application)
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
 
-	/* monitor the desktop preference */
-	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
-					   desktop_changed_callback,
-					   application);	
+	/* Watch for volume mounts so we can restore open windows */
+	gtk_signal_connect (GTK_OBJECT (nautilus_volume_monitor_get ()),
+			    "volume_mounted",
+			    volume_mounted_callback,
+			    application);
+
+	/* Watch for volume unmounts so we can close open windows */
+	gtk_signal_connect (GTK_OBJECT (nautilus_volume_monitor_get ()),
+			    "volume_unmounted",
+			    volume_unmounted_callback,
+			    application);
 }
 
 NautilusApplication *
@@ -200,22 +216,23 @@ nautilus_application_destroy (GtkObject *object)
 
 	nautilus_bookmarks_exiting ();
 	
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
-					      desktop_changed_callback,
-					      application);
-
 	bonobo_object_unref (BONOBO_OBJECT (application->undo_manager));
 
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
 
-static void
-nautilus_application_check_user_directories (NautilusApplication *application)
+static gboolean
+check_required_directories (NautilusApplication *application)
 {
 	char			*user_directory;
 	char			*user_main_directory;
 	char			*desktop_directory;
 	NautilusStringList	*dir_list;
+	char 			*dir_list_concatenated;
+	char 			*error_string;
+	char 			*dialog_title;
+	GnomeDialog 		*dialog;
+	int			 failed_count;
 	
 	g_assert (NAUTILUS_IS_APPLICATION (application));
 
@@ -225,44 +242,51 @@ nautilus_application_check_user_directories (NautilusApplication *application)
 
 	dir_list = nautilus_string_list_new (TRUE);
 	
-	/* FIXME bugzilla.eazel.com 1115: Need better name for "User Directory"
-	 * and "User Data Directory". Perhaps it's OK to use the word "directory"
-	 * instead of "folder" in this technical error case.
-	 */
-
 	if (!g_file_test (user_directory, G_FILE_TEST_ISDIR)) {
-		nautilus_string_list_insert (dir_list, _("User Directory"));
+		nautilus_string_list_insert (dir_list, user_directory);
 	}
 	g_free (user_directory);
 	    
 	if (!g_file_test (user_main_directory, G_FILE_TEST_ISDIR)) {
-		nautilus_string_list_insert (dir_list, _("User Main Directory"));
+		nautilus_string_list_insert (dir_list, user_main_directory);
 	}
 	g_free (user_main_directory);
 	    
 	if (!g_file_test (desktop_directory, G_FILE_TEST_ISDIR)) {
-		nautilus_string_list_insert (dir_list, _("Desktop Directory"));
+		nautilus_string_list_insert (dir_list, desktop_directory);
 	}
 	g_free (desktop_directory);
 
-	if (nautilus_string_list_get_length (dir_list) > 0) {
-		char *dir_list_concatenated;
-		char *error_string;
+	failed_count = nautilus_string_list_get_length (dir_list);
 
+	if (failed_count != 0) {
 		dir_list_concatenated = nautilus_string_list_as_concatenated_string (dir_list, "\n");
-		
-		error_string = g_strdup_printf ("%s\n\n%s\n\n%s",
-						"The following directories are missing:",
-						dir_list_concatenated,
-						"Please restart Nautilus to fix this problem.");
 
-		nautilus_error_dialog (error_string, _("Missing Directories"), NULL);
+		if (failed_count == 1) {
+			dialog_title = g_strdup (_("Couldn't Create Required Folder"));
+			error_string = g_strdup_printf ("Nautilus could not create the required folder \"%s\". "
+							"Before running Nautilus, please create this folder, or "
+							"set permissions such that Nautilus can create it.", dir_list_concatenated);
+		} else {
+			dialog_title = g_strdup (_("Couldn't Create Required Folders"));
+			error_string = g_strdup_printf ("Nautilus could not create the following required folders:\n\n"
+							"%s\n\n"
+							"Before running Nautilus, please create these folders, or "
+							"set permissions such that Nautilus can create them.", dir_list_concatenated);
+		}
+		
+		dialog = nautilus_error_dialog (error_string, dialog_title, NULL);
+		/* We need the main event loop so the user has a chance to see the dialog. */
+		nautilus_main_event_loop_register (GTK_OBJECT (dialog));
 
 		g_free (dir_list_concatenated);
 		g_free (error_string);
+		g_free (dialog_title);
 	}
 
 	nautilus_string_list_free (dir_list);
+
+	return failed_count == 0;
 }
 
 static int
@@ -298,8 +322,10 @@ void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
 			      gboolean restart_shell,
-			      gboolean stop_desktop,
 			      gboolean start_desktop,
+			      gboolean no_default_window,
+			      gboolean do_first_time_druid_check,
+			      const char *geometry,
 			      const char *urls[])
 {
 	CORBA_Environment ev;
@@ -308,6 +334,7 @@ nautilus_application_startup (NautilusApplication *application,
 	const char *message, *detailed_message;
 	GnomeDialog *dialog;
 	Nautilus_URIList *url_list;
+	const CORBA_char *corba_geometry;
 
 	/* Perform check for nautilus being run as super user */
 	if (!check_for_and_run_as_super_user ()) {
@@ -315,7 +342,7 @@ nautilus_application_startup (NautilusApplication *application,
 	}
 
 	/* Run the first time startup druid if needed. */
-	if (need_to_show_first_time_druid ()) {
+	if (do_first_time_druid_check && need_to_show_first_time_druid ()) {
 		nautilus_first_time_druid_show (application, start_desktop, urls);
 		return;
 	}
@@ -323,7 +350,9 @@ nautilus_application_startup (NautilusApplication *application,
 	/* Check the user's ~/.nautilus directories and post warnings
 	 * if there are problems.
 	 */
-	nautilus_application_check_user_directories (application);
+	if (!check_required_directories (application)) {
+		return;
+	}
 
 	/* initialize the sound machinery */
 	nautilus_sound_initialize ();
@@ -346,11 +375,11 @@ nautilus_application_startup (NautilusApplication *application,
 			break;
 		case OAF_REG_NOT_LISTED:
 			/* Can't register myself due to trouble locating the
-			 * nautilus.oafinfo file. This has happened when you
+			 * Nautilus_Shell.oaf file. This has happened when you
 			 * launch Nautilus with an LD_LIBRARY_PATH that
 			 * doesn't include the directory containg the oaf
 			 * library. It could also happen if the
-			 * nautilus.oafinfo file was not present for some
+			 * Nautilus_Shell.oaf file was not present for some
 			 * reason. Sometimes killing oafd and gconfd fixes
 			 * this problem but we don't exactly understand why,
 			 * since neither of the above causes explain it.
@@ -362,11 +391,11 @@ nautilus_application_startup (NautilusApplication *application,
 			detailed_message = _("Nautilus can't be used now. "
 					     "Rebooting the computer or installing "
 					     "Nautilus again may fix the problem.\n\n"
-					     "OAF couldn't locate the nautilus.oafinfo file. "
+					     "OAF couldn't locate the Nautilus_Shell.oaf file. "
 					     "One cause of this seems to be an LD_LIBRARY_PATH "
 					     "that does not include the oaf library's directory. "
 					     "Another possible cause would be bad install "
-					     "with a missing nautilus.oafinfo file.\n\n"
+					     "with a missing Nautilus_Shell.oaf file.\n\n"
 					     "Sometimes killing oafd and gconfd fixes "
 					     "the problem, but we don't know why.\n\n"
 					     "We need a much less confusing message here for Nautilus 1.0.");
@@ -424,23 +453,36 @@ nautilus_application_startup (NautilusApplication *application,
 		if (start_desktop) {
 			Nautilus_Shell_start_desktop (shell, &ev);
 		}
+		
+		/* Monitor the preference to show or hide the desktop */
+		nautilus_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
+							       desktop_changed_callback,
+							       application,
+							       GTK_OBJECT (application));
 
-		if (stop_desktop) {
-			Nautilus_Shell_stop_desktop (shell, &ev);
-		}
+		/* CORBA C mapping doesn't allow NULL to be passed
+		   for string parameters */
+		corba_geometry = (geometry != NULL) ? geometry : "";
 
 	  	/* Create the other windows. */
 		if (urls != NULL) {
 			url_list = nautilus_make_uri_list_from_strv (urls);
-			Nautilus_Shell_open_windows (shell, url_list, &ev);
+			Nautilus_Shell_open_windows (shell, url_list, corba_geometry, &ev);
 			CORBA_free (url_list);
-		} else if (!start_desktop && !stop_desktop) {
-			Nautilus_Shell_open_default_window (shell, &ev);
+		} else if (!no_default_window) {
+			Nautilus_Shell_open_default_window (shell, corba_geometry, &ev);
 		}
 	}
 
 	/* We're done with the shell now, so let it go. */
-	bonobo_object_release_unref (shell, &ev);
+	/* HACK: Don't bother releasing the shell in the case where we
+	 * just told it to quit -- that just leads to hangs and does
+	 * no good. We could probably fix this in some fancier way if
+	 * we could figure out a better lifetime rule.
+	 */
+	if (!kill_shell) {
+		bonobo_object_release_unref (shell, &ev);
+	}
 
  out:
 	CORBA_exception_free (&ev);
@@ -567,7 +609,7 @@ desktop_changed_callback (gpointer user_data)
 	NautilusApplication *application;
 	
 	application = NAUTILUS_APPLICATION (user_data);
-	if ( nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP, FALSE)) {
+	if ( nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
 		nautilus_application_open_desktop (application);
 	} else {
 		nautilus_application_close_desktop ();
@@ -613,3 +655,61 @@ need_to_show_first_time_druid (void)
 
 	return result;
 }
+
+/* Called whenever a volume is mounted.s
+ * It would also be cool to restore open windows and
+ * position info saved when the volume was unmounted.
+ */
+static void
+volume_mounted_callback (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
+			 NautilusApplication *application)
+{
+}
+
+/* Called whenever a volume is unmounted. Check and see if there are any windows open
+ * displaying contents on the volume. If there are, close them.
+ * It would also be cool to save open window and position info.
+ */
+static void
+volume_unmounted_callback (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
+			   NautilusApplication *application)
+{
+	GSList *windows, *index, *close_list;
+	NautilusWindow *window;
+	char *text_uri;
+	const char *path;
+	GnomeVFSURI *uri;
+		
+	close_list = NULL;
+	
+	/* Check and see if any of the open windows are displaying contents from the unmounted volume */
+	windows = nautilus_application_windows ();
+	
+	/* Construct a list of windows to be closed */
+	for (index = windows; index != NULL; index = index->next) {
+		window = (NautilusWindow *)index->data;
+		if (window != NULL) {
+			text_uri = nautilus_file_get_uri (window->details->viewed_file);
+			uri = gnome_vfs_uri_new (text_uri);
+			g_free (text_uri);
+			
+			if (uri != NULL) {
+				path = gnome_vfs_uri_get_path (uri);				
+				if (strlen (path) >= strlen (volume->mount_path)) {
+					if (strncmp (path, volume->mount_path, strlen (volume->mount_path)) == 0) {
+						close_list = g_slist_prepend (close_list, window);
+					}				
+				}
+				gnome_vfs_uri_unref (uri);
+			}
+		}
+	}
+	
+	/* Now close all windows in the close list */
+	for (index = close_list; index != NULL; index = index->next) {
+		nautilus_window_close (NAUTILUS_WINDOW (index->data));
+	}
+	
+	g_slist_free (close_list);
+}
+
