@@ -69,9 +69,6 @@ static void	on_select_row 		      (GtkCList	*,
 	       				       int column,
 	       				       GdkEventButton *,
 	       				       gpointer user_data);
-static gboolean	on_text_field_focus_in_event  (GtkWidget *, 
-					       GdkEventFocus *, 
-					       gpointer user_data);
 static gboolean	on_text_field_focus_out_event (GtkWidget *, 
 					       GdkEventFocus *, 
 					       gpointer user_data);
@@ -79,7 +76,12 @@ static void	on_uri_field_changed 	      (GtkEditable *, gpointer user_data);
 static gboolean on_window_delete_event 	      (GtkWidget *, 
 					       GdkEvent *, 
 					       gpointer user_data);
+static void     on_window_hide_event 	      (GtkWidget *, 
+					       gpointer user_data);
+static void     on_window_destroy_event       (GtkWidget *, 
+					       gpointer user_data);
 static void	repopulate		      (void);
+static void	set_up_close_accelerator      (GtkWidget *window);
 
 #define BOOKMARK_LIST_COLUMN_ICON		0
 #define BOOKMARK_LIST_COLUMN_NAME		1
@@ -95,7 +97,6 @@ static void	repopulate		      (void);
 #define BOOKMARKS_WINDOW_INITIAL_WIDTH	500
 #define BOOKMARKS_WINDOW_INITIAL_HEIGHT	200
 
-
 /**
  * create_bookmarks_window:
  * 
@@ -120,9 +121,11 @@ create_bookmarks_window (NautilusBookmarkList *list, GtkObject *undo_manager_sou
 	bookmarks = list;
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	set_up_close_accelerator (window);
 	nautilus_undo_share_undo_manager (GTK_OBJECT (window), undo_manager_source);
 	gtk_container_set_border_width (GTK_CONTAINER (window), GNOME_PAD);
 	gtk_window_set_title (GTK_WINDOW (window), _("Bookmarks"));
+	gtk_window_set_wmclass (GTK_WINDOW (window), "bookmarks", "Nautilus");
 	gtk_widget_set_usize (window, 
 			      BOOKMARKS_WINDOW_MIN_WIDTH, 
 			      BOOKMARKS_WINDOW_MIN_HEIGHT);
@@ -185,7 +188,7 @@ create_bookmarks_window (NautilusBookmarkList *list, GtkObject *undo_manager_sou
 	gtk_box_pack_start (GTK_BOX (right_side), hbox2, FALSE, FALSE, 0);
 
 	remove_button = gtk_button_new_with_label (_("Remove"));
-	nautilus_gtk_button_set_padding (GTK_BUTTON (remove_button), GNOME_PAD_SMALL);
+	nautilus_gtk_button_set_standard_padding (GTK_BUTTON (remove_button));
 	gtk_widget_show (remove_button);
 	gtk_box_pack_start (GTK_BOX (hbox2), remove_button, TRUE, FALSE, 0);
 
@@ -206,15 +209,19 @@ create_bookmarks_window (NautilusBookmarkList *list, GtkObject *undo_manager_sou
                     	    GTK_SIGNAL_FUNC (on_window_delete_event),
                     	    NULL);
                     	    
+	gtk_signal_connect (GTK_OBJECT (window), "hide",
+                    	    on_window_hide_event,
+                    	    NULL);
+                    	    
+	gtk_signal_connect (GTK_OBJECT (window), "destroy",
+                    	    on_window_destroy_event,
+                    	    NULL);
+                    	    
 	name_field_changed_signalID =
 		gtk_signal_connect (GTK_OBJECT (name_field), "changed",
                 	            GTK_SIGNAL_FUNC (on_name_field_changed),
                       		    NULL);
                       		    
-	gtk_signal_connect (GTK_OBJECT (name_field), "focus_in_event",
-      	              	    GTK_SIGNAL_FUNC (on_text_field_focus_in_event),
-                            NULL);
-                            
 	gtk_signal_connect (GTK_OBJECT (name_field), "focus_out_event",
       	              	    GTK_SIGNAL_FUNC (on_text_field_focus_out_event),
                             NULL);
@@ -228,10 +235,6 @@ create_bookmarks_window (NautilusBookmarkList *list, GtkObject *undo_manager_sou
                 	    	    GTK_SIGNAL_FUNC (on_uri_field_changed),
                       		    NULL);
                       		    
-	gtk_signal_connect (GTK_OBJECT (uri_field), "focus_in_event",
-      	              	    GTK_SIGNAL_FUNC (on_text_field_focus_in_event),
-                            NULL);
-                            
 	gtk_signal_connect (GTK_OBJECT (uri_field), "focus_out_event",
         	            GTK_SIGNAL_FUNC (on_text_field_focus_out_event),
               	    	    NULL);
@@ -489,17 +492,6 @@ update_bookmark_from_text ()
 	}
 }
 
-static gboolean
-on_text_field_focus_in_event (GtkWidget *widget,
-			       GdkEventFocus *event,
-			       gpointer user_data)
-{
-	g_assert (NAUTILUS_IS_ENTRY (widget));
-
-	nautilus_entry_select_all (NAUTILUS_ENTRY (widget));
-	return FALSE;
-}
-
 static gboolean
 on_text_field_focus_out_event (GtkWidget *widget,
 			       GdkEventFocus *event,
@@ -508,7 +500,6 @@ on_text_field_focus_out_event (GtkWidget *widget,
 	g_assert (NAUTILUS_IS_ENTRY (widget));
 
 	update_bookmark_from_text ();
-	gtk_editable_select_region (GTK_EDITABLE (widget), -1, -1);
 	return FALSE;
 }
 
@@ -531,29 +522,56 @@ on_uri_field_changed (GtkEditable *editable,
 	text_changed = TRUE;
 }
 
+static void
+save_geometry_and_hide (GtkWindow *window)
+{
+	g_assert (GTK_IS_WINDOW (window));
+
+	nautilus_bookmarks_window_save_geometry (window);
+	gtk_widget_hide (GTK_WIDGET (window));
+}
+
 
 static gboolean
 on_window_delete_event (GtkWidget *widget,
 			GdkEvent *event,
 			gpointer user_data)
+{
+	save_geometry_and_hide (GTK_WINDOW (widget));
+	return TRUE;
+}
+
+static gboolean
+restore_geometry (gpointer data)
+{
+	g_assert (GTK_IS_WINDOW (data));
+
+	nautilus_bookmarks_window_restore_geometry (GTK_WIDGET (data));
+
+	/* Don't call this again */
+	return FALSE;
+}
+
+static void
+on_window_hide_event (GtkWidget *widget,
+		      gpointer user_data)
 {
 	nautilus_bookmarks_window_save_geometry (GTK_WINDOW (widget));
-	
-	/* Hide but don't destroy */
-	gtk_widget_hide (widget);
 
 	/* Disable undo for entry widgets */
 	nautilus_undo_unregister (GTK_OBJECT (name_field));
 	nautilus_undo_unregister (GTK_OBJECT (uri_field));
 
-	/* Seems odd to restore the geometry just after saving it,
-	 * and when the window is hidden, but this insures that
-	 * the next time the window is shown it will have the
-	 * right hints in it to appear in the correct place.
-	 */
-	nautilus_bookmarks_window_restore_geometry (widget);
+	/* restore_geometry only works after window is hidden */
+	gtk_idle_add (restore_geometry, widget);
+}
 
-	return TRUE;
+static void
+on_window_destroy_event (GtkWidget *widget,
+		      	 gpointer user_data)
+{
+	g_message ("destroying bookmarks window");
+	gtk_idle_remove_by_data (widget);
 }
 
 static void
@@ -614,3 +632,35 @@ repopulate (void)
 	  
 	gtk_clist_thaw (GTK_CLIST (bookmark_list_widget));
 }
+
+static int
+handle_close_accelerator (GtkWindow *window, 
+			  GdkEventKey *event, 
+			  gpointer user_data)
+{
+	g_assert (GTK_IS_WINDOW (window));
+	g_assert (event != NULL);
+	g_assert (user_data == NULL);
+
+	if (nautilus_gtk_window_event_is_close_accelerator (window, event)) {		
+		save_geometry_and_hide (window);
+		gtk_signal_emit_stop_by_name 
+			(GTK_OBJECT (window), "key_press_event");
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void
+set_up_close_accelerator (GtkWidget *window)
+{
+	/* Note that we don't call nautilus_gtk_window_set_up_close_accelerator
+	 * here because we have to handle saving geometry before hiding the
+	 * window.
+	 */
+	gtk_signal_connect (GTK_OBJECT (window),
+			    "key_press_event",
+			    GTK_SIGNAL_FUNC (handle_close_accelerator),
+			    NULL);
+}
