@@ -113,9 +113,6 @@ nautilus_desktop_window_new (NautilusApplication *application)
 				 "app_id", "nautilus",
 				 NULL));
 
-	nautilus_window_disable_keyboard_navigation_for_menus (NAUTILUS_WINDOW (window));
-
-
 	/* Special sawmill setting*/
 	gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
 
@@ -126,8 +123,7 @@ nautilus_desktop_window_new (NautilusApplication *application)
 	/* Point window at the desktop folder.
 	 * Note that nautilus_desktop_window_initialize is too early to do this.
 	 */
-	desktop_directory_uri = gnome_vfs_get_uri_from_local_path
-		(desktop_directory_path);
+	desktop_directory_uri = gnome_vfs_get_uri_from_local_path (desktop_directory_path);
 	g_free (desktop_directory_path);
 	nautilus_window_goto_uri (NAUTILUS_WINDOW (window), desktop_directory_uri);
 	g_free (desktop_directory_uri);
@@ -156,36 +152,9 @@ static void
 realize (GtkWidget *widget)
 {
 	NautilusDesktopWindow *window;
-#ifdef UIH
-	GtkContainer *dock_as_container;
-	GList *children, *p;
-	GtkWidget *child;
-#endif
 
 	window = NAUTILUS_DESKTOP_WINDOW (widget);
 
-	/* Hide unused pieces of the BonoboWin.
-	 * We don't want a menu bar, toolbars, or status bar on the desktop.
-	 * But we don't want to hide the client area!
-	 */
-#ifdef UIH
-	gtk_widget_hide (GNOME_APP (window)->menubar);
-	gtk_widget_hide (GNOME_APP (window)->statusbar);
-	dock_as_container = GTK_CONTAINER (GNOME_APP (window)->dock);
-	children = gtk_container_children (dock_as_container);
-	for (p = children; p != NULL; p = p->next) {
-		child = p->data;
-		
-		if (child != gnome_dock_get_client_area (GNOME_DOCK (dock_as_container))) {
-			gtk_widget_ref (child);
-			window->details->unref_list = g_list_prepend
-				(window->details->unref_list, child);
-			gtk_container_remove (dock_as_container, child);
-		}
-	}
-	g_list_free (children);
-#endif
-
 	/* Make sure we get keyboard events */
 	gtk_widget_set_events (widget, gtk_widget_get_events (widget) 
 			      | GDK_KEY_PRESS_MASK | GDK_KEY_PRESS_MASK);
