@@ -1,4 +1,4 @@
- /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
 /*
  * Nautilus
@@ -24,36 +24,41 @@
  */
 
 #include <config.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <string.h>
+#include "nautilus-first-time-druid.h"
+
 #include <ctype.h>
-#include <netdb.h>
-#include <sys/types.h>
-#include <sys/wait.h>
-
-#include <gnome.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gnome.h>
 #include <libgnomevfs/gnome-vfs.h>
-
-#include <nautilus-main.h>
 #include <libnautilus-extensions/nautilus-background.h>
-#include <libnautilus-extensions/nautilus-druid.h>
 #include <libnautilus-extensions/nautilus-druid-page-eazel.h>
+#include <libnautilus-extensions/nautilus-druid.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
-#include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gdk-extensions.h>
-#include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
+#include <libnautilus-extensions/nautilus-glib-extensions.h>
+#include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-image.h>
 #include <libnautilus-extensions/nautilus-label.h>
-#include <libnautilus-extensions/nautilus-radio-button-group.h>
-#include <libnautilus-extensions/nautilus-user-level-manager.h>
+#include <libnautilus-extensions/nautilus-link.h>
 #include <libnautilus-extensions/nautilus-preferences.h>
+#include <libnautilus-extensions/nautilus-radio-button-group.h>
 #include <libnautilus-extensions/nautilus-string.h>
+#include <libnautilus-extensions/nautilus-global-preferences.h>
+#include <libnautilus-extensions/nautilus-stock-dialogs.h>
+#include <nautilus-main.h>
+#include <netdb.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <unistd.h>
 
-#include "nautilus-first-time-druid.h"
+#define SIZE_BODY_LABEL(l)  nautilus_label_make_smaller (NAUTILUS_LABEL (l), 1)
+
+#define SERVICE_UPDATE_ARCHIVE_PATH "/tmp/nautilus_update.tgz"
+#define WELCOME_PACKAGE_URI "http://services.eazel.com/downloads/eazel/updates.tgz"
 
 #define NUMBER_OF_STANDARD_PAGES 5
 
@@ -64,17 +69,27 @@
 #define PROXY_CONFIGURATION_PAGE 4
 
 /* Preference for http proxy settings */
-#define GNOME_VFS_PREFERENCES_HTTP_PROXY "/system/gnome-vfs/http-proxy"
+#define DEFAULT_HTTP_PROXY_PORT 8080
+#define DEFAULT_HTTP_PROXY_PORT_STRING "8080"
+#define GNOME_VFS_PREFERENCES_HTTP_PROXY_HOST "/system/gnome-vfs/http-proxy-host"
+#define GNOME_VFS_PREFERENCES_HTTP_PROXY_PORT "/system/gnome-vfs/http-proxy-port"
+#define GNOME_VFS_PREFERENCES_USE_HTTP_PROXY "/system/gnome-vfs/use-http-proxy"
+
+#define READ_FILE_HANDLE_TAG "Nautilus first time druid read file handle"
 
 /* The number of seconds we'll wait for our experimental DNS resolution */
 #define NETWORK_CHECK_TIMEOUT_SECONDS 15
 #define DEFAULT_DNS_CHECK_HOST "services.eazel.com"
 
+#define GALEON_PREFS_PATH "/.gnome/galeon"
 
-static void     initiate_file_download           (GnomeDruid *druid);
-static gboolean set_http_proxy                   (const char *proxy_url);
-static gboolean attempt_http_proxy_autoconfigure (void);
-static gboolean check_network_connectivity	 (void);
+#define GETLINE_INITIAL_BUFFER_SIZE 256
+
+#define NETSCAPE_PREFS_PATH "/.netscape/preferences.js"
+
+#define EAZEL_SERVICES_LEARN_MORE_URL "http://services.eazel.com/services"
+#define EAZEL_SERVICES_SIGN_UP_URL "http://services.eazel.com/account/register/form"
+#define EAZEL_SERVICES_LOG_IN_URL "eazel:"
 
 
 /* globals */
@@ -94,12 +109,11 @@ static int last_proxy_choice = 1;
 static GtkWidget *port_number_entry;
 static GtkWidget *proxy_address_entry;
 
-/* FIXME -- for the download callback if it returns after the druid has exited */
-static gboolean has_druid_exited = FALSE;
-
 /* Set by set_http_proxy; used by check_network_connectivity */
+
 /* NULL indicates no HTTP proxy */
 static char *http_proxy_host = NULL;
+
 /* The result of the last check_network_connectivity call */
 static enum {
 	Untested,
@@ -107,11 +121,26 @@ static enum {
 	Fail
 } network_status = Untested;
 
+/* Globals used to implement DNS timeout. */
+static gboolean sigalrm_occurred;
+static pid_t child_pid;
+
+
+static void     initiate_file_download           (GnomeDruid *druid);
+static gboolean set_http_proxy                   (const char *proxy_url);
+static gboolean attempt_http_proxy_autoconfigure (void);
+static gboolean check_network_connectivity	 (void);
+
 
 static void
 druid_cancel (GtkWidget *druid)
 {
-	gtk_widget_destroy(gtk_widget_get_toplevel(druid));
+	gtk_widget_destroy (gtk_widget_get_toplevel (druid));
+
+	/* FIXME bugzilla.eazel.com 5050: Why _exit instead of a plain exit? It might be OK
+	 * to do nothing here now that Nautilus knows to quit when
+	 * windows go away.
+	 */
 	_exit (0);
 }
 
@@ -147,75 +176,75 @@ druid_set_first_time_file_flag (void)
 static void
 druid_finished (GtkWidget *druid_page)
 {
-	char *user_main_directory;
+	char *user_main_directory, *desktop_path;
 	const char *signup_uris[2];
 	
+	user_main_directory = nautilus_get_user_main_directory ();
 
-	user_main_directory = nautilus_get_user_main_directory();
-
-	/* FIXME bugzilla.eazel.com 1555:
-	 * Need to post a error dialog if the user's main directory was not created
-	 */
-	if (!g_file_exists (user_main_directory)) {
-		g_print ("FIXME bugzilla.eazel.com 1555\n");
-		g_print ("Need to post a error dialog if the user's main directory was not created\n");
-	}
 	g_free (user_main_directory);
 
 	/* write out the first time file to indicate that we've successfully traversed the druid */
 	druid_set_first_time_file_flag ();
-	signup_uris[1] = NULL;
 
-	/* FIXME Here we check to see if we can resolve hostnames in a timely
-	 * fashion.  If we can't then we silently tell nautilus to start up
+	/* Here we check to see if we can resolve hostnames in a timely
+	 * fashion. If we can't then we silently tell nautilus to start up
 	 * pointing to the home directory and not any of the HTTP addresses--
 	 * we don't want Nautilus to hang indefinitely trying to resolve
-	 * an HTTP address 
+	 * an HTTP address.
 	 */
-
-	if ( Untested == network_status
-		&& (last_signup_choice == 0 || last_signup_choice == 1 )) {
+	/* FIXME bugzilla.eazel.com 5051: Perhaps we can fix the underlying problem instead of
+	 * having this hack here to guess whether the network is broken.
+	 */
+	if (Untested == network_status
+	    && (last_signup_choice == 0 || last_signup_choice == 1)) {
 		check_network_connectivity ();
 	}
-
-	if ( Fail == network_status ) {
+	if (Fail == network_status) {
 		last_signup_choice = 3;
 	}
 	
+	/* Choose the URL based on the signup choice. */
 	switch (last_signup_choice) {
-		case 0:
-			signup_uris[0] = "http://services.eazel.com/services";
-			break;
-		case 1:
-			signup_uris[0] = "http://services.eazel.com/account/register/form";
-			break;
-		case 2:
-			signup_uris[0] = "eazel:";
-			break;
-		case 3:
-		default:
-			signup_uris[0]	= NULL;	
-			break;
+	case 0:
+		signup_uris[0] = EAZEL_SERVICES_LEARN_MORE_URL;
+		break;
+	case 1:
+		signup_uris[0] = EAZEL_SERVICES_SIGN_UP_URL;
+		break;
+	case 2:
+		signup_uris[0] = EAZEL_SERVICES_LOG_IN_URL;
+		break;
+	case 3:
+	default:
+		signup_uris[0]	= NULL;	
+		break;
 	}
-
-	has_druid_exited = TRUE;
+	signup_uris[1] = NULL;
+	
+	/* FIXME bugzilla.eazel.com bug 5681: Services icon on desktop may need better icon and text */
+	/* Create default services icon on the desktop */
+	desktop_path = nautilus_get_desktop_directory ();
+	nautilus_link_local_create (desktop_path, _("Eazel Services"), "big_services_icon.png", 
+				    "eazel:", NAUTILUS_LINK_GENERIC);
+	g_free (desktop_path);
+	
+	/* Time to start. Hooray! */
+	nautilus_application_startup (save_application, FALSE, FALSE, save_manage_desktop, 
+				      FALSE, FALSE, NULL, (signup_uris[0] != NULL) ? signup_uris : NULL);
 	
-	nautilus_application_startup(save_application, FALSE, FALSE, save_manage_desktop, 
-					     FALSE, (signup_uris[0] != NULL) ? &signup_uris[0] : NULL);
-
 	/* Destroy druid last because it may be the only thing keeping the main event loop alive. */
-	gtk_widget_destroy(gtk_widget_get_toplevel(druid_page));
+	gtk_widget_destroy (gtk_widget_get_toplevel (druid_page));
 }
 
 /* set up an event box to serve as the background */
 
-static GtkWidget*
+static GtkWidget *
 set_up_background (NautilusDruidPageEazel *page, const char *background_color)
 {
 	GtkWidget *event_box;
 	NautilusBackground *background;
 	
-	event_box = gtk_event_box_new();
+	event_box = gtk_event_box_new ();
 	
 	background = nautilus_get_widget_background (event_box);
 	nautilus_background_set_color (background, background_color);
@@ -234,7 +263,7 @@ user_level_selection_changed (GtkWidget *radio_button, gpointer user_data)
 	int user_level = GPOINTER_TO_INT (user_data);
 
 	if (GTK_TOGGLE_BUTTON (radio_button)->active) {
-		nautilus_user_level_manager_set_user_level (user_level);
+		nautilus_preferences_set_user_level (user_level);
 	}
 }
 
@@ -242,21 +271,18 @@ user_level_selection_changed (GtkWidget *radio_button, gpointer user_data)
 static void
 signup_selection_changed (GtkWidget *radio_buttons, gpointer user_data)
 {
-
 	last_signup_choice = nautilus_radio_button_group_get_active_index (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons));
 }
 
 static void
 update_selection_changed (GtkWidget *radio_buttons, gpointer user_data)
 {
-
 	last_update_choice = nautilus_radio_button_group_get_active_index (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons));
 }
 
 static void
 proxy_selection_changed (GtkWidget *radio_buttons, gpointer user_data)
 {
-
 	last_proxy_choice = nautilus_radio_button_group_get_active_index (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons));
 }
 
@@ -264,16 +290,12 @@ proxy_selection_changed (GtkWidget *radio_buttons, gpointer user_data)
 /* utility to allocate an anti-aliased description label */
 
 static GtkWidget*
-make_anti_aliased_label (const char *text)
+label_new_left_justified (const char *text)
 {
 	GtkWidget *label;
 	
 	label = nautilus_label_new (text);
-
-	nautilus_label_set_font_from_components (NAUTILUS_LABEL (label), "helvetica", "medium", NULL, NULL);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 12);
-	nautilus_label_set_text_justification (NAUTILUS_LABEL (label),
-			       GTK_JUSTIFY_LEFT);
+	nautilus_label_set_justify (NAUTILUS_LABEL (label), GTK_JUSTIFY_LEFT);
 
 	return label;
 }
@@ -303,18 +325,18 @@ make_hbox_user_level_radio_button (int index, GtkWidget *radio_buttons[],
 				   const char *icon_name,
 				   const char *comment, const char *background)
 {
-	GtkWidget *hbox, *label_box, *icon, *label, *alignment;
+	GtkWidget *hbox, *vbox;
+	GtkWidget *label_box, *icon, *label, *alignment;
 	GtkWidget *comment_vbox, *comment_hbox;
 	GdkPixbuf *icon_pixbuf;
 	char *user_level_name;
 
 	hbox = gtk_hbox_new (FALSE, 0);
 
-	user_level_name = nautilus_user_level_manager_get_user_level_name_for_display (index);
+	user_level_name = nautilus_preferences_get_user_level_name_for_display (index);
 
 	/* Add an "indent" */
 	alignment = gtk_alignment_new (1.0, 1.0, 1.0, 1.0);
-	gtk_widget_show (alignment);
 	gtk_widget_set_usize (alignment, 50, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), alignment, FALSE, FALSE, 0);
 
@@ -335,17 +357,24 @@ make_hbox_user_level_radio_button (int index, GtkWidget *radio_buttons[],
 	label_box = gtk_hbox_new (FALSE, 5);
 
 	icon_pixbuf = create_named_pixbuf (icon_name);
-	icon = nautilus_image_new ();
+	icon = nautilus_image_new (NULL);
 	if (icon_pixbuf != NULL) {
 		nautilus_image_set_pixbuf (NAUTILUS_IMAGE (icon), icon_pixbuf);
 	}
 	gtk_box_pack_start (GTK_BOX (label_box), icon, FALSE, FALSE, 0);
-	label = make_anti_aliased_label (user_level_name);
+	label = label_new_left_justified (user_level_name);
 	g_free (user_level_name);
 
-	nautilus_label_set_font_from_components (NAUTILUS_LABEL (label), "helvetica", "bold", NULL, NULL);
+	nautilus_label_make_bold (NAUTILUS_LABEL (label));
+	SIZE_BODY_LABEL (label);
 
-	gtk_box_pack_start (GTK_BOX (label_box), label, FALSE, FALSE, 0);
+	/* extra vbox to help with alignment */
+	vbox = gtk_vbox_new (FALSE, 0);
+	alignment = gtk_alignment_new (1.0, 1.0, 1.0, 1.0);
+	gtk_widget_set_usize (alignment, -1, 6);
+	gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (label_box), vbox, FALSE, FALSE, 0);
 
 	/* add label to radio button */
 	gtk_container_add (GTK_CONTAINER (radio_buttons[index]), label_box);
@@ -356,13 +385,12 @@ make_hbox_user_level_radio_button (int index, GtkWidget *radio_buttons[],
 
 	/* Add an "indent" */
 	alignment = gtk_alignment_new (1.0, 1.0, 1.0, 1.0);
-	gtk_widget_show (alignment);
 	gtk_widget_set_usize (alignment, 50, -1);
 	gtk_box_pack_start (GTK_BOX (comment_hbox), alignment, FALSE, FALSE, 0);
 
 	/* Make comment label */
-	label = make_anti_aliased_label (comment);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 12);
+	label = label_new_left_justified (comment);
+	SIZE_BODY_LABEL (label);
 
 	gtk_box_pack_start (GTK_BOX (comment_hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show_all (hbox);
@@ -387,8 +415,11 @@ set_up_user_level_page (NautilusDruidPageEazel *page)
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
 
-	label = make_anti_aliased_label (_("User levels provide a way to adjust the software to your\nlevel of technical expertise.  Pick an initial level that you\nfeel comfortable with; you can always change it later."));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 12);
+	label = label_new_left_justified (_("User levels provide a way to adjust the software to your\n"
+					   "level of technical expertise. Pick an initial level that you\n"
+					   "feel comfortable with; you can always change it later."));
+
+	SIZE_BODY_LABEL (label);
 
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -401,21 +432,21 @@ set_up_user_level_page (NautilusDruidPageEazel *page)
 		   "familiar with the working of "
 		   "GNOME and Linux."),
 		 NULL);
-	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 2);
 	hbox = make_hbox_user_level_radio_button
 		(NAUTILUS_USER_LEVEL_INTERMEDIATE, radio_buttons, "intermediate.png",
 		 _("For non-technical users that are comfortable with\n"
 		   "their GNOME and Linux environment."),
 				       NULL);
-	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 2);
 	hbox = make_hbox_user_level_radio_button
 		(NAUTILUS_USER_LEVEL_HACKER, radio_buttons, "expert.png",
 		 _("For users that have the need to be exposed\n"
 		   "to every detail of their operating system."),
 		 NULL);
-	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 2);
 
-	user_level = nautilus_user_level_manager_get_user_level ();
+	user_level = nautilus_preferences_get_user_level ();
 	g_assert (user_level >= NAUTILUS_USER_LEVEL_NOVICE && user_level <= NAUTILUS_USER_LEVEL_HACKER);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_buttons[user_level]), TRUE);
 
@@ -445,7 +476,14 @@ set_up_service_signup_page (NautilusDruidPageEazel *page)
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
 	/* allocate a descriptive label */
-	label = make_anti_aliased_label (_("Eazel offers a growing number of services to help you\n install and maintain new software and manage your files\n across the network.  Choose an option below, and the\ninformation will be presented in Nautilus after you've\nfinished setting up."));
+	label = label_new_left_justified (_("Eazel offers a growing number of services to help you\n"
+					   "install and maintain new software and manage your files\n"
+					   "across the network.  Choose an option below, and the\n"
+					   "information will be presented in Nautilus after you've\n"
+					   "finished setting up."));
+
+	SIZE_BODY_LABEL (label);
+
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 8);
 	
@@ -457,10 +495,14 @@ set_up_service_signup_page (NautilusDruidPageEazel *page)
 	gtk_container_add (GTK_CONTAINER (frame),
 					radio_buttons);
 
-	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("I want to learn more about Eazel services."));
-	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("I want to sign up for Eazel services now."));	
-	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("I've already signed up and want to login now."));
-	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("I don't want to learn about Eazel services at this time."));
+	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons),
+					    _("I want to learn more about Eazel services."));
+	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons),
+					    _("I want to sign up for Eazel services now."));	
+	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons),
+					    _("I've already signed up and want to login now."));
+	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons),
+					    _("I don't want to learn about Eazel services at this time."));
 
 	gtk_signal_connect (GTK_OBJECT (radio_buttons),
 			    "changed",
@@ -487,7 +529,9 @@ set_up_update_page (NautilusDruidPageEazel *page)
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
 	/* allocate a descriptive label */
-	label = make_anti_aliased_label (_("Nautilus will now contact Eazel services to verify your \nweb connection and download the latest updates.  \nClick the Next button to continue."));
+	label = label_new_left_justified (_("Nautilus will now contact Eazel services to quickly verify \nyour web connection and download the latest updates. \nClick the Next button to continue."));
+	SIZE_BODY_LABEL (label);
+
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 8);
 	
@@ -499,8 +543,8 @@ set_up_update_page (NautilusDruidPageEazel *page)
 	gtk_container_add (GTK_CONTAINER (frame),
 					radio_buttons);
 
-	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("Yes, verify my web connection and update Nautilus now."));
-	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("No, I don't wish to connect and update Nautilus now."));	
+	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("Yes, verify my connection and update Nautilus now."));
+	nautilus_radio_button_group_insert (NAUTILUS_RADIO_BUTTON_GROUP (radio_buttons), _("No, don't contact Eazel services at this time."));	
 
 	gtk_signal_connect (GTK_OBJECT (radio_buttons),
 			    "changed",
@@ -512,6 +556,37 @@ set_up_update_page (NautilusDruidPageEazel *page)
 
 }
 
+
+static gint
+proxy_address_entry_key_press (GtkWidget	*widget,
+                 	       GdkEventKey	*event,
+                 	       gpointer		callback_data)
+{
+	char *keysym_name;
+	GtkWidget *focus_target;
+
+	focus_target = GTK_WIDGET (callback_data);
+
+	g_return_val_if_fail (NULL != focus_target, FALSE);
+	
+	keysym_name = gdk_keyval_name (event->keyval);
+
+	if (strcmp (keysym_name, "Tab") == 0) {
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+
+		gtk_widget_grab_focus (focus_target);
+
+		if (GTK_IS_EDITABLE (focus_target)) {
+			gtk_editable_select_region (GTK_EDITABLE (focus_target), 0, -1);
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 /* set up the proxy configuration page */
 static void
 set_up_proxy_config_page (NautilusDruidPageEazel *page)
@@ -531,7 +606,8 @@ set_up_proxy_config_page (NautilusDruidPageEazel *page)
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
 	/* allocate a descriptive label */
-	label = make_anti_aliased_label (_("We are having troubles making an external web connection.  \nSometimes, firewalls require you to specify a web proxy server.  \n Fill in the name of port of your proxy server, if any, below."));
+	label = label_new_left_justified (_("We are having troubles making an external web connection.  \nSometimes, firewalls require you to specify a web proxy server.  \n Fill in the name of port of your proxy server, if any, below."));
+	SIZE_BODY_LABEL (label);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 8);
 	
@@ -574,8 +650,8 @@ set_up_proxy_config_page (NautilusDruidPageEazel *page)
 	
 	/* allocate the proxy label, followed by the entry */
 	label = nautilus_label_new (_("Proxy address:"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 12);
 	gtk_widget_show (label);
+	SIZE_BODY_LABEL (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
 	proxy_label_width = label->allocation.width;
 	proxy_address_entry = gtk_entry_new_with_max_length (24);
@@ -591,11 +667,13 @@ set_up_proxy_config_page (NautilusDruidPageEazel *page)
 	
 	/* allocate the proxy label, followed by the entry */
 	label = nautilus_label_new (_("Port:"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 12);
+	SIZE_BODY_LABEL (label);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
 
 	port_number_entry = gtk_entry_new_with_max_length (5);
+	gtk_entry_set_text (GTK_ENTRY (port_number_entry), DEFAULT_HTTP_PROXY_PORT_STRING);
+	
 	gtk_widget_set_usize (port_number_entry, 48, -1);
 	gtk_widget_show (port_number_entry);
 	gtk_box_pack_start (GTK_BOX (hbox), port_number_entry, FALSE, FALSE, 2);
@@ -604,6 +682,15 @@ set_up_proxy_config_page (NautilusDruidPageEazel *page)
 	gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 2);
 
 
+	/* Slam it so that the <tab> in "Proxy Address" goes to "Port"
+	 * I have no idea why this doesn't just work
+	 */
+
+	gtk_signal_connect (GTK_OBJECT (proxy_address_entry),
+			    "key_press_event",
+			    GTK_SIGNAL_FUNC (proxy_address_entry_key_press),
+			    port_number_entry);
+
 }
 
 /* set up the update feedback page */
@@ -621,14 +708,14 @@ set_up_update_feedback_page (NautilusDruidPageEazel *page)
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
 	/* allocate a descriptive label */
-	label = make_anti_aliased_label (_("We are now contacting the Eazel service to test your \nweb connection and update Nautilus."));
+	label = label_new_left_justified (_("We are now contacting the Eazel service to test your \nweb connection and update Nautilus."));
+	SIZE_BODY_LABEL (label);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 8);
 		
 		
 	download_label = nautilus_label_new (_("Downloading Nautilus updates..."));
-	nautilus_label_set_font_from_components (NAUTILUS_LABEL (download_label), "helvetica", "medium", NULL, NULL);
-	nautilus_label_set_font_size (NAUTILUS_LABEL (download_label), 18);
+	nautilus_label_make_larger (NAUTILUS_LABEL (download_label), 4);
 	
 	gtk_widget_show (download_label);
 	
@@ -641,6 +728,7 @@ next_update_page_callback (GtkWidget *button, GnomeDruid *druid)
 {
 	if (last_update_choice == 0) {
 		/* initiate the file transfer and launch a timer task to track feedback */
+		/* FIXME bugzilla.eazel.com 5053: There's no timer task! */
 		initiate_file_download (druid);
 		
 		/* return FALSE to display the feedback page */
@@ -667,13 +755,13 @@ next_proxy_configuration_page_callback (GtkWidget *button, GnomeDruid *druid)
 {
 	const char *proxy_text;
 	const char *port_text;
+	char *proxy_url;
 
 	proxy_text = gtk_entry_get_text (GTK_ENTRY (proxy_address_entry));
 	port_text = gtk_entry_get_text (GTK_ENTRY (port_number_entry));
 
 	/* Update the http proxy only if there is some user input */
 	if (nautilus_strlen (proxy_text) > 0 && nautilus_strlen (port_text) > 0) {
-		char *proxy_url;
 		proxy_url = g_strdup_printf ("http://%s:%s", proxy_text, port_text);
 		set_http_proxy (proxy_url);
 		g_free (proxy_url);
@@ -712,7 +800,8 @@ set_page_sidebar (NautilusDruidPageEazel *page)
 	}
 }
 
-GtkWidget *nautilus_first_time_druid_show (NautilusApplication *application, gboolean manage_desktop, const char *urls[])
+GtkWidget *
+nautilus_first_time_druid_show (NautilusApplication *application, gboolean manage_desktop, const char *urls[])
 {	
 	GtkWidget *dialog;
 	GtkWidget *druid;
@@ -726,8 +815,9 @@ GtkWidget *nautilus_first_time_druid_show (NautilusApplication *application, gbo
 	dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (dialog),
 			      _("Nautilus First Time Setup"));
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), "initial_preferences", "Nautilus");
   	gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
-  	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
+  	//gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
   	/* Ensure there's a main event loop while the druid is running. */
   	nautilus_main_event_loop_register (GTK_OBJECT (dialog));
 
@@ -754,25 +844,25 @@ GtkWidget *nautilus_first_time_druid_show (NautilusApplication *application, gbo
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
 	/* make the title label */
-	label = make_anti_aliased_label ( _("Welcome to Nautilus!"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 28);
-	nautilus_label_set_drop_shadow_offset (NAUTILUS_LABEL (label), 2);
-	nautilus_label_set_drop_shadow_color (NAUTILUS_LABEL (label), NAUTILUS_RGBA_COLOR_PACK (191, 191, 191, 255));
+	label = label_new_left_justified ( _("Welcome to Nautilus!"));
+	nautilus_label_make_larger (NAUTILUS_LABEL (label), 4);
+	nautilus_label_set_smooth_drop_shadow_offset (NAUTILUS_LABEL (label), 2);
+	nautilus_label_set_smooth_drop_shadow_color (NAUTILUS_LABEL (label), NAUTILUS_RGBA_COLOR_PACK (191, 191, 191, 255));
 	
 	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 16);
+	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, TRUE, 16);
 	
-	label = make_anti_aliased_label ( _("Since this is the first time that you've launched\nNautilus, we'd like to ask you a few questions\nto help personalize it for your use."));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 18);
+	label = label_new_left_justified ( _("Since this is the first time that you've launched\nNautilus, we'd like to ask you a few questions\nto help personalize it for your use."));
+	nautilus_label_make_larger (NAUTILUS_LABEL (label), 4);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 8);
 	
-	label = make_anti_aliased_label ( _("Press the next button to continue."));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 18);
+	label = label_new_left_justified ( _("Press the next button to continue."));
+	nautilus_label_make_larger (NAUTILUS_LABEL (label), 4);
 
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 16);
-	nautilus_label_set_text_justification (NAUTILUS_LABEL (label), GTK_JUSTIFY_LEFT);
+	nautilus_label_set_justify (NAUTILUS_LABEL (label), GTK_JUSTIFY_LEFT);
 	
 	/* set up the final page */
 	nautilus_druid_page_eazel_set_title (NAUTILUS_DRUID_PAGE_EAZEL (finish_page), _("Finished"));
@@ -782,8 +872,8 @@ GtkWidget *nautilus_first_time_druid_show (NautilusApplication *application, gbo
 	gtk_widget_show (main_box);
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
-	label = make_anti_aliased_label ( _("Click the finish button to launch Nautilus.\nWe hope that you enjoy using it!"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (label), 18);
+	label = label_new_left_justified ( _("Click the finish button to launch Nautilus.\nWe hope that you enjoy using it!"));
+	nautilus_label_make_larger (NAUTILUS_LABEL (label), 4);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 8);
 		
@@ -869,17 +959,12 @@ download_callback (GnomeVFSResult result,
 	GnomeDruid *druid;
 	char *temporary_file;
 	char *remove_command;
-
-	/* FIXME this is a cheap work-around to the problem
-	 * that the user can get beyond the download
-	 * page and leave the druid before the download is complete
-	 */
-	if (!has_druid_exited) {
-		druid = GNOME_DRUID (callback_data);
-	} else {
-		druid = NULL;
-	}
 	
+	druid = GNOME_DRUID (callback_data);
+	
+	/* Let the caller know we're done. */
+	gtk_object_remove_no_notify (GTK_OBJECT (druid), READ_FILE_HANDLE_TAG);
+
 	/* check for errors */
 	if (result == GNOME_VFS_OK) {
 		/* there was no error, so write out the file into the /tmp directory */
@@ -891,10 +976,8 @@ download_callback (GnomeVFSResult result,
 		fclose (outfile);
 		g_free (file_contents);
 
-		if (!has_druid_exited) {
-			/* change the message to expanding file */
-			nautilus_label_set_text (NAUTILUS_LABEL (download_label), _("Decoding Update..."));
-		}
+		/* change the message to expanding file */
+		nautilus_label_set_text (NAUTILUS_LABEL (download_label), _("Decoding Update..."));
 		
 		/* expand the directory into the proper place */
 		/* first, formulate the command string */
@@ -911,25 +994,19 @@ download_callback (GnomeVFSResult result,
 		/* Remove the temporary file */
 		expand_result = system (remove_command);
 
-		if (!has_druid_exited) {
-			nautilus_label_set_text (NAUTILUS_LABEL (download_label), _("Update Completed... Press Next to Continue."));
-		}
+		nautilus_label_set_text (NAUTILUS_LABEL (download_label), _("Update Completed... Press Next to Continue."));
 			
 		g_free (user_directory_path);
 		g_free (untar_command);
 		g_free (remove_command);
 
-		if (!has_druid_exited) {
-			/* now that we're done, reenable the buttons */
-			gtk_widget_set_sensitive (druid->next, TRUE);
-			gtk_widget_set_sensitive (druid->back, TRUE);
-		}
-	} else if (has_druid_exited) {
-		return;
+		/* now that we're done, reenable the buttons */
+		gtk_widget_set_sensitive (druid->next, TRUE);
+		gtk_widget_set_sensitive (druid->back, TRUE);
 	} else {
 		/* there was an error; see if we can't find some HTTP proxy config info */
-		/* note that attempt_http_proxy_autoconfigure() returns FALSE if its already been tried */ 
-		if (attempt_http_proxy_autoconfigure()) {
+		/* note that attempt_http_proxy_autoconfigure returns FALSE if it's already been tried */ 
+		if (attempt_http_proxy_autoconfigure ()) {
 			initiate_file_download (druid);
 		} else {
 			/* Autoconfiguration didn't work; prompt the user */
@@ -941,32 +1018,68 @@ download_callback (GnomeVFSResult result,
 /* initiate downloading of the welcome package from the service */
 /* Note that this may be invoked more than once, if the first time fails */
 
+static void
+read_file_handle_cancel_cover (gpointer data)
+{
+	NautilusReadFileHandle *handle;
+
+	handle = data;
+	nautilus_read_file_cancel (handle);
+}
+
 static void
 initiate_file_download (GnomeDruid *druid)
 {
-	NautilusReadFileHandle *file_handle;
-	const char *file_uri = "http://services.eazel.com/downloads/eazel/updates.tgz";
+	NautilusReadFileHandle *handle;
+	static gboolean prevent_re_entry = FALSE;
 
-	/* disable the next and previous buttons during the  file loading process */
+	/* We exercise the event loop below, so we need to make sure that 
+	 * we don't get re-entered
+	 */
+
+	if (prevent_re_entry) {
+		return;
+	}
+
+	prevent_re_entry = TRUE;
+
+	/* disable the next and previous buttons during the file loading process */
 	gtk_widget_set_sensitive (druid->next, FALSE);
 	gtk_widget_set_sensitive (druid->back, FALSE);
 
-	/* FIXME We might hang here for a while; if we do, we don't want
+	/* Cancel any download already in progress. */
+	gtk_object_remove_data (GTK_OBJECT (druid), READ_FILE_HANDLE_TAG);
+			
+	/* FIXME bugzilla.eazel.com 5054: We might hang here for a while; if we do, we don't want
 	 * the user to get forced through the druid again
 	 */
-	druid_set_first_time_file_flag();
+	druid_set_first_time_file_flag ();
 
-#if 0
-	/* initiate the file transfer */
-	file_handle = nautilus_read_entire_file_async (file_uri, download_callback, druid);
-#else
-	if (check_network_connectivity()) {
+	/* We need to exercise the main loop so that gnome-vfs can get its
+	 * gconf callback for the HTTP proxy autoconfiguration case.
+	 * FIXME: note that "druid" can be freed in this event loop
+	 */
+
+	while (gtk_events_pending()) {
+		gtk_main_iteration();
+	}
+
+	prevent_re_entry = FALSE;
+
+	if (check_network_connectivity ()) {
 		/* initiate the file transfer */
-		file_handle = nautilus_read_entire_file_async (file_uri, download_callback, druid);
+		handle = nautilus_read_entire_file_async
+			(WELCOME_PACKAGE_URI, download_callback, druid);
+		if (handle != NULL) {
+			/* cancel the transfer if the druid goes away */
+			gtk_object_set_data_full (GTK_OBJECT (druid),
+						  READ_FILE_HANDLE_TAG,
+						  handle,
+						  read_file_handle_cancel_cover);
+		}
 	} else {
 		download_callback (GNOME_VFS_ERROR_GENERIC, 0, NULL, druid);
 	}
-#endif /* 0 */
 }
 
 /**
@@ -980,8 +1093,17 @@ set_http_proxy (const char *proxy_url)
 {
 	const char *proxy_url_port_part;
 	size_t proxy_len;
-	char *proxy_host_port;
 	char *colon;
+	int port;
+
+	/* DEBUG */
+	g_print ("set_http_proxy: %s\n", proxy_url);
+
+	/* We reset this later */
+	if (http_proxy_host) {
+		g_free (http_proxy_host);
+		http_proxy_host = NULL;
+	}
 
 	/* set the "http_proxy" environment variable */
 
@@ -999,36 +1121,42 @@ set_http_proxy (const char *proxy_url)
 		return FALSE;
 	}
 
+	http_proxy_host = g_strdup (proxy_url_port_part);
+
 	/* chew off trailing / */
-	proxy_host_port = g_strdup (proxy_url_port_part);
-	if ('/' == proxy_host_port[proxy_len - 1]) {
-		proxy_host_port[proxy_len - 1] = 0;
+	if ('/' == http_proxy_host[proxy_len - 1]) {
+		http_proxy_host[proxy_len - 1] = 0;
 	}
 
-	nautilus_preferences_set (GNOME_VFS_PREFERENCES_HTTP_PROXY, proxy_host_port);
-
-	/* Keep it around for check_network_connectivity, trimming off the :port */
-	if (http_proxy_host) {
-		g_free (http_proxy_host);
-	}
-
-	http_proxy_host = proxy_host_port;
-	proxy_host_port = NULL;
-
+	/* Scan for port */
 	if ( NULL != ( colon = strchr (http_proxy_host, (unsigned char)':'))) {
+		char *endptr;
+		
 		*colon = '\0';
+		port = strtoul (colon+1, &endptr, 10);
+
+		/*no integer here*/
+		if ('\0' == *(colon+1) || endptr == colon+1) {
+			port = DEFAULT_HTTP_PROXY_PORT;
+		}
+	} else {
+		port = DEFAULT_HTTP_PROXY_PORT;
 	}
 
+	nautilus_preferences_set (GNOME_VFS_PREFERENCES_HTTP_PROXY_HOST, http_proxy_host);
+	nautilus_preferences_set_integer (GNOME_VFS_PREFERENCES_HTTP_PROXY_PORT, port);
+	nautilus_preferences_set_boolean (GNOME_VFS_PREFERENCES_USE_HTTP_PROXY, TRUE);
+
 	return TRUE;
 }
 
 /**
  * getline_dup
  *
- * reads newline or EOF terminated line from stream, allocating the return
+ * reads newline (or CR) or EOF terminated line from stream, allocating the return
  * buffer as appropriate
- */ 
-#define GETLINE_INITIAL 256
+ **/
+/* FIXME bugzilla.eazel.com 5055: Belongs in a library, not here. */
 static char * 
 getline_dup (FILE* stream)
 {
@@ -1038,13 +1166,13 @@ getline_dup (FILE* stream)
 	int char_read;
 	gboolean done;
 
-	ret = g_malloc( GETLINE_INITIAL * sizeof(char) );
-	ret_size = GETLINE_INITIAL;
+	ret = g_malloc (GETLINE_INITIAL_BUFFER_SIZE);
+	ret_size = GETLINE_INITIAL_BUFFER_SIZE;
 
 	for ( ret_used = 0, done = FALSE ;
 	      !done && (EOF != (char_read = fgetc (stream))) ; 
-	      ret_used++
-	) {
+	      ret_used++) {
+		
 		if (ret_size == (ret_used + 1)) {
 			ret_size *= 2;
 			ret = g_realloc (ret, ret_size); 
@@ -1057,7 +1185,7 @@ getline_dup (FILE* stream)
 		}
 	}
 
-	if ( 0 == ret_used ) {
+	if (ret_used == 0) {
 		g_free (ret);
 		ret = NULL;
 	} else {
@@ -1067,29 +1195,32 @@ getline_dup (FILE* stream)
 	return ret;
 }
 
-#define NETSCAPE_PREFS_PATH "/.netscape/preferences.js"
-
 /* user_pref("network.proxy.http", "localhost");
  * user_pref("network.proxy.http_port", 8080);
  * user_pref("network.proxy.type", 1);
  */
 static char *
-load_nscp_proxy_settings ()
+load_netscape_proxy_settings (void)
 {
-	char * prefs_path = NULL;
-	char * ret = NULL;
-	char * proxy_host = NULL;
-	guint32 proxy_port = 8080;
-	gboolean has_proxy_type = FALSE;
+	char *prefs_path;
+	char *ret ;
+	char *proxy_host;
+	guint32 proxy_port;
+	gboolean has_proxy_type;
 
-	char * line;
-	char * current, *end;
-	FILE * prefs_file;
+	char *line;
+	char *current, *end;
+	FILE *prefs_file;
+
+	ret = NULL;
+	proxy_host = NULL;
+	proxy_port = 8080;
+	has_proxy_type = FALSE;
 
 	prefs_path = g_strconcat (g_get_home_dir (), NETSCAPE_PREFS_PATH, NULL);
-	prefs_file = fopen (prefs_path, "r");
 
-	if ( NULL == prefs_file ) {
+	prefs_file = fopen (prefs_path, "r");
+	if (prefs_file == NULL) {
 		goto error;
 	}
 
@@ -1143,14 +1274,11 @@ load_nscp_proxy_settings ()
 error:
 	g_free (proxy_host);
 	g_free (prefs_path);
-	prefs_path = NULL;
 
 	return ret;
 }
 
 
-#define GALEON_PREFS_PATH "/.gnome/galeon"
-
 /* http_proxy=localhost
  * http_proxy_port=4128
  */
@@ -1223,7 +1351,7 @@ attempt_http_proxy_autoconfigure (void)
 	}
 
 	/* Check Netscape 4.x settings */
-	proxy_url = load_nscp_proxy_settings ();
+	proxy_url = load_netscape_proxy_settings ();
 	if (NULL != proxy_url) {
 		success = TRUE;
 		set_http_proxy (proxy_url);
@@ -1247,15 +1375,11 @@ done:
 	return success;
 }
 
-static gboolean sigalrm_occurred = FALSE;
-static pid_t child_pid;
-
-static void my_sigalrm_handler (int sig)
+static void
+sigalrm_handler (int sig)
 {
 	sigalrm_occurred = TRUE;
 	kill (child_pid, SIGKILL);
-
-	return;
 }
 
 /* Do a simple DNS lookup wrapped in sigalrm to check the presense of the network
@@ -1270,31 +1394,27 @@ check_dns_resolution (const char *host)
 {
 	struct hostent *my_hostent;
 	int child_status;
+	void *sigalrm_old;
 
 	sigalrm_occurred = FALSE;
 
-	if ( 0 == (child_pid = fork())) {
+	child_pid = fork ();
+	if (child_pid == 0) {
 		my_hostent = gethostbyname (host);
 		_exit (0);
 	}
 
-	if (child_pid > 0 ) {
-		void *sigalrm_old;
-		sigalrm_old = signal (SIGALRM, my_sigalrm_handler);
-
+	if (child_pid > 0) {
+		sigalrm_old = signal (SIGALRM, sigalrm_handler);
 		alarm (NETWORK_CHECK_TIMEOUT_SECONDS);
-
 		waitpid (child_pid, &child_status, 0);
-		
 		alarm (0);
-
 		signal (SIGALRM, sigalrm_old);	
 	}
-
-	return ! sigalrm_occurred;
+	
+	return !sigalrm_occurred;
 }
 
-
 static gboolean
 check_network_connectivity (void) 
 {
@@ -1310,8 +1430,7 @@ check_network_connectivity (void)
 		ret = check_dns_resolution (DEFAULT_DNS_CHECK_HOST);
 	}
 
-	network_status = ret ? Success : Fail ;
+	network_status = ret ? Success : Fail;
 
 	return ret;
 }
-
