@@ -57,6 +57,11 @@
 #define NAUTILUS_DND_TEXT_PLAIN_TYPE 	  "text/plain"
 #define NAUTILUS_DND_URL_TYPE		  "_NETSCAPE_URL"
 
+static const char untranslated_location_label[] = N_("Location:");
+static const char untranslated_go_to_label[] = N_("Go To:");
+#define LOCATION_LABEL _(untranslated_location_label)
+#define GO_TO_LABEL _(untranslated_go_to_label)
+
 enum {
 	NAUTILUS_DND_MC_DESKTOP_ICON,
 	NAUTILUS_DND_URI_LIST,
@@ -92,7 +97,7 @@ static NautilusWindow *
 nautilus_location_bar_get_window (GtkWidget *bar)
 {
 
-	return NAUTILUS_WINDOW (gtk_object_get_data (GTK_OBJECT (bar), "associated_window"));
+	return NAUTILUS_WINDOW (gtk_widget_get_ancestor (bar, NAUTILUS_TYPE_WINDOW));
 }
 
 static void
@@ -199,6 +204,25 @@ drag_data_get_callback (GtkWidget *widget,
 	g_free (entry_text);
 }
 
+/* routine that determines the usize for the label widget as larger
+   then the size of the largest string and then sets it to that so
+   that we don't have localization problems. see
+   gtk_label_finalize_lines in gtklabel.c (line 618) for the code that
+   we are imitating here. */
+
+static void
+style_set_handler (GtkWidget *widget, GtkStyle *previous_style)
+{
+	int width;
+
+	width = gdk_string_width (widget->style->font, GO_TO_LABEL);
+	width = MAX (width, gdk_string_width (widget->style->font, LOCATION_LABEL));
+
+	width += 2 * GTK_MISC (widget)->xpad;
+	
+	gtk_widget_set_usize (widget, width, -1);
+}
+
 /* utility routine to determine the string to expand to.  If we don't have anything yet, accept
    the whole string, otherwise accept the largest part common to both */
 
@@ -264,16 +288,20 @@ try_to_expand_path (GtkEditable *editable)
 	uri = gnome_vfs_uri_new (current_path);
 	
 	base_name_ptr = gnome_vfs_uri_get_basename (uri);
-	if (base_name_ptr) {
-		base_name = gnome_vfs_unescape_string (base_name_ptr, NULL);
-		base_length = strlen (base_name);
+	if (base_name_ptr == NULL) {
+		base_name = NULL;
 	} else {
+		base_name = gnome_vfs_unescape_string (base_name_ptr, NULL);
+	}
+
+	if (base_name == NULL) {
 		gnome_vfs_uri_unref (uri);
 		g_free (current_path);
 		return;	
 	}
-		
-	dir_name = gnome_vfs_uri_extract_dirname(uri);
+	
+	base_length = strlen (base_name);
+	dir_name = gnome_vfs_uri_extract_dirname (uri);
 
 	/* get file info for the directory */
 
@@ -439,17 +467,17 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 	GtkWidget *event_box;
 	GtkWidget *hbox;
 
-	bar->last_location = NULL;
-	
 	hbox = gtk_hbox_new (0, FALSE);
 
 	event_box = gtk_event_box_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (event_box),
 					GNOME_PAD_SMALL);
-	label = gtk_label_new (_("Location:"));
+	label = gtk_label_new (LOCATION_LABEL);
 	gtk_container_add   (GTK_CONTAINER (event_box), label);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
-
+	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+	gtk_signal_connect (GTK_OBJECT (label), "style_set", 
+			    style_set_handler, NULL);
 
 	gtk_box_pack_start  (GTK_BOX (hbox), event_box, FALSE, TRUE,
 			     GNOME_PAD_SMALL);
@@ -495,8 +523,6 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 
 	bar->label = GTK_LABEL (label);
 	bar->entry = GTK_ENTRY (entry);	
-
-
 }
 
 
@@ -512,7 +538,8 @@ nautilus_location_bar_new (NautilusWindow *window)
 	/* Clipboard */
 	nautilus_clipboard_set_up_editable
 		(GTK_EDITABLE (location_bar->entry),
-		 nautilus_window_get_ui_container (window));
+		 nautilus_window_get_ui_container (window),
+		 TRUE);
 
 	return bar;
 }
@@ -539,10 +566,7 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 
 	/* remember the original location for later comparison */
 	
-	if (bar->last_location) {
-		g_free (bar->last_location);
-	}
-
+	g_free (bar->last_location);
 	bar->last_location = g_strdup (location);
 	nautilus_location_bar_update_label (bar);
 }
@@ -588,9 +612,9 @@ nautilus_location_bar_update_label (NautilusLocationBar *bar)
 	current_location = nautilus_make_uri_from_input (current_text);
 	
 	if (nautilus_uris_match (bar->last_location, current_location)) {
-		gtk_label_set_text (GTK_LABEL (bar->label), _("Location:"));
+		gtk_label_set_text (GTK_LABEL (bar->label), LOCATION_LABEL);
 	} else {		 
-		gtk_label_set_text (GTK_LABEL (bar->label), _("   Go To:"));
+		gtk_label_set_text (GTK_LABEL (bar->label), GO_TO_LABEL);
 	}	
 	g_free (current_location);
 }
