@@ -110,7 +110,8 @@ nautilus_main_event_loop_register (GtkObject* object)
 gboolean
 nautilus_main_is_event_loop_mainstay (GtkObject* object)
 {
-	return g_slist_length (event_loop_registrants) == 1 && event_loop_registrants->data == object;
+	return g_slist_length (event_loop_registrants) == 1
+		&& event_loop_registrants->data == object;
 }
 
 void
@@ -126,8 +127,9 @@ main (int argc, char *argv[])
 {
 	gboolean kill_shell;
 	gboolean restart_shell;
-	gboolean stop_desktop;
 	gboolean start_desktop;
+	gboolean no_default_window;
+	char *geometry;
 	gboolean perform_self_check;
 	poptContext popt_context;
 	const char **args;
@@ -137,14 +139,21 @@ main (int argc, char *argv[])
 
 	struct poptOption options[] = {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
-		{ "check", '\0', POPT_ARG_NONE, &perform_self_check, 0, N_("Perform high-speed self-check tests."), NULL },
+		{ "check", 'c', POPT_ARG_NONE, &perform_self_check, 0,
+		  N_("Perform a quick set of self-check tests."), NULL },
 #endif
-		{ "quit", '\0', POPT_ARG_NONE, &kill_shell, 0, N_("Quit Nautilus."), NULL },
-		{ "restart", '\0', POPT_ARG_NONE, &restart_shell, 0, N_("Restart Nautilus."), NULL },
-		{ "stop-desktop", '\0', POPT_ARG_NONE, &stop_desktop, 0, N_("Don't draw background and icons on desktop."), NULL },
-		{ "start-desktop", '\0', POPT_ARG_NONE, &start_desktop, 0, N_("Draw background and icons on desktop."), NULL },
+		{ "geometry", 'g', POPT_ARG_STRING, &geometry, 0,
+		  N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
+		{ "no-default-window", 'n', POPT_ARG_NONE, &no_default_window, 0,
+		  N_("Only create windows for explicitly specified URIs."), NULL },
+		{ "quit", 'q', POPT_ARG_NONE, &kill_shell, 0,
+		  N_("Quit Nautilus."), NULL },
+		{ "restart", '\0', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &restart_shell, 0,
+		  N_("Restart Nautilus."), NULL },
+		/* FIXME bugzilla.eazel.com 5228: The help for these OAF options
+		 * should be in a separate section.
+		 */
 		{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, &oaf_popt_options, 0, NULL, NULL },
-		POPT_AUTOHELP
 		{ NULL, '\0', 0, NULL, 0, NULL, NULL }
 	};
 
@@ -181,22 +190,23 @@ main (int argc, char *argv[])
 	nautilus_setenv ("GNOME_DISABLE_CRASH_DIALOG", "1", TRUE);
 
 	/* Get parameters. */
+	geometry = NULL;
 	kill_shell = FALSE;
-	restart_shell = FALSE;
-	stop_desktop = FALSE;
-	start_desktop = FALSE;
+	no_default_window = FALSE;
 	perform_self_check = FALSE;
+	restart_shell = FALSE;
         gnome_init_with_popt_table ("nautilus", VERSION,
 				    argc, argv, options, 0,
 				    &popt_context);
-	
+	gdk_rgb_init ();
+
 	/* Check for argument consistency. */
 	args = poptGetArgs (popt_context);
 	if (perform_self_check && args != NULL) {
 		fprintf (stderr, _("nautilus: --check cannot be used with URIs.\n"));
 		return EXIT_FAILURE;
 	}
-	if (perform_self_check && (kill_shell || restart_shell || stop_desktop || start_desktop)) {
+	if (perform_self_check && (kill_shell || restart_shell)) {
 		fprintf (stderr, _("nautilus: --check cannot be used with other options.\n"));
 		return EXIT_FAILURE;
 	}
@@ -208,16 +218,8 @@ main (int argc, char *argv[])
 		fprintf (stderr, _("nautilus: --restart cannot be used with URIs.\n"));
 		return EXIT_FAILURE;
 	}
-	if (kill_shell && start_desktop) {
-		fprintf (stderr, _("nautilus: --quit and --start-desktop cannot be used together.\n"));
-		return EXIT_FAILURE;
-	}
-	if (restart_shell && start_desktop) {
-		fprintf (stderr, _("nautilus: --restart and --start-desktop cannot be used together.\n"));
-		return EXIT_FAILURE;
-	}
-	if (stop_desktop && start_desktop) {
-		fprintf (stderr, _("nautilus: --stop-desktop and --start-desktop cannot be used together.\n"));
+	if (geometry != NULL && args != NULL && args[0] != NULL && args[1] != NULL) {
+		fprintf (stderr, _("nautilus: --geometry cannot be used with more than one URI.\n"));
 		return EXIT_FAILURE;
 	}
 
@@ -239,10 +241,7 @@ main (int argc, char *argv[])
 	 */
 	nautilus_global_preferences_initialize ();
 
-	/* if desktop flags are not explicitly specified, get it from preferences */
-	if (!start_desktop && !stop_desktop) {
-		start_desktop = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP, FALSE);
-	}
+	start_desktop = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP);
 		
 	/* Do either the self-check or the real work. */
 	if (perform_self_check) {
@@ -260,8 +259,7 @@ main (int argc, char *argv[])
 		application = nautilus_application_new ();
 		nautilus_application_startup
 			(application,
-			 kill_shell, restart_shell,
-			 stop_desktop, start_desktop,
+			 kill_shell, restart_shell, start_desktop, no_default_window, TRUE, geometry,
 			 args);
 		if (is_event_loop_needed ()) {
 			bonobo_main ();
