@@ -46,7 +46,6 @@
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-icon-factory.h>
-#include <libnautilus-extensions/nautilus-label.h>
 #include <libnautilus-extensions/nautilus-entry.h>
 #include <libnautilus-extensions/nautilus-search-uri.h>
 #include <libnautilus-extensions/nautilus-string.h>
@@ -54,6 +53,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
+#include <widgets/gimphwrapbox/gtkhwrapbox.h>
 
 enum {
 	CRITERION_TYPE_CHANGED,
@@ -63,79 +63,129 @@ enum {
 static guint signals[LAST_SIGNAL];
 
 static char * criteria_titles [] = {
+	/* Menu item to search by file name in e.g. "Name starts with b" */
 	N_("Name"),
+	/* Menu item to search by file content in e.g. "Contains contains GNU" */
 	N_("Content"),
+	/* Menu item to search by file type in e.g. "Type is music" */
 	N_("Type"),
+	/* Menu item to search by file size in e.g. "Size larger than 500K" */
 	N_("Size"),
+	/* Menu item to search for files with an emblem attached in e.g. "With emblem marked with Important" */
 	N_("With Emblem"),
+	/* Menu item to search for files by the time they were last modified in e.g. "Last modified before [date]" */
 	N_("Last Modified"),
-	N_("Owned By"),
+	/* Menu item to search for files by their owner in e.g. "Owner is root" */
+	N_("Owner"),
 	NULL
 };
 
 
 static char *name_relations [] = {
+	/* The "contains" in a search for items where "file name contains" */
 	N_("contains"),
+	/* The "starts with" in a search for items where  "file name starts with" */
 	N_("starts with"),
+	/* The "ends with" in a search for items where "file name ends with" */
 	N_("ends with"),
+	/* The "matches glob" (eg *.c) in a search for items where "file name matches glob" */
 	N_("matches glob"),
+	/* The "matches regexp" (eg '[a-z]{2}+' in a search for items where "file name matches regexp" */
 	N_("matches regexp"),
 	NULL
 };
 
 static char *content_relations [] = {
+	/* The "includes all of" in a search for items where "content includes all of" */
 	N_("includes all of"),
+	/* The "includes any of" in a search for items where "content includes any of" */
 	N_("includes any of"),
+	/* The "does not include all of" in a search for items where "content does not include all of" */
 	N_("does not include all of"),
+	/* The "includes none of" in a search for items where "content includes none of" */
 	N_("includes none of"),
 	NULL
 
 };
 
 static char *type_relations [] = {
+	/* The "is" in a search for items where "Type is" */
 	N_("is"),
+	/* The "is not" in a search for items where "Type is not" */
 	N_("is not"),
 	NULL
 };
 
 static char *type_objects [] = {
+	/* A type of file you can search for; context is "Type is regular file" */
 	N_("regular file"),
+	/* A type of file you can search for; context is "Type is text file" */
 	N_("text file"),
+	/* A type of file you can search for; context is "Type is application" */
 	N_("application"),
-	N_("directory"),
+	/* A type of file you can search for; context is "Type is folder" */
+	N_("folder"),
+	/* A type of file you can search for; context is "Type is regular file" */
 	N_("music"),
 	NULL
 };
 
 static char *size_relations [] = {
+	/* In relation to file size; context is "Size larger than" */
 	N_("larger than"),
+	/* In relation to file size; context is "Size smaller than" */
 	N_("smaller than"),
 	NULL
 };
 
 static char *emblem_relations [] = {
+	/* In relation to emblems; context is "With emblem marked with" */
 	N_("marked with"),
+	/* in relation to emblems; context is "With emblem not marked with" */
 	N_("not marked with"),
 	NULL
 };
 
 static char *modified_relations [] = {
+	/* In relation to a file's last modfied time; context is "Last modified is 11/2/00" */
 	N_("is"),
+	/* In relation to a file's last modfied time; context is "Last modified is not 11/2/00" */
 	N_("is not"),
+	/* In relation to a file's last modfied time; context is "Last modified is after 11/2/00" */
 	N_("is after"),
+	/* In relation to a file's last modfied time; context is "Last modified is before 11/2/00" */
 	N_("is before"),
 	"",
+	/* In relation to a file's last modfied time; context is "Last modified is today" */
 	N_("is today"),
+	/* In relation to a file's last modfied time; context is "Last modified is yesterdat" */
 	N_("is yesterday"),
 	"",
+	/* In relation to a file's last modfied time; context is "Last modified is within a week of" */
 	N_("is within a week of"),
+	/* In relation to a file's last modfied time; context is "Last modified is within a month of" */
 	N_("is within a month of"),
 	NULL
 };
 
+static gboolean modified_relation_shows_date [] = {
+	TRUE,
+	TRUE,
+	TRUE,
+	TRUE, 
+	TRUE,     /* Separator */
+	FALSE,    /* is today */
+	FALSE,    /* is yesterday */
+	TRUE,     /* Separator */
+	TRUE,
+	TRUE,
+	/* NULL */
+};
 
 static char *owner_relations [] = {
+	/* In relation to a file's owner; context is "Owner is root" */
 	N_("is"),
+	/* In relation to a file's owner; context is "Owner is not root" */
 	N_("is not"),
 	NULL
 };
@@ -155,6 +205,11 @@ static NautilusSearchBarCriterion * nautilus_search_bar_criterion_new_from_value
 NautilusSearchBarCriterionType      get_next_default_search_criterion_type         (NautilusSearchBarCriterionType type) ;
 static void                         nautilus_search_bar_criterion_initialize_class (NautilusSearchBarCriterionClass *klass);
 static void                         nautilus_search_bar_criterion_initialize       (NautilusSearchBarCriterion *criterion);
+static gboolean                     modified_relation_should_show_value            (int relation_index);
+static void                         hide_date_widget                               (GtkObject *object,
+										    gpointer data);
+static void                         show_date_widget                               (GtkObject *object,
+										    gpointer data);
 static char *                              get_name_location_for                  (int relation_number,
 										   char *name_text);
 static char *                              get_content_location_for               (int relation_number,
@@ -212,7 +267,7 @@ nautilus_search_bar_criterion_destroy (GtkObject *object)
 	criterion = NAUTILUS_SEARCH_BAR_CRITERION (object);
 
 	/* FIXME bugzilla.eazel.com 2437: need more freeage */
-	gtk_signal_disconnect_by_data (GTK_OBJECT (nautilus_signaller_get_current ()),
+	gtk_signal_disconnect_by_data (nautilus_signaller_get_current (),
 				       criterion);
 	/*	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (criterion->details->value_entry), FALSE);
 	nautilus_undo_tear_down_nautilus_entry_for_undo (criterion->details->value_entry);
@@ -236,6 +291,18 @@ nautilus_search_bar_criterion_initialize (NautilusSearchBarCriterion *criterion)
 	criterion->details = g_new0 (NautilusSearchBarCriterionDetails, 1);
 }
 
+static void
+queue_bar_resize_callback (GtkObject *wrap_box,
+			   gpointer bar)
+{
+	NautilusComplexSearchBar *complex_bar;
+
+	complex_bar = NAUTILUS_COMPLEX_SEARCH_BAR (bar);
+	if (GTK_WIDGET_VISIBLE (wrap_box)) {
+		nautilus_complex_search_bar_queue_resize (complex_bar);
+	}
+}
+
 static NautilusSearchBarCriterion *
 nautilus_search_bar_criterion_new_from_values (NautilusSearchBarCriterionType type,
 					       char *relation_options[],
@@ -247,6 +314,7 @@ nautilus_search_bar_criterion_new_from_values (NautilusSearchBarCriterionType ty
 					       NautilusComplexSearchBar *bar)
 {
 	NautilusSearchBarCriterion *criterion;
+	NautilusSearchBarCriterionDetails *details;
 	GtkWidget *search_criteria_option_menu, *search_criteria_menu; 
 	GtkWidget *relation_option_menu, *relation_menu;
 	GtkWidget *value_option_menu, *value_menu; 
@@ -257,39 +325,50 @@ nautilus_search_bar_criterion_new_from_values (NautilusSearchBarCriterionType ty
 
 	criterion = NAUTILUS_SEARCH_BAR_CRITERION (nautilus_search_bar_criterion_new ());
 	nautilus_search_bar_criterion_initialize (criterion);
-	criterion->details->type = type;
-	criterion->details->bar = bar;
+	details = criterion->details;
 
-	gtk_signal_connect (GTK_OBJECT (nautilus_signaller_get_current ()),
+	details->type = type;
+	details->bar = bar;
+	details->box = gtk_hwrap_box_new (FALSE);
+
+	gtk_signal_connect (GTK_OBJECT (details->box),
+			    "need_reallocation",
+			    queue_bar_resize_callback,
+			    bar);
+	gtk_signal_connect (nautilus_signaller_get_current (),
 			    "emblems_changed",
 			    emblems_changed_callback,
 			    (gpointer) criterion);
-				       
+	
 
 	search_criteria_option_menu = gtk_option_menu_new ();
 	search_criteria_menu = gtk_menu_new ();
-	/* Should we even be making the menu here? */
+
 	for (i = 0; criteria_titles[i] != NULL; i++) {
 		GtkWidget *item;
-
+		
 		item = gtk_menu_item_new_with_label (_(criteria_titles[i]));
 
 		gtk_object_set_data (GTK_OBJECT(item), "type", GINT_TO_POINTER(i));
-
 		gtk_signal_connect (GTK_OBJECT (item),
 				    "activate",
 				    criterion_type_changed_callback,
 				    (gpointer) criterion);
 		gtk_menu_append (GTK_MENU (search_criteria_menu),
 				 item);
+		gtk_widget_show (item);
 	}
-
-
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (search_criteria_option_menu),
 				  search_criteria_menu);
-	criterion->details->available_criteria = GTK_OPTION_MENU (search_criteria_option_menu);
-	gtk_widget_show_all (GTK_WIDGET(search_criteria_option_menu));
-
+	details->available_criteria = GTK_OPTION_MENU (search_criteria_option_menu);
+	gtk_widget_show_all (GTK_WIDGET (search_criteria_option_menu));
+	gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+			   GTK_WIDGET (details->available_criteria),
+			   FALSE,
+			   FALSE,
+			   FALSE,
+			   FALSE);
+	
 	relation_option_menu = gtk_option_menu_new ();
 	relation_menu = gtk_menu_new ();
 	for (i = 0; relation_options[i] != NULL; i++) {
@@ -301,57 +380,109 @@ nautilus_search_bar_criterion_new_from_values (NautilusSearchBarCriterionType ty
 		} else {
 			item = gtk_menu_item_new_with_label (_(relation_options[i]));
 		}
+		gtk_widget_show (item);
 		gtk_object_set_data (GTK_OBJECT(item), "type", GINT_TO_POINTER(i));
+		/* Callback to desensitize the date widget for menu items that
+		   don't need a date, like "yesterday" */
+		if (details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION) {
+			if (modified_relation_should_show_value (i)) {
+				gtk_signal_connect (GTK_OBJECT (item), "activate",
+						    show_date_widget,
+						    criterion);
+			}
+			else {
+				gtk_signal_connect (GTK_OBJECT (item), "activate",
+						    hide_date_widget,
+						    criterion);
+			}
+		}
 		gtk_menu_append (GTK_MENU (relation_menu),
 				 item);
 	}
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (relation_option_menu),
 				  relation_menu);
-	criterion->details->relation_menu = GTK_OPTION_MENU (relation_option_menu);
+	details->relation_menu = GTK_OPTION_MENU (relation_option_menu);
 	gtk_widget_show_all (GTK_WIDGET(relation_option_menu));
-
-
+	gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+			   GTK_WIDGET (details->relation_menu),
+			   FALSE,
+			   FALSE,
+			   FALSE,
+			   FALSE);
 
 	g_assert (! (use_value_entry && use_value_menu));
-	criterion->details->use_value_entry = use_value_entry;
+	details->use_value_entry = use_value_entry;
 	if (use_value_entry) {
-		criterion->details->value_entry = NAUTILUS_ENTRY (nautilus_entry_new ());
-		gtk_widget_show_all(GTK_WIDGET(criterion->details->value_entry));
-		nautilus_undo_set_up_nautilus_entry_for_undo (criterion->details->value_entry);
-		nautilus_undo_editable_set_undo_key (GTK_EDITABLE (criterion->details->value_entry), TRUE);
+		details->value_entry = NAUTILUS_ENTRY (nautilus_entry_new ());
+		gtk_widget_show_all(GTK_WIDGET(details->value_entry));
+		gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+				   GTK_WIDGET (details->value_entry),
+				   FALSE,
+				   FALSE,
+				   FALSE,
+				   FALSE);
+
+		nautilus_undo_set_up_nautilus_entry_for_undo (details->value_entry);
+		nautilus_undo_editable_set_undo_key (GTK_EDITABLE (details->value_entry), TRUE);
 	}
-	criterion->details->use_value_menu = use_value_menu;
-	if (use_value_menu && criterion->details->type != NAUTILUS_EMBLEM_SEARCH_CRITERION) {
+	details->use_value_menu = use_value_menu;
+	if (use_value_menu && details->type != NAUTILUS_EMBLEM_SEARCH_CRITERION) {
 		g_return_val_if_fail (value_options != NULL, NULL);
 		value_option_menu = gtk_option_menu_new ();
 		value_menu = gtk_menu_new ();
 		for (i = 0; value_options[i] != NULL; i++) {
 			GtkWidget *item;
 			item = gtk_menu_item_new_with_label (_(value_options[i]));
+			gtk_widget_show (item);
 			gtk_object_set_data (GTK_OBJECT(item), "type", GINT_TO_POINTER(i));
 			gtk_menu_append (GTK_MENU (value_menu),
 					 item);
 		}
 		gtk_option_menu_set_menu (GTK_OPTION_MENU (value_option_menu),
 					  value_menu);
-		criterion->details->value_menu = GTK_OPTION_MENU (value_option_menu);
-		gtk_widget_show_all (GTK_WIDGET(criterion->details->value_menu));
+		details->value_menu = GTK_OPTION_MENU (value_option_menu);
+		gtk_widget_show_all (GTK_WIDGET(details->value_menu));
+		gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+				   GTK_WIDGET (details->value_menu),
+				   FALSE,
+				   FALSE,
+				   FALSE,
+				   FALSE);
 	}
-	criterion->details->use_value_suffix = use_value_suffix;
+	details->use_value_suffix = use_value_suffix;
 	if (use_value_suffix) {
 		g_return_val_if_fail (value_suffix != NULL, NULL);
-		criterion->details->value_suffix = GTK_LABEL (gtk_label_new (value_suffix));
-		gtk_widget_show_all (GTK_WIDGET (criterion->details->value_suffix));
+		details->value_suffix = GTK_LABEL (gtk_label_new (value_suffix));
+		gtk_widget_show_all (GTK_WIDGET (details->value_suffix));
+		gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+				   GTK_WIDGET (details->value_suffix),
+				   FALSE,
+				   FALSE,
+				   FALSE,
+				   FALSE);
 	}
 	/* Special case widgets here */
-	if (criterion->details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION) {
-		criterion->details->date = GNOME_DATE_EDIT (gnome_date_edit_new (time (NULL), FALSE, FALSE));
-		gtk_widget_show_all (GTK_WIDGET (criterion->details->date));
+	if (details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION) {
+		details->date = GNOME_DATE_EDIT (gnome_date_edit_new (time (NULL), FALSE, FALSE));
+		gtk_widget_show_all (GTK_WIDGET (details->date));
+		gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+				   GTK_WIDGET (details->date),
+				   FALSE,
+				   FALSE,
+				   FALSE,
+				   FALSE);
 	}
-	if (criterion->details->type == NAUTILUS_EMBLEM_SEARCH_CRITERION) {
+	if (details->type == NAUTILUS_EMBLEM_SEARCH_CRITERION) {
 		value_option_menu = gtk_option_menu_new ();
-		criterion->details->value_menu = GTK_OPTION_MENU (value_option_menu);
+		details->value_menu = GTK_OPTION_MENU (value_option_menu);
 		make_emblem_value_menu (criterion);
+		gtk_wrap_box_pack (GTK_WRAP_BOX (details->box),
+				   GTK_WIDGET (details->value_menu),
+				   FALSE,
+				   FALSE,
+				   FALSE,
+				   FALSE);
+
 	}
 
 	return criterion;
@@ -453,8 +584,10 @@ nautilus_search_bar_criterion_new_with_type (NautilusSearchBarCriterionType crit
 		new_criterion = NULL;
 		g_assert_not_reached ();
 	}
-	
-		
+	if (new_criterion->details->value_entry) {
+		nautilus_complex_search_bar_set_up_enclosed_entry_for_clipboard 
+			(bar, new_criterion->details->value_entry);
+	}
 	return new_criterion;
 }
 
@@ -582,6 +715,7 @@ nautilus_search_bar_criterion_show (NautilusSearchBarCriterion *criterion)
 	if (criterion->details->use_value_menu) {
 		gtk_widget_show (GTK_WIDGET (criterion->details->value_menu));
 	}
+	gtk_widget_show (criterion->details->box);
 }
 
 
@@ -599,6 +733,7 @@ nautilus_search_bar_criterion_hide (NautilusSearchBarCriterion *criterion)
 	if (criterion->details->use_value_menu) {
 		gtk_widget_hide (GTK_WIDGET (criterion->details->value_menu));
 	}
+	gtk_widget_hide (criterion->details->box);
 }
 
 void                              
@@ -607,7 +742,7 @@ nautilus_search_bar_criterion_update_valid_criteria_choices (NautilusSearchBarCr
 {
 	GtkWidget *old_menu, *new_menu;
 	GtkWidget *item;
-	int i;
+	guint i;
 
 	/* We remove the whole menu and put in a new one. */
 	new_menu = gtk_menu_new ();
@@ -623,6 +758,7 @@ nautilus_search_bar_criterion_update_valid_criteria_choices (NautilusSearchBarCr
 				    (gpointer) criterion);
 		gtk_menu_append (GTK_MENU (new_menu),
 				 item);
+		gtk_widget_show (item);
 		if (i == criterion->details->type) {
 			gtk_menu_set_active (GTK_MENU (new_menu), i);
 		}
@@ -675,6 +811,40 @@ nautilus_search_uri_get_first_criterion (const char *search_uri)
 	return first_criterion;
 }
 
+static gboolean                     
+modified_relation_should_show_value (int relation_index)
+{
+	return modified_relation_shows_date[relation_index];
+}
+
+static void
+hide_date_widget (GtkObject *object,
+		  gpointer data)
+{
+	NautilusSearchBarCriterion *criterion;
+	NautilusSearchBarCriterionDetails *details;
+
+	criterion = NAUTILUS_SEARCH_BAR_CRITERION (data);
+	details = criterion->details;
+
+	gtk_widget_hide (GTK_WIDGET (details->date));
+	nautilus_complex_search_bar_queue_resize (details->bar);
+}
+
+static void
+show_date_widget (GtkObject *object,
+		  gpointer data)
+{
+	NautilusSearchBarCriterion *criterion;
+	NautilusSearchBarCriterionDetails *details;
+
+	criterion = NAUTILUS_SEARCH_BAR_CRITERION (data);
+	details = criterion->details;
+
+	gtk_widget_show (GTK_WIDGET (details->date));
+	nautilus_complex_search_bar_queue_resize (details->bar);
+}
+
 
 static char *                              
 get_name_location_for (int relation_number, char *name_text)
@@ -736,16 +906,24 @@ get_size_location_for (int relation_number,
 {
 	const char *possible_relations[] = { "larger_than", "smaller_than" };
 	int entered_size;
+	gboolean int_conversion_success;
 	
 	g_assert (relation_number == 0 || relation_number == 1);
 	/* We put a 'K' after the size, so multiply what the user
 	   entered by 1000 */
-	entered_size = strtol (size_text, NULL, 10);
-	/* FIXME bugzilla.eazel.com 2438:  Need error handling here */
-	g_return_val_if_fail (entered_size >= 0, NULL);
-	return g_strdup_printf ("%s %s %d", NAUTILUS_SEARCH_URI_TEXT_SIZE, 
-				possible_relations[relation_number], 
-				entered_size * 1000);
+	int_conversion_success = nautilus_str_to_int (size_text, 
+						      &entered_size);
+	
+	if (int_conversion_success) {
+		return g_strdup_printf ("%s %s %d", NAUTILUS_SEARCH_URI_TEXT_SIZE, 
+					possible_relations[relation_number], 
+					entered_size * 1024);
+	}
+	else {
+		return g_strdup_printf ("%s %s %s", NAUTILUS_SEARCH_URI_TEXT_SIZE, 
+					possible_relations[relation_number], 
+					size_text);
+	}
 
 }
 
@@ -816,7 +994,7 @@ static char *
 get_owner_location_for (int relation_number,
 			char *owner_text)
 {
-	const char *possible_relations[] = { "is", "is not" };
+	const char *possible_relations[] = { "is", "is_not" };
 	g_assert (relation_number == 0 || relation_number == 1);
 	return g_strdup_printf ("%s %s %s",
 				NAUTILUS_SEARCH_URI_TEXT_OWNER,
@@ -830,8 +1008,7 @@ make_emblem_value_menu (NautilusSearchBarCriterion *criterion)
 	NautilusCustomizationData *customization_data;
 	GtkWidget *temp_hbox;
 	GtkWidget *menu_item;
-	char *emblem_file_name;
-	char *emblem_display_name;
+	char *emblem_name, *dot_pos;
 	GtkWidget *emblem_pixmap_widget;
 	GtkWidget *emblem_label;
 	GtkWidget *value_menu; 
@@ -844,36 +1021,38 @@ make_emblem_value_menu (NautilusSearchBarCriterion *criterion)
 							      NAUTILUS_ICON_SIZE_FOR_MENUS, 
 							      NAUTILUS_ICON_SIZE_FOR_MENUS);
 	while (nautilus_customization_data_get_next_element_for_display (customization_data,
-									 &emblem_file_name,
+									 &emblem_name,
 									 &emblem_pixmap_widget,
 									 &emblem_label) == GNOME_VFS_OK) {
 		
+		/* remove the suffix, if any, to make the emblem name */
+		dot_pos = strrchr (emblem_name, '.');
+		if (dot_pos) {
+			*dot_pos = '\0';
+		}
 		
-		
-		gtk_label_get (GTK_LABEL (emblem_label),
-			       &emblem_display_name);
-		if (strcmp (emblem_display_name, "Erase") == 0) {
+		if (strcmp (emblem_name, "erase") == 0) {
 			gtk_widget_destroy (emblem_pixmap_widget);
 			gtk_widget_destroy (emblem_label);
+			g_free (emblem_name);
 			continue;
 		}
 		menu_item = gtk_menu_item_new ();
 		
-		gtk_object_set_data (GTK_OBJECT (menu_item), "emblem name", emblem_display_name);
-		g_assert (strcmp (emblem_display_name, "Erase") != 0);
+		gtk_object_set_data_full (GTK_OBJECT (menu_item), "emblem name",
+					  emblem_name, (GtkDestroyNotify) g_free);
 		temp_hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 		gtk_box_pack_start (GTK_BOX (temp_hbox), emblem_pixmap_widget, FALSE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (temp_hbox), emblem_label, FALSE, FALSE, 0);
 		gtk_container_add (GTK_CONTAINER (menu_item), temp_hbox);
+		gtk_widget_show_all (menu_item);
 		gtk_menu_append (GTK_MENU (value_menu), menu_item);
 	}
 	
-		
+	gtk_widget_show_all (GTK_WIDGET (criterion->details->value_menu));		
 	gtk_option_menu_set_menu (criterion->details->value_menu,
 				  value_menu);
-	gtk_widget_show_all (GTK_WIDGET (criterion->details->value_menu));
 	criterion->details->use_value_menu = TRUE;
-	g_free (emblem_file_name);
 }
 
 
