@@ -39,6 +39,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
@@ -59,8 +60,10 @@ static void nautilus_shell_initialize_class (NautilusShellClass     *klass);
 static void destroy                         (GtkObject              *shell);
 static void corba_open_windows              (PortableServer_Servant  servant,
 					     const Nautilus_URIList *list,
+					     const CORBA_char       *geometry,
 					     CORBA_Environment      *ev);
 static void corba_open_default_window       (PortableServer_Servant  servant,
+					     const CORBA_char       *geometry,
 					     CORBA_Environment      *ev);
 static void corba_start_desktop             (PortableServer_Servant  servant,
 					     CORBA_Environment      *ev);
@@ -72,6 +75,10 @@ static void corba_restart                    (PortableServer_Servant  servant,
 					     CORBA_Environment      *ev);
 static gboolean restore_window_states	    (NautilusShell *shell);
 
+/* Keep window from shrinking down ridiculously small; numbers are somewhat arbitrary */
+#define APPLICATION_WINDOW_MIN_WIDTH	300
+#define APPLICATION_WINDOW_MIN_HEIGHT	100
+
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusShell, nautilus_shell, BONOBO_OBJECT_TYPE)
 
 static void
@@ -169,6 +176,7 @@ display_caveat (GtkWindow *parent_window)
 				   NULL);
   	gtk_container_set_border_width (GTK_CONTAINER (dialog), GNOME_PAD);
   	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), "caveat", "Nautilus");
 
   	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
   	gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD);
@@ -200,17 +208,18 @@ display_caveat (GtkWindow *parent_window)
 	}
 
   	text = nautilus_label_new
-		(_("Thank you for trying out Nautilus Preview Release 2.\n"
+		(_("Thank you for your interest in Nautilus.\n "
 		   "\n"
-		   "Nautilus is still under construction. Some operations may not "
-		   "have their intended consequences, so be careful and make "
-		   "sure any important files on your system are backed up."
+		   "As with any software under development, you should exercise caution when "
+		   "using Nautilus.  Eazel does not provide any guarantee that it will work "
+		   "properly, or assume any liability for your use of it.  Please use it at your "
+		   "own risk.\n"
 		   "\n"
-		   "Please visit http://www.eazel.com/feedback.html to provide feedback."));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (text), 14);
-	nautilus_label_set_text_justification (NAUTILUS_LABEL (text), GTK_JUSTIFY_LEFT);
-	nautilus_label_set_line_wrap (NAUTILUS_LABEL (text), TRUE);
-	nautilus_label_set_line_wrap_width (NAUTILUS_LABEL (text), 300);
+		   "Please visit http://www.eazel.com/feedback.html to provide feedback, "
+		   "comments, and suggestions."));
+	nautilus_label_make_larger (NAUTILUS_LABEL (text), 1);
+	nautilus_label_set_justify (NAUTILUS_LABEL (text), GTK_JUSTIFY_LEFT);
+	nautilus_label_set_wrap (NAUTILUS_LABEL (text), TRUE);
 	gtk_widget_show (text);
   	gtk_box_pack_start (GTK_BOX (hbox), text, FALSE, FALSE, 0);
 
@@ -240,11 +249,19 @@ display_caveat_first_time (NautilusShell *shell, NautilusWindow *window)
 }
 
 static void
-open_window (NautilusShell *shell, const char *uri)
+open_window (NautilusShell *shell, const char *uri, const char *geometry)
 {
 	NautilusWindow *window;
 
 	window = nautilus_application_create_window (shell->details->application);
+
+	if (geometry != NULL) {
+		nautilus_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
+								      geometry,
+								      APPLICATION_WINDOW_MIN_WIDTH,
+								      APPLICATION_WINDOW_MIN_HEIGHT);
+	}
+
 	if (uri == NULL) {
 		nautilus_window_go_home (window);
 	} else {
@@ -256,31 +273,33 @@ open_window (NautilusShell *shell, const char *uri)
 static void
 corba_open_windows (PortableServer_Servant servant,
 		    const Nautilus_URIList *list,
+		    const CORBA_char *geometry,
 		    CORBA_Environment *ev)
 {
 	NautilusShell *shell;
-	int i;
+	guint i;
 
 	shell = NAUTILUS_SHELL (((BonoboObjectServant *) servant)->bonobo_object);
 
 	/* Open windows at each requested location. */
 	for (i = 0; i < list->_length; i++) {
 		g_assert (list->_buffer[i] != NULL);
-		open_window (shell, list->_buffer[i]);
+		open_window (shell, list->_buffer[i], geometry);
 	}
 }
 
 static void
 corba_open_default_window (PortableServer_Servant servant,
+			   const CORBA_char *geometry,
 			   CORBA_Environment *ev)
 {
 	NautilusShell *shell;
 
 	shell = NAUTILUS_SHELL (((BonoboObjectServant *) servant)->bonobo_object);
 
-	if (! restore_window_states (shell)) {
+	if (!restore_window_states (shell)) {
 		/* Open a window pointing at the default location. */
-		open_window (shell, NULL);
+		open_window (shell, NULL, geometry);
 	}
 }
 
