@@ -36,6 +36,7 @@
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-scalable-font.h>
+#include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-theme.h>
 #include <math.h>
 #include <stdio.h>
@@ -181,11 +182,17 @@ nautilus_sidebar_tabs_load_theme_data (NautilusSidebarTabs *sidebar_tabs)
 	/* load the tab_pieces image if necessary */
 	tab_pieces = nautilus_theme_get_theme_data ("sidebar", "TAB_PIECE_IMAGES");
 	tab_piece_theme = nautilus_theme_get_theme_data ("sidebar", "TAB_PIECE_THEME");
+	
 	if (tab_pieces) {
-		if (tab_piece_theme) {
-			tab_piece_path = nautilus_theme_get_image_path_from_theme (tab_pieces, tab_piece_theme);		
-		} else {
-			tab_piece_path = nautilus_theme_get_image_path (tab_pieces);
+		/* check for "none" to force non-bitmap tabs, necessary since the default has bitmap ones now */
+		if (nautilus_strcmp (tab_pieces, "none") == 0) {
+			tab_piece_path = NULL;	
+		}  else {
+			if (tab_piece_theme) {
+				tab_piece_path = nautilus_theme_get_image_path_from_theme (tab_pieces, tab_piece_theme);		
+			} else {
+				tab_piece_path = nautilus_theme_get_image_path (tab_pieces);
+			}
 		}
 		g_free (tab_pieces);
 		g_free (tab_piece_theme);
@@ -220,9 +227,13 @@ nautilus_sidebar_tabs_load_theme_data (NautilusSidebarTabs *sidebar_tabs)
 	if (sidebar_tabs->details->tab_font) {
 		gtk_object_unref (GTK_OBJECT (sidebar_tabs->details->tab_font));
 	}
+
+	/* FIXME bugzilla.eazel.com 5456: 
+	 * Hard coded font family and size.
+	 */
 	
 	/* use the default font.  In the future, it should fetch the font name and properties from the theme */
-	sidebar_tabs->details->tab_font = NAUTILUS_SCALABLE_FONT (nautilus_scalable_font_new ("helvetica", "bold", NULL, NULL));
+	sidebar_tabs->details->tab_font = nautilus_scalable_font_new ("helvetica", "bold", NULL, NULL);
 	sidebar_tabs->details->font_size = 12;
 }
 
@@ -467,33 +478,6 @@ nautilus_sidebar_tabs_size_allocate(GtkWidget *widget, GtkAllocation *allocation
     }
 }
 
-/* utility to allocate a gdk_pixbuf and fill it with a specified color */
-
-static GdkPixbuf*
-make_colored_pixbuf (int pixbuf_width, int pixbuf_height, GdkColor *color)
-{
-	GdkPixbuf *pixbuf;
-	int row, col, stride;
-	char *pixels, *row_pixels;
-	
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, pixbuf_width, pixbuf_height);
-	
-	pixels = gdk_pixbuf_get_pixels (pixbuf);
-	stride = gdk_pixbuf_get_rowstride (pixbuf);
-	
-	/* loop through and set each pixel */
-	for (row = 0; row < pixbuf_height; row++) {
-		row_pixels =  (pixels + (row * stride));
-		for (col = 0; col < pixbuf_width; col++) {		
-			*row_pixels++ = color->red;
-			*row_pixels++ = color->green;
-			*row_pixels++ = color->blue;
-			*row_pixels++ = 255;
-		}
-	}
-	return pixbuf;
-}
-
 /* draw a single tab using the default, non-themed approach */
 static int
 draw_one_tab_plain (NautilusSidebarTabs *sidebar_tabs, GdkGC *gc,
@@ -505,6 +489,7 @@ draw_one_tab_plain (NautilusSidebarTabs *sidebar_tabs, GdkGC *gc,
 	int		total_width;
 	GtkWidget	*widget;
 	GdkPixbuf	*temp_pixbuf;
+	GdkColor	*foreground_color;
 	
 	g_assert (NAUTILUS_IS_SIDEBAR_TABS (sidebar_tabs));
 
@@ -532,9 +517,11 @@ draw_one_tab_plain (NautilusSidebarTabs *sidebar_tabs, GdkGC *gc,
 	/* FIXME bugzilla.eazel.com 668: 
 	 * we must "ellipsize" the name if it doesn't fit, for now, assume it does 
 	 */
+	 
+	foreground_color = prelight_flag ? &sidebar_tabs->details->prelight_color : &sidebar_tabs->details->tab_color;
 		
 	/* fill the tab rectangle with the tab color */
-	gdk_gc_set_foreground (gc, prelight_flag ? &sidebar_tabs->details->prelight_color : &sidebar_tabs->details->tab_color);
+	gdk_gc_set_foreground (gc, foreground_color);
 	gdk_draw_rectangle (widget->window, gc, TRUE, x, y + 1, total_width, sidebar_tabs->details->tab_height - 1); 
 	
 
@@ -550,7 +537,8 @@ draw_one_tab_plain (NautilusSidebarTabs *sidebar_tabs, GdkGC *gc,
 	gdk_draw_line(widget->window, gc, x + 2, y + 2, x + 2, y + sidebar_tabs->details->tab_height - 1);
 	
 	/* allocate the pixbuf and fill it with the background color */
-	temp_pixbuf = make_colored_pixbuf (name_width + 1, name_height + 1, prelight_flag ? &sidebar_tabs->details->prelight_color : &sidebar_tabs->details->tab_color);
+	temp_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, name_width + 1, name_height + 1);
+	nautilus_gdk_pixbuf_fill_rectangle_with_color (temp_pixbuf, NULL, nautilus_gdk_color_to_rgb (foreground_color));
 	
 	/* draw the name into the pixbuf using anti-aliased text */
 	nautilus_scalable_font_draw_text (sidebar_tabs->details->tab_font, temp_pixbuf, 
@@ -558,14 +546,16 @@ draw_one_tab_plain (NautilusSidebarTabs *sidebar_tabs, GdkGC *gc,
 					  NULL,
 					  sidebar_tabs->details->font_size, sidebar_tabs->details->font_size,
 					  tab_name, strlen (tab_name),
-					  prelight_flag ? NAUTILUS_RGB_COLOR_WHITE : NAUTILUS_RGB_COLOR_BLACK, 255, FALSE);
+					  prelight_flag ? NAUTILUS_RGB_COLOR_WHITE : NAUTILUS_RGB_COLOR_BLACK,
+					  NAUTILUS_OPACITY_FULLY_OPAQUE);
 	
 	nautilus_scalable_font_draw_text (sidebar_tabs->details->tab_font, temp_pixbuf, 
 					  0, 0,
 					  NULL,
 					  sidebar_tabs->details->font_size, sidebar_tabs->details->font_size,
 					  tab_name, strlen (tab_name),
-					  prelight_flag ? NAUTILUS_RGB_COLOR_BLACK : NAUTILUS_RGB_COLOR_WHITE, 255, FALSE);
+					  prelight_flag ? NAUTILUS_RGB_COLOR_BLACK : NAUTILUS_RGB_COLOR_WHITE,
+					  NAUTILUS_OPACITY_FULLY_OPAQUE);
 	
 	/* blit the pixbuf to the drawable, then release it */
 
@@ -707,22 +697,27 @@ draw_one_tab_themed (NautilusSidebarTabs *sidebar_tabs, GdkPixbuf *tab_pixbuf,
 
 	text_x = text_x_pos + 1;
 	text_y = y - widget->allocation.y + THEMED_TAB_TEXT_V_OFFSET;
-		
-	nautilus_scalable_font_draw_text (sidebar_tabs->details->tab_font, tab_pixbuf, 
+	
+	/* make sure we can at least draw some of it */
+	if (text_x < gdk_pixbuf_get_width (tab_pixbuf)) {	
+		nautilus_scalable_font_draw_text (sidebar_tabs->details->tab_font, tab_pixbuf, 
 					  text_x, text_y,
 					  NULL,
 					  sidebar_tabs->details->font_size, sidebar_tabs->details->font_size,
 					  tab_name, strlen (tab_name),
-					  NAUTILUS_RGB_COLOR_BLACK, 255, FALSE);
-	text_x -= 1;
-	text_y -= 1;
+					  NAUTILUS_RGB_COLOR_BLACK,
+					  NAUTILUS_OPACITY_FULLY_OPAQUE);
+		text_x -= 1;
+		text_y -= 1;
 
-	nautilus_scalable_font_draw_text (sidebar_tabs->details->tab_font, tab_pixbuf,
+		nautilus_scalable_font_draw_text (sidebar_tabs->details->tab_font, tab_pixbuf,
 					  text_x, text_y,
 					  NULL,
 					  sidebar_tabs->details->font_size, sidebar_tabs->details->font_size,
 					  tab_name, strlen (tab_name),
-					  NAUTILUS_RGB_COLOR_WHITE, 255, FALSE);
+					  NAUTILUS_RGB_COLOR_WHITE,
+					  NAUTILUS_OPACITY_FULLY_OPAQUE);
+	}
 	
 	/* set up the bounds rectangle for later hit-testing */
 	if (tab_rect) {
@@ -1034,12 +1029,24 @@ draw_or_layout_all_tabs (NautilusSidebarTabs *sidebar_tabs, gboolean layout_only
 					if (needs_compositing) {
 						int dest_x = last_x_pos + tab_width - widget->allocation.x;
 						int dest_y = last_y_pos - widget->allocation.y;
+						int src_width = gdk_pixbuf_get_width (temp_pixbuf);
+						int src_height = gdk_pixbuf_get_height (temp_pixbuf);
+						int dest_width = gdk_pixbuf_get_width (tab_pixbuf);
+						int dest_height = gdk_pixbuf_get_height (tab_pixbuf);
+						
+						/* clip it in both dimensions */
+						if ((dest_x + src_width) > dest_width) {
+							src_width = dest_width - dest_x;
+						}
+						if ((dest_y + src_height) > dest_height) {
+							src_height = dest_height - dest_y;
+						}
 						
 						gdk_pixbuf_composite (temp_pixbuf, tab_pixbuf,
 							dest_x,
 							dest_y,
-							gdk_pixbuf_get_width (temp_pixbuf),
-							gdk_pixbuf_get_height (temp_pixbuf),
+							src_width,
+							src_height,
 							dest_x,
 							dest_y,
 							1.0, 1.0,
