@@ -27,6 +27,7 @@
 
 #include <config.h>
 #include "nautilus-sidebar-title.h"
+#include "nautilus-sidebar.h"
 
 #include <ctype.h>
 #include <string.h>
@@ -40,6 +41,7 @@
 #include <libnautilus-extensions/nautilus-background.h>
 #include <libnautilus-extensions/nautilus-file-attributes.h>
 #include <libnautilus-extensions/nautilus-gdk-extensions.h>
+#include <libnautilus-extensions/nautilus-gdk-font-extensions.h>
 #include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
@@ -47,47 +49,36 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-icon-factory.h>
 #include <libnautilus-extensions/nautilus-metadata.h>
+#include <libnautilus-extensions/nautilus-search-uri.h>
 #include <libnautilus-extensions/nautilus-font-factory.h>
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-theme.h>
-#include <libnautilus-extensions/nautilus-label.h>
-#include <libnautilus-extensions/nautilus-image.h>
-
-static void nautilus_sidebar_title_initialize_class    (NautilusSidebarTitleClass *klass);
-static void nautilus_sidebar_title_destroy             (GtkObject                 *object);
-static void nautilus_sidebar_title_initialize          (NautilusSidebarTitle      *pixmap);
-static void nautilus_sidebar_title_size_allocate       (GtkWidget                 *widget,
-							GtkAllocation             *allocation);
-static void nautilus_sidebar_title_theme_changed       (gpointer                   user_data);
-static void update_icon                                (NautilusSidebarTitle      *sidebar_title);
-static void sidebar_create_smooth_components_if_needed (NautilusSidebarTitle      *sidebar_title);
-static void sidebar_create_normal_components_if_needed (NautilusSidebarTitle      *sidebar_title);
-static void smooth_graphics_mode_changed_callback         (gpointer                   callback_data);
+#include <libnautilus-extensions/nautilus-label-with-background.h>
+#include <libnautilus-extensions/nautilus-image-with-background.h>
 
+static void                nautilus_sidebar_title_initialize_class (NautilusSidebarTitleClass *klass);
+static void                nautilus_sidebar_title_destroy          (GtkObject                 *object);
+static void                nautilus_sidebar_title_initialize       (NautilusSidebarTitle      *pixmap);
+static void                nautilus_sidebar_title_size_allocate    (GtkWidget                 *widget,
+								    GtkAllocation             *allocation);
+static void                nautilus_sidebar_title_theme_changed    (gpointer                   user_data);
+static void                update_icon                             (NautilusSidebarTitle      *sidebar_title);
+static GtkWidget *         sidebar_title_create_title_label        (void);
+static GtkWidget *         sidebar_title_create_more_info_label    (void);
+static void                smooth_graphics_mode_changed_callback   (gpointer                   callback_data);
+static NautilusBackground *nautilus_sidebar_title_background       (NautilusSidebarTitle      *sidebar_title);
 
 struct NautilusSidebarTitleDetails {
 	NautilusFile		*file;
 	guint			file_changed_connection;
 	char			*title_text;
-
 	GtkWidget		*icon;
-
-	GtkWidget		*normal_title_label;
-	GtkWidget		*normal_more_info_label;
-
-	GtkWidget		*label_box;
-	GtkWidget		*more_info_box;
-
-	GtkWidget		*smooth_title_label;
-	NautilusScalableFont	*smooth_title_label_font;
-	GtkWidget		*smooth_more_info_label;
-	NautilusScalableFont	*smooth_more_info_label_font;
-
+	GtkWidget		*title_label;
+	GtkWidget		*more_info_label;
 	GtkWidget		*emblem_box;
 	GtkWidget		*notes;
 
 	int			shadow_offset;
-	gboolean		smooth_graphics;
 };
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusSidebarTitle, nautilus_sidebar_title, gtk_vbox_get_type ())
@@ -105,11 +96,31 @@ nautilus_sidebar_title_initialize_class (NautilusSidebarTitleClass *class)
 	widget_class->size_allocate = nautilus_sidebar_title_size_allocate;
 }
 
+static void
+appearance_changed_callback (NautilusBackground *background, NautilusSidebarTitle *sidebar_title)
+{
+	nautilus_sidebar_title_select_text_color (sidebar_title);
+}		
+
+static void
+realize_callback (NautilusSidebarTitle *sidebar_title)
+{
+	NautilusBackground *background;
+	
+	background = nautilus_sidebar_title_background (sidebar_title);
+
+	g_return_if_fail (background != NULL);
+
+	gtk_signal_connect_while_alive (GTK_OBJECT (background),
+					"appearance_changed",
+					appearance_changed_callback,
+					sidebar_title,
+					GTK_OBJECT (sidebar_title));
+}
+
 static void
 nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 { 
-	GdkFont *font;
-
 	sidebar_title->details = g_new0 (NautilusSidebarTitleDetails, 1);
 
 	/* Register to find out about icon theme changes */
@@ -117,19 +128,22 @@ nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 					       "icons_changed",
 					       update_icon,
 					       GTK_OBJECT (sidebar_title));
+	gtk_signal_connect (GTK_OBJECT (sidebar_title), "realize", realize_callback, NULL);
 
-	sidebar_title->details->label_box = gtk_vbox_new (FALSE, 0);
-	sidebar_title->details->more_info_box = gtk_vbox_new (FALSE, 0);
-
-	sidebar_title->details->icon = nautilus_image_new ();
-
+	/* Create the icon */
+	sidebar_title->details->icon = nautilus_image_new_with_background (NULL);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->icon, 0, 0, 0);
-	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->label_box, 0, 0, 0);
-	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->more_info_box, 0, 0, 3);
-
 	gtk_widget_show (sidebar_title->details->icon);
-	gtk_widget_show (sidebar_title->details->label_box);
-	gtk_widget_show (sidebar_title->details->more_info_box);
+
+	/* Create the title label */
+	sidebar_title->details->title_label = sidebar_title_create_title_label ();
+	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->title_label, 0, 0, 0);
+	gtk_widget_show (sidebar_title->details->title_label);
+
+	/* Create the more info label */
+	sidebar_title->details->more_info_label = sidebar_title_create_more_info_label ();
+	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->more_info_label, 0, 0, 0);
+	gtk_widget_show (sidebar_title->details->more_info_label);
 
 	sidebar_title->details->emblem_box = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (sidebar_title->details->emblem_box);
@@ -137,9 +151,6 @@ nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 
 	sidebar_title->details->notes = GTK_WIDGET (gtk_label_new (NULL));
 	gtk_label_set_line_wrap (GTK_LABEL (sidebar_title->details->notes), TRUE);
-	font = nautilus_font_factory_get_font_from_preferences (12);
-	nautilus_gtk_widget_set_font (sidebar_title->details->notes, font);
-	gdk_font_unref (font);
 	gtk_widget_show (sidebar_title->details->notes);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->notes, 0, 0, 0);
 
@@ -199,27 +210,55 @@ nautilus_sidebar_title_new (void)
 	return gtk_widget_new (nautilus_sidebar_title_get_type (), NULL);
 }
 
-/* utility to set up the style of a widget to have a particular color */
-static void
-set_widget_color (GtkWidget *widget, const char* color_spec)
+static NautilusBackground *
+nautilus_sidebar_title_background (NautilusSidebarTitle *sidebar_title)
 {
-	GtkStyle *style;
-	GdkColor color;
-	
-	style = gtk_widget_get_style (widget);
-	
-	/* Make a copy of the style. */
-	style = gtk_style_copy (style);
+	GtkWidget *widget;
+	GtkWidget *sidebar;
+	NautilusBackground *background;
 
-	nautilus_gdk_color_parse_with_white_default (color_spec, &color);
-	style->fg[GTK_STATE_NORMAL] = color;
-	style->base[GTK_STATE_NORMAL] = color;
-	style->fg[GTK_STATE_ACTIVE] = color;
-	style->base[GTK_STATE_ACTIVE] = color;
+	g_return_val_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (sidebar_title), NULL);
 	
-	/* Put the style in the widget. */
-	gtk_widget_set_style (widget, style);
-	gtk_style_unref (style);
+	widget = GTK_WIDGET (sidebar_title)->parent;
+
+	 
+	if (widget != NULL) {
+		sidebar = widget->parent;
+		g_return_val_if_fail (NAUTILUS_IS_SIDEBAR (sidebar), NULL);
+		background = nautilus_get_widget_background (sidebar);
+		g_return_val_if_fail (NAUTILUS_IS_BACKGROUND (background), NULL);
+		return background;
+	} else {
+		/* FIXME bugzilla.eazel.com 5042
+		 * It would be preferable to assert widget != NULL and not have
+		 * this else case. Doing this would require us to be carful when
+		 * nautilus_sidebar_title_select_text_color is called - which would
+		 * probably involve doing more of the sidebar_title initialization
+		 * at realize time.
+		 */
+		return NULL;
+	}
+}
+
+/* utility to determine if the sidebar is using the default theme */
+static gboolean
+nautilus_sidebar_title_background_is_default (NautilusSidebarTitle *sidebar_title)
+{
+	char *background_color, *background_image;
+	gboolean is_default;
+	
+	background_color = nautilus_file_get_metadata (sidebar_title->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+						       NULL);
+	background_image = nautilus_file_get_metadata (sidebar_title->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+						       NULL);
+	
+	is_default = background_color == NULL && background_image == NULL;
+	g_free (background_color);
+	g_free (background_image);
+	
+	return is_default;
 }
 
 /* utility that returns true if the title is over a dark background.  We do this by finding the
@@ -227,54 +266,65 @@ set_widget_color (GtkWidget *widget, const char* color_spec)
 void
 nautilus_sidebar_title_select_text_color (NautilusSidebarTitle *sidebar_title)
 {
-	GtkWidget *widget, *sidebar;
 	NautilusBackground *background;
-	char *sidebar_title_color, *sidebar_info_title_color;
+	char *sidebar_title_color;
+	char *sidebar_info_title_color;
 	char *sidebar_title_shadow_color;
 	
-	widget = GTK_WIDGET (sidebar_title);
-	sidebar = widget->parent;
-	if (sidebar) {
-		sidebar = GTK_WIDGET (sidebar)->parent;
-		background = nautilus_get_widget_background (sidebar);
-		
-		/* FIXME bugzilla.eazel.com 2496: for now, both the title and info colors are the same */
-		if (nautilus_background_is_dark (background)) {
-			sidebar_title_color = g_strdup("rgb:FFFF/FFFF/FFFF");
-			sidebar_info_title_color = g_strdup("rgb:FFFF/FFFF/FFFF");
-			sidebar_title_shadow_color = g_strdup("rgb:0000/0000/0000");
-		
-		} else {
-			sidebar_title_color = g_strdup("rgb:0000/0000/0000");
-			sidebar_info_title_color = g_strdup("rgb:0000/0000/0000");
-			sidebar_title_shadow_color = g_strdup("rgb:FFFF/FFFF/FFFF");
-		}
-
-		if (sidebar_title->details->smooth_graphics) {
-			nautilus_label_set_text_color (NAUTILUS_LABEL (sidebar_title->details->smooth_title_label),
-						       nautilus_parse_rgb_with_white_default (sidebar_title_color));
-			
-			nautilus_label_set_drop_shadow_color (NAUTILUS_LABEL (sidebar_title->details->smooth_title_label),
-						       nautilus_parse_rgb_with_white_default (sidebar_title_shadow_color));
-			
-			nautilus_label_set_drop_shadow_offset (NAUTILUS_LABEL (sidebar_title->details->smooth_title_label)
-						       ,sidebar_title->details->shadow_offset);
-
-			nautilus_label_set_text_color (NAUTILUS_LABEL (sidebar_title->details->smooth_more_info_label),
-						       nautilus_parse_rgb_with_white_default (sidebar_info_title_color));
-			
-			nautilus_label_set_drop_shadow_color (NAUTILUS_LABEL (sidebar_title->details->smooth_more_info_label),
-						       nautilus_parse_rgb_with_white_default (sidebar_title_shadow_color));
-			
-			nautilus_label_set_drop_shadow_offset (NAUTILUS_LABEL (sidebar_title->details->smooth_more_info_label)
-						       ,sidebar_title->details->shadow_offset);
-
-		}
-		else {
-			set_widget_color (sidebar_title->details->normal_title_label, sidebar_title_color);
-			set_widget_color (sidebar_title->details->normal_more_info_label, sidebar_info_title_color);
-		}
+	/* if the background is set to the default, the theme can explicitly define the title colors.  Check if
+	 * the background has been customized and if the theme specified any colors
+	 */
+	sidebar_title_color = NULL;
+	sidebar_info_title_color = NULL;
+	sidebar_title_shadow_color = NULL;
 	
+	background = nautilus_sidebar_title_background (sidebar_title);
+	if (background != NULL) {
+		if (nautilus_sidebar_title_background_is_default (sidebar_title)) {
+			sidebar_title_color = nautilus_theme_get_theme_data ("sidebar", "TITLE_COLOR");
+			sidebar_info_title_color = nautilus_theme_get_theme_data ("sidebar", "TITLE_INFO_COLOR");
+			sidebar_title_shadow_color = nautilus_theme_get_theme_data ("sidebar", "TITLE_SHADOW_COLOR"); 
+		}
+		
+		if (sidebar_title_color == NULL) {
+			/* FIXME bugzilla.eazel.com 2496: for now, both the title and info colors are the same */
+			if (nautilus_background_is_dark (background)) {
+				sidebar_title_color = g_strdup("rgb:FFFF/FFFF/FFFF");
+				sidebar_info_title_color = g_strdup("rgb:FFFF/FFFF/FFFF");
+				sidebar_title_shadow_color = g_strdup("rgb:0000/0000/0000");
+				
+			} else {
+				sidebar_title_color = g_strdup("rgb:0000/0000/0000");
+				sidebar_info_title_color = g_strdup("rgb:0000/0000/0000");
+				sidebar_title_shadow_color = g_strdup("rgb:FFFF/FFFF/FFFF");
+			}
+		} else {
+			if (sidebar_info_title_color == NULL) {
+				sidebar_info_title_color = g_strdup (sidebar_title_color);
+			}
+			if (sidebar_title_shadow_color == NULL) {
+				sidebar_title_shadow_color = g_strdup("rgb:FFFF/FFFF/FFFF");
+			}
+		}
+
+		nautilus_label_set_text_color (NAUTILUS_LABEL (sidebar_title->details->title_label),
+						       nautilus_parse_rgb_with_white_default (sidebar_title_color));
+		
+		nautilus_label_set_smooth_drop_shadow_color (NAUTILUS_LABEL (sidebar_title->details->title_label),
+							      nautilus_parse_rgb_with_white_default (sidebar_title_shadow_color));
+		
+		nautilus_label_set_smooth_drop_shadow_offset (NAUTILUS_LABEL (sidebar_title->details->title_label),
+							       sidebar_title->details->shadow_offset);
+		
+		nautilus_label_set_text_color (NAUTILUS_LABEL (sidebar_title->details->more_info_label),
+						       nautilus_parse_rgb_with_white_default (sidebar_info_title_color));
+		
+		nautilus_label_set_smooth_drop_shadow_color (NAUTILUS_LABEL (sidebar_title->details->more_info_label),
+							      nautilus_parse_rgb_with_white_default (sidebar_title_shadow_color));
+		
+		nautilus_label_set_smooth_drop_shadow_offset (NAUTILUS_LABEL (sidebar_title->details->more_info_label),
+							       sidebar_title->details->shadow_offset);
+		
 		g_free (sidebar_title_color);	
 		g_free (sidebar_info_title_color);	
 		g_free (sidebar_title_shadow_color);
@@ -295,7 +345,7 @@ nautilus_sidebar_title_theme_changed (gpointer user_data)
 		sidebar_title->details->shadow_offset = atoi (shadow_offset_str);	
 		g_free (shadow_offset_str);
 	} else {
-		sidebar_title->details->shadow_offset = 0;	
+		sidebar_title->details->shadow_offset = 1;	
 	}
 	
 	nautilus_sidebar_title_select_text_color (sidebar_title);
@@ -308,7 +358,7 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 	GdkPixbuf *pixbuf;
 	char *uri, *icon_path;
 	
-	/* FIXME: currently, components can't specify their own sidebar icon.  This
+	/* FIXME bugzilla.eazel.com 5043: currently, components can't specify their own sidebar icon.  This
 	  needs to be added to the framework, but for now we special-case some
 	  important ones here */
 	
@@ -320,12 +370,16 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 	
 	if (nautilus_istr_has_prefix (uri, "eazel:") || nautilus_istr_has_prefix (uri, "eazel-services:")) {
 		icon_path = nautilus_theme_get_image_path ("big_services_icon.png");
+	} else if (nautilus_istr_has_prefix (uri, "http:")) {
+		icon_path = nautilus_theme_get_image_path ("i-web-72.png");
 	} else if (nautilus_istr_has_prefix (uri, "man:")) {
 		icon_path = nautilus_theme_get_image_path ("manual.png");
+	} else if (nautilus_istr_has_prefix (uri, "hardware:")) {
+		icon_path = nautilus_theme_get_image_path ("computer.svg");
 	}
 	
 	if (icon_path != NULL) {
-		pixbuf = gdk_pixbuf_new_from_file (icon_path);
+		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_path, NULL, NAUTILUS_ICON_SIZE_LARGE, TRUE);
 	} else if (nautilus_icon_factory_is_icon_ready_for_file (sidebar_title->details->file)) {
 		pixbuf = nautilus_icon_factory_get_pixbuf_for_file (sidebar_title->details->file,
 								    "accept",
@@ -339,51 +393,43 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 	g_free (icon_path);
 	
 	nautilus_image_set_pixbuf (NAUTILUS_IMAGE (sidebar_title->details->icon), pixbuf);
-	
-	if (pixbuf == NULL) {
-		gtk_widget_hide (sidebar_title->details->icon);
-	} else {
-		gdk_pixbuf_unref (pixbuf);
-		gtk_widget_show (sidebar_title->details->icon);
-	}
 }
 
 static void
 update_font (NautilusSidebarTitle *sidebar_title)
 {
-	if (sidebar_title->details->smooth_graphics) {
-		guint largest_size;
-		
-		const guint font_sizes[4] = { 20, 18, 14, 12 };
-		
-		largest_size = nautilus_scalable_font_largest_fitting_font_size (sidebar_title->details->smooth_title_label_font,
-										 sidebar_title->details->title_text,
-										 GTK_WIDGET (sidebar_title)->allocation.width - 4,
-										 font_sizes,
-										 NAUTILUS_N_ELEMENTS (font_sizes));
-		
-		nautilus_label_set_font_size (NAUTILUS_LABEL (sidebar_title->details->smooth_title_label), largest_size);
-	}
-	else {
-		GdkFont *label_font;
-		
-		/* FIXME bugzilla.eazel.com 1103: Make this use the font
-		 * factory and be failsafe if the given font is not found.
-		 */
-		
-		/* leave 4 pixels of slop so the text doesn't go right up to the edge */
-		label_font = nautilus_get_largest_fitting_font
-			(sidebar_title->details->title_text,
-			 GTK_WIDGET (sidebar_title)->allocation.width - 4,
-			 "-adobe-helvetica-bold-r-normal-*-%d-*-*-*-*-*-*-*");
+	guint largest_size;
+	const guint font_sizes[4] = { 20, 18, 14, 12 };
+	GdkFont *label_font;
+	NautilusScalableFont *smooth_font;
 
-		if (label_font == NULL) {
-			label_font = nautilus_font_factory_get_fallback_font ();
-		}
+	smooth_font = nautilus_label_get_smooth_font (NAUTILUS_LABEL (sidebar_title->details->title_label));
+	
+	largest_size = nautilus_scalable_font_largest_fitting_font_size (smooth_font,
+									 sidebar_title->details->title_text,
+									 GTK_WIDGET (sidebar_title)->allocation.width - 4,
+									 font_sizes,
+									 NAUTILUS_N_ELEMENTS (font_sizes));
+	gtk_object_unref (GTK_OBJECT (smooth_font));
+	
+	nautilus_label_set_smooth_font_size (NAUTILUS_LABEL (sidebar_title->details->title_label), largest_size);
+	
+	/* FIXME bugzilla.eazel.com 1103: Make this use the font
+	 * factory and be failsafe if the given font is not found.
+	 */
 		
-		nautilus_gtk_widget_set_font (sidebar_title->details->normal_title_label, label_font);
-		gdk_font_unref (label_font);
+	/* leave 4 pixels of slop so the text doesn't go right up to the edge */
+	label_font = nautilus_get_largest_fitting_font
+		(sidebar_title->details->title_text,
+		 GTK_WIDGET (sidebar_title)->allocation.width - 4,
+		 "-adobe-helvetica-bold-r-normal-*-%d-*-*-*-*-*-*-*");
+	
+	if (label_font == NULL) {
+		label_font = nautilus_font_factory_get_fallback_font ();
 	}
+	
+	nautilus_gtk_widget_set_font (sidebar_title->details->title_label, label_font);
+	gdk_font_unref (label_font);
 }
 
 /* set up the filename label */
@@ -391,15 +437,8 @@ static void
 update_title (NautilusSidebarTitle *sidebar_title)
 {
 	/* FIXME bugzilla.eazel.com 2500: We could defer showing the title until the icon is ready. */
-
-	if (sidebar_title->details->smooth_graphics) {
-		nautilus_label_set_text (NAUTILUS_LABEL (sidebar_title->details->smooth_title_label),
-					 sidebar_title->details->title_text);
-	}
-	else {
-		gtk_label_set_text (GTK_LABEL (sidebar_title->details->normal_title_label),
-				    sidebar_title->details->title_text);
-	}
+	nautilus_label_set_text (NAUTILUS_LABEL (sidebar_title->details->title_label),
+				  sidebar_title->details->title_text);
 
 	update_font (sidebar_title);
 }
@@ -417,39 +456,58 @@ append_and_eat (GString *string, const char *separator, char *new_string)
 	g_free (new_string);
 }
 
+static gboolean
+file_is_search_location (NautilusFile *file)
+{
+	char *uri;
+	gboolean is_search_uri;
+
+	uri = nautilus_file_get_uri (file);
+	is_search_uri = nautilus_is_search_uri (uri);
+	g_free (uri);
+
+	return is_search_uri;
+}
+
 static void
 update_more_info (NautilusSidebarTitle *sidebar_title)
 {
 	NautilusFile *file;
 	GString *info_string;
-	char *type_string;
-
+	char *type_string, *search_string, *search_uri;
+	
 	file = sidebar_title->details->file;
 	
 	/* FIXME bugzilla.eazel.com 2500: We could defer showing info until the icon is ready. */
-
-	info_string = g_string_new (NULL);
-	type_string = nautilus_file_get_string_attribute (file, "type");
-	if (type_string != NULL) {
-		append_and_eat (info_string, NULL, type_string);
-		append_and_eat (info_string, ", ",
+	/* Adding this special case for search results to 
+	   correspond to the fix for bug 2341.  */
+	if (file != NULL && file_is_search_location (file)) {
+		search_uri = nautilus_file_get_uri (file);
+		search_string = nautilus_search_uri_to_human (search_uri);
+		g_free (search_uri);
+		info_string = g_string_new (search_string);
+		g_free (search_string);
+		append_and_eat (info_string, "\n ",
 				nautilus_file_get_string_attribute (file, "size"));
-	} else {
-		append_and_eat (info_string, NULL,
-				nautilus_file_get_string_attribute (file, "size"));
-	}
-	append_and_eat (info_string, "\n",
-			nautilus_file_get_string_attribute (file, "date_modified"));
-	g_string_append_c (info_string, '\0');
-
-	if (sidebar_title->details->smooth_graphics) {
-		nautilus_label_set_text (NAUTILUS_LABEL (sidebar_title->details->smooth_more_info_label),
-					 info_string->str);
 	}
 	else {
-		gtk_label_set_text (GTK_LABEL (sidebar_title->details->normal_more_info_label), info_string->str);
+		info_string = g_string_new (NULL);
+		type_string = nautilus_file_get_string_attribute (file, "type");
+		if (type_string != NULL) {
+			append_and_eat (info_string, NULL, type_string);
+			append_and_eat (info_string, ", ",
+					nautilus_file_get_string_attribute (file, "size"));
+		} else {
+			append_and_eat (info_string, NULL,
+					nautilus_file_get_string_attribute (file, "size"));
+		}
+		append_and_eat (info_string, "\n",
+				nautilus_file_get_string_attribute (file, "date_modified"));
+		g_string_append_c (info_string, '\0');
 	}
 
+	nautilus_label_set_text (NAUTILUS_LABEL (sidebar_title->details->more_info_label),
+				  info_string->str);
 
 	g_string_free (info_string, TRUE);
 }
@@ -460,7 +518,7 @@ add_emblem (NautilusSidebarTitle *sidebar_title, GdkPixbuf *pixbuf)
 {
 	GtkWidget *image_widget;
 
-	image_widget = nautilus_image_new ();
+	image_widget = nautilus_image_new_with_background (NULL);
 	nautilus_image_set_pixbuf (NAUTILUS_IMAGE (image_widget), pixbuf);	
   	gtk_widget_show (image_widget);
 	gtk_container_add (GTK_CONTAINER (sidebar_title->details->emblem_box), image_widget);	
@@ -473,12 +531,20 @@ update_emblems (NautilusSidebarTitle *sidebar_title)
 	GdkPixbuf *pixbuf;
 
 	/* FIXME bugzilla.eazel.com 2500: We could defer showing emblems until the icon is ready. */
-
+	/* exit if we don't have the file yet */
+	if (sidebar_title->details->file == NULL) {
+		return;
+	}
+	
 	/* First, deallocate any existing ones */
 	gtk_container_foreach (GTK_CONTAINER (sidebar_title->details->emblem_box),
 			       (GtkCallback) gtk_widget_destroy,
 			       NULL);
 
+	/* make sure we have the file */
+	if (sidebar_title->details->file) {
+	}
+	
 	/* fetch the emblem icons from metadata */
 	icons = nautilus_icon_factory_get_emblem_icons_for_file (sidebar_title->details->file, FALSE, NULL);
 
@@ -488,7 +554,7 @@ update_emblems (NautilusSidebarTitle *sidebar_title)
 			(p->data,
 			 NAUTILUS_ICON_SIZE_STANDARD, NAUTILUS_ICON_SIZE_STANDARD,
 			 NAUTILUS_ICON_SIZE_STANDARD, NAUTILUS_ICON_SIZE_STANDARD,
-			 NULL);
+			 NULL, FALSE);
 		if (pixbuf != NULL) {
 			add_emblem (sidebar_title, pixbuf);
 			gdk_pixbuf_unref (pixbuf);
@@ -604,122 +670,35 @@ nautilus_sidebar_title_hit_test_icon (NautilusSidebarTitle *sidebar_title, int x
 	return nautilus_point_in_widget (sidebar_title->details->icon, x, y);
 }
 
-static void
-sidebar_create_smooth_components_if_needed (NautilusSidebarTitle *sidebar_title)
+static GtkWidget *
+sidebar_title_create_title_label (void)
 { 
-	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (sidebar_title));
-	g_return_if_fail (sidebar_title->details->label_box != NULL);
-	g_return_if_fail (sidebar_title->details->more_info_box != NULL);
+	GtkWidget *title_label;
 
-	/* Label */
-	if (sidebar_title->details->smooth_title_label == NULL) {
-		sidebar_title->details->smooth_title_label = nautilus_label_new ("");
-		
-		sidebar_title->details->smooth_title_label_font = 
-			NAUTILUS_SCALABLE_FONT (nautilus_scalable_font_new ("helvetica", "bold", NULL, NULL));
-		
-		nautilus_label_set_font (NAUTILUS_LABEL (sidebar_title->details->smooth_title_label),
-					 sidebar_title->details->smooth_title_label_font);
-		
-		gtk_box_pack_start (GTK_BOX (sidebar_title->details->label_box), sidebar_title->details->smooth_title_label, 0, 0, 0);
-	}
-
-	/* More info */
-	if (sidebar_title->details->smooth_more_info_label == NULL) {
-		sidebar_title->details->smooth_more_info_label = nautilus_label_new ("");
-		
-		sidebar_title->details->smooth_more_info_label_font = 
-			NAUTILUS_SCALABLE_FONT (nautilus_scalable_font_new ("helvetica", "medium", NULL, NULL));
-		
-		nautilus_label_set_font (NAUTILUS_LABEL (sidebar_title->details->smooth_more_info_label),
-					 sidebar_title->details->smooth_more_info_label_font);
-		
-		nautilus_label_set_font_size (NAUTILUS_LABEL (sidebar_title->details->smooth_more_info_label), 12);
-
-		gtk_box_pack_start (GTK_BOX (sidebar_title->details->more_info_box), sidebar_title->details->smooth_more_info_label, 0, 0, 0);
-	}
+	title_label = nautilus_label_new_with_background ("");
+	nautilus_label_make_bold (NAUTILUS_LABEL (title_label));
+	nautilus_label_set_wrap (NAUTILUS_LABEL (title_label), TRUE);
+	nautilus_label_set_justify (NAUTILUS_LABEL (title_label), GTK_JUSTIFY_CENTER);
+	
+	return title_label;
 }
 
-static void
-sidebar_create_normal_components_if_needed (NautilusSidebarTitle *sidebar_title)
-{ 
-	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (sidebar_title));
-	g_return_if_fail (sidebar_title->details->label_box != NULL);
-	g_return_if_fail (sidebar_title->details->more_info_box != NULL);
+static GtkWidget *
+sidebar_title_create_more_info_label (void)
+{
+	GtkWidget *more_info_label;
 
-	/* Label */
-	if (sidebar_title->details->normal_title_label == NULL) {
-		sidebar_title->details->normal_title_label = GTK_WIDGET (gtk_label_new (NULL));
-		gtk_label_set_line_wrap (GTK_LABEL (sidebar_title->details->normal_title_label), TRUE);
-		
-		gtk_box_pack_start (GTK_BOX (sidebar_title->details->label_box), sidebar_title->details->normal_title_label, 0, 0, 0);
-	}
-
-	/* More info */
-	if (sidebar_title->details->normal_more_info_label == NULL) {
-		GdkFont *font;
-
-		sidebar_title->details->normal_more_info_label = GTK_WIDGET (gtk_label_new (NULL));
-		font = nautilus_font_factory_get_font_from_preferences (12);
-		nautilus_gtk_widget_set_font (sidebar_title->details->normal_more_info_label, font);
-		gdk_font_unref (font);
-		gtk_box_pack_start (GTK_BOX (sidebar_title->details->more_info_box), sidebar_title->details->normal_more_info_label, 0, 0, 0);
-	}
+	more_info_label = nautilus_label_new_with_background ("");
+	nautilus_label_make_smaller (NAUTILUS_LABEL (more_info_label), 2);
+	nautilus_label_set_justify (NAUTILUS_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
+	
+	return more_info_label;
 }
 
 static void
 smooth_graphics_mode_changed_callback (gpointer callback_data)
 {
-	NautilusSidebarTitle *sidebar_title;
-
 	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (callback_data));
-
-	sidebar_title = NAUTILUS_SIDEBAR_TITLE (callback_data);
-
-	sidebar_title->details->smooth_graphics = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
-										    TRUE);
-
-	if (sidebar_title->details->smooth_graphics) {
-		sidebar_create_smooth_components_if_needed (sidebar_title);
-	}
-	else {
-		sidebar_create_normal_components_if_needed (sidebar_title);
-	}
-
-	if (sidebar_title->details->smooth_graphics) {
-		nautilus_image_set_alpha_mode (NAUTILUS_IMAGE (sidebar_title->details->icon), NAUTILUS_IMAGE_FULL_ALPHA);
-
-		if (sidebar_title->details->normal_title_label != NULL) {
-			gtk_widget_hide (sidebar_title->details->normal_title_label);
-		}
-
-		if (sidebar_title->details->normal_more_info_label != NULL) {
-			gtk_widget_hide (sidebar_title->details->normal_more_info_label);
-		}
-
-		g_assert (GTK_IS_WIDGET (sidebar_title->details->smooth_title_label));
-		gtk_widget_show (sidebar_title->details->smooth_title_label);
-
-		g_assert (GTK_IS_WIDGET (sidebar_title->details->smooth_more_info_label));
-		gtk_widget_show (sidebar_title->details->smooth_more_info_label);
-	}
-	else {
-		nautilus_image_set_alpha_mode (NAUTILUS_IMAGE (sidebar_title->details->icon), NAUTILUS_IMAGE_THRESHOLD_ALPHA);
-
-		if (sidebar_title->details->smooth_title_label != NULL) {
-			gtk_widget_hide (sidebar_title->details->smooth_title_label);
-		}
-
-		if (sidebar_title->details->smooth_more_info_label != NULL) {
-			gtk_widget_hide (sidebar_title->details->smooth_more_info_label);
-		}
-
-		g_assert (GTK_IS_WIDGET (sidebar_title->details->normal_title_label));
-		gtk_widget_show (sidebar_title->details->normal_title_label);
-
-		g_assert (GTK_IS_WIDGET (sidebar_title->details->normal_more_info_label));
-		gtk_widget_show (sidebar_title->details->normal_more_info_label);
-	}
-
-	update_all (sidebar_title);
+	
+	update_all (NAUTILUS_SIDEBAR_TITLE (callback_data));
 }
