@@ -91,8 +91,12 @@ struct NautilusSidebarDetails {
 
 static void     nautilus_sidebar_initialize_class   (GtkObjectClass   *object_klass);
 static void     nautilus_sidebar_initialize         (GtkObject        *object);
+static void	nautilus_sidebar_deactivate_panel   (NautilusSidebar *sidebar);
+
 static gboolean nautilus_sidebar_press_event        (GtkWidget        *widget,
 						     GdkEventButton   *event);
+static gboolean nautilus_sidebar_release_event      (GtkWidget        *widget,
+						     GdkEventButton   *event);
 static gboolean nautilus_sidebar_leave_event        (GtkWidget        *widget,
 						     GdkEventCrossing *event);
 static gboolean nautilus_sidebar_motion_event       (GtkWidget        *widget,
@@ -115,9 +119,11 @@ static void     nautilus_sidebar_update_buttons     (NautilusSidebar  *sidebar);
 static void     add_command_buttons                 (NautilusSidebar  *sidebar,
 						     GList            *application_list);
 
-/* FIXME bugzilla.eazel.com 1245: hardwired sizes */
+static void	background_metadata_changed_callback	    (NautilusSidebar  *sidebar);
+
 #define DEFAULT_TAB_COLOR "rgb:9999/9999/9999"
 
+/* FIXME bugzilla.eazel.com 1245: hardwired sizes */
 #define SIDEBAR_MINIMUM_WIDTH 1
 #define SIDEBAR_MINIMUM_HEIGHT 400
 
@@ -174,6 +180,7 @@ nautilus_sidebar_initialize_class (GtkObjectClass *object_klass)
 	widget_class->motion_notify_event = nautilus_sidebar_motion_event;
 	widget_class->leave_notify_event = nautilus_sidebar_leave_event;
 	widget_class->button_press_event  = nautilus_sidebar_press_event;
+	widget_class->button_release_event  = nautilus_sidebar_release_event;
 	widget_class->size_allocate = nautilus_sidebar_size_allocate;
 
 	/* add the "location changed" signal */
@@ -323,7 +330,7 @@ nautilus_sidebar_sidebar_panel_enabled (const char *panel_iid)
         char *key;
 
 	key = nautilus_sidebar_get_sidebar_panel_key (panel_iid);
-        enabled = nautilus_preferences_get_boolean (key, FALSE);
+        enabled = nautilus_preferences_get_boolean (key);
         g_free (key);
         return enabled;
 }
@@ -340,14 +347,44 @@ reset_background_callback (GtkWidget *menu_item, GtkWidget *sidebar)
 	}
 }
 
+/* utility routine that checks if the active panel matches the passed-in object id */
+static gboolean
+nautilus_sidebar_active_panel_matches_id (NautilusSidebar *sidebar, const char *id)
+{
+	GtkWidget *current_view;
+	const char *current_iid;
+	
+	if (sidebar->details->selected_index < 0) {
+		return FALSE;
+	}
+	current_view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (sidebar->details->notebook),
+							sidebar->details->selected_index);
+	current_iid = nautilus_view_frame_get_iid (NAUTILUS_VIEW_FRAME (current_view));
+	return nautilus_strcmp (current_iid, id) == 0;	
+}
+
+/* if the active panel matches the passed in id, hide it. */
+void
+nautilus_sidebar_hide_active_panel_if_matches (NautilusSidebar *sidebar, const char *sidebar_id)
+{
+	if (nautilus_sidebar_active_panel_matches_id (sidebar, sidebar_id)) {
+		nautilus_sidebar_deactivate_panel (sidebar);
+	}
+}
+
 /* callback for sidebar panel menu items to toggle their visibility */
 static void
 toggle_sidebar_panel (GtkWidget *widget, char *sidebar_id)
 {
+ 	NautilusSidebar *sidebar;
         char *key;
+	
+	sidebar = NAUTILUS_SIDEBAR (gtk_object_get_user_data (GTK_OBJECT (widget)));
 
+	nautilus_sidebar_hide_active_panel_if_matches (sidebar, sidebar_id);
+		
 	key = nautilus_sidebar_get_sidebar_panel_key (sidebar_id);
-	nautilus_preferences_set_boolean (key, !nautilus_preferences_get_boolean (key, FALSE));
+	nautilus_preferences_set_boolean (key, !nautilus_preferences_get_boolean (key));
 	g_free (key); 
 }
 
@@ -359,7 +396,7 @@ nautilus_sidebar_add_panel_items(NautilusSidebar *sidebar, GtkWidget *menu)
 	CORBA_Environment ev;
 	const char *query;
         OAF_ServerInfoList *oaf_result;
-	int i;
+	guint i;
 	gboolean enabled;
 	GList *name_list;
 	GtkWidget *menu_item;
@@ -387,6 +424,7 @@ nautilus_sidebar_add_panel_items(NautilusSidebar *sidebar, GtkWidget *menu)
 				gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM(menu_item), TRUE);
 				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), enabled);
 				gtk_widget_show (menu_item);
+				gtk_object_set_user_data (GTK_OBJECT (menu_item), sidebar);
 				gtk_menu_append (GTK_MENU(menu), menu_item);
 				gtk_signal_connect_full (GTK_OBJECT (menu_item), "activate",
 							 GTK_SIGNAL_FUNC (toggle_sidebar_panel),
@@ -679,13 +717,22 @@ receive_dropped_color (NautilusSidebar *sidebar,
 		nautilus_sidebar_tabs_receive_dropped_color
 			(sidebar->details->sidebar_tabs,
 			 x, y, selection_data);
-		
+
+		/* Block so we don't respond to our own metadata changes.
+		 */
+		gtk_signal_handler_block_by_func (GTK_OBJECT (sidebar->details->file),
+						  background_metadata_changed_callback,
+						  sidebar);
+						  
 		nautilus_file_set_metadata
 			(sidebar->details->file,
 			 NAUTILUS_METADATA_KEY_SIDEBAR_TAB_COLOR,
 			 DEFAULT_TAB_COLOR,
 			 color_spec);
-		
+
+		gtk_signal_handler_unblock_by_func (GTK_OBJECT (sidebar->details->file),
+						    background_metadata_changed_callback,
+						    sidebar);
 		break;
 	case TITLE_TAB_PART:
 		/* color dropped on title tab */
@@ -693,11 +740,21 @@ receive_dropped_color (NautilusSidebar *sidebar,
 			(sidebar->details->title_tab,
 			 x, y, selection_data);
 		
+		/* Block so we don't respond to our own metadata changes.
+		 */
+		gtk_signal_handler_block_by_func (GTK_OBJECT (sidebar->details->file),
+						  background_metadata_changed_callback,
+						  sidebar);
+
 		nautilus_file_set_metadata
 			(sidebar->details->file,
 			 NAUTILUS_METADATA_KEY_SIDEBAR_TITLE_TAB_COLOR,
 			 DEFAULT_TAB_COLOR,
 			 color_spec);
+
+		gtk_signal_handler_unblock_by_func (GTK_OBJECT (sidebar->details->file),
+						    background_metadata_changed_callback,
+						    sidebar);
 		break;
 	case ICON_PART:
 	case BACKGROUND_PART:
@@ -705,10 +762,6 @@ receive_dropped_color (NautilusSidebar *sidebar,
 		nautilus_background_receive_dropped_color
 			(nautilus_get_widget_background (GTK_WIDGET (sidebar)),
 			 GTK_WIDGET (sidebar), x, y, selection_data);
-		
-		/* regenerate the display */
-		nautilus_sidebar_update_appearance (sidebar);  	
-		
 		break;
 	}
 	g_free(color_spec);
@@ -798,21 +851,17 @@ nautilus_sidebar_remove_panel (NautilusSidebar *sidebar,
 	int page_num;
 	char *description;
 	
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (panel));
+
 	page_num = gtk_notebook_page_num (GTK_NOTEBOOK (sidebar->details->notebook),
 					  GTK_WIDGET (panel));
-
-
-	/* FIXME bugzilla.eazel.com 1840: 
-	 * This g_return_if_fail gets hit in cases where the sidebar panel
-	 * fails when loading, which causes the panel's tab to be left behind.
-	 */
-	g_return_if_fail (page_num >= 0);
-
-	gtk_notebook_remove_page (GTK_NOTEBOOK (sidebar->details->notebook),
-				  page_num);
-
 	description = nautilus_view_frame_get_label (panel);
 
+	if (page_num >= 0) {
+		gtk_notebook_remove_page (GTK_NOTEBOOK (sidebar->details->notebook),
+				  page_num);
+	}
+	
 	/* Remove the tab associated with this panel */
 	nautilus_sidebar_tabs_remove_view (sidebar->details->sidebar_tabs, description);
 
@@ -860,7 +909,7 @@ nautilus_sidebar_activate_panel (NautilusSidebar *sidebar, int which_view)
 
 /* utility to deactivate the active panel */
 static void
-nautilus_sidebar_deactivate_panel(NautilusSidebar *sidebar)
+nautilus_sidebar_deactivate_panel (NautilusSidebar *sidebar)
 {
 	if (sidebar->details->selected_index >= 0) {
 		gtk_widget_hide (GTK_WIDGET (sidebar->details->notebook));
@@ -927,18 +976,12 @@ nautilus_sidebar_leave_event (GtkWidget *widget, GdkEventCrossing *event)
 	return TRUE;
 }
 
-/* hit-test the index tabs and activate if necessary */
-
+/* handle the context menu if necessary */
 static gboolean
 nautilus_sidebar_press_event (GtkWidget *widget, GdkEventButton *event)
 {
-	int title_top, title_bottom;
-	GtkWidget *menu;
 	NautilusSidebar *sidebar;
-	NautilusSidebarTabs *sidebar_tabs;
-	NautilusSidebarTabs *title_tab;
-	int rounded_y;
-	int which_tab;
+	GtkWidget *menu;
 		
 	if (widget->window != event->window) {
 		return FALSE;
@@ -952,9 +995,27 @@ nautilus_sidebar_press_event (GtkWidget *widget, GdkEventButton *event)
 		nautilus_pop_up_context_menu (GTK_MENU(menu),
 				      NAUTILUS_DEFAULT_POPUP_MENU_DISPLACEMENT,
 				      NAUTILUS_DEFAULT_POPUP_MENU_DISPLACEMENT,
-				      event->button);
-		return TRUE;
+				      event);
+	}	
+	return TRUE;
+}
+
+/* handle the sidebar tabs on the upstroke */
+static gboolean
+nautilus_sidebar_release_event (GtkWidget *widget, GdkEventButton *event)
+{
+	int title_top, title_bottom;
+	NautilusSidebar *sidebar;
+	NautilusSidebarTabs *sidebar_tabs;
+	NautilusSidebarTabs *title_tab;
+	int rounded_y;
+	int which_tab;
+		
+	if (widget->window != event->window) {
+		return FALSE;
 	}
+
+	sidebar = NAUTILUS_SIDEBAR (widget);
 	
 	sidebar_tabs = sidebar->details->sidebar_tabs;
 	title_tab = sidebar->details->title_tab;
@@ -985,11 +1046,13 @@ nautilus_sidebar_press_event (GtkWidget *widget, GdkEventButton *event)
 	return TRUE;
 }
 
-/* handle the background changed signal by writing out the settings to metadata */
+/* Handle the background changed signal by writing out the settings to metadata.
+ */
 static void
 background_settings_changed_callback (NautilusBackground *background, NautilusSidebar *sidebar)
 {
-	char *color_spec, *image;
+	char *image;
+	char *color;
 
 	g_assert (NAUTILUS_IS_BACKGROUND (background));
 	g_assert (NAUTILUS_IS_SIDEBAR (sidebar));
@@ -998,47 +1061,43 @@ background_settings_changed_callback (NautilusBackground *background, NautilusSi
 		return;
 	}
 	
-	color_spec = nautilus_background_get_color (background);
+	/* Block so we don't respond to our own metadata changes.
+	 */
+	gtk_signal_handler_block_by_func (GTK_OBJECT (sidebar->details->file),
+					  background_metadata_changed_callback,
+					  sidebar);
+
+	color = nautilus_background_get_color (background);
+	image = nautilus_background_get_image_uri (background);
+	
 	nautilus_file_set_metadata (sidebar->details->file,
 				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-				    sidebar->details->default_background_color,
-				    color_spec);
-	g_free (color_spec);
+				    NULL,
+				    color);
 
-	image = nautilus_background_get_image_uri (background);
 	nautilus_file_set_metadata (sidebar->details->file,
 				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-				    sidebar->details->default_background_image,
-				    image);	
+				    NULL,
+				    image);
+				    
+	/* Block so this fn is not reinvoked due to nautilus_background_set_combine_mode */
+	gtk_signal_handler_block_by_func (GTK_OBJECT (background),
+					  background_settings_changed_callback,
+					  sidebar);
+	/* Combine mode uses dithering to avoid striations in gradients.
+	 */
+	nautilus_background_set_combine_mode (background, nautilus_gradient_is_gradient (color));
+	
+	gtk_signal_handler_unblock_by_func (GTK_OBJECT (background),
+					    background_settings_changed_callback,
+					    sidebar);
+
+	g_free (color);
 	g_free (image);
-}
 
-/* we generally want to ignore the appearance changed signal, but we need it to redraw the
-   the sidebar in the case where we're loading the background image, so check for that */
-static void
-background_appearance_changed_callback (NautilusBackground *background, NautilusSidebar *sidebar)
-{
-	gboolean is_default_color, is_default_image;
-	char *background_image, *background_color;
-
-	background_color = nautilus_file_get_metadata (sidebar->details->file,
-						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-						       sidebar->details->default_background_color);
-
-	background_image = nautilus_file_get_metadata (sidebar->details->file,
-						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-						       sidebar->details->default_background_image);
-	
-	nautilus_sidebar_title_select_text_color (sidebar->details->title);
-	
-	is_default_color = !nautilus_strcmp(background_color, sidebar->details->default_background_color);
-	is_default_image = !nautilus_strcmp(background_image, sidebar->details->default_background_image);
-	
-	if (is_default_color && is_default_image) {
-		nautilus_sidebar_update_appearance (sidebar);  	
-	}
-	g_free (background_color);
-	g_free (background_image);
+	gtk_signal_handler_unblock_by_func (GTK_OBJECT (sidebar->details->file),
+					    background_metadata_changed_callback,
+					    sidebar);
 }
 
 /* handle the background reset signal by writing out NULL to metadata and setting the backgrounds
@@ -1046,39 +1105,36 @@ background_appearance_changed_callback (NautilusBackground *background, Nautilus
 static void
 background_reset_callback (NautilusBackground *background, NautilusSidebar *sidebar)
 {
-	char *combine_mode;
-	
+	g_assert (NAUTILUS_IS_BACKGROUND (background));
+	g_assert (NAUTILUS_IS_SIDEBAR (sidebar));
+
 	if (sidebar->details->file == NULL) {
 		return;
 	}
-	
-	/* set up the defaults, but don't write the metdata */
-	gtk_signal_handler_block_by_func (GTK_OBJECT (background),
-					  background_settings_changed_callback,
+
+	/* Block so we don't respond to our own metadata changes.
+	 */
+	gtk_signal_handler_block_by_func (GTK_OBJECT (sidebar->details->file),
+					  background_metadata_changed_callback,
 					  sidebar);
-	nautilus_background_set_color (background, sidebar->details->default_background_color);	
-	nautilus_background_set_image_uri (background, sidebar->details->default_background_image);
-	
-	combine_mode = nautilus_theme_get_theme_data ("sidebar", "COMBINE");
-	nautilus_background_set_combine_mode (background, combine_mode != NULL);
-	g_free (combine_mode);
-	
-	gtk_signal_handler_unblock_by_func (GTK_OBJECT(background),
-					    background_settings_changed_callback,
-					    sidebar);
-					   
-	/* reset the metadata */
+
 	nautilus_file_set_metadata (sidebar->details->file,
 				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-				    sidebar->details->default_background_color,
-				    NULL);
-	nautilus_file_set_metadata (sidebar->details->file,
-				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-				    sidebar->details->default_background_image,
+				    NULL,
 				    NULL);
 
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (background),
-				      "reset");
+	nautilus_file_set_metadata (sidebar->details->file,
+				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+				    NULL,
+				    NULL);
+
+	gtk_signal_handler_unblock_by_func (GTK_OBJECT (sidebar->details->file),
+					    background_metadata_changed_callback,
+					    sidebar);
+
+	/* Force a read from the metadata to set the defaults
+	 */
+	background_metadata_changed_callback (sidebar);
 }
 
 static GtkWindow *
@@ -1186,9 +1242,14 @@ add_command_buttons (NautilusSidebar *sidebar, GList *application_list)
 				    FALSE, FALSE, 
 				    0);
 
-		/* FIXME bugzilla.eazel.com 2510: Security hole? Can't use a string from the
-		 * MIME file as a printf format string without first
-		 * checking it over somehow.
+		/* FIXME bugzilla.eazel.com 2510: Security hole?
+		 * Unsafe to use a string from the MIME file as a
+		 * printf format string without first checking it over
+		 * somehow. We can do a search and replace on the "%s"
+		 * part instead, which should work.
+		 */
+		/* FIXME: Doing a +7 does not turn a URI into a path
+		 * name.
 		 */
 		temp_str = g_strdup_printf
 			("'%s'", 
@@ -1208,7 +1269,7 @@ add_command_buttons (NautilusSidebar *sidebar, GList *application_list)
 
 	/* Catch-all button after all the others. */
 	temp_button = gtk_button_new_with_label (_("Open with..."));
-	gtk_signal_connect (GTK_OBJECT (temp_button),  "clicked",
+	gtk_signal_connect (GTK_OBJECT (temp_button), "clicked",
 			    open_with_callback, NULL);
 	gtk_object_set_user_data (GTK_OBJECT (temp_button), sidebar);
 	gtk_widget_show (temp_button);
@@ -1317,8 +1378,10 @@ static void
 nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 {
 	NautilusBackground *background;
-	char *background_color, *color_spec;
-	char *background_image, *combine_mode;
+	char *color_spec;
+	char *background_color;
+	char *background_image;
+	gboolean combine;
 
 	g_return_if_fail (NAUTILUS_IS_SIDEBAR (sidebar));
 	
@@ -1330,10 +1393,6 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 				    "settings_changed",
 				    background_settings_changed_callback,
 				    sidebar);
-		gtk_signal_connect (GTK_OBJECT (background),
-				    "appearance_changed",
-				    background_appearance_changed_callback,
-				    sidebar);
 		gtk_signal_connect (GTK_OBJECT (background),
 				    "reset",
 				    background_reset_callback,
@@ -1341,34 +1400,39 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 	}
 	
 	/* Set up the background color and image from the metadata. */
-	background_image = NULL;
-	background_color = nautilus_file_get_metadata (sidebar->details->file,
-						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-						       NULL);
-	if (background_color == NULL) {
+
+	if (nautilus_sidebar_background_is_default (sidebar)) {
+		char* combine_str;
 		background_color = g_strdup (sidebar->details->default_background_color);
+		background_image = g_strdup (sidebar->details->default_background_image);
+		combine_str = nautilus_theme_get_theme_data ("sidebar", "COMBINE");
+		combine = combine_str != NULL;
+		g_free (combine_str);
+	} else {
+		background_color = nautilus_file_get_metadata (sidebar->details->file,
+							       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+							       NULL);
 		background_image = nautilus_file_get_metadata (sidebar->details->file,
 							       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-							       sidebar->details->default_background_image);
-	}	
+							       NULL);
+		       
+		/* Combine mode uses dithering to avoid striations in gradients.
+		 */
+		combine = nautilus_gradient_is_gradient (background_color);
+	}
 		
-	/* disable the settings_changed callback, so the background doesn't get
-	   written out, since it might be the theme-dependent default */
+	/* Block so we don't write these settings out in response to our set calls below */
 	gtk_signal_handler_block_by_func (GTK_OBJECT (background),
 					  background_settings_changed_callback,
 					  sidebar);
-	
-	nautilus_background_set_color (background, background_color);	
-	g_free (background_color);
-	
-	nautilus_background_set_image_uri (background, background_image);
-	g_free (background_image);
 
-	combine_mode = nautilus_theme_get_theme_data ("sidebar", "COMBINE");
-	nautilus_background_set_combine_mode (background, combine_mode != NULL);
-	g_free (combine_mode);
+	nautilus_background_set_image_uri (background, background_image);
+	nautilus_background_set_color (background, background_color);
+	nautilus_background_set_combine_mode (background, combine);
+
+	g_free (background_color);
+	g_free (background_image);
 	
-	/* set up the color for the tabs */
 	color_spec = nautilus_file_get_metadata (sidebar->details->file,
 						 NAUTILUS_METADATA_KEY_SIDEBAR_TAB_COLOR,
 						 DEFAULT_TAB_COLOR);
@@ -1381,7 +1445,6 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 	nautilus_sidebar_tabs_set_color(sidebar->details->title_tab, color_spec);
 	g_free (color_spec);
 
-	/* re-enable the background_changed signal */
 	gtk_signal_handler_unblock_by_func (GTK_OBJECT (background),
 					    background_settings_changed_callback,
 					    sidebar);
@@ -1389,7 +1452,7 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 
 
 static void
-nautilus_sidebar_update_all (NautilusSidebar *sidebar)
+background_metadata_changed_callback (NautilusSidebar *sidebar)
 {
 	GList *attributes;
 	gboolean ready;
@@ -1445,14 +1508,14 @@ nautilus_sidebar_set_uri (NautilusSidebar *sidebar,
 	sidebar->details->file_changed_connection =
 		gtk_signal_connect_object (GTK_OBJECT (sidebar->details->file),
 					   "changed",
-					   nautilus_sidebar_update_all,
+					   background_metadata_changed_callback,
 					   GTK_OBJECT (sidebar));
 
 	attributes = nautilus_mime_actions_get_minimum_file_attributes ();
 	nautilus_file_monitor_add (sidebar->details->file, sidebar, attributes);
 	g_list_free (attributes);
 
-	nautilus_sidebar_update_all (sidebar);
+	background_metadata_changed_callback (sidebar);
 
 	/* tell the title widget about it */
 	nautilus_sidebar_title_set_file (sidebar->details->title,
@@ -1481,6 +1544,7 @@ nautilus_sidebar_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	
 	if (widget->allocation.width != sidebar->details->old_width) {
 		sidebar->details->old_width = widget->allocation.width;
- 		nautilus_preferences_set_enum(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH, widget->allocation.width);
+ 		nautilus_preferences_set_integer (NAUTILUS_PREFERENCES_SIDEBAR_WIDTH,
+					      widget->allocation.width);
 	}	
 }
