@@ -154,10 +154,9 @@ nautilus_switchable_search_bar_new (NautilusWindow *window)
 	gtk_container_add (GTK_CONTAINER (bar), hbox);
 
 	gtk_widget_show_all (hbox);
-	nautilus_switchable_search_bar_set_mode
+	nautilus_switchable_search_bar_set_mode 
 		(bar, 
-		 nautilus_preferences_get_enum (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
-						NAUTILUS_SIMPLE_SEARCH_BAR));
+		 nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE));
 
 	/* React to future preference changes. */
 	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
@@ -210,11 +209,6 @@ nautilus_switchable_search_bar_set_mode (NautilusSwitchableSearchBar *bar,
 	}
 	g_free (location);
 
-	/* FIXME bugzilla.eazel.com 1860: 
-	 * Switching to the complex search bar, which is taller, leaves
-	 * the navigation bar forever at the taller height, even after switching
-	 * back to the simple search bar or the location bar.
-	 */
 	switch (mode) {
 	case NAUTILUS_SIMPLE_SEARCH_BAR:
 		gtk_widget_show (bar->simple_search_bar);
@@ -289,8 +283,7 @@ nautilus_search_uri_to_search_bar_mode (const char *uri)
 {
 	NautilusSearchBarMode preferred_mode;
 
-	preferred_mode = nautilus_preferences_get_enum (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
-							NAUTILUS_SIMPLE_SEARCH_BAR);
+	preferred_mode = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE);
 	if (nautilus_search_uri_is_displayable_by_mode (uri, preferred_mode)) {
 		return preferred_mode;
 	}
