@@ -53,6 +53,7 @@
 #include <libnautilus-extensions/nautilus-file.h>
 #include <libnautilus-extensions/nautilus-font-factory.h>
 #include <libnautilus-extensions/nautilus-gdk-extensions.h>
+#include <libnautilus-extensions/nautilus-gdk-font-extensions.h>
 #include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
@@ -69,12 +70,10 @@
 struct NautilusThemeSelectorDetails {
 	GtkWidget *container;
 
-	GtkWidget *title_box;
 	GtkWidget *title_label;
 	GtkWidget *help_label;
 	
 	GtkWidget *theme_list;
-	GtkWidget *bottom_box;
 	
 	GtkWidget *add_button;
 	GtkWidget *add_button_label;	
@@ -108,17 +107,26 @@ static gboolean nautilus_theme_selector_delete_event_callback (GtkWidget *widget
 static void  nautilus_theme_selector_theme_changed	(gpointer user_data);
 static void  populate_list_with_themes 			(NautilusThemeSelector *theme_selector);
 
-static void  theme_select_row_callback (GtkCList * clist, int row, int column, GdkEventButton *event, NautilusThemeSelector *theme_selector); 
+static void  theme_select_row_callback			(GtkCList              *clist,
+							 int                    row,
+							 int                    column,
+							 GdkEventButton        *event,
+							 NautilusThemeSelector *theme_selector); 
+static void  theme_style_set_callback			(GtkWidget             *widget, 
+							 GtkStyle              *previous_style,
+							 NautilusThemeSelector *theme_selector); 
+
 static void  exit_remove_mode 				(NautilusThemeSelector *theme_selector);
 static void  set_help_label				(NautilusThemeSelector *theme_selector,
 							 gboolean remove_mode);
+static void  setup_font_sizes_for_row			(NautilusThemeSelector *theme_selector,
+							 int theme_index);
+static void  setup_font_sizes_for_all_rows		(NautilusThemeSelector *theme_selector);
+static void  clear_style_for_all_rows			(NautilusThemeSelector *theme_selector);
 							 
 #define THEME_SELECTOR_WIDTH  460
 #define THEME_SELECTOR_HEIGHT 264
 
-#define SELECTOR_BACKGROUND_COLOR "rgb:FFFF/FFFF/FFFF"
-#define SELECTOR_TITLE_BAR_COLOR  "rgb:D6D6/D6D6/D6D6"
-
 static NautilusThemeSelector *main_theme_selector = NULL;
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusThemeSelector, nautilus_theme_selector, GTK_TYPE_WINDOW)
@@ -134,15 +142,23 @@ nautilus_theme_selector_initialize_class (GtkObjectClass *object_klass)
 	object_klass->destroy = nautilus_theme_selector_destroy;
 }
 
-/* initialize the instance's fields, create the necessary subviews, etc. */
+/* handle the "done" button by destroying the window */
+static void
+done_button_callback (GtkWidget *widget, GtkWidget *theme_selector)
+{
+	gtk_widget_destroy (theme_selector);
+}
 
+/* initialize the instance's fields, create the necessary subviews, etc. */
 static void
 nautilus_theme_selector_initialize (GtkObject *object)
 {
-	NautilusBackground *background;
  	NautilusThemeSelector *theme_selector;
  	GtkWidget* widget, *temp_box, *temp_hbox, *temp_frame;
 	GtkWidget *scrollwindow;
+	GtkWidget *title_box;
+	GtkWidget *bottom_box;
+	GtkWidget *temp_button, *temp_label;
 	
 	theme_selector = NAUTILUS_THEME_SELECTOR (object);
 	widget = GTK_WIDGET (object);
@@ -153,12 +169,10 @@ nautilus_theme_selector_initialize (GtkObject *object)
 	gtk_widget_set_usize (widget, THEME_SELECTOR_WIDTH, THEME_SELECTOR_HEIGHT);
 	gtk_container_set_border_width (GTK_CONTAINER (widget), 0);				
 
-	/* set the title */
-	gtk_window_set_title (GTK_WINDOW(widget), _("Nautilus Theme Selector"));
-	
-	/* set up the background */	
-	background = nautilus_get_widget_background (GTK_WIDGET (theme_selector));
-	nautilus_background_set_color (background, SELECTOR_BACKGROUND_COLOR);	
+	/* set the title and standard close accelerator */
+	gtk_window_set_title (GTK_WINDOW (widget), _("Nautilus Theme Selector"));
+	gtk_window_set_wmclass(GTK_WINDOW(widget), "theme_selector", "Nautilus");
+	nautilus_gtk_window_set_up_close_accelerator (GTK_WINDOW (widget));
 	
 	/* create the container box */  
   	theme_selector->details->container =  gtk_vbox_new (FALSE, 0);
@@ -169,19 +183,16 @@ nautilus_theme_selector_initialize (GtkObject *object)
 	
   	/* create the title box */
   	
-  	theme_selector->details->title_box = gtk_event_box_new();
-	gtk_container_set_border_width (GTK_CONTAINER (theme_selector->details->title_box), 0);				
+  	title_box = gtk_event_box_new();
+	gtk_container_set_border_width (GTK_CONTAINER (title_box), 0);				
  
- 	background = nautilus_get_widget_background (GTK_WIDGET (theme_selector->details->title_box));
-	nautilus_background_set_color (background, SELECTOR_TITLE_BAR_COLOR);	
-
-  	gtk_widget_show(theme_selector->details->title_box);
-	gtk_box_pack_start (GTK_BOX(theme_selector->details->container), theme_selector->details->title_box, FALSE, FALSE, 0);
+  	gtk_widget_show(title_box);
+	gtk_box_pack_start (GTK_BOX(theme_selector->details->container), title_box, FALSE, FALSE, 0);
   	
   	temp_frame = gtk_frame_new(NULL);
-  	gtk_frame_set_shadow_type(GTK_FRAME(temp_frame), GTK_SHADOW_OUT);
+  	gtk_frame_set_shadow_type(GTK_FRAME(temp_frame), GTK_SHADOW_NONE);
   	gtk_widget_show(temp_frame);
-  	gtk_container_add(GTK_CONTAINER(theme_selector->details->title_box), temp_frame);
+  	gtk_container_add(GTK_CONTAINER(title_box), temp_frame);
   	
   	temp_hbox = gtk_hbox_new(FALSE, 0);
   	gtk_widget_show(temp_hbox);
@@ -189,81 +200,106 @@ nautilus_theme_selector_initialize (GtkObject *object)
  	
 	/* add the title label */
 	theme_selector->details->title_label = nautilus_label_new (_("Nautilus Theme:"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (theme_selector->details->title_label), 18);
+	nautilus_label_make_larger (NAUTILUS_LABEL (theme_selector->details->title_label), 2);
+	nautilus_label_make_bold (NAUTILUS_LABEL (theme_selector->details->title_label));
 
   	gtk_widget_show(theme_selector->details->title_label);
 	gtk_box_pack_start (GTK_BOX(temp_hbox), theme_selector->details->title_label, FALSE, FALSE, 8);
  
  	/* add the help label */
 	theme_selector->details->help_label = nautilus_label_new ("");
-	nautilus_label_set_font_size (NAUTILUS_LABEL (theme_selector->details->help_label), 12);
 	set_help_label (theme_selector, FALSE);
-  	gtk_widget_show(theme_selector->details->help_label);
+	nautilus_label_make_smaller (NAUTILUS_LABEL (theme_selector->details->help_label), 2);
+	nautilus_label_set_justify (NAUTILUS_LABEL (theme_selector->details->help_label), GTK_JUSTIFY_RIGHT);
+  	
+	gtk_widget_show(theme_selector->details->help_label);
 	gtk_box_pack_end (GTK_BOX(temp_hbox), theme_selector->details->help_label, FALSE, FALSE, 8);
  
  	/* add the main part of the content, which is a list view, embedded in a scrollwindow */
 	
+	/* FIXME bugzilla.eazel.com 5951: 
+	 * We cant hard code the geometry of the clist columns here.
+	 * There is at least 2 things that will break this:
+	 *
+	 * 1) The user picking a larger (or even smaller) font that the default in 
+	 *    the control center.
+	 * 
+	 * 2) A theme having a long name that is obviously now known at compile time.
+	 *    For example, "Crux-Eggplant" is too long for the hard coded defaults herre..
+	 */
 	theme_selector->details->theme_list = gtk_clist_new (3);
 	gtk_clist_set_row_height   (GTK_CLIST (theme_selector->details->theme_list), 48);
 	gtk_clist_set_column_width (GTK_CLIST(theme_selector->details->theme_list), 0, 72);
-	gtk_clist_set_column_width (GTK_CLIST(theme_selector->details->theme_list), 1, 80);
+	gtk_clist_set_column_width (GTK_CLIST(theme_selector->details->theme_list), 1, 130);
 	gtk_clist_set_column_width (GTK_CLIST(theme_selector->details->theme_list), 2, 180);
 	
-	gtk_clist_set_shadow_type  (GTK_CLIST (theme_selector->details->theme_list), GTK_SHADOW_NONE);
+	gtk_clist_set_shadow_type  (GTK_CLIST (theme_selector->details->theme_list), GTK_SHADOW_IN);
 			
 	scrollwindow = gtk_scrolled_window_new (NULL, gtk_clist_get_vadjustment (GTK_CLIST (theme_selector->details->theme_list)));
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (scrollwindow), theme_selector->details->theme_list);	
+	gtk_container_set_border_width (GTK_CONTAINER (scrollwindow), 6);
 	gtk_clist_set_selection_mode (GTK_CLIST (theme_selector->details->theme_list), GTK_SELECTION_BROWSE);
 
 	gtk_box_pack_start (GTK_BOX (theme_selector->details->container), scrollwindow, TRUE, TRUE, 0);	
 	gtk_widget_show (theme_selector->details->theme_list);
 	gtk_widget_show (scrollwindow);
 
-	/* connect a signal to let us know when the column titles are clicked */
-	gtk_signal_connect(GTK_OBJECT(theme_selector->details->theme_list), "select_row",
-				GTK_SIGNAL_FUNC(theme_select_row_callback), theme_selector);
+	/* connect a signal to let us know when a row is selected */
+	gtk_signal_connect (GTK_OBJECT (theme_selector->details->theme_list),
+			    "select_row",
+			    GTK_SIGNAL_FUNC (theme_select_row_callback),
+			    theme_selector);
+
+	/* we will have to reset some style stuff when a new style is set,
+	 * so that we follow gtk+ themes correctly */
+	gtk_signal_connect (GTK_OBJECT (theme_selector->details->theme_list),
+			    "style_set",
+			    GTK_SIGNAL_FUNC (theme_style_set_callback),
+			    theme_selector);
 
   	/* add the bottom box to hold the command buttons */
   	temp_box = gtk_event_box_new();
 	gtk_container_set_border_width (GTK_CONTAINER (temp_box), 0);				
   	gtk_widget_show(temp_box);
 
- 	background = nautilus_get_widget_background (GTK_WIDGET (temp_box));
-	nautilus_background_set_color (background, SELECTOR_TITLE_BAR_COLOR);	
-
   	temp_frame = gtk_frame_new(NULL);
-  	gtk_frame_set_shadow_type(GTK_FRAME(temp_frame), GTK_SHADOW_IN);
+  	gtk_frame_set_shadow_type(GTK_FRAME(temp_frame), GTK_SHADOW_NONE);
   	gtk_widget_show(temp_frame);
   	gtk_container_add(GTK_CONTAINER(temp_box), temp_frame);
 
-  	theme_selector->details->bottom_box = gtk_hbox_new(FALSE, 0);
-  	gtk_widget_show(theme_selector->details->bottom_box);
+  	bottom_box = gtk_hbox_new (FALSE, 0);
+  	gtk_widget_show (bottom_box);
+	gtk_container_set_border_width (GTK_CONTAINER (bottom_box), 4);
 	gtk_box_pack_end (GTK_BOX(theme_selector->details->container), temp_box, FALSE, FALSE, 0);
-  	gtk_container_add (GTK_CONTAINER (temp_frame), theme_selector->details->bottom_box);
-  	
+  	gtk_container_add (GTK_CONTAINER (temp_frame), bottom_box);
+ 
+ 	/* create the done button */
+ 	temp_button = gtk_button_new ();
+	gtk_widget_show(temp_button);
+	temp_label = gtk_label_new (_("  Done  "));
+	gtk_widget_show(temp_label);
+	gtk_container_add (GTK_CONTAINER(temp_button), temp_label);
+	gtk_box_pack_end (GTK_BOX(bottom_box), temp_button, FALSE, FALSE, 4);  
+ 	gtk_signal_connect(GTK_OBJECT (temp_button), "clicked", GTK_SIGNAL_FUNC (done_button_callback), theme_selector);
+ 	
   	/* create the "add new" button */
   	theme_selector->details->add_button = gtk_button_new ();
 	gtk_widget_show(theme_selector->details->add_button);
 	
-	theme_selector->details->add_button_label = nautilus_label_new (_("Add new theme"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (theme_selector->details->add_button_label), 14);
-
+	theme_selector->details->add_button_label = gtk_label_new (_("  Add new theme  "));
 	gtk_widget_show(theme_selector->details->add_button_label);
 	gtk_container_add (GTK_CONTAINER(theme_selector->details->add_button), theme_selector->details->add_button_label);
-	gtk_box_pack_end (GTK_BOX(theme_selector->details->bottom_box), theme_selector->details->add_button, FALSE, FALSE, 4);
+	gtk_box_pack_end (GTK_BOX (bottom_box), theme_selector->details->add_button, FALSE, FALSE, 4);
  	  
  	gtk_signal_connect(GTK_OBJECT (theme_selector->details->add_button), "clicked", GTK_SIGNAL_FUNC (add_new_theme_button_callback), theme_selector);
 	
 	/* now create the "remove" button */
   	theme_selector->details->remove_button = gtk_button_new();
-	
-	theme_selector->details->remove_button_label = nautilus_label_new (_("Remove theme"));
-	nautilus_label_set_font_size (NAUTILUS_LABEL (theme_selector->details->remove_button_label), 14);
-	
+	theme_selector->details->remove_button_label = gtk_label_new (_("  Remove theme  "));
 	gtk_widget_show(theme_selector->details->remove_button_label);
 	gtk_container_add (GTK_CONTAINER(theme_selector->details->remove_button), theme_selector->details->remove_button_label);
-	gtk_box_pack_end (GTK_BOX (theme_selector->details->bottom_box),
+	gtk_box_pack_end (GTK_BOX (bottom_box),
 			  theme_selector->details->remove_button,
 			  FALSE,
 			  FALSE,
@@ -370,7 +406,7 @@ add_theme_to_icons (GtkWidget *widget, gpointer *data)
 	g_free (temp_path);
 	
 	if (!g_file_exists (xml_path)) {
-		char *message = g_strdup_printf (_("Sorry, but %s is not a valid theme directory."), theme_path);
+		char *message = g_strdup_printf (_("Sorry, but \"%s\" is not a valid theme folder."), theme_path);
 		nautilus_error_dialog (message, _("Couldn't add theme"), GTK_WINDOW (theme_selector));
 		g_free (message);
 	} else {
@@ -399,7 +435,7 @@ add_theme_to_icons (GtkWidget *widget, gpointer *data)
 		g_free (theme_destination_path);
 		
 		if (result != GNOME_VFS_OK) {
-			char *message = g_strdup_printf (_("Sorry, but the theme %s couldn't be installed."), theme_path);
+			char *message = g_strdup_printf (_("Sorry, but the \"%s\" theme couldn't be installed."), theme_path);
 			nautilus_error_dialog (message, _("Couldn't install theme"), GTK_WINDOW (theme_selector));
 			g_free (message);
 		
@@ -441,7 +477,7 @@ add_new_theme_button_callback(GtkWidget *widget, NautilusThemeSelector *theme_se
 		GtkFileSelection *file_dialog;
 
 		theme_selector->details->dialog = gtk_file_selection_new
-			(_("Select a theme directory to add as a new theme:"));
+			(_("Select a theme folder to add as a new theme:"));
 		file_dialog = GTK_FILE_SELECTION (theme_selector->details->dialog);
 		
 		gtk_signal_connect (GTK_OBJECT (theme_selector->details->dialog),
@@ -459,28 +495,26 @@ add_new_theme_button_callback(GtkWidget *widget, NautilusThemeSelector *theme_se
 					   GTK_OBJECT(file_dialog));
 
 		gtk_window_set_position (GTK_WINDOW (file_dialog), GTK_WIN_POS_MOUSE);
+		gtk_window_set_transient_for (GTK_WINDOW (file_dialog), GTK_WINDOW (theme_selector));
+ 		gtk_window_set_wmclass (GTK_WINDOW (file_dialog), "file_selector", "Nautilus");
 		gtk_widget_show (GTK_WIDGET(file_dialog));
 	}
 }
 
-/* handle the "remove" button */
 static void
-remove_button_callback(GtkWidget *widget, NautilusThemeSelector *theme_selector)
+remove_button_callback (GtkWidget *widget, NautilusThemeSelector *theme_selector)
 {
 	if (theme_selector->details->remove_mode) {
 		return;
 	}
 	
 	theme_selector->details->remove_mode = TRUE;
-	/* change the label to the remove message */
-	nautilus_label_set_text (NAUTILUS_LABEL(theme_selector->details->help_label),
-	_("Click on a theme to remove it."));
+
+	nautilus_label_set_text (NAUTILUS_LABEL (theme_selector->details->help_label),
+				 _("Click on a theme to remove it."));
+	gtk_label_set_text (GTK_LABEL (theme_selector->details->add_button_label),
+				 _("Cancel Remove"));
 	
-	/* change the add button label */
-	nautilus_label_set_text (NAUTILUS_LABEL(theme_selector->details->add_button_label),
-	_("Cancel Remove"));
-	
-	/* regenerate the list */
 	populate_list_with_themes (theme_selector);
 }
 
@@ -523,11 +557,12 @@ static void
 set_help_label (NautilusThemeSelector *theme_selector, gboolean remove_mode)
 {
 	if (remove_mode) {
-		nautilus_label_set_text (NAUTILUS_LABEL(theme_selector->details->help_label),
+		nautilus_label_set_text (NAUTILUS_LABEL (theme_selector->details->help_label),
 					_("Click on a theme to remove it."));	
 	} else {
-		nautilus_label_set_text (NAUTILUS_LABEL(theme_selector->details->help_label),
-					_("Click on a theme to change the\nappearance of Nautilus."));	
+		nautilus_label_set_text (NAUTILUS_LABEL (theme_selector->details->help_label),
+					_("Click on a theme to change the\n"
+					  "appearance of Nautilus."));	
 
 	}
 }
@@ -536,11 +571,10 @@ static void
 exit_remove_mode (NautilusThemeSelector *theme_selector)
 {
 	theme_selector->details->remove_mode = FALSE;
-	set_help_label (theme_selector, TRUE);
+	set_help_label (theme_selector, FALSE);
 	
 	/* change the add button label back to it's normal state */
-	nautilus_label_set_text (NAUTILUS_LABEL(theme_selector->details->add_button_label), _("Add New Theme"));
-	
+	gtk_label_set_text (GTK_LABEL (theme_selector->details->add_button_label), _("Add New Theme"));
 	populate_list_with_themes (theme_selector);	
 }
 
@@ -567,7 +601,8 @@ theme_select_row_callback (GtkCList * clist, int row, int column, GdkEventButton
 		if (nautilus_strcmp (theme_name, current_theme) == 0) {
 			g_free (current_theme);
 			exit_remove_mode (theme_selector);
-			nautilus_error_dialog (_("Sorry, but you can't remove the current theme.  Please change to another theme before removing this one"),
+			nautilus_error_dialog (_("Sorry, but you can't remove the current theme. "
+						 "Please change to another theme before removing this one."),
 				       _("Can't delete current theme"),
 				       GTK_WINDOW (theme_selector));
 			theme_selector->details->handling_theme_change = FALSE;	
@@ -609,6 +644,16 @@ theme_select_row_callback (GtkCList * clist, int row, int column, GdkEventButton
 
 }
 
+/* handle changing of gtk themes */
+static void
+theme_style_set_callback (GtkWidget             *widget, 
+			  GtkStyle              *previous_style,
+			  NautilusThemeSelector *theme_selector)
+{
+	clear_style_for_all_rows (theme_selector);
+	setup_font_sizes_for_all_rows (theme_selector);
+}
+
 static gboolean
 vfs_file_exists (const char *file_uri)
 {
@@ -624,18 +669,18 @@ vfs_file_exists (const char *file_uri)
 
 /* utility routine to test for the presence of an icon file */
 static gboolean
-has_image_file(const char *path_uri, const char *dir_name, const char *image_file)
+has_image_file (const char *path_uri, const char *dir_name, const char *image_file)
 {
 	char* image_uri;
 	gboolean exists;
 	
-	image_uri = g_strdup_printf("%s/%s/%s.png", path_uri, dir_name, image_file);
+	image_uri = g_strdup_printf ("%s/%s/%s.png", path_uri, dir_name, image_file);
 	exists = vfs_file_exists (image_uri);
 	g_free (image_uri);
 	if (exists)
 		return TRUE;
 
-	image_uri = g_strdup_printf("%s/%s/%s.svg", path_uri, dir_name, image_file);
+	image_uri = g_strdup_printf ("%s/%s/%s.svg", path_uri, dir_name, image_file);
 	exists = vfs_file_exists (image_uri);
 	g_free (image_uri);
 	return exists;
@@ -643,43 +688,48 @@ has_image_file(const char *path_uri, const char *dir_name, const char *image_fil
 
 /* derive the theme description from the theme name by reading its xml file */
 static char*
-make_theme_description (const char *theme_name, const char *theme_path_uri)
+get_theme_description_and_display_name (const char *theme_name, const char *theme_path_uri, char** theme_display_name)
 {
 	char *theme_file_name, *theme_path, *theme_local_path;
 	char *description_result, *temp_str;
 	xmlDocPtr theme_document;
-	xmlNodePtr description_node;
 	
 	description_result = NULL;
+	if (theme_display_name) {
+		*theme_display_name = NULL;
+	}
 	
 	theme_file_name = g_strdup_printf ("%s.xml", theme_name);	
 	theme_local_path = gnome_vfs_get_local_path_from_uri (theme_path_uri);
 	theme_path = nautilus_make_path (theme_local_path, theme_file_name);
 	g_free (theme_local_path);
 		
-	if (theme_path) {
+	if (theme_path != NULL) {
 		/* read the xml document */
 		theme_document = xmlParseFile(theme_path);
 		
 		if (theme_document != NULL) {
-			/* fetch the description mode, of any */		
-			description_node = nautilus_xml_get_child_by_name (xmlDocGetRootElement (theme_document), "description");
-			if (description_node) {		
-				temp_str = xmlGetProp(description_node, "TEXT");
-				if (temp_str)
-					description_result = g_strdup (temp_str);
+			/* fetch the description, if any */		
+			temp_str = nautilus_xml_get_property_translated (xmlDocGetRootElement (theme_document), "description");
+			description_result = g_strdup (temp_str);
+			xmlFree (temp_str);
+			
+			if (theme_display_name) {
+				temp_str = nautilus_xml_get_property_translated (xmlDocGetRootElement (theme_document), "name");
+				*theme_display_name = g_strdup (temp_str);
+				xmlFree (temp_str);
 			}
-		
+			
 			xmlFreeDoc (theme_document);
 		}
-			
+		
 		g_free (theme_path);
 	}
-		
+	
 	g_free (theme_file_name);
 	if (description_result)
 		return description_result;
-	return g_strdup_printf (_("No information available for the  %s theme"), theme_name);
+	return g_strdup_printf (_("No description available for the \"%s\" theme"), *theme_display_name == NULL ? theme_name : *theme_display_name);
 }
 
 
@@ -704,6 +754,41 @@ set_preferred_font_for_cell (NautilusThemeSelector *theme_selector, int theme_in
 	gdk_font_unref (name_font);	
 }
 
+/* setup the font sizes for a row */
+static void
+setup_font_sizes_for_row (NautilusThemeSelector *theme_selector, int theme_index)
+{
+	set_preferred_font_for_cell (theme_selector, theme_index, 1, 18);
+	set_preferred_font_for_cell (theme_selector, theme_index, 2, 10);
+}
+
+/* setup the font sizes for all rows */
+static void
+setup_font_sizes_for_all_rows (NautilusThemeSelector *theme_selector)
+{
+	int i;
+
+	for (i = 0; i < GTK_CLIST (theme_selector->details->theme_list)->rows; i++) {
+		setup_font_sizes_for_row (theme_selector, i);
+	}
+}
+
+/* reset style for all rows */
+static void
+clear_style_for_all_rows (NautilusThemeSelector *theme_selector)
+{
+	int i;
+	GtkCList *clist;
+
+	clist = GTK_CLIST (theme_selector->details->theme_list);
+
+	for (i = 0; i < clist->rows; i++) {
+		gtk_clist_set_cell_style (clist, i, 0, NULL);
+		gtk_clist_set_cell_style (clist, i, 1, NULL);
+		gtk_clist_set_cell_style (clist, i, 2, NULL);
+	}
+}
+
 
 /* utility to add a theme folder to the list */
 
@@ -716,7 +801,7 @@ add_theme (NautilusThemeSelector *theme_selector, const char *theme_path_uri, co
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 	
-	char      *clist_entry[3];
+	char       *clist_entry[3];
 	
 	/* generate a pixbuf to represent the theme */
 	theme_pixbuf = nautilus_theme_make_selector (theme_name);
@@ -731,23 +816,28 @@ add_theme (NautilusThemeSelector *theme_selector, const char *theme_path_uri, co
 	pix_widget = GTK_WIDGET (gtk_pixmap_new (pixmap, mask));
 	gtk_widget_show (pix_widget);
 
-	/* install it in the list view */	
-	
+	/* install it in the list view */		
 	clist_entry[0] = NULL;
-	clist_entry[1] = g_strdup (theme_name);
-	clist_entry[2] = make_theme_description (theme_name, theme_path_uri);
+	clist_entry[2] = get_theme_description_and_display_name (theme_name, theme_path_uri, &clist_entry[1]);
+	if (clist_entry[1] == NULL) {
+		clist_entry[1] = g_strdup (theme_name);
+	}
 	
 	gtk_clist_append (GTK_CLIST(theme_selector->details->theme_list), clist_entry);
+
+	g_free (clist_entry[1]);
+	g_free (clist_entry[2]);
 	
 	/* set up the theme logo image */ 
 	gtk_clist_set_pixmap (GTK_CLIST(theme_selector->details->theme_list), theme_index, 0, pixmap , mask);
-	gtk_clist_set_row_data (GTK_CLIST(theme_selector->details->theme_list),
-					 theme_index, g_strdup (theme_name));
+	gtk_clist_set_row_data_full (GTK_CLIST (theme_selector->details->theme_list),
+				     theme_index,
+				     g_strdup (theme_name),
+				     g_free /*destroy notification*/);
 	
 	/* set up the fonts for the theme name and description */
 	
-	set_preferred_font_for_cell (theme_selector, theme_index, 1, 18);
-	set_preferred_font_for_cell (theme_selector, theme_index, 2, 10);
+	setup_font_sizes_for_row (theme_selector, theme_index);
 		
 	gdk_pixmap_unref (pixmap);
 	if (mask != NULL) {
