@@ -32,7 +32,7 @@
 
 #include "nautilus-application.h"
 #include "nautilus-component-adapter-factory.h"
-#include "nautilus-history-frame.h"
+#include "nautilus-signaller.h"
 #include "nautilus-window.h"
 #include <gtk/gtksignal.h>
 #include <libnautilus-extensions/nautilus-bonobo-extensions.h>
@@ -40,7 +40,12 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-undo-manager.h>
 #include <libnautilus/nautilus-view.h>
-#include <libnautilus/nautilus-zoomable.h>
+
+#include <bonobo/bonobo-zoomable-frame.h>
+#include <bonobo/bonobo-zoomable.h>
+
+/* Milliseconds */
+#define ATTACH_CLIENT_TIMEOUT	12000
 
 enum {
 	CHANGE_SELECTION,
@@ -51,10 +56,12 @@ enum {
 	LOAD_COMPLETE,
 	LOAD_PROGRESS_CHANGED,
 	LOAD_UNDERWAY,
-	OPEN_LOCATION,
-	OPEN_LOCATION_IN_NEW_WINDOW,
+	OPEN_LOCATION_FORCE_NEW_WINDOW,
+	OPEN_LOCATION_IN_THIS_WINDOW,
+	OPEN_LOCATION_PREFER_EXISTING_WINDOW,
 	TITLE_CHANGED,
 	ZOOM_LEVEL_CHANGED,
+	ZOOM_PARAMETERS_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -84,11 +91,15 @@ struct NautilusViewFrameDetails {
 
 static void nautilus_view_frame_initialize       (NautilusViewFrame      *view);
 static void nautilus_view_frame_destroy          (GtkObject              *view);
+static void nautilus_view_frame_finalize         (GtkObject              *view);
 static void nautilus_view_frame_initialize_class (NautilusViewFrameClass *klass);
+static void send_history                         (NautilusViewFrame      *view);
 
 static guint signals[LAST_SIGNAL];
 
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusViewFrame, nautilus_view_frame, NAUTILUS_TYPE_GENEROUS_BIN)
+NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusViewFrame,
+				   nautilus_view_frame,
+				   NAUTILUS_TYPE_GENEROUS_BIN)
 
 static void
 nautilus_view_frame_initialize_class (NautilusViewFrameClass *klass)
@@ -97,6 +108,7 @@ nautilus_view_frame_initialize_class (NautilusViewFrameClass *klass)
 	
 	object_class = GTK_OBJECT_CLASS (klass);
 	object_class->destroy = nautilus_view_frame_destroy;
+	object_class->finalize = nautilus_view_frame_finalize;
 	
 	signals[CHANGE_SELECTION] = gtk_signal_new
 		("change_selection",
@@ -162,22 +174,30 @@ nautilus_view_frame_initialize_class (NautilusViewFrameClass *klass)
 				    load_underway),
 		 gtk_marshal_NONE__NONE,
 		 GTK_TYPE_NONE, 0);
-	signals[OPEN_LOCATION] = gtk_signal_new
-		("open_location",
+	signals[OPEN_LOCATION_FORCE_NEW_WINDOW] = gtk_signal_new
+		("open_location_force_new_window",
 		 GTK_RUN_LAST,
 		 object_class->type,
 		 GTK_SIGNAL_OFFSET (NautilusViewFrameClass, 
-				    open_location),
-		 gtk_marshal_NONE__STRING,
-		 GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
-	signals[OPEN_LOCATION_IN_NEW_WINDOW] = gtk_signal_new
-		("open_location_in_new_window",
-		 GTK_RUN_LAST,
-		 object_class->type,
-		 GTK_SIGNAL_OFFSET (NautilusViewFrameClass, 
-				    open_location_in_new_window),
+				    open_location_force_new_window),
 		 nautilus_gtk_marshal_NONE__STRING_POINTER,
 		 GTK_TYPE_NONE, 2, GTK_TYPE_STRING, GTK_TYPE_POINTER);
+	signals[OPEN_LOCATION_IN_THIS_WINDOW] = gtk_signal_new
+		("open_location_in_this_window",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusViewFrameClass, 
+				    open_location_in_this_window),
+		 gtk_marshal_NONE__STRING,
+		 GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
+	signals[OPEN_LOCATION_PREFER_EXISTING_WINDOW] = gtk_signal_new
+		("open_location_prefer_existing_window",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusViewFrameClass, 
+				    open_location_in_this_window),
+		 gtk_marshal_NONE__STRING,
+		 GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
 	signals[TITLE_CHANGED] = gtk_signal_new
 		("title_changed",
 		 GTK_RUN_LAST,
@@ -194,6 +214,14 @@ nautilus_view_frame_initialize_class (NautilusViewFrameClass *klass)
 				    zoom_level_changed),
 		 gtk_marshal_NONE__NONE,
 		 GTK_TYPE_NONE, 0);
+	signals[ZOOM_PARAMETERS_CHANGED] = gtk_signal_new
+		("zoom_parameters_changed",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusViewFrameClass, 
+				    zoom_parameters_changed),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 	
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
@@ -204,6 +232,15 @@ nautilus_view_frame_initialize (NautilusViewFrame *view)
 	GTK_WIDGET_SET_FLAGS (view, GTK_NO_WINDOW);
 
 	view->details = g_new0 (NautilusViewFrameDetails, 1);
+
+	gtk_signal_connect_object_while_alive (nautilus_signaller_get_current (),
+					       "history_list_changed",
+					       send_history,
+					       GTK_OBJECT (view));
+	gtk_signal_connect_object_while_alive (GTK_OBJECT (nautilus_icon_factory_get ()),
+					       "icons_changed",
+					       send_history,
+					       GTK_OBJECT (view));
 }
 
 static void
@@ -215,7 +252,7 @@ nautilus_view_frame_destroy_client (NautilusViewFrame *view)
 		return;
 	}
 	
-	CORBA_exception_init(&ev);
+	CORBA_exception_init (&ev);
 	
 	g_free (view->iid);
 	view->iid = NULL;
@@ -226,19 +263,21 @@ nautilus_view_frame_destroy_client (NautilusViewFrame *view)
 	gtk_container_remove (GTK_CONTAINER (view), view->client_widget);
 	view->client_widget = NULL;
 
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		bonobo_object_release_unref (view->zoomable, &ev);
-	}
-	view->zoomable = CORBA_OBJECT_NIL;
-
 	bonobo_object_unref (view->view_frame);
 	view->view_frame = NULL;
-	/* we can NULL those since we just unref'ed them 
-	   with the aggregate view frame. */
-	view->history_frame = CORBA_OBJECT_NIL;
-	view->zoomable_frame = CORBA_OBJECT_NIL;
+
+	/* We can NULL this since we just unref'ed it as part of the
+	 * aggregate view frame.
+	 */
+	view->zoomable_frame = NULL;
 
 	CORBA_exception_free (&ev);
+	
+	if (view->details->ui_container->win != NULL) {
+		bonobo_window_deregister_dead_components (view->details->ui_container->win);
+	}
+	
+	bonobo_object_unref (BONOBO_OBJECT (view->details->ui_container));
 
 	if (view->details->check_if_view_is_gone_timeout_id != 0) {
 		g_source_remove (view->details->check_if_view_is_gone_timeout_id);
@@ -249,20 +288,31 @@ nautilus_view_frame_destroy_client (NautilusViewFrame *view)
 static void
 nautilus_view_frame_destroy (GtkObject *object)
 {
-	NautilusViewFrame *frame;
+	NautilusViewFrame *view;
 
-	frame = NAUTILUS_VIEW_FRAME (object);
+	view = NAUTILUS_VIEW_FRAME (object);
 	
-	nautilus_view_frame_destroy_client (frame);
-
-	g_free (frame->details->title);
-	g_free (frame->details->label);
-	g_free (frame->details->activation_iid);
-	g_free (frame->details);
+	nautilus_view_frame_destroy_client (view);
+	view->details->state = VIEW_FRAME_EMPTY;
 	
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
 
+static void
+nautilus_view_frame_finalize (GtkObject *object)
+{
+	NautilusViewFrame *view;
+
+	view = NAUTILUS_VIEW_FRAME (object);
+
+	g_free (view->details->title);
+	g_free (view->details->label);
+	g_free (view->details->activation_iid);
+	g_free (view->details);
+	
+	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, finalize, (object));
+}
+
 /* stimulus: successful load_client call */
 static void
 view_frame_activating (NautilusViewFrame *view)
@@ -286,7 +336,6 @@ view_frame_activating (NautilusViewFrame *view)
 	g_assert_not_reached ();
 }
 
-
 /* stimulus: unsuccessful activated_component call */
 static void
 view_frame_not_activated (NautilusViewFrame *view)
@@ -319,6 +368,7 @@ view_frame_activated (NautilusViewFrame *view)
 	switch (view->details->state) {
 	case VIEW_FRAME_ACTIVATING:
 		view->details->state = VIEW_FRAME_NO_LOCATION;
+		send_history (view);
 		return;
 	case VIEW_FRAME_EMPTY:
 	case VIEW_FRAME_NO_LOCATION:
@@ -443,9 +493,7 @@ view_frame_wait_is_over (NautilusViewFrame *view)
 	g_assert_not_reached ();
 }
 
-
 /* stimulus: report_load_complete */
-
 static void
 view_frame_loaded (NautilusViewFrame *view)
 {
@@ -475,7 +523,6 @@ view_frame_loaded (NautilusViewFrame *view)
 }
 
 /* stimulus: report_load_failed */
-
 static void
 view_frame_failed (NautilusViewFrame *view)
 {
@@ -504,6 +551,7 @@ nautilus_view_frame_handle_client_destroy (GtkWidget *widget,
 					   NautilusViewFrame *view)
 {
 	g_assert (NAUTILUS_IS_VIEW_FRAME (view));
+	bonobo_window_deregister_dead_components (view->details->ui_container->win);
 	view_frame_failed (view);
 }
 
@@ -514,6 +562,7 @@ nautilus_view_frame_handle_client_gone (GtkObject *object,
 					NautilusViewFrame *view)
 {
 	g_assert (NAUTILUS_IS_VIEW_FRAME (view));
+	bonobo_window_deregister_dead_components (view->details->ui_container->win);
 	view_frame_failed (view);
 }
 
@@ -525,12 +574,19 @@ nautilus_view_frame_new (BonoboUIContainer *ui_container,
 	
 	view_frame = NAUTILUS_VIEW_FRAME (gtk_widget_new (nautilus_view_frame_get_type (), NULL));
 	
+	bonobo_object_ref (BONOBO_OBJECT (ui_container));
 	view_frame->details->ui_container = ui_container;
 	view_frame->undo_manager = undo_manager;
 	
 	return view_frame;
 }
 
+static Nautilus_ViewFrame
+get_CORBA_object (NautilusViewFrame *view)
+{
+	return bonobo_object_corba_objref (BONOBO_OBJECT (view->client_object));
+}
+
 static gboolean
 check_if_view_is_gone (gpointer data)
 {
@@ -542,8 +598,9 @@ check_if_view_is_gone (gpointer data)
 
 	CORBA_exception_init (&ev);
 	ok = TRUE;
-	if (CORBA_Object_non_existent (bonobo_object_corba_objref (BONOBO_OBJECT (view->client_object)), &ev)) {
+	if (CORBA_Object_non_existent (get_CORBA_object (view), &ev)) {
 		view->details->check_if_view_is_gone_timeout_id = 0;
+		bonobo_window_deregister_dead_components (view->details->ui_container->win);
 		view_frame_failed (view);
 		ok = FALSE;
 	}
@@ -552,24 +609,57 @@ check_if_view_is_gone (gpointer data)
 	return ok;
 }
 
+static void
+zoom_level_changed_callback (BonoboZoomableFrame *zframe, float zoom_level, NautilusViewFrame *view)
+{
+	g_return_if_fail (zframe != NULL);
+	g_return_if_fail (BONOBO_IS_ZOOMABLE_FRAME (zframe));
+	g_return_if_fail (view != NULL);
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	gtk_signal_emit (GTK_OBJECT (view), signals[ZOOM_LEVEL_CHANGED], zoom_level);
+}
+
+static void
+zoom_parameters_changed_callback (BonoboZoomableFrame *zframe, NautilusViewFrame *view)
+{
+	g_return_if_fail (zframe != NULL);
+	g_return_if_fail (BONOBO_IS_ZOOMABLE_FRAME (zframe));
+	g_return_if_fail (view != NULL);
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	gtk_signal_emit (GTK_OBJECT (view), signals[ZOOM_PARAMETERS_CHANGED]);
+}
+
 static gboolean
-nautilus_view_frame_set_to_component (NautilusViewFrame *view, BonoboObjectClient *component)
+attach_client (NautilusViewFrame *view, BonoboObjectClient *client)
 {
 	CORBA_Environment ev;
 	Nautilus_View adapted;
 	Bonobo_Control control;
 	BonoboControlFrame *control_frame;
-	
+	NautilusComponentAdapterFactory *adapter_factory;
+	Bonobo_Zoomable zoomable;
+  	
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), FALSE);
+
+	CORBA_exception_init (&ev);
 
 	/* Either create an adapter or query for the Nautilus:View
 	 * interface. Either way, we don't need to keep the original
 	 * reference around once that happens.
 	 */
 
+	adapter_factory = nautilus_component_adapter_factory_get ();
+
+	if (adapter_factory == NULL) {
+		return FALSE;
+	}
+
 	adapted = nautilus_component_adapter_factory_create_adapter 
-		(nautilus_component_adapter_factory_get (),
-		 component);
-	bonobo_object_unref (BONOBO_OBJECT (component));
+		(adapter_factory,
+		 client);
+	bonobo_object_unref (BONOBO_OBJECT (client));
 
 	/* Handle case where we don't know how to host this component. */
   	if (adapted == CORBA_OBJECT_NIL) {
@@ -590,14 +680,26 @@ nautilus_view_frame_set_to_component (NautilusViewFrame *view, BonoboObjectClien
 		(BONOBO_OBJECT (view->client_object),
 		 "IDL:Bonobo/Control:1.0");
 	g_assert (control != CORBA_OBJECT_NIL);
-	view->zoomable = bonobo_object_query_interface
-		(BONOBO_OBJECT (view->client_object), 
-		 "IDL:Nautilus/Zoomable:1.0");
-	
+
+	/* Add a zoomable frame interface. */
+	zoomable = Bonobo_Unknown_queryInterface
+		(control, "IDL:Bonobo/Zoomable:1.0", &ev);
+	if (ev._major == CORBA_NO_EXCEPTION && !CORBA_Object_is_nil (zoomable, &ev)) {
+		view->zoomable_frame = bonobo_zoomable_frame_new ();
+
+		gtk_signal_connect (GTK_OBJECT (view->zoomable_frame), "zoom_level_changed",
+				    GTK_SIGNAL_FUNC (zoom_level_changed_callback), view);
+		gtk_signal_connect (GTK_OBJECT (view->zoomable_frame), "zoom_parameters_changed",
+				    GTK_SIGNAL_FUNC (zoom_parameters_changed_callback), view);
+
+		bonobo_zoomable_frame_bind_to_zoomable (view->zoomable_frame, zoomable);
+		bonobo_object_release_unref (zoomable, &ev);
+	}
+
 	/* Start with a view frame interface. */
 	view->view_frame = impl_Nautilus_ViewFrame__create (view, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
-		/* FIXME: Cleanup needed here. */
+		/* FIXME bugzilla.eazel.com 5041: Cleanup needed here. */
 		return FALSE;
 	}
 
@@ -608,28 +710,18 @@ nautilus_view_frame_set_to_component (NautilusViewFrame *view, BonoboObjectClien
 	bonobo_control_frame_bind_to_control (control_frame, control);
 	view->client_widget = bonobo_control_frame_get_widget (control_frame);
 
-	/* Add a zoomable frame interface. */
-	view->zoomable_frame = impl_Nautilus_ZoomableFrame__create (view, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		/* FIXME: Cleanup needed here. */
-		return FALSE;
-	}
-	bonobo_object_add_interface (BONOBO_OBJECT (view->view_frame), 
-	                             BONOBO_OBJECT (view->zoomable_frame));
-
-	/* Add a history frame interface. */
-	view->history_frame = impl_Nautilus_HistoryFrame__create (view, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		/* FIXME: Cleanup needed here. */
-		return FALSE;
-	}
-	bonobo_object_add_interface (BONOBO_OBJECT (view->view_frame), 
-	                             BONOBO_OBJECT (view->history_frame));
+ 	if (view->zoomable_frame != NULL) {
+ 		bonobo_object_add_interface (BONOBO_OBJECT (view->view_frame), 
+ 					     BONOBO_OBJECT (view->zoomable_frame));
+  
+ 		gtk_signal_emit_by_name (GTK_OBJECT (view->zoomable_frame),
+					 "zoom_parameters_changed");
+ 	}
 
 	/* Add an undo context interface. */
 	nautilus_undo_manager_add_interface
         	(view->undo_manager, BONOBO_OBJECT (view->view_frame));
-	
+
 	bonobo_object_release_unref (control, NULL);
 	
 	CORBA_exception_free (&ev);
@@ -656,7 +748,7 @@ nautilus_view_frame_set_to_component (NautilusViewFrame *view, BonoboObjectClien
 	 */
 	g_assert (view->details->check_if_view_is_gone_timeout_id == 0);
 	view->details->check_if_view_is_gone_timeout_id
-		= g_timeout_add (10000, check_if_view_is_gone, view);
+		= g_timeout_add (ATTACH_CLIENT_TIMEOUT, check_if_view_is_gone, view);
 
 	return TRUE;
 }
@@ -670,7 +762,7 @@ activation_callback (CORBA_Object object_reference, gpointer data)
 	view = (NautilusViewFrame *) data;
 
 	bonobo_object = bonobo_object_client_from_corba (object_reference);
-	nautilus_view_frame_set_to_component (view, bonobo_object);
+	attach_client (view, bonobo_object);
 
 	gtk_signal_emit (GTK_OBJECT (view), signals[CLIENT_LOADED],
 			 bonobo_object);
@@ -682,6 +774,10 @@ nautilus_view_frame_load_client_async (NautilusViewFrame *view,
 {
 	NautilusBonoboActivate *activate_structure;
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_activating (view);
 	view->details->activation_iid = g_strdup (iid);
 	activate_structure = nautilus_bonobo_activate_from_id
@@ -707,6 +803,11 @@ nautilus_view_frame_load_client (NautilusViewFrame *view, const char *iid)
 	BonoboObjectClient *component;
   	
 	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), FALSE);
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return FALSE;
+	}
+
 	g_return_val_if_fail (view->details->state == VIEW_FRAME_EMPTY, FALSE);
 
 	if (iid == NULL) {
@@ -714,6 +815,7 @@ nautilus_view_frame_load_client (NautilusViewFrame *view, const char *iid)
         }
 
 	view_frame_activating (view);
+
 	component = bonobo_object_activate (iid, 0);
 	if (component == NULL) {
 		view_frame_not_activated (view);
@@ -721,21 +823,32 @@ nautilus_view_frame_load_client (NautilusViewFrame *view, const char *iid)
         }
 
 	view->details->activation_iid = g_strdup (iid);
+
+	if (!attach_client (view, component)) {
+		view_frame_not_activated (view);
+		return FALSE;
+	}
+
 	view_frame_activated (view);
 
-	return nautilus_view_frame_set_to_component (view, component);
+	return TRUE;
 }
 
 void 
 nautilus_view_frame_stop_activation (NautilusViewFrame *view)
 {
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	nautilus_bonobo_activate_stop (view->details->activate_structure);
 	view_frame_stop_activation (view);
 	nautilus_bonobo_activate_free (view->details->activate_structure);
 	view->details->activate_structure = NULL;
 }
 
-
 void
 nautilus_view_frame_load_location (NautilusViewFrame *view,
                                    const char *location)
@@ -744,16 +857,19 @@ nautilus_view_frame_load_location (NautilusViewFrame *view,
 	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	g_free (view->details->title);
 	view->details->title = NULL;
 	gtk_signal_emit (GTK_OBJECT (view), signals[TITLE_CHANGED]);
 
 	view_frame_wait (view);
 	
-
 	/* ORBit does a bad job with Nautilus_URI, so it's not const char *. */
 	CORBA_exception_init (&ev);
-	Nautilus_View_load_location (bonobo_object_corba_objref (BONOBO_OBJECT (view->client_object)),
+	Nautilus_View_load_location (get_CORBA_object (view),
 				     (Nautilus_URI) location, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		view_frame_failed (view);
@@ -767,10 +883,13 @@ nautilus_view_frame_stop_loading (NautilusViewFrame *view)
 	CORBA_Environment ev;
 	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
 	
 	CORBA_exception_init (&ev);
-	Nautilus_View_stop_loading (bonobo_object_corba_objref (BONOBO_OBJECT (view->client_object)),
-				    &ev);
+	Nautilus_View_stop_loading (get_CORBA_object (view), &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		view_frame_failed (view);
 	}
@@ -782,16 +901,18 @@ nautilus_view_frame_selection_changed (NautilusViewFrame *view,
                                        GList *selection)
 {
 	Nautilus_URIList *uri_list;
-	
 	CORBA_Environment ev;
 	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 	
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	uri_list = nautilus_uri_list_from_g_list (selection);
 	
 	CORBA_exception_init (&ev);
-	Nautilus_View_selection_changed (bonobo_object_corba_objref (BONOBO_OBJECT (view->client_object)),
-					 uri_list, &ev);
+	Nautilus_View_selection_changed (get_CORBA_object (view), uri_list, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		view_frame_failed (view);
 	}
@@ -808,10 +929,12 @@ nautilus_view_frame_title_changed (NautilusViewFrame *view,
 	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 	
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	CORBA_exception_init (&ev);
-	Nautilus_View_title_changed (bonobo_object_corba_objref (BONOBO_OBJECT (view->client_object)),
-				     title,
-				     &ev);
+	Nautilus_View_title_changed (get_CORBA_object (view), title, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		view_frame_failed (view);
 	}
@@ -821,177 +944,190 @@ nautilus_view_frame_title_changed (NautilusViewFrame *view,
 gboolean
 nautilus_view_frame_is_zoomable (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	gboolean is_zoomable;
-	
 	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), FALSE);
-	
-	CORBA_exception_init (&ev);
-	is_zoomable = !CORBA_Object_is_nil (view->zoomable, &ev);
-	CORBA_exception_free (&ev);
-	
-	return is_zoomable;
-}
 
-gdouble
-nautilus_view_frame_get_zoom_level (NautilusViewFrame *view)
-{
-	CORBA_Environment ev;
-	double level;
-	
-	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0);
-	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		level = Nautilus_Zoomable__get_zoom_level (view->zoomable, &ev);
-	} else {
-		level = 1.0;
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return FALSE;
 	}
 
-	CORBA_exception_free (&ev);
-	
-	return level;
+	return view->zoomable_frame != NULL;
+}
+
+double
+nautilus_view_frame_get_zoom_level (NautilusViewFrame *view)
+{
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0.0);
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return 0.0;
+	}
+
+	if (view->zoomable_frame == NULL) {
+		return 0.0;
+	}
+
+	return (double) bonobo_zoomable_frame_get_zoom_level (view->zoomable_frame);
 }
 
 void
 nautilus_view_frame_set_zoom_level (NautilusViewFrame *view,
 				    double zoom_level)
 {
-	CORBA_Environment ev;
-	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
-	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		Nautilus_Zoomable__set_zoom_level (view->zoomable, zoom_level, &ev);
-	} else {
-		/* do nothing */
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
 	}
-	
-	CORBA_exception_free (&ev);
+
+	if (view->zoomable_frame == NULL) {
+		return;
+	}
+
+	bonobo_zoomable_frame_set_zoom_level (view->zoomable_frame, (float) zoom_level);
 }
 
-gdouble
+double
 nautilus_view_frame_get_min_zoom_level (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	double level;
-	
-	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0);
-	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		level = Nautilus_Zoomable__get_min_zoom_level (view->zoomable, &ev);
-	} else {
-		level = 1.0;
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0.0);
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return 0.0;
 	}
-	
-	CORBA_exception_free (&ev);
-	
-	return level;
+
+	if (view->zoomable_frame == NULL) {
+		return 0.0;
+	}
+
+	return (double) bonobo_zoomable_frame_get_min_zoom_level (view->zoomable_frame);
 }
 
 double
 nautilus_view_frame_get_max_zoom_level (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	double level;
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0.0);
 	
-	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0);
-	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		level = Nautilus_Zoomable__get_max_zoom_level (view->zoomable, &ev);
-	} else {
-		level = 1.0;
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return 0.0;
 	}
+
+	if (view->zoomable_frame == NULL) {
+		return 0.0;
+	}
+
+	return (double) bonobo_zoomable_frame_get_max_zoom_level (view->zoomable_frame);
+}
+
+gboolean
+nautilus_view_frame_get_has_min_zoom_level (NautilusViewFrame *view)
+{
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), FALSE);
 	
-	CORBA_exception_free (&ev);
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return FALSE;
+	}
+
+	if (view->zoomable_frame == NULL) {
+		return FALSE;
+	}
+
+	return bonobo_zoomable_frame_has_min_zoom_level (view->zoomable_frame);
+}
+
+gboolean
+nautilus_view_frame_get_has_max_zoom_level (NautilusViewFrame *view)
+{
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), FALSE);
 	
-	return level;
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return FALSE;
+	}
+
+	if (view->zoomable_frame == NULL) {
+		return FALSE;
+	}
+
+	return bonobo_zoomable_frame_has_max_zoom_level (view->zoomable_frame);
+}
+
+gboolean
+nautilus_view_frame_get_is_continuous (NautilusViewFrame *view)
+{
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), FALSE);
+	
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return FALSE;
+	}
+
+	if (view->zoomable_frame == NULL) {
+		return FALSE;
+	}
+
+	return bonobo_zoomable_frame_is_continuous (view->zoomable_frame);
 }
 
 GList *
 nautilus_view_frame_get_preferred_zoom_levels (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	GList *levels;
-	Nautilus_ZoomLevelList *zoom_levels;
-	
-	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0);
-	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		zoom_levels = Nautilus_Zoomable__get_preferred_zoom_levels (view->zoomable, &ev);
-		levels = nautilus_g_list_from_ZoomLevelList (zoom_levels);
-		CORBA_free (zoom_levels);
-		
-	} else {
-		levels = NULL;
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), NULL);
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return FALSE;
 	}
-	
-	CORBA_exception_free (&ev);
-	
-	return levels;
+
+	if (view->zoomable_frame == NULL) {
+		return NULL;
+	}
+
+	return bonobo_zoomable_frame_get_preferred_zoom_levels (view->zoomable_frame);
 }
 
 void
 nautilus_view_frame_zoom_in (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
-	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		Nautilus_Zoomable_zoom_in (view->zoomable, &ev);
-	} else {
-		/* do nothing */
+ 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
 	}
-	
-	CORBA_exception_free (&ev);
+
+	if (view->zoomable_frame == NULL) {
+		return;
+	}
+
+	bonobo_zoomable_frame_zoom_in (view->zoomable_frame);
 }
 
 void
 nautilus_view_frame_zoom_out (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		Nautilus_Zoomable_zoom_out (view->zoomable, &ev);
-	} else {
-		/* do nothing */
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
 	}
-	
-	CORBA_exception_free (&ev);
+
+	if (view->zoomable_frame == NULL) {
+		return;
+	}
+
+	bonobo_zoomable_frame_zoom_out (view->zoomable_frame);
 }
 
 void
 nautilus_view_frame_zoom_to_fit (NautilusViewFrame *view)
 {
-	CORBA_Environment ev;
-	
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 	
-	CORBA_exception_init (&ev);
-	
-	if (!CORBA_Object_is_nil (view->zoomable, &ev)) {
-		Nautilus_Zoomable_zoom_to_fit (view->zoomable, &ev);
-	} else {
-		/* do nothing */
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
 	}
-	
-	CORBA_exception_free (&ev);
+
+	if (view->zoomable_frame == NULL) {
+		return;
+	}
+
+	bonobo_zoomable_frame_zoom_to_fit (view->zoomable_frame);
 }
 
 const char *
@@ -1002,26 +1138,49 @@ nautilus_view_frame_get_iid (NautilusViewFrame *view)
 }
 
 void
-nautilus_view_frame_open_location (NautilusViewFrame *view,
-                                   const char *location)
+nautilus_view_frame_open_location_in_this_window (NautilusViewFrame *view,
+						  const char *location)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_wait_is_over (view);
 	gtk_signal_emit (GTK_OBJECT (view),
-			 signals[OPEN_LOCATION], location);
+			 signals[OPEN_LOCATION_IN_THIS_WINDOW], location);
 }
 
 void
-nautilus_view_frame_open_location_in_new_window (NautilusViewFrame *view,
-                                                 const char *location,
-						 GList *selection)
+nautilus_view_frame_open_location_prefer_existing_window (NautilusViewFrame *view,
+							  const char *location)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_wait_is_over (view);
 	gtk_signal_emit (GTK_OBJECT (view),
-			 signals[OPEN_LOCATION_IN_NEW_WINDOW],
+			 signals[OPEN_LOCATION_PREFER_EXISTING_WINDOW], location);
+}
+
+void
+nautilus_view_frame_open_location_force_new_window (NautilusViewFrame *view,
+						    const char *location,
+						    GList *selection)
+{
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
+	view_frame_wait_is_over (view);
+	gtk_signal_emit (GTK_OBJECT (view),
+			 signals[OPEN_LOCATION_FORCE_NEW_WINDOW],
 			 location, selection);
 }
 
@@ -1031,6 +1190,10 @@ nautilus_view_frame_report_selection_change (NautilusViewFrame *view,
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_wait_is_over (view);
 	gtk_signal_emit (GTK_OBJECT (view),
 			 signals[CHANGE_SELECTION], selection);
@@ -1042,6 +1205,10 @@ nautilus_view_frame_report_status (NautilusViewFrame *view,
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_wait_is_over (view);
 	gtk_signal_emit (GTK_OBJECT (view),
 			 signals[CHANGE_STATUS], status);
@@ -1052,6 +1219,10 @@ nautilus_view_frame_report_load_underway (NautilusViewFrame *view)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 	
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_underway (view);
 }
 
@@ -1061,6 +1232,10 @@ nautilus_view_frame_report_load_progress (NautilusViewFrame *view,
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_underway (view);
 	gtk_signal_emit (GTK_OBJECT (view),
 			 signals[LOAD_PROGRESS_CHANGED]);
@@ -1071,6 +1246,10 @@ nautilus_view_frame_report_load_complete (NautilusViewFrame *view)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_loaded (view);
 }
 
@@ -1079,6 +1258,10 @@ nautilus_view_frame_report_load_failed (NautilusViewFrame *view)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	view_frame_failed (view);
 }
 
@@ -1091,6 +1274,10 @@ nautilus_view_frame_set_title (NautilusViewFrame *view,
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 	g_return_if_fail (title != NULL);
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	/* Only do work if the title actually changed. */
 	changed = view->details->title == NULL
 		|| strcmp (view->details->title, title) != 0;
@@ -1107,24 +1294,15 @@ nautilus_view_frame_set_title (NautilusViewFrame *view,
 char *
 nautilus_view_frame_get_title (NautilusViewFrame *view)
 {
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), NULL);
+
 	return g_strdup (view->details->title);
 }
 
-void
-nautilus_view_frame_zoom_level_changed (NautilusViewFrame *view,
-                                        double level)
-{
-	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
-
-	view_frame_wait_is_over (view);
-	gtk_signal_emit (GTK_OBJECT (view), signals[ZOOM_LEVEL_CHANGED]);
-}
-
 char *
 nautilus_view_frame_get_label (NautilusViewFrame *view)
 {
 	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), NULL);
-	g_return_val_if_fail (view->details != NULL, NULL);
 
 	return g_strdup (view->details->label);
 }
@@ -1135,6 +1313,10 @@ nautilus_view_frame_set_label (NautilusViewFrame *view,
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	g_free (view->details->label);
 	view->details->label = g_strdup (label);
 }
@@ -1145,21 +1327,23 @@ nautilus_view_frame_activate (NautilusViewFrame *view)
 {
 	BonoboControlFrame *control_frame;
 	
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	if (view->details->state == VIEW_FRAME_FAILED) {
+		return;
+	}
+
 	control_frame = BONOBO_CONTROL_FRAME (bonobo_object_query_local_interface 
 					      (view->view_frame, "IDL:Bonobo/ControlFrame:1.0"));
 	bonobo_control_frame_control_activate (control_frame);
 	bonobo_object_unref (BONOBO_OBJECT (control_frame));
 }
 
-
-Nautilus_HistoryList *
-nautilus_view_frame_get_history_list (NautilusViewFrame *view)
+static Nautilus_History *
+get_history_list (NautilusViewFrame *view)
 {
-	Nautilus_HistoryList *history_list;
+	Nautilus_History *history_list;
 	
-	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), NULL);
-	
-	view_frame_wait_is_over (view);
 	history_list = NULL;
 	gtk_signal_emit (GTK_OBJECT (view), 
   			 signals[GET_HISTORY_LIST],
@@ -1167,6 +1351,41 @@ nautilus_view_frame_get_history_list (NautilusViewFrame *view)
   	return history_list;
 }
 
+static void
+send_history (NautilusViewFrame *view)
+{
+	Nautilus_History *history;
+	CORBA_Environment ev;
+
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+	
+	switch (view->details->state) {
+	case VIEW_FRAME_EMPTY:
+	case VIEW_FRAME_ACTIVATING:
+	case VIEW_FRAME_FAILED:
+		return;
+	case VIEW_FRAME_NO_LOCATION:
+	case VIEW_FRAME_WAITING:
+	case VIEW_FRAME_UNDERWAY:
+	case VIEW_FRAME_LOADED:
+		break;
+	}
+
+	history = get_history_list (view);
+	if (history == NULL) {
+		return;
+	}
+	
+	CORBA_exception_init (&ev);
+	Nautilus_View_history_changed (get_CORBA_object (view), history, &ev);
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		view_frame_failed (view);
+	}
+	CORBA_exception_free (&ev);
+	
+	CORBA_free (history);
+}
+
 gboolean
 nautilus_view_frame_get_is_underway (NautilusViewFrame *view)
 {
@@ -1177,7 +1396,6 @@ nautilus_view_frame_get_is_underway (NautilusViewFrame *view)
 	case VIEW_FRAME_ACTIVATING:
 	case VIEW_FRAME_NO_LOCATION:
 	case VIEW_FRAME_FAILED:
-		g_return_val_if_fail (FALSE, FALSE);
 	case VIEW_FRAME_WAITING:
 	case VIEW_FRAME_LOADED:
 		return FALSE;
