@@ -44,6 +44,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-extensions/nautilus-bonobo-extensions.h>
 #include <libnautilus-extensions/nautilus-debug.h>
@@ -56,7 +57,6 @@
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-undo-manager.h>
-#include <libnautilus-extensions/nautilus-user-level-manager.h>
 #include <libnautilus-extensions/nautilus-xml-extensions.h>
 #include <libnautilus/nautilus-bonobo-ui.h>
 
@@ -72,47 +72,47 @@
  * don't want other code relying on their existence.
  */
 
-#define NAUTILUS_MENU_PATH_SHOW_HIDE_SIDEBAR			"/menu/View/Show Hide Placeholder/Show Hide Sidebar"
-#define NAUTILUS_MENU_PATH_SHOW_HIDE_TOOL_BAR			"/menu/View/Show Hide Placeholder/Show Hide Tool Bar"
-#define NAUTILUS_MENU_PATH_SHOW_HIDE_LOCATION_BAR		"/menu/View/Show Hide Placeholder/Show Hide Location Bar"
-#define NAUTILUS_MENU_PATH_SHOW_HIDE_STATUS_BAR			"/menu/View/Show Hide Placeholder/Show Hide Status Bar"
+#define MENU_PATH_TOGGLE_FIND_MODE			"/menu/File/Toggle Find Mode"
 
-#define NAUTILUS_MENU_PATH_HISTORY_PLACEHOLDER			"/menu/Go/History Placeholder"
+#define MENU_PATH_SHOW_HIDE_SIDEBAR			"/menu/View/Show Hide Placeholder/Show Hide Sidebar"
+#define MENU_PATH_SHOW_HIDE_TOOL_BAR			"/menu/View/Show Hide Placeholder/Show Hide Tool Bar"
+#define MENU_PATH_SHOW_HIDE_LOCATION_BAR		"/menu/View/Show Hide Placeholder/Show Hide Location Bar"
+#define MENU_PATH_SHOW_HIDE_STATUS_BAR			"/menu/View/Show Hide Placeholder/Show Hide Status Bar"
 
-#define NAUTILUS_MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER	"/menu/Bookmarks/Built-in Bookmarks Placeholder"
-#define NAUTILUS_MENU_PATH_BOOKMARKS_PLACEHOLDER		"/menu/Bookmarks/Bookmarks Placeholder"
+#define MENU_PATH_HISTORY_PLACEHOLDER			"/menu/Go/History Placeholder"
 
-#define SWITCH_TO_BEGINNER_VERB					"Switch to Beginner Level"
-#define SWITCH_TO_INTERMEDIATE_VERB				"Switch to Intermediate Level"
-#define SWITCH_TO_ADVANCED_VERB					"Switch to Advanced Level"
+#define MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER	"/menu/Bookmarks/Built-in Bookmarks Placeholder"
+#define MENU_PATH_BOOKMARKS_PLACEHOLDER			"/menu/Bookmarks/Bookmarks Placeholder"
+
+#define COMMAND_PATH_CLOSE_WINDOW			"/commands/Close"
+
+#define SWITCH_TO_BEGINNER_VERB				"Switch to Beginner Level"
+#define SWITCH_TO_INTERMEDIATE_VERB			"Switch to Intermediate Level"
+#define SWITCH_TO_ADVANCED_VERB				"Switch to Advanced Level"
 
-#define NAUTILUS_MENU_PATH_TOGGLE_FIND_MODE			"/menu/File/Toggle Find Mode"
 
 static GtkWindow *bookmarks_window = NULL;
 
-static void                  append_bookmark_to_menu                        (NautilusWindow         *window,
-									     NautilusBookmark *bookmark,
-									     const char 	    *unique_id,
-									     const char             *menu_item_path,
-									     const char 	    *menu_item_name,
-									     gboolean		     is_in_bookmarks_menu);
-static void		     append_dynamic_bookmarks 			    (NautilusWindow *window);
-static NautilusBookmarkList *get_bookmark_list                              (void);
-static void                  refresh_go_menu                   		    (NautilusWindow         *window);
-static void                  refresh_bookmarks_menu            	    	    (NautilusWindow         *window);
-static void		     schedule_refresh_go_menu 		    	    (NautilusWindow 	    *window);
-static void		     schedule_refresh_bookmarks_menu 	    	    (NautilusWindow 	    *window);
-static void                  edit_bookmarks                                 (NautilusWindow         *window);
+static void                  append_bookmark_to_menu                       (NautilusWindow   *window,
+									    NautilusBookmark *bookmark,
+									    const char       *parent_path,
+									    guint             index_in_parent,
+									    gboolean          is_in_bookmarks_menu);
+static void                  append_dynamic_bookmarks                      (NautilusWindow   *window);
+static NautilusBookmarkList *get_bookmark_list                             (void);
+static void                  refresh_go_menu                               (NautilusWindow   *window);
+static void                  refresh_bookmarks_menu                        (NautilusWindow   *window);
+static void                  schedule_refresh_go_menu                      (NautilusWindow   *window);
+static void                  schedule_refresh_bookmarks_menu               (NautilusWindow   *window);
+static void                  edit_bookmarks                                (NautilusWindow   *window);
+static void                  add_bookmark_for_current_location             (NautilusWindow   *window);
 
 /* User level things */
 static guint                 convert_verb_to_user_level                    (const char       *verb);
 static const char *          convert_user_level_to_path                    (guint             user_level);
-static char *                get_customize_user_level_settings_menu_string (void);
 static void                  update_user_level_menu_items                  (NautilusWindow   *window);
-static char *                get_customize_user_level_string               (void);
 static void                  switch_to_user_level                          (NautilusWindow   *window,
 									    int               new_user_level);
-static void                  update_preferences_dialog_title               (void);
 
 
 #define NAUTILUS_MENU_PATH_USER_LEVEL				"/menu/Preferences"
@@ -129,16 +129,6 @@ typedef struct {
         guint changed_handler_id;
 } BookmarkHolder;
 
-
-static const char * normal_menu_paths[] = {
-	NAUTILUS_MENU_PATH_FILE_MENU,
-	NAUTILUS_MENU_PATH_EDIT_MENU,
-	NAUTILUS_MENU_PATH_VIEW_MENU,
-	NAUTILUS_MENU_PATH_GO_MENU,
-	NAUTILUS_MENU_PATH_BOOKMARKS_MENU,
-	NAUTILUS_MENU_PATH_HELP_MENU
-};
-
 static BookmarkHolder *
 bookmark_holder_new (NautilusBookmark *bookmark, 
 		     NautilusWindow *window,
@@ -199,7 +189,7 @@ file_menu_new_window_callback (BonoboUIComponent *component,
 
 	current_window = NAUTILUS_WINDOW (user_data);
 	new_window = nautilus_application_create_window (current_window->application);
-	nautilus_window_goto_uri (new_window, current_window->location);
+	nautilus_window_go_home (new_window);
 }
 
 static void
@@ -421,25 +411,25 @@ nautilus_window_update_show_hide_menu_items (NautilusWindow *window)
 
 	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
 	nautilus_bonobo_set_label (window->details->shell_ui,
-				   NAUTILUS_MENU_PATH_SHOW_HIDE_STATUS_BAR,
+				   MENU_PATH_SHOW_HIDE_STATUS_BAR,
 				   nautilus_window_status_bar_showing (window)
 				   ? _("Hide Status Bar")
 				   : _("Show Status Bar"));
 		
 	nautilus_bonobo_set_label (window->details->shell_ui,
-				   NAUTILUS_MENU_PATH_SHOW_HIDE_SIDEBAR,
+				   MENU_PATH_SHOW_HIDE_SIDEBAR,
 				   nautilus_window_sidebar_showing (window)
 				   ? _("Hide Sidebar")
 				   : _("Show Sidebar"));
 
 	nautilus_bonobo_set_label (window->details->shell_ui,
-				   NAUTILUS_MENU_PATH_SHOW_HIDE_TOOL_BAR,
+				   MENU_PATH_SHOW_HIDE_TOOL_BAR,
 				   nautilus_window_tool_bar_showing (window)
 				   ? _("Hide Tool Bar")
 				   : _("Show Tool Bar"));
 		
 	nautilus_bonobo_set_label (window->details->shell_ui,
-				   NAUTILUS_MENU_PATH_SHOW_HIDE_LOCATION_BAR,
+				   MENU_PATH_SHOW_HIDE_LOCATION_BAR,
 				   nautilus_window_location_bar_showing (window)
 				   ? _("Hide Location Bar")
 				   : _("Show Location Bar"));
@@ -475,7 +465,7 @@ bookmarks_menu_add_bookmark_callback (BonoboUIComponent *component,
 			              gpointer user_data, 
 			              const char *verb)
 {
-        nautilus_window_add_bookmark_for_current_location (NAUTILUS_WINDOW (user_data));
+        add_bookmark_for_current_location (NAUTILUS_WINDOW (user_data));
 }
 
 static void
@@ -486,75 +476,13 @@ bookmarks_menu_edit_bookmarks_callback (BonoboUIComponent *component,
         edit_bookmarks (NAUTILUS_WINDOW (user_data));
 }
 
-static void
-switch_and_show_intermediate_settings_callback (GtkWidget *button, gpointer user_data)
-{
-	g_assert (NAUTILUS_IS_WINDOW (user_data));
-
-	switch_to_user_level (NAUTILUS_WINDOW (user_data), NAUTILUS_USER_LEVEL_INTERMEDIATE);
-	nautilus_global_preferences_show_dialog ();
-}
-
 static void
 user_level_customize_callback (BonoboUIComponent *component, 
 			       gpointer user_data, 
 			       const char *verb)
 {
-	GnomeDialog *dialog;
-	NautilusWindow *window;
-	char *novice_level_name;
-	char *intermediate_level_name;
-	char *expert_level_name;
-	char *prompt;
-	char *dialog_title;
-
-	window = NAUTILUS_WINDOW (user_data);
-
-	if (nautilus_user_level_manager_get_user_level () == NAUTILUS_USER_LEVEL_NOVICE) {
-		novice_level_name = 
-			nautilus_user_level_manager_get_user_level_name_for_display 
-				(NAUTILUS_USER_LEVEL_NOVICE);
-		intermediate_level_name = 
-			nautilus_user_level_manager_get_user_level_name_for_display 
-				(NAUTILUS_USER_LEVEL_INTERMEDIATE);
-		expert_level_name = 
-			nautilus_user_level_manager_get_user_level_name_for_display 
-				(NAUTILUS_USER_LEVEL_HACKER);
-		/* Localizers: This is the message in the dialog that appears if the user chooses "Edit Beginner Settings".
-		 * The first %s is the name of the lowest user level ("Beginner"). The 2nd and 4th %s are the
-		 * names of the middle user level ("Intermediate"). The 3rd %s is the name of the highest user
-		 * level ("Advanced").
-		 */
-		prompt = g_strdup_printf (_("None of the settings for the %s level can be edited. "
-					    "If you want to edit the settings you must choose the %s or %s "
-					    "level. Do you want to switch to the %s level now "
-					    "and edit its settings?"),
-					    novice_level_name,
-					    intermediate_level_name,
-					    expert_level_name,
-					    intermediate_level_name);
-
-		/* Localizers: This is the title of the dialog that appears if the user chooses "Edit Beginner Settings".
-		 * The %s is the name of the middle user level ("Intermediate").
-		 */
-		dialog_title = g_strdup_printf (_("Switch to %s Level?"), intermediate_level_name);
-		dialog = nautilus_yes_no_dialog (prompt,
-						 dialog_title,
-						 _("Switch"),
-						 GNOME_STOCK_BUTTON_CANCEL,
-						 GTK_WINDOW (window));
-		gnome_dialog_button_connect 
-			(dialog, GNOME_OK, switch_and_show_intermediate_settings_callback, window);
-		gnome_dialog_set_default (dialog, GNOME_CANCEL);
-
-		g_free (prompt);
-		g_free (dialog_title);
-		g_free (novice_level_name);
-		g_free (intermediate_level_name);
-		g_free (expert_level_name);
-	} else {
-		nautilus_global_preferences_show_dialog ();
-	}
+	nautilus_global_preferences_set_dialog_title (_("Preferences"));
+	nautilus_global_preferences_show_dialog ();
 }
 
 static void
@@ -573,67 +501,72 @@ change_appearance_callback (BonoboUIComponent *component,
 	nautilus_theme_selector_show ();
 }
 
-static char *
-get_about_box_timestamp_string (void)
-{
-	char *timestamp;
-
-	timestamp = nautilus_get_build_timestamp ();
-	
-	return timestamp ? timestamp : g_strdup ("");
-}
-
 static void
 help_menu_about_nautilus_callback (BonoboUIComponent *component, 
 			           gpointer user_data, 
 			           const char *verb)
 {
 	static GtkWidget *about = NULL;
+	char *time_stamp;
+	const char *authors[] = {
+		"Ali Abdin",
+		"Darin Adler",
+		"Josh Barrow",
+		"Pavel Csler",
+		"J Shane Culpepper",
+		"Mike Engber",
+		"Ramiro Estrugo",
+		"Mike Fleming",
+		"Eli Goldberg",
+		"Andy Hertzfeld",
+		"Susan Kare",
+		"Mathieu Lacage",
+		"George Lebl",
+		"Elliot Lee",
+		"Raph Levien",
+		"Ian McKellar",
+		"Seth Nickell",
+		"Eskil Heyn Olsen",
+		"Ettore Perazzoli",
+		"Robey Pointer",
+		"Gene Z. Ragan",
+		"Arlo Rose",
+		"Rebecca Schulman",
+		"Robin * Slomkowski",
+		"Maciej Stachowiak",
+		"John Sullivan",
+		"Bud Tribble",
+		NULL
+	};
+
 
 	if (about == NULL) {
-
-		const char *authors[] = {
-			"Ali Abdin",
-			"Darin Adler",
-			"Josh Barrow",
-			"Pavel Csler",
-			"J Shane Culpepper",
-			"Mike Engber",
-			"Ramiro Estrugo",
-			"Mike Fleming",
-			"Eli Goldberg",
-			"Andy Hertzfeld",
-			"Susan Kare",
-			"Mathieu Lacage",
-			"George Lebl",
-			"Elliot Lee",
-			"Raph Levien",
-			"Ian McKellar",
-			"Seth Nickell",
-			"Eskil Heyn Olsen",
-			"Ettore Perazzoli",
-			"Robey Pointer",
-			"Gene Z. Ragan",
-			"Arlo Rose",
-			"Rebecca Schulman",
-			"Robin * Slomkowski",
-			"Maciej Stachowiak",
-			"John Sullivan",
-			"Bud Tribble",
-			NULL
-		};
-
-		char *timestamp = get_about_box_timestamp_string ();
-		g_assert (timestamp != NULL);
-
+		time_stamp = nautilus_get_build_time_stamp ();
+		if (time_stamp == NULL) {
+			time_stamp = g_strdup ("");
+		}
+		
+		/* The copyright character in here is correct for
+		 * Latin-1 encoding, but not for UTF-8, so we have
+		 * to change it when we upgrade to GTK 2.0.
+		 */
 		about = nautilus_about_new (_("Nautilus"),
 					    VERSION,
-					    "(C) 1999-2000 Eazel, Inc.",
+					    /* Localize to deal with
+					     * issues in the copyright
+					     * symbol characters -- do
+					     * not translate the
+					     * company name, please.
+					     */
+					    _("Copyright \xA9 1999-2001 Eazel, Inc."),
 					    authors,
-					    _("Nautilus is a graphical shell \nfor GNOME that makes it \neasy to manage your files \nand the rest of your system."),
-					    timestamp);
+					    _("Nautilus is a graphical shell\n"
+					      "for GNOME that makes it\n"
+					      "easy to manage your files\n"
+					      "and the rest of your system."),
+					    time_stamp);
 		
-		g_free (timestamp);
+		g_free (time_stamp);
 	} else {
 		nautilus_about_update_authors (NAUTILUS_ABOUT (about));
 	}
@@ -711,12 +644,12 @@ switch_to_user_level (NautilusWindow *window, int new_user_level)
 		return;
 	}
 
-	old_user_level = nautilus_user_level_manager_get_user_level ();
+	old_user_level = nautilus_preferences_get_user_level ();
 	if (new_user_level == old_user_level) {
 		return;
 	}
 
-	nautilus_user_level_manager_set_user_level (new_user_level);
+	nautilus_preferences_set_user_level (new_user_level);
 	
 	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
 
@@ -828,14 +761,13 @@ activate_bookmark_in_menu_item (BonoboUIComponent *component, gpointer user_data
 static void
 append_bookmark_to_menu (NautilusWindow *window, 
                          NautilusBookmark *bookmark, 
-			 const char *unique_id,
-                         const char *menu_item_path,
-                         const char *menu_item_name,
+                         const char *parent_path,
+			 guint index_in_parent,
                          gboolean is_bookmarks_menu)
 {
 	BookmarkHolder *bookmark_holder;		
 	char *raw_name, *display_name, *truncated_name, *verb_name;
-	char *escaped_id, *ui_path;
+	char *ui_path;
 	GdkPixbuf *pixbuf;
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
@@ -856,52 +788,86 @@ append_bookmark_to_menu (NautilusWindow *window,
 
 	/* Create menu item with pixbuf */
 	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, NAUTILUS_ICON_SIZE_FOR_MENUS);
-	nautilus_bonobo_add_menu_item (window->details->shell_ui, unique_id, menu_item_path, display_name, pixbuf);
+	nautilus_bonobo_add_numbered_menu_item 
+		(window->details->shell_ui, 
+		 parent_path, 
+		 index_in_parent, 
+		 display_name, 
+		 pixbuf);
 	gdk_pixbuf_unref (pixbuf);
+	g_free (display_name);
 	
 	/* Add the status tip */
-	escaped_id = gnome_vfs_escape_string (unique_id);	
-	ui_path = g_strdup_printf ("%s/%s", menu_item_path, escaped_id);
-	nautilus_bonobo_set_tip (window->details->shell_ui, ui_path, _("Go to the specified location"));
+	ui_path = nautilus_bonobo_get_numbered_menu_item_path
+		(window->details->shell_ui, parent_path, index_in_parent);
+	nautilus_bonobo_set_tip (window->details->shell_ui, ui_path, _("Go to the location specified by this bookmark"));
 	g_free (ui_path);
 			
 	/* Add verb to new bookmark menu item */
-	ui_path = g_strdup_printf ("%s/%s", menu_item_path, unique_id);	
-	verb_name = nautilus_bonobo_get_menu_item_verb_name (ui_path);
+	verb_name = nautilus_bonobo_get_numbered_menu_item_command 
+		(window->details->shell_ui, parent_path, index_in_parent);
 	bonobo_ui_component_add_verb_full (window->details->shell_ui, verb_name, 
 					   activate_bookmark_in_menu_item, bookmark_holder, 
 					   bookmark_holder_free_cover);
-	g_free (escaped_id);
-	g_free (ui_path);
-	g_free (display_name);
 	g_free (verb_name);
 }
 
 static char *
 get_static_bookmarks_file_path (void)
 {
-	char *xml_file_path, *user_directory_path;
-	
-	/* first, try to fetch it from the service update directory. Use the one from there
-	 * if there is one, otherwise, get the built-in one from shared data
-	 */
+	char *update_xml_file_path, *built_in_xml_file_path;
+	char *update_uri, *built_in_uri;
+	char *user_directory_path;
+	gboolean update_exists, built_in_exists;
+	GnomeVFSFileInfo update_info, built_in_info;
 	
+	/* see if there is a static bookmarks file in the updates directory and get its mod-date */
 	user_directory_path = nautilus_get_user_directory ();
-	xml_file_path = g_strdup_printf ("%s/updates/%s", user_directory_path, STATIC_BOOKMARKS_FILE_NAME);
+	update_xml_file_path = g_strdup_printf ("%s/updates/%s", user_directory_path, STATIC_BOOKMARKS_FILE_NAME);
+	update_exists = g_file_exists (update_xml_file_path);
 	g_free (user_directory_path);
-	if (g_file_exists (xml_file_path)) {
-		return xml_file_path;
-	}
 	
-	g_free (xml_file_path);
+	/* get the mod date of the built-in static bookmarks file */
+	built_in_xml_file_path = nautilus_make_path (NAUTILUS_DATADIR, STATIC_BOOKMARKS_FILE_NAME);
+	built_in_exists = g_file_exists (built_in_xml_file_path);
 	
-	xml_file_path = nautilus_make_path (NAUTILUS_DATADIR, STATIC_BOOKMARKS_FILE_NAME);
-	if (g_file_exists (xml_file_path)) {
-		return xml_file_path;
+	/* if we only have one file, return its path as the one to use */
+	if (built_in_exists && !update_exists) {
+		g_free (update_xml_file_path);
+		return built_in_xml_file_path;
 	}
-	g_free (xml_file_path);
 
-	return NULL;
+	if (!built_in_exists && update_exists) {
+		g_free (built_in_xml_file_path);
+		return update_xml_file_path;
+	}
+	
+	/* if we have neither file, return NULL */		
+	if (!built_in_exists && !update_exists) {
+		g_free (built_in_xml_file_path);
+		g_free (update_xml_file_path);
+		return NULL;
+	}
+	
+	/* both files exist, so use the one with the most recent mod-date */
+	update_uri = gnome_vfs_get_local_path_from_uri (update_xml_file_path);
+	gnome_vfs_file_info_init (&update_info);
+	gnome_vfs_get_file_info (update_uri, &update_info, GNOME_VFS_FILE_INFO_DEFAULT);
+	g_free (update_uri);
+	
+	built_in_uri = gnome_vfs_get_local_path_from_uri (built_in_xml_file_path);
+	gnome_vfs_file_info_init (&built_in_info);
+	gnome_vfs_get_file_info (built_in_uri, &built_in_info, GNOME_VFS_FILE_INFO_DEFAULT);
+	g_free (built_in_uri);
+
+	/* see which is most recent */
+	if (update_info.mtime > built_in_info.mtime) {
+		g_free (built_in_xml_file_path);
+		return update_xml_file_path;
+	} 
+		
+	g_free (update_xml_file_path);
+	return built_in_xml_file_path;
 }
 
 static void
@@ -919,20 +885,18 @@ create_menu_item_from_node (NautilusWindow *window,
 	NautilusBookmark *bookmark;
 	xmlChar *xml_folder_name;
 	int sub_index;
-	char *index_as_string, *sub_menu_path, *escaped_name;
+	char *sub_menu_path, *escaped_name;
 	
 	g_assert (NAUTILUS_IS_WINDOW (window));
 		
 	if (strcmp (node->name, "bookmark") == 0) {
-		index_as_string = g_strdup_printf ("item_%d", index);
 		bookmark = nautilus_bookmark_new_from_node (node);
-		append_bookmark_to_menu (window, bookmark, index_as_string, menu_path, menu_path, TRUE);		
+		append_bookmark_to_menu (window, bookmark, menu_path, index, TRUE);		
 		gtk_object_unref (GTK_OBJECT (bookmark));
-		g_free (index_as_string);
 	} else if (strcmp (node->name, "separator") == 0) {
 		append_separator (window, menu_path);
 	} else if (strcmp (node->name, "folder") == 0) {
-		xml_folder_name = xmlGetProp (node, "name");
+		xml_folder_name = nautilus_xml_get_property_translated (node, "name");
 		nautilus_bonobo_add_submenu (window->details->shell_ui, menu_path, xml_folder_name);
 
 		/* Construct path and make sure it is escaped properly */
@@ -972,7 +936,7 @@ append_static_bookmarks (NautilusWindow *window, const char *menu_path)
 
 	node = nautilus_xml_get_root_children (doc);
 	index = 0;
-	
+
 	for (index = 0; node != NULL; node = node->next, ++index) {
 		create_menu_item_from_node (window, node, menu_path, index);
 	}
@@ -1023,13 +987,13 @@ nautilus_bookmarks_exiting (void)
 }
 
 /**
- * nautilus_window_add_bookmark_for_current_location
+ * add_bookmark_for_current_location
  * 
  * Add a bookmark for the displayed location to the bookmarks menu.
  * Does nothing if there's already a bookmark for the displayed location.
  */
-void
-nautilus_window_add_bookmark_for_current_location (NautilusWindow *window)
+static void
+add_bookmark_for_current_location (NautilusWindow *window)
 {
 	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
@@ -1043,6 +1007,12 @@ edit_bookmarks (NautilusWindow *window)
 		(get_or_create_bookmarks_window (GTK_OBJECT (window)));
 }
 
+void
+nautilus_window_bookmarks_preference_changed_callback (gpointer user_data)
+{
+	refresh_bookmarks_menu (NAUTILUS_WINDOW (user_data));
+}
+
 static void
 refresh_bookmarks_menu (NautilusWindow *window)
 {
@@ -1051,50 +1021,15 @@ refresh_bookmarks_menu (NautilusWindow *window)
 	/* Unregister any pending call to this function. */
 	nautilus_window_remove_bookmarks_menu_callback (window);
 
+	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
 	nautilus_window_remove_bookmarks_menu_items (window);				
 
-	if (!nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS, 
-					       FALSE)) {
-		append_static_bookmarks (window, NAUTILUS_MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER);
+	if (!nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS)) {
+		append_static_bookmarks (window, MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER);
 	}
 
 	append_dynamic_bookmarks (window);
-}
-
-static void
-remove_underline_accelerator_from_menu_title (NautilusWindow *window, 
-					      const char *menu_path)
-{
-	char *old_label;
-	char *new_label;
-
-	old_label = nautilus_bonobo_get_label (window->details->shell_ui, menu_path);
-	new_label = nautilus_str_strip_chr (old_label, '_');
-	nautilus_bonobo_set_label (window->details->shell_ui, menu_path, new_label);
-
-	g_free (old_label);
-	g_free (new_label);
-}					      
-
-/**
- * nautilus_window_disable_keyboard_navigation_for_menus
- * 
- * Prevents alt-key shortcuts from pulling down menus.
- */
-void
-nautilus_window_disable_keyboard_navigation_for_menus (NautilusWindow *window)
-{
-	int index;
-
-#ifdef UIH
-	/* FIXME bugzilla.eazel.com 2327: 
-	 * This trick doesn't work with the new Bonobo UI code. 
-	 * Changing the label to one without an underscore does not remove the accelerator.
-	 */
-#endif /* Just to remind us where this bug came from */
-	for (index = 0; index < NAUTILUS_N_ELEMENTS (normal_menu_paths); ++index) {
-		remove_underline_accelerator_from_menu_title (window, normal_menu_paths[index]);
-	}
+	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 }
 
 /**
@@ -1115,7 +1050,7 @@ nautilus_window_initialize_bookmarks_menu (NautilusWindow *window)
 	 * showing static bookmarks changes.
 	 */
 	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS,
-					   (NautilusPreferencesCallback)refresh_bookmarks_menu,
+					   nautilus_window_bookmarks_preference_changed_callback,
 					   window);
 		
 	/* Recreate dynamic part of menu if bookmark list changes */
@@ -1142,7 +1077,7 @@ nautilus_window_initialize_go_menu (NautilusWindow *window)
 	/* Recreate bookmarks part of menu if history list changes
 	 * or if icon theme changes.
 	 */
-	gtk_signal_connect_object_while_alive (GTK_OBJECT (nautilus_signaller_get_current ()),
+	gtk_signal_connect_object_while_alive (nautilus_signaller_get_current (),
 			                       "history_list_changed",
 			                       schedule_refresh_go_menu,
 			   	               GTK_OBJECT (window));
@@ -1151,28 +1086,17 @@ nautilus_window_initialize_go_menu (NautilusWindow *window)
 					       "icons_changed",
 					       schedule_refresh_go_menu,
 					       GTK_OBJECT (window));
-
 }
 
 /* handler to receive the user_level_changed signal, so we can update the menu and dialog
    when the user level changes */
 static void
-user_level_changed_callback (GtkObject	*user_level_manager,
-			     gpointer	user_data)
+user_level_changed_callback (gpointer callback_data)
 {
-	g_return_if_fail (user_data != NULL);
-	g_return_if_fail (NAUTILUS_IS_WINDOW (user_data));
+	g_return_if_fail (callback_data != NULL);
+	g_return_if_fail (NAUTILUS_IS_WINDOW (callback_data));
 
-	update_user_level_menu_items (NAUTILUS_WINDOW (user_data));
-
-	/* Hide the customize dialog for novice user level */
-	if (nautilus_user_level_manager_get_user_level () == NAUTILUS_USER_LEVEL_NOVICE) {
-		nautilus_global_preferences_hide_dialog ();
-	}
-	/* Otherwise update its title to reflect the user level */
-	else {
-		update_preferences_dialog_title ();
-	}
+	update_user_level_menu_items (NAUTILUS_WINDOW (callback_data));
 }
 
 static void
@@ -1188,7 +1112,7 @@ add_user_level_menu_item (NautilusWindow *window,
 		return;
 	}
 
-	current_user_level = nautilus_user_level_manager_get_user_level ();
+	current_user_level = nautilus_preferences_get_user_level ();
 
 	icon_name = get_user_level_icon_name (user_level, current_user_level == user_level);
 
@@ -1267,12 +1191,13 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 		BONOBO_UI_VERB_END
 	};
 
+	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
+
 	bonobo_ui_component_add_verb_list_with_data (window->details->shell_ui, verbs, window);
 
         nautilus_window_update_find_menu_item (window);
         nautilus_window_update_show_hide_menu_items (window);
 
-	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
 	add_user_level_menu_item (window, NAUTILUS_MENU_PATH_NOVICE_ITEM, 
 				  NAUTILUS_USER_LEVEL_NOVICE);
 	add_user_level_menu_item (window, NAUTILUS_MENU_PATH_INTERMEDIATE_ITEM, 
@@ -1282,13 +1207,11 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 	update_user_level_menu_items (window);				  
 	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 
-	/* connect to the user level changed signal, so we can update the menu when the
-	   user level changes */
-	gtk_signal_connect_while_alive (GTK_OBJECT (nautilus_user_level_manager_get ()),
-					"user_level_changed",
-					user_level_changed_callback,
-					window,
-					GTK_OBJECT (window));
+	/* Monitor user level changes so that we can update the user level menu pane */
+	nautilus_preferences_add_callback_while_alive ("user_level",
+						       user_level_changed_callback,
+						       window,
+						       GTK_OBJECT (window));
 
         nautilus_window_initialize_bookmarks_menu (window);
         nautilus_window_initialize_go_menu (window);
@@ -1319,20 +1242,20 @@ nautilus_window_remove_go_menu_callback (NautilusWindow *window)
 void
 nautilus_window_remove_bookmarks_menu_items (NautilusWindow *window)
 {
-	nautilus_bonobo_remove_menu_items_and_verbs 
+	nautilus_bonobo_remove_menu_items_and_commands
 		(window->details->shell_ui, 
-		 NAUTILUS_MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER);					   
-	nautilus_bonobo_remove_menu_items_and_verbs 
+		 MENU_PATH_BUILT_IN_BOOKMARKS_PLACEHOLDER);					   
+	nautilus_bonobo_remove_menu_items_and_commands 
 		(window->details->shell_ui, 
-		 NAUTILUS_MENU_PATH_BOOKMARKS_PLACEHOLDER);					   
+		 MENU_PATH_BOOKMARKS_PLACEHOLDER);					   
 }
 
 void
 nautilus_window_remove_go_menu_items (NautilusWindow *window)
 {
-	nautilus_bonobo_remove_menu_items_and_verbs 
+	nautilus_bonobo_remove_menu_items_and_commands 
 		(window->details->shell_ui, 
-		 NAUTILUS_MENU_PATH_HISTORY_PLACEHOLDER);
+		 MENU_PATH_HISTORY_PLACEHOLDER);
 }
 
 void
@@ -1348,7 +1271,7 @@ nautilus_window_update_find_menu_item (NautilusWindow *window)
 			: _("_Find"));
 
 	nautilus_bonobo_set_label (window->details->shell_ui, 
-				   NAUTILUS_MENU_PATH_TOGGLE_FIND_MODE,
+				   MENU_PATH_TOGGLE_FIND_MODE,
 				   label_string);
 	g_free (label_string);
 }
@@ -1360,7 +1283,6 @@ append_dynamic_bookmarks (NautilusWindow *window)
         NautilusBookmarkList *bookmarks;
 	guint bookmark_count;
 	guint index;
-	char *name;
 		
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
@@ -1369,14 +1291,11 @@ append_dynamic_bookmarks (NautilusWindow *window)
 	/* append new set of bookmarks */
 	bookmark_count = nautilus_bookmark_list_length (bookmarks);
 	for (index = 0; index < bookmark_count; ++index) {
-		name = g_strdup_printf ("Bookmark%d", index);
 		append_bookmark_to_menu (window,
 		                         nautilus_bookmark_list_item_at (bookmarks, index),
-		                         name,
-		                         NAUTILUS_MENU_PATH_BOOKMARKS_PLACEHOLDER,
-		                         name,
+		                         MENU_PATH_BOOKMARKS_PLACEHOLDER,
+		                         index,
 		                         TRUE);
-                g_free (name);
 	}
 }
 
@@ -1414,13 +1333,14 @@ refresh_go_menu (NautilusWindow *window)
 {
 	GList *node;
 	int index;
-	char *name;
 	
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	/* Unregister any pending call to this function. */
 	nautilus_window_remove_go_menu_callback (window);
 
+	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
+
 	/* Remove old set of history items. */
 	nautilus_window_remove_go_menu_items (window);
 
@@ -1428,15 +1348,14 @@ refresh_go_menu (NautilusWindow *window)
 	for (node = nautilus_get_history_list (), index = 0;
 	     node != NULL;
 	     node = node->next, index++) {
-		name = g_strdup_printf ("History%d", index); 
 		append_bookmark_to_menu (window,
 					 NAUTILUS_BOOKMARK (node->data),
-					 name,
-					 NAUTILUS_MENU_PATH_HISTORY_PLACEHOLDER,
-					 name,
+					 MENU_PATH_HISTORY_PLACEHOLDER,
+					 index,
 					 FALSE);
-		g_free (name);
 	}
+
+	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 }
 
 static gboolean
@@ -1465,7 +1384,6 @@ schedule_refresh_go_menu (NautilusWindow *window)
 static void
 update_user_level_menu_items (NautilusWindow *window)
 {
-	char *customize_string;
 	int user_level;
 	char *user_level_icon_name;
 	
@@ -1473,22 +1391,13 @@ update_user_level_menu_items (NautilusWindow *window)
         g_assert (NAUTILUS_IS_WINDOW (window));
 
  	/* Update the user radio group to reflect reality */
-	user_level = nautilus_user_level_manager_get_user_level ();
+	user_level = nautilus_preferences_get_user_level ();
 	user_level_icon_name = get_user_level_icon_name (user_level, FALSE);
 
 	nautilus_bonobo_set_icon (window->details->shell_ui,
 				  NAUTILUS_MENU_PATH_USER_LEVEL,
 				  user_level_icon_name);
 	g_free (user_level_icon_name);
-
-
-	customize_string = get_customize_user_level_settings_menu_string ();
-	g_assert (customize_string != NULL);
-	nautilus_bonobo_set_label (window->details->shell_ui,
-				   NAUTILUS_MENU_PATH_USER_LEVEL_CUSTOMIZE,
-				   customize_string);
-
-	g_free (customize_string);
 }
 
 
@@ -1524,60 +1433,3 @@ convert_user_level_to_path (guint user_level)
 	g_assert_not_reached ();
 	return NAUTILUS_MENU_PATH_NOVICE_ITEM;
 }
-
-
-static char *
-get_customize_user_level_string (void)
-{
-	char *user_level_string_for_display;
-	char *title;
-	
-	user_level_string_for_display = 
-		nautilus_user_level_manager_get_user_level_name_for_display 
-			(nautilus_user_level_manager_get_user_level ());
-
-	/* Localizers: This is the title of the user-level settings
-	 * dialog. %s will be replaced with the name of a user level
-	 * ("Beginner", "Intermediate", or "Advanced").
-	 */
-	title = g_strdup_printf (_("Edit %s Settings"), user_level_string_for_display);
-
-	g_free (user_level_string_for_display);
-
-	return title;
-}
-
-static char *
-get_customize_user_level_settings_menu_string (void)
-{
-	char *user_level_string_for_display;
-	char *title;
-	
-	user_level_string_for_display = 
-		nautilus_user_level_manager_get_user_level_name_for_display 
-			(nautilus_user_level_manager_get_user_level ());
-
-	/* Localizers: This is the label for the menu item that brings
-	 * up the user-level settings dialog. %s will be replaced with
-	 * the name of a user level ("Beginner", "Intermediate", or
-	 * "Advanced").
-	 */
-	title = g_strdup_printf (_("Edit %s Settings..."), user_level_string_for_display);
-
-	g_free (user_level_string_for_display);
-
-	return title;
-}
-
-static void
-update_preferences_dialog_title (void)
-{
-	char *dialog_title;
-	
-	dialog_title = get_customize_user_level_string ();
-	g_assert (dialog_title != NULL);
-	
-	nautilus_global_preferences_set_dialog_title (dialog_title);
-
-	g_free (dialog_title);
-}
