@@ -36,15 +36,16 @@
 #include "nautilus-sidebar.h"
 #include "nautilus-signaller.h"
 #include "nautilus-switchable-navigation-bar.h"
-#include "nautilus-throbber.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-service-ui.h"
 #include "nautilus-zoom-control.h"
 #include <bonobo/bonobo-ui-util.h>
+#include <bonobo/bonobo-exception.h>
 #include <ctype.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkmenubar.h>
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkvbox.h>
@@ -55,6 +56,7 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-extensions/nautilus-bonobo-extensions.h>
+#include <libnautilus-extensions/nautilus-drag-window.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-gdk-extensions.h>
 #include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
@@ -83,7 +85,7 @@
 #include "nautilus-desktop-window.h"
 
 /* Milliseconds */
-#define STATUS_BAR_CLEAR_TIMEOUT 5000
+#define STATUS_BAR_CLEAR_TIMEOUT 10000
 
 /* dock items */
 #define LOCATION_BAR_PATH	"/Location Bar"
@@ -102,30 +104,28 @@ enum {
 	ARG_APP
 };
 
-/* Other static variables */
 static GList *history_list = NULL;
 
-static void nautilus_window_initialize_class        (NautilusWindowClass *klass);
-static void nautilus_window_initialize              (NautilusWindow      *window);
-static void nautilus_window_destroy                 (GtkObject           *object);
-static void nautilus_window_set_arg                 (GtkObject           *object,
-						     GtkArg              *arg,
-						     guint                arg_id);
-static void nautilus_window_get_arg                 (GtkObject           *object,
-						     GtkArg              *arg,
-						     guint                arg_id);
-static void nautilus_window_size_request            (GtkWidget           *widget,
-						     GtkRequisition      *requisition);
-static void nautilus_window_realize                 (GtkWidget           *widget);
-static void update_sidebar_panels_from_preferences  (NautilusWindow      *window);
-static void sidebar_panels_changed_callback         (gpointer             user_data);
-static void nautilus_window_show                    (GtkWidget           *widget);
-static void nautilus_window_real_load_content_view_menu (NautilusFile    *file, 
-							 gpointer        callback_data);
+static void nautilus_window_initialize_class       (NautilusWindowClass *klass);
+static void nautilus_window_initialize             (NautilusWindow      *window);
+static void nautilus_window_destroy                (GtkObject           *object);
+static void nautilus_window_set_arg                (GtkObject           *object,
+						    GtkArg              *arg,
+						    guint                arg_id);
+static void nautilus_window_get_arg                (GtkObject           *object,
+						    GtkArg              *arg,
+						    guint                arg_id);
+static void nautilus_window_size_request           (GtkWidget           *widget,
+						    GtkRequisition      *requisition);
+static void nautilus_window_realize                (GtkWidget           *widget);
+static void update_sidebar_panels_from_preferences (NautilusWindow      *window);
+static void sidebar_panels_changed_callback        (gpointer             user_data);
+static void nautilus_window_show                   (GtkWidget           *widget);
+static void cancel_view_as_callback                (NautilusWindow      *window);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusWindow,
 				   nautilus_window,
-				   BONOBO_TYPE_WIN)
+				   BONOBO_TYPE_WINDOW)
 
 static void
 nautilus_window_initialize_class (NautilusWindowClass *klass)
@@ -330,6 +330,104 @@ set_initial_window_geometry (NautilusWindow *window)
 
 }
 
+static void
+menu_bar_no_resize_hack_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+{
+	/* do nothing */
+}
+
+static void
+menu_bar_no_resize_hack_menu_bar_finder (GtkWidget *child, gpointer data)
+{
+	if (GTK_IS_MENU_BAR (child)) {
+		* (GtkObject **) data = GTK_OBJECT (child);
+	}
+}
+
+/* Since there's only one desktop at a time, we can keep track
+ * of our faux-class with a single static.
+ */
+static GtkObjectClass *menu_bar_no_resize_hack_class;
+
+static void
+menu_bar_no_resize_hack_class_free (void)
+{
+	g_free (menu_bar_no_resize_hack_class);
+}
+
+/* This fn is used to keep the desktop menu bar from resizing.
+ * It patches out its class with one where the size_allocate
+ * method has been replaced with a no-op.
+ */
+static void
+menu_bar_no_resize_hack (NautilusWindow *window)
+{
+	GtkObject *menu_bar;
+	GtkTypeQuery *type_query;
+	
+	menu_bar = NULL;
+	
+	nautilus_gtk_container_foreach_deep (GTK_CONTAINER (window),
+					     menu_bar_no_resize_hack_menu_bar_finder,
+					     &menu_bar);
+
+	g_return_if_fail (menu_bar != NULL);
+
+	if (menu_bar_no_resize_hack_class == NULL) {
+		g_atexit (menu_bar_no_resize_hack_class_free);
+	}
+	
+	type_query = gtk_type_query (menu_bar->klass->type);
+	g_free (menu_bar_no_resize_hack_class);
+	menu_bar_no_resize_hack_class = g_memdup (menu_bar->klass, type_query->class_size);
+	g_free (type_query);
+	
+	((GtkWidgetClass *) menu_bar_no_resize_hack_class)->size_allocate
+		= menu_bar_no_resize_hack_size_allocate;
+
+	menu_bar->klass = menu_bar_no_resize_hack_class;
+}
+
+static gboolean
+location_change_at_idle_callback (gpointer callback_data)
+{
+	NautilusWindow *window;
+	char *location;
+
+	window = NAUTILUS_WINDOW (callback_data);
+
+	location = window->details->location_to_change_to_at_idle;
+	window->details->location_to_change_to_at_idle = NULL;
+	window->details->location_change_at_idle_id = 0;
+
+	nautilus_window_goto_uri (window, location);
+	g_free (location);
+
+	return FALSE;
+}
+
+/* handle bonobo events from the throbber -- since they can come in at
+   any time right in the middle of things, defer until idle */
+static void 
+throbber_location_change_request_callback (BonoboListener *listener,
+					   char *event_name, 
+					   CORBA_any *arg,
+					   CORBA_Environment *ev,
+					   gpointer callback_data)
+{
+	NautilusWindow *window;
+
+	window = NAUTILUS_WINDOW (callback_data);
+
+	g_free (window->details->location_to_change_to_at_idle);
+	window->details->location_to_change_to_at_idle = g_strdup (BONOBO_ARG_GET_STRING (arg));
+
+	if (window->details->location_change_at_idle_id == 0) {
+		window->details->location_change_at_idle_id =
+			gtk_idle_add (location_change_at_idle_callback, window);
+	}
+}
+
 static void
 nautilus_window_constructed (NautilusWindow *window)
 {
@@ -337,11 +435,14 @@ nautilus_window_constructed (NautilusWindow *window)
 	GtkWidget *view_as_menu_vbox;
   	int sidebar_width;
 	BonoboControl *location_bar_wrapper;
-
+	EPaned *panel;
+	CORBA_Environment ev;
+	Bonobo_PropertyBag property_bag;
+	
 	/* CORBA and Bonobo setup, which must be done before the location bar setup */
 	window->details->ui_container = bonobo_ui_container_new ();
 	bonobo_ui_container_set_win (window->details->ui_container,
-				     BONOBO_WIN (window));
+				     BONOBO_WINDOW (window));
 
 	/* Load the user interface from the XML file. */
 	window->details->shell_ui = bonobo_ui_component_new ("Nautilus Shell");
@@ -391,10 +492,14 @@ nautilus_window_constructed (NautilusWindow *window)
 	 * It gets shown later, if the view-frame contains something zoomable.
 	 */
 	window->zoom_control = nautilus_zoom_control_new ();
-	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_in", nautilus_window_zoom_in, GTK_OBJECT (window));
-	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_out", nautilus_window_zoom_out, GTK_OBJECT (window));
-	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_to_level", nautilus_window_zoom_to_level, GTK_OBJECT (window));
-	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_to_fit", nautilus_window_zoom_to_fit, GTK_OBJECT (window));
+	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_in",
+				   nautilus_window_zoom_in, GTK_OBJECT (window));
+	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_out",
+				   nautilus_window_zoom_out, GTK_OBJECT (window));
+	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_to_level",
+				   nautilus_window_zoom_to_level, GTK_OBJECT (window));
+	gtk_signal_connect_object (GTK_OBJECT (window->zoom_control), "zoom_to_fit",
+				   nautilus_window_zoom_to_fit, GTK_OBJECT (window));
 	gtk_box_pack_end (GTK_BOX (location_bar_box), window->zoom_control, FALSE, FALSE, 0);
 	
 	gtk_widget_show (location_bar_box);
@@ -406,41 +511,41 @@ nautilus_window_constructed (NautilusWindow *window)
         if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
 		window->content_hbox = gtk_widget_new (NAUTILUS_TYPE_GENEROUS_BIN, NULL);
 	} else {
-		EPaned *panel;
 		set_initial_window_geometry (window);
 	
 		window->content_hbox = nautilus_horizontal_splitter_new ();
 		panel = E_PANED (window->content_hbox);
 		
-		/* FIXME bugzilla.eazel.com 1245: No constant for the default? */
 		/* FIXME bugzilla.eazel.com 1245: Saved in pixels instead of in %? */
 		/* FIXME bugzilla.eazel.com 1245: No reality check on the value? */
-		/* FIXME bugzilla.eazel.com 1245: get_enum? why not get_integer? */
-		sidebar_width = nautilus_preferences_get_enum (NAUTILUS_PREFERENCES_SIDEBAR_WIDTH, 148);
+		sidebar_width = nautilus_preferences_get_integer (NAUTILUS_PREFERENCES_SIDEBAR_WIDTH);
 		e_paned_set_position (E_PANED (window->content_hbox), sidebar_width);
 	}
 	gtk_widget_show (window->content_hbox);
-	bonobo_win_set_contents (BONOBO_WIN (window), window->content_hbox);
-	
-	/* set up the index panel */
+	bonobo_window_set_contents (BONOBO_WINDOW (window), window->content_hbox);
 	
+	/* set up the sidebar */
 	window->sidebar = nautilus_sidebar_new ();
-	gtk_widget_show (GTK_WIDGET (window->sidebar));
-	gtk_signal_connect (GTK_OBJECT (window->sidebar), "location_changed",
-			    goto_uri_callback, window);
 	
 	/* FIXME bugzilla.eazel.com 1243: 
 	 * We should use inheritance instead of these special cases
 	 * for the desktop window.
 	 */
         if (!NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-		e_paned_pack1 (E_PANED(window->content_hbox), GTK_WIDGET(window->sidebar), FALSE, FALSE);
+		gtk_widget_show (GTK_WIDGET (window->sidebar));
+		gtk_signal_connect (GTK_OBJECT (window->sidebar), "location_changed",
+				    goto_uri_callback, window);
+		e_paned_pack1 (E_PANED (window->content_hbox),
+			       GTK_WIDGET (window->sidebar),
+			       FALSE, FALSE);
 	}
 	
-
-
 	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
 
+	/* FIXME bugzilla.eazel.com 1243: 
+	 * We should use inheritance instead of these special cases
+	 * for the desktop window.
+	 */
 	if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
 		nautilus_bonobo_set_hidden (window->details->shell_ui,
 					    LOCATION_BAR_PATH, TRUE);
@@ -450,6 +555,12 @@ nautilus_window_constructed (NautilusWindow *window)
 					    STATUS_BAR_PATH, TRUE);
 		nautilus_bonobo_set_hidden (window->details->shell_ui,
 					    MENU_BAR_PATH, TRUE);
+
+		/* FIXME bugzilla.eazel.com 4752:
+		 * If we ever get the unsigned math errors in
+		 * gtk_menu_item_size_allocate fixed this can be removed.
+		 */
+		menu_bar_no_resize_hack (window);
 	}
 
 	/* Wrap the location bar in a control and set it up. */
@@ -465,9 +576,19 @@ nautilus_window_constructed (NautilusWindow *window)
 	nautilus_window_initialize_menus (window);
 	nautilus_window_initialize_toolbars (window);
 
-	/* watch for throbber locatoin changes, too */
-	gtk_signal_connect (GTK_OBJECT (window->throbber), "location_changed",
-		goto_uri_callback, window);
+	/* watch for throbber location changes, too */
+	if (window->throbber != NULL) {
+		CORBA_exception_init (&ev);
+		property_bag = Bonobo_Control_getProperties (window->throbber, &ev);
+		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {
+			window->details->throbber_location_change_request_listener_id =
+				bonobo_event_source_client_add_listener
+				(property_bag, throbber_location_change_request_callback, 
+				 "Bonobo/Property:change:location", NULL, window); 
+			bonobo_object_release_unref (property_bag, &ev);	
+		}
+		CORBA_exception_free (&ev);
+	}
 	
 	/* Set initial sensitivity of some buttons & menu items 
 	 * now that they're all created.
@@ -481,6 +602,9 @@ nautilus_window_constructed (NautilusWindow *window)
 
 	/* Set up the sidebar panels. */
 	update_sidebar_panels_from_preferences (window);
+
+	/* Register that things may be dragged from this window */
+	nautilus_drag_window_register (GTK_WINDOW (window));
 }
 
 static void
@@ -496,8 +620,8 @@ nautilus_window_set_arg (GtkObject *object,
 		if (GTK_VALUE_STRING (*arg) == NULL) {
 			return;
 		}
-		old_name = bonobo_win_get_name (BONOBO_WIN (object));
-		bonobo_win_set_name (BONOBO_WIN (object), GTK_VALUE_STRING (*arg));
+		old_name = bonobo_window_get_name (BONOBO_WINDOW (object));
+		bonobo_window_set_name (BONOBO_WINDOW (object), GTK_VALUE_STRING (*arg));
 		/* This hack of using the time when the name first
 		 * goes non-NULL to be window-constructed time is
 		 * completely lame. But it works, so for now we leave
@@ -521,7 +645,7 @@ nautilus_window_get_arg (GtkObject *object,
 {
 	switch(arg_id) {
 	case ARG_APP_ID:
-		GTK_VALUE_STRING (*arg) = bonobo_win_get_name (BONOBO_WIN (object));
+		GTK_VALUE_STRING (*arg) = bonobo_window_get_name (BONOBO_WINDOW (object));
 		break;
 	case ARG_APP:
 		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (NAUTILUS_WINDOW (object)->application);
@@ -529,56 +653,44 @@ nautilus_window_get_arg (GtkObject *object,
 	}
 }
 
-static void
-view_disconnect (gpointer data, gpointer user_data)
-{
-	g_assert (NAUTILUS_IS_WINDOW (user_data));
-	g_assert (NAUTILUS_IS_VIEW_FRAME (data));
-
-	nautilus_window_disconnect_view (NAUTILUS_WINDOW (user_data),
-					 NAUTILUS_VIEW_FRAME (data));
-}
-
 static void 
 nautilus_window_destroy (GtkObject *object)
 {
 	NautilusWindow *window;
-
+	CORBA_Environment ev;
+	Bonobo_PropertyBag property_bag;
+	
 	window = NAUTILUS_WINDOW (object);
 
-	/* Let go of the UI. */
+	/* Handle the part of destroy that's private to the view
+	 * management.
+	 */
+	nautilus_window_manage_views_destroy (window);
+
+	/* Get rid of all callbacks. */
+
+	cancel_view_as_callback (window);
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SIDEBAR_PANELS_NAMESPACE,
+					      sidebar_panels_changed_callback,
+					      window);
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS,
+					      nautilus_window_bookmarks_preference_changed_callback,
+					      window);
+	nautilus_window_remove_bookmarks_menu_callback (window);
+	nautilus_window_remove_go_menu_callback (window);
+	nautilus_window_toolbar_remove_theme_callback (window);
+
+	/* Get rid of all owned objects. */
+
 	if (window->details->shell_ui != NULL) {
 		bonobo_ui_component_unset_container (window->details->shell_ui);
 		bonobo_object_unref (BONOBO_OBJECT (window->details->shell_ui));
 	}
 
-	/* Cancel the callback for the View As menu update, if any */
-	nautilus_file_cancel_call_when_ready (window->details->viewed_file, 
-					      nautilus_window_real_load_content_view_menu,
-					      window);
-
-	/* Let go of the file for the current location */
 	nautilus_file_unref (window->details->viewed_file);
 
-	g_free (window->details->dead_view_name);
-
-	/* Dont keep track of sidebar panel changes no more */
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SIDEBAR_PANELS_NAMESPACE,
-					      sidebar_panels_changed_callback,
-					      NULL);
-
-	nautilus_window_remove_bookmarks_menu_callback (window);
-	nautilus_window_remove_go_menu_callback (window);
-	nautilus_window_toolbar_remove_theme_callback (window);
-
-	/* Disconnect view signals here so they don't trigger when
-	 * views are destroyed normally.
-	 */
-	g_list_foreach (window->sidebar_panels, view_disconnect, window);
 	g_list_free (window->sidebar_panels);
 
-	nautilus_window_disconnect_view (window, window->content_view);
-
 	nautilus_view_identifier_free (window->content_view_id);
 	
 	g_free (window->location);
@@ -603,6 +715,24 @@ nautilus_window_destroy (GtkObject *object)
 		bonobo_object_unref (BONOBO_OBJECT (window->details->ui_container));
 	}
 
+	if (window->throbber != NULL) {
+		CORBA_exception_init (&ev);
+		property_bag = Bonobo_Control_getProperties (window->throbber, &ev);
+		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {	
+			bonobo_event_source_client_remove_listener
+				(property_bag,
+				 window->details->throbber_location_change_request_listener_id,
+				 &ev);
+			bonobo_object_release_unref (property_bag, &ev);	
+		}
+
+		bonobo_object_release_unref (window->throbber, &ev);		
+		CORBA_exception_free (&ev);
+	}
+	if (window->details->location_change_at_idle_id != 0) {
+		gtk_idle_remove (window->details->location_change_at_idle_id);
+	}
+
 	g_free (window->details);
 
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (GTK_OBJECT (window)));
@@ -633,49 +763,13 @@ nautilus_window_close (NautilusWindow *window)
 	 * we're in every-location-in-its-own-window mode. Otherwise it
 	 * would be too apparently random when the stored positions change.
 	 */
-	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW,
-					      FALSE)) {
+	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW)) {
 	        nautilus_window_save_geometry (window);
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (window));
 }
 
-/* FIXME: Why is this filter turned off? */
-#if 0
-#include <gdk/gdkx.h>
-#include <gdk/gdkprivate.h>
-
-static GdkFilterReturn
-nautilus_window_filter (GdkXEvent *xev, GdkEvent *event, gpointer data)
-{
-	XEvent *xevent = (XEvent *)xev;
-		
-	if ((Atom) xevent->xclient.data.l[0] == gdk_wm_delete_window) {
-		/* The delete window request specifies a window
-		*  to delete. We don't actually destroy the
-		*  window because "it is only a request". (The
-		*  window might contain vital data that the
-		*  program does not want destroyed). Instead
-		*  the event is passed along to the program,
-		*  which should then destroy the window.
-		*/
-		GDK_NOTE (EVENTS, g_message ("delete window:\t\twindow: %ld", xevent->xclient.window));
-      
-		event->any.type = GDK_DELETE;
-
-		return GDK_FILTER_TRANSLATE;
-	} else if ((Atom) xevent->xclient.data.l[0] == gdk_wm_take_focus) {
-		/* Check and see if we are in a drag. If not, Focus window and bring to front */
-		nautilus_gdk_window_bring_to_front (GTK_WIDGET (data)->window);
-		
-		return GDK_FILTER_TRANSLATE;
-	}
-
-	return GDK_FILTER_REMOVE;
-}
-#endif
-
 static void
 nautilus_window_update_launcher (GdkWindow *window)
 {
@@ -706,20 +800,6 @@ nautilus_window_realize (GtkWidget *widget)
         /* Create our GdkWindow */
 	NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS, realize, (widget));
 
-	/* Set window manager hints so click-drag from window to window works */
-	
-	/* FIXME: need to coordinate with sawfish or this line results in
-	   nautilus windows being unfocusable */
-#if 0
-	nautilus_gdk_window_set_wm_hints_input (widget->window, FALSE);
-#endif
-
-	/* Add custom message filter to handle WM_TAKE_FOCUS */
-	/* FIXME: Why is this filter turned off? */
-#if 0
-	gdk_add_client_message_filter (gdk_wm_protocols, nautilus_window_filter, widget);
-#endif
-
         /* Set the mini icon */
         filename = nautilus_pixmap_file ("nautilus-mini-logo.png");
         if (filename != NULL) {
@@ -770,11 +850,11 @@ nautilus_window_size_request (GtkWidget		*widget,
 	max_width = get_max_forced_width ();
 	max_height = get_max_forced_height ();
 
-	if (requisition->width > max_width) {
+	if (requisition->width > (int) max_width) {
 		requisition->width = max_width;
 	}
 	
-	if (requisition->height > max_height) {
+	if (requisition->height > (int) max_height) {
 		requisition->height = max_height;
 	}
 }
@@ -785,7 +865,7 @@ nautilus_window_size_request (GtkWidget		*widget,
  */
 
 static void
-view_menu_switch_views_callback (GtkWidget *widget, gpointer data)
+view_as_menu_switch_views_callback (GtkWidget *widget, gpointer data)
 {
         NautilusWindow *window;
         NautilusViewIdentifier *identifier;
@@ -801,7 +881,7 @@ view_menu_switch_views_callback (GtkWidget *widget, gpointer data)
 
 /* Note: The identifier parameter ownership is handed off to the menu item. */
 static GtkWidget *
-create_content_view_menu_item (NautilusWindow *window, NautilusViewIdentifier *identifier)
+create_view_as_menu_item (NautilusWindow *window, NautilusViewIdentifier *identifier)
 {
 	GtkWidget *menu_item;
         char *menu_label;
@@ -812,7 +892,7 @@ create_content_view_menu_item (NautilusWindow *window, NautilusViewIdentifier *i
 
 	gtk_signal_connect (GTK_OBJECT (menu_item),
 			    "activate",
-			    view_menu_switch_views_callback, 
+			    view_as_menu_switch_views_callback, 
 			    NULL);
 
 	/* Store copy of iid in item; free when item destroyed. */
@@ -845,7 +925,7 @@ new_gtk_separator (void)
  * content view isn't already in the "View as" option menu.
  */
 static void
-replace_special_current_view_in_content_view_menu (NautilusWindow *window)
+replace_special_current_view_in_view_as_menu (NautilusWindow *window)
 {
 	GtkWidget *menu;
 	GtkWidget *first_menu_item;
@@ -869,7 +949,7 @@ replace_special_current_view_in_content_view_menu (NautilusWindow *window)
 		gtk_menu_prepend (GTK_MENU (menu), new_gtk_separator ());
 	}
 
-	new_menu_item = create_content_view_menu_item (window, nautilus_view_identifier_copy (window->content_view_id));
+	new_menu_item = create_view_as_menu_item (window, nautilus_view_identifier_copy (window->content_view_id));
 	gtk_object_set_data (GTK_OBJECT (new_menu_item), "current content view", GINT_TO_POINTER (TRUE));
 	gtk_menu_prepend (GTK_MENU (menu), new_menu_item);
 
@@ -878,7 +958,7 @@ replace_special_current_view_in_content_view_menu (NautilusWindow *window)
 }
 
 /**
- * nautilus_window_synch_content_view_menu:
+ * nautilus_window_synch_view_as_menu:
  * 
  * Set the visible item of the "View as" option menu to
  * match the current content view.
@@ -886,7 +966,7 @@ replace_special_current_view_in_content_view_menu (NautilusWindow *window)
  * @window: The NautilusWindow whose "View as" option menu should be synched.
  */
 void
-nautilus_window_synch_content_view_menu (NautilusWindow *window)
+nautilus_window_synch_view_as_menu (NautilusWindow *window)
 {
 	GList *children, *child;
 	GtkWidget *menu;
@@ -912,7 +992,7 @@ nautilus_window_synch_content_view_menu (NautilusWindow *window)
 	}
 
 	if (matching_index == -1) {
-		replace_special_current_view_in_content_view_menu (window);
+		replace_special_current_view_in_view_as_menu (window);
 		matching_index = 0;
 	}
 
@@ -925,29 +1005,34 @@ nautilus_window_synch_content_view_menu (NautilusWindow *window)
 static void
 chose_component_callback (NautilusViewIdentifier *identifier, gpointer callback_data)
 {
-	/* Can't assume callback_data is a valid NautilusWindow, because
-	 * it might be garbage if the program is exiting when this dialog
-	 * is up.
-	 */
+	NautilusWindow *window;
 
+	window = NAUTILUS_WINDOW (callback_data);
 	if (identifier != NULL) {
-		g_return_if_fail (NAUTILUS_IS_WINDOW (callback_data));
-		nautilus_window_set_content_view (NAUTILUS_WINDOW (callback_data), identifier);
+		nautilus_window_set_content_view (window, identifier);
 	}
-
-	/* FIXME bugzilla.eazel.com 1334: 
-	 * There should be some global way to signal that the
-	 * file type associations have changed, so that the places that
-	 * display these lists can react. For now, hardwire this case, which
-	 * is the most obvious one by far.
+	
+	/* FIXME bugzilla.eazel.com 1334: There should be some global
+	 * way to signal that the file type associations have changed,
+	 * so that the places that display these lists can react. For
+	 * now, hardwire this case, which is the most obvious one by
+	 * far.
 	 */
-	if (NAUTILUS_IS_WINDOW (callback_data)) {
-		nautilus_window_load_content_view_menu (NAUTILUS_WINDOW (callback_data));
+	nautilus_window_load_view_as_menu (window);
+}
+
+static void
+cancel_chose_component_callback (NautilusWindow *window)
+{
+	if (window->details->viewed_file != NULL) {
+		nautilus_cancel_choose_component_for_file (window->details->viewed_file,
+							   chose_component_callback, 
+							   window);
 	}
 }
 
 static void
-view_menu_choose_view_callback (GtkWidget *widget, gpointer data)
+view_as_menu_choose_view_callback (GtkWidget *widget, gpointer data)
 {
         NautilusWindow *window;
         
@@ -956,14 +1041,18 @@ view_menu_choose_view_callback (GtkWidget *widget, gpointer data)
         
         window = NAUTILUS_WINDOW (gtk_object_get_data (GTK_OBJECT (widget), "window"));
 
-	/* Set the option menu back to its previous setting (Don't leave it
-	 * on this dialog-producing "View as Other..." setting). If the menu choice 
-	 * causes a content view change, this will be updated again later, 
-	 * in nautilus_window_load_content_view_menu. Do this right away so 
-	 * the user never sees the option menu set to "View as Other...".
+	/* Set the option menu back to its previous setting (Don't
+	 * leave it on this dialog-producing "View as Other..."
+	 * setting). If the menu choice causes a content view change,
+	 * this will be updated again later, in
+	 * nautilus_window_load_view_as_menu. Do this right away so
+	 * the user never sees the option menu set to "View as
+	 * Other...".
 	 */
-	nautilus_window_synch_content_view_menu (window);
+	nautilus_window_synch_view_as_menu (window);
 
+	/* Call back when the user chose the component. */
+	cancel_chose_component_callback (window);
 	nautilus_choose_component_for_file (window->details->viewed_file,
 					    GTK_WINDOW (window), 
 					    chose_component_callback, 
@@ -971,28 +1060,30 @@ view_menu_choose_view_callback (GtkWidget *widget, gpointer data)
 }
 
 static void
-view_menu_vfs_method_callback (GtkWidget *widget, gpointer data)
+view_as_menu_vfs_method_callback (GtkWidget *widget, gpointer callback_data)
 {
+	gpointer object_data;
         NautilusWindow *window;
 	char *new_location;
 	char *method;
         
         g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-        g_return_if_fail (NAUTILUS_IS_WINDOW (gtk_object_get_data (GTK_OBJECT (widget), "window")));
-        
-        window = NAUTILUS_WINDOW (gtk_object_get_data (GTK_OBJECT (widget), "window"));
-        method = (char *) (gtk_object_get_data (GTK_OBJECT (widget), "method"));
-	g_return_if_fail (method);
+
+	object_data = gtk_object_get_data (GTK_OBJECT (widget), "window");
+        g_return_if_fail (NAUTILUS_IS_WINDOW (object_data));
+        window = NAUTILUS_WINDOW (object_data);
+
+        method = (char *) gtk_object_get_data (GTK_OBJECT (widget), "method");
+	g_return_if_fail (method != NULL);
 
 	new_location = g_strdup_printf ("%s#%s:/",window->location,method);
 	nautilus_window_goto_uri (window, new_location);
 	g_free (new_location);
-
 }
 
 static void
-nautilus_window_real_load_content_view_menu (NautilusFile *file, 
-					     gpointer      callback_data)
+load_view_as_menu_callback (NautilusFile *file, 
+			    gpointer callback_data)
 {	
 	GList *components;
 	char *method;
@@ -1011,23 +1102,27 @@ nautilus_window_real_load_content_view_menu (NautilusFile *file,
         /* Add a menu item for each view in the preferred list for this location. */
         components = nautilus_mime_get_short_list_components_for_file (window->details->viewed_file);
         for (p = components; p != NULL; p = p->next) {
-                menu_item = create_content_view_menu_item 
+                menu_item = create_view_as_menu_item 
                 	(window, nautilus_view_identifier_new_from_content_view (p->data));
                 gtk_menu_append (GTK_MENU (new_menu), menu_item);
         }
 	gnome_vfs_mime_component_list_free (components);
 
+	/* FIXME bugzilla.eazel.com 5086: This feature has not been
+	 * thoroughly tested and we need to finish it or delete it.
+	 */
 	/* Add a menu item for each special GNOME-VFS method for this
 	 * URI. This is a questionable user interface, since it's a
 	 * one way trip if you choose one of these view menu items, but
 	 * it's better than nothing.
 	 */
-	method = nautilus_mime_get_short_list_methods_for_file (window->details->viewed_file);
-	/* FIXME bugzilla.eazel.com 2466: Name of the function is plural, but it returns only
-	 * one item. That must be fixed.
+	/* FIXME bugzilla.eazel.com 2466: The name of the following
+	 * function is plural, but it returns only one item. That must
+	 * be fixed.
 	 */
+	method = nautilus_mime_get_short_list_methods_for_file (window->details->viewed_file);
 	if (method != NULL) {
-		label = g_strdup_printf (_("View as %s..."), method);
+		label = g_strdup_printf (_("View as %s"), method);
 		menu_item = gtk_menu_item_new_with_label (label);
 		g_free (label);
 
@@ -1036,7 +1131,7 @@ nautilus_window_real_load_content_view_menu (NautilusFile *file,
 					  g_strdup (method), g_free);
         	gtk_signal_connect (GTK_OBJECT (menu_item),
 				    "activate",
-				    view_menu_vfs_method_callback,
+				    view_as_menu_vfs_method_callback,
 				    NULL);
        		gtk_widget_show (menu_item);
        		gtk_menu_append (GTK_MENU (new_menu), menu_item);
@@ -1053,7 +1148,7 @@ nautilus_window_real_load_content_view_menu (NautilusFile *file,
         gtk_object_set_data (GTK_OBJECT (menu_item), "window", window);
         gtk_signal_connect (GTK_OBJECT (menu_item),
         		    "activate",
-        		    view_menu_choose_view_callback,
+        		    view_as_menu_choose_view_callback,
         		    NULL);
        	gtk_widget_show (menu_item);
        	gtk_menu_append (GTK_MENU (new_menu), menu_item);
@@ -1064,12 +1159,19 @@ nautilus_window_real_load_content_view_menu (NautilusFile *file,
         gtk_option_menu_set_menu (GTK_OPTION_MENU (window->view_as_option_menu),
                                   new_menu);
 
+	nautilus_window_synch_view_as_menu (window);
+}
 
-	nautilus_window_synch_content_view_menu (window);
+static void
+cancel_view_as_callback (NautilusWindow *window)
+{
+	nautilus_file_cancel_call_when_ready (window->details->viewed_file, 
+					      load_view_as_menu_callback,
+					      window);
 }
 
 void
-nautilus_window_load_content_view_menu (NautilusWindow *window)
+nautilus_window_load_view_as_menu (NautilusWindow *window)
 {
 	GList *attributes;
 
@@ -1077,9 +1179,11 @@ nautilus_window_load_content_view_menu (NautilusWindow *window)
 
 	attributes = nautilus_mime_actions_get_full_file_attributes ();
 
-	/* FIXME: need to cancel this when appropriate... */
-	nautilus_file_call_when_ready (window->details->viewed_file, attributes, 
-				       nautilus_window_real_load_content_view_menu, window);
+	cancel_view_as_callback (window);
+	nautilus_file_call_when_ready (window->details->viewed_file,
+				       attributes, 
+				       load_view_as_menu_callback,
+				       window);
 
 	g_list_free (attributes);
 }
@@ -1199,7 +1303,7 @@ nautilus_window_go_web_search (NautilusWindow *window)
 
 	nautilus_window_set_search_mode (window, FALSE);
 
-	search_web_uri = nautilus_preferences_get (NAUTILUS_PREFERENCES_SEARCH_WEB_URI, "");
+	search_web_uri = nautilus_preferences_get (NAUTILUS_PREFERENCES_SEARCH_WEB_URI);
 	g_assert (search_web_uri != NULL);
 	
 	nautilus_window_goto_uri (window, search_web_uri);
@@ -1209,13 +1313,11 @@ nautilus_window_go_web_search (NautilusWindow *window)
 void
 nautilus_window_go_home (NautilusWindow *window)
 {
-	char *default_home_uri, *home_uri;
+	char *home_uri;
 
 	nautilus_window_set_search_mode (window, FALSE);
 
-	default_home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
-	home_uri = nautilus_preferences_get (NAUTILUS_PREFERENCES_HOME_URI, default_home_uri);
-	g_free (default_home_uri);
+	home_uri = nautilus_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
 	
 	g_assert (home_uri != NULL);
 	nautilus_window_goto_uri (window, home_uri);
@@ -1225,21 +1327,25 @@ nautilus_window_go_home (NautilusWindow *window)
 void
 nautilus_window_allow_back (NautilusWindow *window, gboolean allow)
 {
-	/* Because of verbs, we set the sensitivity of the menu to
-	 * control both the menu and toolbar.
-	 */
 	nautilus_bonobo_set_sensitive (window->details->shell_ui,
 				       NAUTILUS_COMMAND_BACK, allow);
+	/* Have to handle non-standard Back button explicitly (it's
+	 * non-standard to support right-click menu).
+	 */
+	gtk_widget_set_sensitive 
+		(GTK_WIDGET (window->details->back_button_item), allow);
 }
 
 void
 nautilus_window_allow_forward (NautilusWindow *window, gboolean allow)
 {
-	/* Because of verbs, we set the sensitivity of the menu to
-	 * control both the menu and toolbar.
-	 */
 	nautilus_bonobo_set_sensitive (window->details->shell_ui,
 				       NAUTILUS_COMMAND_FORWARD, allow);
+	/* Have to handle non-standard Forward button explicitly (it's
+	 * non-standard to support right-click menu).
+	 */
+	gtk_widget_set_sensitive 
+		(GTK_WIDGET (window->details->forward_button_item), allow);
 }
 
 void
@@ -1265,21 +1371,27 @@ nautilus_window_allow_reload (NautilusWindow *window, gboolean allow)
 void
 nautilus_window_allow_stop (NautilusWindow *window, gboolean allow)
 {
+	CORBA_Environment ev;
+	Bonobo_PropertyBag property_bag;
+	
 	nautilus_bonobo_set_sensitive (window->details->shell_ui,
 				       NAUTILUS_COMMAND_STOP, allow);
+
 	if (window->throbber != NULL) {
-		if (allow) {
-			nautilus_throbber_start (NAUTILUS_THROBBER (window->throbber));
-		} else {
-			nautilus_throbber_stop (NAUTILUS_THROBBER (window->throbber));
+		CORBA_exception_init (&ev);
+		property_bag = Bonobo_Control_getProperties (window->throbber, &ev);
+		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {
+			bonobo_property_bag_client_set_value_gboolean (property_bag, "throbbing", allow, &ev);
+			bonobo_object_release_unref (property_bag, &ev);
 		}
+		CORBA_exception_free (&ev);
 	}
 }
 
 void
 nautilus_send_history_list_changed (void)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT (nautilus_signaller_get_current ()),
+	gtk_signal_emit_by_name (nautilus_signaller_get_current (),
 			 	 "history_list_changed");
 }
 
@@ -1438,6 +1550,14 @@ nautilus_window_set_content_view_widget (NautilusWindow *window,
 		}
 	}
 
+	/* Display or hide zoom control */
+	if (nautilus_view_frame_is_zoomable (new_view)) {
+		gtk_widget_show (window->zoom_control);
+	} else {
+		gtk_widget_hide (window->zoom_control);
+	}
+
+
 	window->content_view = new_view;
 }
 
@@ -1468,6 +1588,14 @@ update_sidebar_panels_from_preferences (NautilusWindow *window)
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
+	/* FIXME bugzilla.eazel.com 1243: 
+	 * We should use inheritance instead of these special cases
+	 * for the desktop window.
+	 */
+	if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+		return;
+	}
+
 	/* Obtain list of enabled view identifiers */
 	identifier_list = nautilus_global_preferences_get_enabled_sidebar_panel_view_identifiers ();
 	nautilus_window_set_sidebar_panels (window, identifier_list);
@@ -1651,25 +1779,25 @@ nautilus_window_show (GtkWidget *widget)
 	/* Initially show or hide views based on preferences; once the window is displayed
 	 * these can be controlled on a per-window basis from View menu items. 
 	 */
-	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR, TRUE)) {
+	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR)) {
 		nautilus_window_show_tool_bar (window);
 	} else {
 		nautilus_window_hide_tool_bar (window);
 	}
 
-	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR, TRUE)) {
+	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR)) {
 		nautilus_window_show_location_bar (window);
 	} else {
 		nautilus_window_hide_location_bar (window);
 	}
 
-	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR, TRUE)) {
+	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR)) {
 		nautilus_window_show_status_bar (window);
 	} else {
 		nautilus_window_hide_status_bar (window);
 	}
 
-	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR, TRUE)) {
+	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR)) {
 		nautilus_window_show_sidebar (window);
 	} else {
 		nautilus_window_hide_sidebar (window);
@@ -1683,3 +1811,18 @@ nautilus_window_get_ui_container (NautilusWindow *window)
 
 	return bonobo_object_corba_objref (BONOBO_OBJECT (window->details->ui_container));
 }
+
+void
+nautilus_window_set_viewed_file (NautilusWindow *window,
+				 NautilusFile *file)
+{
+	if (window->details->viewed_file == file) {
+		return;
+	}
+
+	nautilus_file_ref (file);
+	cancel_view_as_callback (window);
+	cancel_chose_component_callback (window);
+	nautilus_file_unref (window->details->viewed_file);
+	window->details->viewed_file = file;
+}
