@@ -49,7 +49,7 @@ typedef struct NautilusWindow NautilusWindow;
 #endif
 
 typedef struct {
-        BonoboWinClass parent_spot;
+        BonoboWindowClass parent_spot;
 } NautilusWindowClass;
 
 typedef struct NautilusWindowStateInfo NautilusWindowStateInfo;
@@ -64,7 +64,7 @@ typedef enum {
 typedef struct NautilusWindowDetails NautilusWindowDetails;
 
 struct NautilusWindow {
-        BonoboWin parent_object;
+        BonoboWindow parent_object;
         
         NautilusWindowDetails *details;
         
@@ -79,9 +79,6 @@ struct NautilusWindow {
         /** CORBA-related elements **/
         NautilusApplication *application;
         
-        /* FIXME bugzilla.eazel.com 916: Workaround for Bonobo bug. */
-        gboolean updating_bonobo_radio_menu_item;
-        
         /** State information **/
         
         /* Information about current location/selection */
@@ -103,7 +100,7 @@ struct NautilusWindow {
         
         /* Widgets to keep track of (for state changes, etc) */      
         GtkWidget *zoom_control;
-        GtkWidget *throbber;
+        Bonobo_Unknown throbber;
         
         /* Pending changes */
         NautilusNavigationInfo *pending_ni;
@@ -137,8 +134,6 @@ void             nautilus_window_set_search_mode      (NautilusWindow    *window
                                                        gboolean           search_mode);
 void             nautilus_window_go_home              (NautilusWindow    *window);
 void		 nautilus_window_go_web_search	      (NautilusWindow    *window);
-void		 nautilus_window_disable_keyboard_navigation_for_menus
-						      (NautilusWindow	 *window);
 void             nautilus_window_display_error        (NautilusWindow    *window,
                                                        const char        *error_msg);
 void             nautilus_window_allow_back           (NautilusWindow    *window,
