@@ -69,6 +69,8 @@ struct NautilusZoomControlDetails {
 	double zoom_level;
 	double min_zoom_level;	 
 	double max_zoom_level;
+	gboolean has_min_zoom_level;
+	gboolean has_max_zoom_level;
 	GList *preferred_zoom_levels;
 
 	int y_offset;
@@ -78,6 +80,8 @@ struct NautilusZoomControlDetails {
 	GdkPixbuf *number_strip;
 	PrelightMode prelight_mode;
 
+	GdkFont *label_font;
+
 	gboolean marking_menu_items;
 };
 
@@ -217,6 +221,8 @@ nautilus_zoom_control_initialize (NautilusZoomControl *zoom_control)
 	zoom_control->details->zoom_level = 1.0;
 	zoom_control->details->min_zoom_level = 0.0;
 	zoom_control->details->max_zoom_level = 2.0;
+	zoom_control->details->has_min_zoom_level = TRUE;
+	zoom_control->details->has_max_zoom_level = TRUE;
 	zoom_control->details->preferred_zoom_levels = NULL;
 
 	/* allocate the pixmap that holds the image */
@@ -276,14 +282,13 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 {
 	char buffer[8];
 	char *cur_char;
-	GdkFont *label_font;
 	GdkGC* temp_gc; 
 	int x, y, percent; 
 	int char_height, char_width, char_offset;
 	int num_v_offset, num_h_offset;
 	NautilusZoomControl *zoom_control;
 	GdkPixbuf *number_pixbuf;
-	
+
 	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
 	number_pixbuf = NULL;
 	
@@ -324,13 +329,9 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 
 			x += char_width;
 		}
-	} else {
-              label_font = gdk_fontset_load (_("-bitstream-courier-medium-r-normal-*-9-*-*-*-*-*-*-*"));
-				
-		x = num_h_offset + ((widget->allocation.width - gdk_string_width(label_font, buffer)) >> 1);  
-		
-		gdk_draw_string (widget->window, label_font, temp_gc, x, y, &buffer[0]);
-		gdk_font_unref(label_font);
+	} else {				
+		x = num_h_offset + ((widget->allocation.width - gdk_string_width(zoom_control->details->label_font, buffer)) >> 1);  
+		gdk_draw_string (widget->window, zoom_control->details->label_font, temp_gc, x, y, &buffer[0]);
 	}
 	
 	if (number_pixbuf != zoom_control->details->number_strip) {
@@ -355,7 +356,6 @@ static void
 draw_pixbuf_with_prelight (NautilusZoomControl *zoom_control, GdkPixbuf *pixbuf, int x_pos, int y_pos, PrelightMode mode)
 {
 	GdkPixbuf *temp_pixbuf;
-	
 	temp_pixbuf = pixbuf;
 	if (zoom_control->details->prelight_mode == mode) {
 		temp_pixbuf = nautilus_create_spotlight_pixbuf (temp_pixbuf);
@@ -378,9 +378,9 @@ draw_zoom_control_image (GtkWidget *widget, GdkRectangle *box)
 	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
 
 	/* draw the decrement symbol if necessary, complete with prelighting */
-	if (zoom_control->details->zoom_level > zoom_control->details->min_zoom_level) {
+	if (nautilus_zoom_control_can_zoom_out (zoom_control)) {
 		draw_pixbuf_with_prelight (zoom_control, zoom_control->details->zoom_decrement_image,
-						box->x, box->y, PRELIGHT_MINUS);
+					   box->x, box->y, PRELIGHT_MINUS);
 	} else {
 		width  = gdk_pixbuf_get_width (zoom_control->details->zoom_decrement_image);
 		height = gdk_pixbuf_get_height (zoom_control->details->zoom_decrement_image);		
@@ -396,12 +396,12 @@ draw_zoom_control_image (GtkWidget *widget, GdkRectangle *box)
 	offset = gdk_pixbuf_get_width (zoom_control->details->zoom_decrement_image) + GAP_WIDTH;
 	/* draw the body image, prelighting if necessary */
 	draw_pixbuf_with_prelight (zoom_control, zoom_control->details->zoom_body_image,
-					box->x + offset, box->y, PRELIGHT_CENTER);
+				   box->x + offset, box->y, PRELIGHT_CENTER);
 	
 	offset += gdk_pixbuf_get_width (zoom_control->details->zoom_body_image) + GAP_WIDTH;
 	
 	/* draw the increment symbol if necessary, complete with prelighting */
-	if (zoom_control->details->zoom_level < zoom_control->details->max_zoom_level) {
+	if (nautilus_zoom_control_can_zoom_in (zoom_control)) {
 		draw_pixbuf_with_prelight (zoom_control, zoom_control->details->zoom_increment_image,
 					   box->x + offset, box->y, PRELIGHT_PLUS);
 	} else {
@@ -419,7 +419,7 @@ draw_zoom_control_image (GtkWidget *widget, GdkRectangle *box)
 
 static void
 nautilus_zoom_control_draw (GtkWidget *widget, GdkRectangle *box)
-{ 
+{
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (NAUTILUS_IS_ZOOM_CONTROL (widget));
 
@@ -429,7 +429,6 @@ nautilus_zoom_control_draw (GtkWidget *widget, GdkRectangle *box)
 			       0,
 			       widget->allocation.width,
 			       widget->allocation.height);
-
 	draw_zoom_control_image (widget, box);	
 	draw_number (widget, box);
 }
@@ -449,7 +448,7 @@ nautilus_zoom_control_expose (GtkWidget *widget, GdkEventExpose *event)
 
 	draw_zoom_control_image (widget, &box);	
 	draw_number (widget, &box);
-	
+
 	return FALSE;
 }
 
@@ -483,6 +482,15 @@ nautilus_zoom_control_unload_images (NautilusZoomControl *zoom_control)
 		gdk_pixbuf_unref (zoom_control->details->zoom_increment_image);
 	}
 
+	if (zoom_control->details->number_strip != NULL) {
+		gdk_pixbuf_unref (zoom_control->details->number_strip);
+	}
+
+	if (zoom_control->details->label_font != NULL) {
+		gdk_font_unref(zoom_control->details->label_font);
+		zoom_control->details->label_font = NULL;
+	}
+
 }
 
 static GdkPixbuf*
@@ -515,6 +523,10 @@ nautilus_zoom_control_load_images (NautilusZoomControl *zoom_control)
 	zoom_control->details->zoom_decrement_image = load_themed_image ("decrement.png");
 	zoom_control->details->zoom_increment_image = load_themed_image ("increment.png");
 	zoom_control->details->number_strip = load_themed_image ("number_strip.png");
+	
+	if (zoom_control->details->number_strip == NULL) {
+		zoom_control->details->label_font = gdk_fontset_load (_("-bitstream-courier-medium-r-normal-*-9-*-*-*-*-*-*-*"));
+	}
 
 	nautilus_zoom_control_update_offsets (zoom_control);
 }
@@ -526,6 +538,7 @@ zoom_menu_callback (GtkMenuItem *item, gpointer callback_data)
 {
 	double zoom_level;
 	NautilusZoomControl *zoom_control;
+	gboolean can_zoom;
 		
 	zoom_control = NAUTILUS_ZOOM_CONTROL (callback_data);
 
@@ -535,20 +548,29 @@ zoom_menu_callback (GtkMenuItem *item, gpointer callback_data)
 	}
 
 	zoom_level = * (double *) gtk_object_get_data (GTK_OBJECT (item), "zoom_level");
-	
-	/* Check to see if we can zoom out */
-	if ((zoom_control->details->min_zoom_level <= zoom_level && zoom_level <  zoom_control->details->zoom_level) ||
-	    (zoom_control->details->zoom_level     < zoom_level && zoom_level  <= zoom_control->details->max_zoom_level)) {
+
+	/* Assume we can zoom and then check whether we're right. */
+	can_zoom = TRUE;
+	if (zoom_control->details->has_min_zoom_level &&
+	    zoom_level < zoom_control->details->min_zoom_level)
+		can_zoom = FALSE; /* no, we're below the minimum zoom level. */
+	if (zoom_control->details->has_max_zoom_level &&
+	    zoom_level > zoom_control->details->max_zoom_level)
+		can_zoom = FALSE; /* no, we're beyond the upper zoom level. */
+
+	/* if we can zoom */
+	if (can_zoom) {	
 		gtk_signal_emit (GTK_OBJECT (zoom_control), signals[ZOOM_TO_LEVEL], zoom_level);
 	}
 }
 
 static GtkRadioMenuItem *
-create_zoom_menu_item (GtkMenu *menu, GtkWidget *widget, double zoom_level, GtkRadioMenuItem *previous_radio_item)
+create_zoom_menu_item (GtkMenu *menu, GtkWidget *widget, float zoom_level,
+		       GtkRadioMenuItem *previous_radio_item)
 {
 	GtkWidget *menu_item;
-	double	  *zoom_level_ptr;
-	char	  item_text[8];
+	gchar     *item_text;
+	double    *zoom_level_ptr;
 	NautilusZoomControl *zoom_control;
 	GSList	  *radio_item_group;
 
@@ -558,11 +580,12 @@ create_zoom_menu_item (GtkMenu *menu, GtkWidget *widget, double zoom_level, GtkR
 	 * to set toggle state of other radio items.
 	 */
 	zoom_control->details->marking_menu_items = TRUE;
-	
+
 	/* This is marked for localization in case the % sign is not
 	 * appropriate in some locale. I guess that's unlikely.
 	 */
-	g_snprintf (item_text, sizeof (item_text), _("%.0f%%"), 100.0 * zoom_level);
+	item_text = g_strdup_printf (_("%.0f%%"), 100.0 * zoom_level);
+
 	radio_item_group = previous_radio_item == NULL
 		? NULL
 		: gtk_radio_menu_item_group (previous_radio_item);
@@ -601,7 +624,7 @@ create_zoom_menu(GtkWidget *zoom_control)
 
 	menu_item = NULL;
 	while (p != NULL) {
-		menu_item = create_zoom_menu_item (menu, zoom_control, * (double *) p->data, menu_item);
+		menu_item = create_zoom_menu_item (menu, zoom_control, *(float *) p->data, menu_item);
 		p = g_list_next (p);
 	}
 	
@@ -622,13 +645,13 @@ nautilus_zoom_control_button_press_event (GtkWidget *widget, GdkEventButton *eve
 		nautilus_pop_up_context_menu (create_zoom_menu (widget), 
 					      NAUTILUS_DEFAULT_POPUP_MENU_DISPLACEMENT, 
 					      NAUTILUS_DEFAULT_POPUP_MENU_DISPLACEMENT, 
-					      CONTEXTUAL_MENU_BUTTON);
+					      event);
 		return TRUE;	  
  	}
 	
-	if (event->x < (width / 3) && (zoom_control->details->zoom_level > zoom_control->details->min_zoom_level)) {
+	if ((event->x < (width / 3)) && nautilus_zoom_control_can_zoom_out (zoom_control)) {
 		gtk_signal_emit (GTK_OBJECT (widget), signals[ZOOM_OUT]);			
-	} else if ((event->x > ((2 * width) / 3)) && (zoom_control->details->zoom_level < zoom_control->details->max_zoom_level)) {
+	} else if ((event->x > ((2 * width) / 3)) && nautilus_zoom_control_can_zoom_in (zoom_control)) {
 		gtk_signal_emit (GTK_OBJECT (widget), signals[ZOOM_IN]);			
 	} else if ((event->x >= (center - (width >> 3))) && (event->x <= (center + (width >> 3)))) {
 		gtk_signal_emit (GTK_OBJECT (widget), signals[ZOOM_TO_FIT]);			
@@ -673,9 +696,9 @@ static gboolean nautilus_zoom_control_motion_notify (GtkWidget *widget, GdkEvent
 	center = width >> 1;
 
 	mode = PRELIGHT_NONE;
-	if (x_offset < (width / 3) && (zoom_control->details->zoom_level > zoom_control->details->min_zoom_level)) {
+	if ((x_offset < (width / 3)) && nautilus_zoom_control_can_zoom_out (zoom_control)) {
 		mode = PRELIGHT_MINUS;		
-	} else if ((x_offset > ((2 * width) / 3)) && (zoom_control->details->zoom_level < zoom_control->details->max_zoom_level)) {
+	} else if ((x_offset > ((2 * width) / 3)) && nautilus_zoom_control_can_zoom_in (zoom_control)) {
 		mode = PRELIGHT_PLUS;
 	} else if ((x_offset >= (center - (width >> 3))) && (x_offset <= (center + (width >> 3)))) {
 		mode = PRELIGHT_CENTER;
@@ -708,24 +731,22 @@ nautilus_zoom_control_set_zoom_level (NautilusZoomControl *zoom_control, double
 }
 
 void
-nautilus_zoom_control_set_min_zoom_level (NautilusZoomControl *zoom_control, double  zoom_level)
+nautilus_zoom_control_set_parameters (NautilusZoomControl *zoom_control,
+				      double min_zoom_level,
+				      double max_zoom_level,
+				      gboolean has_min_zoom_level,
+				      gboolean has_max_zoom_level,
+				      GList *zoom_levels)
 {
-	zoom_control->details->min_zoom_level = zoom_level;
-	gtk_widget_queue_draw (GTK_WIDGET (zoom_control));
-}
+	zoom_control->details->min_zoom_level = min_zoom_level;
+	zoom_control->details->max_zoom_level = max_zoom_level;
+	zoom_control->details->has_min_zoom_level = has_min_zoom_level;
+	zoom_control->details->has_max_zoom_level = has_max_zoom_level;
 
-void
-nautilus_zoom_control_set_max_zoom_level (NautilusZoomControl *zoom_control, double zoom_level)
-{
-	zoom_control->details->max_zoom_level = zoom_level;
-	gtk_widget_queue_draw (GTK_WIDGET (zoom_control));
-}
-
-void
-nautilus_zoom_control_set_preferred_zoom_levels (NautilusZoomControl *zoom_control, GList* zoom_levels)
-{
 	nautilus_g_list_free_deep (zoom_control->details->preferred_zoom_levels);
 	zoom_control->details->preferred_zoom_levels = zoom_levels;
+
+	gtk_widget_queue_draw (GTK_WIDGET (zoom_control));
 }
 
 double
@@ -746,4 +767,31 @@ nautilus_zoom_control_get_max_zoom_level (NautilusZoomControl *zoom_control)
 	return zoom_control->details->max_zoom_level;
 }
 
+gboolean
+nautilus_zoom_control_has_min_zoom_level (NautilusZoomControl *zoom_control)
+{
+	return zoom_control->details->has_min_zoom_level;
+}
+	
+gboolean
+nautilus_zoom_control_has_max_zoom_level (NautilusZoomControl *zoom_control)
+{
+	return zoom_control->details->has_max_zoom_level;
+}
+	
+gboolean
+nautilus_zoom_control_can_zoom_in (NautilusZoomControl *zoom_control)
+{
+	return !zoom_control->details->has_max_zoom_level ||
+		(zoom_control->details->zoom_level
+		 < zoom_control->details->max_zoom_level);
+}
 
+
+gboolean
+nautilus_zoom_control_can_zoom_out (NautilusZoomControl *zoom_control)
+{
+	return !zoom_control->details->has_min_zoom_level ||
+		(zoom_control->details->zoom_level
+		 > zoom_control->details->min_zoom_level);
+}
